Package: pep725
Title: Pan-European Phenological Data Analysis
Version: 1.1.0
Authors@R: 
    c(person("Matthias", "Templ", , "matthias.templ@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8638-5276")),
      person("Barbara", "Templ", , "barbara.a.templ@gmail.com", role = c("aut"),
           comment = c(ORCID = "0000-0002-9391-5888"))
           )
Description: Provides a framework for quality-aware analysis of ground-based
    phenological data from the PEP725 Pan-European Phenology Database
    (Templ et al. (2018) <doi:10.1007/s00484-018-1512-8>;
    Templ et al. (2026) <doi:10.1111/nph.70869>) and similar observation
    networks. Implements station-level data quality grading, outlier
    detection, phenological normals (climate baselines), anomaly detection,
    elevation and latitude gradient estimation with robust regression,
    spatial synchrony quantification, partial least squares (PLS) regression
    for identifying temperature-sensitive periods, and sequential
    Mann-Kendall trend analysis. Supports data import from PEP725 files,
    conversion of user-supplied data, and downloadable synthetic datasets
    for teaching without barriers of registration.
    All analysis outputs provide 'print', 'summary', and
    'plot' methods. Interactive spatial visualization is available via
    'leaflet'.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1), data.table, ggplot2
Imports: dplyr, mgcv, methods, patchwork, purrr, robustbase, stats,
        tibble, tidyr, tidyselect, utils, rnaturalearth, sf
Suggests: ggmap, Kendall, knitr, leaflet, miniUI, nlme, quantreg,
        rmarkdown, rnaturalearthdata, pls, shiny, sp, testthat (>=
        3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
RoxygenNote: 7.3.3
BugReports: https://github.com/matthias-da/pep725/issues
URL: https://github.com/matthias-da/pep725
NeedsCompilation: no
Packaged: 2026-04-23 12:11:16 UTC; matthias
Author: Matthias Templ [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8638-5276>),
  Barbara Templ [aut] (ORCID: <https://orcid.org/0000-0002-9391-5888>)
Maintainer: Matthias Templ <matthias.templ@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-24 11:00:02 UTC
Built: R 4.6.0; ; 2026-04-30 00:07:55 UTC; unix

Package: pep725
Title: Pan-European Phenological Data Analysis
Version: 1.1.0
Authors@R: 
    c(person("Matthias", "Templ", , "matthias.templ@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8638-5276")),
      person("Barbara", "Templ", , "barbara.a.templ@gmail.com", role = c("aut"),
           comment = c(ORCID = "0000-0002-9391-5888"))
           )
Description: Provides a framework for quality-aware analysis of ground-based
    phenological data from the PEP725 Pan-European Phenology Database
    (Templ et al. (2018) <doi:10.1007/s00484-018-1512-8>;
    Templ et al. (2026) <doi:10.1111/nph.70869>) and similar observation
    networks. Implements station-level data quality grading, outlier
    detection, phenological normals (climate baselines), anomaly detection,
    elevation and latitude gradient estimation with robust regression,
    spatial synchrony quantification, partial least squares (PLS) regression
    for identifying temperature-sensitive periods, and sequential
    Mann-Kendall trend analysis. Supports data import from PEP725 files,
    conversion of user-supplied data, and downloadable synthetic datasets
    for teaching without barriers of registration.
    All analysis outputs provide 'print', 'summary', and
    'plot' methods. Interactive spatial visualization is available via
    'leaflet'.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1), data.table, ggplot2
Imports: dplyr, mgcv, methods, patchwork, purrr, robustbase, stats,
        tibble, tidyr, tidyselect, utils, rnaturalearth, sf
Suggests: ggmap, Kendall, knitr, leaflet, miniUI, nlme, quantreg,
        rmarkdown, rnaturalearthdata, pls, shiny, sp, testthat (>=
        3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
RoxygenNote: 7.3.3
BugReports: https://github.com/matthias-da/pep725/issues
URL: https://github.com/matthias-da/pep725
NeedsCompilation: no
Packaged: 2026-04-23 12:11:16 UTC; matthias
Author: Matthias Templ [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8638-5276>),
  Barbara Templ [aut] (ORCID: <https://orcid.org/0000-0002-9391-5888>)
Maintainer: Matthias Templ <matthias.templ@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-24 11:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 20:28:17 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",pep)
S3method(plot,pep)
S3method(plot,pep_completeness)
S3method(plot,pep_coverage)
S3method(plot,pep_outliers)
S3method(plot,pep_quality)
S3method(plot,pheno_anomaly)
S3method(plot,pheno_combined)
S3method(plot,pheno_gradient)
S3method(plot,pheno_normals)
S3method(plot,pheno_pls)
S3method(plot,pheno_synchrony)
S3method(plot,pheno_turning)
S3method(plot,second_events)
S3method(print,pep)
S3method(print,pep_completeness)
S3method(print,pep_coverage)
S3method(print,pep_outliers)
S3method(print,pep_quality)
S3method(print,phase_check)
S3method(print,phase_check_multi)
S3method(print,pheno_anomaly)
S3method(print,pheno_combined)
S3method(print,pheno_gradient)
S3method(print,pheno_normals)
S3method(print,pheno_pls)
S3method(print,pheno_synchrony)
S3method(print,pheno_turning)
S3method(print,second_events)
S3method(summary,pep)
S3method(summary,pep_completeness)
S3method(summary,pep_outliers)
S3method(summary,pep_quality)
S3method(summary,pheno_anomaly)
S3method(summary,pheno_normals)
S3method(summary,pheno_pls)
S3method(summary,pheno_synchrony)
S3method(summary,second_events)
export(add_country)
export(add_daylength)
export(as.pep)
export(bbch_description)
export(calc_daylength)
export(calc_max_daylength)
export(calc_thermal_sum)
export(calc_thermal_units)
export(get_phenology_doys)
export(is.pep)
export(kendall_tau)
export(mann_kendall_z)
export(new_pep)
export(pep725_demo)
export(pep_cache_clear)
export(pep_cache_info)
export(pep_check_connectivity)
export(pep_check_phases)
export(pep_check_phases_multi)
export(pep_completeness)
export(pep_coverage)
export(pep_download)
export(pep_flag_outliers)
export(pep_import)
export(pep_outliers_leaflet)
export(pep_plot_outliers)
export(pep_quality)
export(pep_second_events)
export(pep_simulate)
export(pheno_anomaly)
export(pheno_combine)
export(pheno_gradient)
export(pheno_leaflet)
export(pheno_map)
export(pheno_normals)
export(pheno_plot)
export(pheno_plot_hh)
export(pheno_plot_timeseries)
export(pheno_pls)
export(pheno_regional)
export(pheno_regional_hh)
export(pheno_synchrony)
export(pheno_trend_turning)
export(plot_phenology_trends)
export(select_phase)
export(subspecies_report)
export(summarize_subspecies_availability)
import(ggplot2)
import(methods)
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,fifelse)
importFrom(data.table,fread)
importFrom(data.table,melt)
importFrom(data.table,rbindlist)
importFrom(data.table,setDT)
importFrom(data.table,setcolorder)
importFrom(data.table,setnames)
importFrom(data.table,setorderv)
importFrom(data.table,uniqueN)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,coalesce)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,recode)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_quickmap)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(patchwork,wrap_plots)
importFrom(purrr,map)
importFrom(rnaturalearth,ne_countries)
importFrom(robustbase,lmrob)
importFrom(sf,"st_agr<-")
importFrom(sf,st_agr)
importFrom(sf,st_as_sf)
importFrom(sf,st_bbox)
importFrom(sf,st_join)
importFrom(sf,st_make_valid)
importFrom(sf,st_transform)
importFrom(stats,"contrasts<-")
importFrom(stats,IQR)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,contr.sum)
importFrom(stats,lm)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,weighted.mean)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unite)
importFrom(utils,data)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)

# pep725

<p align="center">
  <img src="presentations/pep725_logo_cpt.png" alt="pep725 logo" width="180"/>
</p>

[![R-CMD-check](https://github.com/matthias-da/pep725/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/matthias-da/pep725/actions/workflows/R-CMD-check.yaml)

pep725: an R package for Pan-European Phenological Data Analysis

**pep725** is an R package for analyzing phenological data from the [PEP725 Pan-European Phenology Database](https://pep725.eu/) and beyond. It provides tools for data exploration, quality assessment, climatological baseline calculation, trend analysis and visualization of phenology-climate relationships.

## Citation

If you use this R package, please cite:
Templ and Templ (2026). pep725: An R package for Pan-European Phenological Data Analysis. *The Journal of Open Source Software*, under review. 
Until it gets published, please cite:
Templ et al. (2018). Pan European Phenological database (PEP725): a single point of access for European data. Int. J. Biometeorology, 62(6): 1109-1113. doi:10.1007/s00484-018-1512-8

## Features

**Data Access:**
- Import PEP725 data files with automatic cleaning and formatting
- Download pre-generated synthetic datasets for learning and testing
- Generate synthetic data from your own datasets for teaching and sharing

**Core Analysis:**
- `pheno_normals()` - Calculate climatological baselines (WMO-style reference periods)
- `pheno_anomaly()` - Detect deviations from normal phenology
- `pep_quality()` - Assess data quality and assign reliability grades
- `pheno_gradient()` - Quantify elevation and latitude gradients
- `pheno_synchrony()` - Measure spatial coherence of phenological events

**Quality & Validation:**
- `pep_completeness()` - Assess species/phase coverage across stations
- `pep_check_phases()` - Validate phenological phase sequences
- `pep_flag_outliers()` - Detect outliers (30-day rule, MAD, IQR, z-score)

**Advanced Analysis:**
- `pheno_combine()` - Combined time series from multi-station data (robust/mixed/OLS)
- `pheno_trend_turning()` - Sequential Mann-Kendall trend turning point detection
- `pheno_pls()` - PLS regression for temperature-sensitive phenological periods

**Visualization:**
- `pheno_plot_timeseries()` - DOY trend plots over time
- `plot_phenology_trends()` - Robust regression trend analysis
- `pheno_leaflet()` - Interactive maps for station exploration and selection
- `pheno_map()` - Static maps of station networks

## Installation

```r
# Install from GitHub
# install.packages("devtools")
devtools::install_github("matthias-da/pep725")
```

## Quick Start

```r
library(pep725)

# Download synthetic data (cached locally after first download)
pep <- pep_download()

# Explore the data
print(pep)
summary(pep)
pep_coverage(pep)

# Filter to a species
wheat <- pep[species == "Triticum aestivum"]

# Calculate phenological normals
normals <- pheno_normals(
  wheat,
  period = 1990:2015,
  by = c("country", "phase_id"),
  min_years = 10
)
print(normals)

# Detect anomalies
anomalies <- pheno_anomaly(
  wheat,
  baseline_period = 1990:2010,
  by = c("country", "phase_id")
)
summary(anomalies)

# Assess data quality
quality <- pep_quality(wheat, by = c("s_id", "phase_id"))
summary(quality)
```

## Data Options

| Option | Best For | Requires Internet |
|--------|----------|-------------------|
| `pep_download()` | Learning, tutorials, reproducible examples | First use only |
| `data(pep_seed)` | Quick offline tests, minimal examples | No |
| `pep_simulate()` | Creating shareable datasets from your own data | No |
| `pep_import()` | Research with actual PEP725 observations | For download |
| `as.pep()` | Working with your own phenological data | No |

**Why synthetic data?** The original PEP725 database requires registration and has usage restrictions. Synthetic data preserves statistical properties while being freely shareable for tutorials and reproducible workflows.

### Working with your own phenological data

You can use all analysis functions on any phenological dataset — it does not have to come from PEP725. Convert your data to a `pep` object with `as.pep()`:

```r
library(data.table)

my_data <- data.table(
  s_id     = c(1, 1, 2, 2),
  lon      = c(8.5, 8.5, 9.1, 9.1),
  lat      = c(47.4, 47.4, 46.9, 46.9),
  alt      = c(400, 400, 550, 550),
  genus    = "Prunus",
  species  = "Prunus avium",
  phase_id = 60,
  year     = c(2022, 2023, 2022, 2023),
  day      = c(105, 98, 112, 103)
)

my_pep <- as.pep(my_data)

# Now use any pep725 analysis function
normals <- pheno_normals(my_pep, period = 2022:2023, min_years = 2)
```

Required columns: `s_id`, `lon`, `lat`, `genus`, `species`, `phase_id`, `year`, `day`. Optional: `alt`, `country`, `subspecies`.

## Spatial Analysis

```r
# Elevation gradient analysis
gradient <- pheno_gradient(
  pep,
  variable = "alt",
  species = "Triticum aestivum",
  phase_id = 60,
  method = "robust"
)
print(gradient)
plot(gradient)

# Spatial synchrony
synchrony <- pheno_synchrony(
  pep,
  species = "Triticum aestivum",
  phase_id = 60,
  by = c("country", "year"),
  min_stations = 3
)
summary(synchrony)
```

## Interactive Mapping

```r
# Launch interactive map for station selection
selected <- pheno_leaflet(pep, label_col = "species")

# Use selected stations for analysis
pep_subset <- pep[s_id %in% selected$s_id]
```

## Built-in Datasets

| Dataset | Description |
|---------|-------------|
| `pep_seed` | Small seed dataset for quick tests (1,319 rows) |

## Vignettes

The package includes detailed vignettes:

- **Getting Started with pep725** - Data loading, exploration, and basic usage
- **Phenological Analysis** - Normals, anomalies, quality assessment, and visualization
- **Spatial Phenological Patterns** - Gradients, synchrony, and mapping
- **Data Quality Assessment** - Quality grading, outlier detection, and completeness

```r
vignette("getting-started", package = "pep725")
vignette("phenological-analysis", package = "pep725")
vignette("spatial-patterns", package = "pep725")
vignette("data-quality", package = "pep725")
```

## Requirements

- R >= 4.1
- Google Maps API key required for `pheno_map()` static maps:
  ```r
  ggmap::register_google(key = "your_api_key")
  ```

## License

GPL-3

## Authors

- Matthias Templ (maintainer)
- Barbara Templ

## Acknowledgements
Dr. Barbara Templ was supported by the Berner Nachwuchsförderung (BNF) program of the University of Bern for developing the **pep725** R package.

## Links

- [GitHub Repository](https://github.com/matthias-da/pep725)
- [Report Issues](https://github.com/matthias-da/pep725/issues)
- [PEP725 Database](https://pep725.eu/)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(pep725)
library(data.table)

# Download the synthetic dataset
pep <- pep_download()

# For this vignette, we'll focus on flowering phases
flowering <- pep[phase_id %in% c(60, 65)]
cat("Flowering observations:", nrow(flowering), "\n")
cat("Species:", length(unique(flowering$species)), "\n")
cat("Year range:", min(flowering$year), "-", max(flowering$year), "\n")

## ----flag-basic---------------------------------------------------------------
# Flag outliers using the default 30-day rule
outliers <- pep_flag_outliers(
  pep = flowering,
  method = "30day",
  by = c("species", "phase_id")
)

print(outliers)

## ----flag-methods, eval=FALSE-------------------------------------------------
# # MAD method: flag if > 3 MAD from median (robust)
# outliers_mad <- pep_flag_outliers(flowering, method = "mad", threshold = 3)

## ----iqr-method, eval=FALSE---------------------------------------------------
# # IQR method: flag if outside 1.5 * IQR (standard boxplot rule)
# outliers_iqr <- pep_flag_outliers(flowering, method = "iqr", threshold = 1.5)

## ----zscore-method, eval=FALSE------------------------------------------------
# # Z-score method: flag if |z| > 3 (assumes normal distribution)
# outliers_zscore <- pep_flag_outliers(flowering, method = "zscore", threshold = 3)

## ----gam-method, eval=FALSE---------------------------------------------------
# # Model-based detection across the whole species x phase group.
# outliers_gam <- pep_flag_outliers(
#   apple_flowering,
#   by = c("genus", "species", "phase_id"),
#   method = "gam_residual",
#   threshold = 3.5
# )

## ----mahalanobis-method, eval=FALSE-------------------------------------------
# # Multivariate detection on the phase profile per station-year.
# # by = c("genus", "species") pools across phases so the covariance
# # is estimated from the joint (BBCH 60, BBCH 65, BBCH 89) distribution.
# outliers_mahal <- pep_flag_outliers(
#   apple,
#   by = c("genus", "species"),
#   method = "mahalanobis"
# )

## ----flag-summary-------------------------------------------------------------
summary(outliers)

## ----flag-compare-------------------------------------------------------------
# Check outliers for grapevine
vine_flowering <- pep[species == "Vitis vinifera" & phase_id %in% c(60, 65)]

if (nrow(vine_flowering) > 0) {
  outliers_vine <- pep_flag_outliers(
    pep = vine_flowering,
    method = "30day",
    by = c("species", "phase_id")
  )

  cat("Outlier comparison:\n")
  cat("All flowering species: ", round(100 * mean(outliers$is_outlier), 2), "%\n")
  cat("Grapevine only:        ", round(100 * mean(outliers_vine$is_outlier), 2), "%\n")
} else {
  cat("No grapevine flowering data available for comparison.\n")
}

## ----plot-overview, fig.height=6----------------------------------------------
# Overview of outlier patterns
pep_plot_outliers(outliers, type = "overview")

## ----plot-seasonal, fig.height=5----------------------------------------------
# When do outliers occur in the year?
pep_plot_outliers(outliers, type = "seasonal")

## ----plot-detail, fig.height=5------------------------------------------------
# See outliers in context of all observations
pep_plot_outliers(outliers, type = "detail", n_top = 15)

## ----plot-map, fig.height=5---------------------------------------------------
# Where are outliers located?
pep_plot_outliers(outliers, type = "map")

## ----plot-diagnostic-gam, eval=FALSE------------------------------------------
# # For gam_residual: classical model diagnostics
# outliers_gam <- pep_flag_outliers(
#   apple_flowering,
#   by = c("genus", "species", "phase_id"),
#   method = "gam_residual"
# )
# pep_plot_outliers(outliers_gam, type = "diagnostic")

## ----plot-diagnostic-mahal, eval=FALSE----------------------------------------
# # For mahalanobis: MD-specific diagnostics
# outliers_mahal <- pep_flag_outliers(
#   apple,
#   by = c("genus", "species"),
#   method = "mahalanobis"
# )
# pep_plot_outliers(outliers_mahal, type = "diagnostic")

## ----plot-profile, eval=FALSE-------------------------------------------------
# # Phase-profile parallel coordinates (primary Mahalanobis figure).
# pep_plot_outliers(outliers_mahal, type = "profile")

## ----completeness-basic-------------------------------------------------------
# Check completeness by station and phase
# Use year_range to focus on a specific period
completeness <- pep_completeness(
  pep = flowering,
  by = c("s_id", "phase_id"),
  year_range = c(1990, 2020)
)

print(completeness)

## ----completeness-summary-----------------------------------------------------
summary(completeness)

## ----completeness-filter------------------------------------------------------
# Get stations with good coverage (>= 70%)
good_coverage <- completeness[completeness_pct >= 70]
cat("Stations with >= 70% coverage:", nrow(good_coverage), "\n")

# Use these for trend analysis
good_stations <- unique(good_coverage$s_id)
flowering_complete <- flowering[s_id %in% good_stations]
cat("Observations from complete stations:", nrow(flowering_complete), "\n")

## ----completeness-plot, fig.height=5------------------------------------------
plot(completeness)

## ----check-basic--------------------------------------------------------------
# Check if expected phases are present for apple
apple <- pep[species == "Malus domestica"]

phase_check <- pep_check_phases(
  pep = apple,
  expected = c(60, 65, 87)  # flowering, full flowering, fruit maturity
)

print(phase_check)

## ----check-multi--------------------------------------------------------------
# Check phases for multiple species
multi_check <- pep_check_phases_multi(
  pep = pep,
  species_list = c("Malus domestica", "Vitis vinifera"),
  expected = c(60, 65, 87)
)

print(multi_check)

## ----workflow-----------------------------------------------------------------
# ══════════════════════════════════════════════════════════════════════════════
# STEP 1: Assess temporal completeness
# ══════════════════════════════════════════════════════════════════════════════
# Why: Incomplete stations can bias trend estimates and normals

completeness <- pep_completeness(flowering, by = c("s_id", "phase_id"))
good_stations <- completeness[completeness_pct >= 50, s_id]
fl_filtered <- flowering[s_id %in% good_stations]

cat("Kept", length(good_stations), "stations with >= 50% completeness\n")

# ══════════════════════════════════════════════════════════════════════════════
# STEP 2: Flag statistical outliers
# ══════════════════════════════════════════════════════════════════════════════
# Why: Identify observations that deviate from expected patterns

outliers_wf <- pep_flag_outliers(fl_filtered, method = "mad", threshold = 3)

cat("Flagged", sum(outliers_wf$is_outlier), "outliers",
    "(", round(100 * mean(outliers_wf$is_outlier), 1), "% )\n")

# ══════════════════════════════════════════════════════════════════════════════
# STEP 3: Make informed decisions about exclusion
# ══════════════════════════════════════════════════════════════════════════════
# Key principle: Document your decisions!

# Option A: Strict cleaning (for normals calculation)
fl_strict <- outliers_wf[is_outlier == FALSE]

# Option B: Moderate cleaning (keep moderate outliers)
fl_moderate <- outliers_wf[is_outlier == FALSE | abs(deviation) < 60]

cat("Strict cleaning keeps:", nrow(fl_strict), "obs\n")
cat("Moderate cleaning keeps:", nrow(fl_moderate), "obs\n")

# ══════════════════════════════════════════════════════════════════════════════
# STEP 4: Proceed with analysis on cleaned data
# ══════════════════════════════════════════════════════════════════════════════

normals <- pheno_normals(fl_moderate, period = 1991:2020, min_years = 5)
cat("Normals calculated for", sum(!is.na(normals$mean_doy)), "groups\n")

## ----eval = FALSE-------------------------------------------------------------
# vignette("getting-started", package = "pep725")

## ----eval = FALSE-------------------------------------------------------------
# vignette("phenological-analysis", package = "pep725")

## ----eval = FALSE-------------------------------------------------------------
# vignette("spatial-patterns", package = "pep725")

## ----session------------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----install, eval = FALSE----------------------------------------------------
# # Install from CRAN (when available)
# install.packages("pep725")
# 
# # Or install development version from GitHub
# # First install devtools if you don't have it:
# # install.packages("devtools")
# devtools::install_github("matthias-da/pep725")

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(pep725)

## ----download, message=FALSE--------------------------------------------------
# Download synthetic data (cached locally after first download)
pep <- pep_download()

## ----cachestatus, message=FALSE-----------------------------------------------
# Check cache status - shows file location and size
pep_cache_info()

# If you need to clear the cache (e.g., to get an updated version):
# pep_cache_clear()

## ----load-seed, eval = FALSE--------------------------------------------------
# # Load the included seed dataset (small, for quick tests only)
# data(pep_seed)
# 
# # This is a small subset (~1,300 rows) useful for:
# # - Testing code quickly
# # - Working offline
# # - Running examples in documentation

## ----simulate, eval = FALSE---------------------------------------------------
# # Generate synthetic data based on your real data
# # This preserves statistical properties while anonymizing observations
# pep_synthetic <- pep_simulate(your_real_pep_data)
# 
# # You can then share pep_synthetic freely

## ----import, eval = FALSE-----------------------------------------------------
# # Import from downloaded PEP725 files
# # Point to the folder containing your downloaded CSV files
# pep <- pep_import("path/to/pep725_data/")

## ----own-data, eval = FALSE---------------------------------------------------
# # Prepare your data with the required columns
# my_data <- data.frame(
#   s_id = c("MY001", "MY001", "MY002"),
#   lon = c(10.1, 10.1, 11.3),
#   lat = c(47.5, 47.5, 48.1),
#   genus = c("Malus", "Malus", "Malus"),
#   species = c("Malus domestica", "Malus domestica", "Malus domestica"),
#   phase_id = c(60, 65, 60),
#   year = c(2020, 2020, 2020),
#   day = c(110, 125, 108)
# )
# 
# # Convert to a pep object — validates required columns automatically
# my_pep <- as.pep(my_data)

## ----structure----------------------------------------------------------------
# View the data - the print method shows a summary
print(pep)

## ----first-look, fig.height = 4-----------------------------------------------
# Where are the stations?
plot(pep, type = "map")

# How are phenological events distributed across the year?
plot(pep, type = "histogram")

## ----class--------------------------------------------------------------------
# Check the class hierarchy
class(pep)

## ----summary------------------------------------------------------------------
# Default summary - shows top species by observation count
summary(pep)

## ----summary-phase------------------------------------------------------------
# Summary by phenological phase
summary(pep, by = "phase")

## ----summary-country----------------------------------------------------------
# Summary by country (showing top 5)
summary(pep, by = "country", top = 5)

## ----summary-year-------------------------------------------------------------
# Temporal coverage summary
summary(pep, by = "year")

## ----subset-------------------------------------------------------------------
# Filter to apple data using data.table syntax
# The syntax is: dataset[row_filter, column_selection]
apple <- pep[species == "Malus domestica"]
print(apple)

# Filter by year range
recent <- pep[year >= 2000 & year <= 2015]
print(recent)

# The summary method works on subsets too
summary(recent)

## ----bbch---------------------------------------------------------------------
# Get descriptions for BBCH codes present in your data
codes <- unique(pep$phase_id)
bbch_description(codes)

## ----filter-phase-------------------------------------------------------------
# Get all flowering observations (BBCH 60-69)
flowering <- pep[phase_id >= 60 & phase_id <= 69]
cat("Flowering observations:", nrow(flowering), "\n")

# Get only full flowering (BBCH 65)
full_flowering <- pep[phase_id == 65]
cat("Full flowering observations:", nrow(full_flowering), "\n")

# Get harvest-related observations
harvest <- pep[phase_id %in% c(87, 89)]
cat("Harvest observations:", nrow(harvest), "\n")

## ----coverage-all-------------------------------------------------------------
# Get a complete coverage report
pep_coverage(pep)

## ----coverage-temporal--------------------------------------------------------
# Temporal coverage with a visualization
pep_coverage(pep, kind = "temporal", plot = TRUE)

## ----coverage-geo-------------------------------------------------------------
# Geographical coverage - which countries have most data
pep_coverage(pep, kind = "geographical", top = 5)

## ----coverage-species---------------------------------------------------------
# Species coverage - which species are best represented
pep_coverage(pep, kind = "species", top = 5)

## ----coverage-by-country------------------------------------------------------
# Temporal coverage broken down by country
cov_by_country <- pep_coverage(pep, kind = "temporal", by = "country")
# Show observations by group
cov_by_country$temporal$obs_by_group

## ----quick-analysis-----------------------------------------------------------
# Step 1: Filter to apple flowering (BBCH 60 = first flowers)
apple_flowering <- pep[species == "Malus domestica" & phase_id == 60]

cat("Apple flowering observations:", nrow(apple_flowering), "\n")
cat("Year range:", min(apple_flowering$year), "-", max(apple_flowering$year), "\n")
cat("Countries:", length(unique(apple_flowering$country)), "\n")

# Step 2: Calculate annual mean DOY
# The data.table syntax here means:
#   - Group by year
#   - Calculate mean DOY and count observations
#   - Order by year
annual_mean <- apple_flowering[, .(
  mean_doy = mean(day, na.rm = TRUE),
  n_obs = .N
), by = year][order(year)]

# Step 3: Look at the results
print(annual_mean)

## ----quick-analysis-plot------------------------------------------------------
# Step 4: Plot the trend with ggplot2
library(ggplot2)

p <- ggplot(annual_mean, aes(x = year, y = mean_doy)) +
  geom_point(aes(size = n_obs), color = "steelblue", alpha = 0.6) +
  geom_line(color = "steelblue", alpha = 0.4) +
  geom_smooth(method = "lm", color = "red", linetype = 2, se = FALSE) +
  labs(
    title = "Apple Flowering Date Over Time",
    x = "Year", y = "Mean Day of Year",
    size = "Observations"
  ) +
  theme_minimal()
print(p)

# Step 5: Quantify the trend
trend <- lm(mean_doy ~ year, data = annual_mean)
slope <- coef(trend)[2]
cat("\nTrend:", round(slope * 10, 2), "days per decade\n")
if (slope < 0) {
  cat("Interpretation: Flowering is getting EARLIER\n")
} else {
  cat("Interpretation: Flowering is getting LATER\n")
}

## ----quick-analysis-vine------------------------------------------------------
# Step 1: Filter to grapevine flowering (BBCH 65 = full flowering)
vine_flowering <- pep[species == "Vitis vinifera" & phase_id == 65]

cat("Grapevine flowering observations:", nrow(vine_flowering), "\n")
cat("Year range:", min(vine_flowering$year), "-", max(vine_flowering$year), "\n")
cat("Countries:", length(unique(vine_flowering$country)), "\n")

# Step 2: Calculate annual mean DOY
vine_annual <- vine_flowering[, .(
  mean_doy = mean(day, na.rm = TRUE),
  n_obs = .N
), by = year][order(year)]

# Step 3: Plot the trend
p <- ggplot(vine_annual, aes(x = year, y = mean_doy)) +
  geom_point(aes(size = n_obs), color = "purple", alpha = 0.6) +
  geom_line(color = "purple", alpha = 0.4) +
  geom_smooth(method = "lm", color = "red", linetype = 2, se = FALSE) +
  labs(
    title = "Grapevine Flowering Date Over Time",
    x = "Year", y = "Mean Day of Year",
    size = "Observations"
  ) +
  theme_minimal()
print(p)

trend <- lm(mean_doy ~ year, data = vine_annual)
slope <- coef(trend)[2]
cat("\nTrend:", round(slope * 10, 2), "days per decade\n")

## ----eval = FALSE-------------------------------------------------------------
# vignette("phenological-analysis", package = "pep725")

## ----eval = FALSE-------------------------------------------------------------
# vignette("spatial-patterns", package = "pep725")

## ----eval = FALSE-------------------------------------------------------------
# vignette("data-quality", package = "pep725")

## ----session------------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(pep725)

# Download the synthetic dataset
pep <- pep_download()

# For this vignette, we'll focus on two well-represented species:
# 1. Apple (Malus domestica) - excellent coverage across Europe
# 2. Grapevine (Vitis vinifera) - longest historical records

apple <- pep[species == "Malus domestica"]
vine <- pep[species == "Vitis vinifera"]

# Verify subsets contain data
stopifnot(nrow(apple) > 0, nrow(vine) > 0)

# Quick overview of our data
cat("=== Apple (Malus domestica) ===\n")
cat("Observations:", nrow(apple), "\n")
cat("Year range:", min(apple$year), "-", max(apple$year), "\n")
cat("Stations:", length(unique(apple$s_id)), "\n\n")

cat("=== Grapevine (Vitis vinifera) ===\n")
cat("Observations:", nrow(vine), "\n")
cat("Year range:", min(vine$year), "-", max(vine$year), "\n")
cat("Stations:", length(unique(vine$s_id)), "\n")

## ----normals-basic------------------------------------------------------------
# Calculate normals for apple across countries and phases
# Using 1990-2015 for illustration (ensures enough data in synthetic dataset)
normals <- pheno_normals(
  pep = apple,
  period = 1990:2015,
  by = c("country", "phase_id"),
  min_years = 10
)

print(normals)

## ----normals-summary----------------------------------------------------------
# Get a summary of the normals
summary(normals)

## ----normals-plot, fig.width=7, fig.height=5----------------------------------
plot(normals)

## ----normals-filtered---------------------------------------------------------
# Normals for apple first flowering (BBCH 60) only
flowering_normals <- pheno_normals(
  pep = pep,                          # Can use full dataset
  period = 1990:2015,
  species = "Malus domestica",         # Filter by species
  phase_id = 60,                      # Filter by phase (first flowers)
  by = "country",                     # Group only by country
  min_years = 5                       # Lower threshold for more countries
)

print(flowering_normals)

## ----normals-compare, warning=FALSE, message=FALSE----------------------------
# Calculate normals for two periods
# Period 1: Historical (1961-1990)
period1 <- pheno_normals(
  pep,
  period = 1961:1990,
  species = "Malus",
  phase_id = 60,
  by = "country",
  min_years = 10
)

# Period 2: Recent (1991-2020)
period2 <- pheno_normals(
  pep,
  period = 1991:2020,
  species = "Malus",
  phase_id = 60,
  by = "country",
  min_years = 10
)

# Compare the two periods by merging on country
# Negative shift = flowering got earlier
# Positive shift = flowering got later
if (nrow(period1) > 0 && nrow(period2) > 0) {
  comparison <- merge(
    period1[, .(country, mean_doy_early = mean_doy)],
    period2[, .(country, mean_doy_recent = mean_doy)],
    by = "country"
  )
  comparison[, shift := mean_doy_recent - mean_doy_early]

  cat("Countries compared:", nrow(comparison), "\n")
  cat("Mean shift:", round(mean(comparison$shift, na.rm = TRUE), 1), "days\n")
  cat("Range of shifts:", round(min(comparison$shift, na.rm = TRUE), 1), "to",
      round(max(comparison$shift, na.rm = TRUE), 1), "days\n")

  if (mean(comparison$shift, na.rm = TRUE) < 0) {
    cat("\nInterpretation: On average, flowering has shifted EARLIER\n")
  } else {
    cat("\nInterpretation: On average, flowering has shifted LATER\n")
  }
}

## ----normals-vine, warning=FALSE, message=FALSE-------------------------------
# Calculate normals for grapevine flowering
vine_normals <- pheno_normals(
  pep = vine,
  period = 1990:2015,
  phase_id = 65,  # Full flowering
  by = "country",
  min_years = 5
)

# Compare with apple for countries that have both
if (nrow(vine_normals) > 0) {
  cat("Grapevine flowering normals (BBCH 65):\n")
  print(vine_normals[!is.na(mean_doy), .(country, n_years, mean_doy, sd_doy)])
}

## ----anomalies-basic----------------------------------------------------------
# Calculate anomalies for apple
# Using 1990-2010 as baseline period
anomalies <- pheno_anomaly(
  pep = apple,
  baseline_period = 1990:2010,
  by = c("country", "phase_id"),
  min_years = 5
)

print(anomalies)

## ----anomalies-extreme--------------------------------------------------------
# Find extreme years (is_extreme == TRUE)
extreme <- anomalies[is_extreme == TRUE]

if (nrow(extreme) > 0) {
  cat("Total extreme years found:", nrow(extreme), "\n\n")

  # Show the most extreme early years
  cat("Most extreme EARLY years:\n")
  early <- extreme[direction == "early"][order(anomaly_days)]
  if (nrow(early) > 0) {
    print(early[1:min(5, nrow(early)),
          .(country, phase_id, year, anomaly_days, z_score)])
  }

  # Show the most extreme late years
  cat("\nMost extreme LATE years:\n")
  late <- extreme[direction == "late"][order(-anomaly_days)]
  if (nrow(late) > 0) {
    print(late[1:min(5, nrow(late)),
          .(country, phase_id, year, anomaly_days, z_score)])
  }
} else {
  cat("No extreme years detected in this dataset\n")
}

## ----anomalies-summary--------------------------------------------------------
summary(anomalies)

## ----anomalies-plot, fig.width=7, fig.height=5--------------------------------
plot(anomalies)

## ----quality-basic------------------------------------------------------------
# Assess quality at the station level
quality <- pep_quality(
  pep = apple,
  by = c("s_id", "phase_id")
)

print(quality)

## ----quality-summary----------------------------------------------------------
# Summary of quality across all stations
summary(quality)

## ----quality-plot, fig.height=5, fig.width=11---------------------------------
# Overview plot: grade distribution + station map (requires pep for coordinates)
plot(quality, pep = apple)

## ----quality-plot-grades, fig.height=4----------------------------------------
# Just the grade distribution (no pep data needed)
plot(quality, which = "grades")

## ----quality-plot-map, fig.height=5-------------------------------------------
# Just the map of station quality
plot(quality, which = "map", pep = apple)

## ----quality-filter-----------------------------------------------------------
# Get high-quality stations
high_quality <- quality[quality_grade %in% c("A", "B")]

cat("Quality distribution:\n")
print(table(quality$quality_grade))

cat("\nHigh quality (A or B):", nrow(high_quality), "of", nrow(quality),
    "(", round(100 * nrow(high_quality) / nrow(quality), 1), "%)\n")

# Filter original data to these stations
if (nrow(high_quality) > 0) {
  good_stations <- unique(high_quality$s_id)
  apple_hq <- apple[s_id %in% good_stations]

  cat("\nOriginal apple observations:", nrow(apple), "\n")
  cat("After quality filter:", nrow(apple_hq),
      "(", round(100 * nrow(apple_hq) / nrow(apple), 1), "%)\n")
}

## ----quality-country----------------------------------------------------------
# Country-level quality (coarser assessment)
country_quality <- pep_quality(
  pep = pep,
  by = c("country", "species", "phase_id")
)

print(country_quality)

## ----workflow-----------------------------------------------------------------
cat("=== STEP 1: Data Quality Assessment ===\n\n")

# Assess quality
quality <- pep_quality(apple, by = c("s_id", "phase_id"))
cat("Quality grades:\n")
print(table(quality$quality_grade))

cat("\n=== STEP 2: Filter to Good Quality Data ===\n\n")

# Keep grades A, B, and C (exclude only grade D)
good_stations <- quality[quality_grade %in% c("A", "B", "C"), s_id]
apple_clean <- apple[s_id %in% good_stations]

cat("Original observations:", nrow(apple), "\n")
cat("After quality filter:", nrow(apple_clean), "\n")
cat("Retained:", round(100 * nrow(apple_clean) / nrow(apple), 1), "%\n")

stopifnot(nrow(apple_clean) > 0)

cat("\n=== STEP 3: Calculate Baseline Normals ===\n\n")

normals <- pheno_normals(
  apple_clean,
  period = 1990:2010,
  by = c("country", "phase_id"),
  min_years = 5
)

cat("Normals calculated for", sum(!is.na(normals$mean_doy)), "groups\n")
cat("Mean DOY range:", round(min(normals$mean_doy, na.rm = TRUE), 0),
    "to", round(max(normals$mean_doy, na.rm = TRUE), 0), "\n")

cat("\n=== STEP 4: Calculate Anomalies ===\n\n")

anomalies <- pheno_anomaly(
  apple_clean,
  baseline_period = 1990:2010,
  by = c("country", "phase_id"),
  min_years = 5
)

# Convert to plain data.table so grouped aggregation works correctly
# (the pheno_anomaly S3 class can interfere with [.data.table dispatch)
anomalies_dt <- as.data.table(anomalies)

cat("Anomalies calculated for", sum(!is.na(anomalies_dt$anomaly_days)), "year-groups\n")
cat("Extreme years:", sum(anomalies_dt$is_extreme, na.rm = TRUE), "\n")

cat("\n=== STEP 5: Analyze Trends by Decade ===\n\n")

decade_summary <- anomalies_dt[
  !is.na(anomaly_days),
  .(
    mean_anomaly = round(mean(anomaly_days, na.rm = TRUE), 1),
    sd_anomaly = round(sd(anomaly_days, na.rm = TRUE), 1),
    n_obs = .N,
    pct_extreme = round(100 * sum(is_extreme, na.rm = TRUE) / .N, 1)
  ),
  by = .(decade = floor(year / 10) * 10)
][order(decade)]

print(decade_summary)

cat("\nInterpretation:\n")
first_decade <- decade_summary$mean_anomaly[1]
last_decade <- decade_summary$mean_anomaly[nrow(decade_summary)]
change <- last_decade - first_decade

if (change < -3) {
  cat("- Strong shift toward EARLIER phenology\n")
} else if (change < 0) {
  cat("- Moderate shift toward earlier phenology\n")
} else if (change > 3) {
  cat("- Strong shift toward LATER phenology\n")
} else {
  cat("- No clear directional change\n")
}

## ----timeseries-basic, fig.height=5-------------------------------------------
# Prepare aggregated data for visualization
apple_annual <- apple[, .(
  day = mean(day, na.rm = TRUE),
  n = .N
), by = .(year, species, phase_id)]

# Add human-readable phase labels
apple_annual[, phase := factor(phase_id,
  levels = c(60, 65, 100),
  labels = c("First flowers", "Full flowering", "Harvest"))]

# Remove rows where phase mapping failed
apple_annual <- apple_annual[!is.na(phase)]

# Plot time series with trend lines
if (nrow(apple_annual) > 0) {
  pheno_plot_timeseries(
    apple_annual,
    color_by = "phase",
    smooth = TRUE,
    title = "Apple Phenology Trends"
  )
}

## ----trends-turning, fig.height=5---------------------------------------------
# Get apple flowering data
apple_flowering <- pep[species == "Malus domestica" & phase_id == 60]

# Detect turning points
turning <- pheno_trend_turning(apple_flowering, min_years = 10)
print(turning)

# Visualize the analysis
plot(turning)

## ----kendall-simple-----------------------------------------------------------
# Aggregate to annual means
annual_doy <- apple_flowering[, .(
  mean_doy = mean(day, na.rm = TRUE)
), by = year][order(year)]

# Calculate Mann-Kendall test statistic. Values beyond +/- 1.96 indicate
# significant trends at the 5% level (standard normal approximation).
# The implementation includes the tie correction and continuity correction.
# Note: pep725 (<= 1.0.2) exported this as kendall_tau(), which was a
# misnomer — it returned the Z-statistic, not Kendall's tau. kendall_tau()
# still works (with a deprecation warning); mann_kendall_z() is the new
# recommended name.
tau <- mann_kendall_z(annual_doy$mean_doy)

cat("Mann-Kendall z-statistic:", round(tau, 3), "\n")
cat("\nInterpretation:\n")
if (tau < -2.58) {
  cat("- Highly significant earlier trend (p < 0.01)\n")
} else if (tau < -1.96) {
  cat("- Significant earlier trend (p < 0.05)\n")
} else if (tau > 2.58) {
  cat("- Highly significant later trend (p < 0.01)\n")
} else if (tau > 1.96) {
  cat("- Significant later trend (p < 0.05)\n")
} else {
  cat("- No statistically significant trend\n")
}

## ----regional-climate, eval=FALSE---------------------------------------------
# # Compile regional data and link to temperature anomalies
# regional_data <- pheno_regional(
#   pep = pep,
#   species_name = "Malus domestica",
#   year_min = 1961
# )
# 
# # Visualize phenology alongside temperature anomalies
# pheno_plot(regional_data)

## ----eval = FALSE-------------------------------------------------------------
# vignette("spatial-patterns", package = "pep725")

## ----eval = FALSE-------------------------------------------------------------
# vignette("data-quality", package = "pep725")

## ----session------------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(pep725)
library(data.table)
library(ggplot2)

# Download the synthetic dataset
pep <- pep_download()

# Overview of the data
cat("Stations:", length(unique(pep$s_id)), "\n")
cat("Altitude range:", min(pep$alt, na.rm = TRUE), "-",
    max(pep$alt, na.rm = TRUE), "m\n")
cat("Latitude range:", round(min(pep$lat, na.rm = TRUE), 2), "-",
    round(max(pep$lat, na.rm = TRUE), 2), "N\n")

## ----gradient-altitude--------------------------------------------------------
# Calculate elevational gradient for apple flowering
grad_alt <- pheno_gradient(
  pep = pep,
  variable = "alt",
  species = "Malus domestica",
  phase_id = 60,
  method = "robust"
)

print(grad_alt)

## ----gradient-latitude--------------------------------------------------------
# Calculate latitude gradient
grad_lat <- pheno_gradient(
  pep = pep,
  variable = "lat",
  species = "Malus domestica",
  phase_id = 60,
  method = "robust"
)

print(grad_lat)

## ----gradient-methods---------------------------------------------------------
# Robust regression (default) - handles outliers
grad_robust <- pheno_gradient(pep, variable = "alt",
                              species = "Malus domestica",
                              phase_id = 60, method = "robust")

# Ordinary least squares
grad_ols <- pheno_gradient(pep, variable = "alt",
                           species = "Malus domestica",
                           phase_id = 60, method = "ols")

cat("Robust slope:", round(grad_robust$summary$slope, 2), "days/100m\n")
cat("OLS slope:", round(grad_ols$summary$slope, 2), "days/100m\n")

## ----gradient-vine------------------------------------------------------------
# Calculate gradient for grapevine
vine_data <- pep[species == "Vitis vinifera"]

# Verify data exists before analysis
if (nrow(vine_data) > 0) {
  grad_vine <- pheno_gradient(
    pep = pep,
    variable = "alt",
    species = "Vitis vinifera",
    phase_id = 65,  # Full flowering
    method = "robust"
  )

  cat("Comparison of elevation gradients:\n")
  cat("Apple:     ", round(grad_robust$summary$slope, 2), "days/100m",
      "(R² =", round(grad_robust$summary$r_squared, 3), ")\n")
  cat("Grapevine: ", round(grad_vine$summary$slope, 2), "days/100m",
      "(R² =", round(grad_vine$summary$r_squared, 3), ")\n")
} else {
  cat("No grapevine data available for gradient analysis.\n")
}

## ----gradient-by-region-------------------------------------------------------
# Calculate gradient by country
grad_by_country <- pheno_gradient(
  pep = pep,
  variable = "alt",
  species = "Malus domestica",
  phase_id = 60,
  by = "country",
  method = "robust"
)

print(grad_by_country$summary)

## ----gradient-plot, fig.height=5----------------------------------------------
# Plot the gradient relationship
if (!is.null(grad_alt$data) && nrow(grad_alt$data) > 2) {
  p <- plot(grad_alt)
  print(p)
}

## ----synchrony-basic----------------------------------------------------------
# Calculate synchrony by country and year
sync <- pheno_synchrony(
  pep = pep,
  species = "Malus domestica",
  phase_id = 60,
  by = c("country", "year"),
  min_stations = 3
)

print(sync)

## ----synchrony-summary--------------------------------------------------------
summary(sync)

## ----synchrony-trend----------------------------------------------------------
# Check if trend analysis was performed
if (!is.null(sync$trend) && nrow(sync$trend) > 0) {
  cat("Trend Analysis Results:\n")
  print(sync$trend)

  # Interpret significant trends
  sig_trends <- sync$trend[!is.na(significant) & significant == TRUE]
  if (nrow(sig_trends) > 0) {
    cat("\nSignificant trends detected:\n")
    print(sig_trends[, .(country, slope, direction, p_value)])
  }
}

## ----synchrony-plot, fig.height=5---------------------------------------------
# Plot synchrony time series
if (nrow(sync$data[!is.na(sd_doy)]) > 5) {
  p <- plot(sync)
  print(p)
}

## ----synchrony-simple---------------------------------------------------------
sync_simple <- pheno_synchrony(
  pep = pep,
  species = "Malus domestica",
  phase_id = 60,
  by = c("country", "year"),
  min_stations = 3,
  compute_trend = FALSE
)

# Just the synchrony data
head(sync_simple$data)

## ----synchrony-compare--------------------------------------------------------
# Calculate synchrony for grapevine
vine_check <- pep[species == "Vitis vinifera" & phase_id == 65]

if (nrow(vine_check) > 0) {
  sync_vine <- pheno_synchrony(
    pep = pep,
    species = "Vitis vinifera",
    phase_id = 65,
    by = c("country", "year"),
    min_stations = 3,
    compute_trend = FALSE
  )

  cat("Synchrony comparison (mean SD across years):\n")
  cat("Apple (flowering):    ", round(sync$overall$mean_sd_doy, 1), "days\n")
  cat("Grapevine (flowering):", round(sync_vine$overall$mean_sd_doy, 1), "days\n")
} else {
  cat("Insufficient grapevine data for synchrony comparison.\n")
}

## ----combined-analysis--------------------------------------------------------
# 1. Understand elevation gradient
gradient <- pheno_gradient(
  pep, variable = "alt",
  species = "Malus domestica",
  phase_id = 60
)

cat("Elevation Gradient Analysis:\n")
cat("  Slope:", round(gradient$summary$slope, 2), "days/100m\n")
cat("  R-squared:", round(gradient$summary$r_squared, 3), "\n\n")

# 2. Assess spatial synchrony
synchrony <- pheno_synchrony(
  pep,
  species = "Malus domestica",
  phase_id = 60,
  min_stations = 3
)

cat("Synchrony Analysis:\n")
cat("  Mean SD across stations:", round(synchrony$overall$mean_sd_doy, 1), "days\n")
cat("  Mean CV:", round(synchrony$overall$mean_cv_pct, 1), "%\n")

## ----leaflet-example, eval=FALSE----------------------------------------------
# # Launch interactive map (opens in viewer)
# # Draw rectangles or polygons to select stations
# selected_stations <- pheno_leaflet(pep, label_col = "species")
# 
# # The function returns a data.frame of selected stations
# print(selected_stations)

## ----leaflet-screenshot, echo=FALSE, out.width="100%", fig.cap="The pheno_leaflet() interface showing station locations with drawing tools for selection."----
# Display screenshot if it exists
screenshot_path <- "pheno_leaflet_screenshot.png"
if (file.exists(screenshot_path)) {
  knitr::include_graphics(screenshot_path)
} else {
  # Placeholder message when screenshot doesn't exist
  cat("*[Screenshot: Interactive map showing PEP725 stations across Europe with drawing toolbar for rectangle and polygon selection]*\n")
}

## ----leaflet-filtered, eval=FALSE---------------------------------------------
# # Filter to a specific species for faster loading (recommended)
# apple <- pep[species == "Malus domestica"]
# selected <- pheno_leaflet(apple, label_col = "s_id")
# 
# # Use selected stations for focused analysis
# apple_subset <- apple[s_id %in% selected$s_id]

## ----map-basic, fig.height=6, fig.width=8-------------------------------------
# Basic station map with country borders (no API key needed)
pheno_map(pep, background = "none", color_by = "none", point_size = 1.5)

## ----map-nobs, fig.height=6, fig.width=8--------------------------------------
# Color stations by number of observations
pheno_map(pep, background = "none", color_by = "n_obs", point_size = 1.5)

## ----map-nspecies, fig.height=6, fig.width=8----------------------------------
# Color by species diversity at each station
pheno_map(pep, background = "none", color_by = "n_species", point_size = 1.5)

## ----map-google, eval=FALSE---------------------------------------------------
# # Register your API key first
# ggmap::register_google(key = "your_api_key_here")
# 
# # Then use background = "google"
# pheno_map(pep, background = "google", color_by = "n_obs", zoom = 5)
# 
# # Regional focus with Google Maps
# pheno_map(
#   pep,
#   background = "google",
#   location = c(lon = 8.2, lat = 46.8),
#   zoom = 7,
#   color_by = "n_obs"
# )

## ----map-mean-doy, fig.height=6, fig.width=8----------------------------------
# Map mean phenological timing for flowering (phase 60)
# Earlier timing = darker colors, later = lighter (plasma scale)
pheno_map(pep, background = "none", color_by = "mean_doy",
        phase_id = 60, point_size = 1.5)

## ----map-trend, fig.height=6, fig.width=8-------------------------------------
# Map trends per station
# Blue = phenology getting earlier, Red = getting later
pheno_map(pep, background = "none", color_by = "trend",
        phase_id = 60, period = 1990:2020, min_years = 10, point_size = 1.5)

## ----map-species-cv, fig.height=6, fig.width=8--------------------------------
# Map species variation at each station
# Higher CV = more variation in timing among species
pheno_map(pep, background = "none", color_by = "species_cv",
        phase_id = 60, min_species = 3, point_size = 1.5)

## ----mapping-workflow, eval=FALSE---------------------------------------------
# # 1. Explore stations interactively
# selected <- pheno_leaflet(pep[genus == "Malus"])
# 
# # 2. Subset data to selected region
# pep_region <- pep[s_id %in% selected$s_id]
# 
# # 3. Analyze gradients in the selected region
# gradient <- pheno_gradient(pep_region, variable = "alt",
#                            species = "Malus domestica", phase_id = 60)
# 
# # 4. Assess synchrony
# synchrony <- pheno_synchrony(pep_region, species = "Malus domestica",
#                               phase_id = 60)
# 
# # 5. Create publication map of the region (no API needed)
# pheno_map(pep_region, background = "none", color_by = "mean_doy",
#         phase_id = 60, output_file = "regional_phenology.pdf")

## ----eval = FALSE-------------------------------------------------------------
# vignette("getting-started", package = "pep725")

## ----eval = FALSE-------------------------------------------------------------
# vignette("phenological-analysis", package = "pep725")

## ----eval = FALSE-------------------------------------------------------------
# vignette("data-quality", package = "pep725")

## ----session------------------------------------------------------------------
sessionInfo()

utils::globalVariables(c("grade_num", "worst_grade_num"))

#' Assess Data Quality of Phenological Observations
#'
#' Provides a comprehensive assessment of data completeness, temporal coverage,
#' outlier prevalence, and assigns quality grades to phenological time series.
#'
#' @param pep A \code{pep} object or \code{data.table} containing phenological
#'   observations with columns \code{year}, \code{day}, and grouping variables.
#' @param by Character vector of column names to group by for quality assessment.
#'   Default is \code{c("s_id", "genus", "phase_id")} for station-level assessment.
#' @param year_range Optional integer vector of length 2 specifying the start and
#'   end years for assessment. If \code{NULL} (default), uses the full range
#'   of years in the data.
#' @param species Optional character string to filter by species/genus.
#' @param phase_id Optional integer vector to filter by BBCH phase codes.
#' @param outlier_method Character string specifying outlier detection method.
#'   Either \code{"tukey"} (default, uses 1.5*IQR rule) or \code{"zscore"}
#'   (flags values with |z| > 3).
#' @param na.rm Logical. Should missing values be removed? Default \code{TRUE}.
#'
#' @return A \code{data.table} with the following columns:
#'   \describe{
#'     \item{by variables}{Grouping variables as specified in \code{by}}
#'     \item{n_obs}{Total number of observations}
#'     \item{year_min}{First year of observations}
#'     \item{year_max}{Last year of observations}
#'     \item{year_span}{Total span in years (year_max - year_min + 1)}
#'     \item{n_years}{Number of distinct years with data}
#'     \item{completeness_pct}{Percentage of years with data within the span}
#'     \item{max_gap_years}{Length of the longest gap (consecutive missing years)}
#'     \item{n_gaps}{Number of gaps in the time series}
#'     \item{n_outliers}{Number of statistical outliers detected}
#'     \item{outlier_pct}{Percentage of observations that are outliers}
#'     \item{doy_mean}{Mean day-of-year}
#'     \item{doy_sd}{Standard deviation of day-of-year}
#'     \item{doy_range}{Range of day-of-year values}
#'     \item{quality_grade}{Overall quality grade (A, B, C, or D)}
#'   }
#'
#' @details
#' This function evaluates phenological time series quality based on multiple
#' criteria important for trend analysis and climatological calculations.
#'
#' @section Quality Grading Criteria:
#' \describe{
#'   \item{Grade A}{Completeness >= 80%, no gaps > 2 years, outliers < 2%}
#'   \item{Grade B}{Completeness >= 60%, no gaps > 5 years, outliers < 5%}
#'   \item{Grade C}{Completeness >= 40%, gaps <= 10 years, outliers < 10%}
#'   \item{Grade D}{Below grade C thresholds}
#' }
#'
#' @section Outlier Detection:
#' The Tukey method (default) flags observations outside the interquartile
#' fences: \code{[Q1 - 1.5*IQR, Q3 + 1.5*IQR]}. This is robust to non-normal
#' distributions common in phenological data.
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Subset to one country for speed
#' pep_ch <- pep[country == "Switzerland"]
#'
#' # Assess quality for stations in Switzerland
#' quality <- pep_quality(pep_ch)
#' print(quality)
#'
#' # Filter to high-quality stations only
#' high_quality <- quality[quality_grade %in% c("A", "B")]
#' high_quality
#'
#' # Assess specific species and phase
#' apple_quality <- pep_quality(pep_ch,
#'                              species = "Malus",
#'                              phase_id = 60)
#' apple_quality
#'
#' # Country-level assessment (use subset of countries)
#' pep_subset <- pep[country %in% c("Switzerland", "Austria")]
#' country_quality <- pep_quality(pep_subset,
#'                                by = c("country", "genus", "phase_id"))
#' country_quality
#'
#' # Assess quality within a specific time window
#' modern_quality <- pep_quality(pep_ch,
#'                               year_range = c(1991, 2020))
#' modern_quality
#' }
#'
#' @references
#' Schaber, J. and Badeck, F.-W. (2002). Evaluation of methods for the
#' combination of phenological time series and outlier detection.
#' \emph{Tree Physiology}, 22, 973--982.
#' \doi{10.1093/treephys/22.14.973}
#'
#' @seealso
#' \code{\link{pheno_normals}} for calculating climatological normals,
#' \code{\link{pheno_anomaly}} for anomaly detection
#'
#' @author Matthias Templ
#' @export
pep_quality <- function(pep,
                        by = c("s_id", "genus", "phase_id"),
                        year_range = NULL,
                        species = NULL,
                        phase_id = NULL,
                        outlier_method = c("tukey", "zscore"),
                        na.rm = TRUE) {

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table if needed
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Check required columns
  required_cols <- c("year", "day")
  missing_cols <- setdiff(required_cols, names(pep))
  if (length(missing_cols) > 0) {
    stop("Missing required columns: ", paste(missing_cols, collapse = ", "),
         call. = FALSE)
  }

  # Check that 'by' columns exist
  missing_by <- setdiff(by, names(pep))
  if (length(missing_by) > 0) {
    stop("Column(s) specified in 'by' not found: ",
         paste(missing_by, collapse = ", "),
         call. = FALSE)
  }

  # Validate outlier_method
  outlier_method <- match.arg(outlier_method)

  # Make a copy to avoid modifying original
  dt <- data.table::copy(pep)

  # Apply species filter if specified
  if (!is.null(species)) {
    if ("genus" %in% names(dt)) {
      genus_match <- dt$genus == species
      species_match <- if ("species" %in% names(dt)) dt$species == species else FALSE
      dt <- dt[genus_match | species_match]
      if (nrow(dt) == 0) {
        stop("No observations found for species '", species, "'", call. = FALSE)
      }
    } else if ("species" %in% names(dt)) {
      dt <- dt[dt$species == species]
      if (nrow(dt) == 0) {
        stop("No observations found for species '", species, "'", call. = FALSE)
      }
    } else {
      warning("Neither 'genus' nor 'species' column found. Ignoring species filter.",
              call. = FALSE)
    }
  }

  # Apply phase_id filter if specified
  if (!is.null(phase_id)) {
    if (!"phase_id" %in% names(dt)) {
      warning("'phase_id' column not found. Ignoring phase_id filter.", call. = FALSE)
    } else {
      phase_filter <- phase_id
      dt <- dt[dt$phase_id %in% phase_filter]
      if (nrow(dt) == 0) {
        stop("No observations found for phase_id: ",
             paste(phase_id, collapse = ", "), call. = FALSE)
      }
    }
  }

  # Apply year range filter if specified
  if (!is.null(year_range)) {
    if (!is.numeric(year_range) || length(year_range) != 2) {
      stop("'year_range' must be a numeric vector of length 2", call. = FALSE)
    }
    dt <- dt[year >= year_range[1] & year <= year_range[2]]
    if (nrow(dt) == 0) {
      stop("No observations found in year range: ",
           year_range[1], "-", year_range[2], call. = FALSE)
    }
  }

  # Define outlier detection function
  detect_outliers <- function(x) {
    if (length(x) < 4) return(rep(FALSE, length(x)))

    if (outlier_method == "tukey") {
      q1 <- quantile(x, 0.25, na.rm = na.rm)
      q3 <- quantile(x, 0.75, na.rm = na.rm)
      iqr <- q3 - q1
      lower <- q1 - 1.5 * iqr
      upper <- q3 + 1.5 * iqr
      return(x < lower | x > upper)
    } else {
      # zscore method
      z <- (x - mean(x, na.rm = na.rm)) / sd(x, na.rm = na.rm)
      return(abs(z) > 3)
    }
  }

  # Function to find maximum gap in years
  find_max_gap <- function(years, min_year, max_year) {
    if (length(years) <= 1) return(0)

    all_years <- min_year:max_year
    present <- all_years %in% years

    if (all(present)) return(0)

    # Find runs of missing years
    runs <- rle(!present)
    gaps <- runs$lengths[runs$values]

    if (length(gaps) == 0) return(0)
    return(max(gaps))
  }

  # Function to count gaps
  count_gaps <- function(years, min_year, max_year) {
    if (length(years) <= 1) return(0)

    all_years <- min_year:max_year
    present <- all_years %in% years

    if (all(present)) return(0)

    runs <- rle(!present)
    return(sum(runs$values))
  }

  # Calculate quality metrics by group
  result <- dt[, {
    doy_values <- day
    years_present <- unique(year)
    n_years_present <- length(years_present)

    year_min <- min(year, na.rm = na.rm)
    year_max <- max(year, na.rm = na.rm)
    year_span <- year_max - year_min + 1

    # Completeness
    completeness <- if (year_span > 0) 100 * n_years_present / year_span else 0

    # Gap analysis
    max_gap <- find_max_gap(years_present, year_min, year_max)
    n_gaps <- count_gaps(years_present, year_min, year_max)

    # Outlier detection
    is_outlier <- detect_outliers(doy_values)
    n_outliers <- sum(is_outlier, na.rm = TRUE)
    outlier_pct <- if (length(doy_values) > 0) 100 * n_outliers / length(doy_values) else 0

    # DOY statistics
    doy_mean <- mean(doy_values, na.rm = na.rm)
    doy_sd <- sd(doy_values, na.rm = na.rm)
    doy_range <- max(doy_values, na.rm = na.rm) - min(doy_values, na.rm = na.rm)

    # Quality grading
    grade <- if (completeness >= 80 && max_gap <= 2 && outlier_pct < 2) {
      "A"
    } else if (completeness >= 60 && max_gap <= 5 && outlier_pct < 5) {
      "B"
    } else if (completeness >= 40 && max_gap <= 10 && outlier_pct < 10) {
      "C"
    } else {
      "D"
    }

    list(
      n_obs = as.integer(.N),
      year_min = as.integer(year_min),
      year_max = as.integer(year_max),
      year_span = as.integer(year_span),
      n_years = as.integer(n_years_present),
      completeness_pct = as.numeric(round(completeness, 1)),
      max_gap_years = as.integer(max_gap),
      n_gaps = as.integer(n_gaps),
      n_outliers = as.integer(n_outliers),
      outlier_pct = as.numeric(round(outlier_pct, 1)),
      doy_mean = as.numeric(round(doy_mean, 1)),
      doy_sd = as.numeric(round(doy_sd, 1)),
      doy_range = as.integer(doy_range),
      quality_grade = as.character(grade)
    )
  }, by = by]

  # Order by grouping variables
  data.table::setorderv(result, by)

  # Add class for potential method dispatch
  class(result) <- c("pep_quality", class(result))

  # Store parameters as attributes
  attr(result, "year_range") <- year_range
  attr(result, "outlier_method") <- outlier_method
  attr(result, "species_filter") <- species
  attr(result, "phase_filter") <- phase_id

  result
}


#' Print Method for Data Quality Assessment
#'
#' @param x A \code{pep_quality} object
#' @param n Number of rows to display
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns \code{x}
#' @author Matthias Templ
#' @export
print.pep_quality <- function(x, n = 10, ...) {
  year_range <- attr(x, "year_range")
  outlier_method <- attr(x, "outlier_method")
  species_filter <- attr(x, "species_filter")
  phase_filter <- attr(x, "phase_filter")

  cat("Phenological Data Quality Assessment\n")
  cat(strrep("-", 50), "\n")

  if (!is.null(year_range) && length(year_range) == 2) {
    cat(sprintf("Assessment period: %d-%d\n", year_range[1], year_range[2]))
  }

  if (!is.null(outlier_method)) {
    cat(sprintf("Outlier method: %s\n", outlier_method))
  }

  if (!is.null(species_filter)) {
    cat(sprintf("Species filter: %s\n", species_filter))
  }
  if (!is.null(phase_filter)) {
    cat(sprintf("Phase filter: %s\n", paste(phase_filter, collapse = ", ")))
  }

  # Grade distribution
  grade_table <- table(x$quality_grade)
  cat("\nQuality Grade Distribution:\n")
  for (g in c("A", "B", "C", "D")) {
    count <- if (g %in% names(grade_table)) grade_table[[g]] else 0
    pct <- 100 * count / nrow(x)
    cat(sprintf("  Grade %s: %d (%.1f%%)\n", g, count, pct))
  }

  cat(sprintf("\nTotal groups assessed: %d\n", nrow(x)))
  cat(strrep("-", 50), "\n\n")

  # Print as data.table
  print(data.table::as.data.table(x)[1:min(n, nrow(x))])

  if (nrow(x) > n) {
    cat(sprintf("\n... and %d more rows\n", nrow(x) - n))
  }

  invisible(x)
}


#' Summary Method for Data Quality Assessment
#'
#' @param object A \code{pep_quality} object
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns a summary list
#' @author Matthias Templ
#' @export
summary.pep_quality <- function(object, ...) {
  cat("Phenological Data Quality Summary\n")
  cat(strrep("=", 50), "\n\n")

  cat(sprintf("Total groups assessed: %d\n\n", nrow(object)))

  # Grade distribution
  grade_table <- table(object$quality_grade)
  cat("Quality Grade Distribution:\n")
  for (g in c("A", "B", "C", "D")) {
    count <- if (g %in% names(grade_table)) grade_table[[g]] else 0
    pct <- 100 * count / nrow(object)
    bar <- paste(rep("*", round(pct / 5)), collapse = "")
    cat(sprintf("  Grade %s: %4d (%5.1f%%) %s\n", g, count, pct, bar))
  }

  cat("\nCompleteness Statistics:\n")
  cat(sprintf("  Mean completeness: %.1f%%\n", mean(object$completeness_pct, na.rm = TRUE)))
  cat(sprintf("  Median completeness: %.1f%%\n", median(object$completeness_pct, na.rm = TRUE)))
  cat(sprintf("  Range: %.1f%% - %.1f%%\n",
              min(object$completeness_pct, na.rm = TRUE),
              max(object$completeness_pct, na.rm = TRUE)))

  cat("\nTemporal Coverage:\n")
  cat(sprintf("  Mean years with data: %.1f\n", mean(object$n_years, na.rm = TRUE)))
  cat(sprintf("  Mean year span: %.1f\n", mean(object$year_span, na.rm = TRUE)))
  cat(sprintf("  Mean max gap: %.1f years\n", mean(object$max_gap_years, na.rm = TRUE)))

  cat("\nOutlier Statistics:\n")
  cat(sprintf("  Mean outlier percentage: %.2f%%\n", mean(object$outlier_pct, na.rm = TRUE)))
  cat(sprintf("  Groups with no outliers: %d (%.1f%%)\n",
              sum(object$n_outliers == 0),
              100 * sum(object$n_outliers == 0) / nrow(object)))

  invisible(list(
    n_groups = nrow(object),
    grade_distribution = grade_table,
    mean_completeness = mean(object$completeness_pct, na.rm = TRUE),
    mean_max_gap = mean(object$max_gap_years, na.rm = TRUE),
    mean_outlier_pct = mean(object$outlier_pct, na.rm = TRUE)
  ))
}


#' Plot Method for Data Quality Assessment
#'
#' Creates visualizations of phenological data quality metrics, including
#' grade distribution and a map of station quality.
#'
#' @param x A \code{pep_quality} object.
#' @param which Character. Type of plot to produce:
#'   \itemize{
#'     \item \code{"overview"} (default): Two-panel overview with grade
#'       distribution and station quality map (requires \code{pep} argument)
#'     \item \code{"grades"}: Bar chart of quality grade distribution only
#'     \item \code{"map"}: Map of station locations colored by quality grade
#'       (requires \code{pep} argument for coordinates)
#'   }
#' @param pep Optional \code{pep} object providing station coordinates (lon, lat).
#'   Required for \code{which = "map"} or \code{which = "overview"}.
#' @param show_grades Character vector of grades to display on the map.
#'   Default is \code{c("A", "B", "C", "D")} (all grades). Use e.g.
#'   \code{show_grades = "D"} to show only poor-quality stations.
#' @param alpha Numeric. Transparency of points on the map (0-1). Default is 0.6.
#' @param title Optional character string for the plot title.
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object (invisibly).
#'
#' @details
#' For the map visualization, the function:
#' \itemize{
#'   \item Aggregates quality to one grade per station (worst grade if multiple phases)
#'   \item Uses country borders from the \code{rnaturalearth} package
#'   \item Uses colorblind-friendly colors (blue=A, cyan=B, orange=C, vermillion=D)
#' }
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Assess quality for Swiss stations
#' pep_ch <- pep[country == "Switzerland"]
#' quality <- pep_quality(pep_ch, by = c("s_id", "phase_id"))
#'
#' # Grade distribution only (no pep data needed)
#' plot(quality, which = "grades")
#'
#' # Map of station quality (requires pep for coordinates)
#' plot(quality, which = "map", pep = pep_ch)
#'
#' # Map showing only poor-quality stations
#' plot(quality, which = "map", pep = pep_ch, show_grades = "D")
#'
#' # Map showing problematic stations (C and D grades)
#' plot(quality, which = "map", pep = pep_ch, show_grades = c("C", "D"))
#'
#' # Overview: grades + map
#' plot(quality, pep = pep_ch)
#' }
#'
#' @author Matthias Templ
#' @export
plot.pep_quality <- function(x, which = c("overview", "grades", "map"),
                              pep = NULL, show_grades = c("A", "B", "C", "D"),
                              alpha = 0.6, title = NULL, ...) {
  which <- match.arg(which)

  # Validate show_grades

  show_grades <- match.arg(show_grades, c("A", "B", "C", "D"), several.ok = TRUE)

  # Colorblind-friendly palette (Okabe-Ito inspired)
  # Blue for best, vermillion for worst - distinguishable for most color vision types
  grade_colors <- c("A" = "#0072B2", "B" = "#56B4E9", "C" = "#E69F00", "D" = "#D55E00")

  # Ensure quality_grade is a factor with correct order
  plot_data <- data.table::as.data.table(x)
  plot_data[, quality_grade := factor(quality_grade, levels = c("A", "B", "C", "D"))]

  # Grade distribution bar chart
  plot_grades <- function() {
    grade_counts <- plot_data[, .N, by = quality_grade]
    grade_counts[, pct := 100 * N / sum(N)]

    ggplot2::ggplot(grade_counts, ggplot2::aes(x = quality_grade, y = N, fill = quality_grade)) +
      ggplot2::geom_col(show.legend = FALSE) +
      ggplot2::geom_text(ggplot2::aes(label = sprintf("%d\n(%.0f%%)", N, pct)),
                         vjust = -0.2, size = 3) +
      ggplot2::scale_fill_manual(values = grade_colors) +
      ggplot2::scale_y_continuous(expand = ggplot2::expansion(mult = c(0, 0.15))) +
      ggplot2::labs(x = "Quality Grade", y = "Number of Groups",
                    title = "Quality Grade Distribution") +
      ggplot2::theme_minimal() +
      ggplot2::theme(panel.grid.major.x = ggplot2::element_blank())
  }

  # Map of station quality
  plot_map <- function() {
    if (is.null(pep)) {
      stop("'pep' argument required for map visualization (provides station coordinates)",
           call. = FALSE)
    }

    # Check for required columns
    if (!all(c("s_id", "lon", "lat") %in% names(pep))) {
      stop("'pep' must contain columns: s_id, lon, lat", call. = FALSE)
    }

    # Get unique station coordinates
    stations <- unique(data.table::as.data.table(pep)[, .(s_id, lon, lat)])

    # Aggregate quality to station level (worst grade per station)
    # Convert grade to numeric for aggregation, then back
    grade_numeric <- c("A" = 1, "B" = 2, "C" = 3, "D" = 4)
    plot_data[, grade_num := grade_numeric[as.character(quality_grade)]]

    station_quality <- plot_data[, .(
      worst_grade_num = max(grade_num, na.rm = TRUE),
      n_phases = .N
    ), by = s_id]

    station_quality[, quality_grade := factor(
      names(grade_numeric)[worst_grade_num],
      levels = c("A", "B", "C", "D")
    )]

    # Merge with coordinates
    map_data <- merge(station_quality, stations, by = "s_id", all.x = TRUE)
    map_data <- map_data[!is.na(lon) & !is.na(lat)]

    # Filter to selected grades
    map_data <- map_data[quality_grade %in% show_grades]

    if (nrow(map_data) == 0) {
      stop("No stations with valid coordinates found for grades: ",
           paste(show_grades, collapse = ", "), call. = FALSE)
    }

    # Get country borders from rnaturalearth
    world <- rnaturalearth::ne_countries(scale = "medium", returnclass = "sf")

    # Determine map extent from data (with padding)
    lon_range <- range(map_data$lon, na.rm = TRUE)
    lat_range <- range(map_data$lat, na.rm = TRUE)
    lon_pad <- diff(lon_range) * 0.1
    lat_pad <- diff(lat_range) * 0.1

    # Build subtitle
    grade_str <- if (length(show_grades) == 4) "all grades" else paste("Grade", paste(show_grades, collapse = ", "))
    subtitle <- sprintf("%d stations (%s)", nrow(map_data), grade_str)

    # Create map
    ggplot2::ggplot() +
      ggplot2::geom_sf(data = world, fill = "gray95", color = "gray70", linewidth = 0.2) +
      ggplot2::geom_point(
        data = map_data,
        ggplot2::aes(x = lon, y = lat, color = quality_grade),
        size = 2, alpha = alpha
      ) +
      ggplot2::scale_color_manual(values = grade_colors, name = "Grade", drop = FALSE) +
      ggplot2::coord_sf(
        xlim = c(lon_range[1] - lon_pad, lon_range[2] + lon_pad),
        ylim = c(lat_range[1] - lat_pad, lat_range[2] + lat_pad),
        expand = FALSE
      ) +
      ggplot2::labs(
        title = "Station Quality Map",
        subtitle = subtitle,
        x = "Longitude", y = "Latitude"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        panel.grid = ggplot2::element_line(color = "gray90"),
        legend.position = "right"
      )
  }

  # Generate requested plot type
  if (which == "grades") {
    p <- plot_grades()
    if (!is.null(title)) p <- p + ggplot2::labs(title = title)
    print(p)
    return(invisible(p))
  }

  if (which == "map") {
    p <- plot_map()
    if (!is.null(title)) p <- p + ggplot2::labs(title = title)
    print(p)
    return(invisible(p))
  }

  # Overview: grades + map side by side
  if (is.null(pep)) {
    message("Note: 'pep' argument not provided. Showing grades only.")
    message("For overview with map, use: plot(quality, pep = your_pep_data)")
    p <- plot_grades()
    if (!is.null(title)) p <- p + ggplot2::labs(title = title)
    print(p)
    return(invisible(p))
  }

  p1 <- plot_grades()
  p2 <- plot_map()

  # Use patchwork to combine
  combined <- p1 + p2 +
    patchwork::plot_annotation(
      title = if (!is.null(title)) title else "Data Quality Overview",
      subtitle = sprintf("Total groups: %d | Stations: %d",
                         nrow(x), length(unique(plot_data$s_id)))
    ) +
    patchwork::plot_layout(widths = c(1, 2))

  print(combined)
  invisible(combined)
}

test_that("summary.pep produces output", {
  data(pep_seed)
  pep <- new_pep(pep_seed)
  expect_output(summary(pep))
})

test_that("summary.pep by=species produces output", {
  data(pep_seed)
  pep <- new_pep(pep_seed)
  expect_output(summary(pep, by = "species"))
})

test_that("summary.pep by=phase produces output", {
  data(pep_seed)
  pep <- new_pep(pep_seed)
  expect_output(summary(pep, by = "phase"))
})

test_that("print.pheno_normals produces output", {
  data(pep_seed)
  pep <- new_pep(pep_seed)
  normals <- pheno_normals(pep, period = 2000:2015, by = "phase_id")
  expect_output(print(normals))
})

test_that("print.pep_quality produces output", {
  data(pep_seed)
  pep <- new_pep(pep_seed)
  quality <- pep_quality(pep, by = c("s_id", "phase_id"))
  expect_output(print(quality))
})

test_that("pep_quality() returns correct class", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, by = c("s_id", "genus", "phase_id"))
  expect_s3_class(result, "pep_quality")
  expect_s3_class(result, "data.table")
})

test_that("pep_quality() has expected output columns", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, by = c("s_id", "genus", "phase_id"))
  expected_cols <- c("n_obs", "completeness_pct", "quality_grade",
                     "year_min", "year_max", "year_span", "n_years",
                     "max_gap_years", "n_outliers", "outlier_pct")
  expect_true(all(expected_cols %in% names(result)))
})

test_that("quality grades are valid A/B/C/D", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, by = c("s_id", "genus", "phase_id"))
  expect_true(all(result$quality_grade %in% c("A", "B", "C", "D")))
})

test_that("completeness_pct is between 0 and 100", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, by = c("s_id", "genus", "phase_id"))
  expect_true(all(result$completeness_pct >= 0 & result$completeness_pct <= 100))
})

test_that("year_span equals year_max - year_min + 1", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, by = c("s_id", "genus", "phase_id"))
  expect_equal(result$year_span, result$year_max - result$year_min + 1L)
})

test_that("n_years <= year_span", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, by = c("s_id", "genus", "phase_id"))
  expect_true(all(result$n_years <= result$year_span))
})

test_that("Grade A criteria verified", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, by = c("s_id", "genus", "phase_id"))
  grade_a <- result[quality_grade == "A"]
  if (nrow(grade_a) > 0) {
    expect_true(all(grade_a$completeness_pct >= 80))
    expect_true(all(grade_a$max_gap_years <= 2))
    expect_true(all(grade_a$outlier_pct < 2))
  }
})

test_that("pep_quality() species and phase_id filters work", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, species = "Malus", phase_id = 60,
                        by = c("s_id"))
  expect_gt(nrow(result), 0)
})

test_that("pep_quality() year_range filter works", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, year_range = c(2000, 2015),
                        by = c("s_id", "genus", "phase_id"))
  expect_true(all(result$year_min >= 2000))
  expect_true(all(result$year_max <= 2015))
})

test_that("both outlier methods (tukey, zscore) work", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  tukey <- pep_quality(p, outlier_method = "tukey", by = c("s_id", "genus", "phase_id"))
  zscore <- pep_quality(p, outlier_method = "zscore", by = c("s_id", "genus", "phase_id"))
  expect_equal(attr(tukey, "outlier_method"), "tukey")
  expect_equal(attr(zscore, "outlier_method"), "zscore")
})

test_that("pep_quality() stores attributes", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pep_quality(p, species = "Malus", phase_id = 60,
                        year_range = c(2000, 2015),
                        by = c("s_id"))
  expect_equal(attr(result, "year_range"), c(2000, 2015))
  expect_equal(attr(result, "species_filter"), "Malus")
  expect_equal(attr(result, "phase_filter"), 60)
})

test_that("pep_quality() errors on invalid input", {
  expect_error(pep_quality("not a df"), "must be a data.frame")
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  expect_error(pep_quality(p, by = c("nonexistent")), "not found")
  expect_error(pep_quality(p, year_range = c(2000)), "numeric vector of length 2")
})

# Prevent R CMD check notes
utils::globalVariables(c("year", "day", "tau_prog", "tau_retr", "is_turning", "..by"))

#' Detect Trend Turning Points in Phenological Time Series
#'
#' Applies a sequential Mann-Kendall test to detect approximate trend turning
#' points in phenological time series. This identifies years where trends
#' may have changed direction (e.g., when spring advancement accelerated or
#' reversed).
#'
#' @param pep A \code{pep} object, data.table, or numeric vector of DOY values.
#'   If a data.frame/data.table, must contain \code{year} and \code{day} columns.
#' @param by Character vector of column names to group by before analysis.
#'   Default is \code{NULL} (analyze all data as one series). Common choices:
#'   \code{c("genus", "species", "phase_id")} or \code{c("country", "phase_id")}.
#' @param min_years Integer. Minimum number of years required for analysis.
#'   Default is 10. Shorter series may produce unreliable results.
#' @param aggregate Logical. If \code{TRUE} (default), aggregates multiple
#'   observations per year (e.g., from multiple stations) using the median.
#'   If \code{FALSE}, expects exactly one value per year.
#'
#' @return An object of class \code{pheno_turning} containing:
#'   \describe{
#'     \item{results}{Data.table with columns: year, day (or median_day),
#'       tau_prog (progressive tau), tau_retr (retrograde tau),
#'       is_turning (logical indicating turning points)}
#'     \item{turning_points}{Years identified as potential turning points}
#'     \item{n_years}{Number of years in the series}
#'     \item{by}{Grouping variables used (if any)}
#'     \item{group_results}{If \code{by} is specified, list of results per group}
#'   }
#'
#' @details
#' The sequential Mann-Kendall test calculates two series:
#' \itemize{
#'   \item \strong{Progressive}: Kendall's tau computed from the start forward
#'   \item \strong{Retrograde}: Kendall's tau computed from the end backward
#' }
#'
#' Points where these series cross indicate potential trend turning points.
#' When either series exceeds confidence thresholds (|tau| > 1.96 for 95%,
#' |tau| > 2.58 for 99%) before and after a crossing, the turning point
#' is considered statistically significant.
#'
#' @section Interpretation:
#' \itemize{
#'   \item Positive tau indicates an increasing trend (later DOY = delayed phenology)
#'   \item Negative tau indicates a decreasing trend (earlier DOY = advanced phenology)
#'   \item Crossing points suggest the trend direction may have changed
#' }
#'
#' @examples
#' # Simple vector input (fast)
#' doy_series <- c(120, 118, 122, 115, 110, 108, 112, 105, 102, 100)
#' turning <- pheno_trend_turning(doy_series)
#' print(turning)
#'
#' # Using pep_seed data (no grouping for speed)
#' data(pep_seed)
#' vine <- pep_seed[pep_seed$species == "Vitis vinifera" &
#'                  pep_seed$phase_id == 65, ]
#' if (nrow(vine) > 0) {
#'   turning <- pheno_trend_turning(vine)
#' }
#'
#' @references
#' Sneyers R (1990). On statistical analysis of series of observations.
#' Technical Note No 143. World Meteorological Organization.
#'
#' @seealso
#' \code{\link{pheno_gradient}} for spatial trend analysis,
#' \code{\link{pheno_normals}} for baseline calculations
#'
#' @author Matthias Templ
#' @export
pheno_trend_turning <- function(pep,
                                 by = NULL,
                                 min_years = 10,
                                 aggregate = TRUE) {

  # Handle numeric vector input
  if (is.numeric(pep) && is.null(dim(pep))) {
    x <- pep
    if (length(x) < min_years) {
      stop(sprintf("Time series too short: %d values (minimum %d required)",
                   length(x), min_years), call. = FALSE)
    }
    mk_result <- sequential_mk(x)
    yearly <- data.table::data.table(
      year = seq_along(x),
      median_day = x,
      n_obs = 1L,
      tau_prog = mk_result$progressive,
      tau_retr = mk_result$retrograde,
      is_turning = mk_result$turning_points
    )
    result <- list(
      results = yearly,
      turning_points = yearly[is_turning == TRUE, year],
      n_years = length(x),
      error = NULL,
      by = NULL,
      group_results = NULL
    )
    class(result) <- c("pheno_turning", "list")
    return(result)
  }

  # Input validation for data.frame
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame, data.table, or numeric vector", call. = FALSE)
  }

  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Check required columns
  if (!all(c("year", "day") %in% names(pep))) {
    stop("Data must contain 'year' and 'day' columns", call. = FALSE)
  }

  # Check grouping columns
  if (!is.null(by)) {
    missing_by <- setdiff(by, names(pep))
    if (length(missing_by) > 0) {
      stop("Column(s) not found: ", paste(missing_by, collapse = ", "), call. = FALSE)
    }
  }

  # Function to analyze a single group
  analyze_group <- function(dt) {
    # Aggregate by year if needed
    if (aggregate) {
      yearly <- dt[, .(median_day = median(day, na.rm = TRUE),
                       n_obs = .N), by = year][order(year)]
    } else {
      yearly <- dt[, .(median_day = day[1], n_obs = 1), by = year][order(year)]
    }

    # Remove NA values
    yearly <- yearly[!is.na(median_day)]

    if (nrow(yearly) < min_years) {
      return(list(
        results = yearly,
        turning_points = integer(0),
        n_years = nrow(yearly),
        error = sprintf("Insufficient years: %d (minimum %d)", nrow(yearly), min_years)
      ))
    }

    # Apply sequential Mann-Kendall
    mk_result <- sequential_mk(yearly$median_day)

    # Combine results
    yearly[, tau_prog := mk_result$progressive]
    yearly[, tau_retr := mk_result$retrograde]
    yearly[, is_turning := mk_result$turning_points]

    list(
      results = yearly,
      turning_points = yearly[is_turning == TRUE, year],
      n_years = nrow(yearly),
      error = NULL
    )
  }

  # Process data
  if (is.null(by)) {
    # Single group analysis
    result <- analyze_group(pep)
    result$by <- NULL
    result$group_results <- NULL
  } else {
    # Grouped analysis
    groups <- unique(pep[, ..by])
    group_results <- list()

    for (i in seq_len(nrow(groups))) {
      group_data <- pep[groups[i, , drop = FALSE], on = by, nomatch = NULL]
      group_label <- paste(sapply(by, function(col) as.character(groups[[col]][i])),
                           collapse = " | ")

      group_results[[group_label]] <- analyze_group(group_data)
      group_results[[group_label]]$group <- groups[i, ]
    }

    # Summarize turning points across groups
    all_turning <- lapply(group_results, function(g) {
      if (length(g$turning_points) > 0) {
        data.table::data.table(
          group = names(group_results)[which(sapply(group_results, identical, g))],
          turning_year = g$turning_points
        )
      }
    })
    all_turning <- data.table::rbindlist(all_turning[!sapply(all_turning, is.null)])

    result <- list(
      results = all_turning,
      turning_points = unique(all_turning$turning_year),
      n_groups = nrow(groups),
      by = by,
      group_results = group_results
    )
  }

  class(result) <- c("pheno_turning", "list")
  result
}


#' Sequential Mann-Kendall Test (Sneyers 1975, internal)
#'
#' Computes the Sneyers (1975) progressive and retrograde standardised
#' rank-count statistics \eqn{u(t)} and \eqn{u'(t)} for a time series, and
#' identifies potential trend turning points as intersections of the two
#' curves inside the non-significance band.
#'
#' Let \eqn{n_i} denote the number of \eqn{j < i} with \eqn{x_i > x_j},
#' and \eqn{t_i = \sum_{k=1}^{i} n_k}. Under the null hypothesis of no
#' trend, \eqn{E[t_i] = i(i-1)/4} and \eqn{\mathrm{Var}[t_i] = i(i-1)(2i+5)/72}.
#' The progressive statistic is
#' \eqn{u(t_i) = (t_i - E[t_i])/\sqrt{\mathrm{Var}[t_i]}}.
#' The retrograde \eqn{u'(t)} is the same procedure applied to the
#' reversed series, then reversed and negated so both curves fluctuate
#' around zero under stationarity.
#'
#' A turning point is flagged where the sign of \eqn{u - u'} changes
#' between two consecutive indices and both curves are within the 95\%
#' non-significance band \eqn{|z| < 1.96} at the crossing.
#'
#' Previous versions of this function (pep725 <= 1.0.2) used
#' \eqn{\mathrm{Var} = i(i-1)(2i+5)/18} with a per-index (non-cumulative)
#' \eqn{S} statistic and an ad-hoc \eqn{|u| > 0.5} threshold for the
#' turning-point filter. That formulation did not match Sneyers' procedure.
#'
#' @param x Numeric vector (time series assumed to be equidistant;
#'   callers should drop or impute NAs beforehand).
#' @return List with \code{progressive}, \code{retrograde} (both length
#'   \code{length(x)}), and \code{turning_points} (logical).
#' @references
#' Sneyers, R. (1975). Sur l'analyse statistique des séries d'observations.
#' WMO Technical Note No. 143, WMO No. 415. Geneva: WMO.
#'
#' Moraes, J. M. et al. (1998). Trends in hydrological parameters of a
#' Southern Brazilian watershed. \emph{Ambio} 27:302-307.
#' @keywords internal
sequential_mk <- function(x) {
  n <- length(x)
  if (n < 4) {
    return(list(
      progressive = rep(NA_real_, n),
      retrograde  = rep(NA_real_, n),
      turning_points = rep(FALSE, n)
    ))
  }

  # Rank-count helper: n_i = number of j < i with x_i > x_j.
  rank_counts <- function(v) {
    m <- length(v)
    out <- integer(m)
    for (i in seq_len(m)) {
      if (i == 1L) next
      out[i] <- sum(v[i] > v[seq_len(i - 1L)])
    }
    out
  }

  i_seq <- seq_len(n)
  E_t <- i_seq * (i_seq - 1) / 4
  V_t <- i_seq * (i_seq - 1) * (2 * i_seq + 5) / 72

  # Progressive series
  t_prog <- cumsum(rank_counts(x))
  prog <- (t_prog - E_t) / sqrt(V_t)
  prog[1] <- 0  # Var is zero at i = 1; by convention the curve starts at 0

  # Retrograde: compute u on the reversed series, then reverse the
  # resulting curve (no negation). Under H0 both u and u' fluctuate
  # around zero; under a monotone trend they diverge in opposite
  # directions and do not cross; at a trend change point (transition
  # from stationary to trending, or vice versa) they intersect inside
  # the 95% confidence band.
  t_retr_rev <- cumsum(rank_counts(rev(x)))
  retr_rev <- (t_retr_rev - E_t) / sqrt(V_t)
  retr_rev[1] <- 0
  retr <- rev(retr_rev)

  # Turning points: u and u' cross inside the 95% non-significance band.
  turning <- rep(FALSE, n)
  diff_pr <- prog - retr
  for (i in 2:(n - 1)) {
    if (!is.finite(diff_pr[i]) || !is.finite(diff_pr[i + 1])) next
    if (diff_pr[i] * diff_pr[i + 1] <= 0 &&
        abs(prog[i]) < 1.96 && abs(retr[i]) < 1.96) {
      turning[i] <- TRUE
    }
  }

  list(
    progressive = prog,
    retrograde  = retr,
    turning_points = turning
  )
}


#' Mann-Kendall Z-Statistic
#'
#' Computes the Mann-Kendall Z-statistic for a time series, a non-parametric
#' test of monotonic trend. The Z-statistic is approximately standard normal
#' under the no-trend null hypothesis.
#'
#' The implementation uses the standard tie correction for \code{Var(S)} (so
#' that tied observations do not inflate the test) and the continuity
#' correction \code{Z = (S - sign(S))/sqrt(Var(S))}, matching
#' \code{Kendall::MannKendall()}.
#'
#' @param x Numeric vector (time series assumed to be equidistant).
#'   \code{NA} values are silently removed.
#'
#' @return Numeric. The Mann-Kendall Z-statistic. Positive values indicate an
#'   increasing trend, negative values decreasing. Compare against standard
#'   normal quantiles (e.g., \code{|Z| > 1.96} for \code{p < 0.05}).
#'   Returns \code{NA} if fewer than three non-missing values.
#'
#' @section Note on \code{kendall_tau()}:
#' Earlier versions of pep725 exported a function called \code{kendall_tau()}
#' that actually returned this Mann-Kendall Z-statistic (not the true
#' Kendall's \eqn{\tau} = \eqn{S / (n(n-1)/2)}). \code{kendall_tau()} is now
#' a deprecated alias of \code{mann_kendall_z()}; please update callers.
#'
#' @references
#' Mann, H.B. (1945). Nonparametric tests against trend. \emph{Econometrica}
#' 13, 245-259.
#'
#' Kendall, M.G. (1975). \emph{Rank Correlation Methods}. 4th ed. London:
#' Charles Griffin.
#'
#' @examples
#' # Decreasing trend (earlier phenology)
#' mann_kendall_z(c(120, 118, 115, 112, 110, 108, 105))
#'
#' # No clear trend
#' mann_kendall_z(c(120, 115, 122, 118, 121, 116, 119))
#'
#' @seealso \code{\link{kendall_tau}} (deprecated alias);
#'   \code{\link{pheno_trend_turning}} for a full sequential Mann-Kendall
#'   analysis.
#' @author Matthias Templ
#' @export
mann_kendall_z <- function(x) {
  x <- x[!is.na(x)]
  n <- length(x)

  if (n < 3) {
    return(NA_real_)
  }

  # Vectorised S = sum over i<j of sign(x_j - x_i).
  #   outer(x, x, "-")[i, j] = x[i] - x[j]
  # The *lower* triangle holds entries with i > j, i.e. sign(x[i] - x[j])
  # for i > j, which after re-labelling is the same multiset of pairs we
  # want (sign(x[j] - x[i]) for i < j).
  d <- outer(x, x, "-")
  s <- sum(sign(d[lower.tri(d)]))

  # Tie correction: if ties are present, reduce Var(S) by
  #   sum over tie groups g of t_g (t_g - 1) (2 t_g + 5) / 18.
  tie_counts <- tabulate(match(x, unique(x)))
  tie_counts <- tie_counts[tie_counts > 1]
  tie_term <- sum(tie_counts * (tie_counts - 1) * (2 * tie_counts + 5)) / 18

  var_s <- (n * (n - 1) * (2 * n + 5) - tie_term * 18) / 18

  # Continuity correction: subtract/add 1 from S before standardising.
  if (s > 0) {
    z <- (s - 1) / sqrt(var_s)
  } else if (s < 0) {
    z <- (s + 1) / sqrt(var_s)
  } else {
    z <- 0
  }

  z
}


#' Mann-Kendall Z-Statistic (deprecated alias for \code{mann_kendall_z})
#'
#' This function was misnamed in pep725 <= 1.0.2: it returned the
#' Mann-Kendall Z-statistic, not Kendall's \eqn{\tau = S / (n(n-1)/2)}.
#' Please call \code{\link{mann_kendall_z}} instead for identical (and
#' better: tie-corrected, continuity-corrected) behaviour.
#'
#' Note that this function is now a thin wrapper and will continue to
#' return the Z-statistic until a future major release.
#'
#' @param x Numeric vector (time series assumed to be equidistant).
#'   \code{NA} values are silently removed.
#' @return Numeric. Identical output shape to \code{mann_kendall_z()}.
#'
#' @examples
#' suppressWarnings(kendall_tau(c(120, 118, 115, 112, 110, 108, 105)))
#'
#' @seealso \code{\link{mann_kendall_z}}
#' @author Matthias Templ
#' @export
kendall_tau <- function(x) {
  .Deprecated(
    new = "mann_kendall_z",
    package = "pep725",
    msg = paste(
      "kendall_tau() is deprecated because it is misnamed: it returns the",
      "Mann-Kendall Z-statistic, not Kendall's tau. Use mann_kendall_z()",
      "for identical (and improved: tie-corrected) behaviour."
    )
  )
  mann_kendall_z(x)
}


#' Print Method for Trend Turning Analysis
#'
#' @param x A \code{pheno_turning} object
#' @param ... Additional arguments (unused)
#' @return Invisibly returns \code{x}
#' @export
print.pheno_turning <- function(x, ...) {
  cat("Phenological Trend Turning Point Analysis\n")
  cat(strrep("=", 50), "\n")

  if (!is.null(x$by)) {
    cat("Grouping: ", paste(x$by, collapse = ", "), "\n")
    cat("Number of groups: ", x$n_groups, "\n\n")

    # Summarize groups with turning points
    groups_with_tp <- sum(sapply(x$group_results, function(g) length(g$turning_points) > 0))
    cat(sprintf("Groups with turning points: %d / %d\n", groups_with_tp, x$n_groups))

    if (nrow(x$results) > 0) {
      cat("\nTurning points detected:\n")
      print(x$results)
    } else {
      cat("\nNo turning points detected.\n")
    }
  } else {
    n_years <- attr(x, "n_years")
    if (is.null(n_years)) n_years <- x$n_years

    cat(sprintf("Years analyzed: %d\n", n_years))
    cat(sprintf("Turning points found: %d\n", length(x$turning_points)))

    if (length(x$turning_points) > 0) {
      cat("Turning point years: ", paste(x$turning_points, collapse = ", "), "\n")
    }

    cat("\nTau statistics:\n")
    cat(sprintf("  Final progressive tau: %.3f\n",
                tail(x$results$tau_prog[!is.na(x$results$tau_prog)], 1)))
    cat(sprintf("  Final retrograde tau: %.3f\n",
                tail(x$results$tau_retr[!is.na(x$results$tau_retr)], 1)))
    cat("\nSignificance thresholds: |tau| > 1.96 (95%), |tau| > 2.58 (99%)\n")
  }

  invisible(x)
}


#' Plot Method for Trend Turning Analysis
#'
#' @param x A \code{pheno_turning} object
#' @param group Character. For grouped analysis, which group to plot.
#'   If \code{NULL} (default), plots the first group or ungrouped results.
#' @param show_thresholds Logical. Show significance threshold lines?
#'   Default \code{TRUE}.
#' @param ... Additional arguments passed to ggplot
#' @return A ggplot object (invisibly)
#' @export
plot.pheno_turning <- function(x, group = NULL, show_thresholds = TRUE, ...) {
  # Get the results to plot
  if (!is.null(x$by) && !is.null(x$group_results)) {
    if (is.null(group)) {
      group <- names(x$group_results)[1]
      message("Plotting first group: ", group)
    }
    if (!group %in% names(x$group_results)) {
      stop("Group '", group, "' not found", call. = FALSE)
    }
    results <- x$group_results[[group]]$results
    title_suffix <- paste0(" (", group, ")")
  } else {
    results <- x$results
    title_suffix <- ""
  }

  if (is.null(results) || nrow(results) == 0) {
    stop("No results to plot", call. = FALSE)
  }

  # Prepare data for plotting
  plot_data <- data.table::melt(
    results,
    id.vars = "year",
    measure.vars = c("tau_prog", "tau_retr"),
    variable.name = "series",
    value.name = "tau"
  )
  plot_data[, series := ifelse(series == "tau_prog", "Progressive", "Retrograde")]

  # Mark turning points
  tp_data <- results[is_turning == TRUE]

  # Create plot
  p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = year, y = tau, color = series)) +
    ggplot2::geom_line(linewidth = 1) +
    ggplot2::geom_point(size = 1.5)

  # Add threshold lines
  if (show_thresholds) {
    p <- p +
      ggplot2::geom_hline(yintercept = c(-1.96, 1.96), linetype = "dashed",
                          color = "gray50", alpha = 0.7) +
      ggplot2::geom_hline(yintercept = c(-2.58, 2.58), linetype = "dotted",
                          color = "gray50", alpha = 0.7)
  }

  # Mark turning points
  if (nrow(tp_data) > 0) {
    p <- p +
      ggplot2::geom_vline(xintercept = tp_data$year, linetype = "dashed",
                          color = "red", alpha = 0.5) +
      ggplot2::annotate("point", x = tp_data$year, y = 0,
                        shape = 18, size = 4, color = "red")
  }

  p <- p +
    ggplot2::scale_color_manual(values = c("Progressive" = "steelblue",
                                            "Retrograde" = "darkorange")) +
    ggplot2::labs(
      title = paste0("Sequential Mann-Kendall Trend Analysis", title_suffix),
      subtitle = if (nrow(tp_data) > 0)
        paste("Turning points:", paste(tp_data$year, collapse = ", "))
      else "No turning points detected",
      x = "Year",
      y = "Normalized Kendall Tau",
      color = "Series"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(legend.position = "bottom")

  print(p)
  invisible(p)
}

test_that("sequential_mk progressive u(t) has E[t] = i(i-1)/4 and Var[t] = i(i-1)(2i+5)/72", {
  # Under H0 (no trend), the rank-count-based statistic t_i has
  #   E[t_i]   = i(i-1)/4
  #   Var[t_i] = i(i-1)(2i+5)/72
  # (Sneyers 1975; Mann-Kendall under the null).
  # For the monotone series 1, 2, ..., n we know n_i = i - 1, so
  #   t_i = sum_{k=1}^{i} (k - 1) = i(i-1)/2
  # which gives u(t_i) = (t_i - E[t_i]) / sqrt(Var[t_i]).
  x <- 1:20
  result <- pep725:::sequential_mk(x)

  i <- seq_along(x)
  t_expected <- i * (i - 1) / 2
  E_t <- i * (i - 1) / 4
  V_t <- i * (i - 1) * (2 * i + 5) / 72
  u_expected <- c(0, (t_expected[-1] - E_t[-1]) / sqrt(V_t[-1]))

  expect_equal(result$progressive, u_expected, tolerance = 1e-8)
})

test_that("sequential_mk final progressive u matches Mann-Kendall Z (without continuity correction)", {
  # At i = n, the Sneyers u(t_n) under no ties equals S / sqrt(Var(S))
  # — i.e. the standard Mann-Kendall Z WITHOUT continuity correction.
  # (mann_kendall_z() applies the (S - sign(S)) continuity correction,
  # so we reconstruct the uncorrected Z by hand here.)
  set.seed(7)
  x <- rnorm(30) + seq_len(30) * 0.05
  result <- pep725:::sequential_mk(x)
  d <- outer(x, x, "-")
  s  <- sum(sign(d[lower.tri(d)]))
  vs <- length(x) * (length(x) - 1) * (2 * length(x) + 5) / 18
  z_uncorr <- s / sqrt(vs)
  expect_equal(result$progressive[length(x)], z_uncorr, tolerance = 1e-8)
})

test_that("sequential_mk retrograde is the forward u on the reversed (then reversed back)", {
  # Moraes et al. 1998 convention: u'(t) = rev(u(rev(x))), no negation.
  # Under this convention, prog and retr fluctuate around zero under H0
  # and diverge symmetrically under a monotone trend.
  set.seed(11)
  x <- rnorm(25)
  r <- pep725:::sequential_mk(x)
  r_rev <- pep725:::sequential_mk(rev(x))
  expect_equal(r$retrograde, rev(r_rev$progressive), tolerance = 1e-8)
})

test_that("sequential_mk detects an injected change point near the correct position", {
  # Flat-with-noise for the first 20 years, then a gentle positive drift
  # in the last 10 years. Under Sneyers, u and u' should cross inside the
  # 95% non-significance band near the change. The signal is deliberately
  # weak so the crossing stays within the band (a strong trend would
  # produce the divergence Sneyers expects, outside the band).
  set.seed(42)
  flat <- 100 + rnorm(20, sd = 2)
  rise <- 100 + seq_len(10) * 0.5 + rnorm(10, sd = 2)
  x <- c(flat, rise)
  result <- pep725:::sequential_mk(x)
  tp_idx <- which(result$turning_points)
  # At least one turning point must be flagged, and it must be within
  # 10 years of the injected change (index 20).
  expect_true(length(tp_idx) >= 1)
  expect_true(any(abs(tp_idx - 20) <= 10))
})

test_that("sequential_mk on a pure monotone trend produces no in-band crossings", {
  # A strong monotone trend must not flag a Sneyers turning point: the
  # curves diverge symmetrically and do not cross inside the band.
  x <- 1:30
  result <- pep725:::sequential_mk(x)
  expect_false(any(result$turning_points))
})

test_that("sequential_mk handles short series by returning NA vectors", {
  r <- pep725:::sequential_mk(c(1, 2, 3))
  expect_true(all(is.na(r$progressive)))
  expect_true(all(is.na(r$retrograde)))
  expect_false(any(r$turning_points))
})

# ---- kendall_tau: deprecated wrapper (kept for backwards compatibility) ----

test_that("kendall_tau is soft-deprecated and returns the old z-stat value", {
  # Pre-1.1.0 semantics: returned the Mann-Kendall Z-statistic, misnamed.
  expect_warning(
    x <- kendall_tau(c(100, 105, 110, 115, 120, 125, 130)),
    "deprecated"
  )
  # Old behavior must continue to work for existing callers.
  expect_true(x > 0)
  expect_warning(
    y <- kendall_tau(c(130, 125, 120, 115, 110, 105, 100)),
    "deprecated"
  )
  expect_true(y < 0)
})

test_that("kendall_tau returns NA for length < 3 (deprecated path)", {
  suppressWarnings({
    expect_true(is.na(kendall_tau(c(1, 2))))
    expect_true(is.na(kendall_tau(c(1))))
    expect_true(is.na(kendall_tau(numeric(0))))
  })
})

# ---- mann_kendall_z: the new, correctly-named Z-statistic ----

test_that("mann_kendall_z matches Kendall::MannKendall Z-statistic", {
  skip_if_not_installed("Kendall")
  set.seed(1)
  x <- c(100, 103, 98, 105, 110, 108, 115, 112, 120, 118)
  ref <- Kendall::MannKendall(x)
  # Kendall::MannKendall returns S and var(S); reconstruct Z with continuity
  # correction (matches our implementation).
  s <- as.numeric(ref$S)
  vs <- as.numeric(ref$varS)
  z_ref <- (s - sign(s)) / sqrt(vs)
  expect_equal(mann_kendall_z(x), z_ref, tolerance = 1e-8)
})

test_that("mann_kendall_z applies tie correction to variance", {
  skip_if_not_installed("Kendall")
  # Ties in phenology data are common (multiple stations same DOY).
  x_with_ties <- c(100, 100, 105, 105, 110, 115, 115, 120)
  ref <- Kendall::MannKendall(x_with_ties)
  vs <- as.numeric(ref$varS)
  s <- as.numeric(ref$S)
  z_ref <- (s - sign(s)) / sqrt(vs)
  expect_equal(mann_kendall_z(x_with_ties), z_ref, tolerance = 1e-6)
})

test_that("mann_kendall_z is sign-correct and NA-safe", {
  expect_true(mann_kendall_z(c(1, 2, 3, 4, 5, 6, 7)) > 0)
  expect_true(mann_kendall_z(c(7, 6, 5, 4, 3, 2, 1)) < 0)
  expect_true(is.na(mann_kendall_z(c(1, 2))))
  expect_true(is.na(mann_kendall_z(numeric(0))))
  # NAs are silently stripped
  expect_equal(
    mann_kendall_z(c(1, 2, NA, 3, 4, 5, 6, 7)),
    mann_kendall_z(c(1, 2, 3, 4, 5, 6, 7)),
    tolerance = 1e-10
  )
})

#' Comprehensive Subspecies Availability Report
#'
#' This function produces a complete report for phenological subspecies
#' availability, combining:
#'   (1) a tidy summary table,
#'   (2) a publication-ready wide table, and
#'   (3) a ggplot2 heatmap showing data presence/absence.
#'
#' Users may specify a \strong{genus}, \strong{species}, or one or more
#' \strong{subspecies}. The report evaluates BBCH phases, regions, years,
#' completeness, and several optional availability metrics.
#'
#' @param pep A PEP725-style data frame containing at least:
#'   `genus`, `species`, `subspecies`, `country`, `year`, `phase_id`,
#'   `s_id`, and `day` or `DOY`.
#' @param genus_name Character (optional). Genus to filter.
#' @param species_name Character (optional). Species to filter.
#' @param subspecies_name Character vector (optional). One or more subspecies.
#' @param phases Integer vector. BBCH phases to evaluate.
#' @param years Numeric vector. Years to include.
#' @param subregions Character vector of region/country names.
#' @param include_empty Logical. If TRUE, include subspecies x country rows
#'   even when no observations are present.
#' @param metric Character vector. Metrics to compute. Use `"all"` for:
#'   `"n_obs"`, `"n_stations"`, `"first_year"`, `"last_year"`,
#'   `"median_doy"`, `"completeness"`.
#' @param bbch_names Optional named list mapping BBCH phase numbers to labels,
#'   e.g. `list("65" = "Flowering", "87" = "Maturity")`.
#' @param return_results Logical. If TRUE, returns a list invisibly.
#'
#' @return Invisibly returns a list with:
#'   \describe{
#'     \item{summary_table}{Tidy long-format summary}
#'     \item{wide_table}{Publication-ready wide-format summary}
#'     \item{heatmap}{A ggplot2 object}
#'   }
#'
#' @seealso \code{\link{pheno_trend_turning}} for trend visualization
#'
#' @author Matthias Templ
#' @export
#' @importFrom dplyr filter mutate select group_by summarise arrange left_join rename coalesce across any_of n n_distinct recode %>%
#' @importFrom tidyr pivot_wider pivot_longer unite
#' @import ggplot2
#' @importFrom stats na.omit
#' @importFrom tibble tibble
#' @importFrom purrr map
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Use Alpine subset for faster computation
#' pep_alpine <- pep[country %in% c("Switzerland", "Austria")]
#'
#' # Grapevine subspecies (wine varieties) - longest historical records
#' if (nrow(pep_alpine[species == "Vitis vinifera"]) > 0) {
#'   subspecies_report(
#'     pep_alpine,
#'     subspecies_name = c("Vitis vinifera Riesling",
#'                         "Vitis vinifera MUELLER THURGAU WEISS"),
#'     subregions = c("Switzerland", "Austria"),
#'     metric = "all",
#'     bbch_names = list("65"="Flowering", "81"="Veraison")
#'   )
#'
#'   subspecies_report(
#'     pep_alpine,
#'     species_name = "Vitis vinifera",
#'     subregions = c("Switzerland", "Austria"),
#'     metric = c("n_obs","median_doy","completeness")
#'   )
#'
#'   subspecies_report(
#'     pep_alpine,
#'     genus_name = "Vitis",
#'     subregions = c("Switzerland", "Austria"),
#'     metric = "all"
#'   )
#' }
#' }
subspecies_report <- function(
    pep,
    genus_name      = NULL,
    species_name    = NULL,
    subspecies_name = NULL,     # may be vector
    phases          = c(65, 87),
    years           = 1961:2024,
    subregions      = c("Austria","Germany-North","Germany-South","Switzerland"),
    include_empty   = TRUE,
    metric          = "all",
    bbch_names      = NULL,
    return_results  = TRUE      # return list invisibly
){
  # --------------------------------------------------------------
  # 1) RUN SUMMARY
  # --------------------------------------------------------------
  summary_tbl <- summarize_subspecies_availability(
    pep             = pep,
    genus_name      = genus_name,
    species_name    = species_name,
    subspecies_name = subspecies_name,
    phases          = phases,
    years           = years,
    subregions      = subregions,
    include_empty   = include_empty,
    metric          = metric
  )

  cat("\n========================\nSUBSPECIES SUMMARY TABLE\n========================\n")
  print(summary_tbl)


  # --------------------------------------------------------------
  # 2) WIDE TABLE
  # --------------------------------------------------------------
  wide_tbl <- format_subspecies_wide_table(
    df         = summary_tbl,
    phases     = phases,
    subregions = subregions,
    metric     = metric,
    bbch_names = bbch_names
  )

  cat("\n========================\nWIDE SUMMARY TABLE\n========================\n")
  print(wide_tbl)


  # --------------------------------------------------------------
  # 3) MISSINGNESS HEATMAP
  # --------------------------------------------------------------
  cat("\n========================\nMISSINGNESS HEATMAP\n========================\n")

  p_heat <- plot_subspecies_missingness_heatmap(
    pep             = pep,
    genus_name      = genus_name,
    species_name    = species_name,
    subspecies_name = subspecies_name,
    phases          = phases,
    subregions      = subregions,
    years           = years,
    bbch_names      = bbch_names
  )

  print(p_heat)

  # --------------------------------------------------------------
  # Return all outputs as a list (invisible)
  # --------------------------------------------------------------
  if (return_results) {
    return(invisible(list(
      summary_table = summary_tbl,
      wide_table    = wide_tbl,
      heatmap       = p_heat
    )))
  }
}


#' Summarize Subspecies Phenological Data Availability
#'
#' Computes data availability metrics for specified genus, species, or
#' subspecies across BBCH phases and regions. Metrics include numbers of
#' observations, station counts, first/last observation years, median DOY,
#' and completeness ratios.
#'
#' @inheritParams subspecies_report
#'
#' @param metric Character vector specifying which metrics to compute.
#'   Allowed values:
#'   `"n_obs"`, `"n_stations"`, `"first_year"`, `"last_year"`,
#'   `"median_doy"`, `"completeness"`, or `"all"`.
#'
#' @return A tibble in long format with one row per `subspecies x country`.
#'
#' @author Matthias Templ
#' @export
#' @importFrom dplyr filter mutate group_by summarise left_join rename across any_of n n_distinct coalesce
#' @importFrom tidyr pivot_wider
#' @importFrom tibble tibble
#' @importFrom stats median
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' pep_alpine <- pep[country %in% c("Switzerland", "Austria")]
#'
#' # Grapevine species (longest historical records)
#' if (nrow(pep_alpine[species == "Vitis vinifera"]) > 0) {
#'   summarize_subspecies_availability(
#'     pep_alpine,
#'     species_name = "Vitis vinifera",
#'     subregions = c("Switzerland", "Austria"),
#'     metric = "all"
#'   )
#'
#'   summarize_subspecies_availability(
#'     pep_alpine,
#'     subspecies_name = c("Vitis vinifera Riesling",
#'                         "Vitis vinifera MUELLER THURGAU WEISS"),
#'     subregions = c("Switzerland", "Austria"),
#'     metric = c("n_obs","median_doy")
#'   )
#' }
#' }
summarize_subspecies_availability <- function(
    pep,
    genus_name      = NULL,
    species_name    = NULL,
    subspecies_name = NULL,   # may be vector
    phases          = c(65, 87),
    years           = 1961:2024,
    subregions      = c("Austria","Germany-North","Germany-South","Switzerland"),
    include_empty   = FALSE,
    metric          = c("n_obs", "n_stations", "first_year",
                        "last_year", "median_doy", "completeness")
){
  # ---------------------------------------------------------------
  # Normalize metric argument
  # ---------------------------------------------------------------
  if (identical(metric, "all")) {
    metric <- c("n_obs","n_stations","first_year","last_year","median_doy","completeness")
  }

  metric <- match.arg(
    metric,
    choices = c("n_obs","n_stations","first_year","last_year","median_doy","completeness"),
    several.ok = TRUE
  )

  # ---------------------------------------------------------------
  # 1) Filter PEP + rename DOY
  # ---------------------------------------------------------------
  pep2 <- pep %>%
    dplyr::rename(DOY = dplyr::any_of("day")) %>%
    dplyr::filter(
      year %in% years,
      country %in% subregions,
      phase_id %in% phases
    )

  # ---------------------------------------------------------------
  # 2) Taxonomy filtering (priority)
  # ---------------------------------------------------------------
  if (!is.null(subspecies_name)) {

    pep2 <- pep2 %>%
      dplyr::filter(!is.na(subspecies) & subspecies %in% subspecies_name)

  } else if (!is.null(species_name)) {

    pep2 <- pep2 %>%
      dplyr::filter(
        species == species_name |
          (!is.na(subspecies) & grepl(paste0("^", species_name), subspecies))
      )

  } else if (!is.null(genus_name)) {

    pep2 <- pep2 %>% dplyr::filter(genus == genus_name)
  }

  pep2 <- pep2 %>% dplyr::filter(!is.na(subspecies))

  # force character phase_id for pivot_wider stability
  pep2 <- pep2 %>% dplyr::mutate(phase_id = as.character(phase_id))

  # ---------------------------------------------------------------
  # 3) Phase-specific metrics (A–E)
  # ---------------------------------------------------------------
  phase_summary <- pep2 %>%
    dplyr::group_by(subspecies, country, phase_id) %>%
    dplyr::summarise(
      n_obs      = if ("n_obs"      %in% metric) dplyr::n()               else NA,
      n_stations = if ("n_stations" %in% metric) dplyr::n_distinct(s_id)  else NA,
      first_year = if ("first_year" %in% metric) min(year, na.rm=TRUE)    else NA,
      last_year  = if ("last_year"  %in% metric) max(year, na.rm=TRUE)    else NA,
      median_DOY = if ("median_doy" %in% metric) median(DOY, na.rm=TRUE)  else NA,
      .groups = "drop"
    ) %>%
    tidyr::pivot_wider(
      names_from  = phase_id,
      values_from = c(n_obs, n_stations, first_year, last_year, median_DOY),
      names_glue  = "{.value}_{phase_id}"
    )

  # Ensure all expected columns exist
  expected_cols <- unlist(lapply(
    c("n_obs","n_stations","first_year","last_year","median_DOY"),
    function(prefix) paste0(prefix, "_", phases)
  ))
  for (cl in expected_cols) {
    if (!cl %in% names(phase_summary)) {
      if (grepl("n_obs|n_stations", cl)) phase_summary[[cl]] <- 0 else phase_summary[[cl]] <- NA
    }
  }

  # ---------------------------------------------------------------
  # 4) Completeness ratio (F)
  # ---------------------------------------------------------------
  if ("completeness" %in% metric) {

    completeness_phase <- pep2 %>%
      dplyr::group_by(subspecies, country, phase_id) %>%
      dplyr::summarise(
        years_present = dplyr::n_distinct(year),
        .groups = "drop"
      ) %>%
      tidyr::pivot_wider(
        names_from  = phase_id,
        values_from = years_present,
        names_glue  = "years_present_{phase_id}"
      )

    # Guarantee columns
    for (ph in phases) {
      nm <- paste0("years_present_", ph)
      if (!nm %in% names(completeness_phase)) completeness_phase[[nm]] <- 0
    }

    # Add completeness ratios
    completeness_phase <- completeness_phase %>%
      dplyr::mutate(
        dplyr::across(
          tidyselect::starts_with("years_present_"),
          ~ . / length(years),
          .names = "completeness_{.col}"
        )
      )

    phase_summary <- phase_summary %>%
      dplyr::left_join(completeness_phase, by = c("subspecies","country"))
  }

  # ---------------------------------------------------------------
  # 5) Optionally include requested subspecies × all countries
  # ---------------------------------------------------------------
  if (!is.null(subspecies_name) && include_empty) {

    full_grid <- expand.grid(
      subspecies = subspecies_name,
      country    = subregions,
      stringsAsFactors = FALSE
    )

    phase_summary <- full_grid %>%
      dplyr::left_join(phase_summary, by = c("subspecies","country"))

    # Replace NA values for numeric metrics with 0
    phase_summary <- phase_summary %>%
      dplyr::mutate(
        dplyr::across(
          tidyselect::matches("n_obs_|n_stations_"),
          ~ dplyr::coalesce(., 0)
        )
      )
  }

  # ---------------------------------------------------------------
  # Final ordering
  # ---------------------------------------------------------------
  phase_summary %>% dplyr::arrange(subspecies, country)
}

#' @keywords internal
#' @noRd
#'
#' @title Internal: Plot Subspecies × Country × Phase Presence Heatmap
#'
#' @description
#' Generates a ggplot2 heatmap showing presence/absence of BBCH phase
#' observations per subspecies and country.
#'
#' @inheritParams subspecies_report
#' @importFrom dplyr filter mutate group_by summarise n_distinct recode
#' @importFrom tidyr pivot_wider
plot_subspecies_missingness_heatmap <- function(
    pep,
    species_name = NULL,
    subspecies_name = NULL,
    genus_name = NULL,
    phases = c(65,87),
    subregions = c("Austria","Germany-North","Germany-South","Switzerland"),
    years = 1961:2024,
    bbch_names = NULL
) {

  # Filter
  pep2 <- pep %>%
    dplyr::filter(
      year %in% years,
      country %in% subregions,
      phase_id %in% phases
    )

  # Apply taxonomy filters
  if (!is.null(subspecies_name)) {
    pep2 <- pep2 %>% dplyr::filter(!is.na(subspecies) & subspecies %in% subspecies_name)

  } else if (!is.null(species_name)) {
    pep2 <- pep2 %>% dplyr::filter(
      species == species_name |
        (!is.na(subspecies) & grepl(paste0("^", species_name), subspecies))
    )

  } else if (!is.null(genus_name)) {
    pep2 <- pep2 %>% dplyr::filter(genus == genus_name)
  }

  # CRITICAL FIX: remove unused subspecies levels
  pep2 <- pep2 %>% dplyr::mutate(subspecies = droplevels(subspecies))

  # Summarize presence/absence
  heat <- pep2 %>%
    dplyr::group_by(subspecies, country, phase_id) %>%
    dplyr::summarise(
      present = as.integer(dplyr::n_distinct(year) > 0),
      .groups = "drop"
    ) %>%
    dplyr::mutate(
      present = factor(present, levels = c(0,1))
    )

  # Drop unused levels again after summarise
  heat <- heat %>% dplyr::mutate(subspecies = droplevels(subspecies))

  # Optional BBCH labels
  if (!is.null(bbch_names)) {
    heat <- heat %>%
      dplyr::mutate(phase_label = dplyr::recode(phase_id, !!!bbch_names))
  } else {
    heat <- heat %>% dplyr::mutate(phase_label = phase_id)
  }

  # Plot
  ggplot2::ggplot(
    heat,
    ggplot2::aes(x = country, y = subspecies, fill = present)
  ) +
    ggplot2::geom_tile(color = "grey30") +
    ggplot2::scale_fill_manual(
      values = c("0" = "white", "1" = "steelblue"),
      breaks = c("0","1"),
      labels = c("Absent","Present"),
      name = "Data"
    ) +
    ggplot2::facet_wrap(~ phase_label, nrow = 1) +
    ggplot2::theme_minimal(14) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      panel.grid = ggplot2::element_blank()
    ) +
    ggplot2::labs(
      title = "Subspecies x Country x BBCH Phase: Data Availability",
      x = "",
      y = ""
    )
}

#' @keywords internal
#' @noRd
#'
#' @title Internal: Format Subspecies Summary as Wide Table
#'
#' @description
#' Converts the output from summarize_subspecies_availability()
#' into a wide publication-ready table.
#'
#' @inheritParams subspecies_report
#' @importFrom dplyr arrange
#' @importFrom tidyr pivot_longer unite pivot_wider
format_subspecies_wide_table <- function(
    df,
    phases = c(65,87),
    subregions = NULL,
    metric = "all",
    bbch_names = NULL  # optional: list mapping phase -> name
) {

  # If user provided BBCH names
  if (!is.null(bbch_names)) {
    names(bbch_names) <- phases
    rename_fun <- function(x) {
      for (ph in names(bbch_names)) {
        x <- gsub(paste0("_", ph), paste0("_", bbch_names[[ph]]), x)
      }
      x
    }
    names(df) <- rename_fun(names(df))
  }

  # If subregions not given, derive from the data
  if (is.null(subregions))
    subregions <- sort(unique(df$country))

  # reshape
  wide <- df %>%
    tidyr::pivot_longer(
      cols = -c(subspecies, country),
      names_to = "metric",
      values_to = "value"
    ) %>%
    tidyr::unite("colname", country, metric, remove = TRUE) %>%
    tidyr::pivot_wider(
      names_from = colname,
      values_from = value
    ) %>%
    dplyr::arrange(subspecies)

  return(wide)
}

#' Calculate Daylength (Photoperiod)
#'
#' Computes the astronomical daylength (hours of daylight) for a given
#' day of year and latitude. Photoperiod is an important environmental
#' driver of plant phenology.
#'
#' @param doy Integer or numeric vector. Day of year (1-365/366).
#' @param lat Numeric. Latitude in decimal degrees. Positive for Northern
#'   Hemisphere, negative for Southern Hemisphere.
#' @param method Character. Daylength model:
#'   \describe{
#'     \item{\code{"brock"}}{(Default) Brock (1981) / Spencer (1971)
#'       simple declination model: \eqn{\delta = 23.45^\circ \cdot
#'       \sin(2\pi(284 + \mathrm{DOY})/365)}. Fast and adequate at
#'       mid-latitudes; differs from CBM by up to a few minutes/day
#'       near the poles.}
#'     \item{\code{"cbm"}}{Forsythe et al. (1995) Climate-Budget-Model
#'       formulation. Includes the eccentricity correction of the Earth's
#'       orbit and is generally the most accurate of the four models
#'       Forsythe et al. compared.}
#'   }
#'
#' @return A list with components:
#'   \describe{
#'     \item{daylength}{Daylength in hours}
#'     \item{declination}{Solar declination angle in degrees}
#'   }
#'   If \code{doy} is a vector, returns a data.frame with these columns.
#'
#' @details
#' Both methods assume a "flat horizon" (sunrise/sunset when the geometric
#' centre of the sun crosses the horizon) and do not account for
#' atmospheric refraction, elevation, or local topography.
#'
#' At polar latitudes (\eqn{|lat| > 66.5^\circ}), continuous daylight or
#' darkness may occur around solstices.
#'
#' @section Phenological Relevance:
#' Many phenological events are triggered by photoperiod thresholds:
#' \itemize{
#'   \item Spring bud burst often requires both temperature accumulation
#'     and minimum daylength
#'   \item Autumn leaf senescence may be triggered by shortening days
#'   \item Flowering in many species is photoperiod-dependent
#' }
#'
#' @examples
#' # Daylength at spring equinox (DOY 80) at 50°N
#' calc_daylength(80, 50)
#'
#' # Summer solstice at different latitudes
#' calc_daylength(172, c(30, 45, 60))
#'
#' # Daylength through the year at 45°N
#' yearly <- calc_daylength(1:365, 45)
#' plot(yearly$doy, yearly$daylength, type = "l",
#'      xlab = "Day of Year", ylab = "Daylength (hours)")
#'
#' @references
#' Spencer, J. W. (1971). Fourier series representation of the position of
#' the sun. \emph{Search} 2(5):172. (Declination formula used by the
#' \code{"brock"} method.)
#'
#' Brock, T. D. (1981). Calculating solar radiation for ecological studies.
#' \emph{Ecological Modelling} 14(1-2):1-19.
#' \doi{10.1016/0304-3800(81)90011-9}
#'
#' Forsythe, W. C., Rykiel, E. J., Stahl, R. S., Wu, H., and
#' Schoolfield, R. M. (1995). A model comparison for daylength as a
#' function of latitude and day of year. \emph{Ecological Modelling}
#' 80(1):87-95. \doi{10.1016/0304-3800(94)00034-F}
#' (CBM daylength model used by the \code{"cbm"} method.)
#'
#' @author Matthias Templ
#' @export
calc_daylength <- function(doy, lat, method = c("brock", "cbm")) {
  method <- match.arg(method)
  # Input validation
  if (!is.numeric(doy) || !is.numeric(lat)) {
    stop("'doy' and 'lat' must be numeric", call. = FALSE)
  }

  if (any(doy < 1 | doy > 366, na.rm = TRUE)) {
    warning("Some DOY values outside 1-366 range", call. = FALSE)
  }

  if (any(abs(lat) > 90, na.rm = TRUE)) {
    stop("Latitude must be between -90 and 90 degrees", call. = FALSE)
  }

  # Solar declination (radians)
  if (method == "brock") {
    # Brock (1981) / Spencer (1971): delta = 23.45° * sin(2π(284+DOY)/365)
    declination_rad <- 23.45 * pi / 180 * sin(2 * pi * (284 + doy) / 365)
  } else {
    # Forsythe et al. (1995) CBM model: revolution-angle correction for
    # the Earth's elliptic orbit.
    #   theta = 0.2163108 + 2 * atan(0.9671396 * tan(0.00860 * (J - 186)))
    #   delta = asin(0.39795 * cos(theta))
    theta <- 0.2163108 +
      2 * atan(0.9671396 * tan(0.00860 * (doy - 186)))
    declination_rad <- asin(0.39795 * cos(theta))
  }

  # Convert latitude to radians
  lat_rad <- lat * pi / 180

  # Hour angle at sunrise/sunset: cos(H) = -tan(lat) * tan(delta)
  cos_hour_angle <- -tan(lat_rad) * tan(declination_rad)

  # Handle polar day/night and NA cases
  daylength <- rep(NA_real_, length(cos_hour_angle))
  not_na <- !is.na(cos_hour_angle)

  # Normal case: sun rises and sets
  normal <- not_na & abs(cos_hour_angle) <= 1
  hour_angle <- acos(pmin(pmax(cos_hour_angle, -1), 1))
  daylength[normal] <- 2 * hour_angle[normal] * 12 / pi

  # Polar day: sun never sets (cos_hour_angle < -1)
  polar_day <- not_na & cos_hour_angle < -1
  daylength[polar_day] <- 24

  # Polar night: sun never rises (cos_hour_angle > 1)
  polar_night <- not_na & cos_hour_angle > 1
  daylength[polar_night] <- 0

  # Convert declination to degrees for output
  declination_deg <- declination_rad * 180 / pi

  # Return format depends on input
  if (length(doy) == 1 && length(lat) == 1) {
    list(
      daylength = daylength,
      declination = declination_deg
    )
  } else {
    data.frame(
      doy = doy,
      lat = lat,
      daylength = daylength,
      declination = declination_deg
    )
  }
}


#' Maximum Daylength at a Latitude
#'
#' Calculates the maximum possible daylength (at summer solstice) for
#' a given latitude.
#'
#' @param lat Numeric. Latitude in decimal degrees.
#'
#' @return Numeric. Maximum daylength in hours.
#'
#' @examples
#' # Maximum daylength at different latitudes
#' calc_max_daylength(c(0, 30, 45, 60, 66.5))
#'
#' @author Matthias Templ
#' @export
calc_max_daylength <- function(lat) {
  # Summer solstice is approximately DOY 172 (June 21)
  result <- calc_daylength(172, lat)

  result$daylength
}


#' Add Daylength to Phenological Data
#'
#' Adds a daylength column to a phenological dataset based on the
#' observation day and station latitude.
#'
#' @param pep A \code{pep} object or data.table with \code{day} and
#'   \code{lat} columns.
#'
#' @return The input data with an added \code{daylength} column (hours).
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' pep <- add_daylength(pep)
#' head(pep[, .(day, lat, daylength)])
#' }
#'
#' @seealso \code{\link{calc_daylength}} for the underlying calculation
#' @author Matthias Templ
#' @export
add_daylength <- function(pep) {
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  if (!all(c("day", "lat") %in% names(pep))) {
    stop("Data must contain 'day' and 'lat' columns", call. = FALSE)
  }

  # Calculate daylength for each observation
  # Use set() to avoid issues with pep class's [ method
  daylength_vals <- calc_daylength(pep$day, pep$lat)$daylength
  data.table::set(pep, j = "daylength", value = daylength_vals)

  pep
}

test_that("add_daylength adds daylength column", {
  data(pep_seed)
  result <- add_daylength(pep_seed)
  expect_true("daylength" %in% names(result))
  expect_true(all(result$daylength > 0 & result$daylength <= 24, na.rm = TRUE))
})

test_that("add_daylength errors when columns missing", {
  df <- data.frame(x = 1:5)
  expect_error(add_daylength(df), "day.*lat")
})

test_that("calc_daylength default (brock) matches the legacy formula", {
  # Pinned numeric: DOY 172 (approx. summer solstice) at 45°N.
  # Current implementation: delta = 23.45° * sin(2π(284 + doy)/365)
  # -> cos(H) = -tan(L)*tan(delta) -> daylength = 24*H/π.
  res <- calc_daylength(172, 45)
  expect_gt(res$daylength, 15)
  expect_lt(res$daylength, 16)
})

test_that("calc_daylength 'cbm' method implements Forsythe (1995) CBM", {
  # CBM near summer solstice at 45°N is about 15.4 hours.
  res <- calc_daylength(172, 45, method = "cbm")
  expect_gt(res$daylength, 15)
  expect_lt(res$daylength, 16)
})

test_that("calc_daylength brock and cbm agree within ~15 min at mid-latitudes", {
  brock <- calc_daylength(1:365, 47, method = "brock")
  cbm   <- calc_daylength(1:365, 47, method = "cbm")
  # Methods should differ by less than ~15 minutes/day at mid-latitudes.
  # (Observed max ~12.4 min at a Swiss latitude; difference grows near
  # the poles.)
  expect_true(max(abs(brock$daylength - cbm$daylength), na.rm = TRUE) < 15/60)
})

test_that("calc_daylength cbm handles polar day / polar night", {
  # 80°N at summer solstice: polar day (24h).
  expect_equal(calc_daylength(172, 80, method = "cbm")$daylength, 24)
  # 80°N at winter solstice: polar night (0h).
  expect_equal(calc_daylength(355, 80, method = "cbm")$daylength, 0)
})

test_that("calc_daylength equinox ~12h for both methods", {
  for (m in c("brock", "cbm")) {
    eq <- calc_daylength(80, c(30, 45, 60), method = m)   # spring equinox
    expect_true(all(abs(eq$daylength - 12) < 0.5))
  }
})

test_that("calc_daylength errors on unknown method", {
  expect_error(calc_daylength(172, 45, method = "bogus"))
})

test_that("calc_daylength returns list for scalar inputs", {
  result <- calc_daylength(80, 50)
  expect_type(result, "list")
  expect_named(result, c("daylength", "declination"))
  expect_true(result$daylength > 0 && result$daylength < 24)
})

test_that("calc_daylength returns data.frame for vector inputs", {
  result <- calc_daylength(1:365, 45)
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 365)
  expect_true(all(c("doy", "lat", "daylength", "declination") %in% names(result)))
})

test_that("calc_daylength equinox is approximately 12h", {
  result <- calc_daylength(80, 0)
  expect_true(abs(result$daylength - 12) < 1)
})

test_that("calc_daylength polar day at high latitude in summer", {
  result <- calc_daylength(172, 70)
  expect_equal(result$daylength, 24)
})

test_that("calc_daylength errors on non-numeric", {
  expect_error(calc_daylength("a", 50), "must be numeric")
})

test_that("calc_daylength errors on lat > 90", {
  expect_error(calc_daylength(100, 95), "between -90 and 90")
})

test_that("calc_max_daylength returns longer days at higher latitudes", {
  low <- calc_max_daylength(30)
  high <- calc_max_daylength(60)
  expect_true(high > low)
})

# Prevent R CMD check notes for data.table columns
utils::globalVariables(c("phase_id", "total_obs", "n_present", "complete"))

#' Check for Expected Phenological Phases
#'
#' Validates that expected phenological phases (BBCH codes) are present in the
#' data. Issues informative warnings for missing phases without stopping
#' execution, making it useful for pipeline validation.
#'
#' @param pep A \code{pep} object or \code{data.table} containing phenological
#'   observations with a \code{phase_id} column.
#' @param expected Integer vector of expected BBCH phase codes. If \code{NULL}
#'   (default), checks for common phases: 10 (leaf), 60 (heading/flowering),
#'   65 (full flowering), and 100 (harvest).
#' @param species Optional character string to filter by species/genus before
#'   checking. If specified, only checks phases for this species.
#' @param year_min Optional integer. Only check phases for years >= this value.
#'   Useful for checking data availability within an analysis window.
#' @param year_max Optional integer. Only check phases for years <= this value.
#' @param warn Logical. If \code{TRUE} (default), issues warnings for missing
#'   phases. If \code{FALSE}, only returns the result silently.
#' @param label Character. Optional label to include in warning messages,
#'   useful when checking multiple datasets. Default is \code{"Data"}.
#'
#' @return A list with class \code{phase_check} containing:
#'   \describe{
#'     \item{expected}{The expected phase codes}
#'     \item{present}{Phase codes found in the data}
#'     \item{missing}{Phase codes not found in the data}
#'     \item{extra}{Phase codes in data but not in expected list}
#'     \item{complete}{Logical. TRUE if all expected phases are present}
#'     \item{n_obs}{Named integer vector of observation counts per present phase}
#'     \item{coverage}{Data.table with phase-level statistics}
#'   }
#'
#' @details
#' This function is designed to be used at the start of analysis pipelines
#' to validate that required phenological phases are available. It issues
#' warnings but does not stop execution, allowing users to be informed of
#' potential issues while still proceeding with available data.
#'
#' @section Common BBCH Phases:
#' \describe{
#'   \item{10}{Leaf development}
#'   \item{60}{Beginning of flowering / Heading}
#'   \item{65}{Full flowering / Anthesis}
#'   \item{69}{End of flowering}
#'   \item{87}{Hard dough (cereals)}
#'   \item{100}{Harvest}
#' }
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Use Swiss subset for faster checking
#' pep_ch <- pep[country == "Switzerland"]
#'
#' # Basic check for common phases
#' check <- pep_check_phases(pep_ch)
#'
#' # Check specific phases for grapevine (longest historical records)
#' check <- pep_check_phases(pep_ch,
#'                       expected = c(60, 65, 81),
#'                       species = "Vitis vinifera")
#'
#' # Check within analysis window
#' check <- pep_check_phases(pep_ch,
#'                       expected = c(60, 65),
#'                       year_min = 1991,
#'                       year_max = 2020)
#'
#' # Use in pipeline with custom label
#' vine_data <- pep_ch[species == "Vitis vinifera"]
#' if (nrow(vine_data) > 0) {
#'   pep_check_phases(vine_data, expected = c(60, 65),
#'                label = "Grapevine analysis")
#' }
#'
#' # Silent check (no warnings)
#' result <- pep_check_phases(pep_ch, warn = FALSE)
#' if (!result$complete) {
#'   # Handle missing phases programmatically
#' }
#' }
#'
#' @seealso
#' \code{\link{bbch_description}} for phase code descriptions,
#' \code{\link{pep_completeness}} for detailed completeness assessment,
#' \code{\link{pep_quality}} for quality grading
#'
#' @author Matthias Templ
#' @export
pep_check_phases <- function(pep,
                          expected = NULL,
                          species = NULL,
                          year_min = NULL,
                          year_max = NULL,
                          warn = TRUE,
                          label = "Data") {

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table if needed
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Check required columns
  if (!"phase_id" %in% names(pep)) {
    stop("'pep' must have a 'phase_id' column", call. = FALSE)
  }

  # Default expected phases (common ones)
  if (is.null(expected)) {
    expected <- c(10, 60, 65, 100)
  }

  # Make a copy for filtering
  dt <- data.table::copy(pep)

  # Apply species filter if specified
  if (!is.null(species)) {
    if ("genus" %in% names(dt)) {
      genus_match <- dt$genus == species
      species_match <- if ("species" %in% names(dt)) dt$species == species else FALSE
      dt <- dt[genus_match | species_match]
    } else if ("species" %in% names(dt)) {
      dt <- dt[dt$species == species]
    }

    if (nrow(dt) == 0) {
      if (warn) {
        warning(sprintf("%s: no observations found for species '%s'",
                        label, species), call. = FALSE)
      }
      return(structure(list(
        expected = expected,
        present = integer(0),
        missing = expected,
        extra = integer(0),
        complete = FALSE,
        n_obs = integer(0),
        coverage = NULL,
        species_filter = species,
        year_filter = c(year_min, year_max)
      ), class = "phase_check"))
    }
  }

  # Apply year filter if specified
  if (!is.null(year_min)) {
    if (!"year" %in% names(dt)) {
      warning("'year' column not found. Ignoring year_min filter.", call. = FALSE)
    } else {
      dt <- dt[year >= year_min]
    }
  }

  if (!is.null(year_max)) {
    if (!"year" %in% names(dt)) {
      warning("'year' column not found. Ignoring year_max filter.", call. = FALSE)
    } else {
      dt <- dt[year <= year_max]
    }
  }

  # Check if any data remains after filtering
  if (nrow(dt) == 0) {
    year_str <- ""
    if (!is.null(year_min) || !is.null(year_max)) {
      year_str <- sprintf(" (years %s-%s)",
                          if (is.null(year_min)) "?" else year_min,
                          if (is.null(year_max)) "?" else year_max)
    }
    if (warn) {
      warning(sprintf("%s: no observations found%s", label, year_str),
              call. = FALSE)
    }
    return(structure(list(
      expected = expected,
      present = integer(0),
      missing = expected,
      extra = integer(0),
      complete = FALSE,
      n_obs = integer(0),
      coverage = NULL,
      species_filter = species,
      year_filter = c(year_min, year_max)
    ), class = "phase_check"))
  }

  # Get phases present in the data
  present_phases <- sort(unique(dt$phase_id))

  # Compare with expected
  missing_phases <- setdiff(expected, present_phases)
  extra_phases <- setdiff(present_phases, expected)
  complete <- length(missing_phases) == 0

  # Count observations per phase
  phase_counts <- dt[, .N, by = phase_id][order(phase_id)]
  n_obs <- stats::setNames(phase_counts$N, phase_counts$phase_id)

  # Create coverage summary
  has_year <- "year" %in% names(dt)
  has_station <- "s_id" %in% names(dt)

  coverage <- dt[, {
    list(
      n_obs = .N,
      n_stations = if (has_station) data.table::uniqueN(s_id) else NA_integer_,
      n_years = if (has_year) data.table::uniqueN(year) else NA_integer_,
      year_min = if (has_year) min(year, na.rm = TRUE) else NA_integer_,
      year_max = if (has_year) max(year, na.rm = TRUE) else NA_integer_
    )
  }, by = phase_id][order(phase_id)]

  # Issue warnings for missing phases
  if (warn && length(missing_phases) > 0) {
    year_str <- ""
    if (!is.null(year_min) || !is.null(year_max)) {
      year_str <- sprintf(" (year %s-%s)",
                          if (is.null(year_min)) min(dt$year) else year_min,
                          if (is.null(year_max)) max(dt$year) else year_max)
    }
    warning(sprintf("%s%s: missing phase(s): %s",
                    label, year_str,
                    paste(missing_phases, collapse = ", ")),
            call. = FALSE)
  }

  # Return result
  result <- list(
    expected = expected,
    present = as.integer(present_phases),
    missing = as.integer(missing_phases),
    extra = as.integer(extra_phases),
    complete = complete,
    n_obs = n_obs,
    coverage = coverage,
    species_filter = species,
    year_filter = c(year_min, year_max)
  )

  class(result) <- "phase_check"
  result
}


#' Print Method for Phase Check Results
#'
#' @param x A \code{phase_check} object
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns \code{x}
#' @author Matthias Templ
#' @export
print.phase_check <- function(x, ...) {
  cat("Phenological Phase Availability Check\n")
  cat(strrep("-", 45), "\n")

  # Filters applied
  if (!is.null(x$species_filter)) {
    cat(sprintf("Species filter: %s\n", x$species_filter))
  }
  if (!is.null(x$year_filter) && !all(is.na(x$year_filter))) {
    year_str <- sprintf("%s - %s",
                        if (is.na(x$year_filter[1])) "any" else x$year_filter[1],
                        if (is.na(x$year_filter[2])) "any" else x$year_filter[2])
    cat(sprintf("Year filter: %s\n", year_str))
  }

  # Expected phases
  cat(sprintf("\nExpected phases: %s\n", paste(x$expected, collapse = ", ")))

  # Status
  if (x$complete) {
    cat("Status: COMPLETE - all expected phases present\n")
  } else {
    cat("Status: INCOMPLETE - some phases missing\n")
  }

  # Missing phases
  if (length(x$missing) > 0) {
    cat(sprintf("\nMissing phases: %s\n", paste(x$missing, collapse = ", ")))
  }

  # Present phases with counts
  if (length(x$present) > 0) {
    cat("\nPhases found:\n")
    for (phase in x$present) {
      in_expected <- if (phase %in% x$expected) "" else " [extra]"
      count <- if (as.character(phase) %in% names(x$n_obs))
        x$n_obs[as.character(phase)] else 0
      cat(sprintf("  Phase %3d: %s observations%s\n",
                  phase, format(count, big.mark = ","), in_expected))
    }
  }

  # Coverage table if available
  if (!is.null(x$coverage) && nrow(x$coverage) > 0) {
    cat("\nDetailed coverage:\n")
    print(x$coverage)
  }

  invisible(x)
}


#' Check Multiple Phases Across Species
#'
#' A convenience function to check phase availability across multiple species
#' at once.
#'
#' @param pep A \code{pep} object or \code{data.table}
#' @param species_list Character vector of species/genus names to check
#' @param expected Integer vector of expected BBCH phase codes
#' @param year_min Optional minimum year filter
#' @param year_max Optional maximum year filter
#' @param warn Logical. Issue warnings for missing phases? Default TRUE.
#'
#' @return A \code{data.table} summarizing phase availability per species
#'
#' @examples
#' \donttest{
#' # Check multiple species (use subset for speed)
#' pep <- pep_download()
#' pep_subset <- pep[country %in% c("Switzerland", "Austria")]
#' result <- pep_check_phases_multi(pep_subset,
#'                              species_list = c("Malus domestica",
#'                                               "Vitis vinifera"),
#'                              expected = c(60, 65))
#' print(result)
#' }
#'
#' @seealso \code{\link{pep_check_phases}} for single-species checking
#'
#' @author Matthias Templ
#' @export
pep_check_phases_multi <- function(pep,
                                species_list,
                                expected = c(60, 65, 100),
                                year_min = NULL,
                                year_max = NULL,
                                warn = TRUE) {

  species_list <- as.character(species_list)

  results <- lapply(species_list, function(sp) {
    check <- pep_check_phases(pep,
                          expected = expected,
                          species = sp,
                          year_min = year_min,
                          year_max = year_max,
                          warn = warn,
                          label = sp)

    data.table::data.table(
      species = sp,
      complete = check$complete,
      n_expected = length(expected),
      n_present = sum(expected %in% check$present),
      n_missing = length(check$missing),
      missing_phases = if (length(check$missing) > 0)
        paste(check$missing, collapse = ",") else NA_character_,
      total_obs = sum(check$n_obs)
    )
  })

  result <- data.table::rbindlist(results)
  data.table::setorder(result, -complete, -n_present, -total_obs)

  class(result) <- c("phase_check_multi", class(result))
  attr(result, "expected") <- expected
  attr(result, "year_range") <- c(year_min, year_max)

  result
}


#' Print Method for Multi-Species Phase Check
#'
#' @param x A \code{phase_check_multi} object
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns \code{x}
#' @author Matthias Templ
#' @export
print.phase_check_multi <- function(x, ...) {
  expected <- attr(x, "expected")
  year_range <- attr(x, "year_range")

  cat("Multi-Species Phase Availability Check\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("Expected phases: %s\n", paste(expected, collapse = ", ")))

  if (!is.null(year_range) && !all(is.na(year_range))) {
    cat(sprintf("Year range: %s - %s\n",
                if (is.na(year_range[1])) "any" else year_range[1],
                if (is.na(year_range[2])) "any" else year_range[2]))
  }

  n_complete <- sum(x$complete)
  cat(sprintf("\nSpecies with all phases: %d / %d (%.0f%%)\n",
              n_complete, nrow(x), 100 * n_complete / nrow(x)))

  cat(strrep("-", 50), "\n\n")
  print(data.table::as.data.table(x))

  invisible(x)
}

test_that("pep_check_phases returns phase_check class", {
  data(pep_seed)
  result <- pep_check_phases(pep_seed, warn = FALSE)
  expect_s3_class(result, "phase_check")
  expect_true("expected" %in% names(result))
  expect_true("present" %in% names(result))
  expect_true("missing" %in% names(result))
  expect_true("complete" %in% names(result))
})

test_that("pep_check_phases identifies present phases correctly", {
  data(pep_seed)
  phases_in_data <- sort(unique(pep_seed$phase_id))
  result <- pep_check_phases(pep_seed, expected = phases_in_data, warn = FALSE)
  expect_true(result$complete)
  expect_length(result$missing, 0)
})

test_that("pep_check_phases detects missing phases", {
  data(pep_seed)
  result <- pep_check_phases(pep_seed, expected = c(999), warn = FALSE)
  expect_false(result$complete)
  expect_equal(result$missing, 999L)
})

test_that("pep_check_phases species filter works", {
  data(pep_seed)
  result <- pep_check_phases(pep_seed, species = "Vitis vinifera", warn = FALSE)
  expect_s3_class(result, "phase_check")
})

test_that("pep_check_phases_multi returns correct structure", {
  data(pep_seed)
  species_list <- unique(pep_seed$species)[1:2]
  result <- pep_check_phases_multi(pep_seed,
                                   species_list = species_list,
                                   expected = c(60, 65),
                                   warn = FALSE)
  expect_s3_class(result, "phase_check_multi")
  expect_equal(nrow(result), length(species_list))
  expect_true("complete" %in% names(result))
})

#' Plot Phenological Time Series for Heading and Harvest
#'
#' Visualizes phenological time series for heading and harvest phases,
#' comparing aggregated and spatially filtered PEP725 data.
#'
#' @param data_list A named list returned by \code{\link{pheno_regional_hh}},
#'   containing \code{ts_tidy}.
#' @param phase_select Character. Optional filter for a specific phenological phase
#'   (e.g., "Heading", "Harvest"). If NULL, all phases are shown.
#' @param alpha_lines Numeric. Transparency level for lines (default is 0.6).
#' @param linewidth Numeric. Line width for time series plots (default is 0.7).
#'
#' @return A \code{ggplot} object.
#'
#' @details
#' This function shows DOY trends by phase and data source across years,
#' with month labels on the y-axis for easier interpretation.
#'
#' @importFrom ggplot2 ggplot aes geom_line facet_grid labs theme_bw theme scale_linetype_manual scale_y_continuous
#'
#' @seealso \code{\link{pheno_regional_hh}}
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' out <- pheno_regional_hh(pep,
#'          species_name = "Triticum aestivum")
#' pheno_plot_hh(out)
#' }
#' @author Matthias Templ
#' @export
pheno_plot_hh <- function(
    data_list,
    phase_select = NULL,
    alpha_lines = 0.6,
    linewidth = 0.7
) {
  # Month scale helpers
  DOY1st  <- c(1, 32, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335)
  DOYmid  <- c(15, 46, 74, 105, 135, 166, 196, 227, 258, 288, 319, 349)
  LABELS  <- c("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec")

  month_scale <- scale_y_continuous(
    limits = c(120, 260),
    breaks = DOYmid, labels = LABELS,
    minor_breaks = DOY1st
  )

  ts_tidy <- data_list$ts_tidy

  if (is.null(ts_tidy)) {
    stop("data_list must contain 'ts_tidy'. Use pheno_regional_hh() to prepare data.")
  }

  # Filter by phase if specified
  if (!is.null(phase_select)) {
    ts_tidy <- ts_tidy[phase == phase_select]
  }

  p <- ggplot(ts_tidy, aes(year, DOY, color = source)) +
    geom_line(aes(linetype = source), linewidth = linewidth, alpha = alpha_lines, na.rm = TRUE) +
    facet_grid(phase ~ panel, scales = "free_y") +
    month_scale +
    labs(x = "", y = NULL, title = "Heading/Harvest phenology") +
    theme_bw() +
    theme(legend.position = "top")

  return(p)
}

#' @importFrom utils data globalVariables head tail txtProgressBar setTxtProgressBar
#' @importFrom stats sd quantile rnorm residuals lm pnorm as.formula complete.cases contr.sum coef predict
#' @importFrom data.table copy setDT fifelse rbindlist fread setcolorder setnames setorderv
.onAttach <- function(libname, pkgname) {
  packageStartupMessage("pep725 loaded.")
 packageStartupMessage("To get phenological data, use:")
  packageStartupMessage("  pep <- pep_download()        # Download synthetic data")
  packageStartupMessage("  data(pep_seed)               # Load small seed dataset")
  packageStartupMessage("  pep <- pep_import('path/')   # Import real PEP725 data")
}

#' Compile Regional Phenological Time Series for Heading and Harvest
#'
#' Generates harmonized phenological time series for heading (BBCH 60) and
#' harvest (BBCH 100) phases of a specified species and spatial region.
#' Optionally integrates GISS climate data for climate-sensitivity studies.
#'
#' @param pep A data.table containing PEP725 phenological data.
#' @param giss Optional. A data.table with GISS global temperature anomalies
#'   (\code{year}, \code{dT}, \code{dT_sm}). If \code{NULL}, GISS-related
#'   outputs are omitted.
#' @param lon_min Minimum longitude for bounding box filter (default is 4.2)
#' @param lon_max Maximum longitude for bounding box filter (default is 8.1)
#' @param lat_min Minimum latitude for bounding box filter (default is 44.7)
#' @param lat_max Maximum latitude for bounding box filter (default is 48.1)
#' @param species_name Name of the species to filter (default is
#'   \code{"Triticum aestivum"})
#' @param year_min Minimum year for all time series outputs (default is 1961)
#' @param pep_for_giss Which PEP725 dataset to use for merging with GISS climate
#'   anomalies. Either \code{"aggregated"} (global) or \code{"near"} (spatially
#'   filtered). Ignored if \code{giss} is \code{NULL}.
#'
#' @return A named list with the following elements:
#' \describe{
#'   \item{\code{ts_tidy}}{Long-format time series with consistent phase and source labeling}
#'   \item{\code{pep_agg}}{Aggregated PEP725 time series (global species-level)}
#'   \item{\code{pep_cgi}}{Spatially filtered PEP725 time series (based on
#'     lat/lon box)}
#'   \item{\code{giss}}{GISS global temperature anomalies (if provided)}
#'   \item{\code{pep_giss}}{PEP time series merged with GISS anomalies (if provided)}
#'   \item{\code{species}}{Species name used for filtering}
#' }
#'
#' @details
#' This function filters the PEP725 dataset by species and optionally by spatial
#' box. It aggregates phenological phases (Heading = 60, Harvest = 100) and
#' computes mean day-of-year (DOY) per year. If GISS data is provided,
#' temperature anomalies are merged with the selected PEP time series to support
#' phenology-climate analysis. Warnings are issued if phases are missing for
#' selected periods.
#'
#' @seealso \code{\link{pep_import}}, \code{\link{pheno_plot_hh}}
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' out <- pheno_regional_hh(pep,
#'          species_name = "Triticum aestivum")
#' pheno_plot_hh(out)
#' }
#' @author Matthias Templ
#' @export
pheno_regional_hh <- function(pep,
                            giss = NULL,
    lon_min = 4.2,  lon_max = 8.1,
    lat_min = 44.7, lat_max = 48.1,
    species_name = "Triticum aestivum",
    year_min   = 1961,
    pep_for_giss = c("aggregated","near")
) {
  pep_for_giss <- match.arg(pep_for_giss)

  # keep species first (faster), then build both aggregated tables
  stopifnot("species" %in% names(pep))
  pep_species <- pep[species %in% species_name]

  # Global PEP aggregation
  PEPData <- pep_species[
    , .(n = .N, mean_day = mean(day, na.rm = TRUE)),
    by = .(species, year, phase_id)
  ][order(species, year, mean_day, phase_id)]

  # Box-filtered (near Changins) PEP aggregation
  pep_box <- pep_species[
    lat > lat_min & lat < lat_max & lon > lon_min & lon < lon_max
  ]
  PEPDataChangins <- pep_box[
    , .(n = .N, mean_day = mean(day, na.rm = TRUE)),
    by = .(species, year, phase_id)
  ][order(species, year, mean_day, phase_id)]

  # --- smarter warning helper: only within analysis window ----------------
  check_phases <- function(dt, label, yr_min) {
    dt_win <- dt[year >= yr_min]
    if (nrow(dt_win) == 0L) {
      warning(sprintf("%s: no rows at or after year_min = %d.", label, yr_min), call. = FALSE)
      return(invisible())
    }
    miss <- setdiff(c("Heading", "Harvest"), unique(dt_win$phase))
    if (length(miss) > 0) {
      warning(sprintf("%s (year >= %d): missing phase(s): %s",
                      label, yr_min, paste(miss, collapse = ", ")), call. = FALSE)
    }
    invisible()
  }

  # --- PEP helper (explicit args; no ..) ---------------------------------
  phase_name <- function(id) ifelse(id == 60, "Heading", "Harvest")
  pepfun <- function(dt, label, sp, yrmin) {
    out <- dt[
      species == sp & year >= yrmin & phase_id %in% c(60, 100),
      .(year, phase = phase_name(phase_id), DOY = mean_day)
    ][order(year)][
      , `:=`(source = label, site = label)
    ][]
    check_phases(out, label, yrmin)  # warn only for analysis window
    out
  }

  pep_agg <- pepfun(PEPData,         "PEP725 (aggregated)",    species_name, year_min)
  pep_cgi <- pepfun(PEPDataChangins, "PEP725 (near Changins)", species_name, year_min)

  # --- One tidy dataset for both time-series panels ----------------------
  ts_tidy <- rbindlist(list(
    copy(pep_agg)[, panel := "PEP aggregated"],
    copy(pep_cgi)[, panel := "PEP near Changins"]
  ), use.names = TRUE)[
    year >= year_min
  ][]

  # --- GISS + merge with selected PEP (optional) -------------------------
  giss_out <- NULL
  pep_giss <- NULL
  if (!is.null(giss)) {
    giss_out <- giss[year >= year_min, .(year, dTgl = dT, Tgl = dT + 14)][order(year)]
    pep_sel <- switch(pep_for_giss, aggregated = pep_agg, near = pep_cgi)
    pep_giss <- merge(pep_sel, giss_out, by = "year")
    attr(pep_giss, "pep_source") <- pep_for_giss
  }

  list(
    ts_tidy  = ts_tidy,
    pep_agg  = pep_agg,
    pep_cgi  = pep_cgi,
    giss     = giss_out,
    pep_giss = pep_giss,
    species  = species_name
  )
}

# Prevent R CMD check notes for data.table columns
utils::globalVariables(c("n_obs", "completeness_pct", ".data"))

#' Assess Species and Phase Completeness of Phenological Data
#'
#' Provides comprehensive assessment of data completeness across species,
#' phases, years, and optionally countries. Useful for identifying data gaps
#' and planning analyses.
#'
#' @param pep A \code{pep} object or \code{data.table} containing phenological
#'   observations with columns \code{year}, \code{day}, \code{genus},
#'   \code{species}, and \code{phase_id}.
#' @param by Character vector specifying grouping dimensions. Options include
#'   \code{"species"}, \code{"genus"}, \code{"phase_id"}, \code{"year"},
#'   \code{"country"}, and \code{"s_id"}. Default is
#'   \code{c("genus", "species", "phase_id")}.
#' @param year_range Optional integer vector of length 2 specifying the start
#'   and end years for assessment. If \code{NULL} (default), uses the full
#'   range of years in the data.
#' @param min_obs Integer. Minimum number of observations required to include

#'   a group in the output. Default is 1.
#' @param include_years Logical. If \code{TRUE}, includes year-level detail
#'   in the output (creates a row per group × year). Default is \code{FALSE}
#'   for summary mode.
#'
#' @return A \code{data.table} of class \code{pep_completeness} with columns:
#'   \describe{
#'     \item{by variables}{Grouping variables as specified in \code{by}}
#'     \item{n_obs}{Total number of observations}
#'     \item{n_stations}{Number of unique stations}
#'     \item{n_years}{Number of years with data}
#'     \item{year_min}{First year of observations}
#'     \item{year_max}{Last year of observations}
#'     \item{year_span}{Total span in years}
#'     \item{completeness_pct}{Percentage of years with data within span}
#'     \item{median_doy}{Median day of year}
#'     \item{iqr_doy}{Interquartile range of day of year}
#'   }
#'
#' @details
#' This function assesses data coverage across multiple dimensions, helping
#' users understand where data is available and where gaps exist. It is
#' particularly useful for:
#' \itemize{
#'   \item Identifying which species × phase combinations have sufficient data
#'   \item Finding temporal gaps in time series
#'   \item Comparing coverage across countries or regions
#'   \item Planning analyses that require minimum observation counts
#' }
#'
#' @section Completeness Calculation:
#' Completeness is calculated as the percentage of years with at least one
#' observation within the year span (year_max - year_min + 1). A completeness
#' of 100% means every year in the span has data.
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Use Swiss subset for faster computation
#' pep_ch <- pep[country == "Switzerland"]
#'
#' # Basic completeness by species and phase
#' comp <- pep_completeness(pep_ch)
#' print(comp)
#'
#' # Filter to well-observed combinations
#' comp_good <- comp[n_obs >= 100 & completeness_pct >= 80]
#'
#' # Completeness by country (use Alpine countries)
#' pep_alpine <- pep[country %in% c("Switzerland", "Austria")]
#' comp_country <- pep_completeness(pep_alpine,
#'                                  by = c("country", "genus", "phase_id"))
#'
#' # Year-level detail for a specific period
#' comp_yearly <- pep_completeness(pep_ch,
#'                                  year_range = c(1991, 2020),
#'                                  include_years = TRUE)
#'
#' # Visualize completeness
#' plot(comp)
#' }
#'
#' @seealso
#' \code{\link{pep_check_phases}} for validating expected phases exist,
#' \code{\link{pep_quality}} for quality grading,
#' \code{\link{pep_coverage}} for overall data coverage
#'
#' @author Matthias Templ
#' @export
pep_completeness <- function(pep,
                              by = c("genus", "species", "phase_id"),
                              year_range = NULL,
                              min_obs = 1,
                              include_years = FALSE) {

  # Input validation

if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table if needed
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Check required columns
  required_cols <- c("year", "day")
  missing_cols <- setdiff(required_cols, names(pep))
  if (length(missing_cols) > 0) {
    stop("Missing required columns: ", paste(missing_cols, collapse = ", "),
         call. = FALSE)
  }

  # Check that 'by' columns exist
  missing_by <- setdiff(by, names(pep))
  if (length(missing_by) > 0) {
    stop("Column(s) specified in 'by' not found: ",
         paste(missing_by, collapse = ", "),
         call. = FALSE)
  }

  # Make a copy to avoid modifying original
  dt <- data.table::copy(pep)

  # Apply year range filter if specified
  if (!is.null(year_range)) {
    if (!is.numeric(year_range) || length(year_range) != 2) {
      stop("'year_range' must be a numeric vector of length 2", call. = FALSE)
    }
    dt <- dt[year >= year_range[1] & year <= year_range[2]]
    if (nrow(dt) == 0) {
      stop("No observations found in year range: ",
           year_range[1], "-", year_range[2], call. = FALSE)
    }
  }

  # Determine if s_id exists for station counting
  has_stations <- "s_id" %in% names(dt)

  if (include_years) {
    # Year-level detail mode
    by_with_year <- c(by, "year")

    result <- dt[, {
      list(
        n_obs = as.integer(.N),
        n_stations = if (has_stations) data.table::uniqueN(s_id) else NA_integer_,
        median_doy = as.numeric(round(median(day, na.rm = TRUE), 1)),
        iqr_doy = as.numeric(round(stats::IQR(day, na.rm = TRUE), 1))
      )
    }, by = by_with_year]

  } else {
    # Summary mode (default)
    result <- dt[, {
      years_present <- unique(year)
      n_years_present <- length(years_present)
      year_min <- min(year, na.rm = TRUE)
      year_max <- max(year, na.rm = TRUE)
      year_span <- year_max - year_min + 1
      completeness <- if (year_span > 0) 100 * n_years_present / year_span else 0

      list(
        n_obs = as.integer(.N),
        n_stations = if (has_stations) data.table::uniqueN(s_id) else NA_integer_,
        n_years = as.integer(n_years_present),
        year_min = as.integer(year_min),
        year_max = as.integer(year_max),
        year_span = as.integer(year_span),
        completeness_pct = as.numeric(round(completeness, 1)),
        median_doy = as.numeric(round(median(day, na.rm = TRUE), 1)),
        iqr_doy = as.numeric(round(stats::IQR(day, na.rm = TRUE), 1))
      )
    }, by = by]
  }

  # Apply minimum observation filter
  result <- result[n_obs >= min_obs]

  # Order by grouping variables
  data.table::setorderv(result, by)

  # Add class for method dispatch
  class(result) <- c("pep_completeness", class(result))

  # Store parameters as attributes
  attr(result, "by") <- by
  attr(result, "year_range") <- year_range
  attr(result, "min_obs") <- min_obs
  attr(result, "include_years") <- include_years

  result
}


#' Print Method for Completeness Assessment
#'
#' @param x A \code{pep_completeness} object
#' @param n Number of rows to display
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns \code{x}
#' @author Matthias Templ
#' @export
print.pep_completeness <- function(x, n = 15, ...) {
  by <- attr(x, "by")
  year_range <- attr(x, "year_range")
  min_obs <- attr(x, "min_obs")
  include_years <- attr(x, "include_years")

  cat("Phenological Data Completeness Assessment\n")
  cat(strrep("-", 50), "\n")

  cat(sprintf("Grouping by: %s\n", paste(by, collapse = ", ")))

  if (!is.null(year_range) && length(year_range) == 2) {
    cat(sprintf("Year range: %d-%d\n", year_range[1], year_range[2]))
  }

  if (min_obs > 1) {
    cat(sprintf("Minimum observations: %d\n", min_obs))
  }

  cat(sprintf("Total groups: %d\n", nrow(x)))

  if (!include_years && "completeness_pct" %in% names(x)) {
    # Summary statistics
    cat(sprintf("Mean completeness: %.1f%%\n", mean(x$completeness_pct, na.rm = TRUE)))
    cat(sprintf("Groups with >80%% completeness: %d (%.1f%%)\n",
                sum(x$completeness_pct >= 80, na.rm = TRUE),
                100 * sum(x$completeness_pct >= 80, na.rm = TRUE) / nrow(x)))
  }

  cat(strrep("-", 50), "\n\n")

  # Print as data.table
  print(data.table::as.data.table(x)[1:min(n, nrow(x))])

  if (nrow(x) > n) {
    cat(sprintf("\n... and %d more rows\n", nrow(x) - n))
  }

  invisible(x)
}


#' Summary Method for Completeness Assessment
#'
#' @param object A \code{pep_completeness} object
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns a summary list
#' @author Matthias Templ
#' @export
summary.pep_completeness <- function(object, ...) {
  by <- attr(object, "by")
  include_years <- attr(object, "include_years")

  cat("Phenological Data Completeness Summary\n")
  cat(strrep("=", 50), "\n\n")

  cat(sprintf("Total groups: %d\n", nrow(object)))
  cat(sprintf("Total observations: %s\n", format(sum(object$n_obs), big.mark = ",")))

  if ("n_stations" %in% names(object) && !all(is.na(object$n_stations))) {
    cat(sprintf("Total unique stations: %s\n",
                format(sum(object$n_stations, na.rm = TRUE), big.mark = ",")))
  }

  if (!include_years && "completeness_pct" %in% names(object)) {
    cat("\nCompleteness Distribution:\n")
    breaks <- c(0, 20, 40, 60, 80, 100)
    labels <- c("0-20%", "20-40%", "40-60%", "60-80%", "80-100%")
    bins <- cut(object$completeness_pct, breaks = breaks, labels = labels,
                include.lowest = TRUE, right = TRUE)
    bin_table <- table(bins)
    for (i in seq_along(bin_table)) {
      pct <- 100 * bin_table[i] / nrow(object)
      bar <- paste(rep("*", round(pct / 5)), collapse = "")
      cat(sprintf("  %s: %4d (%5.1f%%) %s\n", labels[i], bin_table[i], pct, bar))
    }

    cat("\nYear Coverage Statistics:\n")
    cat(sprintf("  Mean year span: %.1f years\n", mean(object$year_span, na.rm = TRUE)))
    cat(sprintf("  Mean years with data: %.1f\n", mean(object$n_years, na.rm = TRUE)))
    cat(sprintf("  Earliest data: %d\n", min(object$year_min, na.rm = TRUE)))
    cat(sprintf("  Latest data: %d\n", max(object$year_max, na.rm = TRUE)))
  }

  cat("\nObservation Statistics:\n")
  cat(sprintf("  Mean observations per group: %.1f\n", mean(object$n_obs, na.rm = TRUE)))
  cat(sprintf("  Median observations per group: %.0f\n", median(object$n_obs, na.rm = TRUE)))
  cat(sprintf("  Max observations: %s\n", format(max(object$n_obs), big.mark = ",")))

  if ("median_doy" %in% names(object)) {
    cat("\nPhenology Statistics:\n")
    cat(sprintf("  DOY range: %.0f - %.0f\n",
                min(object$median_doy, na.rm = TRUE),
                max(object$median_doy, na.rm = TRUE)))
  }

  invisible(list(
    n_groups = nrow(object),
    total_obs = sum(object$n_obs),
    mean_completeness = if ("completeness_pct" %in% names(object))
      mean(object$completeness_pct, na.rm = TRUE) else NA
  ))
}


#' Plot Method for Completeness Assessment
#'
#' Creates visualizations of data completeness, including heatmaps and
#' bar charts.
#'
#' @param x A \code{pep_completeness} object
#' @param type Character. Type of plot: \code{"heatmap"} for completeness
#'   heatmap (requires 2 grouping variables), \code{"bar"} for observation
#'   counts, \code{"timeline"} for year coverage. Default is \code{"heatmap"}.
#' @param top Integer. Number of top groups to display. Default is 20.
#' @param ... Additional arguments (unused)
#'
#' @return A ggplot object (invisibly)
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' pep_ch <- pep[country == "Switzerland"]
#' comp <- pep_completeness(pep_ch, by = c("genus", "phase_id"))
#' plot(comp, type = "heatmap")
#' plot(comp, type = "bar", top = 15)
#' }
#'
#' @author Matthias Templ
#' @export
plot.pep_completeness <- function(x, type = c("heatmap", "bar", "timeline"),
                                   top = 20, ...) {
  type <- match.arg(type)
  by <- attr(x, "by")
  include_years <- attr(x, "include_years")

  if (type == "heatmap") {
    # Need at least 2 grouping variables for heatmap
    if (length(by) < 2) {
      warning("Heatmap requires at least 2 grouping variables. Switching to 'bar' plot.",
              call. = FALSE)
      type <- "bar"
    }
  }

  if (type == "heatmap" && !include_years) {
    # Create species × phase heatmap
    dt <- data.table::as.data.table(x)

    # Use first two 'by' variables for axes
    x_var <- by[1]
    y_var <- by[2]

    # Get top groups by observation count
    if (nrow(dt) > top * top) {
      top_x <- dt[, .(total = sum(n_obs)), by = x_var][order(-total)][1:min(top, .N)][[x_var]]
      top_y <- dt[, .(total = sum(n_obs)), by = y_var][order(-total)][1:min(top, .N)][[y_var]]
      dt <- dt[get(x_var) %in% top_x & get(y_var) %in% top_y]
    }

    # Aggregate if there are more grouping variables
    if (length(by) > 2) {
      dt <- dt[, .(
        n_obs = sum(n_obs),
        completeness_pct = mean(completeness_pct, na.rm = TRUE)
      ), by = c(x_var, y_var)]
    }

    fill_var <- if ("completeness_pct" %in% names(dt)) "completeness_pct" else "n_obs"
    fill_label <- if (fill_var == "completeness_pct") "Completeness (%)" else "Observations"

    p <- ggplot2::ggplot(dt, ggplot2::aes(
      x = .data[[x_var]],
      y = .data[[y_var]],
      fill = .data[[fill_var]]
    )) +
      ggplot2::geom_tile(color = "white", linewidth = 0.5) +
      ggplot2::scale_fill_viridis_c(option = "C", na.value = "grey90") +
      ggplot2::labs(
        title = "Data Completeness Heatmap",
        x = x_var,
        y = y_var,
        fill = fill_label
      ) +
      ggplot2::theme_minimal(base_size = 11) +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, vjust = 1),
        panel.grid = ggplot2::element_blank()
      )

  } else if (type == "bar") {
    dt <- data.table::as.data.table(x)[order(-n_obs)][1:min(top, nrow(x))]

    # Create label from grouping variables
    if (length(by) == 1) {
      dt$label <- as.character(dt[[by[1]]])
    } else {
      dt$label <- do.call(paste, c(dt[, ..by], sep = " | "))
    }
    dt$label <- factor(dt$label, levels = rev(dt$label))

    p <- ggplot2::ggplot(dt, ggplot2::aes(x = n_obs, y = label)) +
      ggplot2::geom_col(fill = "steelblue", alpha = 0.8) +
      ggplot2::labs(
        title = sprintf("Top %d Groups by Observation Count", min(top, nrow(x))),
        x = "Number of Observations",
        y = ""
      ) +
      ggplot2::theme_minimal(base_size = 11)

  } else if (type == "timeline") {
    if (!include_years && "year_min" %in% names(x) && "year_max" %in% names(x)) {
      dt <- data.table::as.data.table(x)[order(-n_obs)][1:min(top, nrow(x))]

      # Create label
      if (length(by) == 1) {
        dt$label <- as.character(dt[[by[1]]])
      } else {
        dt$label <- do.call(paste, c(dt[, ..by], sep = " | "))
      }
      dt$label <- factor(dt$label, levels = rev(dt$label))

      p <- ggplot2::ggplot(dt, ggplot2::aes(y = label)) +
        ggplot2::geom_segment(
          ggplot2::aes(x = year_min, xend = year_max, yend = label),
          linewidth = 3, color = "steelblue", alpha = 0.7
        ) +
        ggplot2::geom_point(ggplot2::aes(x = year_min), size = 2, color = "steelblue") +
        ggplot2::geom_point(ggplot2::aes(x = year_max), size = 2, color = "steelblue") +
        ggplot2::labs(
          title = "Temporal Coverage by Group",
          x = "Year",
          y = ""
        ) +
        ggplot2::theme_minimal(base_size = 11)
    } else {
      warning("Timeline plot requires summary mode with year_min/year_max columns.",
              call. = FALSE)
      return(invisible(NULL))
    }
  }

  print(p)
  invisible(p)
}

test_that("pep_completeness returns pep_completeness class", {
  data(pep_seed)
  result <- pep_completeness(pep_seed)
  expect_s3_class(result, "pep_completeness")
})

test_that("pep_completeness has expected columns", {
  data(pep_seed)
  result <- pep_completeness(pep_seed)
  expected_cols <- c("n_obs", "n_stations", "n_years", "year_min", "year_max",
                     "year_span", "completeness_pct", "median_doy", "iqr_doy")
  expect_true(all(expected_cols %in% names(result)))
})

test_that("pep_completeness year_range filter works", {
  data(pep_seed)
  result <- pep_completeness(pep_seed, year_range = c(2000, 2010))
  expect_true(all(result$year_min >= 2000))
  expect_true(all(result$year_max <= 2010))
})

test_that("pep_completeness min_obs filter works", {
  data(pep_seed)
  result <- pep_completeness(pep_seed, min_obs = 50)
  expect_true(all(result$n_obs >= 50))
})

test_that("pep_completeness include_years adds year-level detail", {
  data(pep_seed)
  result <- pep_completeness(pep_seed, include_years = TRUE)
  expect_true("year" %in% names(result))
})

#' Plot Phenological Time Series
#'
#' Generates time series plots from processed phenology data. Shows DOY trends
#' with faceting by phenophase and spatial scope.
#'
#' @param data_list A named list of prepared data objects, typically output from
#'   \code{\link{pheno_regional}}. Must contain element \code{ts_tidy}.
#' @param alpha_lines Alpha transparency for time series lines (default is \code{0.6}).
#' @param linewidth Line width for time series lines (default is \code{0.7}).
#'
#' @return A \code{ggplot} object for visual inspection and further customization.
#'
#' @details
#' This function visualizes DOY trends with faceting by phenophase and spatial scope.
#'
#' @seealso \code{\link{pheno_regional}}
#'
#' @examples
#' # Construct a minimal data_list with ts_tidy
#' ts_tidy <- data.frame(
#'   year = rep(1990:1995, 2),
#'   DOY = c(120, 118, 115, 119, 113, 110, 125, 122, 120, 124, 118, 115),
#'   source = rep(c("PEP725 (aggregated)", "PEP725 (near Changins)"), each = 6),
#'   phase = "Heading",
#'   panel = "DOY"
#' )
#' pheno_plot(list(ts_tidy = ts_tidy))
#'
#' @import ggplot2
#' @author Matthias Templ
#' @export
pheno_plot <- function(
    data_list,
    alpha_lines = 0.6,
    linewidth = 0.7
) {
  ts_tidy <- data_list$ts_tidy

  if (is.null(ts_tidy)) {
    stop("data_list must contain 'ts_tidy'. Use pheno_regional() to prepare data.")
  }

  p <- ggplot(ts_tidy, aes(year, DOY, color = source)) +
    geom_line(aes(linetype = source), linewidth = linewidth, alpha = alpha_lines, na.rm = TRUE) +
    facet_grid(phase ~ panel, scales = "free_y") +
    labs(x = "", y = NULL, title = "Phenology time series") +
    theme_bw() +
    theme(legend.position = "top") +
    scale_linetype_manual(values = c(
      "PEP725 (aggregated)"       = "solid",
      "PEP725 (near Changins)"    = "solid"
    ))

  return(p)
}

#' Calculate Thermal Units (Growing Degree Days)
#'
#' Computes accumulated thermal units (growing degree days, GDD) from daily
#' temperature data. Thermal time is a fundamental concept in phenology,
#' representing the heat accumulation that drives plant development.
#'
#' @param tmin Numeric vector of daily minimum temperatures (°C).
#' @param tmax Numeric vector of daily maximum temperatures (°C). If \code{NULL},
#'   \code{tmin} is treated as daily mean temperature.
#' @param t_base Numeric. Base temperature (°C) below which no development occurs.
#'   Default is 5°C, typical for temperate crops.
#' @param t_cap Numeric. Optional upper temperature cap (°C). Temperatures above
#'   this are set to \code{t_cap} before GDD calculation. Default \code{NULL}
#'   (no cap). Use ~30°C for crops where high heat doesn't increase development.
#' @param method Character. Calculation method:
#'   \describe{
#'     \item{"average"}{(Default) GDD = max(0, (tmax + tmin)/2 - t_base)}
#'     \item{"modified"}{Sets negative daily GDD to 0 before averaging}
#'     \item{"single_sine"}{Sine wave approximation for more accurate heat units}
#'   }
#' @param cumulative Logical. If \code{TRUE} (default), returns cumulative sum.
#'   If \code{FALSE}, returns daily values.
#' @param na.rm Logical. Remove NA values in accumulation? Default \code{TRUE}.
#'
#' @return Numeric vector of thermal units (degree-days). Same length as input.
#'   If \code{cumulative = TRUE}, returns running sum.
#'
#' @details
#' Growing Degree Days (GDD) quantify heat accumulation above a base temperature.
#' Plants require a certain thermal sum to reach phenological stages (e.g.,
#' flowering typically requires 500-1500 GDD depending on species).
#'
#' The base temperature represents the minimum temperature for growth:
#' \itemize{
#'   \item Cool-season crops (wheat, barley): 0-5°C
#'   \item Warm-season crops (maize, soybean): 10°C
#'   \item Fruit trees (apple, cherry): 4-7°C
#'   \item Grapevine: 10°C
#' }
#'
#' @section Methods:
#' \describe{
#'   \item{average}{Simple average: GDD = max(0, (Tmax + Tmin)/2 - Tbase).
#'     Most common method, used by many agricultural agencies.}
#'   \item{modified}{Like average, but Tmin and Tmax are first bounded to
#'     the range from Tbase to Tcap before averaging. Prevents negative contributions.}
#'   \item{single_sine}{Approximates the daily temperature curve as a sine
#'     wave, computing the mean excess over Tbase. More accurate when
#'     temperatures cross the base threshold during the day. Implements the
#'     Baskerville-Emin (1969) / Snyder (1985) single-sine formula: with
#'     \eqn{T_{avg} = (T_{min} + T_{max})/2}, \eqn{T_{amp} = (T_{max} - T_{min})/2},
#'     and \eqn{\theta = \mathrm{acos}\left((T_{base} - T_{avg})/T_{amp}\right)},
#'     the daily GDD in the mixed case
#'     (\eqn{T_{min} < T_{base} < T_{max}}) is
#'     \eqn{\mathrm{GDD} = \frac{1}{\pi}\left[(T_{avg} - T_{base})\,\theta +
#'       T_{amp}\,\sin(\theta)\right]}.}
#' }
#'
#' @examples
#' # Daily temperatures for a week
#' tmin <- c(5, 7, 8, 10, 12, 11, 9)
#' tmax <- c(15, 18, 20, 22, 25, 23, 19)
#'
#' # Cumulative GDD with base 10°C
#' calc_thermal_units(tmin, tmax, t_base = 10)
#'
#' # Daily GDD values
#' calc_thermal_units(tmin, tmax, t_base = 10, cumulative = FALSE)
#'
#' # With upper cap at 30°C
#' calc_thermal_units(tmin, tmax, t_base = 10, t_cap = 30)
#'
#' # Using mean temperature only
#' tmean <- (tmin + tmax) / 2
#' calc_thermal_units(tmean, t_base = 10)
#'
#' @references
#' McMaster, G.S., Wilhelm, W.W. (1997). Growing degree-days: one equation,
#' two interpretations. \emph{Agricultural and Forest Meteorology} 87:291-300.
#'
#' Baskerville, G.L., Emin, P. (1969). Rapid estimation of heat accumulation
#' from maximum and minimum temperatures. \emph{Ecology} 50:514-517.
#'
#' Snyder, R.L. (1985). Hand calculating degree days. \emph{Agricultural and
#' Forest Meteorology} 35:353-358.
#'
#' @seealso \code{\link{calc_thermal_sum}} for computing thermal sum at
#'   phenological observations
#'
#' @author Matthias Templ
#' @export
calc_thermal_units <- function(tmin,
                                tmax = NULL,
                                t_base = 5,
                                t_cap = NULL,
                                method = c("average", "modified", "single_sine"),
                                cumulative = TRUE,
                                na.rm = TRUE) {

  method <- match.arg(method)

  # Input validation

  if (!is.numeric(tmin)) {
    stop("'tmin' must be numeric", call. = FALSE)
  }

  # If tmax not provided, treat tmin as mean temperature
  if (is.null(tmax)) {
    tmean <- tmin
    tmax <- tmin
    tmin_orig <- tmin
  } else {
    if (!is.numeric(tmax)) {
      stop("'tmax' must be numeric", call. = FALSE)
    }
    if (length(tmin) != length(tmax)) {
      stop("'tmin' and 'tmax' must have same length", call. = FALSE)
    }
    tmean <- (tmin + tmax) / 2
    tmin_orig <- tmin
  }

  # Apply temperature cap if specified
  if (!is.null(t_cap)) {
    tmax <- pmin(tmax, t_cap)
    tmin <- pmin(tmin, t_cap)
    tmean <- (tmin + tmax) / 2
  }

  n <- length(tmin)
  gdd <- numeric(n)

  if (method == "average") {
    # Simple averaging method
    # GDD = max(0, (Tmax + Tmin)/2 - Tbase)
    gdd <- pmax(0, tmean - t_base)

  } else if (method == "modified") {
    # Modified method - bound temperatures first
    tmin_adj <- pmax(tmin, t_base)
    tmax_adj <- pmax(tmax, t_base)
    if (!is.null(t_cap)) {
      tmin_adj <- pmin(tmin_adj, t_cap)
      tmax_adj <- pmin(tmax_adj, t_cap)
    }
    gdd <- (tmax_adj + tmin_adj) / 2 - t_base

  } else if (method == "single_sine") {
    # Single sine wave approximation
    # More accurate when temperature crosses base during day
    for (i in seq_len(n)) {
      if (is.na(tmin_orig[i]) || is.na(tmax[i])) {
        gdd[i] <- NA
        next
      }

      t_min_i <- tmin_orig[i]
      t_max_i <- tmax[i]
      t_avg <- (t_min_i + t_max_i) / 2
      t_amp <- (t_max_i - t_min_i) / 2

      if (t_min_i >= t_base) {
        # Entire day above base
        gdd[i] <- t_avg - t_base
      } else if (t_max_i <= t_base) {
        # Entire day below base
        gdd[i] <- 0
      } else {
        # Temperature crosses base - use the Baskerville-Emin (1969)
        # single-sine integration. Model: T(t) = t_avg + t_amp * cos(t)
        # over one full day (t in [-pi, pi]); T > t_base for t in [-theta,
        # theta] where theta = acos((t_base - t_avg) / t_amp). Daily GDD
        # is the mean excess over t_base:
        #   GDD = (1 / pi) * [ (t_avg - t_base) * theta
        #                      + t_amp * sin(theta) ]
        # See Snyder (1985) and UC-IPM degree-day documentation.
        theta <- acos((t_base - t_avg) / t_amp)
        gdd[i] <- (1 / pi) * ((t_avg - t_base) * theta +
                                t_amp * sin(theta))
        gdd[i] <- max(0, gdd[i])
      }
    }
  }

  # Return cumulative or daily values
  if (cumulative) {
    if (na.rm) {
      gdd[is.na(gdd)] <- 0
    }
    gdd <- cumsum(gdd)
  }

  gdd
}


#' Calculate Thermal Sum at Phenological Events
#'
#' Computes the accumulated thermal units (GDD) from a start date to the
#' observed phenological event date. Useful for determining the thermal
#' requirements of different phenophases.
#'
#' @param pep A \code{pep} object or data.table with phenological observations.
#'   Must contain \code{year} and \code{day} columns.
#' @param temp_data A data.frame or data.table with daily temperature data.
#'   Must contain columns: \code{year}, \code{doy} (day of year),
#'   \code{tmin}, \code{tmax} (or \code{tmean}).
#' @param t_base Numeric. Base temperature (°C). Default 5.
#' @param t_start Integer. Start day of year for accumulation. Default 1 (Jan 1).
#'   Use 60 for March 1, etc.
#' @param by Character vector. Columns to match temperature data to phenology
#'   (e.g., \code{"s_id"} for station-specific temperatures). Default \code{NULL}
#'   uses same temperature for all observations in a year.
#' @param method Character. GDD calculation method (see \code{\link{calc_thermal_units}}).
#'
#' @return The input data with an additional \code{thermal_sum} column containing
#'   the accumulated GDD from \code{t_start} to the observed DOY.
#'
#' @details
#' This function joins phenological observations with temperature data and
#' calculates the thermal sum (accumulated GDD) at each observation date.
#' This is useful for:
#' \itemize{
#'   \item Determining thermal requirements of phenophases
#'   \item Comparing thermal sums across years, locations, or species
#'   \item Validating phenological models
#' }
#'
#' @examples
#' \donttest{
#' # Load phenology data
#' data(pep_seed)
#'
#' # Create example temperature data (normally from weather stations/reanalysis)
#' temp <- data.frame(
#'   year = rep(2000:2015, each = 365),
#'   doy = rep(1:365, 16),
#'   tmin = rnorm(365 * 16, mean = 5, sd = 8),
#'   tmax = rnorm(365 * 16, mean = 15, sd = 8)
#' )
#'
#' # Add thermal sum to phenology data
#' pep_thermal <- calc_thermal_sum(pep_seed, temp, t_base = 5)
#' }
#'
#' @seealso \code{\link{calc_thermal_units}} for the underlying GDD calculation
#'
#' @author Matthias Templ
#' @export
calc_thermal_sum <- function(pep,
                              temp_data,
                              t_base = 5,
                              t_start = 1,
                              by = NULL,
                              method = "average") {

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }
  if (!inherits(temp_data, "data.frame")) {
    stop("'temp_data' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }
  if (!inherits(temp_data, "data.table")) {
    temp_data <- data.table::as.data.table(temp_data)
  }

  # Check required columns
  if (!all(c("year", "day") %in% names(pep))) {
    stop("'pep' must have 'year' and 'day' columns", call. = FALSE)
  }
  if (!all(c("year", "doy") %in% names(temp_data))) {
    stop("'temp_data' must have 'year' and 'doy' columns", call. = FALSE)
  }

  # Check for temperature columns
  has_minmax <- all(c("tmin", "tmax") %in% names(temp_data))
  has_mean <- "tmean" %in% names(temp_data)
  if (!has_minmax && !has_mean) {
    stop("'temp_data' must have either 'tmin'/'tmax' or 'tmean' columns", call. = FALSE)
  }

  # Get unique year-location combinations from pep
  pep <- data.table::copy(pep)
  pep[, .row_id := .I]

  # Calculate thermal sum for each observation
  thermal_sums <- numeric(nrow(pep))

  for (i in seq_len(nrow(pep))) {
    obs_year <- pep$year[i]
    obs_doy <- pep$day[i]

    if (is.na(obs_year) || is.na(obs_doy)) {
      thermal_sums[i] <- NA
      next
    }

    # Get temperature data for this year
    if (is.null(by)) {
      year_temp <- temp_data[year == obs_year & doy >= t_start & doy <= obs_doy]
    } else {
      # Match by additional columns (e.g., station)
      filter_expr <- list(year = obs_year)
      for (col in by) {
        if (col %in% names(pep) && col %in% names(temp_data)) {
          filter_expr[[col]] <- pep[[col]][i]
        }
      }
      year_temp <- temp_data[year == obs_year & doy >= t_start & doy <= obs_doy]
      for (col in by) {
        if (col %in% names(year_temp) && col %in% names(pep)) {
          year_temp <- year_temp[get(col) == pep[[col]][i]]
        }
      }
    }

    if (nrow(year_temp) == 0) {
      thermal_sums[i] <- NA
      next
    }

    # Order by DOY
    data.table::setorder(year_temp, doy)

    # Calculate GDD
    if (has_minmax) {
      gdd <- calc_thermal_units(year_temp$tmin, year_temp$tmax,
                                 t_base = t_base, method = method,
                                 cumulative = TRUE)
    } else {
      gdd <- calc_thermal_units(year_temp$tmean, t_base = t_base,
                                 method = method, cumulative = TRUE)
    }

    thermal_sums[i] <- tail(gdd, 1)
  }

  pep[, thermal_sum := thermal_sums]
  pep[, .row_id := NULL]

  pep
}

test_that("calc_thermal_units average method produces non-negative values", {
  tmin <- c(5, 7, 8, 10, 12)
  tmax <- c(15, 18, 20, 22, 25)
  result <- calc_thermal_units(tmin, tmax, t_base = 10, cumulative = FALSE)
  expect_true(all(result >= 0))
})

test_that("calc_thermal_units cumulative returns monotonically increasing values", {
  tmin <- c(5, 7, 8, 10, 12)
  tmax <- c(15, 18, 20, 22, 25)
  result <- calc_thermal_units(tmin, tmax, t_base = 10, cumulative = TRUE)
  expect_true(all(diff(result) >= 0))
})

test_that("calc_thermal_units daily vs cumulative consistency", {
  tmin <- c(5, 7, 8, 10, 12)
  tmax <- c(15, 18, 20, 22, 25)
  daily <- calc_thermal_units(tmin, tmax, t_base = 10, cumulative = FALSE)
  cum <- calc_thermal_units(tmin, tmax, t_base = 10, cumulative = TRUE)
  expect_equal(cumsum(daily), cum)
})

test_that("calc_thermal_units t_cap limits temperature effect", {
  tmin <- c(20, 20)
  tmax <- c(40, 40)
  without_cap <- calc_thermal_units(tmin, tmax, t_base = 10, t_cap = NULL, cumulative = FALSE)
  with_cap <- calc_thermal_units(tmin, tmax, t_base = 10, t_cap = 30, cumulative = FALSE)
  expect_true(all(with_cap <= without_cap))
})

test_that("calc_thermal_units errors on non-numeric tmin", {
  expect_error(calc_thermal_units("a", c(15, 18)), "must be numeric")
})

test_that("calc_thermal_units single_sine method works", {
  tmin <- c(5, 7, 8, 10, 12)
  tmax <- c(15, 18, 20, 22, 25)
  result <- calc_thermal_units(tmin, tmax, t_base = 10, method = "single_sine", cumulative = FALSE)
  expect_length(result, 5)
  expect_true(all(result >= 0))
})

test_that("calc_thermal_units single_sine matches Baskerville-Emin formula", {
  # Pinned numerics against the canonical Baskerville-Emin (1969) /
  # Snyder (1985) / UC-IPM single-sine formula for the mixed case
  # (Tmin < Tbase < Tmax), computed by hand as:
  #   GDD = (1/pi) * [ (Tavg - Tbase) * theta + Tamp * sin(theta) ]
  # where
  #   Tavg  = (Tmin + Tmax) / 2
  #   Tamp  = (Tmax - Tmin) / 2
  #   theta = acos( (Tbase - Tavg) / Tamp )

  # Case A: symmetric around base (Tmin=0, Tmax=10, Tbase=5)
  # Tavg=5, Tamp=5, (Tbase-Tavg)/Tamp=0, theta=pi/2
  # GDD = (1/pi) * [0*pi/2 + 5*sin(pi/2)] = 5/pi
  expect_equal(
    calc_thermal_units(tmin = 0, tmax = 10, t_base = 5,
                       method = "single_sine", cumulative = FALSE),
    5 / pi,
    tolerance = 1e-8
  )

  # Case B: asymmetric (Tmin=2, Tmax=12, Tbase=5)
  # Tavg=7, Tamp=5, (Tbase-Tavg)/Tamp=-0.4, theta=acos(-0.4) ~ 1.9823
  # GDD = (1/pi) * [2 * 1.9823... + 5 * sin(1.9823...)]
  theta_B <- acos(-0.4)
  expected_B <- (1 / pi) * (2 * theta_B + 5 * sin(theta_B))
  expect_equal(
    calc_thermal_units(tmin = 2, tmax = 12, t_base = 5,
                       method = "single_sine", cumulative = FALSE),
    expected_B,
    tolerance = 1e-8
  )

  # Case C: Tmin >= Tbase -> whole day above base -> GDD = Tavg - Tbase
  expect_equal(
    calc_thermal_units(tmin = 10, tmax = 20, t_base = 5,
                       method = "single_sine", cumulative = FALSE),
    (10 + 20) / 2 - 5,
    tolerance = 1e-10
  )

  # Case D: Tmax <= Tbase -> whole day below base -> GDD = 0
  expect_equal(
    calc_thermal_units(tmin = -5, tmax = 2, t_base = 5,
                       method = "single_sine", cumulative = FALSE),
    0,
    tolerance = 1e-10
  )
})

test_that("calc_thermal_units single_sine is bounded by modified method", {
  # Single-sine GDD is always <= the "modified" method GDD, because the
  # modified method clips daily temperatures up to Tbase (never below)
  # and averages, which is the maximum conceivable GDD under a sinusoid
  # that dips below Tbase for part of the day.
  set.seed(1)
  n <- 50
  tmin <- runif(n, -5, 10)
  tmax <- tmin + runif(n, 5, 20)
  t_base <- 5
  ss  <- calc_thermal_units(tmin, tmax, t_base = t_base,
                            method = "single_sine", cumulative = FALSE)
  mod <- calc_thermal_units(tmin, tmax, t_base = t_base,
                            method = "modified", cumulative = FALSE)
  expect_true(all(ss <= mod + 1e-10))
})

utils::globalVariables(c("..na_cols", "..keep_cols",
                         "baseline_q05", "baseline_q95",
                         "n_baseline_years", "n_years"))

#' Calculate Phenological Anomalies
#'
#' Computes deviations from long-term phenological baselines to identify
#' extreme years and detect climate signals. Returns both raw anomalies
#' (in days) and standardized anomalies for cross-species comparison.
#'
#' @param pep A \code{pep} object or \code{data.table} containing phenological
#'   observations with columns \code{year}, \code{day}, and grouping variables.
#' @param baseline_period Integer vector specifying the years to use for
#'   baseline calculation. Default is \code{1961:1990} (pre-acceleration
#'   warming reference period).
#' @param target_years Integer vector specifying years to calculate anomalies
#'   for. If \code{NULL} (default), calculates for all years in the data.
#' @param by Character vector of column names to group by. Must match columns
#'   in the data. Default is \code{c("country", "genus", "phase_id")}.
#' @param species Optional character string to filter by species/genus.
#'   If \code{NULL} (default), all species are included.
#' @param phase_id Optional integer vector to filter by BBCH phase codes.
#'   If \code{NULL} (default), all phases are included.
#' @param robust Logical. If \code{TRUE} (default), uses median and MAD for
#'   baseline and standardization. If \code{FALSE}, uses mean and SD.
#' @param min_years Minimum number of years required in the baseline period
#'   to calculate valid anomalies. Default is 15.
#' @param extreme_threshold Numeric. Z-score threshold for flagging extreme
#'   events. Default is 2 (approximately 95th percentile for normal distribution).
#' @param normals Optional pre-computed \code{pheno_normals} object. If provided,
#'   baseline statistics are taken from this object instead of being computed.
#' @param na.rm Logical. Should missing values be removed? Default \code{TRUE}.
#'
#' @return A \code{data.table} with the following columns:
#'   \describe{
#'     \item{by variables}{Grouping variables as specified in \code{by}}
#'     \item{year}{Year of observation}
#'     \item{observed_doy}{Observed mean day-of-year for the group/year}
#'     \item{baseline_doy}{Long-term baseline (median if robust, mean otherwise)}
#'     \item{baseline_spread}{Baseline spread (MAD if robust, SD otherwise)}
#'     \item{anomaly_days}{Deviation in days (negative = early, positive = late)}
#'     \item{z_score}{Standardized anomaly (anomaly / spread)}
#'     \item{percentile}{Percentile rank relative to baseline distribution}
#'     \item{is_extreme}{Logical flag for extreme events (|z_score| > threshold)}
#'     \item{direction}{Character: "early", "late", or "normal"}
#'   }
#'
#' @details
#' Phenological anomalies quantify how much a given year deviates from
#' long-term expectations. This is crucial for:
#' \itemize{
#'   \item Detecting climate change signals in phenology
#'   \item Identifying extreme phenological years
#'   \item Comparing anomalies across different species/regions
#' }
#'
#' The function supports two approaches:
#' \itemize{
#'   \item \strong{Robust} (default): Uses median for central tendency and MAD
#'     (Median Absolute Deviation) for spread. More resistant to outliers.
#'   \item \strong{Classical}: Uses mean and standard deviation. More sensitive
#'     to extreme values but provides interpretable z-scores under normality.
#' }
#'
#' @section Interpretation:
#' \itemize{
#'   \item \strong{anomaly_days}: Direct interpretation - "X days earlier/later
#'     than normal"
#'   \item \strong{z_score}: Standardized - values > 2 or < -2 are typically
#'     considered extreme
#'   \item \strong{percentile}: Normal approximation from z-score - e.g.,
#'     5th percentile means "earlier than 95\% of baseline years"
#' }
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Grapevine in Switzerland (longest historical records back to 1775)
#' vine_ch <- pep[country == "Switzerland" & species == "Vitis vinifera"]
#'
#' if (nrow(vine_ch) > 0) {
#'   # Calculate anomalies relative to 1961-1990 baseline
#'   anomalies <- pheno_anomaly(vine_ch,
#'                              baseline_period = 1961:1990,
#'                              phase_id = 65)
#'   print(anomalies)
#'
#'   # Find extreme early years
#'   extreme_early <- anomalies[is_extreme == TRUE & direction == "early"]
#'   extreme_early
#'
#'   # Anomalies for recent years only
#'   recent <- pheno_anomaly(vine_ch,
#'                           baseline_period = 1961:1990,
#'                           target_years = 2000:2020,
#'                           phase_id = 65)
#'   recent
#' }
#' }
#'
#' @references
#' Menzel, A. et al. (2006). European phenological response to climate
#' change matches the warming pattern.
#' \emph{Global Change Biology}, 12(10), 1969--1976.
#' \doi{10.1111/j.1365-2486.2006.01193.x}
#'
#' Templ, M., Templ, B., and Filzmoser, P. (2018).
#' Modelling and prediction of phenological data of the beginning of
#' flowering: Austrian indicators of climate change.
#' \emph{International Journal of Biometeorology}, 62, 1319--1334.
#' \doi{10.1007/s00484-018-1534-2}
#'
#' @seealso
#' \code{\link{pheno_normals}} for calculating baseline statistics,
#' \code{\link{pep_download}} for obtaining the main dataset
#'
#' @author Matthias Templ
#' @export
pheno_anomaly <- function(pep,
                          baseline_period = 1961:1990,
                          target_years = NULL,
                          by = c("country", "genus", "phase_id"),
                          species = NULL,
                          phase_id = NULL,
                          robust = TRUE,
                          min_years = 15,
                          extreme_threshold = 2,
                          normals = NULL,
                          na.rm = TRUE) {

  # Input validation
 if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table if needed
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Check required columns
  required_cols <- c("year", "day")
  missing_cols <- setdiff(required_cols, names(pep))
  if (length(missing_cols) > 0) {
    stop("Missing required columns: ", paste(missing_cols, collapse = ", "),
         call. = FALSE)
  }

  # Check that 'by' columns exist
  missing_by <- setdiff(by, names(pep))
  if (length(missing_by) > 0) {
    stop("Column(s) specified in 'by' not found: ",
         paste(missing_by, collapse = ", "),
         call. = FALSE)
  }

  # Validate parameters
  if (!is.numeric(baseline_period) || length(baseline_period) < 2) {
    stop("'baseline_period' must be a numeric vector with at least 2 years",
         call. = FALSE)
  }

  if (!is.null(target_years) && !is.numeric(target_years)) {
    stop("'target_years' must be NULL or a numeric vector", call. = FALSE)
  }

  if (!is.numeric(extreme_threshold) || extreme_threshold <= 0) {
    stop("'extreme_threshold' must be a positive number", call. = FALSE)
  }

  # Make a copy to avoid modifying original
  dt <- data.table::copy(pep)

  # Apply species filter if specified
  if (!is.null(species)) {
    species_filter <- species  # Avoid name collision with column

    if ("genus" %in% names(dt)) {
      # Convert to character to handle factors
      genus_match <- as.character(dt$genus) == species_filter
      species_match <- if ("species" %in% names(dt)) {
        as.character(dt$species) == species_filter
      } else {
        FALSE
      }
      dt <- dt[genus_match | species_match]
      if (nrow(dt) == 0) {
        stop("No observations found for species '", species_filter, "'", call. = FALSE)
      }
    } else if ("species" %in% names(dt)) {
      dt <- dt[as.character(dt$species) == species_filter]
      if (nrow(dt) == 0) {
        stop("No observations found for species '", species_filter, "'", call. = FALSE)
      }
    } else {
      warning("Neither 'genus' nor 'species' column found. Ignoring species filter.",
              call. = FALSE)
    }
  }

  # Apply phase_id filter if specified
  if (!is.null(phase_id)) {
    if (!"phase_id" %in% names(dt)) {
      warning("'phase_id' column not found. Ignoring phase_id filter.", call. = FALSE)
    } else {
      phase_filter <- phase_id
      dt <- dt[dt$phase_id %in% phase_filter]
      if (nrow(dt) == 0) {
        stop("No observations found for phase_id: ",
             paste(phase_id, collapse = ", "), call. = FALSE)
      }
    }
  }

  # Calculate or extract baseline statistics
  if (!is.null(normals)) {
    # Use provided normals
    if (!inherits(normals, "pheno_normals")) {
      warning("'normals' should be a pheno_normals object. Attempting to use anyway.",
              call. = FALSE)
    }

    # Extract baseline period from normals if available
    normals_period <- attr(normals, "period")
    if (!is.null(normals_period)) {
      baseline_period <- normals_period
    }

    baseline_stats <- data.table::as.data.table(normals)

    # Check that normals have compatible grouping
    normals_by <- intersect(by, names(baseline_stats))
    if (length(normals_by) < length(by)) {
      warning("Some grouping variables not found in normals: ",
              paste(setdiff(by, normals_by), collapse = ", "),
              call. = FALSE)
    }

    # Select appropriate baseline columns and keep only what we need
    if (robust) {
      baseline_stats[, baseline_doy := median_doy]
      baseline_stats[, baseline_spread := mad_doy]
    } else {
      baseline_stats[, baseline_doy := mean_doy]
      baseline_stats[, baseline_spread := sd_doy]
    }

    # Keep only grouping + baseline columns to avoid merge artifacts
    keep_cols <- c(normals_by, "baseline_doy", "baseline_spread")
    if ("q05" %in% names(baseline_stats)) {
      baseline_stats[, baseline_q05 := q05]
      baseline_stats[, baseline_q95 := q95]
      keep_cols <- c(keep_cols, "baseline_q05", "baseline_q95")
    }
    if ("n_years" %in% names(baseline_stats)) {
      baseline_stats[, n_baseline_years := n_years]
      keep_cols <- c(keep_cols, "n_baseline_years")
    }
    baseline_stats <- baseline_stats[, ..keep_cols]

  } else {
    # Calculate baseline from data
    baseline_data <- dt[year %in% baseline_period]

    if (nrow(baseline_data) == 0) {
      stop("No observations found in the baseline period: ",
           min(baseline_period), "-", max(baseline_period), call. = FALSE)
    }

    # First aggregate to annual means per group
    annual_means <- baseline_data[, .(
      annual_mean_doy = mean(day, na.rm = na.rm)
    ), by = c(by, "year")]

    # Calculate baseline statistics
    baseline_stats <- annual_means[, {
      doy_values <- annual_mean_doy
      n_yrs <- sum(!is.na(doy_values))

      if (n_yrs >= min_years) {
        if (robust) {
          list(
            n_baseline_years = n_yrs,
            baseline_doy = median(doy_values, na.rm = na.rm),
            baseline_spread = mad(doy_values, na.rm = na.rm),
            baseline_q05 = quantile(doy_values, 0.05, na.rm = na.rm),
            baseline_q95 = quantile(doy_values, 0.95, na.rm = na.rm)
          )
        } else {
          list(
            n_baseline_years = n_yrs,
            baseline_doy = mean(doy_values, na.rm = na.rm),
            baseline_spread = sd(doy_values, na.rm = na.rm),
            baseline_q05 = quantile(doy_values, 0.05, na.rm = na.rm),
            baseline_q95 = quantile(doy_values, 0.95, na.rm = na.rm)
          )
        }
      } else {
        list(
          n_baseline_years = n_yrs,
          baseline_doy = NA_real_,
          baseline_spread = NA_real_,
          baseline_q05 = NA_real_,
          baseline_q95 = NA_real_
        )
      }
    }, by = by]
  }

  # Filter target years if specified
  if (!is.null(target_years)) {
    dt <- dt[year %in% target_years]
    if (nrow(dt) == 0) {
      stop("No observations found for target years: ",
           paste(range(target_years), collapse = "-"), call. = FALSE)
    }
  }

  # Calculate annual observed DOY per group
  observed_annual <- dt[, .(
    observed_doy = mean(day, na.rm = na.rm),
    n_obs = .N
  ), by = c(by, "year")]

  # Merge with baseline statistics
  result <- merge(observed_annual, baseline_stats, by = by, all.x = TRUE)

  # Calculate anomalies
  result[, anomaly_days := observed_doy - baseline_doy]

  # Calculate z-score (handle zero spread)
  result[, z_score := ifelse(
    is.na(baseline_spread) | baseline_spread == 0,
    NA_real_,
    anomaly_days / baseline_spread
  )]

  # Calculate percentile rank from z-score (assumes approximately normal distribution)
  # This is a reasonable approximation for phenological data
  result[, percentile := ifelse(
    is.na(z_score),
    NA_real_,
    100 * pnorm(z_score)
  )]

  # Flag extreme events
  result[, is_extreme := abs(z_score) > extreme_threshold]

  # Add direction
  result[, direction := ifelse(
    is.na(anomaly_days),
    NA_character_,
    ifelse(anomaly_days < -extreme_threshold * baseline_spread, "early",
           ifelse(anomaly_days > extreme_threshold * baseline_spread, "late", "normal"))
  )]

  # Reorder columns for readability
  col_order <- c(by, "year", "observed_doy", "baseline_doy", "baseline_spread",
                 "anomaly_days", "z_score", "percentile", "is_extreme", "direction",
                 "n_obs")

  # Only include columns that exist
  col_order <- intersect(col_order, names(result))
  if ("n_baseline_years" %in% names(result)) {
    col_order <- c(col_order, "n_baseline_years")
  }

  data.table::setcolorder(result, col_order)

  # Order by grouping variables and year
  data.table::setorderv(result, c(by, "year"))

  # Add informative message about groups with insufficient data
  n_insufficient <- sum(is.na(result$baseline_doy))
  if (n_insufficient > 0) {
    message(sprintf(
      "Note: %d observation(s) have insufficient baseline data and return NA anomalies.",
      n_insufficient
    ))
  }

  # Add class for potential method dispatch
  class(result) <- c("pheno_anomaly", class(result))

  # Store parameters as attributes
  attr(result, "baseline_period") <- baseline_period
  attr(result, "target_years") <- target_years
  attr(result, "robust") <- robust
  attr(result, "extreme_threshold") <- extreme_threshold
  attr(result, "species_filter") <- species
  attr(result, "phase_filter") <- phase_id

  result
}


#' Print Method for Phenological Anomalies
#'
#' @param x A \code{pheno_anomaly} object
#' @param n Number of rows to display
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns \code{x}
#' @author Matthias Templ
#' @export
print.pheno_anomaly <- function(x, n = 10, ...) {
  baseline <- attr(x, "baseline_period")
  robust <- attr(x, "robust")
  threshold <- attr(x, "extreme_threshold")
  species_filter <- attr(x, "species_filter")
  phase_filter <- attr(x, "phase_filter")

  cat("Phenological Anomalies\n")
  cat(strrep("-", 50), "\n")

  if (!is.null(baseline) && length(baseline) > 0) {
    cat(sprintf("Baseline period: %d-%d\n", min(baseline), max(baseline)))
  } else {
    cat("Baseline period: (from pre-computed normals)\n")
  }

  method_str <- if (is.null(robust)) "Unknown" else ifelse(robust, "Robust (median/MAD)", "Classical (mean/SD)")
  cat(sprintf("Method: %s\n", method_str))

  if (!is.null(threshold)) {
    cat(sprintf("Extreme threshold: |z| > %.1f\n", threshold))
  }

  if (!is.null(species_filter)) {
    cat(sprintf("Species filter: %s\n", species_filter))
  }
  if (!is.null(phase_filter)) {
    cat(sprintf("Phase filter: %s\n", paste(phase_filter, collapse = ", ")))
  }

  # Summary statistics
  valid <- x[!is.na(anomaly_days)]
  if (nrow(valid) > 0) {
    n_extreme <- sum(valid$is_extreme, na.rm = TRUE)
    n_early <- sum(valid$direction == "early", na.rm = TRUE)
    n_late <- sum(valid$direction == "late", na.rm = TRUE)

    cat(sprintf("\nTotal observations: %d (valid: %d)\n", nrow(x), nrow(valid)))
    cat(sprintf("Extreme events: %d (%.1f%%) - %d early, %d late\n",
                n_extreme, 100 * n_extreme / nrow(valid), n_early, n_late))
    cat(sprintf("Anomaly range: %.1f to %.1f days\n",
                min(valid$anomaly_days), max(valid$anomaly_days)))
  }

  cat(strrep("-", 50), "\n\n")


  # Print as data.table, with NA values listed last
  print_dt <- data.table::as.data.table(x)
  # Order by: 1) NA in grouping columns (e.g. country), 2) NA in anomaly_days
  na_cols <- intersect(c("country", "genus", "species"), names(print_dt))
  if (length(na_cols) > 0) {
    has_na_group <- rowSums(is.na(print_dt[, ..na_cols])) > 0
    print_dt <- print_dt[order(has_na_group, is.na(anomaly_days))]
  } else {
    print_dt <- print_dt[order(is.na(anomaly_days))]
  }
  print(print_dt[1:min(n, nrow(print_dt))])

  if (nrow(x) > n) {
    cat(sprintf("\n... and %d more rows\n", nrow(x) - n))
  }

  invisible(x)
}


#' Summary Method for Phenological Anomalies
#'
#' @param object A \code{pheno_anomaly} object
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns a summary list
#' @author Matthias Templ
#' @export
summary.pheno_anomaly <- function(object, ...) {
  baseline <- attr(object, "baseline_period")

  cat("Phenological Anomalies Summary\n")
  cat(strrep("=", 50), "\n\n")

  if (!is.null(baseline) && length(baseline) > 0) {
    cat(sprintf("Baseline period: %d-%d\n", min(baseline), max(baseline)))
  } else {
    cat("Baseline period: (from pre-computed normals)\n")
  }
  cat(sprintf("Total observations: %d\n", nrow(object)))

  valid <- object[!is.na(anomaly_days)]

  if (nrow(valid) > 0) {
    cat(sprintf("Valid anomalies: %d (%.1f%%)\n\n",
                nrow(valid), 100 * nrow(valid) / nrow(object)))

    cat("Anomaly Statistics:\n")
    cat(sprintf("  Mean anomaly: %.1f days\n", mean(valid$anomaly_days)))
    cat(sprintf("  Median anomaly: %.1f days\n", median(valid$anomaly_days)))
    cat(sprintf("  SD of anomalies: %.1f days\n", sd(valid$anomaly_days)))
    cat(sprintf("  Range: %.1f to %.1f days\n",
                min(valid$anomaly_days), max(valid$anomaly_days)))

    cat("\nExtreme Events:\n")
    n_extreme <- sum(valid$is_extreme, na.rm = TRUE)
    n_early <- sum(valid$direction == "early", na.rm = TRUE)
    n_late <- sum(valid$direction == "late", na.rm = TRUE)

    cat(sprintf("  Total extreme: %d (%.1f%%)\n",
                n_extreme, 100 * n_extreme / nrow(valid)))
    cat(sprintf("  Extreme early: %d\n", n_early))
    cat(sprintf("  Extreme late: %d\n", n_late))

    # Most extreme years
    if (nrow(valid) > 0) {
      cat("\nMost Extreme Observations:\n")
      extreme_sorted <- valid[order(-abs(z_score))][1:min(5, nrow(valid))]
      for (i in seq_len(nrow(extreme_sorted))) {
        row <- extreme_sorted[i]
        cat(sprintf("  %d: %.1f days (%s, z=%.2f)\n",
                    row$year,
                    row$anomaly_days,
                    ifelse(row$anomaly_days < 0, "early", "late"),
                    row$z_score))
      }
    }
  } else {
    cat("\nNo valid anomalies calculated (insufficient baseline data).\n")
  }

  invisible(list(
    n_total = nrow(object),
    n_valid = nrow(valid),
    mean_anomaly = if (nrow(valid) > 0) mean(valid$anomaly_days) else NA,
    n_extreme = if (nrow(valid) > 0) sum(valid$is_extreme, na.rm = TRUE) else 0,
    baseline_period = baseline
  ))
}


#' Plot Method for Phenological Anomalies
#'
#' Visualise phenological anomalies as a colour-coded timeline or as a
#' histogram of anomaly magnitudes.
#'
#' @param x A \code{pheno_anomaly} object returned by
#'   \code{\link{pheno_anomaly}}.
#' @param which Character string selecting the plot type: \code{"timeline"}
#'   (default) shows \code{anomaly_days} per year as bars coloured by
#'   \code{direction} with extreme events marked; \code{"histogram"} shows
#'   the distribution of anomaly values.
#' @param ... Additional arguments (unused).
#'
#' @return A \code{ggplot} object (returned invisibly).
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' vine_ch <- pep[country == "Switzerland" & species == "Vitis vinifera"]
#'
#' if (nrow(vine_ch) > 0) {
#'   a <- pheno_anomaly(vine_ch,
#'                      baseline_period = 1961:1990,
#'                      phase_id = 65,
#'                      by = "phase_id")
#'   plot(a)
#'   plot(a, which = "histogram")
#' }
#' }
#'
#' @seealso \code{\link{pheno_anomaly}}, \code{\link{pheno_normals}}
#' @author Matthias Templ
#' @export
plot.pheno_anomaly <- function(x, which = c("timeline", "histogram"), ...) {
  which <- match.arg(which)

  # Remove rows with NA anomalies
  plot_data <- data.table::as.data.table(x)[!is.na(anomaly_days)]

  if (nrow(plot_data) == 0) {
    stop("No valid anomalies to plot", call. = FALSE)
  }

  # Identify grouping columns
  known_cols <- c("year", "observed_doy", "baseline_doy", "baseline_spread",
                  "anomaly_days", "z_score", "percentile", "is_extreme",
                  "direction", "n_obs", "n_baseline_years",
                  "baseline_q05", "baseline_q95")
  group_cols <- setdiff(names(plot_data), known_cols)

  # Build facet label when there are grouping columns
  has_groups <- length(group_cols) > 0
  if (has_groups) {
    for (gc in group_cols) {
      if (gc == "phase_id") {
        descs <- .bbch_lookup[as.character(plot_data[[gc]])]
        descs[is.na(descs)] <- as.character(plot_data[[gc]][is.na(descs)])
        plot_data[, (gc) := paste0(plot_data[[gc]], " - ", descs)]
      }
    }
    plot_data[, facet_label := do.call(paste, c(.SD, sep = " | ")),
              .SDcols = group_cols]
  }

  # Subtitle from attributes
  baseline <- attr(x, "baseline_period")
  subtitle <- if (!is.null(baseline) && length(baseline) > 0) {
    sprintf("Baseline: %d-%d", min(baseline), max(baseline))
  } else {
    "Baseline: pre-computed normals"
  }

  direction_colors <- c("early" = "steelblue", "late" = "coral",
                         "normal" = "gray70")

  if (which == "timeline") {
    # Ensure direction is a factor for consistent colours
    plot_data[, direction := factor(direction,
                                    levels = c("early", "normal", "late"))]

    p <- ggplot2::ggplot(plot_data,
                         ggplot2::aes(x = year, y = anomaly_days,
                                      fill = direction)) +
      ggplot2::geom_col(width = 0.8) +
      ggplot2::scale_fill_manual(values = direction_colors, drop = FALSE) +
      ggplot2::labs(x = "Year", y = "Anomaly (days)",
                    fill = "Direction",
                    title = "Phenological Anomalies",
                    subtitle = subtitle) +
      ggplot2::theme_minimal()

    # Mark extreme events
    extremes <- plot_data[is_extreme == TRUE]
    if (nrow(extremes) > 0) {
      p <- p + ggplot2::geom_point(data = extremes,
                                   ggplot2::aes(x = year, y = anomaly_days),
                                   shape = 18, size = 2.5, color = "black",
                                   inherit.aes = FALSE)
    }

    # Facet if multiple groups
    if (has_groups) {
      p <- p + ggplot2::facet_wrap(~ facet_label, scales = "free_y")
    }

  } else {
    # Histogram
    p <- ggplot2::ggplot(plot_data,
                         ggplot2::aes(x = anomaly_days)) +
      ggplot2::geom_histogram(fill = "steelblue", color = "white",
                              bins = 30) +
      ggplot2::geom_vline(xintercept = 0, linetype = "dashed",
                          color = "gray40") +
      ggplot2::labs(x = "Anomaly (days)", y = "Count",
                    title = "Distribution of Phenological Anomalies",
                    subtitle = subtitle) +
      ggplot2::theme_minimal()

    if (has_groups) {
      p <- p + ggplot2::facet_wrap(~ facet_label)
    }
  }

  p
}

test_that("pheno_anomaly() returns correct class", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  by = c("genus", "phase_id"), min_years = 10)
  )
  expect_s3_class(result, "pheno_anomaly")
  expect_s3_class(result, "data.table")
})

test_that("pheno_anomaly() has expected columns", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  by = c("genus", "phase_id"), min_years = 10)
  )
  expected_cols <- c("observed_doy", "baseline_doy", "anomaly_days",
                     "z_score", "is_extreme", "direction")
  expect_true(all(expected_cols %in% names(result)))
})

test_that("anomaly_days equals observed_doy minus baseline_doy", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  by = c("genus", "phase_id"), min_years = 10)
  )
  valid <- result[!is.na(anomaly_days)]
  expect_equal(valid$anomaly_days, valid$observed_doy - valid$baseline_doy)
})

test_that("direction values are valid", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  by = c("genus", "phase_id"), min_years = 10)
  )
  valid_dirs <- result$direction[!is.na(result$direction)]
  expect_true(all(valid_dirs %in% c("early", "late", "normal")))
})

test_that("is_extreme is logical", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  by = c("genus", "phase_id"), min_years = 10)
  )
  expect_type(result$is_extreme, "logical")
})

test_that("robust vs classical method both work", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  robust_result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  by = c("genus", "phase_id"), min_years = 10, robust = TRUE)
  )
  classical_result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  by = c("genus", "phase_id"), min_years = 10, robust = FALSE)
  )
  expect_true(attr(robust_result, "robust"))
  expect_false(attr(classical_result, "robust"))
})

test_that("target_years filter works", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  target_years = 2010:2015,
                  by = c("genus", "phase_id"), min_years = 10)
  )
  expect_true(all(result$year >= 2010 & result$year <= 2015))
})

test_that("pheno_anomaly() stores attributes", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  by = c("genus", "phase_id"), min_years = 10,
                  extreme_threshold = 2, robust = TRUE)
  )
  expect_equal(attr(result, "baseline_period"), 1990:2005)
  expect_equal(attr(result, "extreme_threshold"), 2)
  expect_true(attr(result, "robust"))
})

test_that("percentile is between 0 and 100", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- suppressMessages(
    pheno_anomaly(p, baseline_period = 1990:2005,
                  by = c("genus", "phase_id"), min_years = 10)
  )
  valid_pct <- result$percentile[!is.na(result$percentile)]
  expect_true(all(valid_pct >= 0 & valid_pct <= 100))
})

test_that("pheno_anomaly() errors on invalid input", {
  expect_error(pheno_anomaly("not a df"), "must be a data.frame")
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  expect_error(
    pheno_anomaly(p, baseline_period = "bad"),
    "numeric vector"
  )
  expect_error(
    pheno_anomaly(p, extreme_threshold = -1),
    "positive number"
  )
})

#' @title PEP725 Phenological Data Class
#'
#' @description
#' S3 class for PEP725 phenological data that inherits from \code{data.table}.
#' Provides validation, informative print/summary methods, and convenient defaults.
#'
#' @name pep-class
#' @aliases pep-class
#'
#' @importFrom data.table as.data.table uniqueN .N .SD setDT copy fifelse rbindlist fread setcolorder setnames setorderv melt
#' @importFrom ggplot2 ggplot aes geom_point geom_histogram geom_line labs theme_minimal coord_quickmap
#' @importFrom stats IQR median sd quantile rnorm residuals lm pnorm as.formula complete.cases contr.sum coef predict contrasts<-
#' @importFrom utils head tail txtProgressBar setTxtProgressBar
#' @importFrom patchwork wrap_plots
#' @import methods
NULL

# Prevent R CMD check notes for data.table columns and other non-standard evaluation
utils::globalVariables(c(
  # Core pep columns
  "s_id", "lon", "lat", "genus", "species", "phase_id", "year", "day",
  "n_obs", "country", "phase_name",
  # data.table special symbols
  ".N", ".I", ".SD", ":=", ".row_id", ".row_idx", ".has_na",
  # Analysis variables
  "Country", "DOY", "DOYHd", "DOYHv", "HarvestDOY", "HarvestYEAR", "HeadingDOY",
  "Integral", "N", "PLZ", "Scenario", "Tgl", "alt", "altitude", "annual_mean_doy",
  "anomaly_days", "b", "baseline_doy", "baseline_spread", "bound", "colname",
  "dT", "dTgl", "d_mean", "daylength", "days_decade", "direction", "doy",
  "doy_ctrl", "doy_end", "doy_end_ctrl", "doy_end_scen", "doy_scen", "doy_start",
  "doy_start_ctrl", "doy_start_scen", "doy_synth", "elevation", "filter_type",
  "first_year", "geometry", "giss", "inside", "interpretation", "iqr_doy",
  "is_extreme", "label", "label_y", "last_year", "mad_doy", "mean_DOY", "mean_day",
  "mean_doy", "median_DOY", "median_day", "median_doy", "n_species", "n_stations",
  "observed_doy", "panel", "pct_outliers", "percentile", "phase", "pheno",
  "present", "q25", "q75", "qc_ori_flag", "r_squared", "s_ctrl", "s_scen",
  "sd_doy", "se", "series", "significant", "slope", "station_mean_doy",
  "subspecies", "tau", "thermal_sum", "tmp_end_ctrl", "tmp_end_scen",
  "tmp_start_ctrl", "tmp_start_scen", "total", "value", "var_value", "x", "y",
  "yearHd", "yearHv", "year_max", "year_min", "years_present", "z_score",
  # pep_quality plot variables
  "quality_grade", "pct", "completeness_pct", "outlier_pct",
  # plot.pheno_normals / plot.pheno_anomaly labels
  "group_label", "facet_label"
))

# =============================================================================
# Constructor
# =============================================================================

#' Create a PEP725 Phenological Data Object
#'
#' Constructor function that creates a validated \code{pep} object from a
#' data.frame or data.table. The object inherits from \code{data.table} and
#' retains all its functionality.
#'
#' @param x A data.frame or data.table containing PEP725 phenological data.
#' @param validate Logical. If \code{TRUE} (default), validates that required
#'   columns are present and have correct types.
#'
#' @return An object of class \code{c("pep", "data.table", "data.frame")}.
#'
#' @details
#' Required columns:
#' \itemize{
#'   \item \code{s_id} - Station ID
#'   \item \code{lon}, \code{lat} - Coordinates
#'   \item \code{genus}, \code{species} - Plant taxonomy
#'   \item \code{phase_id} - BBCH phenological phase code
#'   \item \code{year}, \code{day} - Observation timing
#' }
#'
#' @examples
#' \donttest{
#' # From imported data
#' dt <- pep_download()
#' pep_data <- new_pep(dt)
#' }
#'
#' @author Matthias Templ
#' @export
new_pep <- function(x, validate = TRUE) {
  # Ensure it's a data.table


  if (!inherits(x, "data.table")) {
    x <- as.data.table(x)
  }

  if (validate) {
    validate_pep(x)
  }

  # Add class (prepend to preserve data.table inheritance)
  # Remove any existing "pep" to avoid duplication
  class(x) <- c("pep", setdiff(class(x), "pep"))

  x
}

#' Validate PEP725 Data Structure
#'
#' Internal function to validate that a data object has the required
#' structure for PEP725 phenological data.
#'
#' @param x A data.frame or data.table to validate.
#' @return Invisibly returns \code{TRUE} if valid; otherwise throws an error.
#' @keywords internal
validate_pep <- function(x) {
  required_cols <- c("s_id", "lon", "lat", "genus", "species", "phase_id", "year", "day")

  missing_cols <- setdiff(required_cols, names(x))

  if (length(missing_cols) > 0) {
    stop(
      "Missing required columns for pep object: ",
      paste(missing_cols, collapse = ", "),
      call. = FALSE
    )
  }

  # Type checks
  if (!is.numeric(x$lon) || !is.numeric(x$lat)) {
    stop("Columns 'lon' and 'lat' must be numeric", call. = FALSE)
  }

  if (!is.numeric(x$year) || !is.numeric(x$day)) {
    stop("Columns 'year' and 'day' must be numeric/integer", call. = FALSE)
  }

  if (!is.numeric(x$phase_id)) {
    stop("Column 'phase_id' must be numeric/integer (BBCH code)", call. = FALSE)
  }

  invisible(TRUE)
}

# =============================================================================
# Type checking and coercion
# =============================================================================

#' Test if Object is a PEP725 Data Object
#'
#' @param x An R object.
#' @return Logical; \code{TRUE} if \code{x} is of class \code{pep}.
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' is.pep(pep)
#' is.pep(data.frame(x = 1))
#' }
#'
#' @author Matthias Templ
#' @export
is.pep <- function(x) {


  inherits(x, "pep")
}

#' Coerce to PEP725 Data Object
#'
#' @param x A data.frame or data.table to coerce.
#' @param ... Additional arguments (currently unused).
#' @return An object of class \code{pep}.
#'
#' @author Matthias Templ
#' @export
as.pep <- function(x, ...) {
  new_pep(x)
}

# =============================================================================
# Print method
# =============================================================================

#' Print Method for PEP725 Data
#'
#' Displays a concise summary of the PEP725 phenological dataset.
#'
#' @param x A \code{pep} object.
#' @param n Number of rows to display (default 5).
#' @param ... Additional arguments passed to print.
#'
#' @return Invisibly returns \code{x}.
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' print(pep)
#' }
#'
#' @author Matthias Templ
#' @export
print.pep <- function(x, n = 5, ...) {
  # Header
  cat("PEP725 Phenological Data\n")
  cat(rep("-", 50), "\n", sep = "")

  # Key statistics
  n_obs <- format(nrow(x), big.mark = ",")
  n_stations <- format(length(unique(x$s_id)), big.mark = ",")
  n_species <- length(unique(paste(x$genus, x$species)))
  n_phases <- length(unique(x$phase_id))
  year_range <- range(x$year, na.rm = TRUE)

  cat(sprintf("Observations: %s\n", n_obs))
  cat(sprintf("Stations:     %s\n", n_stations))
  cat(sprintf("Species:      %d\n", n_species))
  phase_codes <- paste(sort(unique(x$phase_id))[1:min(5, n_phases)], collapse = ", ")
  if (n_phases > 5) phase_codes <- paste0(phase_codes, ", ...")
  cat(sprintf("Phases:       %d (BBCH codes: %s)\n", n_phases, phase_codes))
  cat(sprintf("Years:        %d - %d\n", year_range[1], year_range[2]))

  # Geographic extent
  lon_range <- range(x$lon, na.rm = TRUE)
  lat_range <- range(x$lat, na.rm = TRUE)
  cat(sprintf("Extent:       lon [%.2f, %.2f], lat [%.2f, %.2f]\n",
              lon_range[1], lon_range[2], lat_range[1], lat_range[2]))

  # Country info if available
  if ("country" %in% names(x)) {
    countries <- sort(unique(x$country))
    n_countries <- length(countries)
    cat(sprintf("Countries:    %d (%s%s)\n",
                n_countries,
                paste(head(countries, 5), collapse = ", "),
                if (n_countries > 5) ", ..." else ""))
  }

  cat(rep("-", 50), "\n", sep = "")

  # Show first few rows (using data.table print)
  cat(sprintf("First %d rows:\n", min(n, nrow(x))))
  print(as.data.table(x)[1:min(n, nrow(x)), .(s_id, lon, lat, genus, species, phase_id, year, day)])

  invisible(x)
}

# =============================================================================
# Summary method
# =============================================================================

#' Summary Method for PEP725 Data
#'
#' Provides a detailed phenological summary of the dataset, including
#' breakdowns by species, phase, country, and temporal coverage.
#'
#' @param object A \code{pep} object.
#' @param by Character. Summarize by \code{"species"}, \code{"phase"},
#'   \code{"country"}, or \code{"year"}. Default is \code{"species"}.
#' @param top Integer. Number of top entries to show (default 10).
#' @param ... Additional arguments (currently unused).
#'
#' @return A \code{pep_summary} object (list) containing summary tables,
#'   printed to console.
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' summary(pep)
#' summary(pep, by = "phase")
#' summary(pep, by = "country")
#' }
#'
#' @author Matthias Templ
#' @export
summary.pep <- function(object, by = c("species", "phase", "country", "year"),
                        top = 10, ...) {
  by <- match.arg(by)
  x <- object

  cat("PEP725 Phenological Data Summary\n")
  cat(rep("=", 50), "\n\n", sep = "")

  # Overall statistics
  cat("OVERVIEW\n")
  cat(sprintf("  Total observations: %s\n", format(nrow(x), big.mark = ",")))
  cat(sprintf("  Unique stations:    %s\n", format(length(unique(x$s_id)), big.mark = ",")))
  cat(sprintf("  Year range:         %d - %d\n", min(x$year), max(x$year)))
  cat(sprintf("  DOY range:          %d - %d\n", min(x$day, na.rm = TRUE), max(x$day, na.rm = TRUE)))
  cat("\n")

  # Summary by selected dimension
  result <- switch(by,
    species = {
      cat(sprintf("TOP %d SPECIES (by observation count)\n", top))
      summary_dt <- x[, .(
        n_obs = .N,
        n_stations = uniqueN(s_id),
        year_min = min(year),
        year_max = max(year),
        median_doy = as.integer(median(day, na.rm = TRUE))
      ), by = .(genus, species)][order(-n_obs)][1:min(top, .N)]

      print(summary_dt)
      summary_dt
    },
    phase = {
      cat("OBSERVATIONS BY BBCH PHASE\n")

      summary_dt <- x[, .(
        n_obs = .N,
        n_species = uniqueN(paste(genus, species)),
        median_doy = as.integer(median(day, na.rm = TRUE)),
        iqr_doy = as.integer(IQR(day, na.rm = TRUE))
      ), by = phase_id][order(phase_id)]

      summary_dt[, phase_name := .bbch_lookup[as.character(phase_id)]]
      summary_dt[is.na(phase_name), phase_name := "Other"]

      print(summary_dt[, .(phase_id, phase_name, n_obs, n_species, median_doy, iqr_doy)])
      summary_dt
    },
    country = {
      if (!"country" %in% names(x)) {
        cat("  (No 'country' column - use add_country() first)\n")
        return(invisible(NULL))
      }
      cat(sprintf("TOP %d COUNTRIES (by observation count)\n", top))
      summary_dt <- x[, .(
        n_obs = .N,
        n_stations = uniqueN(s_id),
        n_species = uniqueN(paste(genus, species)),
        year_min = min(year),
        year_max = max(year)
      ), by = country][order(-n_obs)][1:min(top, .N)]

      print(summary_dt)
      summary_dt
    },
    year = {
      cat("TEMPORAL COVERAGE\n")
      summary_dt <- x[, .(
        n_obs = .N,
        n_stations = uniqueN(s_id),
        n_species = uniqueN(paste(genus, species))
      ), by = year][order(year)]

      # Show summary stats
      cat(sprintf("  Years with data:    %d\n", nrow(summary_dt)))
      cat(sprintf("  Median obs/year:    %s\n", format(median(summary_dt$n_obs), big.mark = ",")))
      cat(sprintf("  Peak year:          %d (%s obs)\n",
                  summary_dt[which.max(n_obs), year],
                  format(max(summary_dt$n_obs), big.mark = ",")))
      cat("\n  Last 10 years:\n")
      print(tail(summary_dt, 10))
      summary_dt
    }
  )

  # Return summary object invisibly
  out <- list(
    n_obs = nrow(x),
    n_stations = length(unique(x$s_id)),
    n_species = length(unique(paste(x$genus, x$species))),
    year_range = range(x$year),
    summary_table = result
  )
  class(out) <- "pep_summary"
  invisible(out)
}

# =============================================================================
# Plot method
# =============================================================================

#' Plot Method for PEP725 Data
#'
#' Provides default visualizations for PEP725 phenological data.
#'
#' @param x A \code{pep} object.
#' @param type Character. Type of plot: \code{"map"} for station locations,
#'   \code{"timeseries"} for temporal trends, \code{"histogram"} for DOY distribution.
#'   Default is \code{"map"}.
#' @param ... Additional arguments passed to the underlying plot function.
#'
#' @return A ggplot object (invisibly).
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' plot(pep)
#' plot(pep, type = "timeseries")
#' plot(pep, type = "histogram")
#' }
#'
#' @importFrom ggplot2 ggplot aes geom_point geom_histogram geom_line
#' @importFrom ggplot2 labs theme_minimal coord_quickmap stat_summary
#' @author Matthias Templ
#' @export
plot.pep <- function(x, type = c("map", "timeseries", "histogram"), ...) {
  type <- match.arg(type)

  switch(type,
    map = {
      # Station map
      stations <- unique(x[, .(s_id, lon, lat)])
      p <- ggplot(stations, aes(x = lon, y = lat)) +
        geom_point(alpha = 0.3, size = 0.8, color = "steelblue") +
        coord_quickmap() +
        labs(
          title = "PEP725 Station Locations",
          subtitle = sprintf("%s stations", format(nrow(stations), big.mark = ",")),
          x = "Longitude", y = "Latitude"
        ) +
        theme_minimal()
    },
    timeseries = {
      # Annual observation counts and mean DOY
      annual <- x[, .(n_obs = .N, mean_doy = mean(day, na.rm = TRUE)), by = year]
      p <- ggplot(annual, aes(x = year, y = mean_doy)) +
        geom_line(color = "steelblue", linewidth = 0.8) +
        geom_point(aes(size = n_obs), alpha = 0.5, color = "steelblue") +
        labs(
          title = "Mean Phenological Day of Year Over Time",
          subtitle = "Point size = number of observations",
          x = "Year", y = "Mean DOY",
          size = "Observations"
        ) +
        theme_minimal()
    },
    histogram = {
      # DOY distribution
      p <- ggplot(x, aes(x = day)) +
        geom_histogram(bins = 52, fill = "steelblue", color = "white", alpha = 0.8) +
        labs(
          title = "Distribution of Phenological Events",
          subtitle = sprintf("%s observations", format(nrow(x), big.mark = ",")),
          x = "Day of Year (DOY)", y = "Count"
        ) +
        theme_minimal()
    }
  )

  print(p)
  invisible(p)
}

# =============================================================================
# Subsetting - preserve class
# =============================================================================

#' Subset PEP725 Data While Preserving Class
#'
#' Ensures that subsetting operations return a \code{pep} object
#' when the result still has the required structure.
#'
#' @param x A \code{pep} object.
#' @param ... Arguments passed to the data.table subset method.
#'
#' @return A \code{pep} object if structure is preserved, otherwise a data.table.
#'
#' @author Matthias Templ
#' @export
`[.pep` <- function(x, ...) {
  # Call the data.table method
  class(x) <- class(x)[-1]
  result <- NextMethod()

  # Try to preserve pep class if result is still valid
  if (is.data.frame(result) && nrow(result) > 0) {
    required_cols <- c("s_id", "lon", "lat", "genus", "species", "phase_id", "year", "day")
    if (all(required_cols %in% names(result))) {
      class(result) <- c("pep", class(result))
    }
  }

  result
}

# =============================================================================
# Helper: BBCH code lookup
# =============================================================================

# Internal named vector of BBCH codes -> descriptions.
# Single source of truth used by bbch_description(), summary.pep(),
# select_phase(), and pheno_regional().
.bbch_lookup <- c(
  "0" = "Dry seed / Dormancy",
  "1" = "Seed imbibition",
  "5" = "Radicle emerged",
  "7" = "Coleoptile emerged",
  "9" = "Emergence",
  "10" = "First leaf through coleoptile",
  "11" = "First leaf unfolded",
  "12" = "2 leaves unfolded",
  "13" = "3 leaves unfolded",
  "15" = "True leaves",
  "19" = "9+ leaves unfolded",
  "21" = "Beginning of tillering",
  "29" = "End of tillering",
  "30" = "Beginning of stem elongation",
  "31" = "First node detectable",
  "37" = "Flag leaf just visible",
  "39" = "Flag leaf ligule visible",
  "41" = "Early boot",
  "45" = "Mid boot",
  "49" = "Late boot",
  "51" = "Beginning of heading",
  "55" = "Middle of heading",
  "59" = "End of heading",
  "60" = "Beginning of flowering / Heading complete",
  "61" = "Beginning of flowering",
  "65" = "Full flowering / Anthesis",
  "69" = "End of flowering",
  "70" = "Milk development",
  "71" = "Watery ripe",
  "73" = "Early milk",
  "75" = "Medium milk",
  "77" = "Late milk",
  "80" = "Dough development",
  "81" = "Beginning of ripening or fruit colouration",
  "83" = "Early dough",
  "85" = "Soft dough",
  "87" = "Hard dough",
  "89" = "Fully ripe",
  "90" = "Over-ripe / Maturity",
  "92" = "Over-ripe, grain hard",
  "95" = "50% of leaves fallen",
  "97" = "Plant dead",
  "99" = "Harvested product",
  "100" = "Harvest",
  "111" = "First cut for silage",
  "131" = "First cut for hay",
  "151" = "Start of harvest for silage (corn, grass)",
  "205" = "Autumn coloring >= 50%"
)

#' Get BBCH Phase Description
#'
#' Returns human-readable descriptions for BBCH phenological codes.
#'
#' @param codes Integer vector of BBCH codes.
#' @param na.rm Logical. If \code{TRUE} (default), removes NA codes and codes
#'   without a known description from the output.
#' @param sort Logical. If \code{TRUE} (default), sorts the output by phase code.
#'
#' @return A data.frame with columns \code{phase_id} and \code{description},
#'   ordered by phase code (if \code{sort = TRUE}).
#'
#' @examples
#' bbch_description(c(60, 65, 100))
#' bbch_description(c(100, 60, 65, NA), na.rm = TRUE, sort = TRUE)
#'
#' @author Matthias Templ
#' @export
bbch_description <- function(codes, na.rm = TRUE, sort = TRUE) {
  bbch_lookup <- .bbch_lookup

  # Remove NA codes if requested
  if (na.rm) {
    codes <- codes[!is.na(codes)]
  }

  # Get unique codes
  codes <- unique(codes)

  # Look up descriptions
  descriptions <- bbch_lookup[as.character(codes)]

  # Remove unknown phases if na.rm is TRUE

  if (na.rm) {
    known <- !is.na(descriptions)
    codes <- codes[known]
    descriptions <- descriptions[known]
  } else {
    descriptions[is.na(descriptions)] <- "Unknown phase"
  }

  # Create result data.frame
  result <- data.frame(
    phase_id = as.integer(codes),
    description = unname(descriptions),
    stringsAsFactors = FALSE
  )

  # Sort by phase_id if requested
  if (sort) {
    result <- result[order(result$phase_id), ]
    rownames(result) <- NULL
  }

  result
}

# =============================================================================
# Coverage function
# =============================================================================

#' Assess Data Coverage of PEP725 Phenological Data
#'
#' Provides comprehensive coverage statistics for phenological datasets,
#' including temporal, geographical, and species dimensions.
#'
#' @param x A \code{pep} object or data.frame with phenological data.
#' @param kind Character. Type of coverage to assess:
#'   \itemize{
#'     \item \code{"all"} (default): All coverage types combined
#'     \item \code{"temporal"}: Year range, gaps, observations per year
#'     \item \code{"geographical"}: Countries, stations, coordinate ranges
#'     \item \code{"species"}: Species and genus diversity
#'   }
#' @param top Integer. Number of top entries to show in tables (default: 10).
#' @param plot Logical. If \code{TRUE}, produces a visualization of the
#'   coverage. Default is \code{FALSE}.
#' @param by Character. Optional grouping variable for more detailed analysis
#'   (e.g., \code{"country"}, \code{"species"}). Only used for temporal coverage.
#'
#' @return A list of class \code{pep_coverage} containing coverage statistics.
#'   The structure depends on \code{kind}:
#'   \describe{
#'     \item{temporal}{Year range, number of years, gaps, observations per year}
#'     \item{geographical}{Countries, stations, coordinate bounds, altitude range}
#'     \item{species}{Genera, species, subspecies counts and details}
#'     \item{all}{All of the above combined}
#'   }
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' pep_ch <- pep[country == "Switzerland"]
#'
#' # Full coverage report
#' pep_coverage(pep_ch)
#'
#' # Temporal coverage only
#' pep_coverage(pep_ch, kind = "temporal")
#'
#' # Species coverage, top 5
#' pep_coverage(pep_ch, kind = "species", top = 5)
#' }
#'
#' @author Matthias Templ
#' @export
pep_coverage <- function(x, kind = c("all", "temporal", "geographical", "species"),
                     top = 10, plot = FALSE, by = NULL) {

kind <- match.arg(kind)

  # Ensure data.table
  if (!inherits(x, "data.table")) {
    x <- data.table::as.data.table(x)
  }

  result <- list()
  class(result) <- "pep_coverage"
  attr(result, "kind") <- kind

# ---------------------------------------------------------------------------
  # Temporal coverage
  # ---------------------------------------------------------------------------
  if (kind %in% c("all", "temporal")) {
    years <- sort(unique(x$year))
    year_range <- range(years)
    n_years <- length(years)

    # Find gaps (missing years in sequence)
    expected_years <- seq(min(years), max(years))
    missing_years <- setdiff(expected_years, years)

    # Observations per year
    obs_by_year <- x[, .(
      n_obs = .N,
      n_stations = uniqueN(s_id),
      n_species = uniqueN(paste(genus, species))
    ), by = year][order(year)]

    # By group if specified
    if (!is.null(by) && by %in% names(x)) {
      obs_by_year_group <- x[, .(
        n_obs = .N,
        n_stations = uniqueN(s_id),
        year_min = min(year),
        year_max = max(year),
        n_years = uniqueN(year)
      ), by = by][order(-n_obs)]
    } else {
      obs_by_year_group <- NULL
    }

    result$temporal <- list(
      year_range = year_range,
      n_years = n_years,
      missing_years = if (length(missing_years) > 0) missing_years else NULL,
      n_gaps = length(missing_years),
      obs_by_year = obs_by_year,
      obs_by_group = obs_by_year_group,
      median_obs_per_year = median(obs_by_year$n_obs),
      peak_year = obs_by_year[which.max(n_obs), year],
      peak_obs = max(obs_by_year$n_obs)
    )
  }

  # ---------------------------------------------------------------------------
  # Geographical coverage
  # ---------------------------------------------------------------------------
  if (kind %in% c("all", "geographical")) {
    # Countries
    if ("country" %in% names(x)) {
      countries <- x[, .(
        n_obs = .N,
        n_stations = uniqueN(s_id),
        n_species = uniqueN(paste(genus, species))
      ), by = country][order(-n_obs)]
    } else {
      countries <- NULL
    }

    # Stations
    stations <- x[, .(
      n_obs = .N,
      lon = lon[1],
      lat = lat[1],
      alt = if ("alt" %in% names(x)) alt[1] else NA_real_
    ), by = s_id]

    # Coordinate bounds
    lon_range <- range(x$lon, na.rm = TRUE)
    lat_range <- range(x$lat, na.rm = TRUE)

    # Altitude range
    if ("alt" %in% names(x)) {
      alt_range <- range(x$alt, na.rm = TRUE)
      alt_range[!is.finite(alt_range)] <- NA
    } else {
      alt_range <- c(NA, NA)
    }

    result$geographical <- list(
      n_countries = if (!is.null(countries)) nrow(countries) else NA,
      countries = if (!is.null(countries)) countries[1:min(top, nrow(countries))] else NULL,
      n_stations = nrow(stations),
      lon_range = lon_range,
      lat_range = lat_range,
      alt_range = alt_range,
      stations_sample = stations[sample(.N, min(top, .N))]
    )
  }

  # ---------------------------------------------------------------------------
  # Species coverage
  # ---------------------------------------------------------------------------
  if (kind %in% c("all", "species")) {
    # Genera
    genera <- x[, .(
      n_obs = .N,
      n_species = uniqueN(species),
      n_stations = uniqueN(s_id)
    ), by = genus][order(-n_obs)]

    # Species
    species_tbl <- x[, .(
      n_obs = .N,
      n_stations = uniqueN(s_id),
      year_min = min(year),
      year_max = max(year)
    ), by = .(genus, species)][order(-n_obs)]

    # Subspecies (if available)
    if ("subspecies" %in% names(x)) {
      subspecies_tbl <- x[!is.na(subspecies) & subspecies != "", .(
        n_obs = .N,
        n_stations = uniqueN(s_id)
      ), by = .(genus, species, subspecies)][order(-n_obs)]
    } else {
      subspecies_tbl <- NULL
    }

    result$species <- list(
      n_genera = nrow(genera),
      n_species = nrow(species_tbl),
      n_subspecies = if (!is.null(subspecies_tbl)) nrow(subspecies_tbl) else 0,
      genera = genera[1:min(top, nrow(genera))],
      species = species_tbl[1:min(top, nrow(species_tbl))],
      subspecies = if (!is.null(subspecies_tbl) && nrow(subspecies_tbl) > 0)
        subspecies_tbl[1:min(top, nrow(subspecies_tbl))] else NULL
    )
  }

  # ---------------------------------------------------------------------------
  # Plot if requested
  # ---------------------------------------------------------------------------
  if (plot) {
    p <- plot.pep_coverage(result)
    print(p)
  }

  result
}


#' Print Method for PEP Coverage
#'
#' @param x A \code{pep_coverage} object.
#' @param ... Additional arguments (unused).
#'
#' @return Invisibly returns \code{x}.
#' @author Matthias Templ
#' @export
print.pep_coverage <- function(x, ...) {
  kind <- attr(x, "kind")

  cat("PEP725 Data Coverage Report\n")
  cat(rep("=", 50), "\n\n", sep = "")

  # Temporal
  if (!is.null(x$temporal)) {
    cat("TEMPORAL COVERAGE\n")
    cat(rep("-", 30), "\n", sep = "")
    cat(sprintf("  Year range:      %d - %d (%d years)\n",
                x$temporal$year_range[1], x$temporal$year_range[2],
                x$temporal$n_years))
    cat(sprintf("  Missing years:   %d gaps\n", x$temporal$n_gaps))
    if (x$temporal$n_gaps > 0 && x$temporal$n_gaps <= 10) {
      cat(sprintf("                   (%s)\n",
                  paste(x$temporal$missing_years, collapse = ", ")))
    }
    cat(sprintf("  Median obs/year: %s\n",
                format(x$temporal$median_obs_per_year, big.mark = ",")))
    cat(sprintf("  Peak year:       %d (%s obs)\n",
                x$temporal$peak_year,
                format(x$temporal$peak_obs, big.mark = ",")))
    cat("\n")

    if (!is.null(x$temporal$obs_by_group)) {
      cat("  Coverage by group:\n")
      print(x$temporal$obs_by_group[1:min(5, nrow(x$temporal$obs_by_group))])
      cat("\n")
    }
  }

  # Geographical
  if (!is.null(x$geographical)) {
    cat("GEOGRAPHICAL COVERAGE\n")
    cat(rep("-", 30), "\n", sep = "")
    if (!is.na(x$geographical$n_countries)) {
      cat(sprintf("  Countries:       %d\n", x$geographical$n_countries))
    }
    cat(sprintf("  Stations:        %s\n",
                format(x$geographical$n_stations, big.mark = ",")))
    cat(sprintf("  Longitude:       %.2f to %.2f\n",
                x$geographical$lon_range[1], x$geographical$lon_range[2]))
    cat(sprintf("  Latitude:        %.2f to %.2f\n",
                x$geographical$lat_range[1], x$geographical$lat_range[2]))
    if (!all(is.na(x$geographical$alt_range))) {
      cat(sprintf("  Altitude:        %d to %d m\n",
                  as.integer(x$geographical$alt_range[1]),
                  as.integer(x$geographical$alt_range[2])))
    }
    cat("\n")

    if (!is.null(x$geographical$countries)) {
      cat("  Top countries:\n")
      print(x$geographical$countries)
      cat("\n")
    }
  }

  # Species
  if (!is.null(x$species)) {
    cat("SPECIES COVERAGE\n")
    cat(rep("-", 30), "\n", sep = "")
    cat(sprintf("  Genera:          %d\n", x$species$n_genera))
    cat(sprintf("  Species:         %d\n", x$species$n_species))
    if (x$species$n_subspecies > 0) {
      cat(sprintf("  Subspecies:      %d\n", x$species$n_subspecies))
    }
    cat("\n  Top genera:\n")
    print(x$species$genera)
    cat("\n  Top species:\n")
    print(x$species$species)
    cat("\n")
  }

  invisible(x)
}


#' Plot Method for PEP Coverage
#'
#' @param x A \code{pep_coverage} object.
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object (or list of ggplot objects).
#' @author Matthias Templ
#' @export
plot.pep_coverage <- function(x, ...) {
  kind <- attr(x, "kind")
  plots <- list()

  # Temporal plot
  if (!is.null(x$temporal)) {
    plot_data <- data.table::copy(x$temporal$obs_by_year)

    # Add missing years with n_obs = 0 to show gaps
    if (x$temporal$n_gaps > 0) {
      missing_dt <- data.table::data.table(
        year = x$temporal$missing_years,
        n_obs = 0L,
        n_stations = 0L,
        n_species = 0L
      )
      plot_data <- rbind(plot_data, missing_dt)[order(year)]
    }

    p_temporal <- ggplot2::ggplot(plot_data,
                                   ggplot2::aes(x = year, y = n_obs)) +
      ggplot2::geom_line(color = "steelblue") +
      ggplot2::geom_point(ggplot2::aes(color = n_obs == 0), size = 1.5) +
      ggplot2::scale_color_manual(values = c("FALSE" = "steelblue", "TRUE" = "red"),
                                   guide = "none") +
      ggplot2::labs(title = "Temporal Coverage",
                    x = "Year", y = "Number of Observations") +
      ggplot2::theme_minimal()
    plots$temporal <- p_temporal
  }

  # Geographical plot (simple: stations by country)
  if (!is.null(x$geographical) && !is.null(x$geographical$countries)) {
    countries_top <- x$geographical$countries[1:min(10, nrow(x$geographical$countries))]
    countries_top$country <- factor(countries_top$country,
                                     levels = rev(countries_top$country))
    p_geo <- ggplot2::ggplot(countries_top,
                              ggplot2::aes(x = n_stations, y = country)) +
      ggplot2::geom_col(fill = "forestgreen", alpha = 0.7) +
      ggplot2::labs(title = "Geographical Coverage",
                    x = "Number of Stations", y = "") +
      ggplot2::theme_minimal()
    plots$geographical <- p_geo
  }

  # Species plot
  if (!is.null(x$species)) {
    species_top <- x$species$species[1:min(10, nrow(x$species$species))]
    species_top$label <- paste(species_top$genus, species_top$species)
    species_top$label <- factor(species_top$label, levels = rev(species_top$label))
    p_species <- ggplot2::ggplot(species_top,
                                  ggplot2::aes(x = n_obs, y = label)) +
      ggplot2::geom_col(fill = "coral", alpha = 0.7) +
      ggplot2::labs(title = "Species Coverage",
                    x = "Number of Observations", y = "") +
      ggplot2::theme_minimal()
    plots$species <- p_species
  }

  # Combine plots if multiple
  if (length(plots) > 1) {
    combined <- patchwork::wrap_plots(plots, ncol = 1)
    return(combined)
  } else if (length(plots) == 1) {
    return(plots[[1]])
  }

  invisible(NULL)
}

test_that("new_pep() creates correct class from pep_seed", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  expect_s3_class(p, "pep")
  expect_s3_class(p, "data.table")
  expect_s3_class(p, "data.frame")
  expect_equal(class(p), c("pep", "data.table", "data.frame"))
})

test_that("new_pep() converts plain data.frame to data.table", {
  data(pep_seed, package = "pep725")
  df <- as.data.frame(pep_seed)
  expect_false(inherits(df, "data.table"))
  p <- new_pep(df)
  expect_s3_class(p, "pep")
  expect_s3_class(p, "data.table")
})

test_that("new_pep() errors on missing required columns", {
  dt <- data.table::data.table(s_id = 1, lon = 8, lat = 47)
  expect_error(new_pep(dt), "Missing required columns")
})

test_that("new_pep() errors on non-numeric lon/lat", {
  dt <- data.table::data.table(
    s_id = 1, lon = "a", lat = "b", genus = "Malus",
    species = "Malus domestica", phase_id = 60, year = 2000, day = 100
  )
  expect_error(new_pep(dt), "lon.*lat.*must be numeric")
})

test_that("new_pep() errors on non-numeric year/day", {
  dt <- data.table::data.table(
    s_id = 1, lon = 8, lat = 47, genus = "Malus",
    species = "Malus domestica", phase_id = 60, year = "x", day = "y"
  )
  expect_error(new_pep(dt), "year.*day.*must be numeric")
})

test_that("new_pep() errors on non-numeric phase_id", {
  dt <- data.table::data.table(
    s_id = 1, lon = 8, lat = 47, genus = "Malus",
    species = "Malus domestica", phase_id = "abc", year = 2000, day = 100
  )
  expect_error(new_pep(dt), "phase_id.*must be numeric")
})

test_that("new_pep() with validate = FALSE skips validation", {
  dt <- data.table::data.table(x = 1)
  p <- new_pep(dt, validate = FALSE)
  expect_s3_class(p, "pep")
})

test_that("is.pep() returns TRUE for pep objects", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  expect_true(is.pep(p))
})

test_that("is.pep() returns FALSE for non-pep objects", {
  expect_false(is.pep(data.frame(x = 1)))
  expect_false(is.pep(42))
})

test_that("as.pep() works as alias for new_pep()", {
  data(pep_seed, package = "pep725")
  p <- as.pep(pep_seed)
  expect_s3_class(p, "pep")
})

test_that("[.pep preserves class when required columns present", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  sub <- p[year >= 2000]
  expect_s3_class(sub, "pep")
})

test_that("[.pep drops class when required columns removed", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  sub <- p[, .(s_id, year)]
  expect_false(inherits(sub, "pep"))
})

test_that("[.pep drops class for zero-row result", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  sub <- p[year > 9999]
  expect_false(inherits(sub, "pep"))
})

test_that("bbch_description() returns data.frame with correct columns", {
  result <- bbch_description(c(60, 65, 100))
  expect_s3_class(result, "data.frame")
  expect_true(all(c("phase_id", "description") %in% names(result)))
  expect_equal(nrow(result), 3)
})

test_that("bbch_description() handles NA codes", {
  result <- bbch_description(c(60, NA, 65), na.rm = TRUE)
  expect_false(any(is.na(result$phase_id)))
})

test_that("print.pep() produces output", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  expect_output(print(p), "PEP725 Phenological Data")
})

test_that("pep_coverage returns pep_coverage object", {
  data(pep_seed)
  result <- pep_coverage(pep_seed)
  expect_s3_class(result, "pep_coverage")
})

test_that("pep_coverage kind=species works", {
  data(pep_seed)
  result <- pep_coverage(pep_seed, kind = "species")
  expect_s3_class(result, "pep_coverage")
})

test_that("pep_coverage kind=temporal works", {
  data(pep_seed)
  result <- pep_coverage(pep_seed, kind = "temporal")
  expect_s3_class(result, "pep_coverage")
})

test_that("pheno_normals() returns correct class", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, by = c("genus", "phase_id"),
                          min_years = 10)
  expect_s3_class(result, "pheno_normals")
  expect_s3_class(result, "data.table")
})

test_that("pheno_normals() has expected output columns", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, by = c("genus", "phase_id"),
                          min_years = 10)
  expected_cols <- c("n_years", "mean_doy", "median_doy", "sd_doy",
                     "iqr_doy", "mad_doy", "q25", "q75", "period")
  expect_true(all(expected_cols %in% names(result)))
})

test_that("pheno_normals() min_years threshold returns NA for insufficient data", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  # pep_seed has 26 years (1990-2015), so min_years = 30 should cause NAs
  result <- suppressMessages(
    pheno_normals(p, period = 1990:2015, by = c("genus", "phase_id"),
                  min_years = 30)
  )
  expect_true(all(is.na(result$mean_doy)))
})

test_that("pheno_normals() species filter works", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, species = "Malus",
                          by = c("phase_id"), min_years = 10)
  expect_gt(nrow(result), 0)
})

test_that("pheno_normals() phase_id filter works", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, phase_id = 60,
                          by = c("genus"), min_years = 10)
  expect_gt(nrow(result), 0)
})

test_that("pheno_normals() stores attributes", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, species = "Malus",
                          phase_id = 60, by = c("country"), min_years = 10)
  expect_equal(attr(result, "period"), 1990:2015)
  expect_equal(attr(result, "min_years"), 10)
  expect_equal(attr(result, "species_filter"), "Malus")
  expect_equal(attr(result, "phase_filter"), 60)
})

test_that("pheno_normals() period column is correctly formatted", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, by = c("genus", "phase_id"),
                          min_years = 10)
  expect_equal(unique(result$period), "1990-2015")
})

test_that("pheno_normals() custom by grouping works", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, by = c("genus"),
                          min_years = 10)
  expect_true("genus" %in% names(result))
  expect_gt(nrow(result), 0)
})

test_that("pheno_normals() statistics are plausible", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, by = c("genus", "phase_id"),
                          min_years = 10)
  valid <- result[!is.na(mean_doy)]
  expect_true(all(valid$mean_doy >= 1 & valid$mean_doy <= 365))
  expect_true(all(valid$sd_doy >= 0))
  expect_true(all(valid$q25 <= valid$q75))
})

test_that("pheno_normals() default probs produce q05/q10/q25/q75/q90/q95", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, by = c("genus", "phase_id"),
                          min_years = 10)
  expect_true(all(c("q05", "q10", "q25", "q75", "q90", "q95") %in% names(result)))
})

test_that("pheno_normals() custom probs generate correctly named columns", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  # Non-default probs must produce column names derived from the levels,
  # not silently re-use q05..q95.
  result <- pheno_normals(p, period = 1990:2015, by = c("genus", "phase_id"),
                          min_years = 10,
                          probs = c(0.025, 0.25, 0.75, 0.975))
  expect_true("q02_5"  %in% names(result))
  expect_true("q25"    %in% names(result))
  expect_true("q75"    %in% names(result))
  expect_true("q97_5"  %in% names(result))
  # The old hardcoded names must not be present when the user did not ask.
  expect_false("q05" %in% names(result))
  expect_false("q95" %in% names(result))
  # Values must correspond to actual quantile levels.
  valid <- result[!is.na(q25)]
  expect_true(all(valid$q02_5 <= valid$q25))
  expect_true(all(valid$q75   <= valid$q97_5))
})

test_that("pheno_normals() stores probs as attribute", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_normals(p, period = 1990:2015, by = c("genus", "phase_id"),
                          min_years = 10,
                          probs = c(0.1, 0.9))
  expect_equal(attr(result, "probs"), c(0.1, 0.9))
})

test_that("pheno_normals() accepts a two-element probs without error", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  expect_no_error(
    pheno_normals(p, period = 1990:2015, by = c("genus", "phase_id"),
                  min_years = 10, probs = c(0.1, 0.9))
  )
})

test_that("pheno_normals() errors on invalid input", {
  expect_error(pheno_normals("not a df"), "must be a data.frame")
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  expect_error(pheno_normals(p, period = "bad"), "numeric vector")
  expect_error(pheno_normals(p, by = c("nonexistent")), "not found")
  expect_error(
    pheno_normals(p, species = "ZZZnonexistent", period = 1990:2015,
                  by = c("phase_id")),
    "No observations"
  )
})

test_that("pheno_gradient() returns correct class", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_gradient(p, variable = "alt", species = "Malus", phase_id = 60)
  expect_s3_class(result, "pheno_gradient")
  expect_true(is.list(result))
})

test_that("pheno_gradient() result has expected structure", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_gradient(p, variable = "alt", species = "Malus", phase_id = 60)
  expect_true(all(c("summary", "model", "data", "variable", "method") %in% names(result)))
})

test_that("pheno_gradient() summary has expected columns", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_gradient(p, variable = "alt", species = "Malus", phase_id = 60)
  expected_cols <- c("slope", "intercept", "r_squared", "rmse",
                     "n_points", "interpretation")
  expect_true(all(expected_cols %in% names(result$summary)))
})

test_that("pheno_gradient() latitude gradient works", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_gradient(p, variable = "lat", species = "Malus", phase_id = 60)
  expect_equal(result$variable, "lat")
  expect_false(is.na(result$summary$slope))
})

test_that("pheno_gradient() OLS method returns lm model", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_gradient(p, variable = "alt", species = "Malus",
                           phase_id = 60, method = "ols")
  expect_s3_class(result$model, "lm")
})

test_that("pheno_gradient() robust method returns lmrob model", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_gradient(p, variable = "alt", species = "Malus",
                           phase_id = 60, method = "robust")
  expect_s3_class(result$model, "lmrob")
})

test_that("pheno_gradient() by = 'country' produces multiple summary rows", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_gradient(p, variable = "alt", species = "Malus",
                           phase_id = 60, by = "country")
  expect_gt(nrow(result$summary), 0)
  expect_true("country" %in% names(result$summary))
})

test_that("pheno_gradient() r_squared is between 0 and 1", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_gradient(p, variable = "alt", species = "Malus", phase_id = 60)
  r2 <- result$summary$r_squared
  expect_true(!is.na(r2) && r2 >= 0 && r2 <= 1)
})

test_that("pheno_gradient() interpretation text is populated", {
  data(pep_seed, package = "pep725")
  p <- new_pep(pep_seed)
  result <- pheno_gradient(p, variable = "alt", species = "Malus", phase_id = 60)
  expect_true(nchar(result$summary$interpretation[1]) > 0)
})

test_that("pheno_gradient() errors on invalid input", {
  expect_error(pheno_gradient("not a df"), "must be a data.frame")
  data(pep_seed, package = "pep725")
  # Remove alt column to trigger altitude error
  dt <- data.table::copy(pep_seed)
  dt[, alt := NULL]
  expect_error(pheno_gradient(dt, variable = "alt"), "Altitude column not found")
})

utils::globalVariables(c("lon", "lat", "s_id", "year", "phase_id", "day",
                         "is_outlier", "deviation", "n_obs", "n_outliers",
                         "max_abs_dev", "mean_abs_dev", "popup_html"))

#' Interactive Leaflet Map of Outlier-Flagged Stations
#'
#' Renders a Leaflet map of the stations in a \code{pep_outliers} object,
#' with each station plotted as a circle marker whose size encodes the
#' number of flagged observations and whose colour encodes the maximum
#' absolute residual (or Mahalanobis distance) for the station. Popups
#' list the worst-offending station-years and their residual values —
#' useful for interactive inspection during QC.
#'
#' Unlike \code{\link{pheno_leaflet}} (which is a selection gadget built
#' on Shiny + miniUI), this is a pure visualisation: it returns a
#' \code{leaflet} htmlwidget so you can embed it in an R Markdown
#' report, include it in a vignette, or call \code{print()} on it for
#' interactive exploration.
#'
#' @param x A \code{pep_outliers} object from \code{\link{pep_flag_outliers}}.
#'   Must contain \code{lon}, \code{lat}, \code{s_id}, \code{year},
#'   \code{phase_id}, \code{day}, \code{is_outlier}, and \code{deviation}
#'   columns.
#' @param outlier_only Logical. If \code{TRUE} (default), only stations
#'   with at least one flagged observation are shown. If \code{FALSE},
#'   every station appears (non-flagged stations are drawn small and
#'   grey for context).
#' @param top_n Integer. Maximum number of station-year records included
#'   in a station's popup. Default 10.
#' @param radius_range Numeric length-2. Minimum and maximum circle
#'   marker radius (in pixels) mapped to log1p(n_outliers). Default
#'   \code{c(4, 14)}.
#' @param palette Character. Name of a \code{RColorBrewer} or viridis
#'   palette for the colour scale. Default \code{"plasma"}.
#' @param ... Additional arguments reserved for future expansion.
#'
#' @return A \code{leaflet} htmlwidget. Call \code{print()} to display
#'   in the RStudio Viewer or embed it in an Rmd file.
#'
#' @details
#' **Colour** encodes the station's maximum absolute residual
#' (for univariate methods: \code{max(|deviation|)} in days; for
#' \code{method = "mahalanobis"}: the maximum robust Mahalanobis
#' distance). **Size** encodes \code{log1p(n_outliers)} — a station
#' flagged once has a small marker, a station flagged 30 times has a
#' much larger one.
#'
#' For \code{method = "gam_residual"} and \code{"mahalanobis"} this is
#' the natural companion to the Schaber-Badeck-style text tables and
#' the static diagnostic figure, because spatial structure in outliers
#' (a local observer network, a regional climate anomaly) is often
#' only visible on a map.
#'
#' @examples
#' \dontrun{
#' pep <- pep_download()
#' pep_ch <- pep[country == "Switzerland" & species == "Malus domestica"]
#' flagged <- pep_flag_outliers(
#'   pep_ch,
#'   by = c("genus", "species", "phase_id"),
#'   method = "gam_residual"
#' )
#' pep_outliers_leaflet(flagged)
#' }
#'
#' @seealso \code{\link{pep_flag_outliers}} — detection,
#'   \code{\link{pep_plot_outliers}} — static diagnostic plots,
#'   \code{\link{pheno_leaflet}} — station-selection Shiny gadget.
#' @author Matthias Templ
#' @export
pep_outliers_leaflet <- function(x,
                                 outlier_only = TRUE,
                                 top_n = 10,
                                 radius_range = c(4, 14),
                                 palette = "plasma",
                                 ...) {
  if (!inherits(x, "pep_outliers")) {
    stop("'x' must be a pep_outliers object from pep_flag_outliers().",
         call. = FALSE)
  }
  required <- c("lon", "lat", "s_id", "is_outlier", "deviation")
  missing_cols <- setdiff(required, names(x))
  if (length(missing_cols) > 0) {
    stop("'x' is missing required columns: ",
         paste(missing_cols, collapse = ", "),
         call. = FALSE)
  }
  if (!requireNamespace("leaflet", quietly = TRUE)) {
    stop("Package 'leaflet' is required for pep_outliers_leaflet(). ",
         "Install it with: install.packages('leaflet').",
         call. = FALSE)
  }

  method <- attr(x, "method")
  threshold <- attr(x, "threshold")

  dt <- data.table::as.data.table(x)
  # Station-level aggregates.
  st <- dt[, .(
    lon = lon[1],
    lat = lat[1],
    n_obs = .N,
    n_outliers = sum(is_outlier, na.rm = TRUE),
    max_abs_dev  = suppressWarnings(
      max(abs(deviation[is_outlier %in% TRUE]), na.rm = TRUE)),
    mean_abs_dev = mean(abs(deviation[is_outlier %in% TRUE]), na.rm = TRUE)
  ), by = s_id]
  # Replace -Inf / NaN (stations with no outliers) with zero so the colour
  # scale handles them.
  st[!is.finite(max_abs_dev),  max_abs_dev  := 0]
  st[is.na(mean_abs_dev),      mean_abs_dev := 0]

  if (outlier_only) {
    st <- st[n_outliers > 0L]
    if (nrow(st) == 0L) {
      message("No flagged stations to plot. Returning an empty map.")
    }
  }

  # Per-station popup: top_n worst offenders.
  .build_popup <- function(s) {
    sub <- dt[s_id == s & is_outlier %in% TRUE]
    if (nrow(sub) == 0L) {
      return(sprintf("<b>Station %s</b><br/>No flagged observations.", s))
    }
    sub <- sub[order(-abs(deviation))][seq_len(min(.N, top_n))]
    rows <- paste0(
      "<tr><td>", sub$year,
      "</td><td>", sub$phase_id,
      "</td><td>", round(sub$day, 0),
      "</td><td>", sprintf("%+.1f", sub$deviation),
      "</td></tr>",
      collapse = "")
    sprintf(paste0(
      "<b>Station %s</b><br/>",
      "<i>%d flagged of %d observations</i><br/>",
      "<table style='font-size:10px;border-collapse:collapse' ",
      "cellpadding='2'>",
      "<tr><th>Year</th><th>Phase</th><th>DOY</th><th>Dev</th></tr>",
      "%s</table>"),
      s,
      dt[s_id == s, sum(is_outlier, na.rm = TRUE)],
      dt[s_id == s, .N],
      rows)
  }
  st[, popup_html := vapply(s_id, .build_popup, character(1))]

  # Colour & size scales. Guard against empty or all-zero domains that
  # leaflet::colorNumeric refuses.
  max_dev <- if (nrow(st) == 0L) 1 else {
    m0 <- suppressWarnings(max(st$max_abs_dev, na.rm = TRUE))
    if (!is.finite(m0) || m0 <= 0) 1 else m0 + 1e-8
  }
  pal_fun <- leaflet::colorNumeric(palette = palette,
                                   domain = c(0, max_dev))
  radius <- if (nrow(st) == 0L) numeric(0) else {
    max_n <- max(st$n_outliers, na.rm = TRUE)
    if (max_n == 0L) rep(radius_range[1], nrow(st)) else {
      scale <- log1p(st$n_outliers) / log1p(max_n)
      radius_range[1] + scale * diff(radius_range)
    }
  }

  legend_title <- if (identical(method, "mahalanobis"))
    "Max robust MD" else "Max |residual| (d)"
  map_subtitle <- if (!is.null(method))
    sprintf("method = %s%s", method,
            if (!is.null(threshold) && is.finite(threshold))
              sprintf(", threshold = %s",
                      format(threshold, digits = 3)) else "") else ""

  m <- leaflet::leaflet(data = st) |>
    leaflet::addProviderTiles("CartoDB.Positron")
  if (nrow(st) > 0L) {
    m <- m |>
      leaflet::addCircleMarkers(
        lng = ~lon, lat = ~lat,
        radius = radius,
        color = ~pal_fun(max_abs_dev),
        stroke = TRUE, weight = 1, opacity = 0.9,
        fillColor = ~pal_fun(max_abs_dev), fillOpacity = 0.65,
        popup = ~popup_html,
        label = ~sprintf("Station %s (%d outliers)", s_id, n_outliers)
      ) |>
      leaflet::addLegend(
        position = "bottomright",
        pal = pal_fun, values = ~max_abs_dev,
        title = legend_title,
        opacity = 0.8
      )
  }
  if (nzchar(map_subtitle)) {
    m <- m |>
      leaflet::addControl(
        html = sprintf(
          "<div style='background:white;padding:3px 6px;font-size:11px'>%s</div>",
          map_subtitle),
        position = "topleft"
      )
  }
  m
}

make_flagged_fixture <- function() {
  set.seed(42)
  n_st <- 10; n_yr <- 20
  grid <- data.table::CJ(s_id = seq_len(n_st), year = 1990:(1989 + n_yr))
  d <- grid
  d[, alt := rep(runif(n_st, 200, 2000), each = n_yr)]
  d[, lat := rep(runif(n_st, 46, 50), each = n_yr)]
  d[, lon := rep(runif(n_st, 5, 15), each = n_yr)]
  d[, day := 110 + 0.02 * alt + rnorm(.N, sd = 4)]
  d[, `:=`(genus = "Malus", species = "Malus domestica", phase_id = 60L)]
  # Inject three clear outliers
  d[s_id == 1 & year == 2000, day := day + 50]
  d[s_id == 2 & year == 1995, day := day - 45]
  d[s_id == 3 & year == 2005, day := day + 40]
  pep_flag_outliers(d, by = c("genus","species","phase_id"),
                    method = "30day")
}

test_that("pep_outliers_leaflet returns a leaflet htmlwidget", {
  skip_if_not_installed("leaflet")
  flagged <- make_flagged_fixture()
  m <- pep_outliers_leaflet(flagged)
  expect_s3_class(m, "leaflet")
  expect_s3_class(m, "htmlwidget")
})

test_that("pep_outliers_leaflet respects outlier_only", {
  skip_if_not_installed("leaflet")
  flagged <- make_flagged_fixture()
  m_all  <- pep_outliers_leaflet(flagged, outlier_only = FALSE)
  m_only <- pep_outliers_leaflet(flagged, outlier_only = TRUE)
  # The only-outliers map has fewer or equal markers.
  n_all  <- length(m_all$x$calls[vapply(m_all$x$calls,
                                         function(c) c$method == "addCircleMarkers",
                                         logical(1))])
  n_only <- length(m_only$x$calls[vapply(m_only$x$calls,
                                          function(c) c$method == "addCircleMarkers",
                                          logical(1))])
  # Both paths should produce exactly one addCircleMarkers call.
  expect_equal(n_all, 1L)
  expect_equal(n_only, 1L)
})

test_that("pep_outliers_leaflet errors on wrong input class", {
  skip_if_not_installed("leaflet")
  expect_error(pep_outliers_leaflet(data.frame(x = 1)),
               "pep_outliers object")
})

test_that("pep_outliers_leaflet errors on missing columns", {
  skip_if_not_installed("leaflet")
  flagged <- make_flagged_fixture()
  # Strip required columns
  flagged2 <- data.table::copy(flagged)
  class(flagged2) <- c("pep_outliers", class(flagged2))
  flagged2[, c("lon", "lat") := NULL]
  expect_error(pep_outliers_leaflet(flagged2), "missing required columns")
})

test_that("pep_outliers_leaflet handles a fixture with zero flagged stations", {
  skip_if_not_installed("leaflet")
  d <- data.table::data.table(
    s_id = rep(1:3, each = 6),
    year = rep(2000:2005, 3),
    day = c(110, 111, 109, 108, 112, 110,
            115, 116, 114, 113, 117, 115,
            120, 121, 119, 118, 122, 120),
    phase_id = 60L,
    lon = rep(c(5, 7, 9), each = 6),
    lat = rep(c(47, 48, 49), each = 6),
    genus = "Malus", species = "Malus domestica"
  )
  flagged <- pep_flag_outliers(d, by = c("genus","species","phase_id"),
                               method = "30day")
  expect_message(m <- pep_outliers_leaflet(flagged),
                 "No flagged stations")
  expect_s3_class(m, "leaflet")
})

# Prevent R CMD check notes
utils::globalVariables(c("day", "year", "s_id", "expected_doy", "deviation",
                         "is_outlier", "outlier_type", "group_median", "group_mad",
                         "..by", "..group_cols"))

#' Flag Phenological Outliers
#'
#' Identifies outlier observations in phenological data using the 30-day rule
#' or statistical methods. Based on the approach described by Schaber & Badeck
#' (2002) for quality control of phenological observations.
#'
#' @param pep A \code{pep} object or data.table with phenological observations.
#'   Must contain \code{year}, \code{day}, and grouping columns.
#' @param by Character vector of columns defining groups for outlier detection.
#'   Default \code{c("s_id", "genus", "species", "phase_id")} detects outliers
#'   within each station-species-phase combination.
#' @param method Character. Outlier detection method:
#'   \describe{
#'     \item{"30day"}{(Default) Flag observations >30 days from group mean/median.
#'       The classic Schaber-Badeck approach for phenological data.}
#'     \item{"mad"}{Use Median Absolute Deviation. Flag if |deviation| > k * MAD,
#'       where k is set by \code{threshold}.}
#'     \item{"iqr"}{Use Interquartile Range. Flag if outside (Q1 - k*IQR, Q3 + k*IQR).}
#'     \item{"zscore"}{Flag if |z-score| > threshold (default 3).}
#'     \item{"gam_residual"}{Fit a GAM (via \code{mgcv::gam()}) per group
#'       using the supplied \code{formula} — by default accounting for year
#'       trend, elevation, latitude, and a station random intercept — and
#'       flag observations whose robust-z-scored residual exceeds
#'       \code{threshold}. Detects covariate-inconsistent anomalies that
#'       the within-group \code{"30day"} rule misses. Falls back to
#'       \code{"30day"} on groups with fewer than \code{min_n_per_group}
#'       observations or when the model fails to converge.}
#'     \item{"mahalanobis"}{Treat each station-year as a vector of DOYs
#'       across phases and flag station-years whose **robust** Mahalanobis
#'       distance exceeds the \code{threshold}. The centre and covariance
#'       are estimated by the Minimum Covariance Determinant (MCD)
#'       estimator of Rousseeuw (1984), via
#'       \code{robustbase::covMcd()} — using the classical, non-robust
#'       covariance here would let the outliers contaminate the
#'       estimate and mask themselves. Under MCD the squared distance is
#'       still approximately \eqn{\chi^2_p}, so the default threshold
#'       is \eqn{\sqrt{\chi^2_{0.975, p}}}, where \eqn{p} is the number
#'       of phases present in the group. Detects station-years whose
#'       pattern across phases is jointly inconsistent (e.g.\ BBCH 60
#'       and 65 impossibly close) even when each marginal DOY looks
#'       fine. All rows within a flagged station-year are marked; the
#'       \code{deviation} column stores the robust Mahalanobis distance.
#'       Groups with too few complete station-years (fewer than
#'       \eqn{\max(p + 5, 15)}) or a singular MCD fit fall back to the
#'       30-day rule.}
#'   }
#' @param threshold Numeric. Threshold for outlier detection.
#'   \describe{
#'     \item{For "30day":}{Days deviation to flag as outlier. Default 30.}
#'     \item{For "mad":}{Number of MADs. Default 3 (~2.5 SD equivalent).}
#'     \item{For "iqr":}{IQR multiplier. Default 1.5 (standard Tukey rule).}
#'     \item{For "zscore":}{Z-score threshold. Default 3.}
#'     \item{For "gam_residual":}{Robust-z threshold on residuals. Default 3.5.}
#'   }
#' @param center Character. Central tendency measure: \code{"median"} (default,
#'   more robust) or \code{"mean"}.
#' @param min_obs Integer. Minimum observations per group required for outlier
#'   detection. Groups with fewer observations are skipped. Default 5.
#' @param formula Optional \code{formula} for \code{method = "gam_residual"}.
#'   Default is
#'   \code{day ~ s(year) + s(alt) + s(lat) + s(s_id, bs = "re")}; any
#'   smooth term referencing a column missing from \code{pep} is silently
#'   dropped.
#' @param min_n_per_group Integer. Minimum group size required before fitting
#'   the GAM in \code{method = "gam_residual"}. Groups below this threshold
#'   fall back to the \code{"30day"} rule. Default 50.
#' @param flag_only Logical. If \code{TRUE} (default), adds outlier flag columns
#'   to data. If \code{FALSE}, removes flagged outliers from data.
#'
#' @return If \code{flag_only = TRUE}: The input data with additional columns:
#'   \describe{
#'     \item{is_outlier}{Logical. TRUE for flagged outliers.}
#'     \item{deviation}{Numeric. Days deviation from expected value.}
#'     \item{expected_doy}{Numeric. Expected DOY for the group.}
#'   }
#'   If \code{flag_only = FALSE}: Data with outliers removed.
#'
#' @details
#' The 30-day rule (Schaber & Badeck 2002) is widely used for phenological data
#' quality control. It flags observations that deviate more than 30 days from
#' the expected value for that station-species-phase combination. This threshold
#' is based on the observation that legitimate phenological variation rarely
#' exceeds one month.
#'
#' @section When to Use Each Method:
#' \describe{
#'   \item{30day}{Standard for phenological QC. Use when the 30-day biological
#'     threshold is meaningful.}
#'   \item{mad}{Robust to existing outliers. Good for initial screening of
#'     potentially contaminated data.}
#'   \item{iqr}{Standard statistical approach. Useful when comparing with
#'     other quality metrics.}
#'   \item{zscore}{Parametric approach. Use when data is approximately normal.}
#' }
#'
#' @examples
#' \donttest{
#' data(pep_seed)
#'
#' # Flag outliers using 30-day rule
#' pep_flagged <- pep_flag_outliers(pep_seed)
#' table(pep_flagged$is_outlier)
#'
#' # View flagged observations
#' pep_flagged[is_outlier == TRUE]
#'
#' # Remove outliers instead of flagging
#' pep_clean <- pep_flag_outliers(pep_seed, flag_only = FALSE)
#'
#' # Use MAD method with stricter threshold
#' pep_flagged <- pep_flag_outliers(pep_seed, method = "mad", threshold = 2.5)
#'
#' # Group by country instead of station
#' pep_flagged <- pep_flag_outliers(pep_seed,
#'                               by = c("country", "genus", "phase_id"))
#' }
#'
#' @references
#' Schaber J, Badeck F-W (2002). Evaluation of methods for the combination of
#' phenological time series and outlier detection. Tree Physiology 22:973-982.
#'
#' Rousseeuw P J (1984). Least median of squares regression.
#' \emph{Journal of the American Statistical Association} 79(388):871-880.
#' (Minimum Covariance Determinant estimator used by
#' \code{method = "mahalanobis"}.)
#'
#' @seealso \code{\link{pep_quality}} for comprehensive quality assessment,
#'   \code{\link{pheno_combine}} which uses residuals for outlier detection
#'
#' @author Matthias Templ
#' @export
pep_flag_outliers <- function(pep,
                           by = c("s_id", "genus", "species", "phase_id"),
                           method = c("30day", "mad", "iqr", "zscore",
                                      "gam_residual", "mahalanobis"),
                           threshold = NULL,
                           center = c("median", "mean"),
                           min_obs = 5,
                           formula = day ~ s(year) + s(alt) + s(lat) +
                             s(s_id, bs = "re"),
                           min_n_per_group = 50,
                           flag_only = TRUE) {

  method <- match.arg(method)
  center <- match.arg(center)

  # Set default thresholds by method
  if (is.null(threshold)) {
    threshold <- switch(method,
                        "30day" = 30,
                        "mad" = 3,
                        "iqr" = 1.5,
                        "zscore" = 3,
                        "gam_residual" = 3.5,
                        # Mahalanobis default: set per-group based on the
                        # number of phases found in the data; handled below.
                        "mahalanobis" = NA_real_)
  }

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Check required columns
  if (!"day" %in% names(pep)) {
    stop("'pep' must have a 'day' column", call. = FALSE)
  }

  # Check grouping columns exist
  by <- intersect(by, names(pep))
  if (length(by) == 0) {
    warning("No grouping columns found. Using all data as one group.", call. = FALSE)
    by <- NULL
  }


  # Make a copy and remove pep class to avoid [.pep interference
  dt <- data.table::copy(pep)
  class(dt) <- c("data.table", "data.frame")

  # Initialize outlier columns
  data.table::set(dt, j = "is_outlier", value = FALSE)
  data.table::set(dt, j = "deviation", value = NA_real_)
  data.table::set(dt, j = "expected_doy", value = NA_real_)

  # Function to detect outliers in a group
  detect_in_group <- function(doy_values) {
    n <- sum(!is.na(doy_values))
    if (n < min_obs) {
      return(list(
        is_outlier = rep(FALSE, length(doy_values)),
        deviation = rep(NA_real_, length(doy_values)),
        expected = rep(NA_real_, length(doy_values))
      ))
    }

    # Calculate center
    if (center == "median") {
      center_val <- median(doy_values, na.rm = TRUE)
    } else {
      center_val <- mean(doy_values, na.rm = TRUE)
    }

    # Calculate deviations
    dev <- doy_values - center_val

    # Detect outliers based on method
    outliers <- rep(FALSE, length(doy_values))

    if (method == "30day") {
      # Simple threshold: |deviation| > 30 days
      outliers <- abs(dev) > threshold

    } else if (method == "mad") {
      # MAD-based detection
      mad_val <- mad(doy_values, na.rm = TRUE, constant = 1.4826)
      if (mad_val > 0) {
        outliers <- abs(dev) > threshold * mad_val
      }

    } else if (method == "iqr") {
      # IQR-based detection (Tukey)
      q <- quantile(doy_values, c(0.25, 0.75), na.rm = TRUE)
      iqr_val <- q[2] - q[1]
      lower <- q[1] - threshold * iqr_val
      upper <- q[2] + threshold * iqr_val
      outliers <- doy_values < lower | doy_values > upper

    } else if (method == "zscore") {
      # Z-score based detection
      sd_val <- sd(doy_values, na.rm = TRUE)
      if (sd_val > 0) {
        z <- dev / sd_val
        outliers <- abs(z) > threshold
      }
    }

    # Handle NAs
    outliers[is.na(doy_values)] <- NA

    list(
      is_outlier = outliers,
      deviation = dev,
      expected = rep(center_val, length(doy_values))
    )
  }

  # GAM-residual detection operates on whole groups (with all covariates
  # visible at once), so we can't use the scalar detect_in_group() helper.
  # Dispatch here; other methods use the per-group univariate path below.
  if (method == "gam_residual") {
    if (!requireNamespace("mgcv", quietly = TRUE)) {
      stop("Package 'mgcv' is required for method = 'gam_residual'. ",
           "Install it with: install.packages('mgcv').",
           call. = FALSE)
    }
    .fit_group_gam <- function(sub, threshold, min_n_per_group, formula) {
      n_sub <- nrow(sub)
      fallback <- function() {
        center_val <- stats::median(sub$day, na.rm = TRUE)
        dev <- sub$day - center_val
        list(is_outlier = abs(dev) > 30,
             deviation  = dev,
             expected   = rep(center_val, n_sub))
      }
      if (n_sub < min_n_per_group) return(fallback())
      # Drop smooth terms referencing absent columns so the caller can
      # pass a generic default formula even when e.g. alt is missing.
      vars_needed <- all.vars(formula)
      missing_vars <- setdiff(vars_needed, names(sub))
      if (length(missing_vars) > 0) {
        f_terms <- attr(stats::terms(formula), "term.labels")
        keep <- vapply(f_terms,
                       function(tm) !any(missing_vars %in% all.vars(str2lang(tm))),
                       logical(1))
        f_kept <- paste(f_terms[keep], collapse = " + ")
        if (nchar(f_kept) == 0) return(fallback())
        formula <- stats::as.formula(paste("day ~", f_kept))
      }
      # s_id must be a factor for the random-intercept smooth term; silent
      # coercion is safe because it's not used as numeric elsewhere here.
      if ("s_id" %in% names(sub)) {
        sub$s_id <- factor(sub$s_id)
      }
      fit <- tryCatch(
        suppressWarnings(mgcv::gam(formula, data = sub, method = "REML")),
        error = function(e) NULL
      )
      if (is.null(fit)) return(fallback())
      res   <- as.numeric(stats::residuals(fit, type = "response"))
      fitted_vals <- as.numeric(stats::fitted(fit))
      scale <- stats::mad(res, constant = 1.4826)
      if (is.na(scale) || scale == 0) return(fallback())
      z <- res / scale
      list(is_outlier = abs(z) > threshold,
           deviation  = res,
           expected   = fitted_vals)
    }

    if (is.null(by) || length(by) == 0) {
      g <- .fit_group_gam(dt, threshold, min_n_per_group, formula)
      dt[, is_outlier := g$is_outlier]
      dt[, deviation := g$deviation]
      dt[, expected_doy := g$expected]
    } else {
      n_fallback <- 0L
      total_groups <- 0L
      dt[, c("is_outlier", "deviation", "expected_doy") := {
        total_groups <<- total_groups + 1L
        res_g <- .fit_group_gam(.SD, threshold, min_n_per_group, formula)
        if (.N < min_n_per_group) n_fallback <<- n_fallback + 1L
        list(res_g$is_outlier, res_g$deviation, res_g$expected)
      }, by = by]
      if (n_fallback > 0L) {
        message(sprintf(
          paste0("pep_flag_outliers(method = 'gam_residual'): %d of %d ",
                 "group(s) had fewer than min_n_per_group = %d observations ",
                 "and fell back to the 30-day rule."),
          n_fallback, total_groups, min_n_per_group
        ))
      }
    }
  } else if (method == "mahalanobis") {
    if (!requireNamespace("robustbase", quietly = TRUE)) {
      stop("Package 'robustbase' is required for method = 'mahalanobis'. ",
           "Install it with: install.packages('robustbase').",
           call. = FALSE)
    }
    if (!all(c("s_id", "year", "phase_id") %in% names(dt))) {
      stop("method = 'mahalanobis' requires 's_id', 'year', 'phase_id', ",
           "and 'day' columns in pep.",
           call. = FALSE)
    }

    .fit_group_mahalanobis <- function(sub, threshold) {
      n_sub <- nrow(sub)
      fallback <- function() {
        center_val <- stats::median(sub$day, na.rm = TRUE)
        dev <- sub$day - center_val
        list(is_outlier = abs(dev) > 30,
             deviation  = dev,
             expected   = rep(center_val, n_sub))
      }
      # Pivot to a (station-year) x phase wide matrix. Rows with any NA
      # phase are dropped from the covariance fit but still returned in
      # the long-format output (with NA MD).
      wide <- data.table::dcast(
        data.table::as.data.table(sub),
        s_id + year ~ phase_id,
        value.var = "day",
        fun.aggregate = mean, na.rm = TRUE
      )
      phase_cols <- setdiff(names(wide), c("s_id", "year"))
      phase_cols <- phase_cols[!grepl("^V[0-9]+$", phase_cols)]
      p <- length(phase_cols)
      if (p < 2) return(fallback())
      M <- as.matrix(wide[, phase_cols, with = FALSE])
      M[is.nan(M)] <- NA_real_
      complete <- stats::complete.cases(M)
      if (sum(complete) < max(p + 5, 15)) return(fallback())
      if (is.na(threshold)) {
        threshold <- sqrt(stats::qchisq(0.975, df = p))
      }
      fit <- tryCatch(
        robustbase::covMcd(M[complete, , drop = FALSE]),
        error = function(e) NULL
      )
      if (is.null(fit) || any(!is.finite(diag(fit$cov)))) {
        return(fallback())
      }
      diffs <- sweep(M, 2, fit$center, "-")
      inv_cov <- tryCatch(solve(fit$cov), error = function(e) NULL)
      if (is.null(inv_cov)) return(fallback())
      md <- sqrt(rowSums((diffs %*% inv_cov) * diffs))
      wide[, md := md]
      # Join the (s_id, year) MD back onto the long-format sub.
      long <- merge(
        data.table::as.data.table(sub)[, .I, by = .(s_id, year)],
        wide[, .(s_id, year, md)],
        by = c("s_id", "year"),
        all.x = TRUE
      )
      long <- long[order(I)]
      list(is_outlier = !is.na(long$md) & long$md > threshold,
           deviation  = long$md,
           expected   = rep(NA_real_, n_sub))
    }

    if (is.null(by) || length(by) == 0) {
      g <- .fit_group_mahalanobis(dt, threshold)
      dt[, is_outlier := g$is_outlier]
      dt[, deviation := g$deviation]
      dt[, expected_doy := g$expected]
    } else {
      dt[, c("is_outlier", "deviation", "expected_doy") := {
        res_g <- .fit_group_mahalanobis(.SD, threshold)
        list(res_g$is_outlier, res_g$deviation, res_g$expected)
      }, by = by]
    }
    # Record a representative threshold on the attribute (the chi-square
    # default depends on the number of phases; if user passed an explicit
    # threshold keep that, otherwise use the default for the actual
    # number of phases in the data).
    if (is.na(threshold)) {
      p_phases <- data.table::uniqueN(dt$phase_id)
      threshold <- sqrt(stats::qchisq(0.975, df = p_phases))
    }
  } else if (is.null(by) || length(by) == 0) {
    # Single group
    result <- detect_in_group(dt$day)
    dt[, is_outlier := result$is_outlier]
    dt[, deviation := result$deviation]
    dt[, expected_doy := result$expected]
  } else {
    # Process each group
    group_cols <- by

    # Use data.table grouping
    dt[, c("is_outlier", "deviation", "expected_doy") := {
      result <- detect_in_group(day)
      list(result$is_outlier, result$deviation, result$expected)
    }, by = group_cols]
  }

  # Add outlier type label
  data.table::set(dt, j = "outlier_type", value = data.table::fifelse(
    dt$is_outlier & !is.na(dt$is_outlier),
    data.table::fifelse(dt$deviation > 0, "late", "early"),
    NA_character_
  ))

  # Return flagged data or cleaned data
  if (flag_only) {
    # Add class for potential method dispatch
    class(dt) <- c("pep_outliers", class(dt))
    attr(dt, "method") <- method
    attr(dt, "threshold") <- threshold
    attr(dt, "by") <- by
    return(dt)
  } else {
    # Remove outliers
    dt_clean <- dt[is_outlier == FALSE | is.na(is_outlier)]
    dt_clean[, c("is_outlier", "deviation", "expected_doy", "outlier_type") := NULL]
    return(dt_clean)
  }
}


#' Print Method for Outlier Detection Results
#'
#' @param x A \code{pep_outliers} object
#' @param ... Additional arguments (unused)
#' @return Invisibly returns \code{x}
#' @export
print.pep_outliers <- function(x, ...) {
  method <- attr(x, "method")
  threshold <- attr(x, "threshold")
  by <- attr(x, "by")

  cat("Phenological Outlier Detection Results\n")
  cat(strrep("=", 45), "\n")

  if (!is.null(method) && length(method) == 1) {
    cat("Method:", method, "\n")
    cat("Threshold:", threshold,
        switch(method,
               "30day" = "days",
               "mad" = "MADs",
               "iqr" = "IQRs",
               "zscore" = "SDs"), "\n")
  }
  if (!is.null(by)) {
    cat("Grouping:", paste(by, collapse = ", "), "\n")
  }

  cat("\n")
  n_total <- nrow(x)
  n_outliers <- sum(x$is_outlier, na.rm = TRUE)
  n_early <- sum(x$outlier_type == "early", na.rm = TRUE)
  n_late <- sum(x$outlier_type == "late", na.rm = TRUE)

  cat(sprintf("Total observations: %s\n", format(n_total, big.mark = ",")))
  cat(sprintf("Outliers flagged: %d (%.2f%%)\n", n_outliers, 100 * n_outliers / n_total))
  cat(sprintf("  - Early outliers: %d\n", n_early))
  cat(sprintf("  - Late outliers: %d\n", n_late))

  if (n_outliers > 0) {
    cat("\nDeviation summary for outliers:\n")
    outlier_dev <- x$deviation[x$is_outlier == TRUE]
    cat(sprintf("  Min: %.1f days\n", min(outlier_dev, na.rm = TRUE)))
    cat(sprintf("  Max: %.1f days\n", max(outlier_dev, na.rm = TRUE)))
    cat(sprintf("  Mean |deviation|: %.1f days\n", mean(abs(outlier_dev), na.rm = TRUE)))
  }

  invisible(x)
}


#' Summary Method for Outlier Detection Results
#'
#' @param object A \code{pep_outliers} object
#' @param ... Additional arguments (unused)
#' @return Invisibly returns a summary data.table
#' @export
summary.pep_outliers <- function(object, ...) {
  by <- attr(object, "by")

  cat("Phenological Outlier Summary\n")
  cat(strrep("=", 45), "\n\n")

  # Overall summary
  n_total <- nrow(object)
  n_outliers <- sum(object$is_outlier, na.rm = TRUE)

  cat(sprintf("Total observations: %s\n", format(n_total, big.mark = ",")))
  cat(sprintf("Total outliers: %d (%.2f%%)\n\n", n_outliers, 100 * n_outliers / n_total))

  # Summary by group if grouping was used
  if (!is.null(by) && length(by) > 0) {
    group_summary <- object[, .(
      n_obs = .N,
      n_outliers = sum(is_outlier, na.rm = TRUE),
      pct_outliers = round(100 * sum(is_outlier, na.rm = TRUE) / .N, 2),
      mean_dev = round(mean(abs(deviation), na.rm = TRUE), 1)
    ), by = by][order(-n_outliers)]

    # Remove pep_outliers class to avoid recursion in print
    class(group_summary) <- class(group_summary)[class(group_summary) != "pep_outliers"]

    cat("Groups with most outliers:\n")
    print(head(group_summary[n_outliers > 0], 10))
  }

  # Distribution of deviations
  cat("\nDeviation distribution (all data):\n")
  dev_quantiles <- quantile(object$deviation, c(0.01, 0.05, 0.25, 0.5, 0.75, 0.95, 0.99),
                            na.rm = TRUE)
  print(round(dev_quantiles, 1))

  invisible(object)
}


#' Plot Method for Outlier Detection Results
#'
#' @param x A \code{pep_outliers} object
#' @param type Character. Plot type: "histogram" for deviation distribution,
#'   "scatter" for DOY vs expected, "timeline" for outliers over time.
#' @param ... Additional arguments
#' @return A ggplot object (invisibly)
#' @export
plot.pep_outliers <- function(x, type = c("histogram", "scatter", "timeline"), ...) {
  type <- match.arg(type)
  threshold <- attr(x, "threshold")
  method <- attr(x, "method")

  if (type == "histogram") {
    # Deviation distribution
    p <- ggplot2::ggplot(x, ggplot2::aes(x = deviation, fill = is_outlier)) +
      ggplot2::geom_histogram(bins = 50, alpha = 0.9, position = "stack") +
      ggplot2::scale_fill_manual(values = c("FALSE" = "steelblue", "TRUE" = "red"),
                                  labels = c("Normal", "Outlier"),
                                  na.value = "gray50") +
      ggplot2::labs(
        title = "Distribution of Deviations from Expected DOY",
        subtitle = sprintf("Method: %s, Threshold: %s", method, threshold),
        x = "Deviation (days)",
        y = "Count",
        fill = ""
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(legend.position = "bottom")

    if (method == "30day") {
      p <- p +
        ggplot2::geom_vline(xintercept = c(-threshold, threshold),
                            linetype = "dashed", color = "red", alpha = 0.7)
    }

  } else if (type == "scatter") {
    # Observed vs expected DOY
    p <- ggplot2::ggplot(x, ggplot2::aes(x = expected_doy, y = day, color = is_outlier)) +
      ggplot2::geom_point(alpha = 0.5, size = 1) +
      ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed") +
      ggplot2::scale_color_manual(values = c("FALSE" = "steelblue", "TRUE" = "red"),
                                   labels = c("Normal", "Outlier"),
                                   na.value = "gray50") +
      ggplot2::labs(
        title = "Observed vs Expected Day of Year",
        x = "Expected DOY (group median/mean)",
        y = "Observed DOY",
        color = ""
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(legend.position = "bottom")

  } else if (type == "timeline") {
    # Outliers over time
    if (!"year" %in% names(x)) {
      stop("'year' column required for timeline plot", call. = FALSE)
    }

    year_summary <- x[, .(
      n_obs = .N,
      n_outliers = sum(is_outlier, na.rm = TRUE),
      pct_outliers = 100 * sum(is_outlier, na.rm = TRUE) / .N
    ), by = year][order(year)]

    p <- ggplot2::ggplot(year_summary, ggplot2::aes(x = year, y = pct_outliers)) +
      ggplot2::geom_line(color = "steelblue", linewidth = 1) +
      ggplot2::geom_point(ggplot2::aes(size = n_obs), color = "steelblue", alpha = 0.7) +
      ggplot2::labs(
        title = "Outlier Percentage Over Time",
        x = "Year",
        y = "Outliers (%)",
        size = "Observations"
      ) +
      ggplot2::theme_minimal()
  }

  print(p)
  invisible(p)
}

test_that("pep_flag_outliers returns pep_outliers class with flag columns", {
  data(pep_seed)
  result <- pep_flag_outliers(pep_seed)
  expect_s3_class(result, "pep_outliers")
  expect_true(all(c("is_outlier", "deviation", "expected_doy") %in% names(result)))
})

test_that("pep_flag_outliers 30day method flags large deviations", {
  data(pep_seed)
  result <- pep_flag_outliers(pep_seed, method = "30day")
  outliers <- result[result$is_outlier == TRUE & !is.na(result$is_outlier), ]
  if (nrow(outliers) > 0) {
    expect_true(all(abs(outliers$deviation) > 30))
  }
})

test_that("pep_flag_outliers mad method works", {
  data(pep_seed)
  result <- pep_flag_outliers(pep_seed, method = "mad")
  expect_s3_class(result, "pep_outliers")
  expect_equal(attr(result, "method"), "mad")
})

test_that("pep_flag_outliers iqr method works", {
  data(pep_seed)
  result <- pep_flag_outliers(pep_seed, method = "iqr")
  expect_s3_class(result, "pep_outliers")
  expect_equal(attr(result, "method"), "iqr")
})

test_that("pep_flag_outliers flag_only=FALSE removes outliers", {
  data(pep_seed)
  flagged <- pep_flag_outliers(pep_seed, method = "30day")
  cleaned <- pep_flag_outliers(pep_seed, method = "30day", flag_only = FALSE)
  n_outliers <- sum(flagged$is_outlier, na.rm = TRUE)
  if (n_outliers > 0) {
    expect_true(nrow(cleaned) < nrow(flagged))
  }
  expect_false("is_outlier" %in% names(cleaned))
})

test_that("pep_flag_outliers errors on non-data.frame", {
  expect_error(pep_flag_outliers("not a df"), "must be a data.frame")
})

make_multiphase_fixture <- function(seed = 1, n_stations = 40, n_years = 25,
                                    contaminate = TRUE) {
  set.seed(seed)
  stations <- data.table::data.table(
    s_id = seq_len(n_stations),
    alt  = runif(n_stations, 200, 2000),
    lat  = runif(n_stations, 45, 52),
    lon  = runif(n_stations, 5, 15)
  )
  years <- 1990:(1990 + n_years - 1)
  phases <- c(60L, 65L, 89L)   # three BBCH phases: start/peak/senescence
  grid <- data.table::CJ(s_id = stations$s_id, year = years, phase_id = phases)
  dat  <- merge(grid, stations, by = "s_id")

  # True phenology: each phase has a different baseline, with a small
  # correlated shift per station-year (shared weather signal) and i.i.d.
  # per-phase noise. The correlation across phases is the multivariate
  # signal that Mahalanobis should exploit.
  ref_by_phase <- c(`60` = 110, `65` = 120, `89` = 250)
  sy <- data.table::CJ(s_id = stations$s_id, year = years)
  sy[, shift := rnorm(.N, sd = 4)]
  dat <- merge(dat, sy, by = c("s_id", "year"))
  dat[, day := ref_by_phase[as.character(phase_id)] +
        0.02 * alt + shift + rnorm(.N, sd = 2)]
  dat[, `:=`(genus = "Malus", species = "Malus domestica", shift = NULL)]

  if (contaminate) {
    # Contamination target: station 1 in year 2000 has BBCH 60 = normal
    # (early spring), but BBCH 65 appears only 1 day later (impossible -
    # full-flowering is typically ~10 days after first-flowering). The
    # *marginal* DOY for phase 60 is fine; for phase 65 it's unusually
    # *early*. Neither is extreme on its own scale but the JOINT (60, 65)
    # vector is an outlier after accounting for the natural correlation.
    dat[s_id == 1 & year == 2000 & phase_id == 65L,
        day := dat[s_id == 1 & year == 2000 & phase_id == 60L, day] + 1]
  }
  dat[]
}

test_that("mahalanobis method flags a multivariate-but-marginal-OK outlier", {
  skip_if_not_installed("robustbase")
  dat <- make_multiphase_fixture(seed = 1)
  res <- pep_flag_outliers(
    dat,
    by = c("genus", "species"),
    method = "mahalanobis",
    threshold = sqrt(qchisq(0.975, df = 3))
  )
  expect_s3_class(res, "pep_outliers")
  # The contaminated station-year must be flagged on all its phase rows
  # (Mahalanobis flags the whole station-year vector, so every
  # observation inside that station-year inherits the flag).
  hits <- res[s_id == 1 & year == 2000, is_outlier]
  expect_true(all(hits))
  # A randomly-chosen clean station-year should not be flagged.
  clean <- res[s_id == 10 & year == 2005, is_outlier]
  expect_false(any(clean))
})

test_that("mahalanobis stores per-observation MD in deviation column", {
  skip_if_not_installed("robustbase")
  dat <- make_multiphase_fixture(seed = 2)
  res <- pep_flag_outliers(
    dat,
    by = c("genus", "species"),
    method = "mahalanobis",
    threshold = sqrt(qchisq(0.975, df = 3))
  )
  # Every row in the same station-year shares the same MD value.
  sy_mds <- res[, .(u = data.table::uniqueN(deviation)), by = .(s_id, year)]
  expect_true(all(sy_mds$u == 1))
  # MDs are non-negative and approximately chi-distributed in their bulk.
  expect_true(all(res$deviation >= 0, na.rm = TRUE))
  expect_lt(stats::median(res$deviation, na.rm = TRUE),
            sqrt(qchisq(0.5, df = 3)) * 3)
})

test_that("mahalanobis falls back gracefully on incomplete / degenerate groups", {
  skip_if_not_installed("robustbase")
  dat <- make_multiphase_fixture(seed = 3, n_stations = 5, n_years = 5)
  # 75 observations across 3 phases -> 25 station-years; covMcd will
  # usually fit, but the group is too small per-phase for a meaningful
  # robust covariance. Should return a valid pep_outliers object without
  # erroring.
  expect_no_error(
    res <- pep_flag_outliers(
      dat,
      by = c("genus", "species"),
      method = "mahalanobis"
    )
  )
  expect_s3_class(res, "pep_outliers")
})

test_that("mahalanobis respects flag_only = FALSE", {
  skip_if_not_installed("robustbase")
  dat <- make_multiphase_fixture(seed = 4)
  n_before <- nrow(dat)
  cleaned <- pep_flag_outliers(
    dat,
    by = c("genus", "species"),
    method = "mahalanobis",
    flag_only = FALSE
  )
  expect_lt(nrow(cleaned), n_before)
  expect_false("is_outlier" %in% names(cleaned))
})

test_that("pep_plot_outliers(type = 'profile') renders a ggplot for Mahalanobis", {
  skip_if_not_installed("robustbase")
  dat <- make_multiphase_fixture(seed = 7)
  res <- pep_flag_outliers(
    dat,
    by = c("genus", "species"),
    method = "mahalanobis"
  )
  p <- pep_plot_outliers(res, type = "profile")
  expect_s3_class(p, "ggplot")
})

test_that("pep_plot_outliers(type = 'diagnostic') handles Mahalanobis without expected_doy", {
  skip_if_not_installed("robustbase")
  dat <- make_multiphase_fixture(seed = 8)
  res <- pep_flag_outliers(
    dat,
    by = c("genus", "species"),
    method = "mahalanobis"
  )
  # Must not error even though expected_doy is all NA.
  expect_no_error(p <- pep_plot_outliers(res, type = "diagnostic"))
  # The diagnostic is a patchwork composition.
  expect_s3_class(p, "patchwork")
})

test_that("attr(result, 'threshold') is populated for Mahalanobis", {
  skip_if_not_installed("robustbase")
  dat <- make_multiphase_fixture(seed = 9)
  res <- pep_flag_outliers(
    dat,
    by = c("genus", "species"),
    method = "mahalanobis"
  )
  expect_true(is.finite(attr(res, "threshold")))
  # With 3 phases, the default cut-off is sqrt(qchisq(0.975, df = 3)).
  expect_equal(
    attr(res, "threshold"),
    sqrt(qchisq(0.975, df = 3)),
    tolerance = 1e-6
  )
})

# --- Fixture: two populations of stations, one contaminated observation ---
make_phenology_fixture <- function(seed = 1, n_stations = 30, n_years = 30,
                                   contaminate = TRUE) {
  set.seed(seed)
  stations <- data.table::data.table(
    s_id = seq_len(n_stations),
    alt  = runif(n_stations, 200, 2000),
    lat  = runif(n_stations, 45, 52),
    lon  = runif(n_stations, 5, 15)
  )
  years <- 1990:(1990 + n_years - 1)
  grid <- data.table::CJ(s_id = stations$s_id, year = years)
  dat <- merge(grid, stations, by = "s_id")
  # True phenology model: day depends on year (trend), altitude, latitude,
  # plus per-station random intercept and small noise.
  station_re <- rnorm(n_stations, sd = 3)
  dat[, day := 110 +
        0.015 * alt +                       # later at higher elevation
        1.2   * (lat - 47) +                # later further north
        -0.05 * (year - 1990) +             # trend: earlier over time
        station_re[s_id] +                  # station random intercept
        rnorm(.N, sd = 4)]                  # i.i.d. noise
  dat[, `:=`(genus = "Malus", species = "Malus domestica", phase_id = 60L)]
  if (contaminate) {
    # Inject a clearly anomalous (s_id = 1, year = 2000) observation 50 days
    # late — consistent with the station's own history (so the classic
    # 30-day-from-group-median rule may miss it if the station itself is
    # high-altitude and therefore late by a lot to begin with) but
    # inconsistent with the global covariate-driven expectation.
    dat[s_id == 1 & year == 2000, day := day + 50]
    # Also inject one "normal-looking-in-isolation" but covariate-inconsistent
    # point: a lowland station with a high-altitude DOY.
    dat[s_id == 2 & year == 2005, day := day + 30]
  }
  dat[]
}

test_that("gam_residual method flags a covariate-inconsistent outlier that 30day misses", {
  skip_if_not_installed("mgcv")
  dat <- make_phenology_fixture(seed = 1)
  # Use by = c(genus, species, phase_id) so the GAM pools across all stations.
  res_gam <- pep_flag_outliers(
    dat,
    by = c("genus", "species", "phase_id"),
    method = "gam_residual",
    threshold = 3.5
  )
  expect_s3_class(res_gam, "pep_outliers")
  expect_true("is_outlier" %in% names(res_gam))
  expect_true("deviation"  %in% names(res_gam))

  # The injected (s_id=1, year=2000) contamination must be flagged.
  hit <- res_gam[s_id == 1 & year == 2000, is_outlier]
  expect_true(isTRUE(hit))
})

test_that("gam_residual uses residuals from the supplied formula", {
  skip_if_not_installed("mgcv")
  dat <- make_phenology_fixture(seed = 2)
  res <- pep_flag_outliers(
    dat,
    by = c("genus", "species", "phase_id"),
    method = "gam_residual",
    formula = day ~ s(year) + s(alt) + s(lat) + s(s_id, bs = "re"),
    threshold = 3.5
  )
  # The stored deviation should now represent GAM residuals, not
  # (day - station_median). Under a reasonable GAM, most residuals
  # should be small (|dev| < 15 days) except for the contamination.
  clean <- res[!(s_id %in% c(1, 2) & year %in% c(2000, 2005))]
  expect_lt(quantile(abs(clean$deviation), 0.95, na.rm = TRUE), 15)
})

test_that("gam_residual falls back to 30day when group is too small", {
  skip_if_not_installed("mgcv")
  dat <- make_phenology_fixture(seed = 3, n_stations = 3, n_years = 4)
  # 12 total observations: below min_n_per_group default -> fallback.
  expect_message(
    res <- pep_flag_outliers(
      dat,
      by = c("genus", "species", "phase_id"),
      method = "gam_residual",
      min_n_per_group = 50
    ),
    "fall|fallback|too few|fewer"
  )
  # Result is still a valid pep_outliers object with the usual columns.
  expect_s3_class(res, "pep_outliers")
  expect_true(all(c("is_outlier", "deviation", "expected_doy") %in% names(res)))
})

test_that("gam_residual respects flag_only = FALSE and removes outliers", {
  skip_if_not_installed("mgcv")
  dat <- make_phenology_fixture(seed = 4)
  n_before <- nrow(dat)
  cleaned <- pep_flag_outliers(
    dat,
    by = c("genus", "species", "phase_id"),
    method = "gam_residual",
    flag_only = FALSE,
    threshold = 3.5
  )
  expect_lt(nrow(cleaned), n_before)
  expect_false("is_outlier" %in% names(cleaned))
})

test_that("gam_residual: attr(result, 'method') == 'gam_residual'", {
  skip_if_not_installed("mgcv")
  dat <- make_phenology_fixture(seed = 5)
  res <- pep_flag_outliers(
    dat,
    by = c("genus", "species", "phase_id"),
    method = "gam_residual",
    threshold = 3.5
  )
  expect_equal(attr(res, "method"), "gam_residual")
})

# Prevent R CMD check notes
utils::globalVariables(c("day", "year", "s_id", "is_outlier", "deviation",
                         "expected_doy", "outlier_type", "lon", "lat",
                         "phase_id", "genus", "species", "month", "season",
                         "n_outliers", "pct_outliers", "n_obs", "country",
                         "outlier_category", "..density..",
                         "mean_deviation", "total_outliers",
                         "alt", "theoretical", "sample", "max_abs_dev",
                         "abs_dev",
                         # diagnostic (Mahalanobis branch)
                         "md", "flagged", "mean_md", "max_md", "z",
                         # profile
                         "sy_flagged", "sy_key", "phase_f"))

#' Visualize Phenological Outliers for Inspection
#'
#' Creates diagnostic plots to help distinguish between data errors and
#' biologically meaningful extreme events (e.g., second flowering). Multiple
#' plot types reveal different aspects of outlier patterns.
#'
#' @param x A \code{pep_outliers} object from \code{\link{pep_flag_outliers}}.
#' @param type Character. Type of plot to produce:
#'   \describe{
#'     \item{"overview"}{(Default) Multi-panel overview with distribution,
#'       seasonal pattern, and summary statistics.}
#'     \item{"seasonal"}{Distribution of outliers by month/season, useful for
#'       detecting second flowering (late-season outliers for spring phases).}
#'     \item{"map"}{Geographic distribution of outliers (requires lon/lat).}
#'     \item{"detail"}{Detailed view of individual outlier events with context.}
#'     \item{"station"}{Station-level outlier patterns over time.}
#'     \item{"doy_context"}{Shows outliers in context of full DOY distribution
#'       per phase, highlighting potential second events.}
#'     \item{"diagnostic"}{Paper- and vignette-ready 4-panel figure of
#'       model fit quality: (A) residuals vs fitted; (B) Q-Q plot of
#'       residuals; (C) |residual| vs altitude (if available) or year;
#'       (D) spatial map of maximum |residual| per station. Designed to
#'       accompany \code{method = "gam_residual"} but usable for every
#'       method (uses the \code{deviation} column as the residual). For
#'       \code{method = "mahalanobis"} the panels automatically switch
#'       to MD-specific diagnostics (sorted MD with chi-square
#'       threshold; Q-Q against \eqn{\chi^2_p}; mean / max MD over time;
#'       per-station worst-case MD map).}
#'     \item{"profile"}{Parallel-coordinates plot of the phase profile
#'       (one line per station-year across phases), with flagged
#'       station-years highlighted and the robust central profile
#'       overlaid. Designed for \code{method = "mahalanobis"} where the
#'       outlierness lives in the *shape* of a station-year across
#'       phases, not in any single DOY. The primary paper figure for
#'       the multivariate method.}
#'   }
#' @param phase_id Optional integer vector to filter specific phases for plotting.
#' @param outlier_only Logical. If TRUE (default for some types), show only
#'   outlier observations. If FALSE, show all data with outliers highlighted.
#' @param late_threshold Integer. DOY threshold for classifying "late" outliers
#'   as potential second events (default 250, ~September 7).
#' @param n_top Integer. For detail view, number of most extreme outliers to show.
#'   Default 20.
#' @param ... Additional arguments passed to plotting functions.
#'
#' @return A \code{ggplot} object (or list of ggplots for "overview").
#'
#' @details
#' This function is designed to support quality control workflows where the

#' goal is to distinguish:
#' \enumerate{
#'   \item \strong{Data errors}: Typos, wrong dates, mis-coded phases
#'   \item \strong{Observer errors}: Misidentified species or phases
#'   \item \strong{Biologically meaningful extremes}: Second flowering, delayed
#'     development, climate-driven anomalies
#' }
#'
#' For detecting potential \strong{second flowering events}:
#' \itemize{
#'   \item Use \code{type = "seasonal"} to see if late-season outliers cluster
#'     in autumn months
#'   \item Use \code{type = "doy_context"} to visualize outliers relative to
#'     the main flowering distribution
#'   \item Focus on flowering phases (60, 65) with late DOY values (>250)
#' }
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Use Swiss subset for faster outlier detection
#' pep_ch <- pep[country == "Switzerland"]
#' pep_flagged <- pep_flag_outliers(pep_ch)
#'
#' # Overview of all outliers
#' pep_plot_outliers(pep_flagged, type = "overview")
#'
#' # Seasonal distribution - look for autumn outliers in flowering phases
#' pep_plot_outliers(pep_flagged, type = "seasonal", phase_id = c(60, 65))
#'
#' # Geographic pattern
#' pep_plot_outliers(pep_flagged, type = "map")
#'
#' # Detailed view of most extreme outliers
#' pep_plot_outliers(pep_flagged, type = "detail", n_top = 30)
#'
#' # DOY context for flowering phase - spot potential second flowering
#' pep_plot_outliers(pep_flagged, type = "doy_context", phase_id = 65)
#' }
#'
#' @seealso \code{\link{pep_flag_outliers}} for detecting outliers,
#'   \code{\link{pep_quality}} for comprehensive quality assessment
#'
#' @author Matthias Templ
#' @export
#' @import ggplot2
pep_plot_outliers <- function(x,
                          type = c("overview", "seasonal", "map", "detail",
                                   "station", "doy_context", "diagnostic",
                                   "profile"),
                          phase_id = NULL,
                          outlier_only = NULL,
                          late_threshold = 250,
                          n_top = 20,
                          ...) {

  type <- match.arg(type)

  # Validate input

  if (!inherits(x, "pep_outliers")) {
    stop("'x' must be a pep_outliers object from pep_flag_outliers()", call. = FALSE)
  }

  # Get attributes
  method <- attr(x, "method")
  threshold <- attr(x, "threshold")

  # Filter by phase if requested
  if (!is.null(phase_id)) {
    if (!"phase_id" %in% names(x)) {
      warning("'phase_id' column not found, ignoring filter", call. = FALSE
      )
    } else {
      x <- x[x$phase_id %in% phase_id, ]
    }
  }

  # Add helper columns
  dt <- data.table::copy(x)

  # Add month from DOY (approximate)
  dt[, month := ceiling(day / 30.44)]
  dt[month > 12, month := 12]
  dt[month < 1, month := 1]

  # Add season
  dt[, season := data.table::fcase(
    month %in% c(12, 1, 2), "Winter",
    month %in% 3:5, "Spring",
    month %in% 6:8, "Summer",
    month %in% 9:11, "Autumn"
  )]
  dt[, season := factor(season, levels = c("Spring", "Summer", "Autumn", "Winter"))]

  # Categorize outliers for potential second events
  dt[, outlier_category := data.table::fcase(
    !is_outlier | is.na(is_outlier), "Normal",
    is_outlier & deviation < 0, "Early outlier",
    is_outlier & deviation > 0 & day < late_threshold, "Late outlier",
    is_outlier & deviation > 0 & day >= late_threshold, "Very late (potential 2nd event)"
  )]
  dt[, outlier_category := factor(outlier_category,
                                   levels = c("Normal", "Early outlier",
                                              "Late outlier", "Very late (potential 2nd event)"))]

  # ============================================================
  # Plot type: OVERVIEW
  # ============================================================
  if (type == "overview") {
    return(pep_plot_outliers_overview(dt, method, threshold, late_threshold))
  }

  # ============================================================
  # Plot type: SEASONAL
  # ============================================================
  if (type == "seasonal") {
    return(pep_plot_outliers_seasonal(dt, late_threshold))
  }

  # ============================================================
  # Plot type: MAP
  # ============================================================
  if (type == "map") {
    return(pep_plot_outliers_map(dt))
  }

  # ============================================================
  # Plot type: DETAIL
  # ============================================================
  if (type == "detail") {
    return(pep_plot_outliers_detail(dt, n_top))
  }

  # ============================================================
  # Plot type: STATION
  # ============================================================
  if (type == "station") {
    return(pep_plot_outliers_station(dt))
  }

  # ============================================================
  # Plot type: DOY_CONTEXT
  # ============================================================
  if (type == "doy_context") {
    return(pep_plot_outliers_doy_context(dt, late_threshold))
  }

  # ============================================================
  # Plot type: DIAGNOSTIC (paper/vignette figure)
  # ============================================================
  if (type == "diagnostic") {
    return(pep_plot_outliers_diagnostic(dt, method, threshold))
  }

  # ============================================================
  # Plot type: PROFILE (phase-profile parallel coordinates)
  # ============================================================
  if (type == "profile") {
    return(pep_plot_outliers_profile(dt, method, threshold))
  }
}


#' @keywords internal
pep_plot_outliers_profile <- function(dt, method, threshold) {
  # Parallel-coordinates plot of phase profiles. Each station-year
  # contributes one line across the BBCH phases; flagged station-years
  # (by any detector, but this plot is designed for Mahalanobis) are
  # highlighted in red.
  if (!all(c("phase_id", "day", "s_id", "year") %in% names(dt))) {
    stop("Profile plot requires 's_id', 'year', 'phase_id', 'day' columns.",
         call. = FALSE)
  }

  dt <- data.table::copy(data.table::as.data.table(dt))
  # is_outlier may be NA for rows the detector didn't reach; treat as FALSE
  # for the plot (we want a single flag per station-year).
  dt[is.na(is_outlier), is_outlier := FALSE]

  # Per station-year: flagged if ANY phase row is flagged.
  dt[, sy_flagged := any(is_outlier), by = .(s_id, year)]
  dt[, sy_key := paste(s_id, year, sep = "_")]
  dt[, phase_f := factor(phase_id)]

  # Robust per-phase centre (median across all station-years) as reference
  # — this is what the MCD estimator is (approximately) targeting for the
  # centre of the ellipsoid.
  ref <- dt[, .(day = stats::median(day, na.rm = TRUE)), by = phase_f]
  ref[, sy_key := "__reference__"]
  ref[, sy_flagged := FALSE]
  ref[, is_outlier := FALSE]

  # Draw flagged on top of non-flagged; reference as a thick black line.
  non_flag <- dt[sy_flagged == FALSE]
  flag     <- dt[sy_flagged == TRUE]

  p <- ggplot(non_flag,
              aes(x = phase_f, y = day, group = sy_key)) +
    geom_line(color = "gray70", alpha = 0.35, linewidth = 0.3) +
    geom_line(data = flag,
              aes(x = phase_f, y = day, group = sy_key),
              color = "red", alpha = 0.8, linewidth = 0.7) +
    geom_line(data = ref,
              aes(x = phase_f, y = day, group = sy_key),
              color = "black", linewidth = 1.2, alpha = 0.9) +
    geom_point(data = ref,
               aes(x = phase_f, y = day),
               color = "black", size = 2.5) +
    labs(
      x = "BBCH phase",
      y = "Day of year (DOY)",
      title = "Phase-profile parallel coordinates",
      subtitle = sprintf(
        paste0("%d station-years (red = flagged%s; black = robust ",
               "median profile)"),
        data.table::uniqueN(dt[, .(s_id, year)]),
        if (identical(method, "mahalanobis"))
          sprintf(", MD threshold = %s",
                  format(threshold, digits = 3)) else "")
    ) +
    theme_minimal(base_size = 11) +
    theme(plot.title = element_text(face = "bold"),
          plot.subtitle = element_text(color = "gray30", size = 9))

  # Facet by species if available and > 1 species.
  if ("species" %in% names(dt) &&
      data.table::uniqueN(dt$species) > 1) {
    p <- p + facet_wrap(~species, scales = "free_y")
  }
  p
}


#' @keywords internal
pep_plot_outliers_diagnostic <- function(dt, method, threshold) {
  # Paper-ready 4-panel diagnostic. For the Mahalanobis method the
  # `deviation` column holds unitless distances (not residuals in days)
  # and `expected_doy` is NA, so the residual/fitted/Q-Q axes don't
  # apply — we replace them with panels tailored to the MD output.
  is_mahal <- identical(method, "mahalanobis")
  dt <- data.table::copy(dt)
  dt[, abs_dev := abs(deviation)]

  if (is_mahal) {
    # --- Mahalanobis-specific diagnostic ---------------------------------
    # Panel A: sorted MD with the chi-square-based threshold highlighted.
    sy <- dt[, .(md = deviation[1],
                 flagged = any(is_outlier, na.rm = TRUE)),
             by = .(s_id, year)]
    sy <- sy[order(md)]
    sy[, rank := seq_len(.N)]
    pA <- ggplot(sy, aes(x = rank, y = md, color = flagged)) +
      geom_point(alpha = 0.6, size = 0.8) +
      geom_hline(yintercept = threshold, linetype = "dashed",
                 color = "red", linewidth = 0.5) +
      scale_color_manual(values = c(`TRUE` = "red", `FALSE` = "gray50"),
                         na.value = "gray80") +
      labs(x = "Station-year rank (sorted)",
           y = "Robust Mahalanobis distance",
           title = "A. Sorted MD with chi-sq threshold") +
      theme_minimal(base_size = 10) +
      theme(legend.position = "none",
            plot.title = element_text(face = "bold", size = 10))

    # Panel B: MD^2 vs chi-square quantiles (Q-Q plot of squared MD).
    # Under multivariate normality MD^2 ~ chi^2_p. We don't know p for
    # sure here; infer it from the number of distinct phases.
    p_dim <- data.table::uniqueN(dt$phase_id)
    md_sq <- sort(sy$md[is.finite(sy$md)]^2)
    qq_df <- data.frame(
      sample = md_sq,
      theoretical = stats::qchisq(stats::ppoints(length(md_sq)), df = p_dim)
    )
    pB <- ggplot(qq_df, aes(x = theoretical, y = sample)) +
      geom_point(alpha = 0.4, size = 0.7, color = "steelblue") +
      geom_abline(slope = 1, intercept = 0,
                  linetype = "dashed", color = "gray40") +
      labs(x = sprintf("Theoretical chi^2_%d quantiles", p_dim),
           y = "Observed MD^2",
           title = sprintf("B. Q-Q against chi^2_%d", p_dim)) +
      theme_minimal(base_size = 10) +
      theme(plot.title = element_text(face = "bold", size = 10))

    # Panel C: MD over time, aggregated to per-year mean (highlights
    # epochs with anomalous observations across the network).
    if ("year" %in% names(dt)) {
      yr <- sy[, .(mean_md = mean(md, na.rm = TRUE),
                   max_md  = max(md, na.rm = TRUE),
                   n_flag  = sum(flagged)), by = year][order(year)]
      pC <- ggplot(yr, aes(x = year)) +
        geom_line(aes(y = mean_md), color = "steelblue", linewidth = 0.6) +
        geom_line(aes(y = max_md),  color = "red",
                  linewidth = 0.4, linetype = "dotted") +
        geom_hline(yintercept = threshold, linetype = "dashed",
                   color = "red", linewidth = 0.3) +
        labs(x = "Year", y = "MD (mean / max per year)",
             title = "C. Mean and max MD over time") +
        theme_minimal(base_size = 10) +
        theme(plot.title = element_text(face = "bold", size = 10))
    } else {
      pC <- ggplot() + theme_void() + labs(title = "C. year unavailable")
    }
  } else {
    # --- Residual-based methods (default path) --------------------------
    # Panel A: residual vs fitted (expected DOY).
    if (all(!is.na(dt$expected_doy))) {
      pA <- ggplot(dt, aes(x = expected_doy, y = deviation)) +
        geom_point(aes(color = is_outlier), alpha = 0.35, size = 0.8) +
        geom_hline(yintercept = 0, linetype = "dashed", color = "gray40") +
        ggplot2::geom_smooth(method = "loess", se = FALSE,
                             color = "steelblue", linewidth = 0.6) +
        scale_color_manual(values = c("FALSE" = "gray50", "TRUE" = "red"),
                           labels = c("Normal", "Flagged"),
                           na.value = "gray80") +
        labs(x = "Fitted / expected DOY", y = "Residual (days)",
             color = NULL,
             title = "A. Residuals vs fitted") +
        theme_minimal(base_size = 10) +
        theme(legend.position = "none",
              plot.title = element_text(face = "bold", size = 10))
    } else {
      pA <- ggplot() + theme_void() +
        labs(title = "A. Residuals vs fitted: not available")
    }

    # Panel B: Q-Q plot of standardised residuals against Normal.
    dev_std <- (dt$deviation - stats::median(dt$deviation, na.rm = TRUE)) /
      stats::mad(dt$deviation, na.rm = TRUE)
    qq_df <- data.frame(z = sort(dev_std[is.finite(dev_std)]))
    qq_df$theoretical <- stats::qnorm(stats::ppoints(nrow(qq_df)))
    pB <- ggplot(qq_df, aes(x = theoretical, y = z)) +
      geom_point(alpha = 0.3, size = 0.7, color = "steelblue") +
      geom_abline(slope = 1, intercept = 0,
                  linetype = "dashed", color = "gray40") +
      labs(x = "Theoretical N(0,1) quantiles",
           y = "Robust-z of residuals",
           title = "B. Q-Q: residual tail behaviour") +
      theme_minimal(base_size = 10) +
      theme(plot.title = element_text(face = "bold", size = 10))

    # Panel C: |residual| vs altitude (if present) or year (fallback).
    covariate <- if ("alt" %in% names(dt) && any(!is.na(dt$alt))) "alt" else
      if ("year" %in% names(dt)) "year" else NULL
    if (!is.null(covariate)) {
      pC <- ggplot(dt, aes(x = .data[[covariate]], y = abs_dev)) +
        geom_point(aes(color = is_outlier), alpha = 0.3, size = 0.7) +
        ggplot2::geom_smooth(method = "loess", se = FALSE,
                             color = "steelblue", linewidth = 0.6) +
        scale_color_manual(values = c("FALSE" = "gray50", "TRUE" = "red"),
                           na.value = "gray80") +
        labs(x = switch(covariate,
                        alt = "Altitude (m)",
                        year = "Year"),
             y = "|Residual| (days)",
             title = sprintf("C. |residual| vs %s",
                             switch(covariate, alt = "altitude", year = "year"))) +
        theme_minimal(base_size = 10) +
        theme(legend.position = "none",
              plot.title = element_text(face = "bold", size = 10))
    } else {
      pC <- ggplot() + theme_void() +
        labs(title = "C. covariate unavailable")
    }
  }

  # Panel D: spatial map of max |residual| (or max MD) per station.
  d_label <- if (is_mahal) "Max robust MD" else "Max |residual|"
  d_title <- if (is_mahal) "D. Per-station worst-case MD"
             else         "D. Per-station worst-case residual"
  if (all(c("lon", "lat") %in% names(dt))) {
    station_max <- dt[, .(max_abs_dev = max(abs_dev, na.rm = TRUE),
                          lon = lon[1], lat = lat[1]), by = s_id]
    pD <- ggplot(station_max, aes(x = lon, y = lat)) +
      geom_point(aes(color = max_abs_dev), size = 1.6, alpha = 0.7) +
      ggplot2::scale_color_viridis_c(option = "magma", direction = -1,
                                     name = d_label) +
      coord_quickmap() +
      labs(x = "Longitude", y = "Latitude",
           title = d_title) +
      theme_minimal(base_size = 10) +
      theme(plot.title = element_text(face = "bold", size = 10),
            legend.position = "right")
  } else {
    pD <- ggplot() + theme_void() +
      labs(title = "D. spatial map unavailable (no lon/lat)")
  }

  subtitle <- sprintf("method = %s, threshold = %s",
                      method, format(threshold, digits = 3))
  combined <- patchwork::wrap_plots(pA, pB, pC, pD, ncol = 2) +
    patchwork::plot_annotation(
      title = "Outlier-detection diagnostic",
      subtitle = subtitle,
      theme = ggplot2::theme(
        plot.title = ggplot2::element_text(face = "bold", size = 12),
        plot.subtitle = ggplot2::element_text(color = "gray30", size = 9)
      )
    )
  combined
}


# ============================================================
# Internal helper functions
# ============================================================

#' @keywords internal
pep_plot_outliers_overview <- function(dt, method, threshold, late_threshold) {

  # Calculate summary statistics
  n_total <- nrow(dt)
  n_outliers <- sum(dt$is_outlier, na.rm = TRUE)
  n_early <- sum(dt$outlier_category == "Early outlier", na.rm = TRUE)
  n_late <- sum(dt$outlier_category == "Late outlier", na.rm = TRUE)
  n_very_late <- sum(dt$outlier_category == "Very late (potential 2nd event)", na.rm = TRUE)

  # Panel 1: Deviation histogram
  p1 <- ggplot(dt, aes(x = deviation, fill = outlier_category)) +
    geom_histogram(bins = 60, alpha = 0.8) +
    scale_fill_manual(
      values = c("Normal" = "gray70",
                 "Early outlier" = "steelblue",
                 "Late outlier" = "orange",
                 "Very late (potential 2nd event)" = "red"),
      drop = FALSE
    ) +
    geom_vline(xintercept = 0, linetype = "dashed", color = "black") +
    labs(
      title = "Deviation from Expected DOY",
      x = "Deviation (days)",
      y = "Count",
      fill = ""
    ) +
    theme_minimal() +
    theme(legend.position = "bottom",
          legend.text = element_text(size = 8))

  # Panel 2: Seasonal distribution of outliers only
  outliers_only <- dt[is_outlier == TRUE]
  if (nrow(outliers_only) > 0) {
    p2 <- ggplot(outliers_only, aes(x = month, fill = outlier_category)) +
      geom_bar(alpha = 0.8) +
      scale_x_continuous(breaks = 1:12,
                         labels = c("J","F","M","A","M","J","J","A","S","O","N","D")) +
      scale_fill_manual(
        values = c("Early outlier" = "steelblue",
                   "Late outlier" = "orange",
                   "Very late (potential 2nd event)" = "red"),
        drop = FALSE
      ) +
      labs(
        title = "Outliers by Month",
        subtitle = "Red = potential second events",
        x = "Month",
        y = "Count",
        fill = ""
      ) +
      theme_minimal() +
      theme(legend.position = "none")
  } else {
    p2 <- ggplot() +
      annotate("text", x = 0.5, y = 0.5, label = "No outliers found") +
      theme_void()
  }

  # Panel 3: DOY distribution with outliers highlighted
  p3 <- ggplot(dt, aes(x = day, fill = is_outlier)) +
    geom_histogram(bins = 73, alpha = 0.7, position = "identity") +
    scale_fill_manual(values = c("FALSE" = "gray60", "TRUE" = "red"),
                      labels = c("Normal", "Outlier"),
                      na.value = "gray80") +
    geom_vline(xintercept = late_threshold, linetype = "dashed",
               color = "darkred", linewidth = 0.8) +
    annotate("text", x = late_threshold + 5, y = Inf, vjust = 2,
             label = paste0("Late threshold (DOY ", late_threshold, ")"),
             color = "darkred", size = 3, hjust = 0) +
    labs(
      title = "Day of Year Distribution",
      x = "Day of Year",
      y = "Count",
      fill = ""
    ) +
    theme_minimal() +
    theme(legend.position = "bottom")

  # Panel 4: Summary text
  summary_text <- sprintf(
    "OUTLIER SUMMARY\n\nTotal observations: %s\nTotal outliers: %d (%.1f%%)\n\nEarly outliers: %d\nLate outliers: %d\nVery late (DOY >= %d): %d\n\nMethod: %s\nThreshold: %s",
    format(n_total, big.mark = ","),
    n_outliers,
    100 * n_outliers / n_total,
    n_early,
    n_late,
    late_threshold,
    n_very_late,
    method,
    threshold
  )

  p4 <- ggplot() +
    annotate("text", x = 0, y = 0.5, label = summary_text,
             hjust = 0, vjust = 0.5, size = 3.5, family = "mono") +
    xlim(-0.1, 1) + ylim(0, 1) +
    theme_void() +
    labs(title = "Summary Statistics")

  # Combine panels
  combined <- patchwork::wrap_plots(p1, p2, p3, p4, ncol = 2)
  print(combined)
  invisible(list(deviation = p1, monthly = p2, doy = p3, summary = p4))
}


#' @keywords internal
pep_plot_outliers_seasonal <- function(dt, late_threshold) {

  outliers_only <- dt[is_outlier == TRUE]

  if (nrow(outliers_only) == 0) {
    message("No outliers to plot")
    return(invisible(NULL))
  }

  # By month and category
  p1 <- ggplot(outliers_only, aes(x = factor(month), fill = outlier_category)) +
    geom_bar(position = "stack", alpha = 0.8) +
    scale_x_discrete(labels = c("Jan","Feb","Mar","Apr","May","Jun",
                                "Jul","Aug","Sep","Oct","Nov","Dec")) +
    scale_fill_manual(
      values = c("Early outlier" = "steelblue",
                 "Late outlier" = "orange",
                 "Very late (potential 2nd event)" = "red")
    ) +
    labs(
      title = "Seasonal Distribution of Outliers",
      subtitle = sprintf("Red bars (DOY >= %d) may indicate second flowering or other repeated events", late_threshold),
      x = "Month",
      y = "Number of Outliers",
      fill = "Category"
    ) +
    theme_minimal() +
    theme(axis.text.x = element_text(angle = 45, hjust = 1),
          legend.position = "bottom")

  # If phase_id available, facet by phase
  if ("phase_id" %in% names(outliers_only) && length(unique(outliers_only$phase_id)) > 1) {
    p1 <- p1 + facet_wrap(~phase_id, scales = "free_y")
  }

  print(p1)
  invisible(p1)
}


#' @keywords internal
pep_plot_outliers_map <- function(dt) {

  if (!all(c("lon", "lat") %in% names(dt))) {
    stop("Map plot requires 'lon' and 'lat' columns", call. = FALSE)
  }

  outliers_only <- dt[is_outlier == TRUE]

  if (nrow(outliers_only) == 0) {
    message("No outliers to plot on map")
    return(invisible(NULL))
  }

  # Aggregate by station
  station_summary <- outliers_only[, .(
    n_outliers = .N,
    mean_deviation = mean(abs(deviation), na.rm = TRUE),
    lon = lon[1],
    lat = lat[1]
  ), by = s_id]

  p <- ggplot(station_summary, aes(x = lon, y = lat)) +
    geom_point(aes(size = n_outliers, color = mean_deviation), alpha = 0.6) +
    scale_color_viridis_c(option = "plasma", name = "Mean |deviation|") +
    scale_size_continuous(range = c(1, 8), name = "N outliers") +
    coord_quickmap() +
    labs(
      title = "Geographic Distribution of Outliers",
      subtitle = sprintf("Showing %d stations with outliers", nrow(station_summary)),
      x = "Longitude",
      y = "Latitude"
    ) +
    theme_minimal()

  print(p)
  invisible(p)
}


#' @keywords internal
pep_plot_outliers_detail <- function(dt, n_top) {

  outliers_only <- dt[is_outlier == TRUE]

  if (nrow(outliers_only) == 0) {
    message("No outliers to show in detail")
    return(invisible(NULL))
  }

  # Get most extreme outliers
  outliers_only <- outliers_only[order(-abs(deviation))]
  top_outliers <- head(outliers_only, n_top)

  # Create label for each observation
  if ("species" %in% names(top_outliers)) {
    top_outliers[, label := sprintf("%s\n%s (phase %s)\nYear: %d",
                                     s_id, species, phase_id, year)]
  } else {
    top_outliers[, label := sprintf("%s (phase %s)\nYear: %d",
                                     s_id, phase_id, year)]
  }

  # Order by deviation
  top_outliers[, label := factor(label, levels = rev(label))]

  p <- ggplot(top_outliers, aes(x = deviation, y = label, fill = outlier_category)) +
    geom_col(alpha = 0.8) +
    geom_vline(xintercept = 0, linetype = "solid", color = "black") +
    scale_fill_manual(
      values = c("Early outlier" = "steelblue",
                 "Late outlier" = "orange",
                 "Very late (potential 2nd event)" = "red")
    ) +
    geom_text(aes(label = sprintf("DOY: %d", day)),
              hjust = ifelse(top_outliers$deviation > 0, -0.1, 1.1),
              size = 2.5) +
    labs(
      title = sprintf("Top %d Most Extreme Outliers", n_top),
      subtitle = "Deviation from expected DOY for station-species-phase combination",
      x = "Deviation (days)",
      y = "",
      fill = "Category"
    ) +
    theme_minimal() +
    theme(axis.text.y = element_text(size = 7),
          legend.position = "bottom")

  print(p)
  invisible(p)
}


#' @keywords internal
pep_plot_outliers_station <- function(dt) {

  if (!all(c("year", "s_id") %in% names(dt))) {
    stop("Station plot requires 'year' and 's_id' columns", call. = FALSE)
  }

  # Summarize by station and year
  station_year <- dt[, .(
    n_obs = .N,
    n_outliers = sum(is_outlier, na.rm = TRUE),
    pct_outliers = 100 * sum(is_outlier, na.rm = TRUE) / .N
  ), by = .(s_id, year)]

  # Find stations with most outliers
  station_totals <- station_year[, .(total_outliers = sum(n_outliers)), by = s_id]
  top_stations <- head(station_totals[order(-total_outliers)], 20)$s_id

  station_year_top <- station_year[s_id %in% top_stations]

  p <- ggplot(station_year_top, aes(x = year, y = s_id, fill = pct_outliers)) +
    geom_tile() +
    scale_fill_viridis_c(option = "inferno", name = "Outliers (%)") +
    labs(
      title = "Outlier Patterns by Station Over Time",
      subtitle = "Top 20 stations by total outliers",
      x = "Year",
      y = "Station ID"
    ) +
    theme_minimal() +
    theme(axis.text.y = element_text(size = 6))

  print(p)
  invisible(p)
}


#' @keywords internal
pep_plot_outliers_doy_context <- function(dt, late_threshold) {

  # Show full DOY distribution with outliers marked
  p <- ggplot(dt, aes(x = day)) +
    geom_density(fill = "gray70", alpha = 0.5) +
    geom_rug(data = dt[is_outlier == TRUE],
             aes(color = outlier_category), alpha = 0.7, linewidth = 1) +
    scale_color_manual(
      values = c("Early outlier" = "steelblue",
                 "Late outlier" = "orange",
                 "Very late (potential 2nd event)" = "red")
    ) +
    geom_vline(xintercept = late_threshold, linetype = "dashed",
               color = "darkred", linewidth = 0.8) +
    annotate("text", x = late_threshold, y = Inf,
             label = sprintf(" DOY %d\n (potential 2nd events)", late_threshold),
             hjust = 0, vjust = 1.5, color = "darkred", size = 3) +
    labs(
      title = "DOY Distribution with Outliers",
      subtitle = "Density shows normal observations; rug marks show outlier positions",
      x = "Day of Year",
      y = "Density",
      color = "Outlier type"
    ) +
    theme_minimal() +
    theme(legend.position = "bottom")

  # Facet by phase if multiple phases
  if ("phase_id" %in% names(dt) && length(unique(dt$phase_id)) > 1) {
    p <- p + facet_wrap(~phase_id, scales = "free_y")
  }

  print(p)
  invisible(p)
}

#' Plot Phenological Time Series
#'
#' Visualizes phenological day-of-year (DOY) trends over time since 1961.
#' Designed for aggregated or site-level phenology data from PEP725.
#'
#' @param data A data.frame or data.table containing at least `year`, `day` (day-of-year), and one grouping variable
#'             (e.g. `species`, `phase`, or `functional_group`).
#' @param color_by Character. Column name used for color grouping (default = "species").
#' @param facet_by Character or NULL. Optional column name for faceting (e.g. "functional_group").
#' @param alpha_lines Numeric. Transparency level for time-series lines (default = 0.7).
#' @param linewidth Numeric. Line width (default = 0.8).
#' @param smooth Logical. If TRUE, adds a linear trend smoother (default = TRUE).
#' @param se Logical. Whether to display confidence interval for smoother (default = FALSE).
#' @param year_min Numeric. Minimum year to plot (default = 1961).
#' @param title Character. Optional plot title.
#'
#' @return A ggplot object.
#' @author Matthias Templ
#' @export
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Use Alpine subset for faster computation
#' pep_alpine <- pep[country %in% c("Switzerland", "Austria")]
#'
#' # Example: flowering DOY by genus
#' pheno_plot_timeseries(data = pep_alpine[phase_id == 65],
#'                       color_by = "genus",
#'                       facet_by = NULL, smooth = TRUE)
#'
#' # Example: single species, colored by site (grapevine has longest records)
#' vine_data <- pep_alpine[species == "Vitis vinifera" & phase_id == 65]
#' if (nrow(vine_data) > 0) {
#'   pheno_plot_timeseries(
#'     data = vine_data,
#'     color_by = "s_id",
#'     smooth = TRUE,
#'     title = "Grapevine flowering (BBCH 65) across sites"
#'   )
#' }
#'
#' # Example: facets by genus
#' pheno_plot_timeseries(
#'   data = pep_alpine[phase_id == 65],
#'   color_by = "species",
#'   facet_by = "genus",
#'   smooth = TRUE
#' )
#' }
pheno_plot_timeseries <- function(
    data,
    color_by = "species",
    facet_by = NULL,
    alpha_lines = 0.7,
    linewidth = 0.8,
    smooth = TRUE,
    se = FALSE,
    year_min = 1961,
    title = "Phenological time series of flowering (BBCH 65)"
) {
  # local data.table copy (avoid mutating the caller's object)
  data <- as.data.table(data)
  data <- data[year >= year_min]

  # --- base plot ---
  p <- ggplot(data, aes(x = year, y = day, color = .data[[color_by]])) +
    geom_line(alpha = alpha_lines, linewidth = linewidth, na.rm = TRUE) +
    theme_bw(base_size = 11) +
    labs(
      x = "Year",
      y = "Day of Year (DOY)",
      color = gsub("_", " ", color_by),
      title = title
    ) +
    theme(legend.position = "top")

  # --- add smoother if requested ---
  if (smooth) {
    p <- p + geom_smooth(method = "lm", se = se, linetype = "solid", linewidth = 0.8)
  }

  # --- optional facetting ---
  if (!is.null(facet_by)) {
    p <- p + facet_wrap(as.formula(paste("~", facet_by)), scales = "free_y")
  }

  return(p)
}

test_that("pheno_plot_timeseries does not mutate the caller's data", {
  # Regression test for the setDT(data) caller-mutation bug.
  # A data.frame passed in must remain a data.frame after the call.

  df <- data.frame(
    year = 1990:2020,
    day = 100 + rnorm(31, 0, 3),
    species = "Malus domestica"
  )
  class_before <- class(df)

  p <- pheno_plot_timeseries(df)

  expect_identical(class(df), class_before)
  expect_false("data.table" %in% class(df))
  expect_s3_class(p, "ggplot")
})

test_that("pheno_plot_timeseries accepts a data.table without side-effects", {
  dt <- data.table::data.table(
    year = 1990:2020,
    day = 100 + rnorm(31, 0, 3),
    species = "Malus domestica"
  )
  # data.table already has the right class; we just want to make sure the
  # function doesn't error and returns a ggplot.
  p <- pheno_plot_timeseries(dt)
  expect_s3_class(p, "ggplot")
})

utils::globalVariables(c("a"))

#' Plot Robust Phenological Trends
#'
#' This function visualizes long-term phenological trends for one genus or species,
#' using robust MM-regression (`robustbase::lmrob`) applied to mean annual DOYs.
#'
#' The function produces a faceted panel plot showing:
#' \itemize{
#'   \item annual mean DOY and interquartile range,
#'   \item robust trend lines for past and future periods,
#'   \item CTRL (1991-2020) and PGW (2066-2095) climate windows,
#'   \item optional layouts (country x phase or phase x country).
#' }
#'
#' @param pep A PEP725-style data frame with columns \code{year}, \code{DOY},
#'   \code{phase_id}, \code{country}, \code{s_id}, \code{genus}, \code{species}.
#' @param genus_name Character string specifying a genus to filter (optional).
#' @param species_name Character string specifying a species to filter (optional).
#'   If both genus and species are provided, the species filter is applied last.
#' @param subspecies_name Character string specifying a subspecies to filter (optional).
#'   Only one of genus_name, species_name, or subspecies_name can be specified.
#' @param phases Integer vector of phenological phase IDs to include
#'   (default: \code{c(65, 87)} for flowering and maturity of fruit).
#' @param common_stations Logical. If \code{TRUE} (default), only stations with observations
#'   for all selected phases are included. If \code{FALSE}, all stations with any of the selected phases are included.
#' @param combine_regions Logical. If \code{TRUE}, combines all regions into a single panel. Default is \code{FALSE}.
#' @param combine_layout Character. Layout for combined regions plot: either \code{"vertical"} or \code{"horizontal"}.
#' @param years Numeric vector of years to include in the analysis.
#'   Default is \code{1961:2024}.
#' @param calib_years Numeric vector specifying the calibration window for robust
#'   regression (default: \code{1991:2020}).
#' @param pred_years Numeric vector of years for projecting future phenology.
#'   Default is \code{1990:2090}.
#' @param subregions Character vector of country names to include
#'   (default: DACH region).
#' @param giss_data Deprecated. Previously used for climate data integration;
#'   now ignored.
#' @param layout Either \code{"country_phase"} (default) or \code{"phase_country"}
#'   to control facet arrangement.
#' @param title Optional plot title. If \code{NULL}, the function generates one
#'   from genus and species information.
#'
#' @details
#' The function performs the following steps:
#' \enumerate{
#'   \item Filter PEP data by species, phases, regions, years.
#'   \item Aggregate DOYs by year-country-phase.
#'   \item Fit robust regression \eqn{DOY \sim year} over the calibration window.
#'   \item Predict past and future trends.
#'   \item Draw ribbon (IQR), annual means, trend lines, climate windows.
#'   \item Facet by country or by phase.
#' }
#'
#' The robust regression uses \code{robustbase::lmrob}, which is resistant to
#' outliers and non-normality, and therefore ideal for phenological time series.
#'
#' @return A \code{ggplot} object.
#'
#' @author Matthias Templ
#' @export
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' plot_phenology_trends(
#'   pep,
#'   species_name = "Vitis vinifera",
#'   phases = c(65, 81),
#'   layout = "country_phase"
#' )
#' }
plot_phenology_trends <- function(
    pep,
    genus_name = NULL,
    species_name = NULL,
    subspecies_name = NULL,   # NEW
    phases = c(65, 87),
    common_stations = TRUE,
    combine_regions = FALSE,   # NEW ARGUMENT
    combine_layout = c("vertical", "horizontal"),
    years = 1961:2024,
    calib_years = 1991:2020,
    pred_years = NULL,
    subregions = c("Austria","Germany-North","Germany-South","Switzerland"),
    giss_data = NULL,
    layout = c("country_phase", "phase_country"),
    title = NULL
){

  layout <- match.arg(layout)
  combine_layout <- match.arg(combine_layout)

  # ------------------------------------------------------------------
  # 0. Validate taxonomy input (only one allowed)
  # ------------------------------------------------------------------
  taxo_args <- list(
    genus = genus_name,
    species = species_name,
    subspecies = subspecies_name
  )
  if (sum(!sapply(taxo_args, is.null)) > 1) {
    stop("Please specify only ONE of: genus_name, species_name, subspecies_name.")
  }

  # ------------------------------------------------------------------
  # 1. Filter PEP data
  # ------------------------------------------------------------------
  pep2 <- pep %>%
    dplyr::rename(DOY = dplyr::any_of("day")) %>%
    dplyr::filter(year %in% years)

  # TAXONOMY FILTER ---------------------------------------------------
  if (!is.null(subspecies_name)) {
    pep2 <- pep2 %>% dplyr::filter(!is.na(subspecies) & subspecies == subspecies_name)

  } else if (!is.null(species_name)) {
    pep2 <- pep2 %>% dplyr::filter(species == species_name)

  } else if (!is.null(genus_name)) {
    pep2 <- pep2 %>% dplyr::filter(genus == genus_name)
  }

  # Phase filtering
  pep2 <- pep2 %>% dplyr::filter(phase_id %in% phases)

  # Region filter
  pep2 <- pep2 %>% dplyr::filter(country %in% subregions)

  if(nrow(pep2) == 0){
    stop("No data available after filtering of years, phases and subregions. Please check your filters.")
  }

  # Keep only stations with all phases (optional)
  if(common_stations){
    common_ids <- Reduce(intersect,
                         lapply(phases, function(p) pep2$s_id[pep2$phase_id == p]))
    pep2 <- pep2 %>% dplyr::filter(s_id %in% common_ids)
  }

  if(nrow(pep2) == 0){
    if(!common_stations){
      stop("No data available after filtering. Try common_stations = FALSE.")
    } else {
      stop("No data available after filtering of years, phases, subregions and common_stations.")
    }
  }

  # ------------------------------------------------------------------
  # 2. Aggregate DOYs per country-year-phase
  # ------------------------------------------------------------------
  agg <- pep2 %>%
    dplyr::group_by(country, year, phase_id) %>%
    dplyr::summarise(
      mean_DOY = mean(DOY, na.rm=TRUE),
      q25      = quantile(DOY, 0.25, na.rm=TRUE),
      q75      = quantile(DOY, 0.75, na.rm=TRUE),
      .groups="drop"
    )


  # ------------------------------------------------------------------
  # 2b. NEW: combine all regions into one series
  # ------------------------------------------------------------------
  if (combine_regions) {

    message("Combining all regions into a single aggregated trend...")

    agg <- agg %>%
      dplyr::group_by(year, phase_id) %>%
      dplyr::summarise(
        mean_DOY = mean(mean_DOY, na.rm = TRUE),
        q25      = mean(q25, na.rm = TRUE),
        q75      = mean(q75, na.rm = TRUE),
        .groups = "drop"
      ) %>%
      dplyr::mutate(country = "All regions")

    subregions <- "All regions"
  }


  # ------------------------------------------------------------------
  # 3. Robust regression: DOY ~ year (MM estimator)
  # ------------------------------------------------------------------
  robust_fit_year <- function(df){
    sub <- df %>%
      dplyr::filter(
        year >= min(calib_years),
        year <= max(calib_years),
        is.finite(mean_DOY)
      )

    if (nrow(sub) < 3)
      return(tibble::tibble(a = NA_real_, b = NA_real_))

    mod <- tryCatch(
      robustbase::lmrob(mean_DOY ~ year, data = sub),
      error = function(e) NULL
    )

    if (is.null(mod))
      return(tibble::tibble(a = NA_real_, b = NA_real_))

    tibble::tibble(
      a = unname(coef(mod)[1]),
      b = unname(coef(mod)[2])
    )
  }

  fits <- agg %>%
    dplyr::group_by(phase_id, dplyr::across(dplyr::any_of("country"))) %>%
    dplyr::group_modify(~ robust_fit_year(.x)) %>%
    dplyr::ungroup() %>%
    dplyr::filter(!is.na(a)) %>%
    dplyr::mutate(
      days_decade = b * 10,
      label = sprintf("%+.2f d/dec", days_decade)
    )


  # ------------------------------------------------------------------
  # 5. Predictions (regression line until PGW)
  # ------------------------------------------------------------------
  if (is.null(pred_years)) {
    pred_years <- seq(min(calib_years), 2095)
  }

  pred <- fits %>%
    dplyr::group_by(phase_id, dplyr::across(dplyr::any_of("country"))) %>%
    dplyr::do(tibble::tibble(
      year = pred_years,
      mean_DOY = .$a + .$b * pred_years
    )) %>%
    dplyr::ungroup()


  # ------------------------------------------------------------------
  # 6. Title construction
  # ------------------------------------------------------------------
  if (is.null(title)) {
    if (!is.null(subspecies_name))
      title <- subspecies_name
    else if (!is.null(species_name))
      title <- species_name
    else if (!is.null(genus_name))
      title <- genus_name
    else
      title <- "Phenology -- Robust Trend"
  }

  # Slope annotation at year = 2085
  label_x <- 2085
  fits <- fits %>% dplyr::mutate(label_y = a + b * label_x)


  # ------------------------------------------------------------------
  # 7. Build plot
  # ------------------------------------------------------------------
  p <- ggplot2::ggplot() +
    ggplot2::geom_ribbon(
      data = agg,
      ggplot2::aes(year, ymin = q25, ymax = q75),
      alpha = 0.18
    ) +
    ggplot2::geom_line(
      data = agg,
      ggplot2::aes(year, mean_DOY)
    ) +
    ggplot2::geom_line(
      data = pred,
      ggplot2::aes(year, mean_DOY),
      linewidth = 0.7,
      linetype = "dashed",
      colour = "red"
    ) +
    ggplot2::geom_text(
      data = fits,
      ggplot2::aes(x = label_x, y = label_y, label = label),
      hjust = 0.7,
      vjust = -0.6,
      fontface = "italic",
      size = 2.5
    ) +
    ggplot2::geom_vline(
      xintercept = c(min(calib_years), max(calib_years)),
      linetype = 1,
      colour = "blue",
      linewidth = 0.5
    ) +
    ggplot2::geom_vline(
      xintercept = c(2066, 2095),
      linetype = 5,
      colour = "red",
      linewidth = 0.5
    ) +
    ggplot2::annotate(
      "text",
      x = mean(calib_years) + 5,
      y = min(agg$mean_DOY, na.rm = TRUE),
      label = "CTRL",
      size = 2.7,
      colour = "blue"
    ) +
    ggplot2::annotate(
      "text",
      x = 2080,
      y = min(agg$mean_DOY, na.rm = TRUE),
      label = "PGW",
      size = 2.7,
      colour = "red"
    ) +
    ggplot2::labs(
      title = title,
      x = "",
      y = "Day of Year (DOY)"
    ) +
    ggplot2::scale_x_continuous(breaks = seq(1900, 2100, 25)) +
    ggplot2::theme_bw(base_size = 12)


  # ------------------------------------------------------------------
  # 8. Faceting logic
  # ------------------------------------------------------------------
  if (!combine_regions) {

    # Original behavior
    if (layout == "country_phase") {
      p <- p + ggplot2::facet_grid(country ~ phase_id)
    } else {
      p <- p + ggplot2::facet_grid(phase_id ~ country)
    }

  } else {

    # Combined region plot: facet only by phase
    if (length(phases) > 1) {

      if (combine_layout == "vertical") {
        p <- p + ggplot2::facet_wrap(~ phase_id, ncol = 1)

      } else if (combine_layout == "horizontal") {
        p <- p + ggplot2::facet_wrap(~ phase_id, nrow = 1)

      }

    } else {
      message("Plotting combined-region trend for BBCH ", phases)
    }

  }

  return(p)
}

#' Add Country Information to a Dataset Based on Latitude/Longitude
#'
#' This function adds a new column \code{country} by performing a fast spatial
#' join between point coordinates (\code{lat}, \code{lon}) and world country
#' polygons from \pkg{rnaturalearth}. Geometry is handled via \pkg{sf}.
#'
#' The input data must contain numeric columns named \code{lat} and \code{lon}
#' in WGS84 (EPSG:4326).  Rows with \code{NA} coordinates receive
#' \code{country = NA}.
#'
#' After assigning countries, Germany is automatically split into
#' \code{"Germany-North"} and \code{"Germany-South"} using a latitude threshold:
#' \itemize{
#'   \item lat >= 50 → \code{"Germany-North"}
#'   \item lat  < 50 → \code{"Germany-South"}
#' }
#'
#' If the input already contains a \code{country} column it is removed and
#' replaced by the spatially derived one.
#'
#' @param dt A \code{data.frame} or \code{data.table} containing columns
#'   \code{lat} and \code{lon}.
#' @param keep_geometry Logical. If \code{TRUE}, the sf geometry column is kept.
#'   Default is \code{FALSE}.
#'
#' @return A \code{data.table} with a new column \code{country}.
#' @author Matthias Templ
#'
#' @examples
#' \donttest{
#' small <- data.table(
#'   lat = c(48.2, 51.0, 47.5),
#'   lon = c(16.3, 10.1, 8.5)
#' )
#'
#' pep_with_country <- add_country(small)
#' pep_with_country
#' }
#'
#' @export
add_country <- function(dt, keep_geometry = FALSE) {

  # Check input
  if (!("lat" %in% names(dt)) || !("lon" %in% names(dt))) {
    stop("Input data must contain columns 'lat' and 'lon'.", call. = FALSE)
  }

  # Check required packages
  if (!requireNamespace("sf", quietly = TRUE))
    stop("Package 'sf' is required. Install it with install.packages('sf').",
         call. = FALSE)
  if (!requireNamespace("rnaturalearth", quietly = TRUE))
    stop("Package 'rnaturalearth' is required. Install it with install.packages('rnaturalearth').",
         call. = FALSE)
  if (!requireNamespace("rnaturalearthdata", quietly = TRUE))
    stop("Package 'rnaturalearthdata' is required. Install it with install.packages('rnaturalearthdata').",
         call. = FALSE)

  message("Assigning countries based on coordinates ...")

  # Convert to data.table (copy to avoid modifying original)
  dt <- data.table::copy(data.table::as.data.table(dt))

  # Remove pre-existing country column to avoid duplicates
  if ("country" %in% names(dt)) {
    dt[, country := NULL]
  }

  # Handle NA coordinates: set aside rows, join valid rows only
  has_na_coords <- is.na(dt$lon) | is.na(dt$lat)
  n_na <- sum(has_na_coords)
  if (n_na > 0) {
    message(sprintf("Note: %d row(s) have NA coordinates and will get country = NA.", n_na))
  }

  if (all(has_na_coords)) {
    dt[, country := NA_character_]
    return(dt)
  }

  # Split into valid and NA-coordinate rows
  if (n_na > 0) {
    dt[, .row_idx := .I]
    dt_valid <- dt[!has_na_coords]
    dt_na <- dt[has_na_coords]
    dt_na[, country := NA_character_]
  } else {
    dt_valid <- dt
  }

  # Convert to sf POINT object with WGS84 coordinates
  pep_sf <- sf::st_as_sf(
    dt_valid,
    coords = c("lon", "lat"),
    crs = 4326,
    remove = FALSE
  )

  # Load world polygons
  world <- rnaturalearth::ne_countries(scale = "medium", returnclass = "sf")[, c("name")]
  world <- sf::st_make_valid(world)

  # Reproject both datasets (faster join in planar CRS)
  pep_sf <- sf::st_transform(pep_sf, 3857)
  world  <- sf::st_transform(world, 3857)

  sf::st_agr(world) <- "constant"

  # Spatial join
  pep_joined <- sf::st_join(pep_sf, world, left = TRUE)

  # Convert back to data.table
  dt_result <- data.table::as.data.table(pep_joined)

  # Remove geometry unless requested
  if (!keep_geometry) {
    dt_result[, geometry := NULL]
  }

  data.table::setnames(dt_result, "name", "country")

  # Recombine with NA-coordinate rows
  if (n_na > 0) {
    dt_result <- data.table::rbindlist(list(dt_result, dt_na),
                                       use.names = TRUE, fill = TRUE)
    data.table::setorderv(dt_result, ".row_idx")
    dt_result[, .row_idx := NULL]
  }

  # Germany split
  dt_result[
    country == "Germany" & lat >= 50,
    country := "Germany-North"
  ]

  dt_result[
    country == "Germany" & lat < 50,
    country := "Germany-South"
  ]

  dt_result
}

#' Compile Regional Phenology Data and Climate Sensitivity Inputs
#'
#' Extracts and aggregates phenological observations for a selected species and
#' phenological phase from the PEP725 dataset (globally and spatially filtered).
#' Optionally links them with global temperature anomalies (GISS data) for
#' climate impact studies.
#'
#' @param pep A data.table containing the full PEP725 dataset. Must include at
#'   least columns: \code{species}, \code{year}, \code{phase_id}, \code{day},
#'   \code{lat}, \code{lon}.
#' @param giss Optional. A data.table with GISS global temperature anomalies,
#'   containing columns \code{year}, \code{dT}, and \code{dT_sm}. If \code{NULL},
#'   GISS-related outputs are omitted.
#' @param lon_min Minimum longitude for spatial filtering of PEP data (default is \code{4.2})
#' @param lon_max Maximum longitude for spatial filtering of PEP data (default is \code{8.1})
#' @param lat_min Minimum latitude for spatial filtering of PEP data (default is \code{44.7})
#' @param lat_max Maximum latitude for spatial filtering of PEP data (default is \code{48.1})
#' @param species_name Character name of the species to be extracted (default is \code{"Triticum aestivum"})
#' @param functional_group Character. Optional functional group name to filter by (e.g., "C4_summer_cereals"). If provided, overwrites species_name filtering.
#' @param year_min Minimum year to include in all outputs (default is \code{1961})
#' @param pep_for_giss Selects which PEP data subset to use for merging with GISS
#'   data. Either \code{"near"} (box-filtered) or \code{"aggregated"} (global).
#'   Ignored if \code{giss} is \code{NULL}.
#' @param phase Integer or vector of phase_id(s) to extract (default is \code{60} = Heading). Named phase mappings are applied.
#'
#' @return A named list with the following elements:
#' \describe{
#'   \item{\code{pep_agg}}{Global (non-spatially filtered) PEP725 time series for the selected phase(s)}
#'   \item{\code{pep_cgi}}{Spatially filtered PEP725 time series based on lat/lon bounding box}
#'   \item{\code{giss}}{Processed GISS global temperature anomaly data (if provided)}
#'   \item{\code{pep_giss}}{Merged data frame of PEP phenology and GISS anomalies (if provided)}
#'   \item{\code{species}}{Character species name used for filtering}
#'   \item{\code{functional_group}}{Functional group if specified, otherwise \code{NA}}
#'   \item{\code{phase}}{Integer phase ID(s) used for filtering}
#' }
#'
#' @details
#' The function aggregates PEP observations by species, year, and phase,
#' calculates mean day-of-year (DOY), applies spatial filtering if needed,
#' and attaches GISS climate anomalies. It also maps \code{phase_id}s to
#' descriptive names and warns if expected phenological phase are missing
#' in the selected data.
#'
#' If GISS data is provided, temperature anomalies are merged with the selected
#' PEP time series to support phenology-climate analysis.
#'
#' @seealso \code{\link{pheno_plot}}, \code{\link{pep_download}}
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' out <- pheno_regional(pep, species_name = "Triticum aestivum", phase = 60)
#' str(out)
#' }
#'
#' @author Matthias Templ
#' @export
pheno_regional <- function(
    pep,
    giss = NULL,
    lon_min = 4.2, lon_max = 8.1,
    lat_min = 44.7, lat_max = 48.1,
    species_name = "Triticum aestivum",
    functional_group = NULL,
    year_min = 1961,
    pep_for_giss = c("near", "aggregated"),
    phase = 60  # <- default to Heading
) {

  if(!is.null(functional_group)) {
    warning("functional_group overwrites species_name.")
  }



  pep_for_giss <- match.arg(pep_for_giss)

  # pep_species <- pep[species %in% species_name]

  if (!is.null(functional_group)) {
    fg <- as.character(functional_group)
    if (!"functional_group" %in% names(pep)) {
      stop("Column 'functional_group' not found in data.", call. = FALSE)
    }
    pep_filtered <- pep[as.character(pep$functional_group) == fg]
    message("Filtering by functional group: ", fg)
    effective_label <- fg
  } else {
    species_name <- as.character(species_name)
    pep_filtered <- pep[as.character(species) == species_name]
    effective_label <- species_name
  }

  group_var <- if (!is.null(functional_group)) "functional_group" else "species"

  PEPData <- pep_filtered[
    , .(n = .N, mean_day = mean(day, na.rm = TRUE)),
    by = c(group_var, "year", "phase_id")
  ][order(get(group_var), year, mean_day, phase_id)]

  # # Global Aggregated PEP
  # PEPData <- pep_filtered[
  #   , .(n = .N, mean_day = mean(day, na.rm = TRUE)),
  #   by = .(species, year, phase_id)
  # ][order(species, year, mean_day, phase_id)]

  # Box-filtered (near Changins)
  pep_box <- pep_filtered[
    lat > lat_min & lat < lat_max & lon > lon_min & lon < lon_max
  ]
  # PEPDataChangins <- pep_box[
  #   , .(n = .N, mean_day = mean(day, na.rm = TRUE)),
  #   by = .(species, year, phase_id)
  # ][order(species, year, mean_day, phase_id)]
  PEPDataChangins <- pep_box[
    , .(n = .N, mean_day = mean(day, na.rm = TRUE)),
    by = c(group_var, "year", "phase_id")
  ][order(get(group_var), year, mean_day, phase_id)]

  # --- Helper: flexible phase name mapping + warnings ---------------------
  phase_lookup <- .bbch_lookup
  phase_name <- function(id) {
    res <- as.character(phase_lookup[as.character(id)])
    if (any(is.na(res))) {
      warning(sprintf("Unmapped phase_id(s): %s",
                      paste(setdiff(id, as.integer(names(phase_lookup))), collapse = ", ")),
              call. = FALSE)
    }
    res
  }

  # pepfun_single <- function(dt, label, sp, yrmin, phase) {
  #   out <- dt[
  #     species == sp & year >= yrmin & phase_id %in% phase,
  #     .(year, phase = phase_name(phase_id), DOY = mean_day)
  #   ][order(year)][
  #     , `:=`(source = label, site = label)
  #   ][]
  #   expected_phase <- phase_name(phase)
  #   observed_phase <- unique(out$phase)
  #   mean_day <- dt$mean_day
  #   missing <- setdiff(expected_phase, observed_phase)
  #   if (length(missing) > 0L) {
  #     warning(sprintf(
  #       "%s (year >= %d): missing phase(s): %s",
  #       label, yrmin, paste(missing, collapse = ", ")
  #     ), call. = FALSE)
  #   }
  #   return(out)
  # }
  pepfun_single <- function(dt, label, sp, yrmin, phase, by_col = "species") {
    # Ensure the column to filter by exists
    if (!by_col %in% names(dt)) {
      stop(sprintf("Column '%s' not found in dataset.", by_col), call. = FALSE)
    }

    # Helper for dynamic filtering (species or functional_group)
    out <- dt[
      get(by_col) == sp & year >= yrmin & phase_id %in% phase,
      .(year, phase_id = phase_id, phase = phase_name(phase_id), DOY = mean_day)
    ][order(year)][
      , `:=`(source = label, site = label)
    ][]

    # --- Quality check and messaging ----------------------------------------
    expected_phase <- phase_name(phase)
    observed_phase <- unique(out$phase)
    mean_day <- dt$mean_day
    missing <- setdiff(expected_phase, observed_phase)

    if (length(missing) > 0L) {
      warning(sprintf(
        "%s (year >= %d): missing phase(s): %s",
        label, yrmin, paste(missing, collapse = ", ")
      ), call. = FALSE)
    }

    # Stamp the group label (species or functional_group) into the output
    out[, species := sp]
    out[, filter_type := by_col]

    return(out)
  }

  # --- Apply phase filtering and label ------------------------------------
  # pep_agg <- pepfun_single(PEPData, "PEP725 (aggregated)", species_name, year_min, phase)
  # pep_cgi <- pepfun_single(PEPDataChangins, "PEP725 (near Changins)", species_name, year_min, phase)
  # pep_agg <- pepfun_single(PEPData, "PEP725 (aggregated)", effective_label, year_min, phase)
  pep_agg <- pepfun_single(
    PEPData,
    label = "PEP725 (aggregated)",
    sp = effective_label,
    yrmin = year_min,
    phase = phase,
    by_col = if (!is.null(functional_group)) "functional_group" else "species"
  )
  # pep_cgi <- pepfun_single(PEPDataChangins, "PEP725 (near Changins)", effective_label, year_min, phase)
  pep_cgi <- pepfun_single(
    PEPDataChangins,
    label = "PEP725 (near Changins)",
    sp = effective_label,
    yrmin = year_min,
    phase = phase,
    by_col = if (!is.null(functional_group)) "functional_group" else "species"
  )

  if (nrow(pep_cgi) == 0L) {
    warning("No matching records found near Changins for functional group: ", functional_group, call. = FALSE)
  }

  # --- GISS subset and merge (optional) ------------------------------------
  giss_out <- NULL
  pep_giss <- NULL
  if (!is.null(giss)) {
    giss_out <- giss[year >= year_min, .(year, dTgl = dT, Tgl = dT + 14)][order(year)]
    pep_sel <- switch(pep_for_giss, aggregated = pep_agg, near = pep_cgi)
    pep_giss <- merge(pep_sel, giss_out, by = "year")
    attr(pep_giss, "pep_source") <- pep_for_giss

    if (nrow(pep_giss) == 0L) {
      warning("No GISS merge possible (pep_giss is empty) -- check date/region/phase", call. = FALSE)
    }
  }


  # Add species label and mean_day alias
  pep_agg[, species := if (!is.null(functional_group)) functional_group else species_name]
  pep_agg$mean_day <- pep_agg$DOY
  pep_cgi[, species := if (!is.null(functional_group)) functional_group else species_name]
  pep_cgi$mean_day <- pep_cgi$DOY

  # Return
  list(
    pep_agg  = pep_agg,
    pep_cgi  = pep_cgi,
    giss     = giss_out,
    pep_giss = pep_giss,
    species = if (is.null(functional_group)) species_name else NA_character_,
    functional_group = if (!is.null(functional_group)) functional_group else NA_character_,
    phase  = phase
  )
}

#' Analyze Phenological Gradients with Elevation or Latitude
#'
#' Quantifies how phenological timing shifts with altitude or latitude,
#' calculating the phenological lapse rate using robust regression methods.
#'
#' @param pep A \code{pep} object or \code{data.table} containing phenological
#'   observations with columns \code{year}, \code{day}, and either \code{alt}
#'   (altitude) or \code{lat} (latitude).
#' @param variable Character string specifying the gradient variable.
#'   Either \code{"alt"} (altitude/elevation) or \code{"lat"} (latitude).
#'   Default is \code{"alt"}.
#' @param species Optional character string to filter by species/genus.
#' @param phase_id Optional integer to filter by a single BBCH phase code.
#' @param year_range Optional integer vector of length 2 specifying years to include.
#' @param by Optional character vector of grouping variables (e.g., "country").
#'   If provided, separate gradients are calculated for each group.
#' @param method Character string specifying regression method:
#'   \itemize{
#'     \item \code{"robust"} (default): Robust regression using \code{robustbase::lmrob}
#'     \item \code{"ols"}: Ordinary least squares regression
#'     \item \code{"quantile"}: Quantile regression for median using \code{quantreg::rq}
#'   }
#' @param aggregate Logical. If \code{TRUE} (default), first aggregate to station
#'   means before fitting. If \code{FALSE}, use all individual observations.
#' @param na.rm Logical. Should missing values be removed? Default \code{TRUE}.
#'
#' @return A list with class \code{pheno_gradient} containing:
#'   \describe{
#'     \item{summary}{A \code{data.table} with slope, intercept, R-squared,
#'       RMSE, n_stations, and interpretation}
#'     \item{model}{The fitted model object(s)}
#'     \item{data}{The data used for fitting}
#'     \item{variable}{The gradient variable used}
#'     \item{method}{The regression method used}
#'   }
#'
#' @details
#' Phenological gradients describe how the timing of biological events changes
#' with environmental factors. This function calculates:
#'
#' \itemize{
#'   \item \strong{Altitude gradient}: Typically 2-4 days delay per 100m elevation
#'     gain in temperate regions
#'   \item \strong{Latitude gradient}: Typically 2-5 days delay per degree north
#'     in Europe
#' }
#'
#' The robust regression method (default) is recommended because phenological
#' data often contains outliers from observation errors or microclimate effects.
#'
#' @section Interpretation:
#' The slope represents:
#' \itemize{
#'   \item For altitude: days delay per 100 meters elevation increase
#'   \item For latitude: days delay per degree latitude increase
#' }
#' Positive slopes indicate later phenology at higher elevations/latitudes.
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Subset to Alpine countries (good elevation variation)
#' pep_subset <- pep[country %in% c("Switzerland", "Austria")]
#'
#' # Altitude gradient for apple flowering
#' grad_alt <- pheno_gradient(pep_subset,
#'                            variable = "alt",
#'                            species = "Malus",
#'                            phase_id = 60)
#' print(grad_alt)
#'
#' # Latitude gradient
#' grad_lat <- pheno_gradient(pep_subset,
#'                            variable = "lat",
#'                            species = "Malus",
#'                            phase_id = 60)
#' print(grad_lat)
#'
#' # Compare regression methods
#' grad_ols <- pheno_gradient(pep_subset, species = "Malus",
#'                            phase_id = 60, method = "ols")
#' print(grad_ols)
#'
#' grad_robust <- pheno_gradient(pep_subset, species = "Malus",
#'                               phase_id = 60, method = "robust")
#' print(grad_robust)
#'
#' # Gradient by country (separate regression per country)
#' grad_by_country <- pheno_gradient(pep_subset,
#'                                   variable = "alt",
#'                                   species = "Malus",
#'                                   phase_id = 60,
#'                                   by = "country")
#' print(grad_by_country)
#' }
#'
#' @references
#' Defila, C. and Clot, B. (2001). Phytophenological trends in Switzerland.
#' \emph{International Journal of Biometeorology}, 45, 203--207.
#' \doi{10.1007/s004840100101}
#'
#' Ziello, C. et al. (2009). Influence of altitude on phenology of selected
#' plant species in the Alpine region (1971--2000).
#' \emph{Climate Research}, 39, 227--234.
#' \doi{10.3354/cr00822}
#'
#' @seealso
#' \code{\link{pheno_normals}} for calculating climatological normals,
#' \code{\link[robustbase]{lmrob}} for robust regression details
#'
#' @author Matthias Templ
#' @export
#' @importFrom robustbase lmrob
pheno_gradient <- function(pep,
                           variable = c("alt", "lat"),
                           species = NULL,
                           phase_id = NULL,
                           year_range = NULL,
                           by = NULL,
                           method = c("robust", "ols", "quantile"),
                           aggregate = TRUE,
                           na.rm = TRUE) {

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table if needed
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Match arguments
  variable <- match.arg(variable)
  method <- match.arg(method)

  # Make a copy early to avoid modifying original by reference
  dt <- data.table::copy(pep)

  # Check required columns
  required_cols <- c("year", "day", variable)
  if (variable == "alt" && !"alt" %in% names(dt)) {
    # Try alternative column names
    if ("altitude" %in% names(dt)) {
      dt[, alt := altitude]
    } else if ("elevation" %in% names(dt)) {
      dt[, alt := elevation]
    } else {
      stop("Altitude column not found. Expected 'alt', 'altitude', or 'elevation'.",
           call. = FALSE)
    }
  }

  missing_cols <- setdiff(required_cols, names(dt))
  if (length(missing_cols) > 0) {
    stop("Missing required columns: ", paste(missing_cols, collapse = ", "),
         call. = FALSE)
  }

  # Check quantreg availability for quantile method
  if (method == "quantile" && !requireNamespace("quantreg", quietly = TRUE)) {
    stop("Package 'quantreg' is required for quantile regression. ",
         "Install it with: install.packages('quantreg')", call. = FALSE)
  }

  # Apply species filter
  if (!is.null(species)) {
    if ("genus" %in% names(dt)) {
      genus_match <- dt$genus == species
      species_match <- if ("species" %in% names(dt)) dt$species == species else FALSE
      dt <- dt[genus_match | species_match]
      if (nrow(dt) == 0) {
        stop("No observations found for species '", species, "'", call. = FALSE)
      }
    } else if ("species" %in% names(dt)) {
      dt <- dt[dt$species == species]
    } else {
      warning("Neither 'genus' nor 'species' column found.", call. = FALSE)
    }
  }

  # Apply phase_id filter
  if (!is.null(phase_id)) {
    if (!"phase_id" %in% names(dt)) {
      warning("'phase_id' column not found.", call. = FALSE)
    } else {
      if (length(phase_id) > 1) {
        warning("Multiple phase_ids provided. Using first one only for gradient analysis.",
                call. = FALSE)
        phase_id <- phase_id[1]
      }
      dt <- dt[dt$phase_id == phase_id]
      if (nrow(dt) == 0) {
        stop("No observations found for phase_id: ", phase_id, call. = FALSE)
      }
    }
  }

  # Apply year range filter
  if (!is.null(year_range)) {
    if (!is.numeric(year_range) || length(year_range) != 2) {
      stop("'year_range' must be a numeric vector of length 2", call. = FALSE)
    }
    dt <- dt[year >= year_range[1] & year <= year_range[2]]
    if (nrow(dt) == 0) {
      stop("No observations found in year range.", call. = FALSE)
    }
  }

  # Remove rows with missing gradient variable
  dt <- dt[!is.na(get(variable)) & !is.na(day)]

  if (nrow(dt) == 0) {
    stop("No valid observations after removing missing values.", call. = FALSE)
  }

  # Function to fit gradient model
  fit_gradient <- function(data, var_name, method_name) {
    # Prepare data for modeling
    if (aggregate && "s_id" %in% names(data)) {
      # Aggregate to station means
      agg_data <- data[, .(
        mean_doy = mean(day, na.rm = na.rm),
        var_value = mean(get(var_name), na.rm = na.rm),
        n_obs = .N
      ), by = "s_id"]
    } else {
      agg_data <- data.table::data.table(
        mean_doy = data$day,
        var_value = data[[var_name]],
        n_obs = 1L
      )
    }

    # Remove any remaining NAs
    agg_data <- agg_data[!is.na(mean_doy) & !is.na(var_value)]

    if (nrow(agg_data) < 3) {
      return(list(
        slope = NA_real_,
        intercept = NA_real_,
        r_squared = NA_real_,
        rmse = NA_real_,
        n_points = nrow(agg_data),
        model = NULL,
        data = agg_data,
        error = "Insufficient data points"
      ))
    }

    # Fit model based on method
    tryCatch({
      if (method_name == "robust") {
        model <- robustbase::lmrob(mean_doy ~ var_value, data = agg_data)
        coefs <- coef(model)
        r_sq <- summary(model)$r.squared
      } else if (method_name == "ols") {
        model <- lm(mean_doy ~ var_value, data = agg_data)
        coefs <- coef(model)
        r_sq <- summary(model)$r.squared
      } else {
        # quantile regression
        model <- quantreg::rq(mean_doy ~ var_value, data = agg_data, tau = 0.5)
        coefs <- coef(model)
        # Pseudo R-squared for quantile regression
        null_model <- quantreg::rq(mean_doy ~ 1, data = agg_data, tau = 0.5)
        r_sq <- 1 - sum(abs(residuals(model))) / sum(abs(residuals(null_model)))
      }

      # Calculate RMSE
      rmse <- sqrt(mean(residuals(model)^2))

      # Scale slope appropriately
      if (var_name == "alt") {
        # Report as days per 100m
        slope_scaled <- coefs[2] * 100
        unit <- "days per 100m"
      } else {
        # Report as days per degree
        slope_scaled <- coefs[2]
        unit <- "days per degree"
      }

      list(
        slope = as.numeric(slope_scaled),
        slope_raw = as.numeric(coefs[2]),
        intercept = as.numeric(coefs[1]),
        r_squared = as.numeric(r_sq),
        rmse = as.numeric(rmse),
        n_points = nrow(agg_data),
        unit = unit,
        model = model,
        data = agg_data,
        error = NULL
      )
    }, error = function(e) {
      list(
        slope = NA_real_,
        intercept = NA_real_,
        r_squared = NA_real_,
        rmse = NA_real_,
        n_points = nrow(agg_data),
        model = NULL,
        data = agg_data,
        error = as.character(e$message)
      )
    })
  }

  # Create interpretation text (vectorized)
  create_interpretation <- function(slope, var_name, r_sq) {
    # Handle single value case
    if (length(slope) == 1) {
      if (is.na(slope)) return(NA_character_)

      direction <- if (slope > 0) "later" else "earlier"
      abs_slope <- abs(slope)

      if (var_name == "alt") {
        return(sprintf("Phenology is %.1f days %s per 100m elevation increase (R-sq = %.2f)",
                abs_slope, direction, r_sq))
      } else {
        return(sprintf("Phenology is %.1f days %s per degree latitude increase (R-sq = %.2f)",
                abs_slope, direction, r_sq))
      }
    }

    # Handle vector case
    mapply(function(s, r) {
      if (is.na(s)) return(NA_character_)

      direction <- if (s > 0) "later" else "earlier"
      abs_slope <- abs(s)

      if (var_name == "alt") {
        sprintf("Phenology is %.1f days %s per 100m elevation increase (R-sq = %.2f)",
                abs_slope, direction, r)
      } else {
        sprintf("Phenology is %.1f days %s per degree latitude increase (R-sq = %.2f)",
                abs_slope, direction, r)
      }
    }, slope, r_sq, USE.NAMES = FALSE)
  }

  # Fit models
  if (is.null(by)) {
    # Single gradient for all data
    result_fit <- fit_gradient(dt, variable, method)

    summary_dt <- data.table::data.table(
      variable = variable,
      method = method,
      slope = result_fit$slope,
      intercept = result_fit$intercept,
      r_squared = result_fit$r_squared,
      rmse = result_fit$rmse,
      n_points = result_fit$n_points,
      interpretation = create_interpretation(result_fit$slope, variable, result_fit$r_squared)
    )

    result <- list(
      summary = summary_dt,
      model = result_fit$model,
      data = result_fit$data,
      variable = variable,
      method = method,
      species = species,
      phase_id = phase_id
    )

  } else {
    # Gradient by groups
    if (!all(by %in% names(dt))) {
      stop("Some grouping variables not found in data: ",
           paste(setdiff(by, names(dt)), collapse = ", "), call. = FALSE)
    }

    # Fit gradient for each group
    group_results <- dt[, {
      fit <- fit_gradient(.SD, variable, method)
      list(
        slope = fit$slope,
        intercept = fit$intercept,
        r_squared = fit$r_squared,
        rmse = fit$rmse,
        n_points = fit$n_points
      )
    }, by = by]

    # Add interpretation
    group_results[, interpretation := create_interpretation(slope, variable, r_squared)]

    # Add metadata columns
    group_results[, variable := variable]
    group_results[, method := method]

    # Aggregate data for plotting (same as fit_gradient does internally)
    if (aggregate && "s_id" %in% names(dt)) {
      plot_data <- dt[, .(
        mean_doy = mean(day, na.rm = na.rm),
        var_value = mean(get(variable), na.rm = na.rm),
        n_obs = .N
      ), by = c("s_id", by)]
    } else {
      plot_data <- data.table::data.table(
        mean_doy = dt$day,
        var_value = dt[[variable]],
        n_obs = 1L
      )
      if (!is.null(by)) {
        for (b in by) plot_data[, (b) := dt[[b]]]
      }
    }
    plot_data <- plot_data[!is.na(mean_doy) & !is.na(var_value)]

    result <- list(
      summary = group_results,
      model = NULL,  # Multiple models, not stored individually
      data = plot_data,
      variable = variable,
      method = method,
      by = by,
      species = species,
      phase_id = phase_id
    )
  }

  class(result) <- c("pheno_gradient", "list")
  result
}


#' Print Method for Phenological Gradient Analysis
#'
#' @param x A \code{pheno_gradient} object
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns \code{x}
#' @author Matthias Templ
#' @export
print.pheno_gradient <- function(x, ...) {
  cat("Phenological Gradient Analysis\n")
  cat(strrep("-", 50), "\n")
  cat(sprintf("Gradient variable: %s\n", x$variable))
  cat(sprintf("Regression method: %s\n", x$method))

  if (!is.null(x$species)) {
    cat(sprintf("Species: %s\n", x$species))
  }
  if (!is.null(x$phase_id)) {
    cat(sprintf("Phase ID: %d\n", x$phase_id))
  }

  cat(strrep("-", 50), "\n\n")

  # Print summary table
  print(x$summary)

  # Print interpretation for single gradient
  if (nrow(x$summary) == 1 && !is.na(x$summary$interpretation[1])) {
    cat("\nInterpretation:\n")
    cat("  ", x$summary$interpretation[1], "\n")
  }

  invisible(x)
}


#' Plot Method for Phenological Gradient Analysis
#'
#' @param x A \code{pheno_gradient} object
#' @param ... Additional arguments passed to ggplot
#'
#' @return A ggplot object
#' @author Matthias Templ
#' @export
plot.pheno_gradient <- function(x, ...) {
  if (is.null(x$data) || nrow(x$data) == 0) {
    stop("No data available for plotting", call. = FALSE)
  }

  # Prepare plot data
  plot_data <- x$data

  # Determine x-axis label
  if (x$variable == "alt") {
    x_lab <- "Altitude (m)"
    x_var <- "var_value"
  } else {
    x_lab <- "Latitude (degrees N)"
    x_var <- "var_value"
  }

  # Create plot
  p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = var_value, y = mean_doy)) +
    ggplot2::geom_point(alpha = 0.6) +
    ggplot2::geom_smooth(method = "lm", se = TRUE, color = "steelblue") +
    ggplot2::labs(
      x = x_lab,
      y = "Day of Year",
      title = sprintf("Phenological %s Gradient",
                      ifelse(x$variable == "alt", "Elevation", "Latitude")),
      subtitle = sprintf("Method: %s", x$method)
    ) +
    ggplot2::theme_minimal()

  # Facet by group if available
  if (!is.null(x$by) && length(x$by) > 0) {
    facet_formula <- stats::as.formula(paste("~", paste(x$by, collapse = " + ")))
    p <- p + ggplot2::facet_wrap(facet_formula, scales = "free")
  }

  p
}

#' Analyze Phenological Synchrony Across Stations
#'
#' Measures how synchronized phenological events are across stations within
#' regions and whether synchrony is changing over time. Higher synchrony
#' indicates more uniform timing of phenological events.
#'
#' @param pep A \code{pep} object or \code{data.table} containing phenological
#'   observations with columns \code{year}, \code{day}, and \code{s_id} (station ID).
#' @param species Optional character string to filter by species/genus.
#' @param phase_id Optional integer to filter by a single BBCH phase code.
#' @param by Character vector of column names to group by for synchrony calculation.
#'   Default is \code{c("country", "year")} for annual synchrony by country.
#' @param min_stations Minimum number of stations required to calculate synchrony.
#'   Default is 5. Groups with fewer stations return NA.
#' @param compute_trend Logical. If \code{TRUE} (default), computes trend in
#'   synchrony over time using robust regression.
#' @param na.rm Logical. Should missing values be removed? Default \code{TRUE}.
#'
#' @return A \code{pheno_synchrony} object (list) containing:
#'   \describe{
#'     \item{data}{A \code{data.table} with synchrony metrics per group:
#'       n_stations, mean_doy, sd_doy, cv_pct, and quality indicators}
#'     \item{trend}{If \code{compute_trend = TRUE}, a data.table with trend
#'       analysis results per region (slope, p-value, direction)}
#'     \item{overall}{Summary statistics across all groups}
#'   }
#'
#' @details
#' Synchrony measures how similar phenological timing is across different
#' stations within the same region and year. This is important for:
#' \itemize{
#'   \item Understanding spatial coherence of phenological signals
#'   \item Detecting changes in spatial variability over time
#'   \item Assessing network representativeness
#' }
#'
#' @section Synchrony Metrics:
#' \describe{
#'   \item{sd_doy}{Standard deviation across stations - lower values indicate
#'     higher synchrony}
#'   \item{cv_pct}{Coefficient of variation (SD/mean * 100) - relative measure
#'     that allows comparison across phases with different mean timing}
#'   \item{range_doy}{Range of DOY values across stations}
#' }
#'
#' @section Trend Interpretation:
#' \itemize{
#'   \item Negative trend in SD: Increasing synchrony over time
#'   \item Positive trend in SD: Decreasing synchrony (more variable)
#' }
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Subset to two countries for speed
#' pep_subset <- pep[country %in% c("Switzerland", "Austria")]
#'
#' # Calculate synchrony for apple flowering by country and year
#' sync <- pheno_synchrony(pep_subset,
#'                         species = "Malus",
#'                         phase_id = 60)
#' print(sync)
#'
#' # Get trend results (robust regression per country)
#' sync$trend
#'
#' # Synchrony without trend analysis (faster)
#' sync_simple <- pheno_synchrony(pep_subset,
#'                                species = "Malus",
#'                                phase_id = 60,
#'                                compute_trend = FALSE)
#' sync_simple
#'
#' # Custom grouping variables
#' sync_detailed <- pheno_synchrony(pep_subset,
#'                                  species = "Malus",
#'                                  phase_id = 60,
#'                                  by = c("country", "year"))
#' sync_detailed
#' }
#'
#' @references
#' Rosenzweig, C. et al. (2008). Attributing physical and biological
#' impacts to anthropogenic climate change.
#' \emph{Nature}, 453, 353--357.
#' \doi{10.1038/nature06937}
#'
#' @seealso
#' \code{\link{pheno_normals}} for climatological statistics,
#' \code{\link{pheno_anomaly}} for anomaly detection
#'
#' @author Matthias Templ
#' @export
pheno_synchrony <- function(pep,
                            species = NULL,
                            phase_id = NULL,
                            by = c("country", "year"),
                            min_stations = 5,
                            compute_trend = TRUE,
                            na.rm = TRUE) {

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table if needed
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Check required columns
  required_cols <- c("year", "day", "s_id")
  missing_cols <- setdiff(required_cols, names(pep))
  if (length(missing_cols) > 0) {
    stop("Missing required columns: ", paste(missing_cols, collapse = ", "),
         call. = FALSE)
  }

  # Check that 'by' columns exist
  missing_by <- setdiff(by, names(pep))
  if (length(missing_by) > 0) {
    stop("Column(s) specified in 'by' not found: ",
         paste(missing_by, collapse = ", "),
         call. = FALSE)
  }

  # Validate min_stations
  if (!is.numeric(min_stations) || min_stations < 2) {
    stop("'min_stations' must be a numeric value >= 2", call. = FALSE)
  }

  # Make a copy to avoid modifying original
  dt <- data.table::copy(pep)

  # Apply species filter
  if (!is.null(species)) {
    if ("genus" %in% names(dt)) {
      genus_match <- dt$genus == species
      species_match <- if ("species" %in% names(dt)) dt$species == species else FALSE
      dt <- dt[genus_match | species_match]
      if (nrow(dt) == 0) {
        stop("No observations found for species '", species, "'", call. = FALSE)
      }
    } else if ("species" %in% names(dt)) {
      dt <- dt[dt$species == species]
    } else {
      warning("Neither 'genus' nor 'species' column found.", call. = FALSE)
    }
  }

  # Apply phase_id filter
  if (!is.null(phase_id)) {
    if (!"phase_id" %in% names(dt)) {
      warning("'phase_id' column not found.", call. = FALSE)
    } else {
      if (length(phase_id) > 1) {
        warning("Multiple phase_ids provided. Using first one only.",
                call. = FALSE)
        phase_id <- phase_id[1]
      }
      pid <- phase_id
      dt <- dt[dt$phase_id == pid]
      if (nrow(dt) == 0) {
        stop("No observations found for phase_id: ", phase_id, call. = FALSE)
      }
    }
  }

  # Remove missing values
  dt <- dt[!is.na(day)]

  # First aggregate to station-level means within each group
  # This handles multiple observations per station-year
  station_means <- dt[, .(
    station_mean_doy = mean(day, na.rm = na.rm)
  ), by = c(by, "s_id")]

  # Calculate synchrony metrics by group
  sync_data <- station_means[, {
    doy_values <- station_mean_doy
    n_stations <- .N

    if (n_stations >= min_stations) {
      mean_doy <- mean(doy_values, na.rm = na.rm)
      sd_doy <- sd(doy_values, na.rm = na.rm)
      cv_pct <- if (!is.na(mean_doy) && mean_doy != 0) {
        100 * sd_doy / mean_doy
      } else {
        NA_real_
      }
      range_doy <- max(doy_values, na.rm = na.rm) - min(doy_values, na.rm = na.rm)
      iqr_doy <- IQR(doy_values, na.rm = na.rm)

      list(
        n_stations = as.integer(n_stations),
        mean_doy = round(mean_doy, 1),
        sd_doy = round(sd_doy, 2),
        cv_pct = round(cv_pct, 2),
        range_doy = as.integer(range_doy),
        iqr_doy = round(iqr_doy, 1),
        min_doy = as.integer(min(doy_values, na.rm = na.rm)),
        max_doy = as.integer(max(doy_values, na.rm = na.rm))
      )
    } else {
      list(
        n_stations = as.integer(n_stations),
        mean_doy = NA_real_,
        sd_doy = NA_real_,
        cv_pct = NA_real_,
        range_doy = NA_integer_,
        iqr_doy = NA_real_,
        min_doy = NA_integer_,
        max_doy = NA_integer_
      )
    }
  }, by = by]

  # Order results
  data.table::setorderv(sync_data, by)

  # Calculate trend in synchrony if requested
  trend_data <- NULL
  if (compute_trend && "year" %in% by) {
    # Identify grouping variables other than year
    region_vars <- setdiff(by, "year")

    if (length(region_vars) > 0) {
      # Calculate trend for each region
      trend_data <- sync_data[!is.na(sd_doy), {
        if (.N >= 10) {  # Need at least 10 years for trend
          tryCatch({
            # Use robust regression
            model <- robustbase::lmrob(sd_doy ~ year, data = .SD)
            coefs <- coef(model)
            summ <- summary(model)

            list(
              n_years = as.integer(.N),
              year_min = as.integer(min(year)),
              year_max = as.integer(max(year)),
              slope = as.numeric(round(coefs[2], 4)),
              intercept = as.numeric(round(coefs[1], 2)),
              r_squared = as.numeric(round(summ$r.squared, 3)),
              p_value = as.numeric(round(summ$coefficients[2, 4], 4)),
              direction = as.character(ifelse(coefs[2] < 0,
                                              "increasing synchrony",
                                              "decreasing synchrony")),
              significant = as.logical(summ$coefficients[2, 4] < 0.05)
            )
          }, error = function(e) {
            list(
              n_years = as.integer(.N),
              year_min = as.integer(min(year)),
              year_max = as.integer(max(year)),
              slope = NA_real_,
              intercept = NA_real_,
              r_squared = NA_real_,
              p_value = NA_real_,
              direction = NA_character_,
              significant = NA
            )
          })
        } else {
          list(
            n_years = as.integer(.N),
            year_min = as.integer(min(year)),
            year_max = as.integer(max(year)),
            slope = NA_real_,
            intercept = NA_real_,
            r_squared = NA_real_,
            p_value = NA_real_,
            direction = NA_character_,
            significant = NA
          )
        }
      }, by = region_vars]
    } else {
      # Single region (no grouping besides year)
      if (sum(!is.na(sync_data$sd_doy)) >= 10) {
        valid_data <- sync_data[!is.na(sd_doy)]
        tryCatch({
          model <- robustbase::lmrob(sd_doy ~ year, data = valid_data)
          coefs <- coef(model)
          summ <- summary(model)

          trend_data <- data.table::data.table(
            n_years = nrow(valid_data),
            year_min = min(valid_data$year),
            year_max = max(valid_data$year),
            slope = round(coefs[2], 4),
            intercept = round(coefs[1], 2),
            r_squared = round(summ$r.squared, 3),
            p_value = round(summ$coefficients[2, 4], 4),
            direction = ifelse(coefs[2] < 0, "increasing synchrony", "decreasing synchrony"),
            significant = summ$coefficients[2, 4] < 0.05
          )
        }, error = function(e) {
          trend_data <- NULL
        })
      }
    }
  }

  # Calculate overall statistics
  valid_sync <- sync_data[!is.na(sd_doy)]
  overall <- list(
    n_groups = nrow(sync_data),
    n_valid_groups = nrow(valid_sync),
    mean_sd_doy = if (nrow(valid_sync) > 0) mean(valid_sync$sd_doy, na.rm = TRUE) else NA,
    median_sd_doy = if (nrow(valid_sync) > 0) median(valid_sync$sd_doy, na.rm = TRUE) else NA,
    mean_cv_pct = if (nrow(valid_sync) > 0) mean(valid_sync$cv_pct, na.rm = TRUE) else NA,
    mean_n_stations = if (nrow(valid_sync) > 0) mean(valid_sync$n_stations, na.rm = TRUE) else NA
  )

  # Build result
  result <- list(
    data = sync_data,
    trend = trend_data,
    overall = overall,
    by = by,
    min_stations = min_stations,
    species = species,
    phase_id = phase_id
  )

  class(result) <- c("pheno_synchrony", "list")
  result
}


#' Print Method for Phenological Synchrony Analysis
#'
#' @param x A \code{pheno_synchrony} object
#' @param n Number of rows to display
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns \code{x}
#' @author Matthias Templ
#' @export
print.pheno_synchrony <- function(x, n = 10, ...) {
  cat("Phenological Synchrony Analysis\n")
  cat(strrep("-", 50), "\n")

  if (!is.null(x$species)) {
    cat(sprintf("Species: %s\n", x$species))
  }
  if (!is.null(x$phase_id)) {
    cat(sprintf("Phase ID: %d\n", x$phase_id))
  }
  cat(sprintf("Grouping: %s\n", paste(x$by, collapse = ", ")))
  cat(sprintf("Min stations required: %d\n", x$min_stations))

  cat(strrep("-", 50), "\n\n")

  # Overall statistics
  cat("Overall Statistics:\n")
  cat(sprintf("  Total groups: %d (valid: %d)\n",
              x$overall$n_groups, x$overall$n_valid_groups))
  cat(sprintf("  Mean SD across stations: %.1f days\n", x$overall$mean_sd_doy))
  cat(sprintf("  Mean CV: %.1f%%\n", x$overall$mean_cv_pct))
  cat(sprintf("  Mean stations per group: %.1f\n", x$overall$mean_n_stations))

  cat("\n")

  # Print data
  cat("Synchrony Data:\n")
  print(x$data[1:min(n, nrow(x$data))])

  if (nrow(x$data) > n) {
    cat(sprintf("\n... and %d more rows\n", nrow(x$data) - n))
  }

  # Print trend if available
  if (!is.null(x$trend) && nrow(x$trend) > 0) {
    cat("\nTrend Analysis (change in SD over time):\n")
    print(x$trend)

    # Interpretation
    sig_trends <- x$trend[significant == TRUE]
    if (nrow(sig_trends) > 0) {
      cat("\nSignificant trends detected:\n")
      for (i in seq_len(nrow(sig_trends))) {
        row <- sig_trends[i]
        if (length(setdiff(x$by, "year")) > 0) {
          region_name <- paste(as.character(row[, setdiff(x$by, "year"), with = FALSE]),
                               collapse = ", ")
        } else {
          region_name <- "Overall"
        }
        cat(sprintf("  %s: %s (p = %.4f)\n",
                    region_name, row$direction, row$p_value))
      }
    }
  }

  invisible(x)
}


#' Summary Method for Phenological Synchrony Analysis
#'
#' @param object A \code{pheno_synchrony} object
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns a summary list
#' @author Matthias Templ
#' @export
summary.pheno_synchrony <- function(object, ...) {
  cat("Phenological Synchrony Summary\n")
  cat(strrep("=", 50), "\n\n")

  if (!is.null(object$species)) {
    cat(sprintf("Species: %s\n", object$species))
  }
  if (!is.null(object$phase_id)) {
    cat(sprintf("Phase ID: %d\n", object$phase_id))
  }

  cat(sprintf("\nGroups analyzed: %d\n", object$overall$n_groups))
  cat(sprintf("Groups with sufficient stations: %d (%.1f%%)\n",
              object$overall$n_valid_groups,
              100 * object$overall$n_valid_groups / object$overall$n_groups))

  cat("\nSynchrony Statistics (across all valid groups):\n")
  cat(sprintf("  Mean SD: %.1f days\n", object$overall$mean_sd_doy))
  cat(sprintf("  Median SD: %.1f days\n", object$overall$median_sd_doy))
  cat(sprintf("  Mean CV: %.1f%%\n", object$overall$mean_cv_pct))

  # SD distribution
  valid_data <- object$data[!is.na(sd_doy)]
  if (nrow(valid_data) > 0) {
    cat("\nSD Distribution:\n")
    cat(sprintf("  Min: %.1f days\n", min(valid_data$sd_doy)))
    cat(sprintf("  25th percentile: %.1f days\n", quantile(valid_data$sd_doy, 0.25)))
    cat(sprintf("  Median: %.1f days\n", median(valid_data$sd_doy)))
    cat(sprintf("  75th percentile: %.1f days\n", quantile(valid_data$sd_doy, 0.75)))
    cat(sprintf("  Max: %.1f days\n", max(valid_data$sd_doy)))
  }

  # Trend summary
  if (!is.null(object$trend) && nrow(object$trend) > 0) {
    cat("\nTrend Summary:\n")
    n_sig <- sum(object$trend$significant, na.rm = TRUE)
    n_increasing <- sum(object$trend$direction == "increasing synchrony" &
                        object$trend$significant, na.rm = TRUE)
    n_decreasing <- sum(object$trend$direction == "decreasing synchrony" &
                        object$trend$significant, na.rm = TRUE)

    cat(sprintf("  Regions with significant trends: %d of %d\n",
                n_sig, nrow(object$trend)))
    cat(sprintf("  Increasing synchrony: %d\n", n_increasing))
    cat(sprintf("  Decreasing synchrony: %d\n", n_decreasing))
  }

  invisible(object$overall)
}


#' Plot Method for Phenological Synchrony Analysis
#'
#' Creates a time series plot of synchrony (SD) over years.
#'
#' @param x A \code{pheno_synchrony} object
#' @param region Optional region to plot (for multi-region analyses)
#' @param ... Additional arguments passed to ggplot
#'
#' @return A ggplot object
#' @author Matthias Templ
#' @export
plot.pheno_synchrony <- function(x, region = NULL, ...) {
  if (!"year" %in% x$by) {
    stop("Plot requires 'year' in grouping variables", call. = FALSE)
  }

  plot_data <- x$data[!is.na(sd_doy)]

  if (nrow(plot_data) == 0) {
    stop("No valid data for plotting", call. = FALSE)
  }

  # Filter to specific region if requested
  region_vars <- setdiff(x$by, "year")
  if (!is.null(region) && length(region_vars) > 0) {
    # Simple filter for single region variable
    if (length(region_vars) == 1) {
      plot_data <- plot_data[get(region_vars) == region]
    }
  }

  # Create base plot
  if (length(region_vars) > 0 && is.null(region)) {
    # Multiple regions - faceted plot
    p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = year, y = sd_doy)) +
      ggplot2::geom_line(alpha = 0.7) +
      ggplot2::geom_point(ggplot2::aes(size = n_stations), alpha = 0.6) +
      ggplot2::geom_smooth(method = "lm", se = TRUE, color = "steelblue", linewidth = 0.8) +
      ggplot2::facet_wrap(as.formula(paste("~", region_vars[1])), scales = "free_y") +
      ggplot2::labs(
        x = "Year",
        y = "SD across stations (days)",
        title = "Phenological Synchrony Over Time",
        subtitle = "Lower SD = Higher synchrony",
        size = "N stations"
      ) +
      ggplot2::theme_minimal()
  } else {
    # Single region or overall
    p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = year, y = sd_doy)) +
      ggplot2::geom_line(alpha = 0.7) +
      ggplot2::geom_point(ggplot2::aes(size = n_stations), alpha = 0.6) +
      ggplot2::geom_smooth(method = "lm", se = TRUE, color = "steelblue", linewidth = 0.8) +
      ggplot2::labs(
        x = "Year",
        y = "SD across stations (days)",
        title = "Phenological Synchrony Over Time",
        subtitle = ifelse(!is.null(region),
                          paste("Region:", region),
                          "Lower SD = Higher synchrony"),
        size = "N stations"
      ) +
      ggplot2::theme_minimal()
  }

  p
}

#' Select and Label Phenophases from PEP Time Series
#'
#' Extracts a subset of phenological observations from a processed PEP time series (e.g., output from species-level aggregation),
#' filters by species, year, and phenophase ID, and attaches meaningful phase labels based on standard BBCH codes.
#'
#' @param dt A \code{data.table} containing columns \code{species}, \code{year}, \code{phase_id}, and \code{mean_day}.
#' Typically produced by aggregating PEP725 data (e.g., via \code{\link{pheno_regional}}).
#' @param label A character string used as identifier for source and site columns in the output.
#' @param sp Species name (as character string) to filter, e.g., \code{"Triticum aestivum"}.
#' @param yrmin Minimum year (inclusive) for filtering.
#' @param phases Integer vector of phase IDs to include (default: \code{c(60, 100)} corresponding to "Heading" and "Harvest").
#'
#' @return A \code{data.table} with columns:
#' \describe{
#'   \item{\code{year}}{Year of observation}
#'   \item{\code{phase}}{Mapped BBCH phenological stage name}
#'   \item{\code{DOY}}{Mean day-of-year for this phase, species, and year}
#'   \item{\code{source}}{Set to \code{label}}
#'   \item{\code{site}}{Set to \code{label}}
#' }
#'
#' @details
#' This function uses an internal lookup table to map numeric \code{phase_id} codes to descriptive BBCH phase names (e.g., \code{60 = "Heading"}, \code{100 = "Harvest"}).
#' Unmapped phase IDs will trigger a warning. If expected phases are missing from the result, a separate warning is issued.
#'
#' The function is particularly useful for visualizing specific growth stages or comparing phenological trends across datasets and locations.
#'
#' @seealso \code{\link{pheno_regional}}, \code{\link{pep_download}}, \code{\link{pheno_plot}}
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' agg <- pheno_regional(pep, species_name = "Triticum aestivum", phase = 60)$pep_agg
#' df <- select_phase(agg, label = "PEP725", sp = "Triticum aestivum", yrmin = 1961)
#' head(df)
#' }
#'
#' @author Matthias Templ
#' @export
select_phase <- function(dt, label, sp, yrmin, phases = c(60, 100)) {
  phase_lookup <- .bbch_lookup

  # Mapping function
  phase_name <- function(id) {
    res <- as.character(phase_lookup[as.character(id)])
    if (any(is.na(res))) {
      warning(sprintf("Unmapped phase_id(s): %s",
                      paste(setdiff(id, as.integer(names(phase_lookup))), collapse = ", ")),
              call. = FALSE)
    }
    res
  }

  # Core logic
  out <- dt[
    species == sp &
      year >= yrmin & phase_id %in% phases,
    .(year, phase = phase_name(phase_id), DOY = mean_day)
  ][order(year)][
    , `:=`(source = label, site = label)
  ][]

  # Optional: warn if expected phases are missing
  expected_phases <- phase_name(phases)
  observed_phases <- unique(out$phase)
  missing <- setdiff(expected_phases, observed_phases)
  if (length(missing) > 0L) {
    warning(sprintf(
      "%s (year >= %d): missing phase(s): %s",
      label, yrmin, paste(missing, collapse = ", ")
    ), call. = FALSE)
  }

  return(out)
}

#' Download Synthetic PEP725 Data
#'
#' Downloads a pre-generated synthetic version of PEP725 phenological data
#' from an external repository. The data is cached locally after the first
#' download to avoid repeated downloads.
#'
#' @param url Character string specifying the URL to download from.
#'   Default uses the official pep725 package data repository.
#' @param cache Logical. If \code{TRUE} (default), cache the downloaded data
#'   locally for future use.
#' @param force Logical. If \code{TRUE}, force re-download even if cached
#'   data exists. Default is \code{FALSE}.
#' @param quiet Logical. If \code{TRUE}, suppress download progress messages.
#'   Default is \code{FALSE}.
#'
#' @return A \code{pep} object containing synthetic phenological data with
#'   the same structure as original PEP725 data.
#'
#' @details
#' The synthetic data preserves the statistical properties and structure of
#' real PEP725 data (stations, species, phases, temporal patterns) while
#' ensuring data privacy. It is suitable for:
#' \itemize{
#'   \item Learning and testing package functions
#'   \item Reproducing examples from vignettes
#'   \item Developing analysis workflows before accessing real data
#' }
#'
#' The data is cached in the platform-appropriate user cache directory
#' determined by \code{\link[tools]{R_user_dir}("pep725", which = "cache")}.
#' Typical locations:
#' \itemize{
#'   \item Windows: \code{\%LOCALAPPDATA\%/R/cache/R/pep725/}
#'   \item macOS: \code{~/Library/Caches/org.R-project.R/R/pep725/}
#'   \item Linux: \code{~/.cache/R/pep725/}
#' }
#'
#' @section Data Access:
#' If you have access to real PEP725 data, you can import it using
#' \code{\link{pep_import}} instead.
#'
#' @examples
#' \donttest{
#' # Download synthetic data
#' pep <- pep_download()
#'
#' # Explore the data
#' print(pep)
#' summary(pep)
#'
#' # Force re-download
#' pep <- pep_download(force = TRUE)
#' }
#'
#' @seealso
#' \code{\link{pep_simulate}} for creating your own synthetic data,
#' \code{\link{pep_import}} for importing real PEP725 data
#'
#' @author Matthias Templ
#' @export
pep_download <- function(url = NULL,
                         cache = TRUE,
                         force = FALSE,
                         quiet = FALSE) {

 # Default URL - update this to your actual data repository
 if (is.null(url)) {
    url <- "https://github.com/matthias-da/pep725_releases/raw/main/downloads/v0.1.0/pep_synth.rda"
  }

  # Determine cache directory
  # Use tempdir() during R CMD check to avoid writing to user directories
  # (flagged by _R_CHECK_THINGS_IN_OTHER_DIRS_)
  in_check <- nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_")) ||
    nzchar(Sys.getenv("_R_CHECK_THINGS_IN_OTHER_DIRS_")) ||
    !identical(Sys.getenv("R_TESTS"), "")
  if (in_check) {
    cache_dir <- tempdir()
  } else {
    cache_dir <- tools::R_user_dir("pep725", which = "cache")
  }
  cache_file <- file.path(cache_dir, "pep_synth.rda")

  # Check if cached version exists
  if (cache && file.exists(cache_file) && !force) {
    if (!quiet) {
      message("Loading cached synthetic PEP data from: ", cache_file)
    }

    pep <- tryCatch({
      env <- new.env()
      var_names <- load(cache_file, envir = env)
      obj <- get(var_names[1], envir = env)
      new_pep(obj)
    }, error = function(e) {
      warning("Cached file appears corrupted. Re-downloading...", call. = FALSE)
      NULL
    })

    if (!is.null(pep)) {
      return(pep)
    }
  }

  # Download the data
  if (!quiet) {
    message("Downloading synthetic PEP data from: ", url)
  }

  # Create temp file for download
  temp_file <- tempfile(fileext = ".rda")

  # Download with error handling
  download_success <- tryCatch({
    utils::download.file(
      url = url,
      destfile = temp_file,
      mode = "wb",
      quiet = quiet
    )
    TRUE
  }, error = function(e) {
    FALSE
  }, warning = function(w) {
    # download.file returns warnings for HTTP errors
    FALSE
  })
  if (!download_success || !file.exists(temp_file) || file.size(temp_file) == 0) {
    stop(
      "Failed to download synthetic PEP data.\n",
      "Please check your internet connection and try again.\n",
      "URL: ", url, "\n\n",
      "Alternatively, you can create synthetic data using pep_simulate() ",
      "if you have access to real PEP725 data.",
      call. = FALSE
    )
  }

  # Read the downloaded data and validate via new_pep()
  pep <- tryCatch({
    env <- new.env()
    var_names <- load(temp_file, envir = env)
    obj <- get(var_names[1], envir = env)
    new_pep(obj)
  }, error = function(e) {
    stop(
      "Downloaded file could not be read or is not valid PEP data.\n",
      "Error: ", e$message,
      call. = FALSE
    )
  })

  # Cache the data
  if (cache) {
    # Create cache directory if it doesn't exist
    if (!dir.exists(cache_dir)) {
      dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
    }

    tryCatch({
      save(pep, file = cache_file)
      if (!quiet) {
        message("Data cached to: ", cache_file)
      }
    }, error = function(e) {
      warning("Could not cache data: ", e$message, call. = FALSE)
    })
  }

  # Clean up temp file
  unlink(temp_file)

  if (!quiet) {
    message("Download complete. ",
            format(nrow(pep), big.mark = ","), " observations loaded.")
  }

  pep
}


#' Clear Cached PEP Data
#'
#' Removes cached synthetic PEP data from the local cache directory.
#'
#' @param quiet Logical. If \code{TRUE}, suppress messages. Default is \code{FALSE}.
#'
#' @return Invisible \code{TRUE} if cache was cleared, \code{FALSE} otherwise.
#'
#' @examples
#' \donttest{
#' # Clear the cache
#' pep_cache_clear()
#'
#' # Next download will fetch fresh data
#' pep <- pep_download()
#' }
#'
#' @seealso \code{\link{pep_download}}
#'
#' @author Matthias Templ
#' @export
pep_cache_clear <- function(quiet = FALSE) {
  in_check <- nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_")) ||
    nzchar(Sys.getenv("_R_CHECK_THINGS_IN_OTHER_DIRS_")) ||
    !identical(Sys.getenv("R_TESTS"), "")
  if (in_check) {
    cache_dir <- tempdir()
  } else {
    cache_dir <- tools::R_user_dir("pep725", which = "cache")
  }
  cache_file <- file.path(cache_dir, "pep_synth.rda")

  if (file.exists(cache_file)) {
    unlink(cache_file)
    if (!quiet) {
      message("Cache cleared: ", cache_file)
    }
    return(invisible(TRUE))
  } else {
    if (!quiet) {
      message("No cached data found.")
    }
    return(invisible(FALSE))
  }
}


#' Get Cache Information
#'
#' Returns information about the cached PEP data.
#'
#' @return A list with cache information, or NULL if no cache exists.
#'
#' @examples
#' \donttest{
#' # Check cache status
#' pep_cache_info()
#' }
#'
#' @author Matthias Templ
#' @export
pep_cache_info <- function() {
  in_check <- nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_")) ||
    nzchar(Sys.getenv("_R_CHECK_THINGS_IN_OTHER_DIRS_")) ||
    !identical(Sys.getenv("R_TESTS"), "")
  if (in_check) {
    cache_dir <- tempdir()
  } else {
    cache_dir <- tools::R_user_dir("pep725", which = "cache")
  }
  cache_file <- file.path(cache_dir, "pep_synth.rda")

  if (file.exists(cache_file)) {
    info <- file.info(cache_file)
    list(
      path = cache_file,
      size_mb = round(info$size / 1024^2, 2),
      modified = info$mtime,
      exists = TRUE
    )
  } else {
    list(
      path = cache_file,
      size_mb = NA,
      modified = NA,
      exists = FALSE
    )
  }
}

# Prevent R CMD check notes
utils::globalVariables(c("lon", "lat", "n", "n_species", "mean_doy", "trend",
                         "species_cv", "year", "day", "species", "s_id",
                         "annual_mean", "species_mean"))

#' Plot Phenology Station Maps
#'
#' Generates a map of PEP725 phenological station locations with optional coloring
#' by various statistics including mean phenological timing, trends, and species
#' variation.
#'
#' @param pep A `data.table` or `data.frame` with phenological observations.
#'   Required columns depend on \code{color_by}:
#'   \itemize{
#'     \item Always required: \code{lon}, \code{lat}
#'     \item For \code{"n_species"}, \code{"mean_doy"}, \code{"species_cv"}: \code{species}
#'     \item For \code{"mean_doy"}, \code{"trend"}: \code{year}, \code{day}
#'   }
#' @param location Named vector with center longitude and latitude for the map.
#'   Defaults to near Changins: \code{c(lon = 6.233, lat = 46.400)}.
#'   Used primarily for \code{background = "google"}.
#' @param zoom Zoom level for the map. For \code{background = "google"}, this is
#'   the Google Maps zoom level (4 = Europe, 7 = regional). For
#'   \code{background = "none"}, this controls the padding around data extent
#'   (4 = wide view, 7 = tight view). Default is 4.
#' @param background Character. Map background type:
#'   \describe{
#'     \item{"google"}{Google Maps satellite/terrain background. Requires API key
#'       (set via \code{ggmap::register_google()} or \code{key} parameter).}
#'     \item{"none"}{Simple map with country borders from Natural Earth data.
#'       No API key required. Good for publications and vignettes.}
#'   }
#' @param color_by Character. What to color stations by:
#'   \describe{
#'     \item{"none"}{Black points showing station locations only}
#'     \item{"n_obs"}{Number of observations per station}
#'     \item{"n_species"}{Number of species recorded per station}
#'     \item{"mean_doy"}{Mean day-of-year (phenological timing) per station}
#'     \item{"trend"}{Trend in days/year per station (positive = later, negative = earlier)}
#'     \item{"species_cv"}{Coefficient of variation in timing across species at each station
#'       (higher = more variation among species)}
#'   }
#' @param phase_id Integer. BBCH phase code to filter by. Recommended for
#'   \code{"mean_doy"} and \code{"trend"} to ensure meaningful comparisons.
#'   If \code{NULL} (default), all phases are included.
#' @param period Integer vector of years to include. Default is all years.
#'   For trend calculation, at least 10 years are recommended.
#' @param min_years Integer. Minimum years required for trend calculation.
#'   Stations with fewer years show as NA. Default is 10.
#' @param min_species Integer. Minimum species required for \code{species_cv}.
#'   Default is 3.
#' @param point_size Size of station points (default: 0.8).
#' @param output_file Optional file path to export the plot (e.g. \code{"map.pdf"}).
#' @param key Google Maps API key. Only needed for \code{background = "google"}.
#'   You can set globally via \code{ggmap::register_google()} instead.
#'
#' @return A ggplot map object.
#'
#' @details
#' The function supports two background types:
#'
#' \strong{Google Maps background} (\code{background = "google"}):
#' Provides detailed satellite or terrain imagery but requires a Google Maps API key.
#' Register at \url{https://console.cloud.google.com/} and enable the Maps Static API.
#'
#' \strong{No background} (\code{background = "none"}):
#' Uses country outlines from Natural Earth data. No API key required, making it
#' suitable for:
#' \itemize{
#'   \item CRAN package vignettes (no external API calls)
#'   \item Reproducible research (no authentication needed)
#'   \item Publication-quality maps with clean appearance
#' }
#'
#' The color options provide insights into:
#' \itemize{
#'   \item \strong{mean_doy}: Spatial patterns in phenological timing. Earlier
#'     timing (lower DOY) typically in southern/lowland areas.
#'   \item \strong{trend}: Where phenology is advancing (negative = earlier over time,
#'     shown in blue) or delaying (positive, shown in red). Uses Kendall's tau
#'     normalized statistic for robustness.
#'   \item \strong{species_cv}: Stations where different species show similar timing
#'     (low CV) vs. divergent timing (high CV). High variation may indicate
#'     species-specific responses to local conditions.
#' }
#'
#' @section Interpreting Trends:
#' The trend is calculated as Kendall's normalized tau statistic, which ranges
#' roughly from -3 to +3 for typical data:
#' \itemize{
#'   \item Negative values (blue): Phenology is getting earlier over time
#'   \item Positive values (red): Phenology is getting later over time
#'   \item Values near 0: No clear trend
#'   \item |tau| > 1.96: Statistically significant at 95% level
#' }
#'
#' @seealso \code{\link{pheno_normals}} for detailed normal calculations,
#'   \code{\link{mann_kendall_z}} for trend calculation,
#'   \code{\link{pheno_synchrony}} for synchrony analysis
#'
#' @import ggplot2
#' @importFrom stats sd coef lm na.omit
#' @importFrom sf st_as_sf st_bbox
#' @examples
#' \donttest{
#' pep <- pep_download()
#' pep_alpine <- pep[country %in% c("Switzerland", "Austria")]
#' pheno_map(pep_alpine, background = "none", color_by = "n_obs")
#' }
#' \dontrun{
#' # With Google Maps background (requires API key)
#' ggmap::register_google(key = "your_api_key_here")
#' pheno_map(pep, background = "google", color_by = "n_species", zoom = 5)
#' }
#' @author Matthias Templ
#' @export
pheno_map <- function(
    pep,
    location = c(lon = 6.233, lat = 46.400),
    zoom = 4,
    background = c("google", "none"),
    color_by = c("none", "n_obs", "n_species", "mean_doy", "trend", "species_cv"),
    phase_id = NULL,
    period = NULL,
    min_years = 10,
    min_species = 3,
    point_size = 0.8,
    output_file = NULL,
    key = NULL
) {
  background <- match.arg(background)
  color_by <- match.arg(color_by)

  # Input validation
  if (!all(c("lon", "lat") %in% names(pep))) {
    stop("'pep' must contain 'lon' and 'lat' columns", call. = FALSE)
  }

  # Convert to data.table if needed
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Make a working copy
  dt <- data.table::copy(pep)

  # Apply phase filter if specified
  if (!is.null(phase_id)) {
    if (!"phase_id" %in% names(dt)) {
      warning("'phase_id' column not found. Ignoring phase_id filter.", call. = FALSE)
    } else {
      dt <- dt[dt$phase_id %in% phase_id]
      if (nrow(dt) == 0) {
        stop("No observations found for phase_id: ", paste(phase_id, collapse = ", "),
             call. = FALSE)
      }
    }
  }

  # Apply period filter if specified
  if (!is.null(period)) {
    if (!"year" %in% names(dt)) {
      warning("'year' column not found. Ignoring period filter.", call. = FALSE)
    } else {
      dt <- dt[dt$year %in% period]
      if (nrow(dt) == 0) {
        stop("No observations found in specified period", call. = FALSE)
      }
    }
  }

  # Create base map depending on background type
  if (background == "google") {
    # Google Maps background (requires API key)
    if (!requireNamespace("ggmap", quietly = TRUE)) {
      stop("Package 'ggmap' is required for background = \"google\". ",
           "Install it with: install.packages(\"ggmap\")", call. = FALSE)
    }
    if (!is.null(key)) {
      ggmap::register_google(key = key)
    }

    gmap <- ggmap::get_map(location = location, zoom = zoom)
    g <- ggmap::ggmap(gmap) +
      theme_minimal()

  } else {

    # No background - use Natural Earth country outlines
    # Get world countries
    world <- rnaturalearth::ne_countries(scale = "medium", returnclass = "sf")

    # Calculate extent from data with padding based on zoom
    lon_range <- range(dt$lon, na.rm = TRUE)
    lat_range <- range(dt$lat, na.rm = TRUE)

    # Padding factor: higher zoom = less padding
    # zoom 4 -> 50% padding, zoom 7 -> 10% padding
    padding_factor <- max(0.1, 0.7 - zoom * 0.1)

    lon_pad <- diff(lon_range) * padding_factor
    lat_pad <- diff(lat_range) * padding_factor

    # Ensure minimum extent for small datasets
    if (lon_pad < 1) lon_pad <- 2
    if (lat_pad < 1) lat_pad <- 2

    lon_min <- lon_range[1] - lon_pad
    lon_max <- lon_range[2] + lon_pad
    lat_min <- lat_range[1] - lat_pad
    lat_max <- lat_range[2] + lat_pad

    g <- ggplot() +
      geom_sf(data = world, fill = "gray95", color = "gray60", linewidth = 0.3) +
      coord_sf(xlim = c(lon_min, lon_max), ylim = c(lat_min, lat_max),
               expand = FALSE) +
      theme_minimal() +
      theme(
        panel.grid.major = element_line(color = "gray85", linewidth = 0.2),
        panel.background = element_rect(fill = "aliceblue", color = NA)
      )
  }

  # Build the plot based on color_by option
  if (color_by == "none") {
    # Just plot unique lon/lat
    unique_coords <- unique(dt[, c("lon", "lat")])
    g <- g + geom_point(data = unique_coords, aes(x = lon, y = lat),
                        size = point_size, color = "black")
    title <- "Phenology Stations"

  } else if (color_by == "n_obs") {
    nobs <- dt[, .(n = .N), by = .(lon, lat)]
    g <- g + geom_point(data = nobs, aes(x = lon, y = lat, color = n),
                        size = point_size) +
      scale_color_viridis_c(name = "No. obs.", option = "plasma")
    title <- "Number of Observations per Station"

  } else if (color_by == "n_species") {
    if (!"species" %in% names(dt)) {
      stop("'species' column required for color_by = 'n_species'", call. = FALSE)
    }
    nsp <- dt[, .(n_species = data.table::uniqueN(species)), by = .(lon, lat)]
    g <- g + geom_point(data = nsp, aes(x = lon, y = lat, color = n_species),
                        size = point_size) +
      scale_color_viridis_c(name = "No. species", option = "plasma")
    title <- "Number of Species per Station"

  } else if (color_by == "mean_doy") {
    # Check required columns
    if (!all(c("day", "year") %in% names(dt))) {
      stop("'day' and 'year' columns required for color_by = 'mean_doy'", call. = FALSE)
    }

    # Calculate mean DOY per station
    # First get annual means, then average across years for robustness
    station_means <- dt[, .(annual_mean = mean(day, na.rm = TRUE)),
                        by = .(lon, lat, year)]
    station_means <- station_means[, .(mean_doy = mean(annual_mean, na.rm = TRUE),
                                        n_years = .N),
                                    by = .(lon, lat)]

    g <- g + geom_point(data = station_means, aes(x = lon, y = lat, color = mean_doy),
                        size = point_size) +
      scale_color_viridis_c(name = "Mean DOY", option = "plasma",
                            direction = -1)  # Earlier = blue, later = yellow
    title <- paste0("Mean Phenological Timing",
                    if (!is.null(phase_id)) paste0(" (Phase ", phase_id, ")") else "")

  } else if (color_by == "trend") {
    # Check required columns
    if (!all(c("day", "year") %in% names(dt))) {
      stop("'day' and 'year' columns required for color_by = 'trend'", call. = FALSE)
    }

    # Calculate trend per station using Kendall's tau
    # First aggregate to annual means per station
    annual_data <- dt[, .(annual_mean = mean(day, na.rm = TRUE)),
                      by = .(lon, lat, year)][order(lon, lat, year)]

    # Calculate trend for each station
    station_trends <- annual_data[, {
      if (.N >= min_years) {
        # Use Mann-Kendall Z-statistic for robust trend estimation
        tau <- mann_kendall_z(annual_mean)
        list(trend = tau, n_years = .N)
      } else {
        list(trend = NA_real_, n_years = .N)
      }
    }, by = .(lon, lat)]

    # Remove stations with NA trends for cleaner plotting
    station_trends_valid <- station_trends[!is.na(trend)]

    if (nrow(station_trends_valid) == 0) {
      warning("No stations have enough years for trend calculation (min_years = ",
              min_years, ")", call. = FALSE)
      # Fall back to showing all stations in gray
      g <- g + geom_point(data = unique(dt[, .(lon, lat)]),
                          aes(x = lon, y = lat),
                          size = point_size, color = "gray50")
    } else {
      # Use diverging color scale: blue = earlier (negative), red = later (positive)
      max_abs <- max(abs(station_trends_valid$trend), na.rm = TRUE)
      g <- g + geom_point(data = station_trends_valid,
                          aes(x = lon, y = lat, color = trend),
                          size = point_size) +
        scale_color_gradient2(name = "Trend\n(Kendall tau)",
                              low = "blue", mid = "white", high = "red",
                              midpoint = 0,
                              limits = c(-max_abs, max_abs))
    }
    title <- paste0("Phenological Trends",
                    if (!is.null(phase_id)) paste0(" (Phase ", phase_id, ")") else "",
                    "\nBlue = earlier, Red = later")

  } else if (color_by == "species_cv") {
    # Check required columns
    if (!all(c("day", "species") %in% names(dt))) {
      stop("'day' and 'species' columns required for color_by = 'species_cv'",
           call. = FALSE)
    }

    # Calculate CV of mean DOY across species at each station
    # First get mean DOY per species per station
    species_means <- dt[, .(species_mean = mean(day, na.rm = TRUE)),
                        by = .(lon, lat, species)]

    # Then calculate CV across species at each station
    station_cv <- species_means[, {
      n_sp <- .N
      if (n_sp >= min_species) {
        mean_val <- mean(species_mean, na.rm = TRUE)
        sd_val <- sd(species_mean, na.rm = TRUE)
        cv <- if (!is.na(mean_val) && mean_val > 0) 100 * sd_val / mean_val else NA_real_
        list(species_cv = cv, n_species = n_sp)
      } else {
        list(species_cv = NA_real_, n_species = n_sp)
      }
    }, by = .(lon, lat)]

    station_cv_valid <- station_cv[!is.na(species_cv)]

    if (nrow(station_cv_valid) == 0) {
      warning("No stations have enough species for CV calculation (min_species = ",
              min_species, ")", call. = FALSE)
      g <- g + geom_point(data = unique(dt[, .(lon, lat)]),
                          aes(x = lon, y = lat),
                          size = point_size, color = "gray50")
    } else {
      g <- g + geom_point(data = station_cv_valid,
                          aes(x = lon, y = lat, color = species_cv),
                          size = point_size) +
        scale_color_viridis_c(name = "Species CV (%)", option = "magma")
    }
    title <- paste0("Species Variation in Timing",
                    if (!is.null(phase_id)) paste0(" (Phase ", phase_id, ")") else "",
                    "\nHigher CV = more variation among species")
  }

  g <- g + labs(title = title) +
    theme(legend.position = "right",
          plot.title = element_text(size = 11))

  # Save if requested
  if (!is.null(output_file)) {
    ggsave(output_file, plot = g, width = 10, height = 8)
  }

  return(g)
}

#' Interactive Leaflet Map to Select PEP725 Stations
#'
#' Launches a Shiny gadget that allows selection of PEP phenological stations
#' by drawing a bounding box or clicking individual points on the map.
#'
#' @param pep A data frame or data.table with columns \code{lon}, \code{lat}, and optionally \code{species}.
#' @param label_col Optional column name to show as label in popups (e.g. \code{"species"} or \code{"s_id"}).
#' @param quiet Logical. If \code{FALSE} (default), displays a message about expected loading time.
#' @return A `data.frame` with selected stations (columns: \code{lon}, \code{lat}, and metadata).
#'
#' @details
#' \strong{Note:} Loading time depends on the size of the dataset. For the full PEP725
#' dataset with millions of observations, expect 10-20 seconds for the map to fully
#' render. Consider filtering the data before calling this function to speed up loading:
#'
#' \preformatted{
#' # Filter to a specific species first
#' wheat <- pep[genus == "Triticum"]
#' selected <- pheno_leaflet(wheat)
#' }
#'
#' @examples
#' if (interactive()) {
#' pep <- pep_download()
#'
#' # For faster loading, filter the data first (recommended)
#' # Grapevine has the longest historical records
#' pep_subset <- pep[species == "Vitis vinifera"]
#'
#' if (nrow(pep_subset) > 0) {
#'   # Launch interactive selection gadget
#'   selected <- pheno_leaflet(pep_subset, label_col = "species")
#'
#'   # Inspect selected subset
#'   print(selected)
#' }
#' }
#' @author Matthias Templ
#' @export
pheno_leaflet <- function(pep, label_col = NULL, quiet = FALSE) {
  pkgs <- c("shiny", "miniUI", "leaflet", "leaflet.extras", "sp")
  missing <- pkgs[!vapply(pkgs, requireNamespace, logical(1), quietly = TRUE)]
  if (length(missing) > 0) {
    msg <- paste0("Package(s) ", paste(missing, collapse = ", "),
         " required for pheno_leaflet().")
    if ("leaflet.extras" %in% missing) {
      msg <- paste0(msg,
        "\nInstall leaflet.extras from r-universe: ",
        'install.packages("leaflet.extras", repos = "https://trafficonese.r-universe.dev")')
    }
    other <- setdiff(missing, "leaflet.extras")
    if (length(other) > 0) {
      msg <- paste0(msg, "\nInstall from CRAN: install.packages(c(",
        paste0('"', other, '"', collapse = ", "), "))")
    }
    stop(msg, call. = FALSE)
  }

  if (!all(c("lon", "lat") %in% names(pep))) stop("pep must contain 'lon' and 'lat' columns.")


  # Inform user about loading time
  n_points <- nrow(pep)
  if (!quiet) {
    message(sprintf(
      "\nLaunching interactive map with %s points...",
      format(n_points, big.mark = ",")
    ))
    if (n_points > 100000) {
      message("Note: Large dataset detected. The map may take 10-20 seconds to fully load.")
      message("Tip: Filter your data first for faster loading (e.g., pep[genus == 'Triticum'])\n")
    } else if (n_points > 10000) {
      message("Note: Map loading may take a few seconds...\n")
    }
  }

  # Minimal label
  label <- if (!is.null(label_col) && label_col %in% names(pep)) {
    pep[[label_col]]
  } else {
    paste0("Lon: ", round(pep$lon, 3), ", Lat: ", round(pep$lat, 3))
  }

  # UI with loading indicator
  ui <- miniUI::miniPage(
    miniUI::gadgetTitleBar("Select PEP Stations"),
    shiny::tags$head(
      shiny::tags$style(shiny::HTML("
        .loading-overlay {
          position: absolute;
          top: 0; left: 0; right: 0; bottom: 0;
          background: rgba(255,255,255,0.9);
          z-index: 1000;
          display: flex;
          flex-direction: column;
          justify-content: center;
          align-items: center;
          font-family: sans-serif;
        }
        .loading-spinner {
          border: 4px solid #f3f3f3;
          border-top: 4px solid #3498db;
          border-radius: 50%;
          width: 40px;
          height: 40px;
          animation: spin 1s linear infinite;
          margin-bottom: 15px;
        }
        @keyframes spin {
          0% { transform: rotate(0deg); }
          100% { transform: rotate(360deg); }
        }
        .loading-text {
          color: #666;
          font-size: 14px;
          text-align: center;
        }
        .loading-subtext {
          color: #999;
          font-size: 12px;
          margin-top: 5px;
        }
      "))
    ),
    miniUI::miniContentPanel(
      shiny::div(
        id = "loading-screen",
        class = "loading-overlay",
        shiny::div(class = "loading-spinner"),
        shiny::div(
          class = "loading-text",
          sprintf("Loading %s stations...", format(n_points, big.mark = ","))
        ),
        shiny::div(
          class = "loading-subtext",
          if (n_points > 100000) "This may take 10-20 seconds" else "Please wait..."
        )
      ),
      leaflet::leafletOutput("map", height = "100%")
    )
  )

  # Server
  server <- function(input, output, session) {
    values <- shiny::reactiveValues(
      selected_points = data.frame(),
      map_ready = FALSE
    )

    output$map <- leaflet::renderLeaflet({
      map <- leaflet::leaflet(pep) %>%
        leaflet::addTiles() %>%
        leaflet::addCircleMarkers(
          lng = ~lon, lat = ~lat,
          radius = 4, stroke = FALSE, fillOpacity = 0.7,
          label = label,
          layerId = ~paste(lon, lat)
        ) %>%
        getExportedValue("leaflet.extras", "addDrawToolbar")(
          targetGroup = "draw",
          polylineOptions = FALSE,
          polygonOptions = TRUE,
          rectangleOptions = TRUE,
          circleOptions = FALSE,
          markerOptions = FALSE,
          editOptions = getExportedValue("leaflet.extras", "editToolbarOptions")(),
          singleFeature = FALSE
        )

      # Signal that map rendering is complete
      values$map_ready <- TRUE
      map
    })

    # Hide loading screen when map is ready
    shiny::observe({
      if (values$map_ready) {
        shiny::removeUI(selector = "#loading-screen", immediate = TRUE)
      }
    })

    # Individual point click
    shiny::observeEvent(input$map_marker_click, {
      click <- input$map_marker_click
      pt <- pep[abs(pep$lon - click$lng) < 1e-6 & abs(pep$lat - click$lat) < 1e-6, ]
      values$selected_points <- unique(rbind(values$selected_points, pt))
    })

    # Polygon/rectangle selection
    shiny::observeEvent(input$map_draw_new_feature, {
      feat <- input$map_draw_new_feature
      if (feat$geometry$type %in% c("Polygon", "Rectangle")) {
        coords <- feat$geometry$coordinates[[1]]
        lng <- sapply(coords, `[[`, 1)
        lat <- sapply(coords, `[[`, 2)
        in_poly <- sp::point.in.polygon(
          point.x = pep$lon, point.y = pep$lat,
          pol.x = lng, pol.y = lat
        ) > 0
        values$selected_points <- unique(rbind(values$selected_points, pep[in_poly, ]))
      }
    })

    # Done button
    shiny::observeEvent(input$done, {
      shiny::stopApp(values$selected_points)
    })

    # Cancel button
    shiny::observeEvent(input$cancel, {
      shiny::stopApp(NULL)
    })
  }

  # Run gadget
  shiny::runGadget(ui, server, viewer = shiny::dialogViewer("Select Stations", width = 900, height = 700))
}

#' Calculate Phenological Normals (Climatology)
#'
#' Computes reference "normal" phenology for a specified period, analogous to
#' WMO climate normals. Returns central tendency, spread, and percentile
#' statistics for phenological day-of-year (DOY) values.
#'
#' @param pep A \code{pep} object or \code{data.table} containing phenological
#'   observations with columns \code{year}, \code{day}, and grouping variables.
#' @param period Integer vector specifying the years to include in the normal
#'   calculation. Default is \code{1991:2020} (current WMO standard period).
#' @param by Character vector of column names to group by. Common choices:
#'   \itemize{
#'     \item \code{c("country", "genus", "phase_id")} - Regional normals by genus/phase
#'     \item \code{c("s_id", "species", "phase_id")} - Station-level normals
#'     \item \code{c("genus", "phase_id")} - Overall normals ignoring geography
#'   }
#'   Default is \code{c("country", "genus", "phase_id")}.
#' @param species Optional character string to filter by species column.
#'   Can be genus name (e.g., "Triticum") or full species (e.g., "Triticum aestivum").
#'   If \code{NULL} (default), all species in the data are included.
#' @param phase_id Optional integer vector to filter by BBCH phase codes.
#'   If \code{NULL} (default), all phases in the data are included.
#' @param min_years Minimum number of years required to calculate valid normals.
#'   Default is 20 (WMO standard). Groups with fewer years return \code{NA}.
#' @param probs Numeric vector of probabilities for percentile calculation.
#'   Default is \code{c(0.05, 0.10, 0.25, 0.75, 0.90, 0.95)}.
#' @param na.rm Logical. Should missing values be removed? Default \code{TRUE}.
#'
#' @return A \code{data.table} with the following columns:
#'   \describe{
#'     \item{by variables}{Grouping variables as specified in \code{by}}
#'     \item{n_years}{Number of years with data in the period}
#'     \item{n_obs}{Total number of observations}
#'     \item{mean_doy}{Arithmetic mean DOY}
#'     \item{median_doy}{Median DOY (more robust to outliers)}
#'     \item{sd_doy}{Standard deviation}
#'     \item{iqr_doy}{Interquartile range}
#'     \item{mad_doy}{Median absolute deviation (robust spread)}
#'     \item{q05, q10, q25, q75, q90, q95}{Percentiles with the default
#'       \code{probs}. Custom \code{probs} produce column names derived from
#'       the actual levels — integer percents pad to two digits
#'       (e.g., \code{q05}, \code{q10}), fractional percents use an
#'       underscore in place of the decimal point (e.g., \code{q02_5} for
#'       the 2.5\% quantile, \code{q97_5} for the 97.5\% quantile).
#'       The mapping is also stored in \code{attr(result, "q_names")}.}
#'     \item{period}{Character string describing the reference period}
#'   }
#'
#' @details
#' Phenological normals provide a baseline for comparing individual years or
#' detecting trends. This function calculates both classical statistics (mean, SD)
#' and robust alternatives (median, MAD, IQR) that are less sensitive to outliers.
#'
#' The function can be used in two ways:
#' \enumerate{
#'   \item \strong{Full dataset}: Pass the complete \code{pep} object and use
#'     \code{species} and \code{phase_id} parameters to filter internally.
#'   \item \strong{Pre-filtered subset}: Filter the data first using data.table
#'     syntax, then pass the subset to the function.
#' }
#'
#' @section Standard Reference Periods:
#' \itemize{
#'   \item \strong{1961-1990}: Historical reference (pre-acceleration of warming)
#'   \item \strong{1991-2020}: Current WMO standard normal period
#' }
#'
#' @examples
#' \donttest{
#' # Download synthetic data first
#' pep <- pep_download()
#'
#' # Use Swiss subset for faster computation
#' pep_ch <- pep[country == "Switzerland"]
#'
#' # Calculate normals for all species and phases
#' normals_all <- pheno_normals(pep_ch)
#'
#' # Normals for apples, flowering phase (60)
#' apple_normals <- pheno_normals(pep_ch,
#'                                species = "Malus",
#'                                phase_id = 60)
#'
#' # Using a pre-filtered subset
#' apples <- pep_ch[genus == "Malus" & phase_id %in% c(60, 65)]
#' apple_normals <- pheno_normals(apples,
#'                                by = c("species", "phase_id"))
#'
#' # Station-level normals for detailed analysis
#' station_normals <- pheno_normals(pep_ch,
#'                                  species = "Malus",
#'                                  phase_id = 60,
#'                                  by = c("s_id"))
#'
#' # Historical reference period
#' historical <- pheno_normals(pep_ch,
#'                             period = 1961:1990,
#'                             species = "Malus")
#'
#' # Compare two periods
#' period1 <- pheno_normals(pep_ch, period = 1961:1990, species = "Malus")
#' period2 <- pheno_normals(pep_ch, period = 1991:2020, species = "Malus")
#' shift <- period2$mean_doy - period1$mean_doy
#' }
#'
#' @references
#' WMO (2017). \emph{WMO Guidelines on the Calculation of Climate Normals}.
#' WMO-No. 1203, World Meteorological Organization, Geneva.
#'
#' Templ, M., Templ, B., and Filzmoser, P. (2018).
#' Modelling and prediction of phenological data of the beginning of
#' flowering: Austrian indicators of climate change.
#' \emph{International Journal of Biometeorology}, 62, 1319--1334.
#' \doi{10.1007/s00484-018-1534-2}
#'
#' @seealso
#' \code{\link{pheno_anomaly}} for calculating anomalies relative to normals,
#' \code{\link{pep_download}} for obtaining the main dataset
#'
#' @author Matthias Templ
#' @export
pheno_normals <- function(pep,
                          period = 1991:2020,
                          by = c("country", "genus", "phase_id"),
                          species = NULL,
                          phase_id = NULL,
                          min_years = 20,
                          probs = c(0.05, 0.10, 0.25, 0.75, 0.90, 0.95),
                          na.rm = TRUE) {

 # Input validation
 if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table if needed
  if (!inherits(pep, "data.table")) {
    pep <- as.data.table(pep)
  }

  # Check required columns
  required_cols <- c("year", "day")
  missing_cols <- setdiff(required_cols, names(pep))
  if (length(missing_cols) > 0) {
    stop("Missing required columns: ", paste(missing_cols, collapse = ", "),
         call. = FALSE)
  }

  # Normalize by = character(0) to NULL
  if (is.character(by) && length(by) == 0) {
    by <- NULL
  }

  # Check that 'by' columns exist
  if (!is.null(by)) {
    missing_by <- setdiff(by, names(pep))
    if (length(missing_by) > 0) {
      stop("Column(s) specified in 'by' not found: ",
           paste(missing_by, collapse = ", "),
           call. = FALSE)
    }
  }

  # Validate period
  if (!is.numeric(period) || length(period) < 2) {
    stop("'period' must be a numeric vector with at least 2 years", call. = FALSE)
  }

  # Validate probs
  if (!is.numeric(probs) || any(probs < 0) || any(probs > 1)) {
    stop("'probs' must be numeric values between 0 and 1", call. = FALSE)
  }
  if (length(probs) < 1) {
    stop("'probs' must contain at least one value", call. = FALSE)
  }
  # Derive quantile column names from levels. Integer percents pad to two
  # digits (q05, q10, q25, q90); fractional percents use '_' for the decimal
  # point to stay syntactically valid (q02_5, q97_5). Defaults of
  # c(0.05, 0.10, 0.25, 0.75, 0.90, 0.95) continue to produce q05..q95.
  .qname <- function(p) {
    pct <- p * 100
    if (isTRUE(all.equal(pct, round(pct)))) {
      sprintf("q%02d", as.integer(round(pct)))
    } else {
      # Fractional percent: "q" + zero-padded integer part + "_" + fractional
      # part (e.g. 2.5 -> "q02_5", 97.5 -> "q97_5").
      int_part <- floor(pct)
      frac_str <- sub("^0\\.", "",
                      formatC(pct - int_part, format = "f", digits = 4))
      frac_str <- sub("0+$", "", frac_str)
      if (nchar(frac_str) == 0) frac_str <- "0"
      sprintf("q%02d_%s", as.integer(int_part), frac_str)
    }
  }
  q_names <- vapply(probs, .qname, character(1))
  if (anyDuplicated(q_names)) {
    stop("'probs' generated duplicate column names: ",
         paste(q_names[duplicated(q_names)], collapse = ", "), call. = FALSE)
  }

 # Make a copy to avoid modifying original
  dt <- copy(pep)

  # Apply species filter if specified
  if (!is.null(species)) {
    species_filter <- species  # Avoid name collision with column

    # Check which column to filter on (genus or species)
    if ("genus" %in% names(dt)) {
      # Try matching against genus first, then species
      # Convert to character to handle factors
      genus_match <- as.character(dt$genus) == species_filter
      species_match <- if ("species" %in% names(dt)) {
        as.character(dt$species) == species_filter
      } else {
        FALSE
      }

      dt <- dt[genus_match | species_match]

      if (nrow(dt) == 0) {
        stop("No observations found for species '", species_filter, "'", call. = FALSE)
      }
    } else if ("species" %in% names(dt)) {
      dt <- dt[as.character(dt$species) == species_filter]
      if (nrow(dt) == 0) {
        stop("No observations found for species '", species_filter, "'", call. = FALSE)
      }
    } else {
      warning("Neither 'genus' nor 'species' column found. Ignoring species filter.",
              call. = FALSE)
    }
  }

  # Apply phase_id filter if specified
  if (!is.null(phase_id)) {
    if (!"phase_id" %in% names(dt)) {
      warning("'phase_id' column not found. Ignoring phase_id filter.", call. = FALSE)
    } else {
      phase_filter <- phase_id  # Avoid name collision with column
      dt <- dt[dt$phase_id %in% phase_filter]
      if (nrow(dt) == 0) {
        stop("No observations found for phase_id: ",
             paste(phase_id, collapse = ", "), call. = FALSE)
      }
    }
  }

  # Filter to specified period
  dt <- dt[year %in% period]

  if (nrow(dt) == 0) {
    stop("No observations found in the specified period: ",
         min(period), "-", max(period), call. = FALSE)
  }

  # Create period label
  period_label <- paste0(min(period), "-", max(period))

  # Calculate normals by group
  # First aggregate to annual means per group to avoid double-counting
  # multiple observations in the same year
  annual_means <- dt[, .(
    annual_mean_doy = mean(day, na.rm = na.rm)
  ), by = c(by, "year")]

  # Now calculate statistics across years
  result <- annual_means[, {
    doy_values <- annual_mean_doy
    n_yrs <- sum(!is.na(doy_values))

    if (n_yrs >= min_years) {
      quantiles <- stats::quantile(doy_values, probs = probs, na.rm = na.rm)
      q_cols <- stats::setNames(as.list(quantiles), q_names)
      c(
        list(
          n_years = n_yrs,
          n_obs = .N,
          mean_doy = mean(doy_values, na.rm = na.rm),
          median_doy = stats::median(doy_values, na.rm = na.rm),
          sd_doy = stats::sd(doy_values, na.rm = na.rm),
          iqr_doy = stats::IQR(doy_values, na.rm = na.rm),
          mad_doy = stats::mad(doy_values, na.rm = na.rm)
        ),
        q_cols,
        list(period = period_label)
      )
    } else {
      q_cols <- stats::setNames(
        as.list(rep(NA_real_, length(q_names))),
        q_names
      )
      c(
        list(
          n_years = n_yrs,
          n_obs = .N,
          mean_doy = NA_real_,
          median_doy = NA_real_,
          sd_doy = NA_real_,
          iqr_doy = NA_real_,
          mad_doy = NA_real_
        ),
        q_cols,
        list(period = period_label)
      )
    }
  }, by = by]

  # Add informative message about groups with insufficient data
  n_insufficient <- sum(result$n_years < min_years)
  if (n_insufficient > 0) {
    message(sprintf(
      "Note: %d group(s) have fewer than %d years of data and return NA values.",
      n_insufficient, min_years
    ))
  }

  # Order by grouping variables
  setorderv(result, by)

  # Add class for potential method dispatch
  class(result) <- c("pheno_normals", class(result))

  # Store parameters as attributes
  attr(result, "period") <- period
  attr(result, "min_years") <- min_years
  attr(result, "species_filter") <- species
  attr(result, "phase_filter") <- phase_id
  attr(result, "probs") <- probs
  attr(result, "q_names") <- q_names

  result
}


#' Print Method for Phenological Normals
#'
#' @param x A \code{pheno_normals} object
#' @param n Number of rows to display
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns \code{x}
#' @author Matthias Templ
#' @export
print.pheno_normals <- function(x, n = 10, ...) {
  period <- attr(x, "period")
  min_years <- attr(x, "min_years")
  species_filter <- attr(x, "species_filter")
  phase_filter <- attr(x, "phase_filter")

  cat("Phenological Normals\n")
  cat(strrep("-", 50), "\n")

  # Handle period safely (may be NULL or empty)
  if (!is.null(period) && length(period) > 0) {
    cat(sprintf("Reference period: %d-%d\n", as.integer(min(period)), as.integer(max(period))))
  }

  if (!is.null(min_years)) {
    cat(sprintf("Minimum years required: %d\n", as.integer(min_years)))
  }

  if (!is.null(species_filter)) {
    cat(sprintf("Species filter: %s\n", species_filter))
  }
  if (!is.null(phase_filter)) {
    cat(sprintf("Phase filter: %s\n", paste(phase_filter, collapse = ", ")))
  }

  cat(sprintf("Groups with valid normals: %d / %d\n",
              sum(!is.na(x$mean_doy)), nrow(x)))
  cat(strrep("-", 50), "\n\n")

  # Convert to data.table for printing

  dt <- as.data.table(x)

  # Identify grouping columns (non-statistic columns)
  q_cols_attr <- attr(x, "q_names")
  if (is.null(q_cols_attr)) {
    # Legacy objects without q_names attribute: fall back to default names.
    q_cols_attr <- c("q05", "q10", "q25", "q75", "q90", "q95")
  }
  stat_cols <- c("n_years", "n_obs", "mean_doy", "median_doy", "sd_doy",
                 "iqr_doy", "mad_doy", q_cols_attr, "period")
  group_cols <- setdiff(names(dt), stat_cols)

  # Sort so that rows with NA in grouping columns appear last
  if (length(group_cols) > 0) {
    # Create indicator: TRUE if any grouping column has NA
    dt[, .has_na := Reduce(`|`, lapply(.SD, is.na)), .SDcols = group_cols]
    setorderv(dt, c(".has_na", group_cols), na.last = TRUE)
    dt[, .has_na := NULL]
  }

  # Print first n rows
  print(dt[1:min(n, nrow(dt))])

  if (nrow(x) > n) {
    cat(sprintf("\n... and %d more rows\n", nrow(x) - n))
  }

  invisible(x)
}


#' Summary Method for Phenological Normals
#'
#' @param object A \code{pheno_normals} object
#' @param ... Additional arguments (unused)
#'
#' @return Invisibly returns a summary list
#' @author Matthias Templ
#' @export
summary.pheno_normals <- function(object, ...) {
  period <- attr(object, "period")

  cat("Phenological Normals Summary\n")
  cat(strrep("=", 50), "\n\n")

  cat(sprintf("Reference period: %d-%d\n", min(period), max(period)))
  cat(sprintf("Total groups: %d\n", nrow(object)))
  cat(sprintf("Groups with valid normals: %d (%.1f%%)\n",
              sum(!is.na(object$mean_doy)),
              100 * sum(!is.na(object$mean_doy)) / nrow(object)))
  cat("\n")

  # Summary of DOY statistics (for valid normals only)
  valid <- object[!is.na(mean_doy)]

  if (nrow(valid) > 0) {
    cat("DOY Statistics (across all groups with valid normals):\n")
    cat(sprintf("  Mean DOY range: %.0f - %.0f\n",
                min(valid$mean_doy), max(valid$mean_doy)))
    cat(sprintf("  Median DOY range: %.0f - %.0f\n",
                min(valid$median_doy), max(valid$median_doy)))
    cat(sprintf("  Typical SD: %.1f days\n",
                median(valid$sd_doy, na.rm = TRUE)))
    cat(sprintf("  Typical IQR: %.1f days\n",
                median(valid$iqr_doy, na.rm = TRUE)))
  }

  invisible(list(
    n_groups = nrow(object),
    n_valid = sum(!is.na(object$mean_doy)),
    period = period
  ))
}


#' Plot Method for Phenological Normals
#'
#' Visualise phenological normals as a dot plot with interquartile-range bars
#' or as a bar chart.
#'
#' @param x A \code{pheno_normals} object returned by \code{\link{pheno_normals}}.
#' @param which Character string selecting the plot type: \code{"dotplot"}
#'   (default) shows \code{median_doy} with Q25--Q75 range via
#'   \code{geom_pointrange}; \code{"bar"} shows \code{median_doy} as columns
#'   with error bars.
#' @param ... Additional arguments (unused).
#'
#' @return A \code{ggplot} object (returned invisibly).
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' pep_ch <- pep[country == "Switzerland"]
#'
#' # Normals by phase for apple
#' n <- pheno_normals(pep_ch, species = "Malus", by = "phase_id")
#' plot(n)
#' plot(n, which = "bar")
#' }
#'
#' @seealso \code{\link{pheno_normals}}, \code{\link{pheno_anomaly}}
#' @author Matthias Templ
#' @export
plot.pheno_normals <- function(x, which = c("dotplot", "bar"), ...) {
  which <- match.arg(which)

  # Remove rows with NA normals
  plot_data <- data.table::as.data.table(x)[!is.na(median_doy)]

  if (nrow(plot_data) == 0) {
    stop("No valid normals to plot", call. = FALSE)
  }

  # Identify grouping columns
  q_cols_attr <- attr(x, "q_names")
  if (is.null(q_cols_attr)) {
    # Legacy objects without q_names attribute: fall back to default names.
    q_cols_attr <- c("q05", "q10", "q25", "q75", "q90", "q95")
  }
  stat_cols <- c("n_years", "n_obs", "mean_doy", "median_doy", "sd_doy",
                 "iqr_doy", "mad_doy", q_cols_attr, "period")
  group_cols <- setdiff(names(plot_data), stat_cols)

  # Build human-readable group label
  if (length(group_cols) > 0) {
    for (gc in group_cols) {
      if (gc == "phase_id") {
        descs <- .bbch_lookup[as.character(plot_data[[gc]])]
        descs[is.na(descs)] <- as.character(plot_data[[gc]][is.na(descs)])
        plot_data[, (gc) := paste0(plot_data[[gc]], " - ", descs)]
      }
    }
    plot_data[, group_label := do.call(paste, c(.SD, sep = " | ")),
              .SDcols = group_cols]
  } else {
    plot_data[, group_label := "All"]
  }

  # Subtitle from attributes
  period <- attr(x, "period")
  subtitle <- if (!is.null(period) && length(period) > 0) {
    sprintf("Reference period: %d-%d  |  n groups: %d",
            min(period), max(period), nrow(plot_data))
  } else {
    sprintf("n groups: %d", nrow(plot_data))
  }

  # Pick a lower/upper quantile pair for the IQR-style bar. Default probs
  # include 0.25/0.75; custom probs may not. Fall back to the widest
  # available pair that spans the median.
  probs_used <- attr(x, "probs")
  if (is.null(probs_used)) probs_used <- c(0.05, 0.10, 0.25, 0.75, 0.90, 0.95)
  lower_idx <- which(probs_used >= 0.20 & probs_used <= 0.30)
  upper_idx <- which(probs_used >= 0.70 & probs_used <= 0.80)
  if (length(lower_idx) == 0) lower_idx <- which.max(probs_used[probs_used < 0.5])
  if (length(upper_idx) == 0) upper_idx <- length(probs_used) -
    which.min(rev(probs_used)[rev(probs_used) > 0.5]) + 1
  # Resolve to actual column names in the result.
  q_lo <- q_cols_attr[lower_idx[1]]
  q_hi <- q_cols_attr[upper_idx[1]]
  has_band <- !is.null(q_lo) && !is.null(q_hi) &&
    !is.na(q_lo) && !is.na(q_hi) &&
    q_lo %in% names(plot_data) && q_hi %in% names(plot_data)

  if (which == "dotplot") {
    p <- ggplot2::ggplot(plot_data,
                         ggplot2::aes(x = median_doy,
                                      y = stats::reorder(group_label, median_doy)))
    if (has_band) {
      p <- p + ggplot2::geom_pointrange(
        ggplot2::aes(xmin = .data[[q_lo]], xmax = .data[[q_hi]]),
        color = "steelblue"
      )
    } else {
      p <- p + ggplot2::geom_point(color = "steelblue", size = 2)
    }
    p <- p +
      ggplot2::labs(x = "Day of Year", y = NULL,
                    title = "Phenological Normals",
                    subtitle = subtitle) +
      ggplot2::theme_minimal()
  } else {
    p <- ggplot2::ggplot(plot_data,
                         ggplot2::aes(x = stats::reorder(group_label, median_doy),
                                      y = median_doy)) +
      ggplot2::geom_col(fill = "steelblue", width = 0.7)
    if (has_band) {
      p <- p + ggplot2::geom_errorbar(
        ggplot2::aes(ymin = .data[[q_lo]], ymax = .data[[q_hi]]),
        width = 0.25
      )
    }
    p <- p +
      ggplot2::labs(x = NULL, y = "Day of Year",
                    title = "Phenological Normals",
                    subtitle = subtitle) +
      ggplot2::coord_flip() +
      ggplot2::theme_minimal()
  }

  p
}

utils::globalVariables(c("med_day", "i.med_day"))

#' Simulate Synthetic PEP Data
#'
#' Generates synthetic phenological observations by fitting a GAM smooth
#' (\code{day ~ s(year)}) per station-species-phase group and replacing the
#' observed \code{day} values with predictions plus Gaussian noise.  The
#' station-year-species-phase structure of the input data is kept intact.
#'
#' @param pep A \code{pep} object or \code{data.table} with at least the
#'   columns \code{species}, \code{phase_id}, \code{s_id}, \code{year}, and
#'   \code{day}.
#' @param min_obs Integer. Minimum number of observations required per
#'   station-species-phase group to attempt GAM fitting.
#'   Groups with fewer observations keep their original \code{day} values
#'   unchanged. Default is 20.
#' @param seed Integer. Random seed for reproducibility. Default is 42.
#' @param progress Logical. Show a progress bar? Default \code{TRUE}.
#'
#' @return A \code{\link[=new_pep]{pep}} object with the same rows and columns
#'   as the input.  The \code{day} column is overwritten with synthetic values
#'   for groups that meet the \code{min_obs} threshold; other rows retain their
#'   original values.
#'
#' @details
#' For each qualifying group (station x species x phase with at least
#' \code{min_obs} observations), a GAM is fitted and synthetic day-of-year
#' values are generated as \code{round(predicted + rnorm(n, sd = residual_sd))}.
#'
#' Groups where the GAM fit fails receive a fallback: the species-phase median
#' day plus Gaussian jitter (sd = 5 days).
#'
#' Groups below the \code{min_obs} threshold are left unchanged.  A message
#' reports how many rows were not simulated.
#'
#' @author Matthias Templ
#' @export
#' @examples
#' \donttest{
#' pep <- pep_download()
#' # Single species + country for fast example
#' vine_ch <- pep[species == "Vitis vinifera" & country == "Switzerland"]
#' pep_synth <- pep_simulate(vine_ch)
#' }
pep_simulate <- function(pep, min_obs = 20, seed = 42, progress = TRUE) {

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  required_cols <- c("species", "phase_id", "s_id", "year", "day")
  missing_cols <- setdiff(required_cols, names(pep))
  if (length(missing_cols) > 0) {
    stop("Missing required columns: ", paste(missing_cols, collapse = ", "),
         call. = FALSE)
  }

  set.seed(seed)
  pep <- copy(pep)
  setDT(pep)

  # Grouping
  groups <- pep[, .N, by = .(species, phase_id, s_id)][N >= min_obs]
  n_groups <- nrow(groups)

  if (n_groups == 0) {
    warning("No groups have >= ", min_obs,
            " observations. Returning data unchanged.", call. = FALSE)
    return(new_pep(pep))
  }

  # Track how many rows are in qualifying groups
  n_total <- nrow(pep)

  # Init progress bar and time
  if (progress) {
    pb <- txtProgressBar(min = 0, max = n_groups, style = 3)
    start_time <- Sys.time()
  }

  # Loop over groups
  for (i in seq_len(n_groups)) {
    group <- groups[i]
    sub <- pep[species == group$species & phase_id == group$phase_id & s_id == group$s_id]

    fit <- try(mgcv::gam(day ~ s(year), data = sub), silent = TRUE)
    if (!inherits(fit, "try-error")) {
      pred <- predict(fit, newdata = sub)
      res_sd <- sd(residuals(fit))
      pep[species == group$species & phase_id == group$phase_id & s_id == group$s_id,
          day := round(pred + rnorm(.N, sd = res_sd))]
    }

    # Progress update
    if (progress) {
      setTxtProgressBar(pb, i)

      if (i %% 500 == 0 || i == n_groups) {
        elapsed <- as.numeric(difftime(Sys.time(), start_time, units = "secs"))
        avg_time <- elapsed / i
        remaining <- round(avg_time * (n_groups - i))
        message(sprintf(" | Est. time left: %ds", remaining))
      }
    }
  }

  if (progress) close(pb)

  # Fallback: replace NAs (rare/failed groups) with species-phase median + jitter
  medians <- pep[!is.na(day), .(med_day = median(day)), by = .(species, phase_id)]
  pep[medians, on = .(species, phase_id), med_day := i.med_day]
  pep[is.na(day) & !is.na(med_day), day := round(med_day + rnorm(.N, mean = 0, sd = 5))]
  pep[, med_day := NULL]

  # Report coverage
  all_groups <- pep[, .N, by = .(species, phase_id, s_id)]
  n_small <- sum(all_groups$N < min_obs)
  if (n_small > 0) {
    n_unchanged <- pep[all_groups[N < min_obs], on = .(species, phase_id, s_id), .N]
    message(sprintf(
      "Note: %d group(s) had fewer than %d observations. %d row(s) (%.1f%%) retain original day values.",
      n_small, min_obs, n_unchanged, 100 * n_unchanged / n_total
    ))
  }

  new_pep(pep)
}

#' Demonstrate the pep725 Package Functions
#'
#' This function provides an interactive demonstration of the main features
#' of the pep725 package for PEP725 phenological data analysis. It showcases
#' data exploration, filtering, summarization, and visualization capabilities.
#'
#' @param which Character vector specifying which demos to run. Options are:
#'   \itemize{
#'     \item \code{"class"} - Demonstrate the pep class (print, summary)
#'     \item \code{"filter"} - Show data filtering and selection
#'     \item \code{"plot"} - Display visualization capabilities
#'     \item \code{"analysis"} - Show analysis functions (normals, anomalies)
#'     \item \code{"all"} - Run all demonstrations (default)
#'   }
#' @param pause Logical. If \code{TRUE} (default), pauses between demos
#'   waiting for user input. Set to \code{FALSE} for non-interactive use.
#'
#' @return Invisibly returns a list containing example outputs from each demo.
#'
#' @details
#' The demonstration downloads synthetic PEP725 data using \code{pep_download()}
#' and walks through typical workflows for phenological data analysis:
#'
#' \enumerate{
#'   \item \strong{Class demo}: Shows the enhanced print and summary methods
#'     for the pep class, demonstrating how to quickly understand the data.
#'   \item \strong{Filter demo}: Demonstrates selecting specific species,
#'     phases, and time periods using data.table syntax.
#'   \item \strong{Plot demo}: Creates visualizations including station maps,
#'     time series, and DOY distributions.
#'   \item \strong{Analysis demo}: Shows phenological normals calculation,
#'     anomaly detection, and data quality assessment.
#' }
#'
#' @examples
#' if (interactive()) {
#' # Run all demos interactively
#' pep725_demo()
#'
#' # Run specific demos
#' pep725_demo(which = "class")
#' pep725_demo(which = c("filter", "plot"))
#'
#' # Run without pausing (for scripts)
#' pep725_demo(pause = FALSE)
#' }
#'
#' @seealso
#' \code{\link{pep_download}} for downloading synthetic data,
#' \code{\link{pheno_normals}} for calculating normals,
#' \code{\link{pheno_anomaly}} for anomaly detection,
#' \code{\link{pep_quality}} for quality assessment
#'
#' @author Matthias Templ
#' @export
pep725_demo <- function(which = "all", pause = interactive()) {

  # Validate input
  valid_demos <- c("all", "class", "filter", "plot", "analysis")
  if (!all(which %in% valid_demos)) {
    stop("Invalid demo selection. Choose from: ",
         paste(valid_demos, collapse = ", "))
  }

  if ("all" %in% which) {
    which <- c("class", "filter", "plot", "analysis")
  }

  results <- list()

  # Helper function for pausing
  wait <- function(msg = "Press [Enter] to continue...") {
    if (pause) {
      readline(prompt = msg)
    }
  }

  # Header
  cat("\n")
  cat(strrep("=", 60), "\n")
  cat("       PEP725 PACKAGE DEMONSTRATION\n")
  cat("       Pan-European Phenological Data Analysis\n")
  cat(strrep("=", 60), "\n\n")

  # Download synthetic data
  cat("Downloading synthetic PEP725 data...\n\n")
  pep <- pep_download()

  cat("Data loaded successfully!\n")
  cat(sprintf("  - %s observations\n", format(nrow(pep), big.mark = ",")))
  cat(sprintf("  - %d stations\n", length(unique(pep$s_id))))
  cat(sprintf("  - %d species\n", length(unique(pep$species))))
  cat(sprintf("  - Years: %d - %d\n\n", min(pep$year), max(pep$year)))

  wait("Press [Enter] to start the demonstration...")

  # =========================================================================
  # DEMO 1: Class features
  # =========================================================================
  if ("class" %in% which) {
    cat("\n")
    cat(strrep("-", 60), "\n")
    cat("DEMO 1: The pep Class - Smart Data Display\n")
    cat(strrep("-", 60), "\n\n")

    cat("The pep object has enhanced print/summary methods.\n")
    cat("Instead of dumping millions of rows, you get a useful overview:\n\n")

    cat(">>> print(pep)\n\n")
    print(pep)

    wait("\nPress [Enter] to see summary by species...")

    cat("\n>>> summary(pep, by = 'species', top = 5)\n\n")
    s1 <- summary(pep, by = "species", top = 5)

    wait("\nPress [Enter] to see summary by phase...")

    cat("\n>>> summary(pep, by = 'phase')\n\n")
    s2 <- summary(pep, by = "phase")

    wait("\nPress [Enter] to see data coverage...")

    cat("\n>>> pep_coverage(pep)\n\n")
    cov <- pep_coverage(pep)

    results$class <- list(summary_species = s1, summary_phase = s2, coverage = cov)

    wait("\nPress [Enter] to continue to filtering demo...")
  }

  # =========================================================================
  # DEMO 2: Data filtering
  # =========================================================================
  if ("filter" %in% which) {
    cat("\n")
    cat(strrep("-", 60), "\n")
    cat("DEMO 2: Data Filtering with data.table syntax\n")
    cat(strrep("-", 60), "\n\n")

    cat("The pep class inherits from data.table, so you can use\n")
    cat("all data.table operations directly:\n\n")

    cat(">>> # Filter wheat observations from 1990 onwards\n")
    cat(">>> wheat <- pep[species == 'Triticum aestivum' & year >= 1990]\n")
    cat(">>> wheat\n\n")

    wheat <- pep[species == "Triticum aestivum" & year >= 1990]
    print(wheat)

    wait("\nPress [Enter] to see subsetting preserves class...")

    cat("\n>>> # Check that class is preserved after subsetting\n")
    cat(">>> class(wheat)\n")
    cat(paste(class(wheat), collapse = ", "), "\n")

    cat("\n>>> # Aggregate by year and phase\n")
    cat(">>> wheat_agg <- wheat[, .(\n")
    cat(">>>   n_obs = .N,\n")
    cat(">>>   mean_doy = mean(day, na.rm = TRUE),\n")
    cat(">>>   sd_doy = sd(day, na.rm = TRUE)\n")
    cat(">>> ), by = .(year, phase_id)]\n\n")

    wheat_agg <- wheat[, .(
      n_obs = .N,
      mean_doy = mean(day, na.rm = TRUE),
      sd_doy = sd(day, na.rm = TRUE)
    ), by = .(year, phase_id)]

    cat("Result:\n")
    print(head(wheat_agg[order(year, phase_id)], 15))

    cat("\n>>> # Look up phase descriptions\n")
    cat(">>> bbch_description(c(60, 65, 100))\n")
    print(bbch_description(c(60, 65, 100)))

    results$filter <- list(wheat = wheat, wheat_agg = wheat_agg)

    wait("\nPress [Enter] to continue to plotting demo...")
  }

  # =========================================================================
  # DEMO 3: Visualization
  # =========================================================================
  if ("plot" %in% which) {
    cat("\n")
    cat(strrep("-", 60), "\n")
    cat("DEMO 3: Visualization with plot.pep()\n")
    cat(strrep("-", 60), "\n\n")

    cat("The pep class has built-in plot methods:\n\n")

    cat(">>> # Station map\n")
    cat(">>> plot(pep, type = 'map')\n\n")

    p1 <- plot(pep, type = "map")

    wait("Press [Enter] to see DOY histogram...")

    cat("\n>>> # DOY distribution\n")
    cat(">>> plot(pep, type = 'histogram')\n\n")

    p2 <- plot(pep, type = "histogram")

    wait("Press [Enter] to see time series...")

    cat("\n>>> # Time series of mean DOY\n")
    cat(">>> plot(pep, type = 'timeseries')\n\n")

    p3 <- plot(pep, type = "timeseries")

    results$plot <- list(map = p1, histogram = p2, timeseries = p3)

    wait("\nPress [Enter] to continue to analysis demo...")
  }

  # =========================================================================
  # DEMO 4: Analysis
  # =========================================================================
  if ("analysis" %in% which) {
    cat("\n")
    cat(strrep("-", 60), "\n")
    cat("DEMO 4: Phenological Analysis Functions\n")
    cat(strrep("-", 60), "\n\n")

    # Get wheat data for analysis
    wheat <- pep[species == "Triticum aestivum"]

    cat(">>> # Calculate phenological normals for wheat\n")
    cat(">>> normals <- pheno_normals(\n")
    cat(">>>   pep = wheat,\n")
    cat(">>>   period = 1990:2015,\n")
    cat(">>>   by = c('country', 'phase_id'),\n")
    cat(">>>   min_years = 10\n")
    cat(">>> )\n\n")

    normals <- pheno_normals(
      pep = wheat,
      period = 1990:2015,
      by = c("country", "phase_id"),
      min_years = 10
    )

    cat("Phenological normals:\n")
    print(normals)

    wait("\nPress [Enter] to see anomaly detection...")

    cat("\n>>> # Detect phenological anomalies\n")
    cat(">>> anomalies <- pheno_anomaly(\n")
    cat(">>>   pep = wheat,\n")
    cat(">>>   baseline_period = 1990:2010,\n")
    cat(">>>   by = c('country', 'phase_id'),\n")
    cat(">>>   min_years = 5\n")
    cat(">>> )\n\n")

    anomalies <- pheno_anomaly(
      pep = wheat,
      baseline_period = 1990:2010,
      by = c("country", "phase_id"),
      min_years = 5
    )

    cat("Anomaly detection results:\n")
    print(anomalies)

    wait("\nPress [Enter] to see quality assessment...")

    cat("\n>>> # Assess data quality\n")
    cat(">>> quality <- pep_quality(\n")
    cat(">>>   pep = wheat,\n")
    cat(">>>   by = c('s_id', 'phase_id')\n")
    cat(">>> )\n\n")

    quality <- pep_quality(
      pep = wheat,
      by = c("s_id", "phase_id")
    )

    cat("Quality assessment:\n")
    print(quality)

    cat("\n>>> summary(quality)\n\n")
    summary(quality)

    results$analysis <- list(
      normals = normals,
      anomalies = anomalies,
      quality = quality
    )

    cat("\n")
  }

  # =========================================================================
  # Summary
  # =========================================================================
  cat(strrep("=", 60), "\n")
  cat("DEMO COMPLETE\n")
  cat(strrep("=", 60), "\n\n")

  cat("Key functions demonstrated:\n")
  cat("
* pep_download()     - Download synthetic PEP725 data
* print(pep)         - Smart data overview
* summary(pep)       - Detailed summaries by species/phase/country
* pep_coverage(pep)  - Assess data coverage
* plot(pep)          - Quick visualizations (map, histogram, timeseries)
* pheno_normals()    - Calculate climatological baselines
* pheno_anomaly()    - Detect deviations from normal
* pep_quality()      - Assess data quality
* bbch_description() - Look up BBCH phase codes
")

  cat("\nFor more information, see:\n")
  cat("* help(package = 'pep725')\n")
  cat("* vignette('getting-started', package = 'pep725')\n")
  cat("* vignette('phenological-analysis', package = 'pep725')\n")
  cat("* vignette('spatial-patterns', package = 'pep725')\n\n")

  invisible(results)
}

#' Import and preprocess PEP725 phenological data
#'
#' This function imports all CSV files from the specified folder,
#' reads them efficiently using \pkg{data.table}, and combines them into a single
#' data table. It performs several preprocessing steps, including:
#' \itemize{
#'   \item Converting selected columns to factors (\code{provider_id}, \code{s_id}, \code{gss_id}, \code{genus}, \code{species}, \code{subspecies}).
#'   \item Replacing missing altitude values coded as \code{-9999} with \code{NA}.
#'   \item Creating a combined altitude variable \code{alt2} using \code{alt} and \code{alt_dem}.
#'   \item Converting the \code{date} column to \code{Date} class.
#'   \item Recoding cultivation season (\code{cult_season}) and quality control flags (\code{qc_ori_flag}) into labeled factors.
#'   \item Removing unused or problematic columns (\code{affected_flag}, \code{qc_flag}).
#' }
#'
#' This function is intended for importing raw PEP725 station data into a standardized
#' format suitable for further phenological analysis, such as sensitivity and trend estimation.
#'
#' @param path path to the folder containing PEP725 CSV files (default is \code{"data/Data_PEP725_all/"}).
#' @param flags Logical indicating whether the pep data contains quality control flags (default is \code{FALSE}).
#' @param add_country Logical indicating whether to add country information based on station coordinates (default is \code{TRUE}).
#' @return A \code{\link[=new_pep]{pep}} object (extends \code{data.table}) containing the combined and preprocessed PEP725 data.
#' @importFrom rnaturalearth ne_countries
#' @importFrom sf st_make_valid st_as_sf st_transform st_join st_agr st_agr<-
#' @seealso \code{\link[data.table]{fread}}, \code{\link[data.table]{rbindlist}}
#' @author Matthias Templ (FHNW)
#' @examples
#' # Small example files shipped with the package
#' path <- system.file("extdata", package = "pep725")
#' pep_data <- pep_import(path = path, add_country = FALSE)
#' print(pep_data)
#' @export
pep_import <- function(path = "data/Data_PEP725_all", flags = FALSE, add_country = TRUE){

  # Validate path
  if (!dir.exists(path)) {
    stop("Directory does not exist: ", path, call. = FALSE)
  }

  # Get all CSV file paths
  files <- list.files(path, pattern = "\\.csv$", full.names = TRUE)

  if (length(files) == 0) {
    stop("No CSV files found in: ", path, call. = FALSE)
  }

  # Read and bind them efficiently
  dt <- rbindlist(lapply(files, fread, sep = ";", fill = TRUE, quote = "",
                         encoding = "UTF-8", na.strings = c("", "NA")))

  if (nrow(dt) == 0) {
    stop("CSV files in '", path, "' contain no data rows", call. = FALSE)
  }

  dt$provider_id <- factor(dt$provider_id)
  dt$s_id <- factor(dt$s_id)
  dt$gss_id <- factor(dt$gss_id)
  dt$genus <- factor(dt$genus)
  dt$species <- factor(dt$species)
  dt$subspecies <- factor(dt$subspecies)
  dt[dt$alt == -9999, "alt"] <- NA
  dt[dt$alt_dem == -9999, "alt_dem"] <- NA
  # create a new altitude variable that combines both alt information
  dt$alt2 <- dt$alt
  dt$alt2[is.na(dt$alt2)] <- dt$alt_dem[is.na(dt$alt2)]
  dt[, date := as.Date(as.character(date))]
  dt$cult_season <- factor(dt$cult_season,
                           levels = 0:2,
                           labels = c("not applicable", "summer cereals - sowing in spring", "winter cereals - sowing in autumn"))
  if(flags){
    dt$affected_flag <- factor(dt$affected_flag,
                               levels = c(0, 1, 10),
                               labels = c("not affected", "affected", "unknown"))
    levels <- c(0L, 101007L, 1801000L, 1801001L, 1801002L)
    labels <- c("default", "spatial consistency check",
                "not tested", "no outlier", "outlier")

    dt[, qc_ori_flag := factor(qc_ori_flag, levels = levels, labels = labels)]
    dt[, c("affected_flag", "qc_flag") := NULL]
  }

  setcolorder(dt, c(
    head(names(dt), which(names(dt) == "alt_dem")),
    "alt2",
    setdiff(names(dt), c(head(names(dt), which(names(dt) == "alt_dem")), "alt2"))
  ))

  if(add_country){
    # Add country information to PEP data using fast spatial join
    message("Adding country information (add_country = TRUE). This may take 1-3 minutes...")

    # Convert to sf POINT object with WGS84 coordinates
    pep_sf <- st_as_sf(
      dt,
      coords = c("lon", "lat"),
      crs = 4326,
      remove = FALSE
    )

    # Load world country polygons
    world <- ne_countries(scale = "medium", returnclass = "sf")[, c("name")]

    # Fix invalid geometries (important)
    world <- st_make_valid(world)

    # Transform both to projected CRS (3857 = fast for intersections)
    pep_sf   <- st_transform(pep_sf, 3857)
    world    <- st_transform(world, 3857)

    # Improve join performance (avoid warnings)
    st_agr(world) <- "constant"

    # Fast spatial join: assign each point the country polygon
    pep_sf_joined <- st_join(pep_sf, world, left = TRUE)

    # Convert back to data.table
    dt <- as.data.table(pep_sf_joined)

    # Remove geometry column (not needed anymore)
    dt[, geometry := NULL]

    # Rename the joined country column
    setnames(dt, "name", "country")

    # Split Germany into south and north
    dt[
      country == "Germany" & lat >= 50,
      country := "Germany-North"
    ]

    dt[
      country == "Germany" & lat < 50,
      country := "Germany-South"
    ]
  }

  # Return as pep class object (validate required columns)
  new_pep(dt)
}

# Prevent R CMD check notes
utils::globalVariables(c("day", "year", "JDay", "Tmean", "Tmin", "Tmax",
                         "pheno_year", ".", "VIP", "coefficient", "period"))

#' Partial Least Squares Analysis for Phenology-Temperature Relationships
#'
#' Performs PLS regression to identify temperature-sensitive periods affecting
#' phenological timing. This is useful for understanding which parts of the
#' year have the strongest influence on phenological events.
#'
#' @param pep A \code{pep} object or data.frame with phenological observations.
#'   Must contain \code{year} and \code{day} columns.
#' @param temp_data A data.frame with daily temperature data. Must contain
#'   columns \code{year}, \code{doy} (day of year), and either \code{Tmean} or

#'   both \code{Tmin} and \code{Tmax}.
#' @param by Character vector of grouping columns to match phenology and
#'   temperature data (e.g., \code{"s_id"} for station-specific analysis).
#'   Default \code{NULL} uses the same temperature for all observations per year.
#' @param method Character. PLS method to use:
#'   \describe{
#'     \item{"robust"}{(Default) Iteratively reweighted PLS using bisquare weights.
#'       Resistant to outliers in both phenology and temperature data.}
#'     \item{"standard"}{Classical PLS regression without robust weighting.}
#'   }
#' @param split_month Integer (1-12). The last month to include in the
#'   phenological year. Default 6 (June) means July-June phenological years.
#'   This is important for spring phenology where relevant temperatures
#'   span the previous autumn/winter.
#' @param runn_mean Integer (odd). Window size for running mean smoothing of
#'   temperatures. Default 11 days. Larger values produce smoother results.
#' @param ncomp Integer or NULL. Number of PLS components. If NULL (default),
#'   automatically determined to explain at least \code{expl_var} percent
#'   of variance.
#' @param expl_var Numeric. Minimum percentage of variance to explain when
#'   automatically selecting components. Default 30.
#' @param max_iter Integer. Maximum iterations for robust method. Default 20.
#' @param tol Numeric. Convergence tolerance for robust method. Default 1e-4.
#'
#' @return An object of class \code{pheno_pls} containing:
#'   \describe{
#'     \item{vip}{Data.frame with Variable Importance in Projection scores
#'       for each day of the phenological year.}
#'     \item{coefficients}{PLS regression coefficients for each day.}
#'     \item{model}{The fitted PLS model object.}
#'     \item{weights}{Observation weights (all 1 for standard method).}
#'     \item{r_squared}{Model R-squared.}
#'     \item{ncomp}{Number of components used.}
#'     \item{method}{Method used ("robust" or "standard").}
#'     \item{n_obs}{Number of observations.}
#'     \item{n_years}{Number of years analyzed.}
#'   }
#'
#' @details
#' PLS regression is particularly useful for phenology-temperature analysis
#' because it handles the high collinearity between daily temperatures
#' (adjacent days are highly correlated) while identifying which periods
#' have the strongest influence on phenological timing.
#'
#' The robust method uses iteratively reweighted PLS with bisquare weights,
#' which downweights observations with large residuals. This is important
#' for phenological data which may contain outliers from observation errors
#' or unusual weather events.
#'
#' @section Interpreting Results:
#' \describe{
#'   \item{VIP scores}{Values > 0.8 indicate important variables. Days with
#'     high VIP scores have strong influence on phenology timing.}
#'   \item{Coefficients}{Negative coefficients indicate that warmer temperatures
#'     during that period lead to earlier phenology (lower DOY).}
#' }
#'
#' @section Phenological Year:
#' Spring phenology (e.g., flowering) is influenced by temperatures from the
#' previous autumn through spring. The \code{split_month} parameter defines
#' where to split the calendar year. With \code{split_month = 6}, a flowering
#' event in April 2020 is analyzed against temperatures from July 2019
#' through June 2020.
#'
#' @examples
#' \donttest{
#' data(pep_seed)
#'
#' # Create example temperature data (fewer years for speed)
#' years <- 2005:2012
#' temp <- data.frame(
#'   year = rep(years, each = 365),
#'   doy = rep(1:365, length(years)),
#'   Tmin = rnorm(365 * length(years), mean = 5, sd = 8),
#'   Tmax = rnorm(365 * length(years), mean = 15, sd = 8)
#' )
#'
#' # Run standard PLS (fast: no CV, fixed components)
#' result <- pheno_pls(pep_seed, temp, method = "standard", ncomp = 2)
#' print(result)
#' }
#'
#' @references
#' Luedeling E, Gassner A (2012). Partial Least Squares Regression for
#' analyzing walnut phenology in California. Agricultural and Forest
#' Meteorology 158-159:104-113.
#'
#' Wold S, Sjostrom M, Eriksson L (2001). PLS-regression: a basic tool of
#' chemometrics. Chemometrics and Intelligent Laboratory Systems 58:109-130.
#'
#' @seealso \code{\link{pheno_trend_turning}} for trend analysis,
#'   \code{\link{pheno_anomaly}} for anomaly detection
#'
#' @author Matthias Templ
#' @importFrom stats mad weighted.mean median coef predict
#' @export
pheno_pls <- function(pep,
                          temp_data,
                          by = NULL,
                          method = c("robust", "standard"),
                          split_month = 6,
                          runn_mean = 11,
                          ncomp = NULL,
                          expl_var = 30,
                          max_iter = 20,
                          tol = 1e-4) {

  method <- match.arg(method)

  # Check for pls package

  if (!requireNamespace("pls", quietly = TRUE)) {
    stop("Package 'pls' is required for PLS analysis. Install with: install.packages('pls')",
         call. = FALSE)
  }

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }
  if (!inherits(temp_data, "data.frame")) {
    stop("'temp_data' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }
  if (!inherits(temp_data, "data.table")) {
    temp_data <- data.table::as.data.table(temp_data)
  }

  # Check required columns
  if (!all(c("year", "day") %in% names(pep))) {
    stop("'pep' must have 'year' and 'day' columns", call. = FALSE)
  }
  if (!all(c("year", "doy") %in% names(temp_data))) {
    stop("'temp_data' must have 'year' and 'doy' columns", call. = FALSE)
  }

  # Check temperature columns
  has_tmean <- "Tmean" %in% names(temp_data)
  has_minmax <- all(c("Tmin", "Tmax") %in% names(temp_data))
  if (!has_tmean && !has_minmax) {
    stop("'temp_data' must have either 'Tmean' or both 'Tmin' and 'Tmax'",
         call. = FALSE)
  }

  # Calculate Tmean if not present
  if (!has_tmean) {
    temp_data[, Tmean := (Tmin + Tmax) / 2]
  }

  # Validate runn_mean is odd

  if (runn_mean %% 2 == 0) {
    runn_mean <- runn_mean + 1
    warning("runn_mean must be odd; using ", runn_mean, call. = FALSE)
  }

  # Aggregate phenology by year (take median DOY per year for analysis)
  pep_agg <- pep[, .(pheno = median(day, na.rm = TRUE)), by = year]
  pep_agg <- pep_agg[!is.na(pheno)]

  # Determine phenological year for each observation
  # For split_month = 6, pheno year 2020 includes Jul 2019 - Jun 2020
  pep_agg[, pheno_year := year]

  # Get unique years with phenology data
  pheno_years <- sort(unique(pep_agg$pheno_year))

  # Build temperature matrix for PLS
  # Each row = one phenological year
  # Each column = one day of the phenological year (365 days)
  n_days <- 365
  temp_matrix <- matrix(NA, nrow = length(pheno_years), ncol = n_days)
  pheno_vector <- numeric(length(pheno_years))

  for (i in seq_along(pheno_years)) {
    py <- pheno_years[i]

    # Get phenology DOY for this year
    pheno_doy <- pep_agg[pheno_year == py, pheno]
    if (length(pheno_doy) == 0 || is.na(pheno_doy)) next
    pheno_vector[i] <- pheno_doy

    # Get temperatures for this phenological year
    # From (split_month + 1) of previous year to split_month of current year
    start_year <- py - 1
    end_year <- py

    # Days from previous year: from July 1 (if split_month = 6) to Dec 31
    days_prev_year <- temp_data[year == start_year & doy > split_month * 30.5]
    days_curr_year <- temp_data[year == end_year & doy <= split_month * 30.5]

    # Combine and order
    year_temps <- rbind(days_prev_year, days_curr_year)
    data.table::setorder(year_temps, year, doy)

    if (nrow(year_temps) < 300) next  # Skip years with insufficient data

    # Take first 365 days and extract Tmean
    temps <- head(year_temps$Tmean, n_days)
    if (length(temps) < n_days) {
      temps <- c(temps, rep(NA, n_days - length(temps)))
    }
    temp_matrix[i, ] <- temps
  }

  # Remove rows with missing data
  valid_rows <- complete.cases(temp_matrix) & !is.na(pheno_vector)
  temp_matrix <- temp_matrix[valid_rows, , drop = FALSE]
  pheno_vector <- pheno_vector[valid_rows]
  pheno_years <- pheno_years[valid_rows]

  if (nrow(temp_matrix) < 5) {
    stop("Insufficient data: need at least 5 complete phenological years",
         call. = FALSE)
  }

  # Apply running mean smoothing
  temp_matrix_smooth <- apply(temp_matrix, 1, function(row) {
    stats::filter(row, rep(1/runn_mean, runn_mean), sides = 2)
  })
  temp_matrix_smooth <- t(temp_matrix_smooth)

  # Handle NAs from smoothing at edges
  half_win <- floor(runn_mean / 2)
  temp_matrix_smooth[, 1:half_win] <- temp_matrix[, 1:half_win]
  temp_matrix_smooth[, (n_days - half_win + 1):n_days] <-
    temp_matrix[, (n_days - half_win + 1):n_days]

  # Create data frame for PLS
  pls_data <- data.frame(pheno = pheno_vector, temp_matrix_smooth)
  colnames(pls_data) <- c("pheno", paste0("day_", 1:n_days))

  # Scale predictors
  temp_scaled <- scale(temp_matrix_smooth)
  temp_center <- attr(temp_scaled, "scaled:center")
  temp_scale <- attr(temp_scaled, "scaled:scale")

  # Initialize weights
  n_obs <- nrow(temp_matrix_smooth)
  weights <- rep(1, n_obs)

  # Fit PLS model
  if (method == "standard") {
    # Standard PLS
    pls_fit <- fit_pls(pheno_vector, temp_scaled, ncomp, expl_var, weights)

  } else {
    # Robust PLS using iteratively reweighted approach
    pls_fit <- fit_pls(pheno_vector, temp_scaled, ncomp, expl_var, weights)

    for (iter in seq_len(max_iter)) {
      # Get residuals from training predictions
      pred <- pls_fit$predictions
      resid <- pheno_vector - pred

      # Calculate robust weights using bisquare function
      weights_new <- bisquare_weights(resid)

      # Check convergence
      weight_change <- max(abs(weights_new - weights))
      weights <- weights_new

      if (weight_change < tol) break

      # Refit with new weights
      pls_fit <- fit_pls(pheno_vector, temp_scaled, pls_fit$ncomp, expl_var, weights)
    }
  }

  # Calculate VIP scores
  vip_scores <- calculate_vip(pls_fit)

  # Get coefficients
  coefs <- pls_fit$coefficients

  # Calculate R-squared using predictions from the pls model
  pred_final <- pls_fit$predictions
  ss_res <- sum(weights * (pheno_vector - pred_final)^2)
  ss_tot <- sum(weights * (pheno_vector - weighted.mean(pheno_vector, weights))^2)
  r_squared <- 1 - ss_res / ss_tot

  # Create day labels for output
  # Days are numbered from start of phenological year
  day_labels <- create_day_labels(split_month, n_days)

  # Prepare output
  result <- list(
    vip = data.frame(
      day = 1:n_days,
      period = day_labels,
      VIP = vip_scores
    ),
    coefficients = data.frame(
      day = 1:n_days,
      period = day_labels,
      coefficient = coefs
    ),
    model = pls_fit,
    weights = weights,
    r_squared = r_squared,
    ncomp = pls_fit$ncomp,
    method = method,
    n_obs = n_obs,
    n_years = length(pheno_years),
    pheno_years = pheno_years,
    split_month = split_month,
    runn_mean = runn_mean
  )

  class(result) <- c("pheno_pls", "list")
  result
}


#' Fit PLS model (internal)
#' @noRd
fit_pls <- function(y, X, ncomp = NULL, expl_var = 30, weights = NULL) {
  n <- length(y)
  p <- ncol(X)

  if (is.null(weights)) weights <- rep(1, n)

  # Create data frame for pls
  pls_df <- data.frame(y = y, X = I(X))

  # Determine number of components if not specified
  max_comp <- min(n - 2, p, 15)

  if (is.null(ncomp)) {
    # Use cross-validation to select components
    suppressWarnings({
      pls_full <- pls::plsr(y ~ X, data = pls_df, ncomp = max_comp,
                            validation = "CV", weights = weights)
    })
    explvar <- cumsum(pls::explvar(pls_full))

    # Find first component that exceeds expl_var threshold
    ncomp <- which(explvar >= expl_var)[1]
    if (is.na(ncomp)) ncomp <- max_comp
    ncomp <- max(1, min(ncomp, max_comp))
  }

  # Fit final model
  pls_fit <- pls::plsr(y ~ X, data = pls_df, ncomp = ncomp, weights = weights)

  # Extract coefficients (on scaled X)
  coefs <- as.vector(coef(pls_fit, ncomp = ncomp))

  # Get predictions on training data
  predictions <- as.vector(predict(pls_fit, newdata = pls_df, ncomp = ncomp))

  list(
    pls_object = pls_fit,
    coefficients = coefs,
    ncomp = ncomp,
    weights = weights,
    predictions = predictions,
    X = X,
    y = y
  )
}


#' Predict from PLS model (internal)
#' @noRd
predict_pls <- function(pls_fit, X = NULL) {
  if (is.null(X)) {
    # Return training predictions
    return(pls_fit$predictions)
  }
  # Predict on new data
  pls_df <- data.frame(X = I(X))
  as.vector(predict(pls_fit$pls_object, newdata = pls_df, ncomp = pls_fit$ncomp))
}


#' Calculate VIP scores (internal)
#' @noRd
calculate_vip <- function(pls_fit) {
  # Extract components from pls object
  pls_obj <- pls_fit$pls_object
  ncomp <- pls_fit$ncomp

  # Get scores and loadings
  W <- pls_obj$loading.weights[, 1:ncomp, drop = FALSE]
  Q <- pls_obj$Yloadings[, 1:ncomp, drop = FALSE]
  T_scores <- pls_obj$scores[, 1:ncomp, drop = FALSE]

  # Calculate explained variance for each component
  SS <- colSums(T_scores^2) * as.vector(Q)^2

  # VIP calculation
  p <- nrow(W)
  vip <- numeric(p)

  for (j in seq_len(p)) {
    ss_j <- sum((W[j, ]^2) * SS)
    vip[j] <- sqrt(p * ss_j / sum(SS))
  }

  vip
}


#' Bisquare weight function for robust estimation (internal)
#' @noRd
bisquare_weights <- function(resid, c = 4.685) {
  # Robust scale estimate using MAD
  s <- mad(resid, center = median(resid), constant = 1.4826)
  if (s < 1e-10) s <- 1

  # Standardize residuals
  u <- resid / s

  # Bisquare weights
  weights <- ifelse(abs(u) <= c,
                    (1 - (u/c)^2)^2,
                    0)

  # Ensure no zero weights (minimum weight)
  weights <- pmax(weights, 0.01)
  weights
}


#' Create day labels for phenological year (internal)
#' @noRd
create_day_labels <- function(split_month, n_days) {
  # Create month-day labels starting from split_month + 1
  months <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun",
              "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
  days_per_month <- c(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)

  # Reorder starting from split_month + 1
  start_idx <- split_month + 1
  month_order <- c(start_idx:12, 1:split_month)

  labels <- character(n_days)
  day_counter <- 1

  for (m in month_order) {
    n_days_month <- days_per_month[m]
    for (d in seq_len(n_days_month)) {
      if (day_counter > n_days) break
      labels[day_counter] <- paste0(months[m], "-", d)
      day_counter <- day_counter + 1
    }
    if (day_counter > n_days) break
  }

  labels
}


#' Print Method for PLS Phenology Results
#'
#' @param x A \code{pheno_pls} object
#' @param ... Additional arguments (unused)
#' @return Invisibly returns \code{x}
#' @export
print.pheno_pls <- function(x, ...) {
  cat("PLS Phenology-Temperature Analysis\n")
  cat(strrep("=", 45), "\n")
  cat("Method:", x$method, "\n")
  cat("Components:", x$ncomp, "\n")
  cat("R-squared:", sprintf("%.3f", x$r_squared), "\n")
  cat("Years analyzed:", x$n_years, "\n")
  cat("Running mean window:", x$runn_mean, "days\n")
  cat("Split month:", x$split_month, "\n\n")

  # Show top VIP periods
  vip_sorted <- x$vip[order(-x$vip$VIP), ]
  cat("Top 10 most important temperature periods:\n")
  print(head(vip_sorted, 10), row.names = FALSE)

  if (x$method == "robust") {
    n_downweighted <- sum(x$weights < 0.5)
    cat("\nRobust fitting: ", n_downweighted, " observations downweighted\n", sep = "")
  }

  invisible(x)
}


#' Summary Method for PLS Phenology Results
#'
#' @param object A \code{pheno_pls} object
#' @param vip_threshold Numeric. VIP threshold for identifying important
#'   periods. Default 0.8.
#' @param ... Additional arguments (unused)
#' @return Invisibly returns a summary data.frame
#' @export
summary.pheno_pls <- function(object, vip_threshold = 0.8, ...) {
  cat("PLS Phenology-Temperature Analysis Summary\n")
  cat(strrep("=", 50), "\n\n")

  cat("Model fit:\n")
  cat("  Method:", object$method, "\n")
  cat("  Components:", object$ncomp, "\n")
  cat("  R-squared:", sprintf("%.3f", object$r_squared), "\n")
  cat("  Years:", object$n_years, "(", min(object$pheno_years), "-",
      max(object$pheno_years), ")\n\n")

  # Identify important periods (VIP > threshold)
  important <- object$vip[object$vip$VIP >= vip_threshold, ]

  if (nrow(important) > 0) {
    cat("Important temperature periods (VIP >=", vip_threshold, "):\n")

    # Group consecutive days into periods
    important_days <- important$day
    if (length(important_days) > 0) {
      # Find runs of consecutive days
      runs <- rle(diff(important_days) == 1)
      periods <- data.frame(
        start = integer(),
        end = integer(),
        mean_vip = numeric(),
        mean_coef = numeric()
      )

      pos <- 1
      for (i in seq_along(runs$lengths)) {
        if (i == 1 || !runs$values[i-1]) {
          start_day <- important_days[pos]
        }
        if (runs$values[i]) {
          end_day <- important_days[pos + runs$lengths[i]]
        } else {
          end_day <- important_days[pos]
        }
        if (!runs$values[i] || i == length(runs$lengths)) {
          days_in_period <- start_day:end_day
          mean_vip <- mean(object$vip$VIP[days_in_period])
          mean_coef <- mean(object$coefficients$coefficient[days_in_period])
          periods <- rbind(periods, data.frame(
            start = start_day,
            end = end_day,
            mean_vip = mean_vip,
            mean_coef = mean_coef
          ))
        }
        pos <- pos + runs$lengths[i]
      }

      # Add period labels
      periods$start_period <- object$vip$period[periods$start]
      periods$end_period <- object$vip$period[periods$end]
      periods$effect <- ifelse(periods$mean_coef < 0, "advancing", "delaying")

      cat("\n")
      for (i in seq_len(nrow(periods))) {
        cat(sprintf("  %s to %s: VIP=%.2f, %s effect\n",
                    periods$start_period[i],
                    periods$end_period[i],
                    periods$mean_vip[i],
                    periods$effect[i]))
      }
    }
  } else {
    cat("No periods with VIP >=", vip_threshold, "found.\n")
  }

  cat("\n")
  invisible(object$vip)
}


#' Plot Method for PLS Phenology Results
#'
#' @param x A \code{pheno_pls} object
#' @param type Character. Plot type: "vip" for VIP scores (default),
#'   "coef" for coefficients, "both" for combined plot.
#' @param vip_threshold Numeric. Threshold line for VIP plot. Default 0.8.
#' @param ... Additional arguments passed to plotting functions
#' @return A ggplot object (invisibly)
#' @export
plot.pheno_pls <- function(x, type = c("vip", "coef", "both"),
                                vip_threshold = 0.8, ...) {
  type <- match.arg(type)

  # Create month breaks for x-axis
  split_month <- x$split_month
  months <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun",
              "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
  days_per_month <- c(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)

  month_order <- c((split_month + 1):12, 1:split_month)
  month_starts <- cumsum(c(1, days_per_month[month_order[-length(month_order)]]))
  month_labels <- months[month_order]

  if (type == "vip") {
    p <- ggplot2::ggplot(x$vip, ggplot2::aes(x = day, y = VIP)) +
      ggplot2::geom_line(color = "steelblue", linewidth = 0.8) +
      ggplot2::geom_hline(yintercept = vip_threshold, linetype = "dashed",
                          color = "red", alpha = 0.7) +
      ggplot2::scale_x_continuous(breaks = month_starts, labels = month_labels) +
      ggplot2::labs(
        title = "Variable Importance in Projection (VIP) Scores",
        subtitle = sprintf("Method: %s, R-sq = %.3f", x$method, x$r_squared),
        x = "Month",
        y = "VIP Score"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = 1))

  } else if (type == "coef") {
    p <- ggplot2::ggplot(x$coefficients, ggplot2::aes(x = day, y = coefficient)) +
      ggplot2::geom_line(color = "darkgreen", linewidth = 0.8) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", alpha = 0.5) +
      ggplot2::scale_x_continuous(breaks = month_starts, labels = month_labels) +
      ggplot2::labs(
        title = "PLS Regression Coefficients",
        subtitle = "Negative = warmer temps lead to earlier phenology",
        x = "Month",
        y = "Coefficient"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = 1))

  } else {
    # Both plots combined
    p1 <- ggplot2::ggplot(x$vip, ggplot2::aes(x = day, y = VIP)) +
      ggplot2::geom_line(color = "steelblue", linewidth = 0.8) +
      ggplot2::geom_hline(yintercept = vip_threshold, linetype = "dashed",
                          color = "red", alpha = 0.7) +
      ggplot2::scale_x_continuous(breaks = month_starts, labels = month_labels) +
      ggplot2::labs(title = "VIP Scores", x = NULL, y = "VIP") +
      ggplot2::theme_minimal() +
      ggplot2::theme(axis.text.x = ggplot2::element_blank())

    p2 <- ggplot2::ggplot(x$coefficients, ggplot2::aes(x = day, y = coefficient)) +
      ggplot2::geom_line(color = "darkgreen", linewidth = 0.8) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", alpha = 0.5) +
      ggplot2::scale_x_continuous(breaks = month_starts, labels = month_labels) +
      ggplot2::labs(title = "Coefficients", x = "Month", y = "Coefficient") +
      ggplot2::theme_minimal() +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = 1))

    p <- patchwork::wrap_plots(p1, p2, ncol = 1)
  }

  print(p)
  invisible(p)
}

# Prevent R CMD check notes
utils::globalVariables(c("year", "day", "s_id", "station", "year_effect",
                         "station_effect", "residual", "fitted", "set_id",
                         "n_sets", "set_size", "..by"))

#' Create Combined Phenological Time Series
#'
#' Estimates a combined (regional/national) phenological time series from
#' multi-station observations by separating year effects from station effects.
#' This is essential for creating representative time series from networks
#' with varying station coverage over time.
#'
#' @param pep A \code{pep} object or \code{data.table} containing phenological
#'   observations with columns \code{year}, \code{day}, and \code{s_id}.
#' @param by Character vector of grouping variables (e.g., \code{c("genus",
#'   "species", "phase_id")}). A separate combined series is created for each
#'   group. Default is \code{NULL} (treat all data as one group).
#' @param method Character. Estimation method:
#'   \describe{
#'     \item{"robust"}{(Default) Least Absolute Deviations (LAD/L1) regression.
#'       Minimizes sum of absolute residuals, robust to outliers.}
#'     \item{"mixed"}{Mixed-effects model with year as fixed effect and station
#'       as random effect. Uses REML estimation.}
#'     \item{"ols"}{Ordinary Least Squares. Sensitive to outliers but provides
#'       standard errors and confidence intervals.}
#'   }
#' @param min_years Integer. Minimum years of overlap required. Default 5.
#' @param min_stations Integer. Minimum stations required. Default 3.
#' @param check_connectivity Logical. If \code{TRUE} (default), checks whether
#'   station-year combinations form a connected set. Warns if data is
#'   disconnected (which prevents valid combined series estimation).
#'
#' @return An object of class \code{pheno_combined} containing:
#'   \describe{
#'     \item{combined}{Data.table with the combined time series: year,
#'       year_effect (the combined DOY), se (standard error, if available)}
#'     \item{station_effects}{Data.table of station effects: s_id, effect, se}
#'     \item{residuals}{Data.table with original data plus fitted values and
#'       residuals for outlier detection}
#'     \item{method}{Estimation method used}
#'     \item{connectivity}{Connectivity check results (if performed)}
#'     \item{by}{Grouping variables used}
#'     \item{group_results}{For grouped analysis, results per group}
#'   }
#'
#' @details
#' The model fitted is: \deqn{DOY_{ij} = \mu_i + \sigma_j + \epsilon_{ij}}
#' where \eqn{\mu_i} is the year effect (combined series), \eqn{\sigma_j} is
#' the station effect, and \eqn{\epsilon_{ij}} is the residual.
#'
#' Station effects represent systematic differences between stations (e.g.,
#' due to altitude, microclimate, or observer differences). The combined
#' series removes these effects to reveal the underlying temporal trend.
#'
#' @section Connectivity:
#' A prerequisite for valid estimation is that station-year combinations
#' form a "connected" set - meaning there must be overlapping years between
#' stations to separate year and station effects. The function checks this
#' automatically and warns if data is disconnected.
#'
#' @section Outlier Detection:
#' With \code{method = "robust"}, large residuals (e.g., > 30 days) indicate
#' potential outliers or data errors. The residuals table can be used to
#' identify and investigate these.
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Grapevine flowering in Switzerland (longest historical records)
#' vine_ch <- pep[species == "Vitis vinifera" &
#'                phase_id == 65 & country == "Switzerland"]
#'
#' if (nrow(vine_ch) > 0) {
#'   # Create combined series using OLS
#'   combined <- pheno_combine(vine_ch, method = "ols")
#'   print(combined)
#'
#'   # Identify potential outliers (residuals > 30 days)
#'   outliers <- combined$residuals[abs(residual) > 30]
#'   outliers
#' }
#' }
#'
#' @references
#' Schaber J, Badeck F-W (2002). Evaluation of methods for the combination
#' of phenological time series and outlier detection. Tree Physiology 22:973-982.
#'
#' @seealso
#' \code{\link{pheno_normals}} for climatological baselines,
#' \code{\link{pheno_trend_turning}} for trend turning point detection,
#' \code{\link{pep_check_connectivity}} for connectivity assessment
#'
#' @author Matthias Templ
#' @export
pheno_combine <- function(pep,
                           by = NULL,
                           method = c("robust", "mixed", "ols"),
                           min_years = 5,
                           min_stations = 3,
                           check_connectivity = TRUE) {

  method <- match.arg(method)

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  # Check required columns
  required <- c("year", "day", "s_id")
  missing <- setdiff(required, names(pep))
  if (length(missing) > 0) {
    stop("Missing required columns: ", paste(missing, collapse = ", "), call. = FALSE)
  }

  # Check grouping columns
  if (!is.null(by)) {
    missing_by <- setdiff(by, names(pep))
    if (length(missing_by) > 0) {
      stop("Column(s) not found: ", paste(missing_by, collapse = ", "), call. = FALSE)
    }
  }

  # Function to fit combined model for one group
  fit_combined <- function(dt, group_label = NULL) {
    # Remove missing values
    dt <- dt[!is.na(day) & !is.na(year) & !is.na(s_id)]

    # Check minimum requirements
    n_years <- data.table::uniqueN(dt$year)
    n_stations <- data.table::uniqueN(dt$s_id)

    if (n_years < min_years || n_stations < min_stations) {
      return(list(
        combined = NULL,
        station_effects = NULL,
        residuals = NULL,
        connectivity = NULL,
        error = sprintf("Insufficient data: %d years, %d stations (need %d years, %d stations)",
                        n_years, n_stations, min_years, min_stations)
      ))
    }

    # Check connectivity if requested
    conn <- NULL
    if (check_connectivity) {
      conn <- check_connectivity_internal(dt)
      if (!conn$is_connected) {
        warning(sprintf("%sData is not fully connected (%d disconnected sets). ",
                        if (!is.null(group_label)) paste0(group_label, ": ") else "",
                        conn$n_sets),
                "Using largest connected subset.", call. = FALSE)
        # Use largest connected set
        dt <- dt[s_id %in% conn$largest_set$stations &
                   year %in% conn$largest_set$years]
      }
    }

    # Prepare factors (drop unused levels to avoid contrast errors)
    dt <- data.table::copy(dt)
    # Remove pep class to avoid interference with := assignment
    class(dt) <- c("data.table", "data.frame")
    dt[, year := factor(year)]
    dt[, station := factor(as.character(s_id))]

    # Fit model based on method
    result <- tryCatch({
      if (method == "robust") {
        fit_lad(dt)
      } else if (method == "mixed") {
        fit_mixed(dt)
      } else {
        fit_ols(dt)
      }
    }, error = function(e) {
      list(error = e$message)
    })

    if (!is.null(result$error)) {
      return(list(
        combined = NULL,
        station_effects = NULL,
        residuals = NULL,
        connectivity = conn,
        error = result$error
      ))
    }

    result$connectivity <- conn
    result$n_obs <- nrow(dt)
    result$n_years <- n_years
    result$n_stations <- n_stations
    result
  }

  # Process data
  if (is.null(by)) {
    result <- fit_combined(pep)
    result$by <- NULL
    result$method <- method
    result$group_results <- NULL
  } else {
    # Grouped analysis
    groups <- unique(pep[, ..by])
    group_results <- list()

    for (i in seq_len(nrow(groups))) {
      # Build filter
      filter_list <- lapply(by, function(col) pep[[col]] == groups[[col]][i])
      filter_idx <- Reduce(`&`, filter_list)
      group_data <- pep[filter_idx]

      group_label <- paste(sapply(by, function(col) as.character(groups[[col]][i])),
                           collapse = " | ")

      group_results[[group_label]] <- fit_combined(group_data, group_label)
    }

    # Combine results
    all_combined <- data.table::rbindlist(
      lapply(names(group_results), function(nm) {
        res <- group_results[[nm]]
        if (!is.null(res$combined)) {
          cbind(group = nm, res$combined)
        }
      }),
      fill = TRUE
    )

    result <- list(
      combined = all_combined,
      method = method,
      by = by,
      n_groups = nrow(groups),
      group_results = group_results
    )
  }

  class(result) <- c("pheno_combined", "list")
  result
}


#' Fit LAD (Robust) Model
#' @keywords internal
fit_lad <- function(dt) {
  # Use quantreg for LAD regression
  if (!requireNamespace("quantreg", quietly = TRUE)) {
    stop("Package 'quantreg' required for robust estimation", call. = FALSE)
  }

  # Fit model: day ~ year + station - 1 (no intercept, with sum-to-zero constraint)
  # Using treatment contrasts and adjusting afterwards for efficiency
  fit <- quantreg::rq(day ~ year + station, data = dt, tau = 0.5)

  coefs <- coef(fit)
  year_levels <- levels(dt$year)
  station_levels <- levels(dt$station)

  # Extract year effects (combined series)
  year_idx <- grep("^year", names(coefs))
  year_effects <- coefs[year_idx]
  names(year_effects) <- gsub("^year", "", names(year_effects))

  # Add intercept to first year and create full series
  intercept <- coefs["(Intercept)"]
  if (!is.na(intercept)) {
    year_effects <- year_effects + intercept
    # First year level is in the intercept
    year_effects <- c(intercept, year_effects)
    names(year_effects)[1] <- year_levels[1]
  }

  # Extract station effects
  station_idx <- grep("^station", names(coefs))
  station_effects <- coefs[station_idx]
  names(station_effects) <- gsub("^station", "", names(station_effects))

  # First station is reference (effect = 0), adjust to sum-to-zero
  station_effects <- c(0, station_effects)
  names(station_effects)[1] <- station_levels[1]
  station_mean <- mean(station_effects)
  station_effects <- station_effects - station_mean

  # Compute residuals
  dt[, fitted := predict(fit)]
  dt[, residual := day - fitted]

  list(
    combined = data.table::data.table(
      year = as.integer(names(year_effects)),
      year_effect = as.numeric(year_effects)
    )[order(year)],
    station_effects = data.table::data.table(
      s_id = names(station_effects),
      station_effect = as.numeric(station_effects)
    ),
    residuals = dt[, .(s_id, year = as.integer(as.character(year)),
                       day, fitted, residual)],
    fit = fit,
    error = NULL
  )
}


#' Fit Mixed-Effects Model
#' @keywords internal
fit_mixed <- function(dt) {
  if (!requireNamespace("nlme", quietly = TRUE)) {
    stop("Package 'nlme' required for mixed-effects estimation", call. = FALSE)
  }

  # Fit: day ~ year (fixed) with station as random effect
  fit <- nlme::lme(day ~ year - 1, random = ~ 1 | station, data = dt,
                   method = "REML")

  # Extract fixed effects (year effects = combined series)
  fixed_effects <- nlme::fixef(fit)
  year_levels <- levels(dt$year)
  names(fixed_effects) <- year_levels

  # Get standard errors from model summary
  fit_summary <- summary(fit)
  se_fixed <- fit_summary$tTable[, "Std.Error"]

  # Extract random effects (station effects)
  random_effects <- nlme::ranef(fit)
  if (is.data.frame(random_effects)) {
    station_levels <- rownames(random_effects)
    random_vals <- random_effects[, 1]
  } else {
    station_levels <- rownames(random_effects[[1]])
    random_vals <- random_effects[[1]][, 1]
  }
  names(random_vals) <- station_levels

  # Compute residuals
  dt[, fitted := predict(fit)]
  dt[, residual := day - fitted]

  combined_dt <- data.table::data.table(
    year = as.integer(names(fixed_effects)),
    year_effect = as.numeric(fixed_effects),
    se = as.numeric(se_fixed)
  )

  # Extract variance components
  vc <- nlme::VarCorr(fit)
  var_station <- as.numeric(vc[1, "Variance"])
  var_residual <- as.numeric(vc[2, "Variance"])

  list(
    combined = combined_dt[order(year)],
    station_effects = data.table::data.table(
      s_id = names(random_vals),
      station_effect = as.numeric(random_vals)
    ),
    residuals = dt[, .(s_id, year = as.integer(as.character(year)),
                       day, fitted, residual)],
    fit = fit,
    var_station = var_station,
    var_residual = var_residual,
    error = NULL
  )
}


#' Fit OLS Model
#' @keywords internal
fit_ols <- function(dt) {
  # Sum-to-zero contrasts for station
  contrasts(dt$station) <- contr.sum(levels(dt$station))

  fit <- lm(day ~ year + station - 1, data = dt)
  coefs <- coef(fit)
  se <- summary(fit)$coefficients[, "Std. Error"]

  year_levels <- levels(dt$year)
  station_levels <- levels(dt$station)

  # Extract year effects
  year_idx <- grep("^year", names(coefs))
  year_effects <- coefs[year_idx]
  year_se <- se[year_idx]
  names(year_effects) <- gsub("^year", "", names(year_effects))
  names(year_se) <- names(year_effects)

  # Extract station effects (sum-to-zero, last level is -sum(others))
  station_idx <- grep("^station", names(coefs))
  station_effects <- coefs[station_idx]
  names(station_effects) <- station_levels[-length(station_levels)]
  # Last station effect is negative sum of others
  station_effects <- c(station_effects, -sum(station_effects))
  names(station_effects)[length(station_effects)] <- station_levels[length(station_levels)]

  # Compute residuals
  dt[, fitted := predict(fit)]
  dt[, residual := day - fitted]

  list(
    combined = data.table::data.table(
      year = as.integer(names(year_effects)),
      year_effect = as.numeric(year_effects),
      se = as.numeric(year_se)
    )[order(year)],
    station_effects = data.table::data.table(
      s_id = names(station_effects),
      station_effect = as.numeric(station_effects)
    ),
    residuals = dt[, .(s_id, year = as.integer(as.character(year)),
                       day, fitted, residual)],
    fit = fit,
    r_squared = summary(fit)$r.squared,
    error = NULL
  )
}


#' Check Connectivity of Station-Year Data (Internal)
#' @keywords internal
check_connectivity_internal <- function(dt) {
  # Create station-year matrix
  stations <- unique(dt$s_id)
  years <- sort(unique(dt$year))

  # Create incidence matrix (1 if station has data in year, 0 otherwise)
  present <- dt[, .(present = 1), by = .(s_id, year)]
  mat <- matrix(0, nrow = length(stations), ncol = length(years),
                dimnames = list(stations, years))

  for (i in seq_len(nrow(present))) {
    mat[as.character(present$s_id[i]), as.character(present$year[i])] <- 1
  }

  # Find connected components using breadth-first search
  n_stations <- length(stations)
  n_years <- length(years)

  station_set <- rep(0, n_stations)
  year_set <- rep(0, n_years)
  current_set <- 0

  for (s in seq_len(n_stations)) {
    if (station_set[s] == 0 && any(mat[s, ] > 0)) {
      current_set <- current_set + 1

      # BFS from this station
      station_queue <- s
      while (length(station_queue) > 0) {
        curr_s <- station_queue[1]
        station_queue <- station_queue[-1]

        if (station_set[curr_s] != 0) next
        station_set[curr_s] <- current_set

        # Find years this station has data
        connected_years <- which(mat[curr_s, ] > 0)
        for (y in connected_years) {
          if (year_set[y] == 0) {
            year_set[y] <- current_set
            # Find other stations with data in this year
            connected_stations <- which(mat[, y] > 0)
            station_queue <- c(station_queue,
                               connected_stations[station_set[connected_stations] == 0])
          }
        }
      }
    }
  }

  # Count sets and their sizes
  n_sets <- max(station_set)
  set_sizes <- sapply(seq_len(n_sets), function(i) {
    sum(mat[station_set == i, year_set == i, drop = FALSE] > 0)
  })

  # Find largest set
  largest_idx <- which.max(set_sizes)

  list(
    is_connected = n_sets == 1,
    n_sets = n_sets,
    set_sizes = set_sizes,
    largest_set = list(
      stations = stations[station_set == largest_idx],
      years = years[year_set == largest_idx],
      n_obs = set_sizes[largest_idx]
    ),
    station_sets = station_set,
    year_sets = year_set
  )
}


#' Check Station-Year Connectivity
#'
#' Checks whether station-year combinations in phenological data form a
#' connected set, which is required for valid combined time series estimation.
#'
#' @param pep A \code{pep} object or data.table with \code{year}, \code{day},
#'   and \code{s_id} columns.
#' @param by Optional grouping variables to check connectivity within groups.
#'
#' @return A list with connectivity information:
#'   \describe{
#'     \item{is_connected}{Logical. TRUE if all data forms one connected set.}
#'     \item{n_sets}{Number of disconnected sets found.}
#'     \item{set_sizes}{Number of observations in each set.}
#'     \item{largest_set}{Stations and years in the largest connected set.}
#'   }
#'
#' @details
#' Connectivity means that there exist overlapping years between stations,
#' allowing year effects to be separated from station effects. If data is
#' disconnected, the combined series cannot be uniquely estimated across
#' the disconnected parts.
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#' pep_alpine <- pep[country %in% c("Switzerland", "Austria")]
#' conn <- pep_check_connectivity(pep_alpine)
#' if (!conn$is_connected) {
#'   warning("Data has ", conn$n_sets, " disconnected sets")
#' }
#' }
#'
#' @seealso \code{\link{pheno_combine}} which uses this internally
#' @author Matthias Templ
#' @export
pep_check_connectivity <- function(pep, by = NULL) {
  if (!inherits(pep, "data.table")) {
    pep <- data.table::as.data.table(pep)
  }

  required <- c("year", "day", "s_id")
  missing <- setdiff(required, names(pep))
  if (length(missing) > 0) {
    stop("Missing required columns: ", paste(missing, collapse = ", "), call. = FALSE)
  }

  if (is.null(by)) {
    result <- check_connectivity_internal(pep)
    class(result) <- c("pep_connectivity", "list")
    return(result)
  }

  # Grouped analysis
  groups <- unique(pep[, ..by])
  results <- list()

  for (i in seq_len(nrow(groups))) {
    filter_list <- lapply(by, function(col) pep[[col]] == groups[[col]][i])
    filter_idx <- Reduce(`&`, filter_list)
    group_data <- pep[filter_idx]

    group_label <- paste(sapply(by, function(col) as.character(groups[[col]][i])),
                         collapse = " | ")
    results[[group_label]] <- check_connectivity_internal(group_data)
  }

  summary_dt <- data.table::rbindlist(lapply(names(results), function(nm) {
    data.table::data.table(
      group = nm,
      is_connected = results[[nm]]$is_connected,
      n_sets = results[[nm]]$n_sets,
      largest_set_size = results[[nm]]$largest_set$n_obs
    )
  }))

  result <- list(
    summary = summary_dt,
    group_results = results,
    by = by
  )
  class(result) <- c("pep_connectivity", "list")
  result
}


#' Print Method for Combined Time Series
#' @param x A \code{pheno_combined} object
#' @param ... Additional arguments (unused)
#' @return Invisibly returns \code{x}
#' @export
print.pheno_combined <- function(x, ...) {
  cat("Combined Phenological Time Series\n")
  cat(strrep("=", 50), "\n")
  cat("Method:", x$method, "\n")

  if (!is.null(x$by)) {
    cat("Grouping:", paste(x$by, collapse = ", "), "\n")
    cat("Number of groups:", x$n_groups, "\n\n")

    # Summary of groups
    successful <- sum(sapply(x$group_results, function(g) is.null(g$error)))
    cat(sprintf("Successfully estimated: %d / %d groups\n", successful, x$n_groups))

    if (!is.null(x$combined) && nrow(x$combined) > 0) {
      cat("\nCombined series (first 10 rows):\n")
      print(head(x$combined, 10))
    }
  } else {
    if (!is.null(x$error)) {
      cat("\nError:", x$error, "\n")
    } else {
      cat(sprintf("Observations: %d\n", x$n_obs))
      cat(sprintf("Years: %d\n", x$n_years))
      cat(sprintf("Stations: %d\n", x$n_stations))

      if (!is.null(x$connectivity) && !x$connectivity$is_connected) {
        cat(sprintf("Warning: Data had %d disconnected sets (used largest)\n",
                    x$connectivity$n_sets))
      }

      cat("\nCombined time series (first 10 rows):\n")
      print(utils::head(x$combined, 10))
      if (nrow(x$combined) > 10) {
        cat(sprintf("  ... %d more rows\n", nrow(x$combined) - 10))
      }

      cat("\nStation effects (first 10):\n")
      print(head(x$station_effects[order(-abs(station_effect))], 10))

      cat("\nResidual summary:\n")
      cat(sprintf("  Mean: %.2f days\n", mean(x$residuals$residual, na.rm = TRUE)))
      cat(sprintf("  SD: %.2f days\n", sd(x$residuals$residual, na.rm = TRUE)))
      cat(sprintf("  |Residual| > 30 days: %d observations\n",
                  sum(abs(x$residuals$residual) > 30, na.rm = TRUE)))
    }
  }

  invisible(x)
}


#' Plot Method for Combined Time Series
#' @param x A \code{pheno_combined} object
#' @param type Character. "series" for combined time series, "stations" for
#'   station effects, "residuals" for residual distribution.
#' @param group For grouped analysis, which group to plot.
#' @param ... Additional arguments
#' @return A ggplot object (invisibly)
#' @export
plot.pheno_combined <- function(x, type = c("series", "stations", "residuals"),
                                 group = NULL, ...) {
  type <- match.arg(type)

  # Get data to plot
  if (!is.null(x$by) && !is.null(x$group_results)) {
    if (is.null(group)) {
      # Find first successful group
      successful <- names(which(sapply(x$group_results, function(g) is.null(g$error))))
      if (length(successful) == 0) {
        stop("No successfully estimated groups to plot", call. = FALSE)
      }
      group <- successful[1]
      message("Plotting group: ", group)
    }
    data <- x$group_results[[group]]
    title_suffix <- paste0("\n(", group, ")")
  } else {
    data <- x
    title_suffix <- ""
  }

  if (is.null(data) || !is.null(data$error)) {
    stop("No valid results to plot", call. = FALSE)
  }

  if (type == "series") {
    p <- ggplot2::ggplot(data$combined, ggplot2::aes(x = year, y = year_effect)) +
      ggplot2::geom_line(color = "steelblue", linewidth = 1) +
      ggplot2::geom_point(color = "steelblue", size = 2)

    if ("se" %in% names(data$combined)) {
      p <- p + ggplot2::geom_ribbon(
        ggplot2::aes(ymin = year_effect - 1.96 * se,
                     ymax = year_effect + 1.96 * se),
        alpha = 0.2, fill = "steelblue"
      )
    }

    p <- p +
      ggplot2::labs(
        title = paste0("Combined Phenological Time Series", title_suffix),
        subtitle = paste("Method:", x$method),
        x = "Year",
        y = "Day of Year (DOY)"
      ) +
      ggplot2::theme_minimal()

  } else if (type == "stations") {
    effects <- data$station_effects[order(station_effect)]
    effects$s_id <- factor(effects$s_id, levels = effects$s_id)

    if (nrow(effects) > 30) {
      # Show only most extreme stations
      effects <- rbind(head(effects, 15), tail(effects, 15))
    }

    p <- ggplot2::ggplot(effects, ggplot2::aes(x = station_effect, y = s_id)) +
      ggplot2::geom_col(fill = "steelblue", alpha = 0.7) +
      ggplot2::geom_vline(xintercept = 0, linetype = "dashed") +
      ggplot2::labs(
        title = paste0("Station Effects", title_suffix),
        subtitle = "Deviation from combined series (days)",
        x = "Station Effect (days)",
        y = "Station ID"
      ) +
      ggplot2::theme_minimal()

  } else {
    p <- ggplot2::ggplot(data$residuals, ggplot2::aes(x = residual)) +
      ggplot2::geom_histogram(bins = 50, fill = "steelblue", alpha = 0.7) +
      ggplot2::geom_vline(xintercept = c(-30, 30), linetype = "dashed",
                          color = "red", alpha = 0.7) +
      ggplot2::labs(
        title = paste0("Residual Distribution", title_suffix),
        subtitle = "Red lines at +/-30 days (potential outliers)",
        x = "Residual (days)",
        y = "Count"
      ) +
      ggplot2::theme_minimal()
  }

  print(p)
  invisible(p)
}

#' Minimal Seed Dataset for Synthetic Data Generation
#'
#' A small subset of phenological data suitable for use with \code{\link{pep_simulate}}
#' to generate synthetic phenological datasets. This dataset contains real structure
#' but limited scope, making it suitable for package examples and testing.
#'
#' @format A data.table with 1,319 rows and 10 variables:
#' \describe{
#'   \item{s_id}{Station identifier (factor)}
#'   \item{lon}{Longitude in decimal degrees}
#'   \item{lat}{Latitude in decimal degrees}
#'   \item{alt}{Altitude in meters}
#'   \item{genus}{Plant genus (factor)}
#'   \item{species}{Full species name (factor)}
#'   \item{phase_id}{BBCH phenological phase code (integer)}
#'   \item{year}{Observation year}
#'   \item{day}{Day of year (DOY) of the phenological event}
#'   \item{country}{Country name}
#' }
#'
#' @details
#' This dataset includes observations for:
#' \itemize{
#'   \item \strong{Species}: Triticum aestivum (wheat), Vitis vinifera (grapevine),
#'     Malus domestica (apple)
#'   \item \strong{Phases}: BBCH 10 (emergence), 60 (flowering), 65 (full flowering)
#'   \item \strong{Countries}: Austria, Germany-South
#'   \item \strong{Years}: 1990-2015
#'   \item \strong{Stations}: 30 stations
#' }
#'
#' For a full synthetic dataset, use \code{\link{pep_download}} to download
#' pre-generated synthetic data from the package repository.
#'
#' @source
#' Derived from PEP725 Pan European Phenology Database \url{https://pep725.eu/}
#'
#' @examples
#' # Load the seed dataset
#' data(pep_seed)
#'
#' # Examine structure
#' str(pep_seed)
#'
#' # Use with pep_simulate to create synthetic data
#' \donttest{
#' pep_synthetic <- pep_simulate(pep_seed)
#' }
#'
#' @seealso
#' \code{\link{pep_simulate}} for generating synthetic data,
#' \code{\link{pep_download}} for downloading full synthetic dataset
#'
#' @name pep_seed
#' @docType data
#' @keywords datasets
NULL

utils::globalVariables(c("a"))

#' Compute CTRL and SCEN Phenology Day-of-Year Values
#'
#' This function computes the mean phenological day-of-year (DOY) for two
#' phenophases (e.g. BBCH 65 and 87) for each country, using either a simple
#' warming-shift model or a GDD-based model.
#'
#' @param pep A data frame containing PEP725 phenology observations, including
#'   columns \code{country}, \code{day}, \code{year}, \code{phase_id}, and \code{genus}.
#' @param phase1 Integer. First phenophase (e.g. 65).
#' @param phase2 Integer. Second phenophase (e.g. 87).
#' @param genus_name Character. Genus to filter (e.g. "Malus").
#' @param phenology_method Either \code{"simple_shift"} or \code{"gdd_cmip6"}.
#' @param scen_warming Numeric. Warming in degC (simple shift model).
#' @param shift_per_deg Numeric. DOY shift per degC (simple shift).
#' @param cmip6_tas Optional list of daily temperature vectors--only required
#'   if using \code{"gdd_cmip6"}.
#' @param gdd_base Numeric. Base temperature for GDD model.
#' @param gdd_threshold1 Numeric. GDD threshold for phase1.
#' @param gdd_threshold2 Numeric. GDD threshold for phase2.
#' @param ctrl_years Integer vector. Years for CTRL period (default 2011:2021).
#' @param scen_years Integer vector. Years for scenario period (default 2085:2095).
#' @param calib_years Integer vector. Years for calibration period (default 1991:2020).
#' @param regions Character vector. Countries/regions to include. If NULL, all are used.
#' @param delta_Tgl Numeric. Global temperature change (degC) for scenario.
#' @param giss_data Data frame with GISS temperature data (columns: \code{year},
#'   \code{dT} or \code{Tgl}). Required for robust_shift method.
#' @author Matthias Templ
#'
#' @return A data frame containing:
#'   \item{Country}{Country name}
#'   \item{doy_start_ctrl}{Start DOY for CTRL period}
#'   \item{doy_end_ctrl}{End DOY for CTRL period}
#'   \item{doy_start_scen}{Start DOY for SCEN period}
#'   \item{doy_end_scen}{End DOY for SCEN period}
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Use Alpine subset for faster computation
#' regions <- c("Austria", "Switzerland")
#' pep_alpine <- pep[country %in% regions]
#'
#' # Simple shift method
#' d1 <- get_phenology_doys(
#'   pep = pep_alpine,
#'   phase1 = 65,
#'   phase2 = 87,
#'   genus_name = "Malus",
#'   regions = regions,
#'   phenology_method = 'simple_shift'
#' )
#' d1
#' }
#'
#' @export
#'
get_phenology_doys <- function(
    pep,
    phase1,
    phase2,
    genus_name,
    phenology_method = c("robust_shift", "simple_shift", "gdd_cmip6"),
    ctrl_years = 2011:2021,       # CTRL for Tgl mean (PGW setup)
    scen_years = 2085:2095,       # PGW-Periode
    calib_years = 1991:2020,       # Calibration period for regression
    scen_warming = 3,
    shift_per_deg = -4,
    cmip6_tas = NULL,
    gdd_base = 5,
    gdd_threshold1 = 150,
    gdd_threshold2 = 350,
    regions = NULL,
    delta_Tgl = 2.09,        # numeric or "estimate"
    giss_data = NULL         # must be provided for robust_shift
){

  phenology_method <- match.arg(phenology_method)

  # ------------------------------------------------------------------
  # 1. Filter genus + regions
  # ------------------------------------------------------------------
  pep <- pep[pep$genus == genus_name, ]

  if (!is.null(regions)) {
    pep <- pep[pep$country %in% regions, ]
  }

  if (nrow(pep) == 0)
    stop("No data left after filtering for genus and selected regions.")

  p1name <- paste0("phase_", phase1)
  p2name <- paste0("phase_", phase2)

  # ================================================================
  # SIMPLE_SHIFT & GDD_CMIP6
  # ================================================================
  if (phenology_method %in% c("robust_shift", "simple_shift", "gdd_cmip6")) {

    ctrl <- pep %>%
      dplyr::filter(
        year >= min(ctrl_years), year <= max(ctrl_years),
        phase_id %in% c(phase1, phase2)
      ) %>%
      dplyr::group_by(country, phase_id) %>%
      dplyr::summarize(d_mean = round(mean(day)), .groups = "drop") %>%
      tidyr::pivot_wider(
        names_from  = phase_id,
        values_from = d_mean,
        names_prefix = "phase_"
      ) %>%
      dplyr::rename(Country = country) %>%
      stats::na.omit()

    # -----------------------------
    # simple shift
    # -----------------------------
    if (phenology_method == "simple_shift") {

      shift_days <- scen_warming * shift_per_deg

      ctrl <- ctrl %>%
        dplyr::mutate(
          doy_start_ctrl = .data[[p1name]],
          doy_end_ctrl   = .data[[p2name]],
          doy_start_scen = pmax(1, doy_start_ctrl + shift_days),
          doy_end_scen   = pmin(366, doy_end_ctrl + shift_days)
        )

      return(ctrl %>% dplyr::select(
        Country, doy_start_ctrl, doy_end_ctrl, doy_start_scen, doy_end_scen))
    }

    # -----------------------------
    # GDD CMIP6
    # -----------------------------
    if (phenology_method == "gdd_cmip6") {

      if (is.null(cmip6_tas))
        stop("cmip6_tas must be supplied for gdd_cmip6 method.")

      compute_doy_gdd <- function(tvec, base, thr) {
        gdd <- pmax(tvec - base, 0)
        cum <- cumsum(gdd)
        idx <- which(cum >= thr)[1]
        if (is.na(idx)) idx <- length(tvec)
        idx
      }

      ctrl <- ctrl %>%
        dplyr::rowwise() %>%
        dplyr::mutate(
          doy_start_ctrl = .data[[p1name]],
          doy_end_ctrl   = .data[[p2name]],
          doy_start_scen =
            compute_doy_gdd(cmip6_tas[[Country]], gdd_base, gdd_threshold1),
          doy_end_scen   =
            compute_doy_gdd(cmip6_tas[[Country]], gdd_base, gdd_threshold2)
        ) %>%
        dplyr::ungroup()

      return(ctrl %>% dplyr::select(
        Country, doy_start_ctrl, doy_end_ctrl, doy_start_scen, doy_end_scen))
    }
  }

  # ================================================================
  # ROBUST_SHIFT (DOY ~ Tgl from GISS)
  # ================================================================
  if (phenology_method == "robust_shift") {

    if (is.null(giss_data))
      stop("Please supply global temperature data via giss_data = ...")



    # --------------------------------------------------------------
    # Validate and automatically create Tgl if missing
    # --------------------------------------------------------------
    if (!("year" %in% names(giss_data)))
      stop("giss_data must contain a 'year' column.")

    if ("Tgl" %in% names(giss_data)) {
      giss_sub <- giss_data
    } else if ("dT" %in% names(giss_data)) {
      message("No column 'Tgl' found. Creating Tgl = dT + 14 (absolute global temp).")
      giss_sub <- giss_data %>%
        dplyr::mutate(Tgl = dT + 14)
    } else {
      stop("giss_data must contain either a 'Tgl' or 'dT' column.")
    }

    # Restrict to useful years
    giss_sub <- giss_sub[giss_sub$year >= 1961, ] %>%
      dplyr::arrange(year)

    Tgl_CTRL <- mean(giss_sub$Tgl[giss_sub$year %in% ctrl_years], na.rm = TRUE)

    # -------------------------
    # delta_Tgl logic
    # -------------------------
    if (is.character(delta_Tgl) && delta_Tgl == "estimate") {

      Tgl_SCEN_est <- mean(giss_sub$Tgl[giss_sub$year %in% scen_years], na.rm = TRUE)
      delta_val <- Tgl_SCEN_est - Tgl_CTRL

      message(sprintf(
        "Estimated delta_Tgl from GISS: %.3f K.", delta_val))

    } else if (is.numeric(delta_Tgl)) {
      delta_val <- delta_Tgl
    } else {
      stop('delta_Tgl must be numeric or "estimate".')
    }

    Tgl_SCEN <- Tgl_CTRL + delta_val

    # -------------------------
    # prep PEP aggregated DOYs
    # -------------------------
    agg <- pep %>%
      dplyr::filter(phase_id %in% c(phase1, phase2)) %>%
      dplyr::group_by(country, phase_id, year) %>%
      dplyr::summarize(mean_DOY = mean(day), .groups = "drop") %>%
      dplyr::left_join(giss_sub, by = "year")

    # -------------------------
    # Robust regression: DOY ~ Tgl (auf calib_years)
    # -------------------------
    robust_fit_Tgl <- function(df) {
      sub <- df %>%
        dplyr::filter(
          year >= min(calib_years), year <= max(calib_years),
          is.finite(mean_DOY),
          is.finite(Tgl)
        )

      if (nrow(sub) < 3)
        return(tibble::tibble(a = NA_real_, b = NA_real_))

      mod <- tryCatch(
        robustbase::lmrob(mean_DOY ~ Tgl, data = sub),
        error = function(e) NULL
      )

      if (is.null(mod))
        return(tibble::tibble(a = NA_real_, b = NA_real_))

      tibble::tibble(
        a = unname(coef(mod)[1]),
        b = unname(coef(mod)[2])
      )
    }

    fits <- agg %>%
      dplyr::group_by(country, phase_id) %>%
      dplyr::group_modify(~ robust_fit_Tgl(.x)) %>%
      dplyr::ungroup() %>%
      dplyr::filter(!is.na(a))

    # Predictions
    fits <- fits %>%
      dplyr::mutate(
        doy_ctrl = a + b * Tgl_CTRL,
        doy_scen = a + b * Tgl_SCEN
      )

    safe_mean <- function(x)
      if (all(is.na(x))) NA_real_ else mean(x, na.rm = TRUE)

    # -------------------------
    # reshape to start/end DOYs
    # -------------------------
    result <- fits %>%
      dplyr::mutate(
        doy_start_ctrl = ifelse(phase_id == phase1, doy_ctrl, NA_real_),
        doy_end_ctrl   = ifelse(phase_id == phase2, doy_ctrl, NA_real_),
        doy_start_scen = ifelse(phase_id == phase1, doy_scen, NA_real_),
        doy_end_scen   = ifelse(phase_id == phase2, doy_scen, NA_real_)
      ) %>%
      dplyr::group_by(country) %>%
      dplyr::summarize(
        doy_start_ctrl = round(safe_mean(doy_start_ctrl)),
        doy_end_ctrl   = round(safe_mean(doy_end_ctrl)),
        doy_start_scen = round(safe_mean(doy_start_scen)),
        doy_end_scen   = round(safe_mean(doy_end_scen)),
        .groups = "drop"
      ) %>%
      dplyr::rename(Country = country) %>%
      # ensure correct order and then constrain to [1,366]
      dplyr::mutate(
        # CTRL
        tmp_start_ctrl = pmin(doy_start_ctrl, doy_end_ctrl),
        tmp_end_ctrl   = pmax(doy_start_ctrl, doy_end_ctrl),
        # SCEN
        tmp_start_scen = pmin(doy_start_scen, doy_end_scen),
        tmp_end_scen   = pmax(doy_start_scen, doy_end_scen)
      ) %>%
      dplyr::transmute(
        Country,
        doy_start_ctrl = pmin(366, pmax(1, tmp_start_ctrl)),
        doy_end_ctrl   = pmin(366, pmax(1, tmp_end_ctrl)),
        doy_start_scen = pmin(366, pmax(1, tmp_start_scen)),
        doy_end_scen   = pmin(366, pmax(1, tmp_end_scen))
      )

    return(result)
  }
}

# get_phenology_doys <- function(
#     pep,
#     phase1,
#     phase2,
#     genus_name,
#     phenology_method = c("simple_shift", "gdd_cmip6"),
#     scen_warming = 3,
#     shift_per_deg = -4,
#     cmip6_tas = NULL,
#     gdd_base = 5,
#     gdd_threshold1 = 150,
#     gdd_threshold2 = 350
# ){
#
#   phenology_method <- match.arg(phenology_method)
#
#   # Filter for genus
#   pep <- pep[pep$genus == genus_name, ]
#
#   p1name <- paste0("phase_", phase1)
#   p2name <- paste0("phase_", phase2)
#
#   # Compute CTRL mean DOYs per country
#   ctrl <- pep %>%
#     dplyr::filter(
#       year >= 2011, year <= 2021,
#       phase_id %in% c(phase1, phase2)
#     ) %>%
#     dplyr::group_by(country, phase_id) %>%
#     dplyr::summarize(d_mean = round(mean(day)), .groups = "drop") %>%
#     tidyr::pivot_wider(
#       names_from = phase_id,
#       values_from = d_mean,
#       names_prefix = "phase_"
#     ) %>%
#     dplyr::rename(Country = country) %>%
#     na.omit()
#
#   # Compute SCEN DOYs
#   if (phenology_method == "simple_shift") {
#
#     shift_days <- scen_warming * shift_per_deg
#
#     ctrl <- ctrl %>%
#       dplyr::mutate(
#         doy_start_ctrl = .data[[p1name]],
#         doy_end_ctrl   = .data[[p2name]],
#         doy_start_scen = pmax(1, doy_start_ctrl + shift_days),
#         doy_end_scen   = pmin(366, doy_end_ctrl + shift_days)
#       )
#
#   } else {
#
#     if (is.null(cmip6_tas)) stop("CMIP6 tas list is required for gdd_cmip6 method.")
#
#     compute_doy_gdd <- function(tvec, base, thr) {
#       gdd <- pmax(tvec - base, 0)
#       cum <- cumsum(gdd)
#       idx <- which(cum >= thr)[1]
#       if (is.na(idx)) idx <- length(tvec)
#       idx
#     }
#
#     ctrl <- ctrl %>%
#       dplyr::rowwise() %>%
#       dplyr::mutate(
#         doy_start_ctrl = .data[[p1name]],
#         doy_end_ctrl   = .data[[p2name]],
#         doy_start_scen =
#           compute_doy_gdd(cmip6_tas[[Country]], gdd_base, gdd_threshold1),
#         doy_end_scen =
#           compute_doy_gdd(cmip6_tas[[Country]], gdd_base, gdd_threshold2)
#       ) %>%
#       dplyr::ungroup()
#   }
#
#   return(ctrl %>% dplyr::select(Country, doy_start_ctrl, doy_end_ctrl,
#                                 doy_start_scen, doy_end_scen))
# }

# Prevent R CMD check notes
utils::globalVariables(c("day", "year", "s_id", "phase_id", "genus", "species",
                         "lon", "lat", "country", "n_events", "event_type",
                         "doy_1", "doy_2", "gap_days", "is_second_event",
                         "median_doy", "mad_doy", "z_score", "month",
                         "expected_window_start", "expected_window_end",
                         "..cols_keep", "n_total", "proportion",
                         "q05", "q95", "doy_max", "doy_min", "month_name",
                         "month_label"))

#' Detect Second Flowering and Other Repeated Phenological Events
#'
#' Identifies potential second flowering events and other irregular repeated
#' phenological observations. These may represent biologically meaningful
#' responses to climate disruption rather than data errors.
#'
#' @param pep A \code{pep} object or data.table with phenological observations.
#' @param phases Integer vector of phase IDs to analyze. Default is
#'   \code{c(60, 65)} for flowering phases. Use \code{NULL} to analyze all phases.
#' @param method Character. Detection method:
#'   \describe{
#'     \item{"late_season"}{(Default) Flag observations occurring after the
#'       expected phenological window (DOY > late_threshold).}
#'     \item{"multiple_per_year"}{Detect multiple observations of the same
#'       phase at the same station within a single year.}
#'     \item{"both"}{Apply both methods.}
#'   }
#' @param late_threshold Integer. DOY threshold for "late season" events.
#'   Default is 250 (~September 7). Events after this date for spring phases
#'   are flagged as potential second events.
#' @param gap_threshold Integer. For "multiple_per_year" method, minimum days
#'   between observations to consider them separate events. Default is 60.
#' @param reference_period Integer vector of years to calculate expected
#'   phenological windows. Default is \code{1991:2020}.
#' @param z_threshold Numeric. Z-score threshold for identifying late events
#'   relative to the reference period. Default is 3.
#' @param min_obs Integer. Minimum observations in reference period to calculate
#'   expected windows. Default is 10.
#'
#' @return A list of class \code{second_events} containing:
#'   \describe{
#'     \item{events}{Data.table of detected second/repeated events with columns:
#'       s_id, species, phase_id, year, day, event_type, expected timing info}
#'     \item{summary}{Summary statistics by species, phase, and event type}
#'     \item{by_year}{Annual counts of detected events}
#'     \item{by_month}{Monthly distribution of events}
#'     \item{total_by_year}{Total observations per year (for relative scaling in plots)}
#'     \item{method}{Detection method used}
#'     \item{params}{Parameters used for detection}
#'   }
#'
#' @details
#' Second flowering and other repeated phenological events are increasingly
#' reported as climate patterns become more irregular. These events may indicate:
#' \itemize{
#'   \item Disrupted coordination between environmental cues and development
#'   \item Response to unusual autumn warmth after summer dormancy
#'   \item Failure of normal seasonal inhibition mechanisms
#' }
#'
#' The function uses two complementary approaches:
#' \enumerate{
#'   \item \strong{Late season detection}: Identifies observations occurring
#'     well after the normal phenological window (e.g., flowering in autumn)
#'   \item \strong{Multiple events}: Finds stations reporting the same phase
#'     multiple times in one year with sufficient gap between observations
#' }
#'
#' @section Interpreting Results:
#' Not all detected events are necessarily "true" second flowering. Consider:
#' \itemize{
#'   \item \strong{High confidence}: Same station reports phase twice with
#'     60+ day gap; late observation is clearly outside normal window
#'   \item \strong{Medium confidence}: Single late observation at a station;
#'     could be second event or data entry error
#'   \item \strong{Requires verification}: Check original data sources,
#'     geographic context, species biology
#' }
#'
#' @examples
#' \donttest{
#' pep <- pep_download()
#'
#' # Use grapevine flowering in Alpine region (long records, second events possible)
#' pep_subset <- pep[species == "Vitis vinifera" & phase_id %in% c(60, 65) &
#'                   country %in% c("Germany-South", "Switzerland", "Austria")]
#'
#' if (nrow(pep_subset) > 0) {
#'   # Method 1: Detect late-season flowering events (default)
#'   second <- pep_second_events(pep_subset, phases = c(60, 65))
#'   print(second)
#'   summary(second)
#'
#'   # Plot from 1980 onwards for clearer trends
#'   plot(second, from_year = 1980)
#'   plot(second, type = "overview", scale = "relative", from_year = 1980)
#'
#'   # Method 2: Detect repeated observations at same station
#'   repeated <- pep_second_events(pep_subset, method = "multiple_per_year")
#'   print(repeated)
#'   plot(repeated, type = "timeline", from_year = 1980)
#'
#'   # Method 3: Combine both detection methods
#'   all_events <- pep_second_events(pep_subset, method = "both")
#'   summary(all_events)
#'   plot(all_events)
#'   plot(all_events, scale = "relative")
#' }
#' }
#'
#' @seealso \code{\link{pep_flag_outliers}} for general outlier detection,
#'   \code{\link{pep_plot_outliers}} for outlier visualization
#'
#' @references
#' Related to emerging research on phenological irregularity and climate
#' disruption of seasonal timing.
#'
#' @author Matthias Templ
#' @export
pep_second_events <- function(pep,
                                  phases = c(60, 65),
                                  method = c("late_season", "multiple_per_year", "both"),
                                  late_threshold = 250,
                                  gap_threshold = 60,
                                  reference_period = 1991:2020,
                                  z_threshold = 3,
                                  min_obs = 10) {

 method <- match.arg(method)

  # Input validation
  if (!inherits(pep, "data.frame")) {
    stop("'pep' must be a data.frame or data.table", call. = FALSE)
  }

  # Convert to data.table and remove pep class to avoid subsetting issues
  dt <- data.table::as.data.table(pep)
  class(dt) <- c("data.table", "data.frame")

  # Check required columns
  required <- c("s_id", "year", "day", "phase_id")
  missing <- setdiff(required, names(dt))
  if (length(missing) > 0) {
    stop("Missing required columns: ", paste(missing, collapse = ", "), call. = FALSE)
  }

  # Filter to specified phases
  if (!is.null(phases)) {
    dt <- dt[phase_id %in% phases]
    if (nrow(dt) == 0) {
      stop("No observations found for specified phases", call. = FALSE)
    }
  }

  # Calculate reference statistics for each group
  ref_stats <- calculate_reference_windows(dt, reference_period, min_obs)

  # Initialize results
  events_late <- NULL
  events_multiple <- NULL

  # ============================================================
  # Method 1: Late season detection
  # ============================================================
  if (method %in% c("late_season", "both")) {
    events_late <- detect_late_season_events(dt, ref_stats, late_threshold, z_threshold)
  }

  # ============================================================
  # Method 2: Multiple observations per year
  # ============================================================
  if (method %in% c("multiple_per_year", "both")) {
    events_multiple <- detect_multiple_events(dt, gap_threshold)
  }

  # Combine results
  if (method == "both") {
    # Combine and deduplicate
    all_events <- data.table::rbindlist(list(events_late, events_multiple), fill = TRUE)
    # Remove duplicates (same observation flagged by both methods)
    all_events <- unique(all_events, by = c("s_id", "year", "day", "phase_id"))
  } else if (method == "late_season") {
    all_events <- events_late
  } else {
    all_events <- events_multiple
  }

  # Add month column
  if (nrow(all_events) > 0) {
    all_events[, month := ceiling(day / 30.44)]
    all_events[month > 12, month := 12]
  }

  # Create summary statistics
  summary_stats <- create_event_summary(all_events)

  # Calculate total observations per year for relative scaling
  total_by_year <- dt[, .(n_total = .N), by = year][order(year)]

  # Create result object
  result <- list(
    events = all_events,
    summary = summary_stats$by_species,
    by_year = summary_stats$by_year,
    by_month = summary_stats$by_month,
    total_by_year = total_by_year,
    reference_stats = ref_stats,
    method = method,
    params = list(
      phases = phases,
      late_threshold = late_threshold,
      gap_threshold = gap_threshold,
      reference_period = reference_period,
      z_threshold = z_threshold
    )
  )

  class(result) <- c("second_events", "list")
  result
}


#' @keywords internal
calculate_reference_windows <- function(dt, reference_period, min_obs) {
  # Calculate expected DOY statistics from reference period
  ref_data <- dt[year %in% reference_period]

  # Group by species and phase
  group_cols <- intersect(c("genus", "species", "phase_id"), names(dt))

  if (length(group_cols) == 0) {
    group_cols <- "phase_id"
  }

  ref_stats <- ref_data[, .(
    n_obs = .N,
    median_doy = as.numeric(median(day, na.rm = TRUE)),
    mad_doy = as.numeric(mad(day, na.rm = TRUE, constant = 1.4826)),
    q05 = as.numeric(quantile(day, 0.05, na.rm = TRUE)),
    q95 = as.numeric(quantile(day, 0.95, na.rm = TRUE)),
    min_doy = as.numeric(min(day, na.rm = TRUE)),
    max_doy = as.numeric(max(day, na.rm = TRUE))
  ), by = group_cols]

  # Only keep groups with sufficient observations
  ref_stats <- ref_stats[n_obs >= min_obs]

  # Calculate expected window (use median +/- 3*MAD or q05-q95)
  ref_stats[, expected_window_start := pmax(q05, median_doy - 3 * mad_doy)]
  ref_stats[, expected_window_end := pmin(q95, median_doy + 3 * mad_doy)]

  ref_stats
}


#' @keywords internal
detect_late_season_events <- function(dt, ref_stats, late_threshold, z_threshold) {
  # Merge reference statistics
  group_cols <- intersect(c("genus", "species", "phase_id"), names(dt))
  if (length(group_cols) == 0) group_cols <- "phase_id"

  # Merge with reference stats
  dt_merged <- merge(dt, ref_stats, by = group_cols, all.x = TRUE)

  # Calculate z-score relative to reference
  dt_merged[, z_score := (day - median_doy) / mad_doy]

  # Flag late season events:
  # 1. DOY > late_threshold AND
  # 2. (z_score > z_threshold OR DOY > expected_window_end)
  dt_merged[, is_second_event :=
              !is.na(median_doy) &
              day > late_threshold &
              (z_score > z_threshold | day > expected_window_end)]

  # Extract flagged events
  events <- dt_merged[is_second_event == TRUE]

  if (nrow(events) > 0) {
    events[, event_type := "late_season"]

    # Keep relevant columns
    cols_keep <- c("s_id", "year", "day", "phase_id", "event_type",
                   "median_doy", "mad_doy", "z_score",
                   "expected_window_start", "expected_window_end")
    cols_keep <- intersect(cols_keep, names(events))

    # Add species info if available
    if ("species" %in% names(events)) cols_keep <- c(cols_keep, "species")
    if ("genus" %in% names(events)) cols_keep <- c(cols_keep, "genus")
    if ("lon" %in% names(events)) cols_keep <- c(cols_keep, "lon")
    if ("lat" %in% names(events)) cols_keep <- c(cols_keep, "lat")
    if ("country" %in% names(events)) cols_keep <- c(cols_keep, "country")

    events <- events[, ..cols_keep]
  }

  events
}


#' @keywords internal
detect_multiple_events <- function(dt, gap_threshold) {
  # Find stations with multiple observations of same phase in same year
  group_cols <- c("s_id", "phase_id", "year")
  if ("species" %in% names(dt)) group_cols <- c(group_cols, "species")

  # Count observations per station-phase-year
  multi <- dt[, .(
    n_events = .N,
    doy_min = min(day, na.rm = TRUE),
    doy_max = max(day, na.rm = TRUE),
    doy_list = list(sort(day))
  ), by = group_cols]

  # Filter to those with multiple observations
  multi <- multi[n_events >= 2]

  # Check if gap between observations exceeds threshold
  multi[, gap_days := doy_max - doy_min]
  multi <- multi[gap_days >= gap_threshold]

  if (nrow(multi) == 0) {
    return(data.table::data.table())
  }

  # Expand back to individual observations and mark as second events
  # Get the later observations (second, third, etc.)
  result_list <- list()

  for (i in seq_len(nrow(multi))) {
    row <- multi[i]
    doys <- unlist(row$doy_list)

    # All observations after the first one are "second events"
    if (length(doys) > 1) {
      second_doys <- doys[-1]  # All but the first

      for (d in second_doys) {
        # Find the original observation
        filter_expr <- dt$s_id == row$s_id &
          dt$phase_id == row$phase_id &
          dt$year == row$year &
          dt$day == d

        if ("species" %in% names(dt)) {
          filter_expr <- filter_expr & dt$species == row$species
        }

        obs <- dt[filter_expr][1]  # Take first match if duplicates

        if (nrow(obs) > 0) {
          obs[, event_type := "multiple_per_year"]
          obs[, doy_1 := doys[1]]  # First (normal) observation
          obs[, doy_2 := d]        # This (second) observation
          obs[, gap_days := d - doys[1]]
          obs[, n_events := length(doys)]
          result_list[[length(result_list) + 1]] <- obs
        }
      }
    }
  }

  if (length(result_list) > 0) {
    events <- data.table::rbindlist(result_list, fill = TRUE)
  } else {
    events <- data.table::data.table()
  }

  events
}


#' @keywords internal
create_event_summary <- function(events) {
  if (nrow(events) == 0) {
    return(list(
      by_species = data.table::data.table(),
      by_year = data.table::data.table(),
      by_month = data.table::data.table()
    ))
  }

  # Summary by species
  group_cols <- intersect(c("species", "phase_id", "event_type"), names(events))
  if (length(group_cols) > 0) {
    by_species <- events[, .(
      n_events = .N,
      n_stations = data.table::uniqueN(s_id),
      n_years = data.table::uniqueN(year),
      mean_doy = mean(day, na.rm = TRUE),
      min_doy = min(day, na.rm = TRUE),
      max_doy = max(day, na.rm = TRUE)
    ), by = group_cols][order(-n_events)]
  } else {
    by_species <- data.table::data.table()
  }

  # Summary by year
  by_year <- events[, .(n_events = .N), by = year][order(year)]

  # Summary by month
  if ("month" %in% names(events)) {
    by_month <- events[, .(n_events = .N), by = month][order(month)]
  } else {
    by_month <- data.table::data.table()
  }

  list(
    by_species = by_species,
    by_year = by_year,
    by_month = by_month
  )
}


#' Print Method for Second Events Detection
#'
#' @param x A \code{second_events} object
#' @param ... Additional arguments (unused)
#' @return Invisibly returns \code{x}
#' @export
print.second_events <- function(x, ...) {
  cat("Second/Repeated Phenological Events Detection\n")
  cat(strrep("=", 50), "\n\n")

  cat("Method:", x$method, "\n")
  cat("Phases analyzed:", paste(x$params$phases, collapse = ", "), "\n")

  if (x$method %in% c("late_season", "both")) {
    cat("Late threshold: DOY", x$params$late_threshold,
        sprintf("(~%s)", format(as.Date(x$params$late_threshold - 1, origin = "2000-01-01"), "%b %d")), "\n")
  }
  if (x$method %in% c("multiple_per_year", "both")) {
    cat("Gap threshold:", x$params$gap_threshold, "days\n")
  }

  cat("\n")
  n_events <- nrow(x$events)
  cat(sprintf("Total events detected: %d\n", n_events))

  if (n_events > 0) {
    cat(sprintf("Unique stations: %d\n", data.table::uniqueN(x$events$s_id)))
    cat(sprintf("Year range: %d - %d\n", min(x$events$year), max(x$events$year)))

    if ("event_type" %in% names(x$events)) {
      cat("\nBy event type:\n")
      print(table(x$events$event_type))
    }

    if (nrow(x$by_month) > 0) {
      cat("\nBy month:\n")
      month_names <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun",
                       "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
      month_tbl <- x$by_month
      if (all(month_tbl$month %in% 1:12)) {
        month_tbl$month_name <- month_names[month_tbl$month]
        print(month_tbl[, .(month_name, n_events)])
      }
    }

    if ("species" %in% names(x$events)) {
      cat("\nTop species:\n")
      species_counts <- x$events[, .(n = .N), by = species][order(-n)]
      print(head(species_counts, 10))
    }
  }

  invisible(x)
}


#' Summary Method for Second Events Detection
#'
#' @param object A \code{second_events} object
#' @param ... Additional arguments (unused)
#' @return The summary data.table (invisibly)
#' @export
summary.second_events <- function(object, ...) {
  cat("Second/Repeated Phenological Events - Summary\n")
  cat(strrep("=", 50), "\n\n")

  if (nrow(object$events) == 0) {
    cat("No events detected.\n")
    return(invisible(NULL))
  }

  cat("Events by species and phase:\n\n")
  print(object$summary)

  cat("\n\nAnnual trend:\n")
  print(object$by_year)

  invisible(object$summary)
}


#' Plot Method for Second Events Detection
#'
#' @param x A \code{second_events} object
#' @param type Character. Plot type: "overview", "timeline", "seasonal", "map"
#' @param scale Character. Scale for y-axis in timeline/overview plots:
#'   \describe{
#'     \item{"absolute"}{(Default) Show raw counts of second events per year.}
#'     \item{"relative"}{Show proportion of second events relative to total
#'       observations per year. This accounts for varying data availability
#'       over time and reveals whether second events are becoming proportionally
#'       more common.}
#'   }
#' @param from_year Integer. If specified, only show data from this year onwards
#'   in timeline and overview plots. Useful for focusing on recent trends.
#'   Default is \code{NULL} (show all years).
#' @param ... Additional arguments
#' @return A ggplot object (invisibly)
#' @export
#' @import ggplot2
plot.second_events <- function(x, type = c("overview", "timeline", "seasonal", "map"),
                               scale = c("absolute", "relative"),
                               from_year = NULL, ...) {
  type <- match.arg(type)
  scale <- match.arg(scale)

  events <- x$events

  if (nrow(events) == 0) {
    message("No events to plot")
    return(invisible(NULL))
  }


  # Filter by from_year if specified
  if (!is.null(from_year)) {
    events <- events[year >= from_year]
    if (nrow(events) == 0) {
      message("No events to plot after filtering to from_year = ", from_year)
      return(invisible(NULL))
    }
  }

  if (type == "overview") {
    # Multi-panel overview
    p1 <- ggplot(events, aes(x = day)) +
      geom_histogram(bins = 50, fill = "darkred", alpha = 0.7) +
      geom_vline(xintercept = x$params$late_threshold, linetype = "dashed", color = "black") +
      labs(title = "DOY Distribution of Second Events",
           x = "Day of Year", y = "Count") +
      theme_minimal()

    # Prepare data for timeline panel based on scale
    # Filter by_year data if from_year specified
    by_year_filtered <- x$by_year
    total_by_year_filtered <- x$total_by_year
    if (!is.null(from_year)) {
      by_year_filtered <- by_year_filtered[year >= from_year]
      if (!is.null(total_by_year_filtered)) {
        total_by_year_filtered <- total_by_year_filtered[year >= from_year]
      }
    }

    if (scale == "relative" && !is.null(total_by_year_filtered)) {
      # Merge with total observations to calculate proportion
      plot_data <- merge(by_year_filtered, total_by_year_filtered, by = "year", all.x = TRUE)
      plot_data[, proportion := n_events / n_total * 100]  # as percentage
      p2 <- ggplot(plot_data, aes(x = year, y = proportion)) +
        geom_col(fill = "darkred", alpha = 0.7) +
        geom_smooth(method = "loess", se = FALSE, color = "black", linewidth = 0.8) +
        labs(title = "Relative Second Events Over Time",
             x = "", y = "% of Total Observations") +
        theme_minimal()
    } else {
      p2 <- ggplot(by_year_filtered, aes(x = year, y = n_events)) +
        geom_col(fill = "darkred", alpha = 0.7) +
        geom_smooth(method = "loess", se = FALSE, color = "black", linewidth = 0.8) +
        labs(title = "Second Events Over Time",
             x = "", y = "Number of Events") +
        theme_minimal()
    }

    # Recalculate monthly distribution from filtered events
    if (nrow(events) > 0 && "month" %in% names(events)) {
      by_month_filtered <- events[, .(n_events = .N), by = month][order(month)]
      month_abbr <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun",
                      "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
      plot_month <- data.table::copy(by_month_filtered)
      plot_month[, month_label := month_abbr[month]]
      plot_month[, month_label := factor(month_label, levels = month_abbr)]
      p3 <- ggplot(plot_month, aes(x = month_label, y = n_events)) +
        geom_col(fill = "darkred", alpha = 0.7) +
        scale_x_discrete(drop = TRUE) +
        labs(title = "Monthly Distribution",
             x = "", y = "Count") +
        theme_minimal()
    } else {
      p3 <- ggplot() + theme_void()
    }

    if ("species" %in% names(events)) {
      top_species <- events[, .(n = .N), by = species][order(-n)][1:min(10, .N)]
      top_species[, species := factor(species, levels = rev(species))]
      p4 <- ggplot(top_species, aes(x = n, y = species)) +
        geom_col(fill = "darkred", alpha = 0.7) +
        labs(title = "Top Species",
             x = "Number of Events", y = "") +
        theme_minimal()
    } else {
      p4 <- ggplot() + theme_void()
    }

    combined <- patchwork::wrap_plots(p1, p2, p3, p4, ncol = 2)
    print(combined)
    return(invisible(combined))
  }

  if (type == "timeline") {
    # Filter by_year data if from_year specified
    by_year_filtered <- x$by_year
    total_by_year_filtered <- x$total_by_year
    if (!is.null(from_year)) {
      by_year_filtered <- by_year_filtered[year >= from_year]
      if (!is.null(total_by_year_filtered)) {
        total_by_year_filtered <- total_by_year_filtered[year >= from_year]
      }
    }

    if (scale == "relative" && !is.null(total_by_year_filtered)) {
      # Merge with total observations to calculate proportion
      plot_data <- merge(by_year_filtered, total_by_year_filtered, by = "year", all.x = TRUE)
      plot_data[, proportion := n_events / n_total * 100]  # as percentage
      p <- ggplot(plot_data, aes(x = year, y = proportion)) +
        geom_line(color = "darkred", linewidth = 1) +
        geom_point(color = "darkred", size = 2) +
        geom_smooth(method = "loess", se = TRUE, alpha = 0.2, color = "black") +
        labs(title = "Trend in Second Flowering Events (Relative)",
             subtitle = sprintf("Method: %s, Late threshold: DOY %d | Proportion of total observations",
                                x$method, x$params$late_threshold),
             x = "Year", y = "% of Total Observations") +
        theme_minimal()
    } else {
      p <- ggplot(by_year_filtered, aes(x = year, y = n_events)) +
        geom_line(color = "darkred", linewidth = 1) +
        geom_point(color = "darkred", size = 2) +
        geom_smooth(method = "loess", se = TRUE, alpha = 0.2, color = "black") +
        labs(title = "Trend in Second Flowering Events",
             subtitle = sprintf("Method: %s, Late threshold: DOY %d",
                                x$method, x$params$late_threshold),
             x = "Year", y = "Number of Events") +
        theme_minimal()
    }

    print(p)
    return(invisible(p))
  }

  if (type == "seasonal") {
    if (!"month" %in% names(events)) {
      events[, month := ceiling(day / 30.44)]
    }
    month_abbr <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun",
                    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
    events[, month_label := month_abbr[month]]
    events[, month_label := factor(month_label, levels = month_abbr)]

    p <- ggplot(events, aes(x = month_label)) +
      geom_bar(fill = "darkred", alpha = 0.7) +
      scale_x_discrete(drop = TRUE) +
      labs(title = "Seasonal Distribution of Second Events",
           subtitle = "When do 'second flowering' events occur?",
           x = "Month", y = "Number of Events") +
      theme_minimal() +
      theme(axis.text.x = element_text(angle = 45, hjust = 1))

    if ("species" %in% names(events) && length(unique(events$species)) <= 6) {
      p <- p + facet_wrap(~species)
    }

    print(p)
    return(invisible(p))
  }

  if (type == "map") {
    if (!all(c("lon", "lat") %in% names(events))) {
      stop("Map plot requires 'lon' and 'lat' columns", call. = FALSE)
    }

    station_summary <- events[, .(
      n_events = .N,
      years = paste(sort(unique(year)), collapse = ", "),
      lon = lon[1],
      lat = lat[1]
    ), by = s_id]

    p <- ggplot(station_summary, aes(x = lon, y = lat)) +
      geom_point(aes(size = n_events, color = n_events), alpha = 0.6) +
      scale_color_viridis_c(option = "plasma") +
      scale_size_continuous(range = c(2, 8)) +
      coord_quickmap() +
      labs(title = "Geographic Distribution of Second Events",
           subtitle = sprintf("%d stations with %d total events",
                              nrow(station_summary), sum(station_summary$n_events)),
           x = "Longitude", y = "Latitude",
           size = "Events", color = "Events") +
      theme_minimal()

    print(p)
    return(invisible(p))
  }
}

---
title: "Phenological Analysis"
author:
  - Matthias Templ^[University of Applied Sciences Northwestern Switzerland (FHNW)]
  - Barbara Templ^[Swiss Federal Institute for Forest, Snow and Landscape Research (WSL)]
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{Phenological Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

*Vignette 3 of 4 for the **pep725** R package.
The most current version is available on
[GitHub](https://github.com/matthias-da/pep725) and
[CRAN](https://CRAN.R-project.org/package=pep725).*

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

This vignette teaches you how to perform core phenological analyses using the
**pep725** package. By the end, you'll be able to:

1. Calculate **phenological normals** (long-term baselines)
2. Detect **anomalies** (years that deviate from normal)
3. Assess **data quality** before analysis
4. Analyze and visualize **trends** over time
5. Link phenology to **climate variables**

These analyses form the foundation of phenological research, whether you're
studying climate change impacts, agricultural timing, or ecosystem dynamics.

### What You'll Learn

| Analysis | What It Tells You | Key Function |
|----------|-------------------|--------------|
| Normals | "What is typical?" | `pheno_normals()` |
| Anomalies | "Which years are unusual?" | `pheno_anomaly()` |
| Quality | "Can I trust this data?" | `pep_quality()` |
| Trends | "Is timing changing?" | `pheno_trend_turning()` |
| Climate links | "Why is it changing?" | `pheno_regional()` |

### Prerequisites

This vignette assumes you've read the **Getting Started** vignette and
understand:

- How to load phenological data
- The structure of the `pep` object
- Basic BBCH codes and DOY (day of year)

## Setup

Let's load the package and prepare our data:

```{r setup, message=FALSE, warning=FALSE}
library(pep725)

# Download the synthetic dataset
pep <- pep_download()

# For this vignette, we'll focus on two well-represented species:
# 1. Apple (Malus domestica) - excellent coverage across Europe
# 2. Grapevine (Vitis vinifera) - longest historical records

apple <- pep[species == "Malus domestica"]
vine <- pep[species == "Vitis vinifera"]

# Verify subsets contain data
stopifnot(nrow(apple) > 0, nrow(vine) > 0)

# Quick overview of our data
cat("=== Apple (Malus domestica) ===\n")
cat("Observations:", nrow(apple), "\n")
cat("Year range:", min(apple$year), "-", max(apple$year), "\n")
cat("Stations:", length(unique(apple$s_id)), "\n\n")

cat("=== Grapevine (Vitis vinifera) ===\n")
cat("Observations:", nrow(vine), "\n")
cat("Year range:", min(vine$year), "-", max(vine$year), "\n")
cat("Stations:", length(unique(vine$s_id)), "\n")
```

**Why these species?**
- *Malus domestica* (apple) has excellent coverage across Europe with clear
  phenological phases (bud burst, flowering, fruit development)
- *Vitis vinifera* (grapevine) has the longest historical records (back to 1775)
  and is economically important for wine production

## Part 1: Phenological Normals

### What are Phenological Normals?

In climatology, "climate normals" are defined as 30-year averages of temperature
or precipitation used to characterise typical climatic conditions.
By analogy, we propose the term **phenological normals** to describe long-term
averages of the timing of recurring biological events.

**Why are normals important?**

1. **Baseline for comparison**: Describing an event as “early” or “late” is only meaningful relative to when it typically occurs.
2. **Detecting change**: Comparing normals from different reference periods (e.g.,
   1961-1990 vs. 1991-2020) highlights long-term shifts in phenological timing
3. **Spatial patterns**: Normals vary across regions, reflecting differences in climate, latitude and elevation
4. **Model validation**: Phenological models predict normals; observations
   validate them

### Standard Reference Periods

The World Meteorological Organization (WMO) defines standard 30-year periods
for calculating normals:

| Period | Use Case |
|--------|----------|
| **1961-1990** | Historical baseline, often used as "pre-warming" reference |
| **1991-2020** | Current WMO standard normal period |
| **Custom** | Any period relevant to your research question |

**Important**: The period you choose affects your conclusions! If you compare
2023 to a 1961-1990 baseline, you might find larger anomalies than comparing
to a 1991-2020 baseline (because the recent period already includes warming).

### Calculating Normals with pheno_normals()

Let's calculate normals for apple:

```{r normals-basic}
# Calculate normals for apple across countries and phases
# Using 1990-2015 for illustration (ensures enough data in synthetic dataset)
normals <- pheno_normals(
  pep = apple,
  period = 1990:2015,
  by = c("country", "phase_id"),
  min_years = 10
)

print(normals)
```

**Understanding the parameters:**

| Parameter | What it does | Our choice |
|-----------|--------------|------------|
| `pep` | Input data | Apple observations |
| `period` | Years to include in normal | 1990-2015 (26 years) |
| `by` | Grouping variables | Country and phase (separate normals for each) |
| `min_years` | Minimum years required | 10 (groups with fewer years get NA) |

### Understanding the Output Statistics

The function returns several statistics for each group. Here's what these mean
and when to use each:

| Statistic | Description | When to Use |
|-----------|-------------|-------------|
| `n_years` | Years with data | Check data availability |
| `mean_doy` | Arithmetic mean DOY | Standard measure, but sensitive to outliers |
| `median_doy` | Middle value | **Recommended** - robust to outliers |
| `sd_doy` | Standard deviation | Describes year-to-year variability |
| `iqr_doy` | Interquartile range | Robust measure of spread |
| `mad_doy` | Median absolute deviation | Most robust spread measure |
| `q05`-`q95` | Percentiles | Define "early" (q05) and "late" (q95) thresholds |

**Which measure should you use?**

- For **central tendency**: Use `median_doy` (robust to outliers)
- For **variability**: Use `mad_doy` or `iqr_doy` (robust)
- For **defining extreme events**: Use percentiles (e.g., q05, q95)

```{r normals-summary}
# Get a summary of the normals
summary(normals)
```

### Visualizing Normals

The `plot()` method shows the median DOY with interquartile range (Q25-Q75) for
each group:

```{r normals-plot, fig.width=7, fig.height=5}
plot(normals)
```

A bar chart variant is also available with `plot(normals, which = "bar")`.

### Filtering to Specific Phases

Often you want normals for a specific phenological phase:

```{r normals-filtered}
# Normals for apple first flowering (BBCH 60) only
flowering_normals <- pheno_normals(
  pep = pep,                          # Can use full dataset
  period = 1990:2015,
  species = "Malus domestica",         # Filter by species
  phase_id = 60,                      # Filter by phase (first flowers)
  by = "country",                     # Group only by country
  min_years = 5                       # Lower threshold for more countries
)

print(flowering_normals)
```

**Note on NA values:** You'll notice some countries show `NA` for all
statistics. This happens when a country has fewer than `min_years` years of
data. This is intentional - calculating a "normal" from just 2-3 years would
be misleading.

The print output tells you "Groups with valid normals: X / Y" so you know
how many groups had sufficient data.

### Comparing Two Time Periods

A powerful application is comparing normals between periods to detect shifts:

```{r normals-compare, warning=FALSE, message=FALSE}
# Calculate normals for two periods
# Period 1: Historical (1961-1990)
period1 <- pheno_normals(
  pep,
  period = 1961:1990,
  species = "Malus",
  phase_id = 60,
  by = "country",
  min_years = 10
)

# Period 2: Recent (1991-2020)
period2 <- pheno_normals(
  pep,
  period = 1991:2020,
  species = "Malus",
  phase_id = 60,
  by = "country",
  min_years = 10
)

# Compare the two periods by merging on country
# Negative shift = flowering got earlier
# Positive shift = flowering got later
if (nrow(period1) > 0 && nrow(period2) > 0) {
  comparison <- merge(
    period1[, .(country, mean_doy_early = mean_doy)],
    period2[, .(country, mean_doy_recent = mean_doy)],
    by = "country"
  )
  comparison[, shift := mean_doy_recent - mean_doy_early]

  cat("Countries compared:", nrow(comparison), "\n")
  cat("Mean shift:", round(mean(comparison$shift, na.rm = TRUE), 1), "days\n")
  cat("Range of shifts:", round(min(comparison$shift, na.rm = TRUE), 1), "to",
      round(max(comparison$shift, na.rm = TRUE), 1), "days\n")

  if (mean(comparison$shift, na.rm = TRUE) < 0) {
    cat("\nInterpretation: On average, flowering has shifted EARLIER\n")
  } else {
    cat("\nInterpretation: On average, flowering has shifted LATER\n")
  }
}
```

**Ecological interpretation**: A shift of -5 days (earlier) over 30 years
corresponds to about 1.7 days per decade.

### Comparing Species: Grapevine with Longer History

Grapevine records extend much further back, enabling comparison across longer
time periods:

```{r normals-vine, warning=FALSE, message=FALSE}
# Calculate normals for grapevine flowering
vine_normals <- pheno_normals(
  pep = vine,
  period = 1990:2015,
  phase_id = 65,  # Full flowering
  by = "country",
  min_years = 5
)

# Compare with apple for countries that have both
if (nrow(vine_normals) > 0) {
  cat("Grapevine flowering normals (BBCH 65):\n")
  print(vine_normals[!is.na(mean_doy), .(country, n_years, mean_doy, sd_doy)])
}
```

## Part 2: Phenological Anomalies

### What are Anomalies?

An **anomaly** is the deviation of a single year from the long-term normal.
Anomalies help you:

1. **Identify extreme years**: Which years had unusually early or late phenology?
2. **Link to climate**: Do years with early phenology correspond to warm springs?
3. **Assess trends**: Are anomalies becoming more frequent or extreme?

### How Anomalies are Calculated

For each year, the anomaly is calculated as:

```
Anomaly = Observed DOY - Normal DOY
```

- **Negative anomaly** = Earlier than normal
- **Positive anomaly** = Later than normal

The anomaly can be expressed in:

- **Days**: Direct difference (e.g., "10 days early")
- **Z-score**: Standardized by variability (e.g., "2 standard deviations early")
- **Percentile**: Position in historical distribution (e.g., "5th percentile")

### Calculating Anomalies with pheno_anomaly()

```{r anomalies-basic}
# Calculate anomalies for apple
# Using 1990-2010 as baseline period
anomalies <- pheno_anomaly(
  pep = apple,
  baseline_period = 1990:2010,
  by = c("country", "phase_id"),
  min_years = 5
)

print(anomalies)
```

### Understanding Anomaly Metrics

The function returns several ways to express the anomaly:

| Metric | Description | Interpretation |
|--------|-------------|----------------|
| `anomaly_days` | Difference in days | Negative = early, positive = late |
| `z_score` | Standardized anomaly | \|z\| > 2 is unusual, > 3 is extreme |
| `percentile` | Position in distribution | <10 very early, >90 very late |
| `is_extreme` | Boolean flag | TRUE if outside normal range |
| `direction` | Text label | "early", "late", or "normal" |

**Which metric to use?**

- **For communication**: Use `anomaly_days` ("flowering was 10 days early")
- **For statistics**: Use `z_score` (allows comparison across regions/phases)
- **For frequency analysis**: Use `percentile` ("a 1-in-20 year event")

### Identifying Extreme Years

Let's find the most extreme years in our data:

```{r anomalies-extreme}
# Find extreme years (is_extreme == TRUE)
extreme <- anomalies[is_extreme == TRUE]

if (nrow(extreme) > 0) {
  cat("Total extreme years found:", nrow(extreme), "\n\n")

  # Show the most extreme early years
  cat("Most extreme EARLY years:\n")
  early <- extreme[direction == "early"][order(anomaly_days)]
  if (nrow(early) > 0) {
    print(early[1:min(5, nrow(early)),
          .(country, phase_id, year, anomaly_days, z_score)])
  }

  # Show the most extreme late years
  cat("\nMost extreme LATE years:\n")
  late <- extreme[direction == "late"][order(-anomaly_days)]
  if (nrow(late) > 0) {
    print(late[1:min(5, nrow(late)),
          .(country, phase_id, year, anomaly_days, z_score)])
  }
} else {
  cat("No extreme years detected in this dataset\n")
}
```

**What makes a year "extreme"?** By default, years with |z-score| > 2 are
flagged as extreme. This corresponds to roughly the most unusual 5% of years
(2.5% early + 2.5% late).

### Summary of Anomalies

```{r anomalies-summary}
summary(anomalies)
```

### Visualizing Anomalies

The `plot()` method shows anomalies over time, coloured by direction (early vs.
late), with extreme events marked:

```{r anomalies-plot, fig.width=7, fig.height=5}
plot(anomalies)
```

A histogram of anomaly magnitudes is available with
`plot(anomalies, which = "histogram")`.

### Robust vs. Classical Methods

Phenological data often contains outliers - observations that are clearly
erroneous or affected by unusual local conditions. The anomaly function offers
two approaches.

The **robust method** (`robust = TRUE`, default and recommended) uses the median as the baseline and the MAD (median absolute deviation) for standardization. This approach is less affected by outliers and should be preferred in most situations.

The **classical method** (`robust = FALSE`) uses the mean as the baseline and the standard deviation (SD) for standardization. This approach is more affected by outliers.

**When to use each:**

- **Robust (default)**: Almost always. Phenological data commonly has outliers.
- **Classical**: Only if you've already cleaned outliers, or for comparison
  with older literature that used classical methods.

## Part 3: Data Quality Assessment

### Why Quality Matters

Before running any analysis, you should understand your data's quality.
Poor quality data can lead to:

- **False trends**: A few erroneous observations can create apparent shifts
- **Inflated variability**: Outliers increase standard deviations
- **Spatial bias**: If quality varies by region, comparisons are compromised

### Quality Dimensions

The `pep_quality()` function assesses three dimensions:

| Dimension | Question | Metric |
|-----------|----------|--------|
| **Completeness** | How many years have data? | % of period covered |
| **Continuity** | Are there gaps? | Maximum consecutive missing years |
| **Consistency** | Are there outliers? | % of observations flagged |

### Running Quality Assessment

```{r quality-basic}
# Assess quality at the station level
quality <- pep_quality(
  pep = apple,
  by = c("s_id", "phase_id")
)

print(quality)
```

### Understanding Quality Grades

The function assigns letter grades based on thresholds:

| Grade | Completeness | Max Gap | Outliers | Interpretation |
|-------|--------------|---------|----------|----------------|
| **A** | ≥ 80% | ≤ 2 years | < 2% | Excellent - use confidently |
| **B** | ≥ 60% | ≤ 5 years | < 5% | Good - suitable for most analyses |
| **C** | ≥ 40% | ≤ 10 years | < 10% | Fair - use with caution |
| **D** | Below C | - | - | Poor - consider excluding |

**What grade do you need?**

- **Trend analysis**: Grade A or B recommended (need continuous data)
- **Normal calculation**: Grade B or C acceptable (gaps less critical)
- **Spatial analysis**: Grade C may suffice (using many stations)

```{r quality-summary}
# Summary of quality across all stations
summary(quality)
```

### Visualizing Quality Assessment

With many stations to assess, visual inspection is essential. The `plot()` method
provides an overview of quality metrics:

```{r quality-plot, fig.height=5, fig.width=11}
# Overview plot: grade distribution + station map (requires pep for coordinates)
plot(quality, pep = apple)
```

**Reading the plot:**

- **Left panel**: Grade distribution shows how many stations fall into each category
- **Right panel**: Map shows station locations colored by their quality grade,
  revealing spatial patterns in data quality

You can also generate individual plots:

```{r quality-plot-grades, fig.height=4}
# Just the grade distribution (no pep data needed)
plot(quality, which = "grades")
```

```{r quality-plot-map, fig.height=5}
# Just the map of station quality
plot(quality, which = "map", pep = apple)
```

### Filtering Data by Quality

A critical step before analysis is filtering to high-quality data:

```{r quality-filter}
# Get high-quality stations
high_quality <- quality[quality_grade %in% c("A", "B")]

cat("Quality distribution:\n")
print(table(quality$quality_grade))

cat("\nHigh quality (A or B):", nrow(high_quality), "of", nrow(quality),
    "(", round(100 * nrow(high_quality) / nrow(quality), 1), "%)\n")

# Filter original data to these stations
if (nrow(high_quality) > 0) {
  good_stations <- unique(high_quality$s_id)
  apple_hq <- apple[s_id %in% good_stations]

  cat("\nOriginal apple observations:", nrow(apple), "\n")
  cat("After quality filter:", nrow(apple_hq),
      "(", round(100 * nrow(apple_hq) / nrow(apple), 1), "%)\n")
}
```

### Assessing Quality at Different Levels

You can assess quality at various aggregation levels:

```{r quality-country}
# Country-level quality (coarser assessment)
country_quality <- pep_quality(
  pep = pep,
  by = c("country", "species", "phase_id")
)

print(country_quality)
```

## Part 4: Complete Analytical Workflow

Now let's put it all together in a realistic workflow:

```{r workflow}
cat("=== STEP 1: Data Quality Assessment ===\n\n")

# Assess quality
quality <- pep_quality(apple, by = c("s_id", "phase_id"))
cat("Quality grades:\n")
print(table(quality$quality_grade))

cat("\n=== STEP 2: Filter to Good Quality Data ===\n\n")

# Keep grades A, B, and C (exclude only grade D)
good_stations <- quality[quality_grade %in% c("A", "B", "C"), s_id]
apple_clean <- apple[s_id %in% good_stations]

cat("Original observations:", nrow(apple), "\n")
cat("After quality filter:", nrow(apple_clean), "\n")
cat("Retained:", round(100 * nrow(apple_clean) / nrow(apple), 1), "%\n")

stopifnot(nrow(apple_clean) > 0)

cat("\n=== STEP 3: Calculate Baseline Normals ===\n\n")

normals <- pheno_normals(
  apple_clean,
  period = 1990:2010,
  by = c("country", "phase_id"),
  min_years = 5
)

cat("Normals calculated for", sum(!is.na(normals$mean_doy)), "groups\n")
cat("Mean DOY range:", round(min(normals$mean_doy, na.rm = TRUE), 0),
    "to", round(max(normals$mean_doy, na.rm = TRUE), 0), "\n")

cat("\n=== STEP 4: Calculate Anomalies ===\n\n")

anomalies <- pheno_anomaly(
  apple_clean,
  baseline_period = 1990:2010,
  by = c("country", "phase_id"),
  min_years = 5
)

# Convert to plain data.table so grouped aggregation works correctly
# (the pheno_anomaly S3 class can interfere with [.data.table dispatch)
anomalies_dt <- as.data.table(anomalies)

cat("Anomalies calculated for", sum(!is.na(anomalies_dt$anomaly_days)), "year-groups\n")
cat("Extreme years:", sum(anomalies_dt$is_extreme, na.rm = TRUE), "\n")

cat("\n=== STEP 5: Analyze Trends by Decade ===\n\n")

decade_summary <- anomalies_dt[
  !is.na(anomaly_days),
  .(
    mean_anomaly = round(mean(anomaly_days, na.rm = TRUE), 1),
    sd_anomaly = round(sd(anomaly_days, na.rm = TRUE), 1),
    n_obs = .N,
    pct_extreme = round(100 * sum(is_extreme, na.rm = TRUE) / .N, 1)
  ),
  by = .(decade = floor(year / 10) * 10)
][order(decade)]

print(decade_summary)

cat("\nInterpretation:\n")
first_decade <- decade_summary$mean_anomaly[1]
last_decade <- decade_summary$mean_anomaly[nrow(decade_summary)]
change <- last_decade - first_decade

if (change < -3) {
  cat("- Strong shift toward EARLIER phenology\n")
} else if (change < 0) {
  cat("- Moderate shift toward earlier phenology\n")
} else if (change > 3) {
  cat("- Strong shift toward LATER phenology\n")
} else {
  cat("- No clear directional change\n")
}
```

## Part 5: Visualizing Trends

### Time Series Plots

The `pheno_plot_timeseries()` function creates publication-ready time series:

```{r timeseries-basic, fig.height=5}
# Prepare aggregated data for visualization
apple_annual <- apple[, .(
  day = mean(day, na.rm = TRUE),
  n = .N
), by = .(year, species, phase_id)]

# Add human-readable phase labels
apple_annual[, phase := factor(phase_id,
  levels = c(60, 65, 100),
  labels = c("First flowers", "Full flowering", "Harvest"))]

# Remove rows where phase mapping failed
apple_annual <- apple_annual[!is.na(phase)]

# Plot time series with trend lines
if (nrow(apple_annual) > 0) {
  pheno_plot_timeseries(
    apple_annual,
    color_by = "phase",
    smooth = TRUE,
    title = "Apple Phenology Trends"
  )
}
```

**Reading the plot:**

- Each line represents a phenological phase
- The smoothed line shows the underlying trend
- Downward trends indicate earlier phenology
- The spread around the line shows year-to-year variability

### Detecting Trend Turning Points

Sometimes trends aren't linear - they may accelerate, slow, or even reverse.
The `pheno_trend_turning()` function uses the sequential Mann-Kendall test to
detect such changes:

```{r trends-turning, fig.height=5}
# Get apple flowering data
apple_flowering <- pep[species == "Malus domestica" & phase_id == 60]

# Detect turning points
turning <- pheno_trend_turning(apple_flowering, min_years = 10)
print(turning)

# Visualize the analysis
plot(turning)
```

**Understanding the plot:**

- **Blue line (Progressive tau)**: Trend calculated from the start forward
- **Orange line (Retrograde tau)**: Trend calculated from the end backward
- **Crossing points**: Where trends may have changed direction
- **Dashed lines**: Statistical significance thresholds (±1.96 for 95%, ±2.58 for 99%)

### Simple Trend Statistics

For a single trend value, use `mann_kendall_z()`:

```{r kendall-simple}
# Aggregate to annual means
annual_doy <- apple_flowering[, .(
  mean_doy = mean(day, na.rm = TRUE)
), by = year][order(year)]

# Calculate Mann-Kendall test statistic. Values beyond +/- 1.96 indicate
# significant trends at the 5% level (standard normal approximation).
# The implementation includes the tie correction and continuity correction.
# Note: pep725 (<= 1.0.2) exported this as kendall_tau(), which was a
# misnomer — it returned the Z-statistic, not Kendall's tau. kendall_tau()
# still works (with a deprecation warning); mann_kendall_z() is the new
# recommended name.
tau <- mann_kendall_z(annual_doy$mean_doy)

cat("Mann-Kendall z-statistic:", round(tau, 3), "\n")
cat("\nInterpretation:\n")
if (tau < -2.58) {
  cat("- Highly significant earlier trend (p < 0.01)\n")
} else if (tau < -1.96) {
  cat("- Significant earlier trend (p < 0.05)\n")
} else if (tau > 2.58) {
  cat("- Highly significant later trend (p < 0.01)\n")
} else if (tau > 1.96) {
  cat("- Significant later trend (p < 0.05)\n")
} else {
  cat("- No statistically significant trend\n")
}
```

**Why Kendall's tau?**

- Non-parametric (doesn't assume normality)
- Robust to outliers
- Works well with ordinal data
- Widely used in phenological research

### Linking Phenology to Climate

A central goal of phenological research is to understand how climate drives
the timing of biological events. The `pheno_regional()` function compiles
multi-station phenological data into regional time series and links them to
global temperature anomalies, enabling you to assess **climate sensitivity** -
how many days earlier (or later) phenology occurs per degree of warming.

```{r regional-climate, eval=FALSE}
# Compile regional data and link to temperature anomalies
regional_data <- pheno_regional(
  pep = pep,
  species_name = "Malus domestica",
  year_min = 1961
)

# Visualize phenology alongside temperature anomalies
pheno_plot(regional_data)
```

**What the output shows:**

- **Regional median DOY** per year, aggregated from individual stations
- **Global temperature anomaly** for context (GISS dataset)
- **Correlation** between phenological timing and temperature

**Typical climate sensitivities** for European spring phenology:

| Phase | Sensitivity | Meaning |
|-------|-------------|---------|
| Leaf unfolding | -2 to -4 days/°C | Moderate response to spring warming |
| Flowering | -3 to -5 days/°C | Strong response, especially for fruit trees |
| Harvest | -1 to -3 days/°C | Weaker response, partly compensated by summer processes |

These values mean that for every 1°C increase in spring temperature, flowering
tends to occur 3-5 days earlier. Over the past 50 years, this has translated
to an advance of roughly 2-5 days per decade across Central Europe.

## Best Practices Summary

### Before Analysis

1. **Check data quality** with `pep_quality()` - always!
2. **Filter to appropriate quality grades** - usually A, B, or C
3. **Verify sample sizes** - are there enough years/stations?

### Calculating Normals

4. **Choose reference period carefully** - it affects your conclusions
5. **Use median_doy** for robust central tendency
6. **Set appropriate min_years** - WMO recommends 30, but 10-20 often works

### Calculating Anomalies

7. **Use robust method** (default) - handles outliers automatically
8. **Consider z-scores** for cross-region/phase comparisons
9. **Check for extreme years** - investigate them!

### Analyzing Trends

10. **Use non-parametric methods** like Kendall's tau
11. **Look for trend changes** with `pheno_trend_turning()`
12. **Consider multiple phases** - they may show different patterns

### Reporting

13. **Document your methods** - period, thresholds, filters
14. **Report sample sizes** - readers need to assess reliability
15. **Include uncertainty** - confidence intervals or standard errors

## Next Steps

You now understand the core analytical functions. Continue to:

- **Spatial Phenological Patterns**: How does phenology vary with elevation
  and latitude? Use `pheno_gradient()` and `pheno_synchrony()`.

```{r eval = FALSE}
vignette("spatial-patterns", package = "pep725")
```

- **Data Quality Assessment**: How to detect outliers and validate data completeness.

```{r eval = FALSE}
vignette("data-quality", package = "pep725")
```

## Session Info

```{r session}
sessionInfo()
```

---
title: "Getting Started with pep725"
author:
  - Matthias Templ^[University of Applied Sciences Northwestern Switzerland (FHNW)]
  - Barbara Templ^[Swiss Federal Institute for Forest, Snow and Landscape Research (WSL)]
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{Getting Started with pep725}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

*Vignette 1 of 4 for the **pep725** R package.
The most current version is available on
[GitHub](https://github.com/matthias-da/pep725) and
[CRAN](https://CRAN.R-project.org/package=pep725).*

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## What is Phenology?
Phenology is the study of cyclic and seasonal natural phenomena, especially in
relation to climate and plant life cycles. In plant science, phenology focuses
on the timing of recurring biological events such as:

- **Budburst**: When dormant buds begin to open in spring
- **Leaf unfolding**: The emergence of new leaves
- **Flowering**: When flowers open (anthesis)
- **Fruit ripening**: The maturation of fruits
- **Leaf senescence**: Autumn coloring and leaf fall

These phenological phases are strongly temperature-dependent and respond sensitively to interannual climate variability as well as long-term climate change. As a result, phenological records provide some of the most direct biological evidence of climate impacts on terrestrial ecosystems.

## The PEP725 Database

The [PEP725 Pan European Phenology Database](https://pep725.eu/) is one of
the world's most comprehensive collections of plant phenological observations. It brings together long-term records from national phenological monitoring networks across Europe into a harmonized and openly accessible research infrastructure.
Key characteristics of PEP725 include [@templ2018; @templ2026]:

- **Over 12 million observations** from across over 30 European countries
- **Records spanning 250+ years** (some stations have data back to the 1750s)
- **Thousands of monitoring stations** spanning a wide range of elevations
- **Broad taxonomic coverage** including agricultural crops, fruit trees, wild plant and forest tree species

Observations in PEP725 are based on standardized phase definitions (largely aligned with BBCH-type schemes) and consistent observation protocols within national networks. This standardization enables comparative analyses across regions, species, and time periods, thus addresses key data integration and reusability challenges commonly encountered in large, heterogeneous phenological datasets [@templ2025].

## Why Use the pep725 R Package?

Working with phenological data presents several challenges:

1. **Data volume**: Millions of observations require efficient data handling
2. **Data quality**: Observations may contain errors or outliers
3. **Standardization**: Different countries use different formats
4. **Analysis complexity**: Calculating trends, normals, and anomalies requires
   specialized methods

The **pep725** package addresses these challenges by providing:

- Efficient data import and handling using `data.table`
- Quality assessment and outlier detection tools
- Functions for calculating phenological normals and anomalies
- Spatial analysis (gradients, synchrony)
- Publication-ready visualizations


## Installation

Before you begin, install the package:

```{r install, eval = FALSE}
# Install from CRAN (when available)
install.packages("pep725")

# Or install development version from GitHub
# First install devtools if you don't have it:
# install.packages("devtools")
devtools::install_github("matthias-da/pep725")
```

## Loading the Package

Once installed, load the package at the start of your R session:

```{r setup, message=FALSE, warning=FALSE}
library(pep725)
```

When the package loads, you'll see a startup message with tips on how to get
data.

## Getting Phenological Data

The **pep725** package offers five ways to obtain data. Understanding which
option to use is important for your workflow:

| Option | Best For | Data Size | Internet Required? |
|--------|----------|-----------|-------------------|
| **1. Download synthetic data** | Learning, tutorials, reproducible examples | ~7.24M rows | First download only |
| **2. Seed dataset** | Quick offline tests, minimal examples | ~1,300 rows | No |
| **3. Generate synthetic data** | Creating shareable versions of your data | Variable | No |
| **4. Import real PEP725 data** | Actual research and publications | ~7.24M rows | For initial download |
| **5. Use your own phenological data** | Regional networks, field trials, citizen science datasets | Variable | No |

### Why Synthetic Data?

You might wonder why we use synthetic data instead of real observations. The
original PEP725 database has usage restrictions that prevent redistribution -
you must register on the pep725.eu website and accept the data use policy. This creates a problem for
tutorials and examples that need to be reproducible.

**Synthetic data** solves this problem. It is generated to preserve the
statistical properties of real data:

- Same distributions of phenological events across seasons
- Similar correlations between variables (e.g. latitude and flowering date)
- Realistic spatial patterns across Europe
- Comparable year-to-year variability

This means you can learn all the analysis techniques using synthetic data, then
apply them confidently to real data when you're ready for actual research.

### Option 1: Download Synthetic Data (Recommended for Learning)

To learn the package, download the pre-generated synthetic dataset:

```{r download, message=FALSE}
# Download synthetic data (cached locally after first download)
pep <- pep_download()
```

**What happens here?**

1. The first time you run this, the package downloads a ~64 MB file from GitHub
2. The file is cached in a local directory (typically in your user folder)
3. Subsequent calls load from the cache instantly - no re-download needed

You can check your cache status or clear it if needed:

```{r cachestatus, message=FALSE}
# Check cache status - shows file location and size
pep_cache_info()

# If you need to clear the cache (e.g., to get an updated version):
# pep_cache_clear()
```

### Option 2: Use the Small Seed Dataset

For quick tests when you don't need the full dataset, use the included seed
data:

```{r load-seed, eval = FALSE}
# Load the included seed dataset (small, for quick tests only)
data(pep_seed)

# This is a small subset (~1,300 rows) useful for:
# - Testing code quickly
# - Working offline
# - Running examples in documentation
```

**When to use this**: If you're developing code and want fast iteration without
waiting for the full dataset to load, the seed dataset is perfect. However,
it's too small for meaningful statistical analyses.

### Option 3: Generate Your Own Synthetic Data

If you have real PEP725 data and want to create a shareable synthetic version
(e.g., for teaching or supplementary materials), use `pep_simulate()`:

```{r simulate, eval = FALSE}
# Generate synthetic data based on your real data
# This preserves statistical properties while anonymizing observations
pep_synthetic <- pep_simulate(your_real_pep_data)

# You can then share pep_synthetic freely
```

**Use case**: You've done an analysis with real data for a paper, and you want
to provide reproducible examples in supplementary materials without violating
data sharing restrictions.

### Option 4: Import Real PEP725 Data (For Research)

For actual research and publications, you'll want real data. Here's the process:

1. **Register** at [pep725.eu](https://pep725.eu/) (free for research)
2. **Download** data for your species/regions of interest
3. **Import** using `pep_import()`

```{r import, eval = FALSE}
# Import from downloaded PEP725 files
# Point to the folder containing your downloaded CSV files
pep <- pep_import("path/to/pep725_data/")
```

The `pep_import()` function automatically cleans and formats the data:

- Converts columns to appropriate types (factors, dates, etc.)
- Handles missing values (e.g., -9999 altitude codes become NA)
- Removes problematic columns
- Optionally adds country names based on coordinates


### Option 5: Use Your Own Plant Phenological Data

Although **pep725** is designed around the structure and conventions of the PEP725 database, it is not limited to PEP725 data.

If your dataset consists of station-based phenological observations with:

- spatial coordinates (longitude and latitude),
- observation year,
- species identifiers,
- phenophase identifiers,
- and day-of-year (DOY) information,

you can use most of the functionality in **pep725** with minimal adaptation.

The key requirement is a *PEP725-like tabular structure* — one row per observation. This structure can usually be achieved with straightforward preprocessing (e.g., renaming columns and ensuring consistent DOY formatting).

Once your data has the required columns, convert it to a `pep` object:

```{r own-data, eval = FALSE}
# Prepare your data with the required columns
my_data <- data.frame(
  s_id = c("MY001", "MY001", "MY002"),
  lon = c(10.1, 10.1, 11.3),
  lat = c(47.5, 47.5, 48.1),
  genus = c("Malus", "Malus", "Malus"),
  species = c("Malus domestica", "Malus domestica", "Malus domestica"),
  phase_id = c(60, 65, 60),
  year = c(2020, 2020, 2020),
  day = c(110, 125, 108)
)

# Convert to a pep object — validates required columns automatically
my_pep <- as.pep(my_data)
```



> **Important Note for Research**
>
> All examples in these vignettes use **synthetic data**. The patterns you see
> are realistic but not actual scientific observations.
>
> **For publishable research, always use real PEP725 data!** After registering
> at [pep725.eu](https://pep725.eu/), simply replace `pep_download()` with
> `pep_import("your/data/path/")`. All the analysis techniques work identically.



## Understanding the Data Structure

Now that we have data loaded, let's understand what we're working with. The
`pep` object is a special type of data frame optimized for large datasets.

```{r structure}
# View the data - the print method shows a summary
print(pep)
```

Let's get a quick visual overview. The `plot()` method for `pep` objects
provides three built-in views — a station map, a time series, and a DOY
histogram:

```{r first-look, fig.height = 4}
# Where are the stations?
plot(pep, type = "map")

# How are phenological events distributed across the year?
plot(pep, type = "histogram")
```

### What You See in the Output

The output shows you:

- **Dimensions**: Number of rows (observations) and columns
- **Column names and types**: What variables are available
- **First few rows**: A preview of actual values

### Key Columns Explained

Each row in the dataset represents **one phenological observation** - a record
of when a specific plant at a specific location reached a particular
developmental stage.

| Column | Description | Example |
|--------|-------------|---------|
| `s_id` | **Station identifier** - A unique code for each monitoring location | "AT0001" |
| `lon`, `lat` | **Geographic coordinates** - Longitude and latitude in decimal degrees | 15.5, 48.2 |
| `alt` | **Altitude** - Elevation above sea level in meters | 350 |
| `genus` | **Plant genus** - The genus name (first part of scientific name) | "Malus" |
| `species` | **Full species name** - Genus + specific epithet | "Malus domestica" |
| `phase_id` | **BBCH code** - Standardized phenological stage (explained below) | 60 |
| `year` | **Observation year** | 2015 |
| `day` | **Day of year (DOY)** - The day when the phase was observed (1-365) | 145 |
| `country` | **Country name** | "Austria" |

### Understanding Day of Year (DOY)

Phenological data uses **Day of Year (DOY)** rather than calendar dates. This
simplifies calculations and comparisons:

- DOY 1 = January 1
- DOY 91 = April 1 (approximately)
- DOY 182 = July 1 (approximately)
- DOY 274 = October 1 (approximately)
- DOY 365/366 = December 31

**Why DOY?** It makes it easy to calculate things like "flowering advanced by
10 days" without dealing with month boundaries.

## The `pep` Class

The package provides a custom class system that extends `data.table` with
phenology-specific methods. Let's verify what class our data has:

```{r class}
# Check the class hierarchy
class(pep)
```

You'll see three classes:

1. `pep` - Our custom class with phenology-specific methods
2. `data.table` - Fast data manipulation (from the data.table package)
3. `data.frame` - Base R compatibility

This means you can use:

- **pep725 functions** designed for phenological analysis
- **data.table syntax** for fast filtering and aggregation
- **Base R functions** that work with data frames

### The Summary Method

The `summary()` function is your first tool for understanding a phenological
dataset. It provides different views depending on the `by` parameter:

```{r summary}
# Default summary - shows top species by observation count
summary(pep)
```

**What this tells you**: Which species have the most data, how many stations
observe each species, and the year range of observations.

```{r summary-phase}
# Summary by phenological phase
summary(pep, by = "phase")
```

**What this tells you**: Which developmental stages are recorded, how many
observations of each, and the typical DOY (useful for understanding the
seasonal distribution of your data).

```{r summary-country}
# Summary by country (showing top 5)
summary(pep, by = "country", top = 5)
```

**What this tells you**: Geographic coverage - which countries contribute most
data, how many stations they have, and species diversity.

```{r summary-year}
# Temporal coverage summary
summary(pep, by = "year")
```

**What this tells you**: How observations are distributed across years - are
there gaps? Is coverage increasing or decreasing over time?

### Subsetting the Data

You'll often want to focus on specific subsets of the data. The package
preserves the `pep` class when you subset, so all methods continue to work:

```{r subset}
# Filter to apple data using data.table syntax
# The syntax is: dataset[row_filter, column_selection]
apple <- pep[species == "Malus domestica"]
print(apple)

# Filter by year range
recent <- pep[year >= 2000 & year <= 2015]
print(recent)

# The summary method works on subsets too
summary(recent)
```

**Tip for beginners**: The `data.table` syntax `pep[condition]` is similar to
base R's `subset()` function but much faster for large datasets. You can
combine multiple conditions with `&` (and) or `|` (or).

## Understanding BBCH Codes

The **BBCH scale** [from Biologische Bundesanstalt, Bundessortenamt und
CHemische Industrie; @meier2018] is a standardized system for describing plant
developmental stages. It's used worldwide, ensuring that "flowering" means the
same thing whether observed in Germany or Spain.

### How BBCH Codes Work

BBCH codes are two-digit numbers where:

- The **first digit** (0-9) indicates the principal growth stage
- The **second digit** indicates the secondary stage within that principal stage

| First Digit | Principal Growth Stage |
|-------------|------------------------|
| 0 | Germination / sprouting |
| 1 | Leaf development |
| 2 | Formation of side shoots / tillering |
| 3 | Stem elongation |
| 4 | Development of harvestable vegetative parts |
| 5 | Inflorescence emergence / heading |
| 6 | Flowering |
| 7 | Development of fruit |
| 8 | Ripening of fruit and seed |
| 9 | Senescence / dormancy |

### Looking Up BBCH Codes

The `bbch_description()` function translates codes to human-readable
descriptions:

```{r bbch}
# Get descriptions for BBCH codes present in your data
codes <- unique(pep$phase_id)
bbch_description(codes)
```

### Commonly Used BBCH Codes in Phenological Research

| Code | Stage | What to Look For |
|------|-------|------------------|
| 10 | First leaves | Cotyledons or first true leaves visible |
| 11 | First leaf unfolded | First true leaf fully unfolded |
| 60 | First flowers open | Beginning of flowering (anthesis) |
| 65 | Full flowering | 50% of flowers open |
| 69 | End of flowering | All petals fallen |
| 85 | Fruit coloring | Softening begins |
| 87 | Fruit ripe | Ready for harvest |
| 89 | Full ripeness | Fruit fully mature |
| 91 | Shoot growth complete | Leaves fully colored |
| 95 | Leaf fall | 50% of leaves fallen |

### Filtering by Phenological Phase

When analyzing phenology, you typically focus on specific phenophases. Here's how:

```{r filter-phase}
# Get all flowering observations (BBCH 60-69)
flowering <- pep[phase_id >= 60 & phase_id <= 69]
cat("Flowering observations:", nrow(flowering), "\n")

# Get only full flowering (BBCH 65)
full_flowering <- pep[phase_id == 65]
cat("Full flowering observations:", nrow(full_flowering), "\n")

# Get harvest-related observations
harvest <- pep[phase_id %in% c(87, 89)]
cat("Harvest observations:", nrow(harvest), "\n")
```

## Exploring Data Coverage

Before diving into analysis, it's crucial to understand what your dataset
contains. The `pep_coverage()` function provides a comprehensive overview.

### Full Coverage Report

```{r coverage-all}
# Get a complete coverage report
pep_coverage(pep)
```

This report shows you three dimensions of coverage:

1. **Temporal**: What years are covered? Any gaps?
2. **Geographical**: Which countries/regions? How many stations?
3. **Species**: Which plants are observed? How many observations each?

### Focused Coverage Analysis

You can focus on specific aspects and optionally generate plots:

```{r coverage-temporal}
# Temporal coverage with a visualization
pep_coverage(pep, kind = "temporal", plot = TRUE)
```

The temporal plot shows observation density over time. Look for:

- **Gaps**: Years with few or no observations
- **Trends**: Is data coverage increasing or decreasing over time?
- **Breakpoints**: Sudden changes (e.g. due to methodology changes)

```{r coverage-geo}
# Geographical coverage - which countries have most data
pep_coverage(pep, kind = "geographical", top = 5)
```

Geographic coverage tells you where your data comes from. This is important
because:

- Different regions have different climates
- Some countries have longer observation histories
- Network density affects spatial analyses

```{r coverage-species}
# Species coverage - which species are best represented
pep_coverage(pep, kind = "species", top = 5)
```

Species coverage shows which plants have enough data for robust analyses. As a
rule of thumb:

- **>10,000 observations**: Excellent for most analyses
- **1,000-10,000 observations**: Good for national/regional analyses
- **<1,000 observations**: Limited; may need to combine with related species

### Coverage by Groups

You can break down coverage by grouping variables:

```{r coverage-by-country}
# Temporal coverage broken down by country
cov_by_country <- pep_coverage(pep, kind = "temporal", by = "country")
# Show observations by group
cov_by_country$temporal$obs_by_group
```

This helps identify which countries have continuous records versus sporadic
observations.

## Your First Analysis: Tracking Flowering Trends

Let's put everything together with a simple but meaningful analysis: tracking
how apple flowering dates have changed over time.

```{r quick-analysis}
# Step 1: Filter to apple flowering (BBCH 60 = first flowers)
apple_flowering <- pep[species == "Malus domestica" & phase_id == 60]

cat("Apple flowering observations:", nrow(apple_flowering), "\n")
cat("Year range:", min(apple_flowering$year), "-", max(apple_flowering$year), "\n")
cat("Countries:", length(unique(apple_flowering$country)), "\n")

# Step 2: Calculate annual mean DOY
# The data.table syntax here means:
#   - Group by year
#   - Calculate mean DOY and count observations
#   - Order by year
annual_mean <- apple_flowering[, .(
  mean_doy = mean(day, na.rm = TRUE),
  n_obs = .N
), by = year][order(year)]

# Step 3: Look at the results
print(annual_mean)
```

Now let's visualize the trend:

```{r quick-analysis-plot}
# Step 4: Plot the trend with ggplot2
library(ggplot2)

p <- ggplot(annual_mean, aes(x = year, y = mean_doy)) +
  geom_point(aes(size = n_obs), color = "steelblue", alpha = 0.6) +
  geom_line(color = "steelblue", alpha = 0.4) +
  geom_smooth(method = "lm", color = "red", linetype = 2, se = FALSE) +
  labs(
    title = "Apple Flowering Date Over Time",
    x = "Year", y = "Mean Day of Year",
    size = "Observations"
  ) +
  theme_minimal()
print(p)

# Step 5: Quantify the trend
trend <- lm(mean_doy ~ year, data = annual_mean)
slope <- coef(trend)[2]
cat("\nTrend:", round(slope * 10, 2), "days per decade\n")
if (slope < 0) {
  cat("Interpretation: Flowering is getting EARLIER\n")
} else {
  cat("Interpretation: Flowering is getting LATER\n")
}
```

**Interpreting the results**: A negative slope means flowering is occurring
earlier in the year - a common finding for spring phenology in a warming
climate. A shift of -2 days per decade means that over 50 years, flowering
has advanced by about 10 days.

### Comparing with Another Species: Grapevine

Let's compare apple with grapevine (*Vitis vinifera*), which has the longest
time series in the database (records back to 1775 in some regions):

```{r quick-analysis-vine}
# Step 1: Filter to grapevine flowering (BBCH 65 = full flowering)
vine_flowering <- pep[species == "Vitis vinifera" & phase_id == 65]

cat("Grapevine flowering observations:", nrow(vine_flowering), "\n")
cat("Year range:", min(vine_flowering$year), "-", max(vine_flowering$year), "\n")
cat("Countries:", length(unique(vine_flowering$country)), "\n")

# Step 2: Calculate annual mean DOY
vine_annual <- vine_flowering[, .(
  mean_doy = mean(day, na.rm = TRUE),
  n_obs = .N
), by = year][order(year)]

# Step 3: Plot the trend
p <- ggplot(vine_annual, aes(x = year, y = mean_doy)) +
  geom_point(aes(size = n_obs), color = "purple", alpha = 0.6) +
  geom_line(color = "purple", alpha = 0.4) +
  geom_smooth(method = "lm", color = "red", linetype = 2, se = FALSE) +
  labs(
    title = "Grapevine Flowering Date Over Time",
    x = "Year", y = "Mean Day of Year",
    size = "Observations"
  ) +
  theme_minimal()
print(p)

trend <- lm(mean_doy ~ year, data = vine_annual)
slope <- coef(trend)[2]
cat("\nTrend:", round(slope * 10, 2), "days per decade\n")
```

**Why grapevine?** *Vitis vinifera* is economically important (wine production),
and its phenology is closely monitored. The long historical records make it
ideal for studying long-term climate trends.

## Common Pitfalls and Tips

As you start working with phenological data, keep these points in mind:

### Data Quality Matters

- **Always check for outliers**: A flowering date of DOY 300 (late October) for
  apple is almost certainly an error
- **Consider sample sizes**: Trends from 5 observations are not reliable
- **Check spatial coverage**: Trends might differ between regions


### Species Considerations

- **Check taxonomy**: "Malus" might include multiple apple species
- **Consider subspecies**: Some records have subspecies/variety information
- **Wild vs. cultivated**: Agricultural species may behave differently than
  wild populations

## Next Steps

Now that you understand the basics, explore the other vignettes for more
advanced analyses:

### Phenological Analysis Vignette

Learn to calculate phenological normals (long-term averages), detect anomalies
(unusual years), and analyze trends using:

- `pheno_normals()` - Calculate climatological baselines
- `pheno_anomaly()` - Identify deviations from normal
- `pheno_trend_turning()` - Detect changes in trend direction

```{r eval = FALSE}
vignette("phenological-analysis", package = "pep725")
```

### Spatial Phenological Patterns Vignette

Analyze how phenology varies across landscapes:

- `pheno_gradient()` - Quantify elevation and latitude effects
- `pheno_synchrony()` - Measure spatial coherence
- `pheno_map()` - Create maps of phenological patterns

```{r eval = FALSE}
vignette("spatial-patterns", package = "pep725")
```

### Data Quality Assessment Vignette

Ensure your data is reliable and investigate unusual observations:

- `pep_flag_outliers()` - Identify suspicious observations
- `pep_plot_outliers()` - Visualize outliers in context

```{r eval = FALSE}
vignette("data-quality", package = "pep725")
```

## Getting Help

If you encounter problems:

1. **Check the function documentation**: `?function_name` (e.g., `?pheno_normals`)
2. **Read the vignettes**: They contain worked examples
3. **Report issues**: [github.com/matthias-da/pep725/issues](https://github.com/matthias-da/pep725/issues)

## Session Info

For reproducibility, here's the R environment used for this vignette:

```{r session}
sessionInfo()
```

## References

---
title: "Spatial Phenological Patterns"
author:
  - Matthias Templ^[University of Applied Sciences Northwestern Switzerland (FHNW)]
  - Barbara Templ^[Swiss Federal Institute for Forest, Snow and Landscape Research (WSL)]
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{Spatial Phenological Patterns}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

*Vignette 4 of 4 for the **pep725** R package.
The most current version is available on
[GitHub](https://github.com/matthias-da/pep725) and
[CRAN](https://CRAN.R-project.org/package=pep725).*

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

Phenological timing varies systematically across space due to environmental
gradients. Understanding these spatial patterns is fundamental to phenological
research because it allows us to:

- **Scale observations**: Extrapolate from point measurements to landscapes
- **Predict impacts**: Forecast how climate change will shift phenological timing
- **Design networks**: Optimize monitoring station placement
- **Validate models**: Compare observed gradients with model predictions

This vignette covers spatial analysis and visualization functions in the
`pep725` package, organized into four parts:

### What You'll Learn

| Section | Key Concepts | Functions |
|---------|--------------|-----------|
| Part 1: Gradients | How phenology changes with altitude & latitude | `pheno_gradient()` |
| Part 2: Synchrony | Spatial coherence of phenological events | `pheno_synchrony()` |
| Part 3: Combined | Integrating gradient and synchrony results | `pheno_gradient()`, `pheno_synchrony()` |
| Part 4: Mapping | Visualizing station networks and patterns | `pheno_leaflet()`, `pheno_map()` |

### Prerequisites

This vignette assumes you have completed the "Getting Started" vignette and are
familiar with:

- The basic structure of PEP725 data
- BBCH phenological phase codes
- Day of Year (DOY) as a timing metric

## Setup

```{r setup, message=FALSE, warning=FALSE}
library(pep725)
library(data.table)
library(ggplot2)

# Download the synthetic dataset
pep <- pep_download()

# Overview of the data
cat("Stations:", length(unique(pep$s_id)), "\n")
cat("Altitude range:", min(pep$alt, na.rm = TRUE), "-",
    max(pep$alt, na.rm = TRUE), "m\n")
cat("Latitude range:", round(min(pep$lat, na.rm = TRUE), 2), "-",
    round(max(pep$lat, na.rm = TRUE), 2), "N\n")
```

---

# Part 1: Phenological Gradients

## What are Phenological Gradients?

A **phenological gradient** describes systematic changes in the timing of biological events along an environmental axis. Such gradients reflect spatial variations in climatic conditions and are fundamental for understanding large-scale patterns in phenology. The two most important phenological gradients are associated with elevation and latitude:

### Altitudinal Gradient

With increasing elevation, air temperature generally decreases. As a result, phenological events tend to occur progressively later at higher altitudes. The rate at which phenological timing shifts with elevation is commonly referred to as the
**phenological lapse rate**.

```
                     Mountain Profile
                          /\
                         /  \  <- Higher elevation = LATER phenology
                        /    \
                       /      \
         ____________/        \____________
              ^                     ^
         Earlier                  Earlier
        (lowland)                (lowland)
```

**Typical values**: In temperate regions of Europe, phenological phases are delayed by **2-4 days per
100 meters** of elevation gain. In practical terms:
- A station at 500m elevation might observe flowering 10-20 days later than a lowland station.
- High-elevation stations (2000m+) can exhibit delays of 40-80 days relative to lowland sites

### Latitudinal Gradient

Moving northward in the Northern Hemisphere is generally associated with cooler climatic conditions and changes in seasonal radiation regimes. Consequently, phenological events tend to occur later at
higher latitudes.

**Typical values**: Across Europe, phenology is delayed by **2-5 days per degree of
latitude** toward the north. For example:
- The latitudinal difference between southern France (43°N) and northern Germany (54°N) is about 11 degrees
- This corresponds to an expected difference of 22-55 days in phenology

### Why Study Gradients?

Understanding phenological gradients is valuable for several reasons:

1. **Scaling observations**: Gradients allow phenological timing to be extrapolated to locations without direct observations, based on elevation or latitude
2. **Validating models**: Phenological and crop models should reproduce observed spatial gradients
3. **Detecting change**: Temporal changes in the strength or shape of phenological gradients may signal climate-driven alterations in biological responses
4. **Understanding adaptation**: Systematic deviations from expected gradients may reveal
   local adaptation, genetic variation or the influence of site-specific factors.
   
## Altitudinal Gradient Analysis

Let's calculate the elevational gradient for apple flowering:

```{r gradient-altitude}
# Calculate elevational gradient for apple flowering
grad_alt <- pheno_gradient(
  pep = pep,
  variable = "alt",
  species = "Malus domestica",
  phase_id = 60,
  method = "robust"
)

print(grad_alt)
```

### Understanding the Function Parameters

| Parameter | Purpose | Your Choice |
|-----------|---------|-------------|
| `pep` | Input data | Your phenological dataset |
| `variable` | Environmental gradient | `"alt"` for elevation, `"lat"` for latitude |
| `species` | Species to analyze | Must match exactly (e.g., "Malus domestica") |
| `phase_id` | BBCH phase code | 60 = flowering |
| `method` | Regression method | `"robust"` recommended for phenological data |

### Interpreting the Results

The output contains several important metrics:

| Metric | What It Tells You | How to Interpret |
|--------|-------------------|------------------|
| `slope` | Days delay per 100m elevation | The phenological lapse rate |
| `intercept` | Predicted DOY at 0m altitude | Theoretical baseline (extrapolation) |
| `r_squared` | Proportion of variance explained | How well elevation predicts phenology |
| `n_points` | Number of stations used | Sample size (affects reliability) |

**Key interpretation points:**

- **Positive slope**: Later phenology at higher elevations (expected)
- **Negative slope**: Earlier phenology at higher elevations (unusual - investigate!)
- **Slope magnitude**: Compare to literature values (2-4 days/100m typical)
- **R-squared**: Values > 0.3 indicate meaningful gradient; < 0.1 suggests weak control

## Latitudinal Gradient Analysis

The same function works for latitude gradients:

```{r gradient-latitude}
# Calculate latitude gradient
grad_lat <- pheno_gradient(
  pep = pep,
  variable = "lat",
  species = "Malus domestica",
  phase_id = 60,
  method = "robust"
)

print(grad_lat)
```

## Comparing Regression Methods

The `pheno_gradient()` function offers three regression methods. Choosing the
right method matters because phenological data often contains outliers.

```{r gradient-methods}
# Robust regression (default) - handles outliers
grad_robust <- pheno_gradient(pep, variable = "alt",
                              species = "Malus domestica",
                              phase_id = 60, method = "robust")

# Ordinary least squares
grad_ols <- pheno_gradient(pep, variable = "alt",
                           species = "Malus domestica",
                           phase_id = 60, method = "ols")

cat("Robust slope:", round(grad_robust$summary$slope, 2), "days/100m\n")
cat("OLS slope:", round(grad_ols$summary$slope, 2), "days/100m\n")
```

### Which Method Should You Use?

| Method | Strengths | Weaknesses | When to Use |
|--------|-----------|------------|-------------|
| `robust` | Resistant to outliers and leverage points | Slightly lower efficiency under ideal conditions (clean data) | Recommended **default choice** for phenological data |
| `ols` | Efficient when assumptions are met | Highly sensitive to outliers | Use after careful data screening |
| `quantile` | Models conditional medians or other quantiles | Less precise estimates (wider confidence intervals) | When focusing on median responses |

**Recommendation**: Start with `method = "robust"` for exploratory and applied analyses. Substantial differences between robust and OLS estimates often indicate influential observations and should prompt further inspection.

### Comparing Species: Grapevine Gradient

Different species may show different gradient strengths. Let's compare apple
with grapevine:

```{r gradient-vine}
# Calculate gradient for grapevine
vine_data <- pep[species == "Vitis vinifera"]

# Verify data exists before analysis
if (nrow(vine_data) > 0) {
  grad_vine <- pheno_gradient(
    pep = pep,
    variable = "alt",
    species = "Vitis vinifera",
    phase_id = 65,  # Full flowering
    method = "robust"
  )

  cat("Comparison of elevation gradients:\n")
  cat("Apple:     ", round(grad_robust$summary$slope, 2), "days/100m",
      "(R² =", round(grad_robust$summary$r_squared, 3), ")\n")
  cat("Grapevine: ", round(grad_vine$summary$slope, 2), "days/100m",
      "(R² =", round(grad_vine$summary$r_squared, 3), ")\n")
} else {
  cat("No grapevine data available for gradient analysis.\n")
}
```

## Gradients by Region

Phenological gradients are not uniform across space. Their magnitude and shape can vary between regions due to:
- Different climate zones
- Local adaptation of species
- Data quality differences

Estimating gradients separately for different regions:

```{r gradient-by-region}
# Calculate gradient by country
grad_by_country <- pheno_gradient(
  pep = pep,
  variable = "alt",
  species = "Malus domestica",
  phase_id = 60,
  by = "country",
  method = "robust"
)

print(grad_by_country$summary)
```

**What to look for:**
- Consistency of slopes: Are slopes consistent across regions? Similar slope estimates across regions suggest a robust and generalizable gradient pattern.
- Explained variance: Are R-squared values similar? (Comparable R-squared values indicate that the gradient explains phenological variability consistently across regions.)
- Sample size adequacy: Do sample sizes support reliable estimates? (Reliable gradient estimates require sufficient data coverage; small sample sizes (e.g. n < 20) should be interpreted with caution.)

## Visualizing Gradients

Visualization plays a key role in interpreting and communicating phenological gradients, helping to assess linearity, uncertainty, and regional differences at a glance:

```{r gradient-plot, fig.height=5}
# Plot the gradient relationship
if (!is.null(grad_alt$data) && nrow(grad_alt$data) > 2) {
  p <- plot(grad_alt)
  print(p)
}
```

**Reading the plot:**
- Each point represents a station-level mean phenophase (averaged across years).
- The fitted line shows the estimated relationship between phenophase and elevation.
- Points deviating strongly from the line indicate locations where elevation alone explains phenology poorly.
- Systematic clustering of residuals may point to regional influences or additional controlling factors.

## Expected Values and Troubleshooting

### Reference Values from Literature

The following table summarises phenological gradient values reported in the literature.
These can serve as benchmarks when interpreting your own results:

| Gradient | Typical Range | Region | Sources |
|----------|---------------|--------|---------|
| Altitude | 2–4 days/100m | Europe (temperate) | Pellerin et al. (2012); Vitasse et al. (2009); Ziello et al. (2009) |
| Altitude | 1–3 days/100m | Europe (warm / low elevation) | Vitasse et al. (2009); Ziello et al. (2009) |
| Latitude | 2–4 days/degree | Europe | Rötzer & Chmielewski (2001) |
| Latitude | 3–5 days/degree | North America | Hopkins (1918); Richardson et al. (2019) |

**Notes:**

- Altitude gradients are reported as delay in days per 100 m of elevation gain.
  Values vary by species: deciduous broadleaves (e.g. oak, hazel) tend toward the
  upper end, while conifers (e.g. spruce) show weaker responses
  (Ziello et al. 2009).
- Latitude gradients are reported as delay in days per degree northward. Hopkins'
  (1918) bioclimatic law predicts approximately 4 days per degree of latitude,
  which remains a useful first approximation.
- Gradient strength may change over time: Chen et al. (2018) showed that altitude
  gradients in Europe have been declining since the 1980s, indicating more uniform
  phenology across elevations under warming.

**Key references:**

- Pellerin, M. et al. (2012). Spring tree phenology in the Alps. *European Journal of Forest Research*, 131, 1957–1965. [doi:10.1007/s10342-012-0646-1](https://doi.org/10.1007/s10342-012-0646-1)
- Vitasse, Y. et al. (2009). Leaf phenology sensitivity to temperature in European trees. *Agricultural and Forest Meteorology*, 149, 735–744. [doi:10.1016/j.agrformet.2008.10.019](https://doi.org/10.1016/j.agrformet.2008.10.019)
- Ziello, C. et al. (2009). Influence of altitude on phenology of selected plant species in the Alpine region. *Climate Research*, 39, 227–234. [doi:10.3354/cr00822](https://doi.org/10.3354/cr00822)
- Rötzer, T. & Chmielewski, F.-M. (2001). Phenological maps of Europe. *Climate Research*, 18, 249–257. [doi:10.3354/cr018249](https://doi.org/10.3354/cr018249)
- Richardson, A.D. et al. (2019). Testing Hopkins' Bioclimatic Law with PhenoCam data. *Applications in Plant Sciences*, 7, e01228. [doi:10.1002/aps3.1228](https://doi.org/10.1002/aps3.1228)
- Chen, L. et al. (2018). Spring phenology at different altitudes is becoming more uniform under global warming in Europe. *Global Change Biology*, 24, 3969–3975. [doi:10.1111/gcb.14288](https://doi.org/10.1111/gcb.14288)

### When Your Results Differ from Expected

| Observation | Possible Causes | What to Do |
|-------------|-----------------|------------|
| Slope too steep (>5 days/100m) | Outliers, microclimate effects | Check for data errors, use robust method |
| Slope too shallow (<1 day/100m) | Limited elevation range, species adaptation | Expand geographic scope |
| Negative slope | Data errors, southern aspect effects | Investigate unusual observations |
| Very low R² | Other factors dominate (precipitation, soil) | Consider multiple regression |

---

# Part 2: Phenological Synchrony

## What is Phenological Synchrony?

**Phenological synchrony** describes the degree to which phenological events occur at similar times across different locations within a region during a given period. It addresses the question: Do stations experience the same phenological phase at roughly the same time?


### Visualizing Synchrony Concepts

```
High Synchrony (SD = 5 days)        Low Synchrony (SD = 25 days)

Station A: ●────────●               Station A: ●────────●
Station B:  ●────────●              Station B:          ●────────●
Station C: ●────────●               Station C:   ●────────●
Station D:  ●────────●              Station D:              ●────────●
           ▲                                    ▲
           All stations flower                  Stations flower at
           within ~10 day window                very different times
```

### Why Study Synchrony?

1. **Ecosystem coherence**: High synchrony suggests strong regional climate
   control, whereas low synchrony indicates a greater influence of local factors.

2. **Monitoring network design**: When synchrony is high, fewer stations may be sufficient to characterize a region; low synchrony implies the need for denser observation networks.

3. **Detection of change**: Temporal changes in synchrony can indicate
   shifts in climate variability, increasing spatial heterogeneity, or emerging local adaptations.

4. **Ecological consequences**: Synchrony influences ecological interactions such as pollination, pest dynamics, and trophic mismatch.

## Calculating Synchrony

The `pheno_synchrony()` function calculates several metrics to quantify spatial
variability:

```{r synchrony-basic}
# Calculate synchrony by country and year
sync <- pheno_synchrony(
  pep = pep,
  species = "Malus domestica",
  phase_id = 60,
  by = c("country", "year"),
  min_stations = 3
)

print(sync)
```

### Understanding the Parameters

| Parameter | Purpose | Typical Values |
|-----------|---------|----------------|
| `pep` | Input dataset | Phenological dataset prepared with `pep725` |
| `species` | Species to analyze | Single species for clear interpretation |
| `phase_id` | Phenophase coded by the BBCH-scale (Meier, 2018) | e.g. 60 (flowering), 65 (full flowering) |
| `by` | Grouping variables | `c("country", "year")` for regional, year-specific analysis |
| `min_stations` | Minimum number of stations per group | 3-5 as a lower bound; 10+ recommended for robust estimates |
| `compute_trend` | Estimate temporal trends in synchrony | TRUE (default) when assessing changes over time |

## Understanding Synchrony Metrics

Different synchrony metrics capture complementary aspects of spatial variability. Selecting an appropriate metric depends on the analysis goal and data characteristics:

| Metric | Formula | Interpretation | When to Use |
|--------|---------|----------------|-------------|
| `sd_doy` | Standard deviation of day-of-year | Absolute variability in days | Comparing synchrony across phases or regions |
| `cv_pct` | (SD/mean) × 100 | Relative variability | Comparing phases with different mean timing |
| `range_doy` | Maximum minus minimum doy | Total observed spread | Assessing extreme differences |
| `iqr_doy` | 75th - 25th percentile | Robust measure of spread | Preferred when outliers are present |

**Interpretation guide:**

The following table provides a rule-of-thumb interpretation of synchrony based on the standard deviation of phenological timing across stations. Thresholds are indicative and should be interpreted in relation to species, phenophase, and spatial extent:

| SD Value | Interpretation |
|----------|----------------|
| < 5 days | Very high synchrony; stations behave very similarly |
| 5-10 days | Moderate synchrony; regional pattern dominates |
| 10-20 days | Low synchrony; local variability is substantial |
| > 20 days | Very low synchrony; local factors dominate |

```{r synchrony-summary}
summary(sync)
```

## Temporal Trends in Synchrony

An important question in phenological analysis is whether spatial synchrony changes over time.

- **Decreasing synchrony**: means growing spatial heterogeneity, for example due to more variable weather or diverging local conditions.
- **Increasing synchrony**: suggests stronger regional coherence, potentially driven by a common climate signal overriding local factors

```{r synchrony-trend}
# Check if trend analysis was performed
if (!is.null(sync$trend) && nrow(sync$trend) > 0) {
  cat("Trend Analysis Results:\n")
  print(sync$trend)

  # Interpret significant trends
  sig_trends <- sync$trend[!is.na(significant) & significant == TRUE]
  if (nrow(sig_trends) > 0) {
    cat("\nSignificant trends detected:\n")
    print(sig_trends[, .(country, slope, direction, p_value)])
  }
}
```

### Interpreting Trend Results

| Trend Direction | Interpretation | Ecological meaning |
|-----------------|---------------|-------------------------|
| Decreasing SD | Increasing synchrony | Phenological timing becoming more spatially coherent |
| Increasing SD | Decreasing synchrony | Growing spatial variability among stations |
| No trend | Stable synchrony | Persistent spatial patterns over time |

**Cautions with trend interpretation:**
- Changes in the observation network (e.g. stations added or removed) can introduce artificial trends.
- Reliable trend detection typically requires at least 15–20 years of data.
- Statistical significance should be interpreted alongside ecological relevance and effect size.

## Visualizing Synchrony Over Time

```{r synchrony-plot, fig.height=5}
# Plot synchrony time series
if (nrow(sync$data[!is.na(sd_doy)]) > 5) {
  p <- plot(sync)
  print(p)
}
```

**Reading the plot:**
- Y-axis shows the synchrony metric (typically the SD of timing)
- The fitted trend line indicates long-term change
- Scatter around trend reflects year-to-year variability in synchrony

## Synchrony Without Trend Analysis

For exploratory analyses or descriptive summaries, synchrony can be calculated without estimating temporal trends:

```{r synchrony-simple}
sync_simple <- pheno_synchrony(
  pep = pep,
  species = "Malus domestica",
  phase_id = 60,
  by = c("country", "year"),
  min_stations = 3,
  compute_trend = FALSE
)

# Just the synchrony data
head(sync_simple$data)
```

This approach is useful for rapid assessment or when time series length is insufficient for robust trend estimation.

### Comparing Synchrony: Grapevine vs. Apple

Different species may show different synchrony levels due to their biology:

```{r synchrony-compare}
# Calculate synchrony for grapevine
vine_check <- pep[species == "Vitis vinifera" & phase_id == 65]

if (nrow(vine_check) > 0) {
  sync_vine <- pheno_synchrony(
    pep = pep,
    species = "Vitis vinifera",
    phase_id = 65,
    by = c("country", "year"),
    min_stations = 3,
    compute_trend = FALSE
  )

  cat("Synchrony comparison (mean SD across years):\n")
  cat("Apple (flowering):    ", round(sync$overall$mean_sd_doy, 1), "days\n")
  cat("Grapevine (flowering):", round(sync_vine$overall$mean_sd_doy, 1), "days\n")
} else {
  cat("Insufficient grapevine data for synchrony comparison.\n")
}
```

---

# Part 3: Combining Gradient and Synchrony Analysis

A comprehensive spatial phenological analysis typically combines gradient and synchrony approaches. While each addresses a different aspect of spatial structure, together they provide a more complete picture of how phenology varies across landscapes.


| Analysis | Question Answered |
|----------|-------------------|
| Gradient | How does MEAN phenological timing change across space? |
| Synchrony | How VARIABLE is phenology within regions? |

Gradients describe systematic spatial shifts (e.g. with elevation or latitude), whereas synchrony captures the degree of spatial coherence around these shifts.

```{r combined-analysis}
# 1. Understand elevation gradient
gradient <- pheno_gradient(
  pep, variable = "alt",
  species = "Malus domestica",
  phase_id = 60
)

cat("Elevation Gradient Analysis:\n")
cat("  Slope:", round(gradient$summary$slope, 2), "days/100m\n")
cat("  R-squared:", round(gradient$summary$r_squared, 3), "\n\n")

# 2. Assess spatial synchrony
synchrony <- pheno_synchrony(
  pep,
  species = "Malus domestica",
  phase_id = 60,
  min_stations = 3
)

cat("Synchrony Analysis:\n")
cat("  Mean SD across stations:", round(synchrony$overall$mean_sd_doy, 1), "days\n")
cat("  Mean CV:", round(synchrony$overall$mean_cv_pct, 1), "%\n")
```

The two analyses answer complementary questions: the gradient quantifies the direction and strength of spatial change, while synchrony indicates how tightly phenological timing aligns around that spatial pattern.

## Interpreting Combined Results

Interpreting gradients and synchrony together helps to identify the dominant controls on phenology within a region:

| Gradient | Synchrony | Interpretation | Example |
|----------|-----------|----------------|---------|
| Strong | High | Clear environmental control with uniform response | Continental climate regions |
| Strong | Low | Environmental control with strong local variation | Mountain regions with microclimates |
| Weak | High | Broad regional climate signal dominates| Coastal or maritime regions |
| Weak | Low | Phenology shaped by complex local factors | Highly fragmented landscapes |

This combined perspective is particularly useful for comparing regions, diagnosing model performance, and identifying where local processes may override large-scale climatic drivers.

---

# Part 4: Mapping Phenological Patterns

Spatial visualization is a critical component of phenological analysis. Before formal modeling, maps help to assess station coverage, identify spatial gaps, and explore geographic patterns in phenological timing. The `pep725` package provides two mapping functions:

| Function | Type | Best suited for |
|----------|------|----------|
| `pheno_leaflet()` | Interactive | Data exploration and station selection |
| `pheno_map()` | Static | Analysis summaries and publication-quality figures |

## Interactive Maps with pheno_leaflet()
The `pheno_leaflet()` function launches an interactive Shiny-based map for exploring
phenological station networks. It is particularly useful during the early stages of analysis for:

- exploring spatial coverage and station density,
- interactively selecting subsets of stations,
- understanding geographic context before aggregation or modeling.

```{r leaflet-example, eval=FALSE}
# Launch interactive map (opens in viewer)
# Draw rectangles or polygons to select stations
selected_stations <- pheno_leaflet(pep, label_col = "species")

# The function returns a data.frame of selected stations
print(selected_stations)
```

```{r leaflet-screenshot, echo=FALSE, out.width="100%", fig.cap="The pheno_leaflet() interface showing station locations with drawing tools for selection."}
# Display screenshot if it exists
screenshot_path <- "pheno_leaflet_screenshot.png"
if (file.exists(screenshot_path)) {
  knitr::include_graphics(screenshot_path)
} else {
  # Placeholder message when screenshot doesn't exist
  cat("*[Screenshot: Interactive map showing PEP725 stations across Europe with drawing toolbar for rectangle and polygon selection]*\n")
}
```

### Features of pheno_leaflet()

The interactive map supports multiple selection and inspection tools:

| Feature | Purpose |
|---------|------------|
| **Click selection** | Select individual stations |
| **Rectangle selection** | Select rectangular geographic areas using the rectangle tool from the toolbar |
| **Polygon selection** | Define custom shapes to select irregular regions |
| **Hover labels** | Move mouse over points to inspect station metadata |
| **Done button** | Click "Done" to finalize and return selected stations |

The function returns a `data.frame` containing all selected stations with their
coordinates and metadata, which can then be directly reused in subsequent analysis.

### Best Practices for Interactive Mapping

**Tip**: For large datasets, filtering before mapping substantially improves performance.

```{r leaflet-filtered, eval=FALSE}
# Filter to a specific species for faster loading (recommended)
apple <- pep[species == "Malus domestica"]
selected <- pheno_leaflet(apple, label_col = "s_id")

# Use selected stations for focused analysis
apple_subset <- apple[s_id %in% selected$s_id]
```

**Why filter first?** The full PEP725 dataset contains millions of observations.
Rendering all these points in an interactive map can take long. Restricting the dataset to a species or region of interest results in faster loading and smoother interaction.

## Static Maps with pheno_map()

The `pheno_map()` function creates static maps suitable for reporting and publication. Two background options are supported:

| `background` | Description | API Key Required? |
|--------------|-------------|-------------------|
| `"none"` | Country borders from Natural Earth | No |
| `"google"` | Google Maps satellite/terrain imagery | Yes |

For most applications, the `background = "none"` option is recommended because it:
- requires no external services (API key setup),
- works reliably in package vignettes,
- produces reproducible, publication-ready maps

### Basic Station Maps

```{r map-basic, fig.height=6, fig.width=8}
# Basic station map with country borders (no API key needed)
pheno_map(pep, background = "none", color_by = "none", point_size = 1.5)
```

This view is useful for assessing overall network coverage and geographic gaps.

### Coloring by Data Attributes

```{r map-nobs, fig.height=6, fig.width=8}
# Color stations by number of observations
pheno_map(pep, background = "none", color_by = "n_obs", point_size = 1.5)
```

```{r map-nspecies, fig.height=6, fig.width=8}
# Color by species diversity at each station
pheno_map(pep, background = "none", color_by = "n_species", point_size = 1.5)
```

### Using Google Maps Background

For satellite imagery (requires API key from Google Cloud Console):

```{r map-google, eval=FALSE}
# Register your API key first
ggmap::register_google(key = "your_api_key_here")

# Then use background = "google"
pheno_map(pep, background = "google", color_by = "n_obs", zoom = 5)

# Regional focus with Google Maps
pheno_map(
  pep,
  background = "google",
  location = c(lon = 8.2, lat = 46.8),
  zoom = 7,
  color_by = "n_obs"
)
```

These maps help identify well-monitored stations and multi-species observation sites.

## Mapping Phenological Patterns

Beyond station metadata, `pheno_map()` can visualize derived phenological quantities, allowing spatial interpretation of biological signals.

### Mean Phenological Timing

```{r map-mean-doy, fig.height=6, fig.width=8}
# Map mean phenological timing for flowering (phase 60)
# Earlier timing = darker colors, later = lighter (plasma scale)
pheno_map(pep, background = "none", color_by = "mean_doy",
        phase_id = 60, point_size = 1.5)
```

This representation reveals spatial gradients in average phenological timing.

### Phenological Trends

```{r map-trend, fig.height=6, fig.width=8}
# Map trends per station
# Blue = phenology getting earlier, Red = getting later
pheno_map(pep, background = "none", color_by = "trend",
        phase_id = 60, period = 1990:2020, min_years = 10, point_size = 1.5)
```

Trend maps highlight regions where phenology is advancing or delaying over time.

### Species-level Variation

```{r map-species-cv, fig.height=6, fig.width=8}
# Map species variation at each station
# Higher CV = more variation in timing among species
pheno_map(pep, background = "none", color_by = "species_cv",
        phase_id = 60, min_species = 3, point_size = 1.5)
```

High inter-species variability may indicate ecological heterogeneity or mixed habitat conditions.

### Understanding `color_by` Options

| `color_by` | Meaning | Typical application |
|------------|---------------|----------------|
| `"none"` | Station locations | Observation network overview |
| `"n_obs"` | Observation density | Identification of well-monitored stations |
| `"n_species"` | Species richness | Finding multi-species stations |
| `"mean_doy"` | Mean timing of day-of-year | Reveal spatial phenology gradients |
| `"trend"` | Temporal change | Climate impact assessment |
| `"species_cv"` | Inter-species variability | Find stations with consistent timing |

### Parameter Reference for pheno_map()

| Parameter | Description | Default |
|-----------|-------------|---------|
| `background` | Map background: "none" or "google" | "google" |
| `location` | Map center as c(lon, lat) | European center |
| `zoom` | Zoom level (4=wide, 7=tight) | 4 |
| `color_by` | Coloring option | "none" |
| `phase_id` | BBCH phase for phenological colors | NULL |
| `period` | Years for trend calculation | All years |
| `min_years` | Minimum years for trend | 10 |
| `min_species` | Minimum species for CV | 3 |
| `point_size` | Marker size | 0.8 |
| `output_file` | Path to save map | NULL (display only) |

## Combining Mapping with Analysis

A typical workflow integrates mapping with spatial analysis:

```{r mapping-workflow, eval=FALSE}
# 1. Explore stations interactively
selected <- pheno_leaflet(pep[genus == "Malus"])

# 2. Subset data to selected region
pep_region <- pep[s_id %in% selected$s_id]

# 3. Analyze gradients in the selected region
gradient <- pheno_gradient(pep_region, variable = "alt",
                           species = "Malus domestica", phase_id = 60)

# 4. Assess synchrony
synchrony <- pheno_synchrony(pep_region, species = "Malus domestica",
                              phase_id = 60)

# 5. Create publication map of the region (no API needed)
pheno_map(pep_region, background = "none", color_by = "mean_doy",
        phase_id = 60, output_file = "regional_phenology.pdf")
```

---

# Best Practices Summary

## For Gradient Analysis

1. **Check sample size**: Need sufficient stations across the gradient range
   (minimum 20 stations, ideally 50+)

2. **Use robust methods**: Phenological data often has outliers from observation
   errors or microclimate effects - always start with `method = "robust"`

3. **Consider regional differences**: Gradients may vary by location - use
   the `by` parameter to check consistency

4. **Validate against literature**: Compare your results to published lapse
   rates (2-4 days/100m for altitude is typical in temperate Europe)

5. **Report uncertainty**: Always include R-squared and sample size when
   reporting gradient estimates

## For Synchrony Analysis

1. **Set appropriate minimum stations**: Too few stations gives unreliable
   estimates (minimum 3, ideally 10+)

2. **Consider temporal scale**: Annual synchrony captures different patterns
   than decadal averages

3. **Account for network changes**: Station additions/removals can create
   artificial trends - check if network composition changed

4. **Use appropriate metrics**: SD for absolute variability, CV for relative
   comparisons across phases

5. **Interpret trends cautiously**: Distinguish statistically significant
   trends from ecologically meaningful changes

## For Mapping

1. **Start with exploration**: Use `pheno_leaflet()` to understand your data
   before formal analysis

2. **Choose appropriate zoom**: Match zoom level to your study area
   (4=Europe, 6=country, 8=region)

3. **Use meaningful colors**: Choose `color_by` option that matches your
   research question

4. **Document selections**: Save selected station lists for reproducibility

5. **Export for publication**: Use `output_file` to create high-quality figures

---

## Summary

This vignette demonstrated how `pep725` supports spatial phenological analysis through complementary methods:

| Topic | Key Function | Main Output |
|-------|--------------|-------------|
| Elevation gradients | `pheno_gradient()` | Days delay per 100m |
| Latitude gradients | `pheno_gradient()` | Days delay per degree |
| Spatial synchrony | `pheno_synchrony()` | SD/CV of timing across stations |
| Interactive mapping | `pheno_leaflet()` | Selected station data.frame |
| Static mapping | `pheno_map()` | Publication-quality maps |

### Key Take-Home Messages

1. **Phenology delays systematically** with increasing altitude and latitude -
   quantifying these gradients helps scale point observations to landscapes

2. **Synchrony tells you about spatial variability** - high synchrony suggests
   strong regional climate control, low synchrony indicates local factors matter

3. **Combine approaches** for complete understanding - gradients show mean
   patterns, synchrony shows variability around those patterns

4. **Mapping is essential** for exploring data, selecting regions, and
   communicating results

## Next Steps

Explore the other vignettes for complementary analyses:

- **Getting Started**: Data access, the `pep` class, and basic exploration

```{r eval = FALSE}
vignette("getting-started", package = "pep725")
```

- **Phenological Analysis**: Normals, anomalies, quality assessment, and trends

```{r eval = FALSE}
vignette("phenological-analysis", package = "pep725")
```

- **Data Quality Assessment**: Outlier detection and data validation

```{r eval = FALSE}
vignette("data-quality", package = "pep725")
```

---

## Session Info

```{r session}
sessionInfo()
```

---
title: "Data Quality Assessment"
author:
  - Matthias Templ^[University of Applied Sciences Northwestern Switzerland (FHNW)]
  - Barbara Templ^[Swiss Federal Institute for Forest, Snow and Landscape Research (WSL)]
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{Data Quality Assessment}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

*Vignette 2 of 4 for the **pep725** R package.
The most current version is available on
[GitHub](https://github.com/matthias-da/pep725) and
[CRAN](https://CRAN.R-project.org/package=pep725).*

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

Data quality assessment is a critical but often overlooked step in phenological
analysis. Before calculating trends, normals, or climate sensitivities, you need
to understand what's in your data and make informed decisions about unusual
observations.

### The Data Quality Challenge

Long-term phenological datasets such as PEP725 are compiled from many observers over decades. This richness enables large-scale analyses, but it also introduces data quality challenges that need to be assessed:

| Issue Type | Examples | Consequence if Ignored |
|------------|----------|------------------------|
| **Recording errors** | Typographical errors (DOY 150 instead of 105), incorrect year | Outliers bias statistics |
| **Identification errors** | Confusion between morphologically similar species | Inconsistent or mixed time series |
| **Protocol changes** | Changes in the definition of phenological phases over time | Artificial shifts or trends |
| **Biological anomalies** | Irregular phenological events as a result of climate extremes | May represent real biological signals |

The following sections introduce practical tools in pep725 package for diagnosing and handling these issues in a transparent and reproducible way.

### What You'll Learn

| Section | Topic | Key Functions |
|---------|-------|---------------|
| Part 1 | Detecting statistical outliers | `pep_flag_outliers()`, `pep_plot_outliers()` |
| Part 2 | Temporal coverage and completeness | `pep_completeness()` |
| Part 3 | Phase presence validation | `pep_check_phases()` |
| Part 4 | Integrated workflow | Combining all approaches |

### Prerequisites

This vignette assumes you are familiar with:

- Basic phenological concepts (from "Getting Started" vignette)
- Day of Year (DOY) as a timing metric
- BBCH phenological phase codes

## Setup

```{r setup, message=FALSE, warning=FALSE}
library(pep725)
library(data.table)

# Download the synthetic dataset
pep <- pep_download()

# For this vignette, we'll focus on flowering phases
flowering <- pep[phase_id %in% c(60, 65)]
cat("Flowering observations:", nrow(flowering), "\n")
cat("Species:", length(unique(flowering$species)), "\n")
cat("Year range:", min(flowering$year), "-", max(flowering$year), "\n")
```

---

# Part 1: Outlier Detection

## Why Detect Outliers?

An **outlier** is an observation that differs substantially from the expected
pattern. In phenological data, outliers may indicate:

1. **Recording errors** that should be excluded from analysis
   - A typo: DOY 250 instead of DOY 150 (roughly 3 months difference)
   - Wrong year entered: observation assigned to incorrect season

2. **Unusual weather events** worth investigating
   - Very warm winter causing early flowering
   - Late frost delaying spring phenology

3. **Second flowering** or other abnormal phenological events
   - Plants flowering again in autumn/winter after spring flowering
   - Increasingly observed with climate change

**The challenge**: You can't simply delete all outliers. Some are errors, but
others are scientifically valuable observations of unusual events.

## Statistical Methods for Outlier Detection

The `pep_flag_outliers()` function provides four statistical methods for identifying
potential outliers. Each method has different assumptions and strengths:

### Method 1: 30-Day Rule (Simple Threshold)

The simplest approach: flag any observation more than 30 days from the median
for that species/phase combination.

```{r flag-basic}
# Flag outliers using the default 30-day rule
outliers <- pep_flag_outliers(
  pep = flowering,
  method = "30day",
  by = c("species", "phase_id")
)

print(outliers)
```

**How it works:**

```
                     Distribution of flowering dates

        ← Early                                    Late →

   ----|----[=======|=======]----|-----
        ^           ^           ^
     -30 days    median      +30 days

   Anything outside the brackets is flagged as an outlier
```

### Understanding the Output

The function adds new columns to your original data:

| Column | What It Contains | How to Use It |
|--------|------------------|---------------|
| `is_outlier` | TRUE/FALSE flag | Filter data: `data[is_outlier == FALSE]` |
| `deviation` | Days from expected | Prioritize investigation: larger = more extreme |
| `expected_doy` | Reference DOY (median) | Understand what "normal" looks like |

### Method 2: MAD (Median Absolute Deviation) - Recommended

The MAD method is **robust to the presence of multiple outliers**. This is
important because if your data contains many outliers, they can distort
mean-based methods.

```{r flag-methods, eval=FALSE}
# MAD method: flag if > 3 MAD from median (robust)
outliers_mad <- pep_flag_outliers(flowering, method = "mad", threshold = 3)
```

**Why MAD is recommended:**

| Scenario | Mean-based method | MAD method |
|----------|-------------------|------------|
| 1-2 outliers | Works reasonably | Works well |
| Many outliers | Outliers inflate SD, may miss more | Stays robust |
| Asymmetric data | Assumes symmetry | Handles asymmetry |

### Method 3: IQR (Interquartile Range) - Also Robust

The IQR method uses quartiles and is familiar from boxplot "whiskers":

```{r iqr-method, eval=FALSE}
# IQR method: flag if outside 1.5 * IQR (standard boxplot rule)
outliers_iqr <- pep_flag_outliers(flowering, method = "iqr", threshold = 1.5)
```

**How it works:**
- Q1 = 25th percentile, Q3 = 75th percentile
- IQR = Q3 - Q1
- Outlier if: value < Q1 - 1.5×IQR OR value > Q3 + 1.5×IQR

### Method 4: Z-Score - Sensitive but Less Robust

The z-score method assumes normally distributed data:

```{r zscore-method, eval=FALSE}
# Z-score method: flag if |z| > 3 (assumes normal distribution)
outliers_zscore <- pep_flag_outliers(flowering, method = "zscore", threshold = 3)
```

**Caution**: The z-score method is sensitive to existing outliers. If your data
has many outliers, they will inflate the standard deviation, making the z-score
threshold less effective at detecting them.

### Method 5: GAM Residual - Model-based, Covariate-aware

The four methods above are all **univariate within a group** — they compare
each observation to the central tendency of its own station-species-phase
combination. That misses three realistic outlier patterns:

1. **Year-effect outliers.** In a year where most of central Europe bloomed
   8 days earlier, a station that flowered on its 30-year median looks
   "late" only because the reference ignores the shared weather signal.
2. **Covariate-inconsistent reports.** A lowland station that reports a
   DOY typical of a mountain station is invisible to a within-station
   detector because the station's own history may be short.
3. **Trend-inconsistent reports.** A station whose DOY is constant while
   every other station in the network has a trend is anomalous only in
   relation to the trend.

`method = "gam_residual"` tackles all three. For each group (typically
species × phase), it fits a GAM of DOY on year, altitude, latitude, and a
station random intercept (customisable via `formula =`) and flags
observations whose residual — standardised by a robust MAD scale — exceeds
the threshold:

```{r gam-method, eval=FALSE}
# Model-based detection across the whole species x phase group.
outliers_gam <- pep_flag_outliers(
  apple_flowering,
  by = c("genus", "species", "phase_id"),
  method = "gam_residual",
  threshold = 3.5
)
```

The `deviation` column now holds GAM residuals in days (not deviations
from a within-station median), and `expected_doy` holds the fitted
values — so the station-year whose residual is 20 days above the fit is
flagged whether or not its station's history made that value look
"normal".

Groups with fewer observations than `min_n_per_group` (default 50) or
where the GAM fails to converge silently fall back to the 30-day rule.

### Method 6: Mahalanobis - Multivariate, Robust (MCD)

All methods above flag observations one at a time. A station-year could
have an entirely reasonable BBCH 60 date and an entirely reasonable BBCH
65 date, but only **one day apart** — biologically impossible, because
first flowering precedes full flowering by ~10 days.

`method = "mahalanobis"` catches this by treating each station-year as
a *vector* across phases and computing the **robust Mahalanobis
distance** of that vector from the centre of the rest of the network.
"Robust" matters: the classical Mahalanobis distance uses the sample
mean and covariance, which are themselves pulled by outliers
(masking). We use the **Minimum Covariance Determinant** estimator
(Rousseeuw 1984, `robustbase::covMcd()`) which gives a high-breakdown
robust centre and scatter, so the outliers stay *outside* the robust
ellipsoid instead of contaminating it:

```{r mahalanobis-method, eval=FALSE}
# Multivariate detection on the phase profile per station-year.
# by = c("genus", "species") pools across phases so the covariance
# is estimated from the joint (BBCH 60, BBCH 65, BBCH 89) distribution.
outliers_mahal <- pep_flag_outliers(
  apple,
  by = c("genus", "species"),
  method = "mahalanobis"
)
```

The default threshold is `sqrt(qchisq(0.975, df = p))` where *p* is the
number of phases — the standard chi-square containment cut-off under
multivariate normality. Flagged station-years get all their phase rows
marked; `deviation` stores the robust MD (not a day-count, so it is
unitless).

### Choosing a Method

| Method | Best For | Threshold Meaning |
|--------|----------|-------------------|
| `30day` | Simple, interpretable threshold | Absolute days from median |
| `mad` | Most situations (recommended for single-phase) | Number of MADs from median |
| `iqr` | Familiar boxplot-style detection | IQR multiplier |
| `zscore` | Clean data, normal distribution | Standard deviations |
| `gam_residual` | Year / elevation / station covariates available | Robust z on GAM residuals |
| `mahalanobis` | Multiple phases per station-year | `sqrt(chi^2_{0.975, p})` default |

### Summary Statistics

```{r flag-summary}
summary(outliers)
```

### Comparing Outlier Rates Across Species

Different species may have different outlier rates due to observation difficulty
or data quality:

```{r flag-compare}
# Check outliers for grapevine
vine_flowering <- pep[species == "Vitis vinifera" & phase_id %in% c(60, 65)]

if (nrow(vine_flowering) > 0) {
  outliers_vine <- pep_flag_outliers(
    pep = vine_flowering,
    method = "30day",
    by = c("species", "phase_id")
  )

  cat("Outlier comparison:\n")
  cat("All flowering species: ", round(100 * mean(outliers$is_outlier), 2), "%\n")
  cat("Grapevine only:        ", round(100 * mean(outliers_vine$is_outlier), 2), "%\n")
} else {
  cat("No grapevine flowering data available for comparison.\n")
}
```

## Visualizing Outliers

Visual inspection is essential for understanding outlier patterns. The
`pep_plot_outliers()` function provides several visualization types —
four for the univariate methods plus two targeted at the model-based
and multivariate methods:

### 1. Overview Plot

Shows the big picture: how many outliers, which species/phases are affected:

```{r plot-overview, fig.height=6}
# Overview of outlier patterns
pep_plot_outliers(outliers, type = "overview")
```

**What to look for:**
- Which species have the most outliers? (Might indicate data quality issues)
- Are outliers concentrated in certain phases? (Might indicate definition problems)
- What's the overall outlier rate? (Typical: 1-5% for well-curated data)

### 2. Seasonal Distribution

When in the year do outliers occur?

```{r plot-seasonal, fig.height=5}
# When do outliers occur in the year?
pep_plot_outliers(outliers, type = "seasonal")
```

**Interpretation guide:**

| Outlier Timing | Likely Explanation |
|----------------|-------------------|
| Very early (DOY < 50) | Possible data errors (flowering in January/February unlikely for most species) |
| Somewhat early | Warm winters or Mediterranean locations |
| Somewhat late | Cool springs or high-altitude stations |
| Very late (DOY > 250) | Possible **second flowering** events - investigate! |

### 3. Detailed Context

See outliers alongside normal observations:

```{r plot-detail, fig.height=5}
# See outliers in context of all observations
pep_plot_outliers(outliers, type = "detail", n_top = 15)
```

**This plot shows:**
- Full distribution of observations (gray)
- Flagged outliers (highlighted)
- The `n_top` parameter controls how many species/phases to show

### 4. Geographic Distribution

Where are outliers located?

```{r plot-map, fig.height=5}
# Where are outliers located?
pep_plot_outliers(outliers, type = "map")
```

**What to look for:**
- Clustered outliers in one region? Might indicate local data quality issues
- Outliers at network edges? Might be at environmental limits
- Random scatter? Suggests individual observation errors

### 5. Model Diagnostic (for `gam_residual` / `mahalanobis`)

`type = "diagnostic"` produces a paper-ready 4-panel figure tailored to
the detection method. For `gam_residual` the panels are classical GAM
diagnostics — residuals vs fitted, Q-Q against Normal, |residual| vs
altitude, and a per-station worst-case-residual map. For `mahalanobis`
the panels switch automatically to MD-specific diagnostics: sorted MD
with the chi-square threshold line, Q-Q of MD² against χ²_*p*,
mean/max MD over time (exposes network-wide anomalous epochs), and
the per-station worst-case MD map.

```{r plot-diagnostic-gam, eval=FALSE}
# For gam_residual: classical model diagnostics
outliers_gam <- pep_flag_outliers(
  apple_flowering,
  by = c("genus", "species", "phase_id"),
  method = "gam_residual"
)
pep_plot_outliers(outliers_gam, type = "diagnostic")
```

![GAM-residual diagnostic on a synthetic fixture with one injected contamination at station 1, year 2000. Panel A: residuals vs fitted; the injected point appears at residual ≈ +50 days. Panel B: Q-Q against N(0,1); the right-tail departure corresponds to the outlier. Panel C: |residual| vs altitude, showing the model has no altitude bias away from the outlier. Panel D: per-station worst-case residual map.](figures/gam_diagnostic.png)

```{r plot-diagnostic-mahal, eval=FALSE}
# For mahalanobis: MD-specific diagnostics
outliers_mahal <- pep_flag_outliers(
  apple,
  by = c("genus", "species"),
  method = "mahalanobis"
)
pep_plot_outliers(outliers_mahal, type = "diagnostic")
```

![Mahalanobis-specific diagnostic on the same synthetic fixture. Panel A: sorted robust Mahalanobis distances across all station-years, with the $\sqrt{\chi^2_{0.975,\,3}} \approx 3.06$ threshold as a red dashed line; flagged points are red. Panel B: Q-Q of observed $\mathrm{MD}^2$ against $\chi^2_3$ — upper-tail departure identifies the outlier mass. Panel C: mean (blue) and max (red dotted) MD per year — the spike at year 2000 coincides with the injected anomaly. Panel D: per-station worst-case MD map.](figures/mahalanobis_diagnostic.png)

**What to look for:**

- **Residuals vs fitted (GAM)**: a level trend with no structure is
  ideal; a funnel or curve signals model misspecification, not outliers.
- **Q-Q (GAM)**: heavy-tailed deviations in the corners are the real
  outlier signal — light tails mean the threshold is picking up noise.
- **Sorted MD with threshold (Mahalanobis)**: the steep rise at the
  right-hand end is where flagged station-years sit; a long flat
  "knee" across the threshold means the threshold is arguably too
  loose or too strict.
- **Mean / max MD over time (Mahalanobis)**: spikes in the max line
  identify *years* in which many stations reported biologically
  inconsistent phase sequences — useful for correlating with known
  data-collection or instrumental events.

### 6. Phase-profile Plot (primary Mahalanobis figure)

The MD detector flags station-years whose *shape* across phases is
atypical, even when each marginal DOY looks fine. `type = "profile"`
plots the phase profile as parallel coordinates — one line per
station-year, flagged ones in red, with the robust median profile
overlaid in black as a reference:

```{r plot-profile, eval=FALSE}
# Phase-profile parallel coordinates (primary Mahalanobis figure).
pep_plot_outliers(outliers_mahal, type = "profile")
```

![Phase-profile parallel-coordinates plot. Each line is one station-year; the grey envelope shows the natural variability of the network, the red lines are station-years flagged by the robust Mahalanobis detector, and the thick black line is the MCD robust per-phase median profile. Red lines that cross the phases at atypical angles — not just at unusual absolute DOYs — are the classic Mahalanobis flags.](figures/mahalanobis_profile.png)

**How to read it:**

- Every grey line is a "normal-looking" station-year; together they
  form the envelope of the robust ellipsoid.
- Red lines are flagged. Look where they bend oddly: a station-year
  whose BBCH 60 → 65 slope is much flatter or steeper than the
  reference is the classic MD flag.
- The black reference profile is the robust (MCD) per-phase median —
  the centre of the ellipsoid. Anything strongly parallel to this is
  safe; anything that crosses multiple phases at unusual angles is
  suspect.

---

# Part 2: Data Completeness

## Why Check Completeness?

**Data completeness** refers to how many years of observations exist for each
station/phase combination. This matters because:

1. **Trend analysis requires continuous data**: Gaps can bias trend estimates
2. **Normals require representative coverage**: WMO recommends 24+ years in a 30-year period
3. **Missing data isn't random**: Stations often drop out or are added systematically

### Visualizing Completeness Issues

```
Station A: ●●●●●●●●●●●●●●●●●●●●●●●●●●●●●●  <- 100% complete
           1990                      2020

Station B: ●●●●●●●●●●○○○○○○○○○○●●●●●●●●●●  <- Gap in middle
           1990                      2020

Station C: ○○○○○○○○○○○○○○○○○○○○●●●●●●●●●●  <- Only recent data
           1990                      2020

Station D: ●●●●●●●●●●●●●●●●●●●●○○○○○○○○○○  <- Discontinued
           1990                      2020
```

Different completeness patterns have different implications:

| Pattern | Implications for Analysis |
|---------|---------------------------|
| Full coverage | Ideal - reliable trends and normals |
| Gap in middle | Careful - may miss important changes |
| Only recent | Cannot compare to historical period |
| Discontinued | Cannot assess recent changes |

## Assessing Completeness

```{r completeness-basic}
# Check completeness by station and phase
# Use year_range to focus on a specific period
completeness <- pep_completeness(
  pep = flowering,
  by = c("s_id", "phase_id"),
  year_range = c(1990, 2020)
)

print(completeness)
```

### Understanding Completeness Metrics

The function calculates several metrics:

| Metric | What It Measures | Interpretation |
|--------|------------------|----------------|
| `n_years` | Total years with data | More = better |
| `completeness_pct` | % of year span covered | 70%+ for trends, 80%+ for normals |
| `year_span` | Total span (year_max - year_min + 1) | Context for completeness |
| `year_min` | Earliest observation | Needed for historical comparison |
| `year_max` | Most recent observation | Needed for current status |

### Summary Statistics

```{r completeness-summary}
summary(completeness)
```

## Filtering by Completeness

Use completeness information to select stations appropriate for your analysis:

```{r completeness-filter}
# Get stations with good coverage (>= 70%)
good_coverage <- completeness[completeness_pct >= 70]
cat("Stations with >= 70% coverage:", nrow(good_coverage), "\n")

# Use these for trend analysis
good_stations <- unique(good_coverage$s_id)
flowering_complete <- flowering[s_id %in% good_stations]
cat("Observations from complete stations:", nrow(flowering_complete), "\n")
```

### Completeness Thresholds for Different Analyses

| Analysis Type | Minimum Completeness | Reasoning |
|---------------|----------------------|-----------|
| Trend detection | 50% (15+ years) | Need enough points for trend fitting |
| Climate sensitivity | 60% | Need variability across climate conditions |
| 30-year normals | 80% (24+ years) | WMO standard requirement |
| Station comparison | Same time period | Avoid bias from different eras |

## Visualizing Completeness

```{r completeness-plot, fig.height=5}
plot(completeness)
```

---

# Part 3: Phase Presence Validation

## Why Check Phase Presence?

Before starting an analysis, it's important to verify that your data contains
the phenological phases you need. The `pep_check_phases()` function validates that
expected BBCH phase codes are present in your data.

**Common questions this helps answer:**

- Does my dataset have flowering observations (phase 60)?
- Are flowering and fruit maturity phases both available for apple?
- Which phases are missing for my species of interest?

## Checking Phase Presence

```{r check-basic}
# Check if expected phases are present for apple
apple <- pep[species == "Malus domestica"]

phase_check <- pep_check_phases(
  pep = apple,
  expected = c(60, 65, 87)  # flowering, full flowering, fruit maturity
)

print(phase_check)
```

### Understanding the Output

The function returns information about phase coverage:

| Output | What It Shows |
|--------|---------------|
| `expected` | The phases you asked for |
| `present` | Which expected phases were found |
| `missing` | Which expected phases are absent |
| `complete` | TRUE if all expected phases are present |
| `n_obs` | Number of observations per phase |

## Checking Multiple Species

To check phase presence across multiple species at once:

```{r check-multi}
# Check phases for multiple species
multi_check <- pep_check_phases_multi(
  pep = pep,
  species_list = c("Malus domestica", "Vitis vinifera"),
  expected = c(60, 65, 87)
)

print(multi_check)
```

### Common Phases to Check

| Plant Type | Common Phases | BBCH Codes |
|------------|---------------|------------|
| Cereals | Heading, Flowering, Harvest | 60, 65, 100 |
| Fruit trees | Flowering, Full flowering, Fruit maturity | 60, 65, 87 |
| Deciduous trees | Leaf unfolding, Flowering, Leaf fall | 11, 60, 95 |

---

# Part 4: Integrated Quality Workflow

## Putting It All Together

Here's a recommended workflow for data quality assessment that combines all the
tools covered in this vignette:

```{r workflow}
# ══════════════════════════════════════════════════════════════════════════════
# STEP 1: Assess temporal completeness
# ══════════════════════════════════════════════════════════════════════════════
# Why: Incomplete stations can bias trend estimates and normals

completeness <- pep_completeness(flowering, by = c("s_id", "phase_id"))
good_stations <- completeness[completeness_pct >= 50, s_id]
fl_filtered <- flowering[s_id %in% good_stations]

cat("Kept", length(good_stations), "stations with >= 50% completeness\n")

# ══════════════════════════════════════════════════════════════════════════════
# STEP 2: Flag statistical outliers
# ══════════════════════════════════════════════════════════════════════════════
# Why: Identify observations that deviate from expected patterns

outliers_wf <- pep_flag_outliers(fl_filtered, method = "mad", threshold = 3)

cat("Flagged", sum(outliers_wf$is_outlier), "outliers",
    "(", round(100 * mean(outliers_wf$is_outlier), 1), "% )\n")

# ══════════════════════════════════════════════════════════════════════════════
# STEP 3: Make informed decisions about exclusion
# ══════════════════════════════════════════════════════════════════════════════
# Key principle: Document your decisions!

# Option A: Strict cleaning (for normals calculation)
fl_strict <- outliers_wf[is_outlier == FALSE]

# Option B: Moderate cleaning (keep moderate outliers)
fl_moderate <- outliers_wf[is_outlier == FALSE | abs(deviation) < 60]

cat("Strict cleaning keeps:", nrow(fl_strict), "obs\n")
cat("Moderate cleaning keeps:", nrow(fl_moderate), "obs\n")

# ══════════════════════════════════════════════════════════════════════════════
# STEP 4: Proceed with analysis on cleaned data
# ══════════════════════════════════════════════════════════════════════════════

normals <- pheno_normals(fl_moderate, period = 1991:2020, min_years = 5)
cat("Normals calculated for", sum(!is.na(normals$mean_doy)), "groups\n")
```

## Documentation Template

When publishing research, document your quality control decisions:

```
Data Quality Control:
1. Completeness filtering: Excluded stations with < 50% temporal coverage
   (reduced dataset from N to M stations)

2. Outlier detection: Used MAD method with threshold = 3
   (flagged X observations as outliers, Y% of total)

3. Outlier disposition:
   - Excluded: Z observations with deviations > 60 days (likely errors)
   - Retained: W observations identified as abnormal events (e.g., second flowering)

4. Phase validation: Checked for sequence violations in [species list]
   (identified V cases for manual review)
```

---

# Best Practices Summary

## For Outlier Detection

1. **Use robust methods**: MAD or IQR methods handle multiple outliers better
   than z-scores. The MAD method is generally recommended.

2. **Check seasonally**: Late-season outliers (DOY > 250) may be biologically
   meaningful events (e.g. second flowering) rather than errors.

3. **Verify extremes**: For observations with very large deviations (> 60 days),
   try to check original data sources or contact data providers.

4. **Document decisions**: Record which outliers you exclude and why. This is
   essential for reproducibility and peer review.

5. **Don't delete automatically**: Human judgment is needed to distinguish
   errors from genuine unusual events.

## For Abnormal Event Detection

1. **Consider species biology**: Some species (e.g., certain Rosaceae) are more
   prone to second flowering than others.

2. **Check geographic patterns**: If multiple nearby stations report late events
   in the same year, this suggests a real regional phenomenon.

3. **Look at weather context**: Link abnormal events to weather extremes -
   drought is a common trigger.

4. **Distinguish from errors**: Very isolated observations (single station,
   single year) need verification before treating them as e.g. second flowering.

5. **Keep for separate analysis**: Abnormal phenological events are scientifically
   interesting (proof of climate change) - don't just delete them!

## For Completeness Assessment

1. **Set appropriate thresholds**: Use 80%+ coverage for calculating normals,
   50%+ for trend analysis.

2. **Consider gap patterns**: Many small gaps are usually better than one long
   gap that might coincide with important climate changes.

3. **Check temporal bias**: Ensure you're not comparing stations with data only
   from early periods to stations with only recent data.

4. **Document station selection**: Report how many stations met your completeness
   criteria and how this affected your sample.

## For Phase Presence Checking

1. **Check early**: Run `pep_check_phases()` at the start of your analysis to
   verify required phases are available in your data.

2. **Know required phases**: Different analyses need different phases -
   know which BBCH codes you need before starting.

3. **Check across species**: Use `pep_check_phases_multi()` to verify data
   availability for all species you plan to analyze.

---

## Summary

This vignette covered data quality tools for phenological analysis:

| Function | Purpose | Key Output |
|----------|---------|------------|
| `pep_flag_outliers()` | Identify unusual observations | `is_outlier` flag, `deviation` in days |
| `pep_plot_outliers()` | Visualize outlier patterns | Four plot types: overview, seasonal, detail, map |
| `pep_completeness()` | Assess temporal coverage | Completeness %, gaps, year range |
| `pep_check_phases()` | Check phase presence | Missing phases, observation counts |

### Key Take-Home Messages

1. **Data quality assessment is not optional** - it's a critical step that
   affects the validity of all downstream analyses.

2. **Not all outliers are errors** - some represent genuine biological phenomena
   like second flowering that deserve further study.

3. **Document your decisions** - quality control choices should be transparent
   and reproducible.

4. **Use robust methods** - the MAD method is recommended for phenological data
   because it handles multiple outliers well.

5. **Visual inspection matters** - plots often reveal patterns that summary
   statistics miss.

## Next Steps

Explore the other vignettes for complementary analyses:

- **Getting Started**: Data access, the `pep` class, and basic exploration

```{r eval = FALSE}
vignette("getting-started", package = "pep725")
```

- **Phenological Analysis**: Normals, anomalies, quality grading, and trends

```{r eval = FALSE}
vignette("phenological-analysis", package = "pep725")
```

- **Spatial Phenological Patterns**: Gradients, synchrony, and mapping

```{r eval = FALSE}
vignette("spatial-patterns", package = "pep725")
```

---

## Session Info

```{r session}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_completeness.R
\name{print.pep_completeness}
\alias{print.pep_completeness}
\title{Print Method for Completeness Assessment}
\usage{
\method{print}{pep_completeness}(x, n = 15, ...)
}
\arguments{
\item{x}{A \code{pep_completeness} object}

\item{n}{Number of rows to display}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}
}
\description{
Print Method for Completeness Assessment
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_phenology_trends.R
\name{plot_phenology_trends}
\alias{plot_phenology_trends}
\title{Plot Robust Phenological Trends}
\usage{
plot_phenology_trends(
  pep,
  genus_name = NULL,
  species_name = NULL,
  subspecies_name = NULL,
  phases = c(65, 87),
  common_stations = TRUE,
  combine_regions = FALSE,
  combine_layout = c("vertical", "horizontal"),
  years = 1961:2024,
  calib_years = 1991:2020,
  pred_years = NULL,
  subregions = c("Austria", "Germany-North", "Germany-South", "Switzerland"),
  giss_data = NULL,
  layout = c("country_phase", "phase_country"),
  title = NULL
)
}
\arguments{
\item{pep}{A PEP725-style data frame with columns \code{year}, \code{DOY},
\code{phase_id}, \code{country}, \code{s_id}, \code{genus}, \code{species}.}

\item{genus_name}{Character string specifying a genus to filter (optional).}

\item{species_name}{Character string specifying a species to filter (optional).
If both genus and species are provided, the species filter is applied last.}

\item{subspecies_name}{Character string specifying a subspecies to filter (optional).
Only one of genus_name, species_name, or subspecies_name can be specified.}

\item{phases}{Integer vector of phenological phase IDs to include
(default: \code{c(65, 87)} for flowering and maturity of fruit).}

\item{common_stations}{Logical. If \code{TRUE} (default), only stations with observations
for all selected phases are included. If \code{FALSE}, all stations with any of the selected phases are included.}

\item{combine_regions}{Logical. If \code{TRUE}, combines all regions into a single panel. Default is \code{FALSE}.}

\item{combine_layout}{Character. Layout for combined regions plot: either \code{"vertical"} or \code{"horizontal"}.}

\item{years}{Numeric vector of years to include in the analysis.
Default is \code{1961:2024}.}

\item{calib_years}{Numeric vector specifying the calibration window for robust
regression (default: \code{1991:2020}).}

\item{pred_years}{Numeric vector of years for projecting future phenology.
Default is \code{1990:2090}.}

\item{subregions}{Character vector of country names to include
(default: DACH region).}

\item{giss_data}{Deprecated. Previously used for climate data integration;
now ignored.}

\item{layout}{Either \code{"country_phase"} (default) or \code{"phase_country"}
to control facet arrangement.}

\item{title}{Optional plot title. If \code{NULL}, the function generates one
from genus and species information.}
}
\value{
A \code{ggplot} object.
}
\description{
This function visualizes long-term phenological trends for one genus or species,
using robust MM-regression (\code{robustbase::lmrob}) applied to mean annual DOYs.
}
\details{
The function produces a faceted panel plot showing:
\itemize{
\item annual mean DOY and interquartile range,
\item robust trend lines for past and future periods,
\item CTRL (1991-2020) and PGW (2066-2095) climate windows,
\item optional layouts (country x phase or phase x country).
}

The function performs the following steps:
\enumerate{
\item Filter PEP data by species, phases, regions, years.
\item Aggregate DOYs by year-country-phase.
\item Fit robust regression \eqn{DOY \sim year} over the calibration window.
\item Predict past and future trends.
\item Draw ribbon (IQR), annual means, trend lines, climate windows.
\item Facet by country or by phase.
}

The robust regression uses \code{robustbase::lmrob}, which is resistant to
outliers and non-normality, and therefore ideal for phenological time series.
}
\examples{
\donttest{
pep <- pep_download()
plot_phenology_trends(
  pep,
  species_name = "Vitis vinifera",
  phases = c(65, 81),
  layout = "country_phase"
)
}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_second_events.R
\name{plot.second_events}
\alias{plot.second_events}
\title{Plot Method for Second Events Detection}
\usage{
\method{plot}{second_events}(
  x,
  type = c("overview", "timeline", "seasonal", "map"),
  scale = c("absolute", "relative"),
  from_year = NULL,
  ...
)
}
\arguments{
\item{x}{A \code{second_events} object}

\item{type}{Character. Plot type: "overview", "timeline", "seasonal", "map"}

\item{scale}{Character. Scale for y-axis in timeline/overview plots:
\describe{
\item{"absolute"}{(Default) Show raw counts of second events per year.}
\item{"relative"}{Show proportion of second events relative to total
observations per year. This accounts for varying data availability
over time and reveals whether second events are becoming proportionally
more common.}
}}

\item{from_year}{Integer. If specified, only show data from this year onwards
in timeline and overview plots. Useful for focusing on recent trends.
Default is \code{NULL} (show all years).}

\item{...}{Additional arguments}
}
\value{
A ggplot object (invisibly)
}
\description{
Plot Method for Second Events Detection
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_combine.R
\name{fit_lad}
\alias{fit_lad}
\title{Fit LAD (Robust) Model}
\usage{
fit_lad(dt)
}
\description{
Fit LAD (Robust) Model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{pep_coverage}
\alias{pep_coverage}
\title{Assess Data Coverage of PEP725 Phenological Data}
\usage{
pep_coverage(
  x,
  kind = c("all", "temporal", "geographical", "species"),
  top = 10,
  plot = FALSE,
  by = NULL
)
}
\arguments{
\item{x}{A \code{pep} object or data.frame with phenological data.}

\item{kind}{Character. Type of coverage to assess:
\itemize{
\item \code{"all"} (default): All coverage types combined
\item \code{"temporal"}: Year range, gaps, observations per year
\item \code{"geographical"}: Countries, stations, coordinate ranges
\item \code{"species"}: Species and genus diversity
}}

\item{top}{Integer. Number of top entries to show in tables (default: 10).}

\item{plot}{Logical. If \code{TRUE}, produces a visualization of the
coverage. Default is \code{FALSE}.}

\item{by}{Character. Optional grouping variable for more detailed analysis
(e.g., \code{"country"}, \code{"species"}). Only used for temporal coverage.}
}
\value{
A list of class \code{pep_coverage} containing coverage statistics.
The structure depends on \code{kind}:
\describe{
\item{temporal}{Year range, number of years, gaps, observations per year}
\item{geographical}{Countries, stations, coordinate bounds, altitude range}
\item{species}{Genera, species, subspecies counts and details}
\item{all}{All of the above combined}
}
}
\description{
Provides comprehensive coverage statistics for phenological datasets,
including temporal, geographical, and species dimensions.
}
\examples{
\donttest{
pep <- pep_download()
pep_ch <- pep[country == "Switzerland"]

# Full coverage report
pep_coverage(pep_ch)

# Temporal coverage only
pep_coverage(pep_ch, kind = "temporal")

# Species coverage, top 5
pep_coverage(pep_ch, kind = "species", top = 5)
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_anomaly.R
\name{plot.pheno_anomaly}
\alias{plot.pheno_anomaly}
\title{Plot Method for Phenological Anomalies}
\usage{
\method{plot}{pheno_anomaly}(x, which = c("timeline", "histogram"), ...)
}
\arguments{
\item{x}{A \code{pheno_anomaly} object returned by
\code{\link{pheno_anomaly}}.}

\item{which}{Character string selecting the plot type: \code{"timeline"}
(default) shows \code{anomaly_days} per year as bars coloured by
\code{direction} with extreme events marked; \code{"histogram"} shows
the distribution of anomaly values.}

\item{...}{Additional arguments (unused).}
}
\value{
A \code{ggplot} object (returned invisibly).
}
\description{
Visualise phenological anomalies as a colour-coded timeline or as a
histogram of anomaly magnitudes.
}
\examples{
\donttest{
pep <- pep_download()
vine_ch <- pep[country == "Switzerland" & species == "Vitis vinifera"]

if (nrow(vine_ch) > 0) {
  a <- pheno_anomaly(vine_ch,
                     baseline_period = 1961:1990,
                     phase_id = 65,
                     by = "phase_id")
  plot(a)
  plot(a, which = "histogram")
}
}

}
\seealso{
\code{\link{pheno_anomaly}}, \code{\link{pheno_normals}}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_completeness.R
\name{plot.pep_completeness}
\alias{plot.pep_completeness}
\title{Plot Method for Completeness Assessment}
\usage{
\method{plot}{pep_completeness}(x, type = c("heatmap", "bar", "timeline"), top = 20, ...)
}
\arguments{
\item{x}{A \code{pep_completeness} object}

\item{type}{Character. Type of plot: \code{"heatmap"} for completeness
heatmap (requires 2 grouping variables), \code{"bar"} for observation
counts, \code{"timeline"} for year coverage. Default is \code{"heatmap"}.}

\item{top}{Integer. Number of top groups to display. Default is 20.}

\item{...}{Additional arguments (unused)}
}
\value{
A ggplot object (invisibly)
}
\description{
Creates visualizations of data completeness, including heatmaps and
bar charts.
}
\examples{
\donttest{
pep <- pep_download()
pep_ch <- pep[country == "Switzerland"]
comp <- pep_completeness(pep_ch, by = c("genus", "phase_id"))
plot(comp, type = "heatmap")
plot(comp, type = "bar", top = 15)
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_check_phases.R
\name{pep_check_phases_multi}
\alias{pep_check_phases_multi}
\title{Check Multiple Phases Across Species}
\usage{
pep_check_phases_multi(
  pep,
  species_list,
  expected = c(60, 65, 100),
  year_min = NULL,
  year_max = NULL,
  warn = TRUE
)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table}}

\item{species_list}{Character vector of species/genus names to check}

\item{expected}{Integer vector of expected BBCH phase codes}

\item{year_min}{Optional minimum year filter}

\item{year_max}{Optional maximum year filter}

\item{warn}{Logical. Issue warnings for missing phases? Default TRUE.}
}
\value{
A \code{data.table} summarizing phase availability per species
}
\description{
A convenience function to check phase availability across multiple species
at once.
}
\examples{
\donttest{
# Check multiple species (use subset for speed)
pep <- pep_download()
pep_subset <- pep[country \%in\% c("Switzerland", "Austria")]
result <- pep_check_phases_multi(pep_subset,
                             species_list = c("Malus domestica",
                                              "Vitis vinifera"),
                             expected = c(60, 65))
print(result)
}

}
\seealso{
\code{\link{pep_check_phases}} for single-species checking
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_gradient.R
\name{plot.pheno_gradient}
\alias{plot.pheno_gradient}
\title{Plot Method for Phenological Gradient Analysis}
\usage{
\method{plot}{pheno_gradient}(x, ...)
}
\arguments{
\item{x}{A \code{pheno_gradient} object}

\item{...}{Additional arguments passed to ggplot}
}
\value{
A ggplot object
}
\description{
Plot Method for Phenological Gradient Analysis
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_trend_turning.R
\name{plot.pheno_turning}
\alias{plot.pheno_turning}
\title{Plot Method for Trend Turning Analysis}
\usage{
\method{plot}{pheno_turning}(x, group = NULL, show_thresholds = TRUE, ...)
}
\arguments{
\item{x}{A \code{pheno_turning} object}

\item{group}{Character. For grouped analysis, which group to plot.
If \code{NULL} (default), plots the first group or ungrouped results.}

\item{show_thresholds}{Logical. Show significance threshold lines?
Default \code{TRUE}.}

\item{...}{Additional arguments passed to ggplot}
}
\value{
A ggplot object (invisibly)
}
\description{
Plot Method for Trend Turning Analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_second_events.R
\name{pep_second_events}
\alias{pep_second_events}
\title{Detect Second Flowering and Other Repeated Phenological Events}
\usage{
pep_second_events(
  pep,
  phases = c(60, 65),
  method = c("late_season", "multiple_per_year", "both"),
  late_threshold = 250,
  gap_threshold = 60,
  reference_period = 1991:2020,
  z_threshold = 3,
  min_obs = 10
)
}
\arguments{
\item{pep}{A \code{pep} object or data.table with phenological observations.}

\item{phases}{Integer vector of phase IDs to analyze. Default is
\code{c(60, 65)} for flowering phases. Use \code{NULL} to analyze all phases.}

\item{method}{Character. Detection method:
\describe{
\item{"late_season"}{(Default) Flag observations occurring after the
expected phenological window (DOY > late_threshold).}
\item{"multiple_per_year"}{Detect multiple observations of the same
phase at the same station within a single year.}
\item{"both"}{Apply both methods.}
}}

\item{late_threshold}{Integer. DOY threshold for "late season" events.
Default is 250 (~September 7). Events after this date for spring phases
are flagged as potential second events.}

\item{gap_threshold}{Integer. For "multiple_per_year" method, minimum days
between observations to consider them separate events. Default is 60.}

\item{reference_period}{Integer vector of years to calculate expected
phenological windows. Default is \code{1991:2020}.}

\item{z_threshold}{Numeric. Z-score threshold for identifying late events
relative to the reference period. Default is 3.}

\item{min_obs}{Integer. Minimum observations in reference period to calculate
expected windows. Default is 10.}
}
\value{
A list of class \code{second_events} containing:
\describe{
\item{events}{Data.table of detected second/repeated events with columns:
s_id, species, phase_id, year, day, event_type, expected timing info}
\item{summary}{Summary statistics by species, phase, and event type}
\item{by_year}{Annual counts of detected events}
\item{by_month}{Monthly distribution of events}
\item{total_by_year}{Total observations per year (for relative scaling in plots)}
\item{method}{Detection method used}
\item{params}{Parameters used for detection}
}
}
\description{
Identifies potential second flowering events and other irregular repeated
phenological observations. These may represent biologically meaningful
responses to climate disruption rather than data errors.
}
\details{
Second flowering and other repeated phenological events are increasingly
reported as climate patterns become more irregular. These events may indicate:
\itemize{
\item Disrupted coordination between environmental cues and development
\item Response to unusual autumn warmth after summer dormancy
\item Failure of normal seasonal inhibition mechanisms
}

The function uses two complementary approaches:
\enumerate{
\item \strong{Late season detection}: Identifies observations occurring
well after the normal phenological window (e.g., flowering in autumn)
\item \strong{Multiple events}: Finds stations reporting the same phase
multiple times in one year with sufficient gap between observations
}
}
\section{Interpreting Results}{

Not all detected events are necessarily "true" second flowering. Consider:
\itemize{
\item \strong{High confidence}: Same station reports phase twice with
60+ day gap; late observation is clearly outside normal window
\item \strong{Medium confidence}: Single late observation at a station;
could be second event or data entry error
\item \strong{Requires verification}: Check original data sources,
geographic context, species biology
}
}

\examples{
\donttest{
pep <- pep_download()

# Use grapevine flowering in Alpine region (long records, second events possible)
pep_subset <- pep[species == "Vitis vinifera" & phase_id \%in\% c(60, 65) &
                  country \%in\% c("Germany-South", "Switzerland", "Austria")]

if (nrow(pep_subset) > 0) {
  # Method 1: Detect late-season flowering events (default)
  second <- pep_second_events(pep_subset, phases = c(60, 65))
  print(second)
  summary(second)

  # Plot from 1980 onwards for clearer trends
  plot(second, from_year = 1980)
  plot(second, type = "overview", scale = "relative", from_year = 1980)

  # Method 2: Detect repeated observations at same station
  repeated <- pep_second_events(pep_subset, method = "multiple_per_year")
  print(repeated)
  plot(repeated, type = "timeline", from_year = 1980)

  # Method 3: Combine both detection methods
  all_events <- pep_second_events(pep_subset, method = "both")
  summary(all_events)
  plot(all_events)
  plot(all_events, scale = "relative")
}
}

}
\references{
Related to emerging research on phenological irregularity and climate
disruption of seasonal timing.
}
\seealso{
\code{\link{pep_flag_outliers}} for general outlier detection,
\code{\link{pep_plot_outliers}} for outlier visualization
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_plot_hh.R
\name{pheno_plot_hh}
\alias{pheno_plot_hh}
\title{Plot Phenological Time Series for Heading and Harvest}
\usage{
pheno_plot_hh(
  data_list,
  phase_select = NULL,
  alpha_lines = 0.6,
  linewidth = 0.7
)
}
\arguments{
\item{data_list}{A named list returned by \code{\link{pheno_regional_hh}},
containing \code{ts_tidy}.}

\item{phase_select}{Character. Optional filter for a specific phenological phase
(e.g., "Heading", "Harvest"). If NULL, all phases are shown.}

\item{alpha_lines}{Numeric. Transparency level for lines (default is 0.6).}

\item{linewidth}{Numeric. Line width for time series plots (default is 0.7).}
}
\value{
A \code{ggplot} object.
}
\description{
Visualizes phenological time series for heading and harvest phases,
comparing aggregated and spatially filtered PEP725 data.
}
\details{
This function shows DOY trends by phase and data source across years,
with month labels on the y-axis for easier interpretation.
}
\examples{
\donttest{
pep <- pep_download()
out <- pheno_regional_hh(pep,
         species_name = "Triticum aestivum")
pheno_plot_hh(out)
}
}
\seealso{
\code{\link{pheno_regional_hh}}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{plot.pep}
\alias{plot.pep}
\title{Plot Method for PEP725 Data}
\usage{
\method{plot}{pep}(x, type = c("map", "timeseries", "histogram"), ...)
}
\arguments{
\item{x}{A \code{pep} object.}

\item{type}{Character. Type of plot: \code{"map"} for station locations,
\code{"timeseries"} for temporal trends, \code{"histogram"} for DOY distribution.
Default is \code{"map"}.}

\item{...}{Additional arguments passed to the underlying plot function.}
}
\value{
A ggplot object (invisibly).
}
\description{
Provides default visualizations for PEP725 phenological data.
}
\examples{
\donttest{
pep <- pep_download()
plot(pep)
plot(pep, type = "timeseries")
plot(pep, type = "histogram")
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{summary.pep}
\alias{summary.pep}
\title{Summary Method for PEP725 Data}
\usage{
\method{summary}{pep}(object, by = c("species", "phase", "country", "year"), top = 10, ...)
}
\arguments{
\item{object}{A \code{pep} object.}

\item{by}{Character. Summarize by \code{"species"}, \code{"phase"},
\code{"country"}, or \code{"year"}. Default is \code{"species"}.}

\item{top}{Integer. Number of top entries to show (default 10).}

\item{...}{Additional arguments (currently unused).}
}
\value{
A \code{pep_summary} object (list) containing summary tables,
printed to console.
}
\description{
Provides a detailed phenological summary of the dataset, including
breakdowns by species, phase, country, and temporal coverage.
}
\examples{
\donttest{
pep <- pep_download()
summary(pep)
summary(pep, by = "phase")
summary(pep, by = "country")
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_simulate.R
\name{pep_simulate}
\alias{pep_simulate}
\title{Simulate Synthetic PEP Data}
\usage{
pep_simulate(pep, min_obs = 20, seed = 42, progress = TRUE)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table} with at least the
columns \code{species}, \code{phase_id}, \code{s_id}, \code{year}, and
\code{day}.}

\item{min_obs}{Integer. Minimum number of observations required per
station-species-phase group to attempt GAM fitting.
Groups with fewer observations keep their original \code{day} values
unchanged. Default is 20.}

\item{seed}{Integer. Random seed for reproducibility. Default is 42.}

\item{progress}{Logical. Show a progress bar? Default \code{TRUE}.}
}
\value{
A \code{\link[=new_pep]{pep}} object with the same rows and columns
as the input.  The \code{day} column is overwritten with synthetic values
for groups that meet the \code{min_obs} threshold; other rows retain their
original values.
}
\description{
Generates synthetic phenological observations by fitting a GAM smooth
(\code{day ~ s(year)}) per station-species-phase group and replacing the
observed \code{day} values with predictions plus Gaussian noise.  The
station-year-species-phase structure of the input data is kept intact.
}
\details{
For each qualifying group (station x species x phase with at least
\code{min_obs} observations), a GAM is fitted and synthetic day-of-year
values are generated as \code{round(predicted + rnorm(n, sd = residual_sd))}.

Groups where the GAM fit fails receive a fallback: the species-phase median
day plus Gaussian jitter (sd = 5 days).

Groups below the \code{min_obs} threshold are left unchanged.  A message
reports how many rows were not simulated.
}
\examples{
\donttest{
pep <- pep_download()
# Single species + country for fast example
vine_ch <- pep[species == "Vitis vinifera" & country == "Switzerland"]
pep_synth <- pep_simulate(vine_ch)
}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_completeness.R
\name{pep_completeness}
\alias{pep_completeness}
\title{Assess Species and Phase Completeness of Phenological Data}
\usage{
pep_completeness(
  pep,
  by = c("genus", "species", "phase_id"),
  year_range = NULL,
  min_obs = 1,
  include_years = FALSE
)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table} containing phenological
observations with columns \code{year}, \code{day}, \code{genus},
\code{species}, and \code{phase_id}.}

\item{by}{Character vector specifying grouping dimensions. Options include
\code{"species"}, \code{"genus"}, \code{"phase_id"}, \code{"year"},
\code{"country"}, and \code{"s_id"}. Default is
\code{c("genus", "species", "phase_id")}.}

\item{year_range}{Optional integer vector of length 2 specifying the start
and end years for assessment. If \code{NULL} (default), uses the full
range of years in the data.}

\item{min_obs}{Integer. Minimum number of observations required to include
a group in the output. Default is 1.}

\item{include_years}{Logical. If \code{TRUE}, includes year-level detail
in the output (creates a row per group × year). Default is \code{FALSE}
for summary mode.}
}
\value{
A \code{data.table} of class \code{pep_completeness} with columns:
\describe{
\item{by variables}{Grouping variables as specified in \code{by}}
\item{n_obs}{Total number of observations}
\item{n_stations}{Number of unique stations}
\item{n_years}{Number of years with data}
\item{year_min}{First year of observations}
\item{year_max}{Last year of observations}
\item{year_span}{Total span in years}
\item{completeness_pct}{Percentage of years with data within span}
\item{median_doy}{Median day of year}
\item{iqr_doy}{Interquartile range of day of year}
}
}
\description{
Provides comprehensive assessment of data completeness across species,
phases, years, and optionally countries. Useful for identifying data gaps
and planning analyses.
}
\details{
This function assesses data coverage across multiple dimensions, helping
users understand where data is available and where gaps exist. It is
particularly useful for:
\itemize{
\item Identifying which species × phase combinations have sufficient data
\item Finding temporal gaps in time series
\item Comparing coverage across countries or regions
\item Planning analyses that require minimum observation counts
}
}
\section{Completeness Calculation}{

Completeness is calculated as the percentage of years with at least one
observation within the year span (year_max - year_min + 1). A completeness
of 100\% means every year in the span has data.
}

\examples{
\donttest{
pep <- pep_download()

# Use Swiss subset for faster computation
pep_ch <- pep[country == "Switzerland"]

# Basic completeness by species and phase
comp <- pep_completeness(pep_ch)
print(comp)

# Filter to well-observed combinations
comp_good <- comp[n_obs >= 100 & completeness_pct >= 80]

# Completeness by country (use Alpine countries)
pep_alpine <- pep[country \%in\% c("Switzerland", "Austria")]
comp_country <- pep_completeness(pep_alpine,
                                 by = c("country", "genus", "phase_id"))

# Year-level detail for a specific period
comp_yearly <- pep_completeness(pep_ch,
                                 year_range = c(1991, 2020),
                                 include_years = TRUE)

# Visualize completeness
plot(comp)
}

}
\seealso{
\code{\link{pep_check_phases}} for validating expected phases exist,
\code{\link{pep_quality}} for quality grading,
\code{\link{pep_coverage}} for overall data coverage
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_thermal_units.R
\name{calc_thermal_sum}
\alias{calc_thermal_sum}
\title{Calculate Thermal Sum at Phenological Events}
\usage{
calc_thermal_sum(
  pep,
  temp_data,
  t_base = 5,
  t_start = 1,
  by = NULL,
  method = "average"
)
}
\arguments{
\item{pep}{A \code{pep} object or data.table with phenological observations.
Must contain \code{year} and \code{day} columns.}

\item{temp_data}{A data.frame or data.table with daily temperature data.
Must contain columns: \code{year}, \code{doy} (day of year),
\code{tmin}, \code{tmax} (or \code{tmean}).}

\item{t_base}{Numeric. Base temperature (°C). Default 5.}

\item{t_start}{Integer. Start day of year for accumulation. Default 1 (Jan 1).
Use 60 for March 1, etc.}

\item{by}{Character vector. Columns to match temperature data to phenology
(e.g., \code{"s_id"} for station-specific temperatures). Default \code{NULL}
uses same temperature for all observations in a year.}

\item{method}{Character. GDD calculation method (see \code{\link{calc_thermal_units}}).}
}
\value{
The input data with an additional \code{thermal_sum} column containing
the accumulated GDD from \code{t_start} to the observed DOY.
}
\description{
Computes the accumulated thermal units (GDD) from a start date to the
observed phenological event date. Useful for determining the thermal
requirements of different phenophases.
}
\details{
This function joins phenological observations with temperature data and
calculates the thermal sum (accumulated GDD) at each observation date.
This is useful for:
\itemize{
\item Determining thermal requirements of phenophases
\item Comparing thermal sums across years, locations, or species
\item Validating phenological models
}
}
\examples{
\donttest{
# Load phenology data
data(pep_seed)

# Create example temperature data (normally from weather stations/reanalysis)
temp <- data.frame(
  year = rep(2000:2015, each = 365),
  doy = rep(1:365, 16),
  tmin = rnorm(365 * 16, mean = 5, sd = 8),
  tmax = rnorm(365 * 16, mean = 15, sd = 8)
)

# Add thermal sum to phenology data
pep_thermal <- calc_thermal_sum(pep_seed, temp, t_base = 5)
}

}
\seealso{
\code{\link{calc_thermal_units}} for the underlying GDD calculation
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_plot.R
\name{pheno_plot}
\alias{pheno_plot}
\title{Plot Phenological Time Series}
\usage{
pheno_plot(data_list, alpha_lines = 0.6, linewidth = 0.7)
}
\arguments{
\item{data_list}{A named list of prepared data objects, typically output from
\code{\link{pheno_regional}}. Must contain element \code{ts_tidy}.}

\item{alpha_lines}{Alpha transparency for time series lines (default is \code{0.6}).}

\item{linewidth}{Line width for time series lines (default is \code{0.7}).}
}
\value{
A \code{ggplot} object for visual inspection and further customization.
}
\description{
Generates time series plots from processed phenology data. Shows DOY trends
with faceting by phenophase and spatial scope.
}
\details{
This function visualizes DOY trends with faceting by phenophase and spatial scope.
}
\examples{
# Construct a minimal data_list with ts_tidy
ts_tidy <- data.frame(
  year = rep(1990:1995, 2),
  DOY = c(120, 118, 115, 119, 113, 110, 125, 122, 120, 124, 118, 115),
  source = rep(c("PEP725 (aggregated)", "PEP725 (near Changins)"), each = 6),
  phase = "Heading",
  panel = "DOY"
)
pheno_plot(list(ts_tidy = ts_tidy))

}
\seealso{
\code{\link{pheno_regional}}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_phase.R
\name{select_phase}
\alias{select_phase}
\title{Select and Label Phenophases from PEP Time Series}
\usage{
select_phase(dt, label, sp, yrmin, phases = c(60, 100))
}
\arguments{
\item{dt}{A \code{data.table} containing columns \code{species}, \code{year}, \code{phase_id}, and \code{mean_day}.
Typically produced by aggregating PEP725 data (e.g., via \code{\link{pheno_regional}}).}

\item{label}{A character string used as identifier for source and site columns in the output.}

\item{sp}{Species name (as character string) to filter, e.g., \code{"Triticum aestivum"}.}

\item{yrmin}{Minimum year (inclusive) for filtering.}

\item{phases}{Integer vector of phase IDs to include (default: \code{c(60, 100)} corresponding to "Heading" and "Harvest").}
}
\value{
A \code{data.table} with columns:
\describe{
\item{\code{year}}{Year of observation}
\item{\code{phase}}{Mapped BBCH phenological stage name}
\item{\code{DOY}}{Mean day-of-year for this phase, species, and year}
\item{\code{source}}{Set to \code{label}}
\item{\code{site}}{Set to \code{label}}
}
}
\description{
Extracts a subset of phenological observations from a processed PEP time series (e.g., output from species-level aggregation),
filters by species, year, and phenophase ID, and attaches meaningful phase labels based on standard BBCH codes.
}
\details{
This function uses an internal lookup table to map numeric \code{phase_id} codes to descriptive BBCH phase names (e.g., \code{60 = "Heading"}, \code{100 = "Harvest"}).
Unmapped phase IDs will trigger a warning. If expected phases are missing from the result, a separate warning is issued.

The function is particularly useful for visualizing specific growth stages or comparing phenological trends across datasets and locations.
}
\examples{
\donttest{
pep <- pep_download()
agg <- pheno_regional(pep, species_name = "Triticum aestivum", phase = 60)$pep_agg
df <- select_phase(agg, label = "PEP725", sp = "Triticum aestivum", yrmin = 1961)
head(df)
}

}
\seealso{
\code{\link{pheno_regional}}, \code{\link{pep_download}}, \code{\link{pheno_plot}}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_trend_turning.R
\name{kendall_tau}
\alias{kendall_tau}
\title{Mann-Kendall Z-Statistic (deprecated alias for \code{mann_kendall_z})}
\usage{
kendall_tau(x)
}
\arguments{
\item{x}{Numeric vector (time series assumed to be equidistant).
\code{NA} values are silently removed.}
}
\value{
Numeric. Identical output shape to \code{mann_kendall_z()}.
}
\description{
This function was misnamed in pep725 <= 1.0.2: it returned the
Mann-Kendall Z-statistic, not Kendall's \eqn{\tau = S / (n(n-1)/2)}.
Please call \code{\link{mann_kendall_z}} instead for identical (and
better: tie-corrected, continuity-corrected) behaviour.
}
\details{
Note that this function is now a thin wrapper and will continue to
return the Z-statistic until a future major release.
}
\examples{
suppressWarnings(kendall_tau(c(120, 118, 115, 112, 110, 108, 105)))

}
\seealso{
\code{\link{mann_kendall_z}}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_normals.R
\name{plot.pheno_normals}
\alias{plot.pheno_normals}
\title{Plot Method for Phenological Normals}
\usage{
\method{plot}{pheno_normals}(x, which = c("dotplot", "bar"), ...)
}
\arguments{
\item{x}{A \code{pheno_normals} object returned by \code{\link{pheno_normals}}.}

\item{which}{Character string selecting the plot type: \code{"dotplot"}
(default) shows \code{median_doy} with Q25--Q75 range via
\code{geom_pointrange}; \code{"bar"} shows \code{median_doy} as columns
with error bars.}

\item{...}{Additional arguments (unused).}
}
\value{
A \code{ggplot} object (returned invisibly).
}
\description{
Visualise phenological normals as a dot plot with interquartile-range bars
or as a bar chart.
}
\examples{
\donttest{
pep <- pep_download()
pep_ch <- pep[country == "Switzerland"]

# Normals by phase for apple
n <- pheno_normals(pep_ch, species = "Malus", by = "phase_id")
plot(n)
plot(n, which = "bar")
}

}
\seealso{
\code{\link{pheno_normals}}, \code{\link{pheno_anomaly}}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_trend_turning.R
\name{sequential_mk}
\alias{sequential_mk}
\title{Sequential Mann-Kendall Test (Sneyers 1975, internal)}
\usage{
sequential_mk(x)
}
\arguments{
\item{x}{Numeric vector (time series assumed to be equidistant;
callers should drop or impute NAs beforehand).}
}
\value{
List with \code{progressive}, \code{retrograde} (both length
\code{length(x)}), and \code{turning_points} (logical).
}
\description{
Computes the Sneyers (1975) progressive and retrograde standardised
rank-count statistics \eqn{u(t)} and \eqn{u'(t)} for a time series, and
identifies potential trend turning points as intersections of the two
curves inside the non-significance band.
}
\details{
Let \eqn{n_i} denote the number of \eqn{j < i} with \eqn{x_i > x_j},
and \eqn{t_i = \sum_{k=1}^{i} n_k}. Under the null hypothesis of no
trend, \eqn{E[t_i] = i(i-1)/4} and \eqn{\mathrm{Var}[t_i] = i(i-1)(2i+5)/72}.
The progressive statistic is
\eqn{u(t_i) = (t_i - E[t_i])/\sqrt{\mathrm{Var}[t_i]}}.
The retrograde \eqn{u'(t)} is the same procedure applied to the
reversed series, then reversed and negated so both curves fluctuate
around zero under stationarity.

A turning point is flagged where the sign of \eqn{u - u'} changes
between two consecutive indices and both curves are within the 95\\%
non-significance band \eqn{|z| < 1.96} at the crossing.

Previous versions of this function (pep725 <= 1.0.2) used
\eqn{\mathrm{Var} = i(i-1)(2i+5)/18} with a per-index (non-cumulative)
\eqn{S} statistic and an ad-hoc \eqn{|u| > 0.5} threshold for the
turning-point filter. That formulation did not match Sneyers' procedure.
}
\references{
Sneyers, R. (1975). Sur l'analyse statistique des séries d'observations.
WMO Technical Note No. 143, WMO No. 415. Geneva: WMO.

Moraes, J. M. et al. (1998). Trends in hydrological parameters of a
Southern Brazilian watershed. \emph{Ambio} 27:302-307.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_download.R
\name{pep_cache_info}
\alias{pep_cache_info}
\title{Get Cache Information}
\usage{
pep_cache_info()
}
\value{
A list with cache information, or NULL if no cache exists.
}
\description{
Returns information about the cached PEP data.
}
\examples{
\donttest{
# Check cache status
pep_cache_info()
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_import.R
\name{pep_import}
\alias{pep_import}
\title{Import and preprocess PEP725 phenological data}
\usage{
pep_import(path = "data/Data_PEP725_all", flags = FALSE, add_country = TRUE)
}
\arguments{
\item{path}{path to the folder containing PEP725 CSV files (default is \code{"data/Data_PEP725_all/"}).}

\item{flags}{Logical indicating whether the pep data contains quality control flags (default is \code{FALSE}).}

\item{add_country}{Logical indicating whether to add country information based on station coordinates (default is \code{TRUE}).}
}
\value{
A \code{\link[=new_pep]{pep}} object (extends \code{data.table}) containing the combined and preprocessed PEP725 data.
}
\description{
This function imports all CSV files from the specified folder,
reads them efficiently using \pkg{data.table}, and combines them into a single
data table. It performs several preprocessing steps, including:
\itemize{
\item Converting selected columns to factors (\code{provider_id}, \code{s_id}, \code{gss_id}, \code{genus}, \code{species}, \code{subspecies}).
\item Replacing missing altitude values coded as \code{-9999} with \code{NA}.
\item Creating a combined altitude variable \code{alt2} using \code{alt} and \code{alt_dem}.
\item Converting the \code{date} column to \code{Date} class.
\item Recoding cultivation season (\code{cult_season}) and quality control flags (\code{qc_ori_flag}) into labeled factors.
\item Removing unused or problematic columns (\code{affected_flag}, \code{qc_flag}).
}
}
\details{
This function is intended for importing raw PEP725 station data into a standardized
format suitable for further phenological analysis, such as sensitivity and trend estimation.
}
\examples{
# Small example files shipped with the package
path <- system.file("extdata", package = "pep725")
pep_data <- pep_import(path = path, add_country = FALSE)
print(pep_data)
}
\seealso{
\code{\link[data.table]{fread}}, \code{\link[data.table]{rbindlist}}
}
\author{
Matthias Templ (FHNW)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{bbch_description}
\alias{bbch_description}
\title{Get BBCH Phase Description}
\usage{
bbch_description(codes, na.rm = TRUE, sort = TRUE)
}
\arguments{
\item{codes}{Integer vector of BBCH codes.}

\item{na.rm}{Logical. If \code{TRUE} (default), removes NA codes and codes
without a known description from the output.}

\item{sort}{Logical. If \code{TRUE} (default), sorts the output by phase code.}
}
\value{
A data.frame with columns \code{phase_id} and \code{description},
ordered by phase code (if \code{sort = TRUE}).
}
\description{
Returns human-readable descriptions for BBCH phenological codes.
}
\examples{
bbch_description(c(60, 65, 100))
bbch_description(c(100, 60, 65, NA), na.rm = TRUE, sort = TRUE)

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_daylength.R
\name{calc_max_daylength}
\alias{calc_max_daylength}
\title{Maximum Daylength at a Latitude}
\usage{
calc_max_daylength(lat)
}
\arguments{
\item{lat}{Numeric. Latitude in decimal degrees.}
}
\value{
Numeric. Maximum daylength in hours.
}
\description{
Calculates the maximum possible daylength (at summer solstice) for
a given latitude.
}
\examples{
# Maximum daylength at different latitudes
calc_max_daylength(c(0, 30, 45, 60, 66.5))

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_leaflet.R
\name{pheno_leaflet}
\alias{pheno_leaflet}
\title{Interactive Leaflet Map to Select PEP725 Stations}
\usage{
pheno_leaflet(pep, label_col = NULL, quiet = FALSE)
}
\arguments{
\item{pep}{A data frame or data.table with columns \code{lon}, \code{lat}, and optionally \code{species}.}

\item{label_col}{Optional column name to show as label in popups (e.g. \code{"species"} or \code{"s_id"}).}

\item{quiet}{Logical. If \code{FALSE} (default), displays a message about expected loading time.}
}
\value{
A \code{data.frame} with selected stations (columns: \code{lon}, \code{lat}, and metadata).
}
\description{
Launches a Shiny gadget that allows selection of PEP phenological stations
by drawing a bounding box or clicking individual points on the map.
}
\details{
\strong{Note:} Loading time depends on the size of the dataset. For the full PEP725
dataset with millions of observations, expect 10-20 seconds for the map to fully
render. Consider filtering the data before calling this function to speed up loading:

\preformatted{
# Filter to a specific species first
wheat <- pep[genus == "Triticum"]
selected <- pheno_leaflet(wheat)
}
}
\examples{
if (interactive()) {
pep <- pep_download()

# For faster loading, filter the data first (recommended)
# Grapevine has the longest historical records
pep_subset <- pep[species == "Vitis vinifera"]

if (nrow(pep_subset) > 0) {
  # Launch interactive selection gadget
  selected <- pheno_leaflet(pep_subset, label_col = "species")

  # Inspect selected subset
  print(selected)
}
}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep725_demo.R
\name{pep725_demo}
\alias{pep725_demo}
\title{Demonstrate the pep725 Package Functions}
\usage{
pep725_demo(which = "all", pause = interactive())
}
\arguments{
\item{which}{Character vector specifying which demos to run. Options are:
\itemize{
\item \code{"class"} - Demonstrate the pep class (print, summary)
\item \code{"filter"} - Show data filtering and selection
\item \code{"plot"} - Display visualization capabilities
\item \code{"analysis"} - Show analysis functions (normals, anomalies)
\item \code{"all"} - Run all demonstrations (default)
}}

\item{pause}{Logical. If \code{TRUE} (default), pauses between demos
waiting for user input. Set to \code{FALSE} for non-interactive use.}
}
\value{
Invisibly returns a list containing example outputs from each demo.
}
\description{
This function provides an interactive demonstration of the main features
of the pep725 package for PEP725 phenological data analysis. It showcases
data exploration, filtering, summarization, and visualization capabilities.
}
\details{
The demonstration downloads synthetic PEP725 data using \code{pep_download()}
and walks through typical workflows for phenological data analysis:

\enumerate{
\item \strong{Class demo}: Shows the enhanced print and summary methods
for the pep class, demonstrating how to quickly understand the data.
\item \strong{Filter demo}: Demonstrates selecting specific species,
phases, and time periods using data.table syntax.
\item \strong{Plot demo}: Creates visualizations including station maps,
time series, and DOY distributions.
\item \strong{Analysis demo}: Shows phenological normals calculation,
anomaly detection, and data quality assessment.
}
}
\examples{
if (interactive()) {
# Run all demos interactively
pep725_demo()

# Run specific demos
pep725_demo(which = "class")
pep725_demo(which = c("filter", "plot"))

# Run without pausing (for scripts)
pep725_demo(pause = FALSE)
}

}
\seealso{
\code{\link{pep_download}} for downloading synthetic data,
\code{\link{pheno_normals}} for calculating normals,
\code{\link{pheno_anomaly}} for anomaly detection,
\code{\link{pep_quality}} for quality assessment
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_check_phases.R
\name{print.phase_check}
\alias{print.phase_check}
\title{Print Method for Phase Check Results}
\usage{
\method{print}{phase_check}(x, ...)
}
\arguments{
\item{x}{A \code{phase_check} object}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}
}
\description{
Print Method for Phase Check Results
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_map.R
\name{pheno_map}
\alias{pheno_map}
\title{Plot Phenology Station Maps}
\usage{
pheno_map(
  pep,
  location = c(lon = 6.233, lat = 46.4),
  zoom = 4,
  background = c("google", "none"),
  color_by = c("none", "n_obs", "n_species", "mean_doy", "trend", "species_cv"),
  phase_id = NULL,
  period = NULL,
  min_years = 10,
  min_species = 3,
  point_size = 0.8,
  output_file = NULL,
  key = NULL
)
}
\arguments{
\item{pep}{A \code{data.table} or \code{data.frame} with phenological observations.
Required columns depend on \code{color_by}:
\itemize{
\item Always required: \code{lon}, \code{lat}
\item For \code{"n_species"}, \code{"mean_doy"}, \code{"species_cv"}: \code{species}
\item For \code{"mean_doy"}, \code{"trend"}: \code{year}, \code{day}
}}

\item{location}{Named vector with center longitude and latitude for the map.
Defaults to near Changins: \code{c(lon = 6.233, lat = 46.400)}.
Used primarily for \code{background = "google"}.}

\item{zoom}{Zoom level for the map. For \code{background = "google"}, this is
the Google Maps zoom level (4 = Europe, 7 = regional). For
\code{background = "none"}, this controls the padding around data extent
(4 = wide view, 7 = tight view). Default is 4.}

\item{background}{Character. Map background type:
\describe{
\item{"google"}{Google Maps satellite/terrain background. Requires API key
(set via \code{ggmap::register_google()} or \code{key} parameter).}
\item{"none"}{Simple map with country borders from Natural Earth data.
No API key required. Good for publications and vignettes.}
}}

\item{color_by}{Character. What to color stations by:
\describe{
\item{"none"}{Black points showing station locations only}
\item{"n_obs"}{Number of observations per station}
\item{"n_species"}{Number of species recorded per station}
\item{"mean_doy"}{Mean day-of-year (phenological timing) per station}
\item{"trend"}{Trend in days/year per station (positive = later, negative = earlier)}
\item{"species_cv"}{Coefficient of variation in timing across species at each station
(higher = more variation among species)}
}}

\item{phase_id}{Integer. BBCH phase code to filter by. Recommended for
\code{"mean_doy"} and \code{"trend"} to ensure meaningful comparisons.
If \code{NULL} (default), all phases are included.}

\item{period}{Integer vector of years to include. Default is all years.
For trend calculation, at least 10 years are recommended.}

\item{min_years}{Integer. Minimum years required for trend calculation.
Stations with fewer years show as NA. Default is 10.}

\item{min_species}{Integer. Minimum species required for \code{species_cv}.
Default is 3.}

\item{point_size}{Size of station points (default: 0.8).}

\item{output_file}{Optional file path to export the plot (e.g. \code{"map.pdf"}).}

\item{key}{Google Maps API key. Only needed for \code{background = "google"}.
You can set globally via \code{ggmap::register_google()} instead.}
}
\value{
A ggplot map object.
}
\description{
Generates a map of PEP725 phenological station locations with optional coloring
by various statistics including mean phenological timing, trends, and species
variation.
}
\details{
The function supports two background types:

\strong{Google Maps background} (\code{background = "google"}):
Provides detailed satellite or terrain imagery but requires a Google Maps API key.
Register at \url{https://console.cloud.google.com/} and enable the Maps Static API.

\strong{No background} (\code{background = "none"}):
Uses country outlines from Natural Earth data. No API key required, making it
suitable for:
\itemize{
\item CRAN package vignettes (no external API calls)
\item Reproducible research (no authentication needed)
\item Publication-quality maps with clean appearance
}

The color options provide insights into:
\itemize{
\item \strong{mean_doy}: Spatial patterns in phenological timing. Earlier
timing (lower DOY) typically in southern/lowland areas.
\item \strong{trend}: Where phenology is advancing (negative = earlier over time,
shown in blue) or delaying (positive, shown in red). Uses Kendall's tau
normalized statistic for robustness.
\item \strong{species_cv}: Stations where different species show similar timing
(low CV) vs. divergent timing (high CV). High variation may indicate
species-specific responses to local conditions.
}
}
\section{Interpreting Trends}{

The trend is calculated as Kendall's normalized tau statistic, which ranges
roughly from -3 to +3 for typical data:
\itemize{
\item Negative values (blue): Phenology is getting earlier over time
\item Positive values (red): Phenology is getting later over time
\item Values near 0: No clear trend
\item |tau| > 1.96: Statistically significant at 95\% level
}
}

\examples{
\donttest{
pep <- pep_download()
pep_alpine <- pep[country \%in\% c("Switzerland", "Austria")]
pheno_map(pep_alpine, background = "none", color_by = "n_obs")
}
\dontrun{
# With Google Maps background (requires API key)
ggmap::register_google(key = "your_api_key_here")
pheno_map(pep, background = "google", color_by = "n_species", zoom = 5)
}
}
\seealso{
\code{\link{pheno_normals}} for detailed normal calculations,
\code{\link{mann_kendall_z}} for trend calculation,
\code{\link{pheno_synchrony}} for synchrony analysis
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_check_phases.R
\name{pep_check_phases}
\alias{pep_check_phases}
\title{Check for Expected Phenological Phases}
\usage{
pep_check_phases(
  pep,
  expected = NULL,
  species = NULL,
  year_min = NULL,
  year_max = NULL,
  warn = TRUE,
  label = "Data"
)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table} containing phenological
observations with a \code{phase_id} column.}

\item{expected}{Integer vector of expected BBCH phase codes. If \code{NULL}
(default), checks for common phases: 10 (leaf), 60 (heading/flowering),
65 (full flowering), and 100 (harvest).}

\item{species}{Optional character string to filter by species/genus before
checking. If specified, only checks phases for this species.}

\item{year_min}{Optional integer. Only check phases for years >= this value.
Useful for checking data availability within an analysis window.}

\item{year_max}{Optional integer. Only check phases for years <= this value.}

\item{warn}{Logical. If \code{TRUE} (default), issues warnings for missing
phases. If \code{FALSE}, only returns the result silently.}

\item{label}{Character. Optional label to include in warning messages,
useful when checking multiple datasets. Default is \code{"Data"}.}
}
\value{
A list with class \code{phase_check} containing:
\describe{
\item{expected}{The expected phase codes}
\item{present}{Phase codes found in the data}
\item{missing}{Phase codes not found in the data}
\item{extra}{Phase codes in data but not in expected list}
\item{complete}{Logical. TRUE if all expected phases are present}
\item{n_obs}{Named integer vector of observation counts per present phase}
\item{coverage}{Data.table with phase-level statistics}
}
}
\description{
Validates that expected phenological phases (BBCH codes) are present in the
data. Issues informative warnings for missing phases without stopping
execution, making it useful for pipeline validation.
}
\details{
This function is designed to be used at the start of analysis pipelines
to validate that required phenological phases are available. It issues
warnings but does not stop execution, allowing users to be informed of
potential issues while still proceeding with available data.
}
\section{Common BBCH Phases}{

\describe{
\item{10}{Leaf development}
\item{60}{Beginning of flowering / Heading}
\item{65}{Full flowering / Anthesis}
\item{69}{End of flowering}
\item{87}{Hard dough (cereals)}
\item{100}{Harvest}
}
}

\examples{
\donttest{
pep <- pep_download()

# Use Swiss subset for faster checking
pep_ch <- pep[country == "Switzerland"]

# Basic check for common phases
check <- pep_check_phases(pep_ch)

# Check specific phases for grapevine (longest historical records)
check <- pep_check_phases(pep_ch,
                      expected = c(60, 65, 81),
                      species = "Vitis vinifera")

# Check within analysis window
check <- pep_check_phases(pep_ch,
                      expected = c(60, 65),
                      year_min = 1991,
                      year_max = 2020)

# Use in pipeline with custom label
vine_data <- pep_ch[species == "Vitis vinifera"]
if (nrow(vine_data) > 0) {
  pep_check_phases(vine_data, expected = c(60, 65),
               label = "Grapevine analysis")
}

# Silent check (no warnings)
result <- pep_check_phases(pep_ch, warn = FALSE)
if (!result$complete) {
  # Handle missing phases programmatically
}
}

}
\seealso{
\code{\link{bbch_description}} for phase code descriptions,
\code{\link{pep_completeness}} for detailed completeness assessment,
\code{\link{pep_quality}} for quality grading
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_flag_outliers.R
\name{plot.pep_outliers}
\alias{plot.pep_outliers}
\title{Plot Method for Outlier Detection Results}
\usage{
\method{plot}{pep_outliers}(x, type = c("histogram", "scatter", "timeline"), ...)
}
\arguments{
\item{x}{A \code{pep_outliers} object}

\item{type}{Character. Plot type: "histogram" for deviation distribution,
"scatter" for DOY vs expected, "timeline" for outliers over time.}

\item{...}{Additional arguments}
}
\value{
A ggplot object (invisibly)
}
\description{
Plot Method for Outlier Detection Results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_outliers_leaflet.R
\name{pep_outliers_leaflet}
\alias{pep_outliers_leaflet}
\title{Interactive Leaflet Map of Outlier-Flagged Stations}
\usage{
pep_outliers_leaflet(
  x,
  outlier_only = TRUE,
  top_n = 10,
  radius_range = c(4, 14),
  palette = "plasma",
  ...
)
}
\arguments{
\item{x}{A \code{pep_outliers} object from \code{\link{pep_flag_outliers}}.
Must contain \code{lon}, \code{lat}, \code{s_id}, \code{year},
\code{phase_id}, \code{day}, \code{is_outlier}, and \code{deviation}
columns.}

\item{outlier_only}{Logical. If \code{TRUE} (default), only stations
with at least one flagged observation are shown. If \code{FALSE},
every station appears (non-flagged stations are drawn small and
grey for context).}

\item{top_n}{Integer. Maximum number of station-year records included
in a station's popup. Default 10.}

\item{radius_range}{Numeric length-2. Minimum and maximum circle
marker radius (in pixels) mapped to log1p(n_outliers). Default
\code{c(4, 14)}.}

\item{palette}{Character. Name of a \code{RColorBrewer} or viridis
palette for the colour scale. Default \code{"plasma"}.}

\item{...}{Additional arguments reserved for future expansion.}
}
\value{
A \code{leaflet} htmlwidget. Call \code{print()} to display
in the RStudio Viewer or embed it in an Rmd file.
}
\description{
Renders a Leaflet map of the stations in a \code{pep_outliers} object,
with each station plotted as a circle marker whose size encodes the
number of flagged observations and whose colour encodes the maximum
absolute residual (or Mahalanobis distance) for the station. Popups
list the worst-offending station-years and their residual values —
useful for interactive inspection during QC.
}
\details{
Unlike \code{\link{pheno_leaflet}} (which is a selection gadget built
on Shiny + miniUI), this is a pure visualisation: it returns a
\code{leaflet} htmlwidget so you can embed it in an R Markdown
report, include it in a vignette, or call \code{print()} on it for
interactive exploration.

\strong{Colour} encodes the station's maximum absolute residual
(for univariate methods: \code{max(|deviation|)} in days; for
\code{method = "mahalanobis"}: the maximum robust Mahalanobis
distance). \strong{Size} encodes \code{log1p(n_outliers)} — a station
flagged once has a small marker, a station flagged 30 times has a
much larger one.

For \code{method = "gam_residual"} and \code{"mahalanobis"} this is
the natural companion to the Schaber-Badeck-style text tables and
the static diagnostic figure, because spatial structure in outliers
(a local observer network, a regional climate anomaly) is often
only visible on a map.
}
\examples{
\dontrun{
pep <- pep_download()
pep_ch <- pep[country == "Switzerland" & species == "Malus domestica"]
flagged <- pep_flag_outliers(
  pep_ch,
  by = c("genus", "species", "phase_id"),
  method = "gam_residual"
)
pep_outliers_leaflet(flagged)
}

}
\seealso{
\code{\link{pep_flag_outliers}} — detection,
\code{\link{pep_plot_outliers}} — static diagnostic plots,
\code{\link{pheno_leaflet}} — station-selection Shiny gadget.
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_trend_turning.R
\name{print.pheno_turning}
\alias{print.pheno_turning}
\title{Print Method for Trend Turning Analysis}
\usage{
\method{print}{pheno_turning}(x, ...)
}
\arguments{
\item{x}{A \code{pheno_turning} object}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}
}
\description{
Print Method for Trend Turning Analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_plot_outliers.R
\name{pep_plot_outliers}
\alias{pep_plot_outliers}
\title{Visualize Phenological Outliers for Inspection}
\usage{
pep_plot_outliers(
  x,
  type = c("overview", "seasonal", "map", "detail", "station", "doy_context",
    "diagnostic", "profile"),
  phase_id = NULL,
  outlier_only = NULL,
  late_threshold = 250,
  n_top = 20,
  ...
)
}
\arguments{
\item{x}{A \code{pep_outliers} object from \code{\link{pep_flag_outliers}}.}

\item{type}{Character. Type of plot to produce:
\describe{
\item{"overview"}{(Default) Multi-panel overview with distribution,
seasonal pattern, and summary statistics.}
\item{"seasonal"}{Distribution of outliers by month/season, useful for
detecting second flowering (late-season outliers for spring phases).}
\item{"map"}{Geographic distribution of outliers (requires lon/lat).}
\item{"detail"}{Detailed view of individual outlier events with context.}
\item{"station"}{Station-level outlier patterns over time.}
\item{"doy_context"}{Shows outliers in context of full DOY distribution
per phase, highlighting potential second events.}
\item{"diagnostic"}{Paper- and vignette-ready 4-panel figure of
model fit quality: (A) residuals vs fitted; (B) Q-Q plot of
residuals; (C) |residual| vs altitude (if available) or year;
(D) spatial map of maximum |residual| per station. Designed to
accompany \code{method = "gam_residual"} but usable for every
method (uses the \code{deviation} column as the residual). For
\code{method = "mahalanobis"} the panels automatically switch
to MD-specific diagnostics (sorted MD with chi-square
threshold; Q-Q against \eqn{\chi^2_p}; mean / max MD over time;
per-station worst-case MD map).}
\item{"profile"}{Parallel-coordinates plot of the phase profile
(one line per station-year across phases), with flagged
station-years highlighted and the robust central profile
overlaid. Designed for \code{method = "mahalanobis"} where the
outlierness lives in the \emph{shape} of a station-year across
phases, not in any single DOY. The primary paper figure for
the multivariate method.}
}}

\item{phase_id}{Optional integer vector to filter specific phases for plotting.}

\item{outlier_only}{Logical. If TRUE (default for some types), show only
outlier observations. If FALSE, show all data with outliers highlighted.}

\item{late_threshold}{Integer. DOY threshold for classifying "late" outliers
as potential second events (default 250, ~September 7).}

\item{n_top}{Integer. For detail view, number of most extreme outliers to show.
Default 20.}

\item{...}{Additional arguments passed to plotting functions.}
}
\value{
A \code{ggplot} object (or list of ggplots for "overview").
}
\description{
Creates diagnostic plots to help distinguish between data errors and
biologically meaningful extreme events (e.g., second flowering). Multiple
plot types reveal different aspects of outlier patterns.
}
\details{
This function is designed to support quality control workflows where the
goal is to distinguish:
\enumerate{
\item \strong{Data errors}: Typos, wrong dates, mis-coded phases
\item \strong{Observer errors}: Misidentified species or phases
\item \strong{Biologically meaningful extremes}: Second flowering, delayed
development, climate-driven anomalies
}

For detecting potential \strong{second flowering events}:
\itemize{
\item Use \code{type = "seasonal"} to see if late-season outliers cluster
in autumn months
\item Use \code{type = "doy_context"} to visualize outliers relative to
the main flowering distribution
\item Focus on flowering phases (60, 65) with late DOY values (>250)
}
}
\examples{
\donttest{
pep <- pep_download()

# Use Swiss subset for faster outlier detection
pep_ch <- pep[country == "Switzerland"]
pep_flagged <- pep_flag_outliers(pep_ch)

# Overview of all outliers
pep_plot_outliers(pep_flagged, type = "overview")

# Seasonal distribution - look for autumn outliers in flowering phases
pep_plot_outliers(pep_flagged, type = "seasonal", phase_id = c(60, 65))

# Geographic pattern
pep_plot_outliers(pep_flagged, type = "map")

# Detailed view of most extreme outliers
pep_plot_outliers(pep_flagged, type = "detail", n_top = 30)

# DOY context for flowering phase - spot potential second flowering
pep_plot_outliers(pep_flagged, type = "doy_context", phase_id = 65)
}

}
\seealso{
\code{\link{pep_flag_outliers}} for detecting outliers,
\code{\link{pep_quality}} for comprehensive quality assessment
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_second_events.R
\name{summary.second_events}
\alias{summary.second_events}
\title{Summary Method for Second Events Detection}
\usage{
\method{summary}{second_events}(object, ...)
}
\arguments{
\item{object}{A \code{second_events} object}

\item{...}{Additional arguments (unused)}
}
\value{
The summary data.table (invisibly)
}
\description{
Summary Method for Second Events Detection
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_normals.R
\name{pheno_normals}
\alias{pheno_normals}
\title{Calculate Phenological Normals (Climatology)}
\usage{
pheno_normals(
  pep,
  period = 1991:2020,
  by = c("country", "genus", "phase_id"),
  species = NULL,
  phase_id = NULL,
  min_years = 20,
  probs = c(0.05, 0.1, 0.25, 0.75, 0.9, 0.95),
  na.rm = TRUE
)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table} containing phenological
observations with columns \code{year}, \code{day}, and grouping variables.}

\item{period}{Integer vector specifying the years to include in the normal
calculation. Default is \code{1991:2020} (current WMO standard period).}

\item{by}{Character vector of column names to group by. Common choices:
\itemize{
\item \code{c("country", "genus", "phase_id")} - Regional normals by genus/phase
\item \code{c("s_id", "species", "phase_id")} - Station-level normals
\item \code{c("genus", "phase_id")} - Overall normals ignoring geography
}
Default is \code{c("country", "genus", "phase_id")}.}

\item{species}{Optional character string to filter by species column.
Can be genus name (e.g., "Triticum") or full species (e.g., "Triticum aestivum").
If \code{NULL} (default), all species in the data are included.}

\item{phase_id}{Optional integer vector to filter by BBCH phase codes.
If \code{NULL} (default), all phases in the data are included.}

\item{min_years}{Minimum number of years required to calculate valid normals.
Default is 20 (WMO standard). Groups with fewer years return \code{NA}.}

\item{probs}{Numeric vector of probabilities for percentile calculation.
Default is \code{c(0.05, 0.10, 0.25, 0.75, 0.90, 0.95)}.}

\item{na.rm}{Logical. Should missing values be removed? Default \code{TRUE}.}
}
\value{
A \code{data.table} with the following columns:
\describe{
\item{by variables}{Grouping variables as specified in \code{by}}
\item{n_years}{Number of years with data in the period}
\item{n_obs}{Total number of observations}
\item{mean_doy}{Arithmetic mean DOY}
\item{median_doy}{Median DOY (more robust to outliers)}
\item{sd_doy}{Standard deviation}
\item{iqr_doy}{Interquartile range}
\item{mad_doy}{Median absolute deviation (robust spread)}
\item{q05, q10, q25, q75, q90, q95}{Percentiles with the default
\code{probs}. Custom \code{probs} produce column names derived from
the actual levels — integer percents pad to two digits
(e.g., \code{q05}, \code{q10}), fractional percents use an
underscore in place of the decimal point (e.g., \code{q02_5} for
the 2.5\\% quantile, \code{q97_5} for the 97.5\\% quantile).
The mapping is also stored in \code{attr(result, "q_names")}.}
\item{period}{Character string describing the reference period}
}
}
\description{
Computes reference "normal" phenology for a specified period, analogous to
WMO climate normals. Returns central tendency, spread, and percentile
statistics for phenological day-of-year (DOY) values.
}
\details{
Phenological normals provide a baseline for comparing individual years or
detecting trends. This function calculates both classical statistics (mean, SD)
and robust alternatives (median, MAD, IQR) that are less sensitive to outliers.

The function can be used in two ways:
\enumerate{
\item \strong{Full dataset}: Pass the complete \code{pep} object and use
\code{species} and \code{phase_id} parameters to filter internally.
\item \strong{Pre-filtered subset}: Filter the data first using data.table
syntax, then pass the subset to the function.
}
}
\section{Standard Reference Periods}{

\itemize{
\item \strong{1961-1990}: Historical reference (pre-acceleration of warming)
\item \strong{1991-2020}: Current WMO standard normal period
}
}

\examples{
\donttest{
# Download synthetic data first
pep <- pep_download()

# Use Swiss subset for faster computation
pep_ch <- pep[country == "Switzerland"]

# Calculate normals for all species and phases
normals_all <- pheno_normals(pep_ch)

# Normals for apples, flowering phase (60)
apple_normals <- pheno_normals(pep_ch,
                               species = "Malus",
                               phase_id = 60)

# Using a pre-filtered subset
apples <- pep_ch[genus == "Malus" & phase_id \%in\% c(60, 65)]
apple_normals <- pheno_normals(apples,
                               by = c("species", "phase_id"))

# Station-level normals for detailed analysis
station_normals <- pheno_normals(pep_ch,
                                 species = "Malus",
                                 phase_id = 60,
                                 by = c("s_id"))

# Historical reference period
historical <- pheno_normals(pep_ch,
                            period = 1961:1990,
                            species = "Malus")

# Compare two periods
period1 <- pheno_normals(pep_ch, period = 1961:1990, species = "Malus")
period2 <- pheno_normals(pep_ch, period = 1991:2020, species = "Malus")
shift <- period2$mean_doy - period1$mean_doy
}

}
\references{
WMO (2017). \emph{WMO Guidelines on the Calculation of Climate Normals}.
WMO-No. 1203, World Meteorological Organization, Geneva.

Templ, M., Templ, B., and Filzmoser, P. (2018).
Modelling and prediction of phenological data of the beginning of
flowering: Austrian indicators of climate change.
\emph{International Journal of Biometeorology}, 62, 1319--1334.
\doi{10.1007/s00484-018-1534-2}
}
\seealso{
\code{\link{pheno_anomaly}} for calculating anomalies relative to normals,
\code{\link{pep_download}} for obtaining the main dataset
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_country.R
\name{add_country}
\alias{add_country}
\title{Add Country Information to a Dataset Based on Latitude/Longitude}
\usage{
add_country(dt, keep_geometry = FALSE)
}
\arguments{
\item{dt}{A \code{data.frame} or \code{data.table} containing columns
\code{lat} and \code{lon}.}

\item{keep_geometry}{Logical. If \code{TRUE}, the sf geometry column is kept.
Default is \code{FALSE}.}
}
\value{
A \code{data.table} with a new column \code{country}.
}
\description{
This function adds a new column \code{country} by performing a fast spatial
join between point coordinates (\code{lat}, \code{lon}) and world country
polygons from \pkg{rnaturalearth}. Geometry is handled via \pkg{sf}.
}
\details{
The input data must contain numeric columns named \code{lat} and \code{lon}
in WGS84 (EPSG:4326).  Rows with \code{NA} coordinates receive
\code{country = NA}.

After assigning countries, Germany is automatically split into
\code{"Germany-North"} and \code{"Germany-South"} using a latitude threshold:
\itemize{
\item lat >= 50 → \code{"Germany-North"}
\item lat  < 50 → \code{"Germany-South"}
}

If the input already contains a \code{country} column it is removed and
replaced by the spatially derived one.
}
\examples{
\donttest{
small <- data.table(
  lat = c(48.2, 51.0, 47.5),
  lon = c(16.3, 10.1, 8.5)
)

pep_with_country <- add_country(small)
pep_with_country
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_quality.R
\name{print.pep_quality}
\alias{print.pep_quality}
\title{Print Method for Data Quality Assessment}
\usage{
\method{print}{pep_quality}(x, n = 10, ...)
}
\arguments{
\item{x}{A \code{pep_quality} object}

\item{n}{Number of rows to display}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}
}
\description{
Print Method for Data Quality Assessment
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subspecies_report.R
\name{summarize_subspecies_availability}
\alias{summarize_subspecies_availability}
\title{Summarize Subspecies Phenological Data Availability}
\usage{
summarize_subspecies_availability(
  pep,
  genus_name = NULL,
  species_name = NULL,
  subspecies_name = NULL,
  phases = c(65, 87),
  years = 1961:2024,
  subregions = c("Austria", "Germany-North", "Germany-South", "Switzerland"),
  include_empty = FALSE,
  metric = c("n_obs", "n_stations", "first_year", "last_year", "median_doy",
    "completeness")
)
}
\arguments{
\item{pep}{A PEP725-style data frame containing at least:
\code{genus}, \code{species}, \code{subspecies}, \code{country}, \code{year}, \code{phase_id},
\code{s_id}, and \code{day} or \code{DOY}.}

\item{genus_name}{Character (optional). Genus to filter.}

\item{species_name}{Character (optional). Species to filter.}

\item{subspecies_name}{Character vector (optional). One or more subspecies.}

\item{phases}{Integer vector. BBCH phases to evaluate.}

\item{years}{Numeric vector. Years to include.}

\item{subregions}{Character vector of region/country names.}

\item{include_empty}{Logical. If TRUE, include subspecies x country rows
even when no observations are present.}

\item{metric}{Character vector specifying which metrics to compute.
Allowed values:
\code{"n_obs"}, \code{"n_stations"}, \code{"first_year"}, \code{"last_year"},
\code{"median_doy"}, \code{"completeness"}, or \code{"all"}.}
}
\value{
A tibble in long format with one row per \verb{subspecies x country}.
}
\description{
Computes data availability metrics for specified genus, species, or
subspecies across BBCH phases and regions. Metrics include numbers of
observations, station counts, first/last observation years, median DOY,
and completeness ratios.
}
\examples{
\donttest{
pep <- pep_download()
pep_alpine <- pep[country \%in\% c("Switzerland", "Austria")]

# Grapevine species (longest historical records)
if (nrow(pep_alpine[species == "Vitis vinifera"]) > 0) {
  summarize_subspecies_availability(
    pep_alpine,
    species_name = "Vitis vinifera",
    subregions = c("Switzerland", "Austria"),
    metric = "all"
  )

  summarize_subspecies_availability(
    pep_alpine,
    subspecies_name = c("Vitis vinifera Riesling",
                        "Vitis vinifera MUELLER THURGAU WEISS"),
    subregions = c("Switzerland", "Austria"),
    metric = c("n_obs","median_doy")
  )
}
}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_gradient.R
\name{pheno_gradient}
\alias{pheno_gradient}
\title{Analyze Phenological Gradients with Elevation or Latitude}
\usage{
pheno_gradient(
  pep,
  variable = c("alt", "lat"),
  species = NULL,
  phase_id = NULL,
  year_range = NULL,
  by = NULL,
  method = c("robust", "ols", "quantile"),
  aggregate = TRUE,
  na.rm = TRUE
)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table} containing phenological
observations with columns \code{year}, \code{day}, and either \code{alt}
(altitude) or \code{lat} (latitude).}

\item{variable}{Character string specifying the gradient variable.
Either \code{"alt"} (altitude/elevation) or \code{"lat"} (latitude).
Default is \code{"alt"}.}

\item{species}{Optional character string to filter by species/genus.}

\item{phase_id}{Optional integer to filter by a single BBCH phase code.}

\item{year_range}{Optional integer vector of length 2 specifying years to include.}

\item{by}{Optional character vector of grouping variables (e.g., "country").
If provided, separate gradients are calculated for each group.}

\item{method}{Character string specifying regression method:
\itemize{
\item \code{"robust"} (default): Robust regression using \code{robustbase::lmrob}
\item \code{"ols"}: Ordinary least squares regression
\item \code{"quantile"}: Quantile regression for median using \code{quantreg::rq}
}}

\item{aggregate}{Logical. If \code{TRUE} (default), first aggregate to station
means before fitting. If \code{FALSE}, use all individual observations.}

\item{na.rm}{Logical. Should missing values be removed? Default \code{TRUE}.}
}
\value{
A list with class \code{pheno_gradient} containing:
\describe{
\item{summary}{A \code{data.table} with slope, intercept, R-squared,
RMSE, n_stations, and interpretation}
\item{model}{The fitted model object(s)}
\item{data}{The data used for fitting}
\item{variable}{The gradient variable used}
\item{method}{The regression method used}
}
}
\description{
Quantifies how phenological timing shifts with altitude or latitude,
calculating the phenological lapse rate using robust regression methods.
}
\details{
Phenological gradients describe how the timing of biological events changes
with environmental factors. This function calculates:

\itemize{
\item \strong{Altitude gradient}: Typically 2-4 days delay per 100m elevation
gain in temperate regions
\item \strong{Latitude gradient}: Typically 2-5 days delay per degree north
in Europe
}

The robust regression method (default) is recommended because phenological
data often contains outliers from observation errors or microclimate effects.
}
\section{Interpretation}{

The slope represents:
\itemize{
\item For altitude: days delay per 100 meters elevation increase
\item For latitude: days delay per degree latitude increase
}
Positive slopes indicate later phenology at higher elevations/latitudes.
}

\examples{
\donttest{
pep <- pep_download()

# Subset to Alpine countries (good elevation variation)
pep_subset <- pep[country \%in\% c("Switzerland", "Austria")]

# Altitude gradient for apple flowering
grad_alt <- pheno_gradient(pep_subset,
                           variable = "alt",
                           species = "Malus",
                           phase_id = 60)
print(grad_alt)

# Latitude gradient
grad_lat <- pheno_gradient(pep_subset,
                           variable = "lat",
                           species = "Malus",
                           phase_id = 60)
print(grad_lat)

# Compare regression methods
grad_ols <- pheno_gradient(pep_subset, species = "Malus",
                           phase_id = 60, method = "ols")
print(grad_ols)

grad_robust <- pheno_gradient(pep_subset, species = "Malus",
                              phase_id = 60, method = "robust")
print(grad_robust)

# Gradient by country (separate regression per country)
grad_by_country <- pheno_gradient(pep_subset,
                                  variable = "alt",
                                  species = "Malus",
                                  phase_id = 60,
                                  by = "country")
print(grad_by_country)
}

}
\references{
Defila, C. and Clot, B. (2001). Phytophenological trends in Switzerland.
\emph{International Journal of Biometeorology}, 45, 203--207.
\doi{10.1007/s004840100101}

Ziello, C. et al. (2009). Influence of altitude on phenology of selected
plant species in the Alpine region (1971--2000).
\emph{Climate Research}, 39, 227--234.
\doi{10.3354/cr00822}
}
\seealso{
\code{\link{pheno_normals}} for calculating climatological normals,
\code{\link[robustbase]{lmrob}} for robust regression details
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{plot.pep_coverage}
\alias{plot.pep_coverage}
\title{Plot Method for PEP Coverage}
\usage{
\method{plot}{pep_coverage}(x, ...)
}
\arguments{
\item{x}{A \code{pep_coverage} object.}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object (or list of ggplot objects).
}
\description{
Plot Method for PEP Coverage
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{as.pep}
\alias{as.pep}
\title{Coerce to PEP725 Data Object}
\usage{
as.pep(x, ...)
}
\arguments{
\item{x}{A data.frame or data.table to coerce.}

\item{...}{Additional arguments (currently unused).}
}
\value{
An object of class \code{pep}.
}
\description{
Coerce to PEP725 Data Object
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_thermal_units.R
\name{calc_thermal_units}
\alias{calc_thermal_units}
\title{Calculate Thermal Units (Growing Degree Days)}
\usage{
calc_thermal_units(
  tmin,
  tmax = NULL,
  t_base = 5,
  t_cap = NULL,
  method = c("average", "modified", "single_sine"),
  cumulative = TRUE,
  na.rm = TRUE
)
}
\arguments{
\item{tmin}{Numeric vector of daily minimum temperatures (°C).}

\item{tmax}{Numeric vector of daily maximum temperatures (°C). If \code{NULL},
\code{tmin} is treated as daily mean temperature.}

\item{t_base}{Numeric. Base temperature (°C) below which no development occurs.
Default is 5°C, typical for temperate crops.}

\item{t_cap}{Numeric. Optional upper temperature cap (°C). Temperatures above
this are set to \code{t_cap} before GDD calculation. Default \code{NULL}
(no cap). Use ~30°C for crops where high heat doesn't increase development.}

\item{method}{Character. Calculation method:
\describe{
\item{"average"}{(Default) GDD = max(0, (tmax + tmin)/2 - t_base)}
\item{"modified"}{Sets negative daily GDD to 0 before averaging}
\item{"single_sine"}{Sine wave approximation for more accurate heat units}
}}

\item{cumulative}{Logical. If \code{TRUE} (default), returns cumulative sum.
If \code{FALSE}, returns daily values.}

\item{na.rm}{Logical. Remove NA values in accumulation? Default \code{TRUE}.}
}
\value{
Numeric vector of thermal units (degree-days). Same length as input.
If \code{cumulative = TRUE}, returns running sum.
}
\description{
Computes accumulated thermal units (growing degree days, GDD) from daily
temperature data. Thermal time is a fundamental concept in phenology,
representing the heat accumulation that drives plant development.
}
\details{
Growing Degree Days (GDD) quantify heat accumulation above a base temperature.
Plants require a certain thermal sum to reach phenological stages (e.g.,
flowering typically requires 500-1500 GDD depending on species).

The base temperature represents the minimum temperature for growth:
\itemize{
\item Cool-season crops (wheat, barley): 0-5°C
\item Warm-season crops (maize, soybean): 10°C
\item Fruit trees (apple, cherry): 4-7°C
\item Grapevine: 10°C
}
}
\section{Methods}{

\describe{
\item{average}{Simple average: GDD = max(0, (Tmax + Tmin)/2 - Tbase).
Most common method, used by many agricultural agencies.}
\item{modified}{Like average, but Tmin and Tmax are first bounded to
the range from Tbase to Tcap before averaging. Prevents negative contributions.}
\item{single_sine}{Approximates the daily temperature curve as a sine
wave, computing the mean excess over Tbase. More accurate when
temperatures cross the base threshold during the day. Implements the
Baskerville-Emin (1969) / Snyder (1985) single-sine formula: with
\eqn{T_{avg} = (T_{min} + T_{max})/2}, \eqn{T_{amp} = (T_{max} - T_{min})/2},
and \eqn{\theta = \mathrm{acos}\left((T_{base} - T_{avg})/T_{amp}\right)},
the daily GDD in the mixed case
(\eqn{T_{min} < T_{base} < T_{max}}) is
\eqn{\mathrm{GDD} = \frac{1}{\pi}\left[(T_{avg} - T_{base})\,\theta +
      T_{amp}\,\sin(\theta)\right]}.}
}
}

\examples{
# Daily temperatures for a week
tmin <- c(5, 7, 8, 10, 12, 11, 9)
tmax <- c(15, 18, 20, 22, 25, 23, 19)

# Cumulative GDD with base 10°C
calc_thermal_units(tmin, tmax, t_base = 10)

# Daily GDD values
calc_thermal_units(tmin, tmax, t_base = 10, cumulative = FALSE)

# With upper cap at 30°C
calc_thermal_units(tmin, tmax, t_base = 10, t_cap = 30)

# Using mean temperature only
tmean <- (tmin + tmax) / 2
calc_thermal_units(tmean, t_base = 10)

}
\references{
McMaster, G.S., Wilhelm, W.W. (1997). Growing degree-days: one equation,
two interpretations. \emph{Agricultural and Forest Meteorology} 87:291-300.

Baskerville, G.L., Emin, P. (1969). Rapid estimation of heat accumulation
from maximum and minimum temperatures. \emph{Ecology} 50:514-517.

Snyder, R.L. (1985). Hand calculating degree days. \emph{Agricultural and
Forest Meteorology} 35:353-358.
}
\seealso{
\code{\link{calc_thermal_sum}} for computing thermal sum at
phenological observations
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_download.R
\name{pep_cache_clear}
\alias{pep_cache_clear}
\title{Clear Cached PEP Data}
\usage{
pep_cache_clear(quiet = FALSE)
}
\arguments{
\item{quiet}{Logical. If \code{TRUE}, suppress messages. Default is \code{FALSE}.}
}
\value{
Invisible \code{TRUE} if cache was cleared, \code{FALSE} otherwise.
}
\description{
Removes cached synthetic PEP data from the local cache directory.
}
\examples{
\donttest{
# Clear the cache
pep_cache_clear()

# Next download will fetch fresh data
pep <- pep_download()
}

}
\seealso{
\code{\link{pep_download}}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_flag_outliers.R
\name{print.pep_outliers}
\alias{print.pep_outliers}
\title{Print Method for Outlier Detection Results}
\usage{
\method{print}{pep_outliers}(x, ...)
}
\arguments{
\item{x}{A \code{pep_outliers} object}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}
}
\description{
Print Method for Outlier Detection Results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{print.pep}
\alias{print.pep}
\title{Print Method for PEP725 Data}
\usage{
\method{print}{pep}(x, n = 5, ...)
}
\arguments{
\item{x}{A \code{pep} object.}

\item{n}{Number of rows to display (default 5).}

\item{...}{Additional arguments passed to print.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a concise summary of the PEP725 phenological dataset.
}
\examples{
\donttest{
pep <- pep_download()
print(pep)
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_combine.R
\name{fit_mixed}
\alias{fit_mixed}
\title{Fit Mixed-Effects Model}
\usage{
fit_mixed(dt)
}
\description{
Fit Mixed-Effects Model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_combine.R
\name{pep_check_connectivity}
\alias{pep_check_connectivity}
\title{Check Station-Year Connectivity}
\usage{
pep_check_connectivity(pep, by = NULL)
}
\arguments{
\item{pep}{A \code{pep} object or data.table with \code{year}, \code{day},
and \code{s_id} columns.}

\item{by}{Optional grouping variables to check connectivity within groups.}
}
\value{
A list with connectivity information:
\describe{
\item{is_connected}{Logical. TRUE if all data forms one connected set.}
\item{n_sets}{Number of disconnected sets found.}
\item{set_sizes}{Number of observations in each set.}
\item{largest_set}{Stations and years in the largest connected set.}
}
}
\description{
Checks whether station-year combinations in phenological data form a
connected set, which is required for valid combined time series estimation.
}
\details{
Connectivity means that there exist overlapping years between stations,
allowing year effects to be separated from station effects. If data is
disconnected, the combined series cannot be uniquely estimated across
the disconnected parts.
}
\examples{
\donttest{
pep <- pep_download()
pep_alpine <- pep[country \%in\% c("Switzerland", "Austria")]
conn <- pep_check_connectivity(pep_alpine)
if (!conn$is_connected) {
  warning("Data has ", conn$n_sets, " disconnected sets")
}
}

}
\seealso{
\code{\link{pheno_combine}} which uses this internally
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_combine.R
\name{pheno_combine}
\alias{pheno_combine}
\title{Create Combined Phenological Time Series}
\usage{
pheno_combine(
  pep,
  by = NULL,
  method = c("robust", "mixed", "ols"),
  min_years = 5,
  min_stations = 3,
  check_connectivity = TRUE
)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table} containing phenological
observations with columns \code{year}, \code{day}, and \code{s_id}.}

\item{by}{Character vector of grouping variables (e.g., \code{c("genus",
  "species", "phase_id")}). A separate combined series is created for each
group. Default is \code{NULL} (treat all data as one group).}

\item{method}{Character. Estimation method:
\describe{
\item{"robust"}{(Default) Least Absolute Deviations (LAD/L1) regression.
Minimizes sum of absolute residuals, robust to outliers.}
\item{"mixed"}{Mixed-effects model with year as fixed effect and station
as random effect. Uses REML estimation.}
\item{"ols"}{Ordinary Least Squares. Sensitive to outliers but provides
standard errors and confidence intervals.}
}}

\item{min_years}{Integer. Minimum years of overlap required. Default 5.}

\item{min_stations}{Integer. Minimum stations required. Default 3.}

\item{check_connectivity}{Logical. If \code{TRUE} (default), checks whether
station-year combinations form a connected set. Warns if data is
disconnected (which prevents valid combined series estimation).}
}
\value{
An object of class \code{pheno_combined} containing:
\describe{
\item{combined}{Data.table with the combined time series: year,
year_effect (the combined DOY), se (standard error, if available)}
\item{station_effects}{Data.table of station effects: s_id, effect, se}
\item{residuals}{Data.table with original data plus fitted values and
residuals for outlier detection}
\item{method}{Estimation method used}
\item{connectivity}{Connectivity check results (if performed)}
\item{by}{Grouping variables used}
\item{group_results}{For grouped analysis, results per group}
}
}
\description{
Estimates a combined (regional/national) phenological time series from
multi-station observations by separating year effects from station effects.
This is essential for creating representative time series from networks
with varying station coverage over time.
}
\details{
The model fitted is: \deqn{DOY_{ij} = \mu_i + \sigma_j + \epsilon_{ij}}
where \eqn{\mu_i} is the year effect (combined series), \eqn{\sigma_j} is
the station effect, and \eqn{\epsilon_{ij}} is the residual.

Station effects represent systematic differences between stations (e.g.,
due to altitude, microclimate, or observer differences). The combined
series removes these effects to reveal the underlying temporal trend.
}
\section{Connectivity}{

A prerequisite for valid estimation is that station-year combinations
form a "connected" set - meaning there must be overlapping years between
stations to separate year and station effects. The function checks this
automatically and warns if data is disconnected.
}

\section{Outlier Detection}{

With \code{method = "robust"}, large residuals (e.g., > 30 days) indicate
potential outliers or data errors. The residuals table can be used to
identify and investigate these.
}

\examples{
\donttest{
pep <- pep_download()

# Grapevine flowering in Switzerland (longest historical records)
vine_ch <- pep[species == "Vitis vinifera" &
               phase_id == 65 & country == "Switzerland"]

if (nrow(vine_ch) > 0) {
  # Create combined series using OLS
  combined <- pheno_combine(vine_ch, method = "ols")
  print(combined)

  # Identify potential outliers (residuals > 30 days)
  outliers <- combined$residuals[abs(residual) > 30]
  outliers
}
}

}
\references{
Schaber J, Badeck F-W (2002). Evaluation of methods for the combination
of phenological time series and outlier detection. Tree Physiology 22:973-982.
}
\seealso{
\code{\link{pheno_normals}} for climatological baselines,
\code{\link{pheno_trend_turning}} for trend turning point detection,
\code{\link{pep_check_connectivity}} for connectivity assessment
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_regional.R
\name{pheno_regional}
\alias{pheno_regional}
\title{Compile Regional Phenology Data and Climate Sensitivity Inputs}
\usage{
pheno_regional(
  pep,
  giss = NULL,
  lon_min = 4.2,
  lon_max = 8.1,
  lat_min = 44.7,
  lat_max = 48.1,
  species_name = "Triticum aestivum",
  functional_group = NULL,
  year_min = 1961,
  pep_for_giss = c("near", "aggregated"),
  phase = 60
)
}
\arguments{
\item{pep}{A data.table containing the full PEP725 dataset. Must include at
least columns: \code{species}, \code{year}, \code{phase_id}, \code{day},
\code{lat}, \code{lon}.}

\item{giss}{Optional. A data.table with GISS global temperature anomalies,
containing columns \code{year}, \code{dT}, and \code{dT_sm}. If \code{NULL},
GISS-related outputs are omitted.}

\item{lon_min}{Minimum longitude for spatial filtering of PEP data (default is \code{4.2})}

\item{lon_max}{Maximum longitude for spatial filtering of PEP data (default is \code{8.1})}

\item{lat_min}{Minimum latitude for spatial filtering of PEP data (default is \code{44.7})}

\item{lat_max}{Maximum latitude for spatial filtering of PEP data (default is \code{48.1})}

\item{species_name}{Character name of the species to be extracted (default is \code{"Triticum aestivum"})}

\item{functional_group}{Character. Optional functional group name to filter by (e.g., "C4_summer_cereals"). If provided, overwrites species_name filtering.}

\item{year_min}{Minimum year to include in all outputs (default is \code{1961})}

\item{pep_for_giss}{Selects which PEP data subset to use for merging with GISS
data. Either \code{"near"} (box-filtered) or \code{"aggregated"} (global).
Ignored if \code{giss} is \code{NULL}.}

\item{phase}{Integer or vector of phase_id(s) to extract (default is \code{60} = Heading). Named phase mappings are applied.}
}
\value{
A named list with the following elements:
\describe{
\item{\code{pep_agg}}{Global (non-spatially filtered) PEP725 time series for the selected phase(s)}
\item{\code{pep_cgi}}{Spatially filtered PEP725 time series based on lat/lon bounding box}
\item{\code{giss}}{Processed GISS global temperature anomaly data (if provided)}
\item{\code{pep_giss}}{Merged data frame of PEP phenology and GISS anomalies (if provided)}
\item{\code{species}}{Character species name used for filtering}
\item{\code{functional_group}}{Functional group if specified, otherwise \code{NA}}
\item{\code{phase}}{Integer phase ID(s) used for filtering}
}
}
\description{
Extracts and aggregates phenological observations for a selected species and
phenological phase from the PEP725 dataset (globally and spatially filtered).
Optionally links them with global temperature anomalies (GISS data) for
climate impact studies.
}
\details{
The function aggregates PEP observations by species, year, and phase,
calculates mean day-of-year (DOY), applies spatial filtering if needed,
and attaches GISS climate anomalies. It also maps \code{phase_id}s to
descriptive names and warns if expected phenological phase are missing
in the selected data.

If GISS data is provided, temperature anomalies are merged with the selected
PEP time series to support phenology-climate analysis.
}
\examples{
\donttest{
pep <- pep_download()

out <- pheno_regional(pep, species_name = "Triticum aestivum", phase = 60)
str(out)
}

}
\seealso{
\code{\link{pheno_plot}}, \code{\link{pep_download}}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_normals.R
\name{print.pheno_normals}
\alias{print.pheno_normals}
\title{Print Method for Phenological Normals}
\usage{
\method{print}{pheno_normals}(x, n = 10, ...)
}
\arguments{
\item{x}{A \code{pheno_normals} object}

\item{n}{Number of rows to display}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}
}
\description{
Print Method for Phenological Normals
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_quality.R
\name{pep_quality}
\alias{pep_quality}
\title{Assess Data Quality of Phenological Observations}
\usage{
pep_quality(
  pep,
  by = c("s_id", "genus", "phase_id"),
  year_range = NULL,
  species = NULL,
  phase_id = NULL,
  outlier_method = c("tukey", "zscore"),
  na.rm = TRUE
)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table} containing phenological
observations with columns \code{year}, \code{day}, and grouping variables.}

\item{by}{Character vector of column names to group by for quality assessment.
Default is \code{c("s_id", "genus", "phase_id")} for station-level assessment.}

\item{year_range}{Optional integer vector of length 2 specifying the start and
end years for assessment. If \code{NULL} (default), uses the full range
of years in the data.}

\item{species}{Optional character string to filter by species/genus.}

\item{phase_id}{Optional integer vector to filter by BBCH phase codes.}

\item{outlier_method}{Character string specifying outlier detection method.
Either \code{"tukey"} (default, uses 1.5*IQR rule) or \code{"zscore"}
(flags values with |z| > 3).}

\item{na.rm}{Logical. Should missing values be removed? Default \code{TRUE}.}
}
\value{
A \code{data.table} with the following columns:
\describe{
\item{by variables}{Grouping variables as specified in \code{by}}
\item{n_obs}{Total number of observations}
\item{year_min}{First year of observations}
\item{year_max}{Last year of observations}
\item{year_span}{Total span in years (year_max - year_min + 1)}
\item{n_years}{Number of distinct years with data}
\item{completeness_pct}{Percentage of years with data within the span}
\item{max_gap_years}{Length of the longest gap (consecutive missing years)}
\item{n_gaps}{Number of gaps in the time series}
\item{n_outliers}{Number of statistical outliers detected}
\item{outlier_pct}{Percentage of observations that are outliers}
\item{doy_mean}{Mean day-of-year}
\item{doy_sd}{Standard deviation of day-of-year}
\item{doy_range}{Range of day-of-year values}
\item{quality_grade}{Overall quality grade (A, B, C, or D)}
}
}
\description{
Provides a comprehensive assessment of data completeness, temporal coverage,
outlier prevalence, and assigns quality grades to phenological time series.
}
\details{
This function evaluates phenological time series quality based on multiple
criteria important for trend analysis and climatological calculations.
}
\section{Quality Grading Criteria}{

\describe{
\item{Grade A}{Completeness >= 80\%, no gaps > 2 years, outliers < 2\%}
\item{Grade B}{Completeness >= 60\%, no gaps > 5 years, outliers < 5\%}
\item{Grade C}{Completeness >= 40\%, gaps <= 10 years, outliers < 10\%}
\item{Grade D}{Below grade C thresholds}
}
}

\section{Outlier Detection}{

The Tukey method (default) flags observations outside the interquartile
fences: \code{[Q1 - 1.5*IQR, Q3 + 1.5*IQR]}. This is robust to non-normal
distributions common in phenological data.
}

\examples{
\donttest{
pep <- pep_download()

# Subset to one country for speed
pep_ch <- pep[country == "Switzerland"]

# Assess quality for stations in Switzerland
quality <- pep_quality(pep_ch)
print(quality)

# Filter to high-quality stations only
high_quality <- quality[quality_grade \%in\% c("A", "B")]
high_quality

# Assess specific species and phase
apple_quality <- pep_quality(pep_ch,
                             species = "Malus",
                             phase_id = 60)
apple_quality

# Country-level assessment (use subset of countries)
pep_subset <- pep[country \%in\% c("Switzerland", "Austria")]
country_quality <- pep_quality(pep_subset,
                               by = c("country", "genus", "phase_id"))
country_quality

# Assess quality within a specific time window
modern_quality <- pep_quality(pep_ch,
                              year_range = c(1991, 2020))
modern_quality
}

}
\references{
Schaber, J. and Badeck, F.-W. (2002). Evaluation of methods for the
combination of phenological time series and outlier detection.
\emph{Tree Physiology}, 22, 973--982.
\doi{10.1093/treephys/22.14.973}
}
\seealso{
\code{\link{pheno_normals}} for calculating climatological normals,
\code{\link{pheno_anomaly}} for anomaly detection
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_pls.R
\name{summary.pheno_pls}
\alias{summary.pheno_pls}
\title{Summary Method for PLS Phenology Results}
\usage{
\method{summary}{pheno_pls}(object, vip_threshold = 0.8, ...)
}
\arguments{
\item{object}{A \code{pheno_pls} object}

\item{vip_threshold}{Numeric. VIP threshold for identifying important
periods. Default 0.8.}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns a summary data.frame
}
\description{
Summary Method for PLS Phenology Results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_timeseries.R
\name{pheno_plot_timeseries}
\alias{pheno_plot_timeseries}
\title{Plot Phenological Time Series}
\usage{
pheno_plot_timeseries(
  data,
  color_by = "species",
  facet_by = NULL,
  alpha_lines = 0.7,
  linewidth = 0.8,
  smooth = TRUE,
  se = FALSE,
  year_min = 1961,
  title = "Phenological time series of flowering (BBCH 65)"
)
}
\arguments{
\item{data}{A data.frame or data.table containing at least \code{year}, \code{day} (day-of-year), and one grouping variable
(e.g. \code{species}, \code{phase}, or \code{functional_group}).}

\item{color_by}{Character. Column name used for color grouping (default = "species").}

\item{facet_by}{Character or NULL. Optional column name for faceting (e.g. "functional_group").}

\item{alpha_lines}{Numeric. Transparency level for time-series lines (default = 0.7).}

\item{linewidth}{Numeric. Line width (default = 0.8).}

\item{smooth}{Logical. If TRUE, adds a linear trend smoother (default = TRUE).}

\item{se}{Logical. Whether to display confidence interval for smoother (default = FALSE).}

\item{year_min}{Numeric. Minimum year to plot (default = 1961).}

\item{title}{Character. Optional plot title.}
}
\value{
A ggplot object.
}
\description{
Visualizes phenological day-of-year (DOY) trends over time since 1961.
Designed for aggregated or site-level phenology data from PEP725.
}
\examples{
\donttest{
pep <- pep_download()

# Use Alpine subset for faster computation
pep_alpine <- pep[country \%in\% c("Switzerland", "Austria")]

# Example: flowering DOY by genus
pheno_plot_timeseries(data = pep_alpine[phase_id == 65],
                      color_by = "genus",
                      facet_by = NULL, smooth = TRUE)

# Example: single species, colored by site (grapevine has longest records)
vine_data <- pep_alpine[species == "Vitis vinifera" & phase_id == 65]
if (nrow(vine_data) > 0) {
  pheno_plot_timeseries(
    data = vine_data,
    color_by = "s_id",
    smooth = TRUE,
    title = "Grapevine flowering (BBCH 65) across sites"
  )
}

# Example: facets by genus
pheno_plot_timeseries(
  data = pep_alpine[phase_id == 65],
  color_by = "species",
  facet_by = "genus",
  smooth = TRUE
)
}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_anomaly.R
\name{pheno_anomaly}
\alias{pheno_anomaly}
\title{Calculate Phenological Anomalies}
\usage{
pheno_anomaly(
  pep,
  baseline_period = 1961:1990,
  target_years = NULL,
  by = c("country", "genus", "phase_id"),
  species = NULL,
  phase_id = NULL,
  robust = TRUE,
  min_years = 15,
  extreme_threshold = 2,
  normals = NULL,
  na.rm = TRUE
)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table} containing phenological
observations with columns \code{year}, \code{day}, and grouping variables.}

\item{baseline_period}{Integer vector specifying the years to use for
baseline calculation. Default is \code{1961:1990} (pre-acceleration
warming reference period).}

\item{target_years}{Integer vector specifying years to calculate anomalies
for. If \code{NULL} (default), calculates for all years in the data.}

\item{by}{Character vector of column names to group by. Must match columns
in the data. Default is \code{c("country", "genus", "phase_id")}.}

\item{species}{Optional character string to filter by species/genus.
If \code{NULL} (default), all species are included.}

\item{phase_id}{Optional integer vector to filter by BBCH phase codes.
If \code{NULL} (default), all phases are included.}

\item{robust}{Logical. If \code{TRUE} (default), uses median and MAD for
baseline and standardization. If \code{FALSE}, uses mean and SD.}

\item{min_years}{Minimum number of years required in the baseline period
to calculate valid anomalies. Default is 15.}

\item{extreme_threshold}{Numeric. Z-score threshold for flagging extreme
events. Default is 2 (approximately 95th percentile for normal distribution).}

\item{normals}{Optional pre-computed \code{pheno_normals} object. If provided,
baseline statistics are taken from this object instead of being computed.}

\item{na.rm}{Logical. Should missing values be removed? Default \code{TRUE}.}
}
\value{
A \code{data.table} with the following columns:
\describe{
\item{by variables}{Grouping variables as specified in \code{by}}
\item{year}{Year of observation}
\item{observed_doy}{Observed mean day-of-year for the group/year}
\item{baseline_doy}{Long-term baseline (median if robust, mean otherwise)}
\item{baseline_spread}{Baseline spread (MAD if robust, SD otherwise)}
\item{anomaly_days}{Deviation in days (negative = early, positive = late)}
\item{z_score}{Standardized anomaly (anomaly / spread)}
\item{percentile}{Percentile rank relative to baseline distribution}
\item{is_extreme}{Logical flag for extreme events (|z_score| > threshold)}
\item{direction}{Character: "early", "late", or "normal"}
}
}
\description{
Computes deviations from long-term phenological baselines to identify
extreme years and detect climate signals. Returns both raw anomalies
(in days) and standardized anomalies for cross-species comparison.
}
\details{
Phenological anomalies quantify how much a given year deviates from
long-term expectations. This is crucial for:
\itemize{
\item Detecting climate change signals in phenology
\item Identifying extreme phenological years
\item Comparing anomalies across different species/regions
}

The function supports two approaches:
\itemize{
\item \strong{Robust} (default): Uses median for central tendency and MAD
(Median Absolute Deviation) for spread. More resistant to outliers.
\item \strong{Classical}: Uses mean and standard deviation. More sensitive
to extreme values but provides interpretable z-scores under normality.
}
}
\section{Interpretation}{

\itemize{
\item \strong{anomaly_days}: Direct interpretation - "X days earlier/later
than normal"
\item \strong{z_score}: Standardized - values > 2 or < -2 are typically
considered extreme
\item \strong{percentile}: Normal approximation from z-score - e.g.,
5th percentile means "earlier than 95\\% of baseline years"
}
}

\examples{
\donttest{
pep <- pep_download()

# Grapevine in Switzerland (longest historical records back to 1775)
vine_ch <- pep[country == "Switzerland" & species == "Vitis vinifera"]

if (nrow(vine_ch) > 0) {
  # Calculate anomalies relative to 1961-1990 baseline
  anomalies <- pheno_anomaly(vine_ch,
                             baseline_period = 1961:1990,
                             phase_id = 65)
  print(anomalies)

  # Find extreme early years
  extreme_early <- anomalies[is_extreme == TRUE & direction == "early"]
  extreme_early

  # Anomalies for recent years only
  recent <- pheno_anomaly(vine_ch,
                          baseline_period = 1961:1990,
                          target_years = 2000:2020,
                          phase_id = 65)
  recent
}
}

}
\references{
Menzel, A. et al. (2006). European phenological response to climate
change matches the warming pattern.
\emph{Global Change Biology}, 12(10), 1969--1976.
\doi{10.1111/j.1365-2486.2006.01193.x}

Templ, M., Templ, B., and Filzmoser, P. (2018).
Modelling and prediction of phenological data of the beginning of
flowering: Austrian indicators of climate change.
\emph{International Journal of Biometeorology}, 62, 1319--1334.
\doi{10.1007/s00484-018-1534-2}
}
\seealso{
\code{\link{pheno_normals}} for calculating baseline statistics,
\code{\link{pep_download}} for obtaining the main dataset
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_normals.R
\name{summary.pheno_normals}
\alias{summary.pheno_normals}
\title{Summary Method for Phenological Normals}
\usage{
\method{summary}{pheno_normals}(object, ...)
}
\arguments{
\item{object}{A \code{pheno_normals} object}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns a summary list
}
\description{
Summary Method for Phenological Normals
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_daylength.R
\name{add_daylength}
\alias{add_daylength}
\title{Add Daylength to Phenological Data}
\usage{
add_daylength(pep)
}
\arguments{
\item{pep}{A \code{pep} object or data.table with \code{day} and
\code{lat} columns.}
}
\value{
The input data with an added \code{daylength} column (hours).
}
\description{
Adds a daylength column to a phenological dataset based on the
observation day and station latitude.
}
\examples{
\donttest{
pep <- pep_download()
pep <- add_daylength(pep)
head(pep[, .(day, lat, daylength)])
}

}
\seealso{
\code{\link{calc_daylength}} for the underlying calculation
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{pep-class}
\alias{pep-class}
\title{PEP725 Phenological Data Class}
\description{
S3 class for PEP725 phenological data that inherits from \code{data.table}.
Provides validation, informative print/summary methods, and convenient defaults.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_trend_turning.R
\name{pheno_trend_turning}
\alias{pheno_trend_turning}
\title{Detect Trend Turning Points in Phenological Time Series}
\usage{
pheno_trend_turning(pep, by = NULL, min_years = 10, aggregate = TRUE)
}
\arguments{
\item{pep}{A \code{pep} object, data.table, or numeric vector of DOY values.
If a data.frame/data.table, must contain \code{year} and \code{day} columns.}

\item{by}{Character vector of column names to group by before analysis.
Default is \code{NULL} (analyze all data as one series). Common choices:
\code{c("genus", "species", "phase_id")} or \code{c("country", "phase_id")}.}

\item{min_years}{Integer. Minimum number of years required for analysis.
Default is 10. Shorter series may produce unreliable results.}

\item{aggregate}{Logical. If \code{TRUE} (default), aggregates multiple
observations per year (e.g., from multiple stations) using the median.
If \code{FALSE}, expects exactly one value per year.}
}
\value{
An object of class \code{pheno_turning} containing:
\describe{
\item{results}{Data.table with columns: year, day (or median_day),
tau_prog (progressive tau), tau_retr (retrograde tau),
is_turning (logical indicating turning points)}
\item{turning_points}{Years identified as potential turning points}
\item{n_years}{Number of years in the series}
\item{by}{Grouping variables used (if any)}
\item{group_results}{If \code{by} is specified, list of results per group}
}
}
\description{
Applies a sequential Mann-Kendall test to detect approximate trend turning
points in phenological time series. This identifies years where trends
may have changed direction (e.g., when spring advancement accelerated or
reversed).
}
\details{
The sequential Mann-Kendall test calculates two series:
\itemize{
\item \strong{Progressive}: Kendall's tau computed from the start forward
\item \strong{Retrograde}: Kendall's tau computed from the end backward
}

Points where these series cross indicate potential trend turning points.
When either series exceeds confidence thresholds (|tau| > 1.96 for 95\%,
|tau| > 2.58 for 99\%) before and after a crossing, the turning point
is considered statistically significant.
}
\section{Interpretation}{

\itemize{
\item Positive tau indicates an increasing trend (later DOY = delayed phenology)
\item Negative tau indicates a decreasing trend (earlier DOY = advanced phenology)
\item Crossing points suggest the trend direction may have changed
}
}

\examples{
# Simple vector input (fast)
doy_series <- c(120, 118, 122, 115, 110, 108, 112, 105, 102, 100)
turning <- pheno_trend_turning(doy_series)
print(turning)

# Using pep_seed data (no grouping for speed)
data(pep_seed)
vine <- pep_seed[pep_seed$species == "Vitis vinifera" &
                 pep_seed$phase_id == 65, ]
if (nrow(vine) > 0) {
  turning <- pheno_trend_turning(vine)
}

}
\references{
Sneyers R (1990). On statistical analysis of series of observations.
Technical Note No 143. World Meteorological Organization.
}
\seealso{
\code{\link{pheno_gradient}} for spatial trend analysis,
\code{\link{pheno_normals}} for baseline calculations
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_phenological_doys.R
\name{get_phenology_doys}
\alias{get_phenology_doys}
\title{Compute CTRL and SCEN Phenology Day-of-Year Values}
\usage{
get_phenology_doys(
  pep,
  phase1,
  phase2,
  genus_name,
  phenology_method = c("robust_shift", "simple_shift", "gdd_cmip6"),
  ctrl_years = 2011:2021,
  scen_years = 2085:2095,
  calib_years = 1991:2020,
  scen_warming = 3,
  shift_per_deg = -4,
  cmip6_tas = NULL,
  gdd_base = 5,
  gdd_threshold1 = 150,
  gdd_threshold2 = 350,
  regions = NULL,
  delta_Tgl = 2.09,
  giss_data = NULL
)
}
\arguments{
\item{pep}{A data frame containing PEP725 phenology observations, including
columns \code{country}, \code{day}, \code{year}, \code{phase_id}, and \code{genus}.}

\item{phase1}{Integer. First phenophase (e.g. 65).}

\item{phase2}{Integer. Second phenophase (e.g. 87).}

\item{genus_name}{Character. Genus to filter (e.g. "Malus").}

\item{phenology_method}{Either \code{"simple_shift"} or \code{"gdd_cmip6"}.}

\item{ctrl_years}{Integer vector. Years for CTRL period (default 2011:2021).}

\item{scen_years}{Integer vector. Years for scenario period (default 2085:2095).}

\item{calib_years}{Integer vector. Years for calibration period (default 1991:2020).}

\item{scen_warming}{Numeric. Warming in degC (simple shift model).}

\item{shift_per_deg}{Numeric. DOY shift per degC (simple shift).}

\item{cmip6_tas}{Optional list of daily temperature vectors--only required
if using \code{"gdd_cmip6"}.}

\item{gdd_base}{Numeric. Base temperature for GDD model.}

\item{gdd_threshold1}{Numeric. GDD threshold for phase1.}

\item{gdd_threshold2}{Numeric. GDD threshold for phase2.}

\item{regions}{Character vector. Countries/regions to include. If NULL, all are used.}

\item{delta_Tgl}{Numeric. Global temperature change (degC) for scenario.}

\item{giss_data}{Data frame with GISS temperature data (columns: \code{year},
\code{dT} or \code{Tgl}). Required for robust_shift method.}
}
\value{
A data frame containing:
\item{Country}{Country name}
\item{doy_start_ctrl}{Start DOY for CTRL period}
\item{doy_end_ctrl}{End DOY for CTRL period}
\item{doy_start_scen}{Start DOY for SCEN period}
\item{doy_end_scen}{End DOY for SCEN period}
}
\description{
This function computes the mean phenological day-of-year (DOY) for two
phenophases (e.g. BBCH 65 and 87) for each country, using either a simple
warming-shift model or a GDD-based model.
}
\examples{
\donttest{
pep <- pep_download()

# Use Alpine subset for faster computation
regions <- c("Austria", "Switzerland")
pep_alpine <- pep[country \%in\% regions]

# Simple shift method
d1 <- get_phenology_doys(
  pep = pep_alpine,
  phase1 = 65,
  phase2 = 87,
  genus_name = "Malus",
  regions = regions,
  phenology_method = 'simple_shift'
)
d1
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_flag_outliers.R
\name{pep_flag_outliers}
\alias{pep_flag_outliers}
\title{Flag Phenological Outliers}
\usage{
pep_flag_outliers(
  pep,
  by = c("s_id", "genus", "species", "phase_id"),
  method = c("30day", "mad", "iqr", "zscore", "gam_residual", "mahalanobis"),
  threshold = NULL,
  center = c("median", "mean"),
  min_obs = 5,
  formula = day ~ s(year) + s(alt) + s(lat) + s(s_id, bs = "re"),
  min_n_per_group = 50,
  flag_only = TRUE
)
}
\arguments{
\item{pep}{A \code{pep} object or data.table with phenological observations.
Must contain \code{year}, \code{day}, and grouping columns.}

\item{by}{Character vector of columns defining groups for outlier detection.
Default \code{c("s_id", "genus", "species", "phase_id")} detects outliers
within each station-species-phase combination.}

\item{method}{Character. Outlier detection method:
\describe{
\item{"30day"}{(Default) Flag observations >30 days from group mean/median.
The classic Schaber-Badeck approach for phenological data.}
\item{"mad"}{Use Median Absolute Deviation. Flag if |deviation| > k * MAD,
where k is set by \code{threshold}.}
\item{"iqr"}{Use Interquartile Range. Flag if outside (Q1 - k\emph{IQR, Q3 + k}IQR).}
\item{"zscore"}{Flag if |z-score| > threshold (default 3).}
\item{"gam_residual"}{Fit a GAM (via \code{mgcv::gam()}) per group
using the supplied \code{formula} — by default accounting for year
trend, elevation, latitude, and a station random intercept — and
flag observations whose robust-z-scored residual exceeds
\code{threshold}. Detects covariate-inconsistent anomalies that
the within-group \code{"30day"} rule misses. Falls back to
\code{"30day"} on groups with fewer than \code{min_n_per_group}
observations or when the model fails to converge.}
\item{"mahalanobis"}{Treat each station-year as a vector of DOYs
across phases and flag station-years whose \strong{robust} Mahalanobis
distance exceeds the \code{threshold}. The centre and covariance
are estimated by the Minimum Covariance Determinant (MCD)
estimator of Rousseeuw (1984), via
\code{robustbase::covMcd()} — using the classical, non-robust
covariance here would let the outliers contaminate the
estimate and mask themselves. Under MCD the squared distance is
still approximately \eqn{\chi^2_p}, so the default threshold
is \eqn{\sqrt{\chi^2_{0.975, p}}}, where \eqn{p} is the number
of phases present in the group. Detects station-years whose
pattern across phases is jointly inconsistent (e.g.\ BBCH 60
and 65 impossibly close) even when each marginal DOY looks
fine. All rows within a flagged station-year are marked; the
\code{deviation} column stores the robust Mahalanobis distance.
Groups with too few complete station-years (fewer than
\eqn{\max(p + 5, 15)}) or a singular MCD fit fall back to the
30-day rule.}
}}

\item{threshold}{Numeric. Threshold for outlier detection.
\describe{
\item{For "30day":}{Days deviation to flag as outlier. Default 30.}
\item{For "mad":}{Number of MADs. Default 3 (~2.5 SD equivalent).}
\item{For "iqr":}{IQR multiplier. Default 1.5 (standard Tukey rule).}
\item{For "zscore":}{Z-score threshold. Default 3.}
\item{For "gam_residual":}{Robust-z threshold on residuals. Default 3.5.}
}}

\item{center}{Character. Central tendency measure: \code{"median"} (default,
more robust) or \code{"mean"}.}

\item{min_obs}{Integer. Minimum observations per group required for outlier
detection. Groups with fewer observations are skipped. Default 5.}

\item{formula}{Optional \code{formula} for \code{method = "gam_residual"}.
Default is
\code{day ~ s(year) + s(alt) + s(lat) + s(s_id, bs = "re")}; any
smooth term referencing a column missing from \code{pep} is silently
dropped.}

\item{min_n_per_group}{Integer. Minimum group size required before fitting
the GAM in \code{method = "gam_residual"}. Groups below this threshold
fall back to the \code{"30day"} rule. Default 50.}

\item{flag_only}{Logical. If \code{TRUE} (default), adds outlier flag columns
to data. If \code{FALSE}, removes flagged outliers from data.}
}
\value{
If \code{flag_only = TRUE}: The input data with additional columns:
\describe{
\item{is_outlier}{Logical. TRUE for flagged outliers.}
\item{deviation}{Numeric. Days deviation from expected value.}
\item{expected_doy}{Numeric. Expected DOY for the group.}
}
If \code{flag_only = FALSE}: Data with outliers removed.
}
\description{
Identifies outlier observations in phenological data using the 30-day rule
or statistical methods. Based on the approach described by Schaber & Badeck
(2002) for quality control of phenological observations.
}
\details{
The 30-day rule (Schaber & Badeck 2002) is widely used for phenological data
quality control. It flags observations that deviate more than 30 days from
the expected value for that station-species-phase combination. This threshold
is based on the observation that legitimate phenological variation rarely
exceeds one month.
}
\section{When to Use Each Method}{

\describe{
\item{30day}{Standard for phenological QC. Use when the 30-day biological
threshold is meaningful.}
\item{mad}{Robust to existing outliers. Good for initial screening of
potentially contaminated data.}
\item{iqr}{Standard statistical approach. Useful when comparing with
other quality metrics.}
\item{zscore}{Parametric approach. Use when data is approximately normal.}
}
}

\examples{
\donttest{
data(pep_seed)

# Flag outliers using 30-day rule
pep_flagged <- pep_flag_outliers(pep_seed)
table(pep_flagged$is_outlier)

# View flagged observations
pep_flagged[is_outlier == TRUE]

# Remove outliers instead of flagging
pep_clean <- pep_flag_outliers(pep_seed, flag_only = FALSE)

# Use MAD method with stricter threshold
pep_flagged <- pep_flag_outliers(pep_seed, method = "mad", threshold = 2.5)

# Group by country instead of station
pep_flagged <- pep_flag_outliers(pep_seed,
                              by = c("country", "genus", "phase_id"))
}

}
\references{
Schaber J, Badeck F-W (2002). Evaluation of methods for the combination of
phenological time series and outlier detection. Tree Physiology 22:973-982.

Rousseeuw P J (1984). Least median of squares regression.
\emph{Journal of the American Statistical Association} 79(388):871-880.
(Minimum Covariance Determinant estimator used by
\code{method = "mahalanobis"}.)
}
\seealso{
\code{\link{pep_quality}} for comprehensive quality assessment,
\code{\link{pheno_combine}} which uses residuals for outlier detection
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_combine.R
\name{print.pheno_combined}
\alias{print.pheno_combined}
\title{Print Method for Combined Time Series}
\usage{
\method{print}{pheno_combined}(x, ...)
}
\arguments{
\item{x}{A \code{pheno_combined} object}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}
}
\description{
Print Method for Combined Time Series
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_download.R
\name{pep_download}
\alias{pep_download}
\title{Download Synthetic PEP725 Data}
\usage{
pep_download(url = NULL, cache = TRUE, force = FALSE, quiet = FALSE)
}
\arguments{
\item{url}{Character string specifying the URL to download from.
Default uses the official pep725 package data repository.}

\item{cache}{Logical. If \code{TRUE} (default), cache the downloaded data
locally for future use.}

\item{force}{Logical. If \code{TRUE}, force re-download even if cached
data exists. Default is \code{FALSE}.}

\item{quiet}{Logical. If \code{TRUE}, suppress download progress messages.
Default is \code{FALSE}.}
}
\value{
A \code{pep} object containing synthetic phenological data with
the same structure as original PEP725 data.
}
\description{
Downloads a pre-generated synthetic version of PEP725 phenological data
from an external repository. The data is cached locally after the first
download to avoid repeated downloads.
}
\details{
The synthetic data preserves the statistical properties and structure of
real PEP725 data (stations, species, phases, temporal patterns) while
ensuring data privacy. It is suitable for:
\itemize{
\item Learning and testing package functions
\item Reproducing examples from vignettes
\item Developing analysis workflows before accessing real data
}

The data is cached in the platform-appropriate user cache directory
determined by \code{\link[tools]{R_user_dir}("pep725", which = "cache")}.
Typical locations:
\itemize{
\item Windows: \code{\%LOCALAPPDATA\%/R/cache/R/pep725/}
\item macOS: \code{~/Library/Caches/org.R-project.R/R/pep725/}
\item Linux: \code{~/.cache/R/pep725/}
}
}
\section{Data Access}{

If you have access to real PEP725 data, you can import it using
\code{\link{pep_import}} instead.
}

\examples{
\donttest{
# Download synthetic data
pep <- pep_download()

# Explore the data
print(pep)
summary(pep)

# Force re-download
pep <- pep_download(force = TRUE)
}

}
\seealso{
\code{\link{pep_simulate}} for creating your own synthetic data,
\code{\link{pep_import}} for importing real PEP725 data
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{new_pep}
\alias{new_pep}
\title{Create a PEP725 Phenological Data Object}
\usage{
new_pep(x, validate = TRUE)
}
\arguments{
\item{x}{A data.frame or data.table containing PEP725 phenological data.}

\item{validate}{Logical. If \code{TRUE} (default), validates that required
columns are present and have correct types.}
}
\value{
An object of class \code{c("pep", "data.table", "data.frame")}.
}
\description{
Constructor function that creates a validated \code{pep} object from a
data.frame or data.table. The object inherits from \code{data.table} and
retains all its functionality.
}
\details{
Required columns:
\itemize{
\item \code{s_id} - Station ID
\item \code{lon}, \code{lat} - Coordinates
\item \code{genus}, \code{species} - Plant taxonomy
\item \code{phase_id} - BBCH phenological phase code
\item \code{year}, \code{day} - Observation timing
}
}
\examples{
\donttest{
# From imported data
dt <- pep_download()
pep_data <- new_pep(dt)
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_second_events.R
\name{print.second_events}
\alias{print.second_events}
\title{Print Method for Second Events Detection}
\usage{
\method{print}{second_events}(x, ...)
}
\arguments{
\item{x}{A \code{second_events} object}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}
}
\description{
Print Method for Second Events Detection
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subspecies_report.R
\name{subspecies_report}
\alias{subspecies_report}
\title{Comprehensive Subspecies Availability Report}
\usage{
subspecies_report(
  pep,
  genus_name = NULL,
  species_name = NULL,
  subspecies_name = NULL,
  phases = c(65, 87),
  years = 1961:2024,
  subregions = c("Austria", "Germany-North", "Germany-South", "Switzerland"),
  include_empty = TRUE,
  metric = "all",
  bbch_names = NULL,
  return_results = TRUE
)
}
\arguments{
\item{pep}{A PEP725-style data frame containing at least:
\code{genus}, \code{species}, \code{subspecies}, \code{country}, \code{year}, \code{phase_id},
\code{s_id}, and \code{day} or \code{DOY}.}

\item{genus_name}{Character (optional). Genus to filter.}

\item{species_name}{Character (optional). Species to filter.}

\item{subspecies_name}{Character vector (optional). One or more subspecies.}

\item{phases}{Integer vector. BBCH phases to evaluate.}

\item{years}{Numeric vector. Years to include.}

\item{subregions}{Character vector of region/country names.}

\item{include_empty}{Logical. If TRUE, include subspecies x country rows
even when no observations are present.}

\item{metric}{Character vector. Metrics to compute. Use \code{"all"} for:
\code{"n_obs"}, \code{"n_stations"}, \code{"first_year"}, \code{"last_year"},
\code{"median_doy"}, \code{"completeness"}.}

\item{bbch_names}{Optional named list mapping BBCH phase numbers to labels,
e.g. \code{list("65" = "Flowering", "87" = "Maturity")}.}

\item{return_results}{Logical. If TRUE, returns a list invisibly.}
}
\value{
Invisibly returns a list with:
\describe{
\item{summary_table}{Tidy long-format summary}
\item{wide_table}{Publication-ready wide-format summary}
\item{heatmap}{A ggplot2 object}
}
}
\description{
This function produces a complete report for phenological subspecies
availability, combining:
(1) a tidy summary table,
(2) a publication-ready wide table, and
(3) a ggplot2 heatmap showing data presence/absence.
}
\details{
Users may specify a \strong{genus}, \strong{species}, or one or more
\strong{subspecies}. The report evaluates BBCH phases, regions, years,
completeness, and several optional availability metrics.
}
\examples{
\donttest{
pep <- pep_download()

# Use Alpine subset for faster computation
pep_alpine <- pep[country \%in\% c("Switzerland", "Austria")]

# Grapevine subspecies (wine varieties) - longest historical records
if (nrow(pep_alpine[species == "Vitis vinifera"]) > 0) {
  subspecies_report(
    pep_alpine,
    subspecies_name = c("Vitis vinifera Riesling",
                        "Vitis vinifera MUELLER THURGAU WEISS"),
    subregions = c("Switzerland", "Austria"),
    metric = "all",
    bbch_names = list("65"="Flowering", "81"="Veraison")
  )

  subspecies_report(
    pep_alpine,
    species_name = "Vitis vinifera",
    subregions = c("Switzerland", "Austria"),
    metric = c("n_obs","median_doy","completeness")
  )

  subspecies_report(
    pep_alpine,
    genus_name = "Vitis",
    subregions = c("Switzerland", "Austria"),
    metric = "all"
  )
}
}
}
\seealso{
\code{\link{pheno_trend_turning}} for trend visualization
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{is.pep}
\alias{is.pep}
\title{Test if Object is a PEP725 Data Object}
\usage{
is.pep(x)
}
\arguments{
\item{x}{An R object.}
}
\value{
Logical; \code{TRUE} if \code{x} is of class \code{pep}.
}
\description{
Test if Object is a PEP725 Data Object
}
\examples{
\donttest{
pep <- pep_download()
is.pep(pep)
is.pep(data.frame(x = 1))
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.R
\docType{data}
\name{pep_seed}
\alias{pep_seed}
\title{Minimal Seed Dataset for Synthetic Data Generation}
\format{
A data.table with 1,319 rows and 10 variables:
\describe{
\item{s_id}{Station identifier (factor)}
\item{lon}{Longitude in decimal degrees}
\item{lat}{Latitude in decimal degrees}
\item{alt}{Altitude in meters}
\item{genus}{Plant genus (factor)}
\item{species}{Full species name (factor)}
\item{phase_id}{BBCH phenological phase code (integer)}
\item{year}{Observation year}
\item{day}{Day of year (DOY) of the phenological event}
\item{country}{Country name}
}
}
\source{
Derived from PEP725 Pan European Phenology Database \url{https://pep725.eu/}
}
\description{
A small subset of phenological data suitable for use with \code{\link{pep_simulate}}
to generate synthetic phenological datasets. This dataset contains real structure
but limited scope, making it suitable for package examples and testing.
}
\details{
This dataset includes observations for:
\itemize{
\item \strong{Species}: Triticum aestivum (wheat), Vitis vinifera (grapevine),
Malus domestica (apple)
\item \strong{Phases}: BBCH 10 (emergence), 60 (flowering), 65 (full flowering)
\item \strong{Countries}: Austria, Germany-South
\item \strong{Years}: 1990-2015
\item \strong{Stations}: 30 stations
}

For a full synthetic dataset, use \code{\link{pep_download}} to download
pre-generated synthetic data from the package repository.
}
\examples{
# Load the seed dataset
data(pep_seed)

# Examine structure
str(pep_seed)

# Use with pep_simulate to create synthetic data
\donttest{
pep_synthetic <- pep_simulate(pep_seed)
}

}
\seealso{
\code{\link{pep_simulate}} for generating synthetic data,
\code{\link{pep_download}} for downloading full synthetic dataset
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_trend_turning.R
\name{mann_kendall_z}
\alias{mann_kendall_z}
\title{Mann-Kendall Z-Statistic}
\usage{
mann_kendall_z(x)
}
\arguments{
\item{x}{Numeric vector (time series assumed to be equidistant).
\code{NA} values are silently removed.}
}
\value{
Numeric. The Mann-Kendall Z-statistic. Positive values indicate an
increasing trend, negative values decreasing. Compare against standard
normal quantiles (e.g., \code{|Z| > 1.96} for \code{p < 0.05}).
Returns \code{NA} if fewer than three non-missing values.
}
\description{
Computes the Mann-Kendall Z-statistic for a time series, a non-parametric
test of monotonic trend. The Z-statistic is approximately standard normal
under the no-trend null hypothesis.
}
\details{
The implementation uses the standard tie correction for \code{Var(S)} (so
that tied observations do not inflate the test) and the continuity
correction \code{Z = (S - sign(S))/sqrt(Var(S))}, matching
\code{Kendall::MannKendall()}.
}
\section{Note on \code{kendall_tau()}}{

Earlier versions of pep725 exported a function called \code{kendall_tau()}
that actually returned this Mann-Kendall Z-statistic (not the true
Kendall's \eqn{\tau} = \eqn{S / (n(n-1)/2)}). \code{kendall_tau()} is now
a deprecated alias of \code{mann_kendall_z()}; please update callers.
}

\examples{
# Decreasing trend (earlier phenology)
mann_kendall_z(c(120, 118, 115, 112, 110, 108, 105))

# No clear trend
mann_kendall_z(c(120, 115, 122, 118, 121, 116, 119))

}
\references{
Mann, H.B. (1945). Nonparametric tests against trend. \emph{Econometrica}
13, 245-259.

Kendall, M.G. (1975). \emph{Rank Correlation Methods}. 4th ed. London:
Charles Griffin.
}
\seealso{
\code{\link{kendall_tau}} (deprecated alias);
\code{\link{pheno_trend_turning}} for a full sequential Mann-Kendall
analysis.
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_pls.R
\name{plot.pheno_pls}
\alias{plot.pheno_pls}
\title{Plot Method for PLS Phenology Results}
\usage{
\method{plot}{pheno_pls}(x, type = c("vip", "coef", "both"), vip_threshold = 0.8, ...)
}
\arguments{
\item{x}{A \code{pheno_pls} object}

\item{type}{Character. Plot type: "vip" for VIP scores (default),
"coef" for coefficients, "both" for combined plot.}

\item{vip_threshold}{Numeric. Threshold line for VIP plot. Default 0.8.}

\item{...}{Additional arguments passed to plotting functions}
}
\value{
A ggplot object (invisibly)
}
\description{
Plot Method for PLS Phenology Results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{[.pep}
\alias{[.pep}
\title{Subset PEP725 Data While Preserving Class}
\usage{
\method{[}{pep}(x, ...)
}
\arguments{
\item{x}{A \code{pep} object.}

\item{...}{Arguments passed to the data.table subset method.}
}
\value{
A \code{pep} object if structure is preserved, otherwise a data.table.
}
\description{
Ensures that subsetting operations return a \code{pep} object
when the result still has the required structure.
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_combine.R
\name{check_connectivity_internal}
\alias{check_connectivity_internal}
\title{Check Connectivity of Station-Year Data (Internal)}
\usage{
check_connectivity_internal(dt)
}
\description{
Check Connectivity of Station-Year Data (Internal)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_synchrony.R
\name{pheno_synchrony}
\alias{pheno_synchrony}
\title{Analyze Phenological Synchrony Across Stations}
\usage{
pheno_synchrony(
  pep,
  species = NULL,
  phase_id = NULL,
  by = c("country", "year"),
  min_stations = 5,
  compute_trend = TRUE,
  na.rm = TRUE
)
}
\arguments{
\item{pep}{A \code{pep} object or \code{data.table} containing phenological
observations with columns \code{year}, \code{day}, and \code{s_id} (station ID).}

\item{species}{Optional character string to filter by species/genus.}

\item{phase_id}{Optional integer to filter by a single BBCH phase code.}

\item{by}{Character vector of column names to group by for synchrony calculation.
Default is \code{c("country", "year")} for annual synchrony by country.}

\item{min_stations}{Minimum number of stations required to calculate synchrony.
Default is 5. Groups with fewer stations return NA.}

\item{compute_trend}{Logical. If \code{TRUE} (default), computes trend in
synchrony over time using robust regression.}

\item{na.rm}{Logical. Should missing values be removed? Default \code{TRUE}.}
}
\value{
A \code{pheno_synchrony} object (list) containing:
\describe{
\item{data}{A \code{data.table} with synchrony metrics per group:
n_stations, mean_doy, sd_doy, cv_pct, and quality indicators}
\item{trend}{If \code{compute_trend = TRUE}, a data.table with trend
analysis results per region (slope, p-value, direction)}
\item{overall}{Summary statistics across all groups}
}
}
\description{
Measures how synchronized phenological events are across stations within
regions and whether synchrony is changing over time. Higher synchrony
indicates more uniform timing of phenological events.
}
\details{
Synchrony measures how similar phenological timing is across different
stations within the same region and year. This is important for:
\itemize{
\item Understanding spatial coherence of phenological signals
\item Detecting changes in spatial variability over time
\item Assessing network representativeness
}
}
\section{Synchrony Metrics}{

\describe{
\item{sd_doy}{Standard deviation across stations - lower values indicate
higher synchrony}
\item{cv_pct}{Coefficient of variation (SD/mean * 100) - relative measure
that allows comparison across phases with different mean timing}
\item{range_doy}{Range of DOY values across stations}
}
}

\section{Trend Interpretation}{

\itemize{
\item Negative trend in SD: Increasing synchrony over time
\item Positive trend in SD: Decreasing synchrony (more variable)
}
}

\examples{
\donttest{
pep <- pep_download()

# Subset to two countries for speed
pep_subset <- pep[country \%in\% c("Switzerland", "Austria")]

# Calculate synchrony for apple flowering by country and year
sync <- pheno_synchrony(pep_subset,
                        species = "Malus",
                        phase_id = 60)
print(sync)

# Get trend results (robust regression per country)
sync$trend

# Synchrony without trend analysis (faster)
sync_simple <- pheno_synchrony(pep_subset,
                               species = "Malus",
                               phase_id = 60,
                               compute_trend = FALSE)
sync_simple

# Custom grouping variables
sync_detailed <- pheno_synchrony(pep_subset,
                                 species = "Malus",
                                 phase_id = 60,
                                 by = c("country", "year"))
sync_detailed
}

}
\references{
Rosenzweig, C. et al. (2008). Attributing physical and biological
impacts to anthropogenic climate change.
\emph{Nature}, 453, 353--357.
\doi{10.1038/nature06937}
}
\seealso{
\code{\link{pheno_normals}} for climatological statistics,
\code{\link{pheno_anomaly}} for anomaly detection
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_anomaly.R
\name{summary.pheno_anomaly}
\alias{summary.pheno_anomaly}
\title{Summary Method for Phenological Anomalies}
\usage{
\method{summary}{pheno_anomaly}(object, ...)
}
\arguments{
\item{object}{A \code{pheno_anomaly} object}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns a summary list
}
\description{
Summary Method for Phenological Anomalies
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_regional_hh.R
\name{pheno_regional_hh}
\alias{pheno_regional_hh}
\title{Compile Regional Phenological Time Series for Heading and Harvest}
\usage{
pheno_regional_hh(
  pep,
  giss = NULL,
  lon_min = 4.2,
  lon_max = 8.1,
  lat_min = 44.7,
  lat_max = 48.1,
  species_name = "Triticum aestivum",
  year_min = 1961,
  pep_for_giss = c("aggregated", "near")
)
}
\arguments{
\item{pep}{A data.table containing PEP725 phenological data.}

\item{giss}{Optional. A data.table with GISS global temperature anomalies
(\code{year}, \code{dT}, \code{dT_sm}). If \code{NULL}, GISS-related
outputs are omitted.}

\item{lon_min}{Minimum longitude for bounding box filter (default is 4.2)}

\item{lon_max}{Maximum longitude for bounding box filter (default is 8.1)}

\item{lat_min}{Minimum latitude for bounding box filter (default is 44.7)}

\item{lat_max}{Maximum latitude for bounding box filter (default is 48.1)}

\item{species_name}{Name of the species to filter (default is
\code{"Triticum aestivum"})}

\item{year_min}{Minimum year for all time series outputs (default is 1961)}

\item{pep_for_giss}{Which PEP725 dataset to use for merging with GISS climate
anomalies. Either \code{"aggregated"} (global) or \code{"near"} (spatially
filtered). Ignored if \code{giss} is \code{NULL}.}
}
\value{
A named list with the following elements:
\describe{
\item{\code{ts_tidy}}{Long-format time series with consistent phase and source labeling}
\item{\code{pep_agg}}{Aggregated PEP725 time series (global species-level)}
\item{\code{pep_cgi}}{Spatially filtered PEP725 time series (based on
lat/lon box)}
\item{\code{giss}}{GISS global temperature anomalies (if provided)}
\item{\code{pep_giss}}{PEP time series merged with GISS anomalies (if provided)}
\item{\code{species}}{Species name used for filtering}
}
}
\description{
Generates harmonized phenological time series for heading (BBCH 60) and
harvest (BBCH 100) phases of a specified species and spatial region.
Optionally integrates GISS climate data for climate-sensitivity studies.
}
\details{
This function filters the PEP725 dataset by species and optionally by spatial
box. It aggregates phenological phases (Heading = 60, Harvest = 100) and
computes mean day-of-year (DOY) per year. If GISS data is provided,
temperature anomalies are merged with the selected PEP time series to support
phenology-climate analysis. Warnings are issued if phases are missing for
selected periods.
}
\examples{
\donttest{
pep <- pep_download()

out <- pheno_regional_hh(pep,
         species_name = "Triticum aestivum")
pheno_plot_hh(out)
}
}
\seealso{
\code{\link{pep_import}}, \code{\link{pheno_plot_hh}}
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_combine.R
\name{plot.pheno_combined}
\alias{plot.pheno_combined}
\title{Plot Method for Combined Time Series}
\usage{
\method{plot}{pheno_combined}(x, type = c("series", "stations", "residuals"), group = NULL, ...)
}
\arguments{
\item{x}{A \code{pheno_combined} object}

\item{type}{Character. "series" for combined time series, "stations" for
station effects, "residuals" for residual distribution.}

\item{group}{For grouped analysis, which group to plot.}

\item{...}{Additional arguments}
}
\value{
A ggplot object (invisibly)
}
\description{
Plot Method for Combined Time Series
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_daylength.R
\name{calc_daylength}
\alias{calc_daylength}
\title{Calculate Daylength (Photoperiod)}
\usage{
calc_daylength(doy, lat, method = c("brock", "cbm"))
}
\arguments{
\item{doy}{Integer or numeric vector. Day of year (1-365/366).}

\item{lat}{Numeric. Latitude in decimal degrees. Positive for Northern
Hemisphere, negative for Southern Hemisphere.}

\item{method}{Character. Daylength model:
\describe{
\item{\code{"brock"}}{(Default) Brock (1981) / Spencer (1971)
simple declination model: \eqn{\delta = 23.45^\circ \cdot
      \sin(2\pi(284 + \mathrm{DOY})/365)}. Fast and adequate at
mid-latitudes; differs from CBM by up to a few minutes/day
near the poles.}
\item{\code{"cbm"}}{Forsythe et al. (1995) Climate-Budget-Model
formulation. Includes the eccentricity correction of the Earth's
orbit and is generally the most accurate of the four models
Forsythe et al. compared.}
}}
}
\value{
A list with components:
\describe{
\item{daylength}{Daylength in hours}
\item{declination}{Solar declination angle in degrees}
}
If \code{doy} is a vector, returns a data.frame with these columns.
}
\description{
Computes the astronomical daylength (hours of daylight) for a given
day of year and latitude. Photoperiod is an important environmental
driver of plant phenology.
}
\details{
Both methods assume a "flat horizon" (sunrise/sunset when the geometric
centre of the sun crosses the horizon) and do not account for
atmospheric refraction, elevation, or local topography.

At polar latitudes (\eqn{|lat| > 66.5^\circ}), continuous daylight or
darkness may occur around solstices.
}
\section{Phenological Relevance}{

Many phenological events are triggered by photoperiod thresholds:
\itemize{
\item Spring bud burst often requires both temperature accumulation
and minimum daylength
\item Autumn leaf senescence may be triggered by shortening days
\item Flowering in many species is photoperiod-dependent
}
}

\examples{
# Daylength at spring equinox (DOY 80) at 50°N
calc_daylength(80, 50)

# Summer solstice at different latitudes
calc_daylength(172, c(30, 45, 60))

# Daylength through the year at 45°N
yearly <- calc_daylength(1:365, 45)
plot(yearly$doy, yearly$daylength, type = "l",
     xlab = "Day of Year", ylab = "Daylength (hours)")

}
\references{
Spencer, J. W. (1971). Fourier series representation of the position of
the sun. \emph{Search} 2(5):172. (Declination formula used by the
\code{"brock"} method.)

Brock, T. D. (1981). Calculating solar radiation for ecological studies.
\emph{Ecological Modelling} 14(1-2):1-19.
\doi{10.1016/0304-3800(81)90011-9}

Forsythe, W. C., Rykiel, E. J., Stahl, R. S., Wu, H., and
Schoolfield, R. M. (1995). A model comparison for daylength as a
function of latitude and day of year. \emph{Ecological Modelling}
80(1):87-95. \doi{10.1016/0304-3800(94)00034-F}
(CBM daylength model used by the \code{"cbm"} method.)
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_pls.R
\name{pheno_pls}
\alias{pheno_pls}
\title{Partial Least Squares Analysis for Phenology-Temperature Relationships}
\usage{
pheno_pls(
  pep,
  temp_data,
  by = NULL,
  method = c("robust", "standard"),
  split_month = 6,
  runn_mean = 11,
  ncomp = NULL,
  expl_var = 30,
  max_iter = 20,
  tol = 1e-04
)
}
\arguments{
\item{pep}{A \code{pep} object or data.frame with phenological observations.
Must contain \code{year} and \code{day} columns.}

\item{temp_data}{A data.frame with daily temperature data. Must contain
columns \code{year}, \code{doy} (day of year), and either \code{Tmean} or
both \code{Tmin} and \code{Tmax}.}

\item{by}{Character vector of grouping columns to match phenology and
temperature data (e.g., \code{"s_id"} for station-specific analysis).
Default \code{NULL} uses the same temperature for all observations per year.}

\item{method}{Character. PLS method to use:
\describe{
\item{"robust"}{(Default) Iteratively reweighted PLS using bisquare weights.
Resistant to outliers in both phenology and temperature data.}
\item{"standard"}{Classical PLS regression without robust weighting.}
}}

\item{split_month}{Integer (1-12). The last month to include in the
phenological year. Default 6 (June) means July-June phenological years.
This is important for spring phenology where relevant temperatures
span the previous autumn/winter.}

\item{runn_mean}{Integer (odd). Window size for running mean smoothing of
temperatures. Default 11 days. Larger values produce smoother results.}

\item{ncomp}{Integer or NULL. Number of PLS components. If NULL (default),
automatically determined to explain at least \code{expl_var} percent
of variance.}

\item{expl_var}{Numeric. Minimum percentage of variance to explain when
automatically selecting components. Default 30.}

\item{max_iter}{Integer. Maximum iterations for robust method. Default 20.}

\item{tol}{Numeric. Convergence tolerance for robust method. Default 1e-4.}
}
\value{
An object of class \code{pheno_pls} containing:
\describe{
\item{vip}{Data.frame with Variable Importance in Projection scores
for each day of the phenological year.}
\item{coefficients}{PLS regression coefficients for each day.}
\item{model}{The fitted PLS model object.}
\item{weights}{Observation weights (all 1 for standard method).}
\item{r_squared}{Model R-squared.}
\item{ncomp}{Number of components used.}
\item{method}{Method used ("robust" or "standard").}
\item{n_obs}{Number of observations.}
\item{n_years}{Number of years analyzed.}
}
}
\description{
Performs PLS regression to identify temperature-sensitive periods affecting
phenological timing. This is useful for understanding which parts of the
year have the strongest influence on phenological events.
}
\details{
PLS regression is particularly useful for phenology-temperature analysis
because it handles the high collinearity between daily temperatures
(adjacent days are highly correlated) while identifying which periods
have the strongest influence on phenological timing.

The robust method uses iteratively reweighted PLS with bisquare weights,
which downweights observations with large residuals. This is important
for phenological data which may contain outliers from observation errors
or unusual weather events.
}
\section{Interpreting Results}{

\describe{
\item{VIP scores}{Values > 0.8 indicate important variables. Days with
high VIP scores have strong influence on phenology timing.}
\item{Coefficients}{Negative coefficients indicate that warmer temperatures
during that period lead to earlier phenology (lower DOY).}
}
}

\section{Phenological Year}{

Spring phenology (e.g., flowering) is influenced by temperatures from the
previous autumn through spring. The \code{split_month} parameter defines
where to split the calendar year. With \code{split_month = 6}, a flowering
event in April 2020 is analyzed against temperatures from July 2019
through June 2020.
}

\examples{
\donttest{
data(pep_seed)

# Create example temperature data (fewer years for speed)
years <- 2005:2012
temp <- data.frame(
  year = rep(years, each = 365),
  doy = rep(1:365, length(years)),
  Tmin = rnorm(365 * length(years), mean = 5, sd = 8),
  Tmax = rnorm(365 * length(years), mean = 15, sd = 8)
)

# Run standard PLS (fast: no CV, fixed components)
result <- pheno_pls(pep_seed, temp, method = "standard", ncomp = 2)
print(result)
}

}
\references{
Luedeling E, Gassner A (2012). Partial Least Squares Regression for
analyzing walnut phenology in California. Agricultural and Forest
Meteorology 158-159:104-113.

Wold S, Sjostrom M, Eriksson L (2001). PLS-regression: a basic tool of
chemometrics. Chemometrics and Intelligent Laboratory Systems 58:109-130.
}
\seealso{
\code{\link{pheno_trend_turning}} for trend analysis,
\code{\link{pheno_anomaly}} for anomaly detection
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_quality.R
\name{plot.pep_quality}
\alias{plot.pep_quality}
\title{Plot Method for Data Quality Assessment}
\usage{
\method{plot}{pep_quality}(
  x,
  which = c("overview", "grades", "map"),
  pep = NULL,
  show_grades = c("A", "B", "C", "D"),
  alpha = 0.6,
  title = NULL,
  ...
)
}
\arguments{
\item{x}{A \code{pep_quality} object.}

\item{which}{Character. Type of plot to produce:
\itemize{
\item \code{"overview"} (default): Two-panel overview with grade
distribution and station quality map (requires \code{pep} argument)
\item \code{"grades"}: Bar chart of quality grade distribution only
\item \code{"map"}: Map of station locations colored by quality grade
(requires \code{pep} argument for coordinates)
}}

\item{pep}{Optional \code{pep} object providing station coordinates (lon, lat).
Required for \code{which = "map"} or \code{which = "overview"}.}

\item{show_grades}{Character vector of grades to display on the map.
Default is \code{c("A", "B", "C", "D")} (all grades). Use e.g.
\code{show_grades = "D"} to show only poor-quality stations.}

\item{alpha}{Numeric. Transparency of points on the map (0-1). Default is 0.6.}

\item{title}{Optional character string for the plot title.}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object (invisibly).
}
\description{
Creates visualizations of phenological data quality metrics, including
grade distribution and a map of station quality.
}
\details{
For the map visualization, the function:
\itemize{
\item Aggregates quality to one grade per station (worst grade if multiple phases)
\item Uses country borders from the \code{rnaturalearth} package
\item Uses colorblind-friendly colors (blue=A, cyan=B, orange=C, vermillion=D)
}
}
\examples{
\donttest{
pep <- pep_download()

# Assess quality for Swiss stations
pep_ch <- pep[country == "Switzerland"]
quality <- pep_quality(pep_ch, by = c("s_id", "phase_id"))

# Grade distribution only (no pep data needed)
plot(quality, which = "grades")

# Map of station quality (requires pep for coordinates)
plot(quality, which = "map", pep = pep_ch)

# Map showing only poor-quality stations
plot(quality, which = "map", pep = pep_ch, show_grades = "D")

# Map showing problematic stations (C and D grades)
plot(quality, which = "map", pep = pep_ch, show_grades = c("C", "D"))

# Overview: grades + map
plot(quality, pep = pep_ch)
}

}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_class.R
\name{validate_pep}
\alias{validate_pep}
\title{Validate PEP725 Data Structure}
\usage{
validate_pep(x)
}
\arguments{
\item{x}{A data.frame or data.table to validate.}
}
\value{
Invisibly returns \code{TRUE} if valid; otherwise throws an error.
}
\description{
Internal function to validate that a data object has the required
structure for PEP725 phenological data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pep_completeness.R
\name{summary.pep_completeness}
\alias{summary.pep_completeness}
\title{Summary Method for Completeness Assessment}
\usage{
\method{summary}{pep_completeness}(object, ...)
}
\arguments{
\item{object}{A \code{pep_completeness} object}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns a summary list
}
\description{
Summary Method for Completeness Assessment
}
\author{
Matthias Templ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pheno_synchrony.R
\name{plot.pheno_synchrony}
\alias{plot.pheno_synchrony}
\title{Plot Method for Phenological Synchrony Analysis}
\usage{
\method{plot}{pheno_synchrony}(x, region = NULL, ...)
}
\arguments{
\item{x}{A \code{pheno_synchrony} object}

\item{region}{Optional region to plot (for multi-region analyses)}

\item{...}{Additional arguments passed to ggplot}
}
\value{
A ggplot object
}
\description{
Creates a time series plot of synchrony (SD) over years.
}
\author{
Matthias Templ
}
